=== modified file '.bzrignore'
--- .bzrignore	2005-07-11 05:46:55 +0000
+++ .bzrignore	2005-10-03 16:53:39 +0000
@@ -1,5 +1,6 @@
 ## *.diff
 ./doc/*.html
+./tutorial.html
 *.py[oc]
 *~
 .arch-ids
@@ -14,3 +15,8 @@
 api
 test.log
 ./tmp
+build
+./ID
+./tags
+.bzr-shelf
+bzrlib/plugins/*

=== added file 'HACKING'
--- HACKING	1970-01-01 00:00:00 +0000
+++ HACKING	2005-10-06 22:15:52 +0000
@@ -0,0 +1,190 @@
+============================
+Guidelines for modifying bzr
+============================
+
+.. contents::
+
+(The current version of this document is available in the file ``HACKING``
+in the source tree, or at http://bazaar-ng.org/hacking.html)
+
+Overall
+=======
+
+* New functionality should have test cases.  Preferably write the
+  test before writing the code.
+
+  In general, you can test at either the command-line level or the
+  internal API level.  Choose whichever is appropriate: if adding a
+  new command, or a new command option, then call through run_bzr().
+  It is not necessary to do both. Tests that test the command line level
+  are appropriate for checking the UI behaves well - bug fixes and
+  core improvements should be tested closer to the code that is doing the
+  work. Command line level tests should be placed in 'blackbox.py'.
+
+* Before fixing a bug, write a test case so that it does not regress.
+
+* Exceptions should be defined inside bzrlib.errors, so that we can
+  see the whole tree at a glance.
+
+* Imports should be done at the top-level of the file, unless there is
+  a strong reason to have them lazily loaded when a particular
+  function runs.  Import statements have a cost, so try to make sure
+  they don't run inside hot functions.
+
+* Module names should always be given fully-qualified,
+  i.e. ``bzrlib.hashcache`` not just ``hashcache``.
+
+Evolving interfaces
+-------------------
+
+If you change the behaviour of an API in an incompatible way, please
+be sure to change its name as well. For instance, if I add a keyword
+parameter to branch.commit - that's fine. On the other hand, if I add
+a keyword parameter to branch.commit which is a *required* transaction
+object, I should rename the api - i.e. to 'branch.commit_transaction'.
+
+This will prevent users of the old api getting surprising results. 
+Instead, they will get an Attribute error as the api is missing, and
+will know to update their code. If in doubt, just ask on #bzr.
+
+Documentation
+=============
+
+If you change the behaviour of a command, please update its docstring
+in bzrlib/commands.py.  This is displayed by the 'bzr help' command.
+
+If you make a user-visible change, please add a note to the NEWS file.
+The description should be written to make sense to someone who's just
+a user of bzr, not a developer: new functions or classes shouldn't be
+mentioned, but new commands, changes in behaviour or fixed nontrivial
+bugs should be listed.  See the existing entries for an idea of what
+should be done.
+
+
+API documentation
+-----------------
+
+Functions, methods, classes and modules should have docstrings
+describing how they are used. 
+
+The first line of the docstring should be a self-contained sentence.
+
+For the special case of Command classes, this acts as the user-visible
+documentation shown by the help command.
+
+The docstrings should be formatted as reStructuredText_ (like this
+document), suitable for processing using the epydoc_ tool into HTML
+documentation.
+
+.. _reStructuredText: http://docutils.sourceforge.net/rst.html
+.. _epydoc: http://epydoc.sourceforge.net/
+
+
+
+Coding style
+============
+
+Please write PEP-8__ compliant code.  
+
+One often-missed requirement is that the first line of docstrings
+should be a self-contained one-sentence summary.
+
+__ http://www.python.org/peps/pep-0008.html
+
+
+
+Naming
+------
+
+Functions, methods or members that are in some sense "private" are given
+a leading underscore prefix.  This is just a hint that code outside the
+implementation should probably not use that interface.
+
+We prefer class names to be concatenated capital words (``TestCase``)
+and variables, methods and functions to be lowercase words joined by
+underscores (``revision_id``, ``get_revision``).
+
+For the purposes of naming some names are treated as single compound
+words: "filename", "revno".
+
+Consider naming classes as nouns and functions/methods as verbs.
+
+
+Standard names
+--------------
+
+``revision_id`` not ``rev_id`` or ``revid``
+
+Functions that transform one thing to another should be named ``x_to_y``
+(not ``x2y`` as occurs in some old code.)
+
+
+Writing output
+==============
+
+(The strategy described here is what we want to get to, but it's not
+consistently followed in the code at the moment.)
+
+bzrlib is intended to be a generically reusable library.  It shouldn't
+write messages to stdout or stderr, because some programs that use it
+might want to display that information through a GUI or some other
+mechanism.
+
+We can distinguish two types of output from the library:
+
+ 1. Structured data representing the progress or result of an
+    operation.  For example, for a commit command this will be a list
+    of the modified files and the finally committed revision number
+    and id.
+
+    These should be exposed either through the return code or by calls
+    to a callback parameter.
+
+    A special case of this is progress indicators for long-lived
+    operations, where the caller should pass a ProgressBar object.
+
+ 2. Unstructured log/debug messages, mostly for the benefit of the
+    developers or users trying to debug problems.  This should always
+    be sent through ``bzrlib.trace`` and Python ``logging``, so that
+    it can be redirected by the client.
+
+The distinction between the two is a bit subjective, but in general if
+there is any chance that a library would want to see something as
+structured data, we should make it so.
+
+The policy about how output is presented in the text-mode client
+should be only in the command-line tool.
+
+
+Writing tests
+=============
+In general tests should be placed in a file named testFOO.py where 
+FOO is the logical thing under test. That file should be placed in the
+tests subdirectory under the package being tested.
+
+For example, tests for merge3 in bzrlib belong in bzrlib/tests/testmerge3.py.
+See bzrlib/selftest/testsampler.py for a template test script.
+
+
+Running tests
+=============
+Currently, bzr selftest is used to invoke tests.
+You can provide a pattern argument to run a subset. For example, 
+to run just the whitebox tests, run::
+
+  bzr selftest -v whitebox
+
+
+Errors and exceptions
+=====================
+
+[write me]
+
+
+Jargon
+======
+
+revno
+    Integer identifier for a revision on the main line of a branch.
+    Revision 0 is always the null revision; others are 1-based
+    indexes into the branch's revision history.

=== added file 'Makefile'
--- Makefile	1970-01-01 00:00:00 +0000
+++ Makefile	2005-10-07 01:11:04 +0000
@@ -0,0 +1,20 @@
+all: executables
+
+executables:
+	chmod u+x ./bzr
+	chmod u+x ./setup.py
+
+check:
+	./bzr selftest
+
+clean: executables
+	./setup.py clean
+	find . -name "*.pyc" | xargs rm
+
+.PHONY: all
+
+
+# build emacs cross-reference
+tag_files=./bzr ./bzrlib/*py ./bzrlib/selftest/*.py
+TAGS: $(tag_files)
+	ctags-exuberant -e $(tag_files)

=== modified file 'NEWS'
--- NEWS	2005-07-23 13:53:01 +0000
+++ NEWS	2005-10-08 08:45:42 +0000
@@ -1,4 +1,218 @@
-DEVELOPMENT HEAD
+bzr 0.1 2005-10-06
+
+
+  BUG FIXES:
+  
+    * bzr-man.py has been updated to work again. Contributed by
+      Rob Weir.
+
+    * Locking is now done with fcntl.lockf which works with NFS
+      file systems. Contributed by Harald Meland.
+
+    * When a merge encounters a file that has been deleted on
+      one side and modified on the other, the old contents are
+      written out to foo.BASE and foo.SIDE, where SIDE is this
+      or OTHER. Contributed by Aaron Bentley.
+
+    * Export was choosing incorrect file paths for the content of
+      the tarball, this has been fixed by Aaron Bentley.
+
+    * Commit will no longer commit without a log message, an 
+      error is returned instead. Contributed by Jelmer Vernooij.
+
+    * If you commit a specific file in a sub directory, any of its
+      parent directories that are added but not listed will be 
+      automatically included. Suggested by Michael Ellerman.
+
+    * bzr commit and upgrade did not correctly record new revisions
+      for files with only a change to their executable status.
+      bzr will correct this when it encounters it. Fixed by
+      Robert Collins
+
+
+  IMPROVEMENTS:
+
+    * 'branch' now accepts a --basis parameter which will take advantage
+      of local history when making a new branch. This allows faster 
+      branching of remote branches. Contributed by Aaron Bentley.
+
+    * New tree format based on weave files, called version 5.
+      Existing branches can be upgraded to this format using 
+      'bzr upgrade'.
+
+    * Symlinks are now versionable. Initial patch by 
+      Erik Toubro Nielsen, updated to head by Robert Collins.
+
+    * Executable bits are tracked on files. Patch from Gustavo
+      Niemeyer.
+
+    * 'bzr status' now shows unknown files inside a selected directory.
+      Patch from Heikki Paajanen.
+
+    * Merge conflicts are recorded in .bzr. Two new commands 'conflicts'
+      and 'resolve' have needed added, which list and remove those 
+      merge conflicts respectively. A conflicted tree cannot be committed
+      in. Contributed by Aaron Bentley.
+
+    * 'rm' is now an alias for 'remove'.
+
+
+  INTERNALS:
+
+    * Test case failures have the exception printed after the log 
+      for your viewing pleasure.
+
+    * InventoryEntry is now an abstract base class, use one of the
+      concrete InventoryDirectory etc classes instead.
+
+    * Branch raises an UnsupportedFormatError when it detects a 
+      bzr branch it cannot understand. This allows for precise
+      handling of such circumstances.
+
+
+  TESTING:
+
+    * Removed testsweet module so that tests can be run after 
+      bzr installed by 'bzr selftest'.
+
+    * 'bzr selftest' command-line arguments can now be partial ids
+      of tests to run, e.g. 'bzr selftest test_weave'
+
+      
+bzr 0.0.9 2005-09-23
+
+  BUG FIXES:
+
+    * Fixed "branch -r" option.
+
+    * Fix remote access to branches containing non-compressed history.
+      (Robert Collins).
+
+    * Better reliability of http server tests.  (John Arbash-Meinel)
+
+    * Merge graph maximum distance calculation fix.  (Aaron Bentley)
+   
+    * Various minor bug in windows support have been fixed, largely in the
+      test suite. Contributed by Alexander Belchenko.
+
+  IMPROVEMENTS:
+
+    * Status now accepts a -r argument to give status between chosen
+      revisions. Contributed by Heikki Paajanen.
+
+    * Revision arguments no longer use +/-/= to control ranges, instead
+      there is a 'before' namespace, which limits the successive namespace.
+      For example '$ bzr log -r date:yesterday..before:date:today' will
+      select everything from yesterday and before today. Contributed by
+      Robey Pointer
+
+    * There is now a bzr.bat file created by distutils when building on 
+      Windows. Contributed by Alexander Belchenko.
+
+  INTERNALS:
+
+    * Removed uuid() as it was unused.
+
+    * Improved 'fetch' code for pulling revisions from one branch into
+      another (used by pull, merged, etc.)
+
+
+bzr 0.0.8 2005-09-20
+
+  IMPROVEMENTS:
+
+    * Adding a file whose parent directory is not versioned will
+      implicitly add the parent, and so on up to the root. This means
+      you should never need to explictly add a directory, they'll just
+      get added when you add a file in the directory.  Contributed by
+      Michael Ellerman.
+
+    * Ignore .DS_Store (contains Mac metadata) by default.  Patch from
+      Nir Soffer.
+
+    * If you set BZR_EDITOR in the environment, it is checked in
+      preference to EDITOR and the config file for the interactive commit
+      editing program. Related to this is a bugfix where a missing program
+      set in EDITOR would cause editing to fail, now the fallback program
+      for the operating system is still tried.
+
+    * Files that are not directories/symlinks/regular files will no longer
+      cause bzr to fail, it will just ignore them by default. You cannot add
+      them to the tree though - they are not versionable.
+
+
+  INTERNALS:
+
+    * Refactor xml packing/unpacking.
+
+  BUG FIXES: 
+
+    * Fixed 'bzr mv' by Ollie Rutherfurd.
+
+    * Fixed strange error when trying to access a nonexistent http
+      branch.
+
+    * Make sure that the hashcache gets written out if it can't be
+      read.
+
+
+  PORTABILITY:
+
+    * Various Windows fixes from Ollie Rutherfurd.
+
+    * Quieten warnings about locking; patch from Matt Lavin.
+
+
+bzr-0.0.7 2005-09-02
+
+  NEW FEATURES:
+
+    * ``bzr shell-complete`` command contributed by Clint Adams to
+      help with intelligent shell completion.
+
+    * New expert command ``bzr find-merge-base`` for debugging merges.
+
+
+  ENHANCEMENTS:
+
+    * Much better merge support.
+
+    * merge3 conflicts are now reported with markers like '<<<<<<<'
+      (seven characters) which is the same as CVS and pleases things
+      like emacs smerge.
+
+
+  BUG FIXES:
+
+    * ``bzr upgrade`` no longer fails when trying to fix trees that
+      mention revisions that are not present.
+
+    * Fixed bugs in listing plugins from ``bzr plugins``.
+
+    * Fix case of $EDITOR containing options for the editor.
+
+    * Fix log -r refusing to show the last revision.
+      (Patch from Goffredo Baroncelli.)
+
+
+  CHANGES:
+
+    * ``bzr log --show-ids`` shows the revision ids of all parents.
+
+    * Externally provided commands on your $BZRPATH no longer need
+      to recognize --bzr-usage to work properly, and can just handle
+      --help themselves.
+
+
+  LIBRARY:
+
+    * Changed trace messages to go through the standard logging
+      framework, so that they can more easily be redirected by
+      libraries.
+
+
+
+bzr-0.0.6 2005-08-18
 
   NEW FEATURES:
 
@@ -14,13 +228,36 @@
       FOO.
       
     * ``bzr add`` with no arguments adds everything under the current directory.
-    
+
+    * ``bzr mv`` does move or rename depending on its arguments, like
+      the Unix command.
+
+    * ``bzr missing`` command shows a summary of the differences
+      between two trees.  (Merged from John Arbash-Meinel.)
+
+    * An email address for commits to a particular tree can be
+      specified by putting it into .bzr/email within a branch.  (Based
+      on a patch from Heikki Paajanen.)
+
+
   ENHANCEMENTS:
 
     * Faster working tree operations.
 
+
   CHANGES:
 
+    * 3rd-party modules shipped with bzr are copied within the bzrlib
+      python package, so that they can be installed by the setup
+      script without clashing with anything already existing on the
+      system.  (Contributed by Gustavo Niemeyer.)
+
+    * Moved plugins directory to bzrlib/, so that there's a standard
+      plugin directory which is not only installed with bzr itself but
+      is also available when using bzr from the development tree.
+      BZR_PLUGIN_PATH and DEFAULT_PLUGIN_PATH are then added to the
+      standard plugins directory.
+
     * When exporting to a tarball with ``bzr export --format tgz``, put 
       everything under a top directory rather than dumping it into the
       current directory.   This can be overridden with the ``--root`` 
@@ -39,6 +276,22 @@
     * By default the commit command refuses to record a revision with
       no changes unless the ``--unchanged`` option is given.
 
+    * The ``--no-plugins``, ``--profile`` and ``--builtin`` command
+      line options must come before the command name because they 
+      affect what commands are available; all other options must come 
+      after the command name because their interpretation depends on
+      it.
+
+    * ``branch`` and ``clone`` added as aliases for ``branch``.
+
+    * Default log format is back to the long format; the compact one
+      is available with ``--short``.
+      
+      
+  BUG FIXES:
+  
+    * Fix bugs in committing only selected files or within a subdirectory.
+
 
 bzr-0.0.5  2005-06-15
   

=== added file 'NEWS.developers'
--- NEWS.developers	1970-01-01 00:00:00 +0000
+++ NEWS.developers	2005-10-06 06:31:16 +0000
@@ -0,0 +1,20 @@
+changes in weave branch:
+
+ * Remove RevisionReference class; Revision.parent_ids is now simply a
+   list of their ids and parent_sha1s is a list of their corresponding
+   sha1s (for old branches only at the moment.)
+
+ * New method-object style interface for Commit() and Fetch().
+
+ * Renamed Branch.last_patch() to Branch.last_revision(), since
+   we call them revisions not patches.
+
+ * Move copy_branch to bzrlib.clone.copy_branch.  The destination
+   directory is created if it doesn't exist.
+
+ * Inventories now identify the files which were present by 
+   giving the revision *of that file*.
+
+ * Inventory and Revision XML contains a version identifier.  
+   This must be consistent with the overall branch version
+   but allows for more flexibility in future upgrades.

=== modified file 'TODO'
--- TODO	2005-07-22 23:43:26 +0000
+++ TODO	2005-09-19 02:52:24 +0000
@@ -17,7 +17,23 @@
 Small things
 ------------
 
-* is_within_any is very very dodgy
+* ``bzr status`` should show some description of patches that are merged but 
+  not yet committed.  Preferably in a very compact format so that they
+  just fit on a single line; this could also be used for say ``log
+  --tiny``::
+
+  2005-11-05      mbp  Flowers for stephane
+
+* Fix tests so that import errors caused by modules don't produce false reports
+  that the tests themselves don't exist.
+
+* Fix tests so that one test failure doesn't prevent other tests from running
+
+* print a message at the end of running the tests telling them that the
+  test log and output exists but can be removed
+
+* tests for running the commit editor, and fix problem of not passing in 
+  multiple arguments
 
 * Merging add of a new file clashing with an existing file doesn't
   work; add gets an error that it's already versioned and the merge
@@ -68,7 +84,9 @@
 * ``bzr status DIR`` should give status on all files under that
   directory.
 
-* ``bzr log DIR`` should give changes to any files within DIR.
+* ``bzr log DIR`` should give changes to any files within DIR; at the
+  moment it only lists things which modify the specific named file
+  (and not its contents)
 
 * ``bzr inventory -r REV`` and perhaps unify this with ``bzr ls``,
   giving options to display ids, types, etc.
@@ -156,13 +174,55 @@
 * Reproducible performance benchmark to measure whether performance is 
   getting better or worse.
 
+* ``bzr log -m foo`` should perhaps error if nothing matches?
+
+* ``bzr diff -r 30 -r 40 foo.c`` or ``bzr diff -r30..40 foo.c``
+
+  If diffing between two branches then we probably want two -r
+  options, since the revisions don't form a range that can be
+  evaluated on either one.
+
+* bzr diff shouldn't diff binary files
+
+* setup.py install when run from a bzr tree should freeze the tree
+  revision-id into the installed bzr.
+
+* bzr script should trap ImportError and perhaps give a better error
+  message?
+
+* revert after a merge should possibly remove all the BASE/THIS/OTHER
+  files to get you back to where you were.
+
+* files that are added and then deleted are still reported as added
+
+* stores should raise KeyError, not IndexError
+
+* merging from a remote branch seems to sometimes raise errors not
+  present locally
+
+* should be possible to give a related branch when pulling from a
+  remote branch to make things faster
+
+* sometimes gives "conflicting add" even when the contents are in fact
+  the same???
+
+* BZRDIR should be in branch.py not __init__.py.
+
+* ``status `` should accept a -r option to show changes relative to a revision,
+  or between revisions
 
 Medium things
 -------------
 
-* Merge revert patch.
-
-* ``bzr mv`` that does either rename or move as in Unix.
+* ~/.bzr.log is not written anymore for some reason.
+
+* merge should add all revision and inventory XML to the local store.
+
+* check should give a warning for revisions that are named in the
+  chain but not actually present in the store.
+
+* remove anything outside of the branch implementation that directly
+  accesses the stores.
 
 * More efficient diff of only selected files.  We should be able to
   just get the id for the selected files, look up their location and
@@ -182,6 +242,8 @@
 
 * More test cases.
 
+  - ``missing`` command
+
   - Selected-file commit
 
   - Impossible selected-file commit: adding things in non-versioned
@@ -201,13 +263,9 @@
 
   Given this we might be able to import patches at 1/second or better.
 
-* Get branch over http.
-
-* Pull pure updates over http.
-
 * revfile compression.
 
-* Split inventory into per-directory files.
+* Split inventory into per-directory files?
 
 * Fix ignore file parsing:
 
@@ -328,9 +386,9 @@
 * Function to list a directory, saying in which revision each file was
   last modified.  Useful for web and gui interfaces, and slow to
   compute one file at a time.
-
-* unittest is standard, but the results are kind of ugly; would be
-  nice to make it cleaner.
+  
+  This will be done when we track file texts by referring to the
+  version that created them. 
 
 * Check locking is correct during merge-related operations.
 
@@ -345,6 +403,33 @@
 
 * Track all merged-in revisions in a versioned add-only metafile.
 
+* ``pull --clobber`` should discard any local changes not present
+  remotely.  Not generally what you want, but possibly useful when
+  you're just mirroring another branch and want to keep tracking it
+  even when they e.g. uncommit or make similar non-forward movements.
+  Also for push I suppose.  Clobber may not be the best name, maybe
+  ``--destroy``?
+
+* ``uncommit`` command that removes a revision from the end of the
+  revision-history; just doing this is enough to remove the commit,
+  and a new commit will automatically be made against the
+  predecessor.  This can be repeated.
+
+  It only makes sense to delete from the tail of history, not from the
+  end.
+
+  The revision, its inventory and texts remain floating in the store.
+  We should perhaps add the revision to a list of removed-commits, so
+  that it can be restored or at least accounted for when checking
+  consistency.  This file would not be versioned, and probably should
+  not propagate when branched.
+
+  If we track merged revisions then we need to update this list too.
+  If the list is stored in a weave it's easy (implicit): the version
+  of the list can remain but it won't be referenced anymore.  It's
+  probably best to just store this list in a weave in the first place
+  and be done.
+
 
 Large things
 ------------

=== modified file 'bzr'
--- bzr	2005-07-26 14:06:11 +0000
+++ bzr	2005-09-29 05:06:14 +0000
@@ -40,9 +40,22 @@
     print >>sys.stderr, "bzr: error: cannot find a suitable python interpreter"
     print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
     sys.exit(1)
-os.unsetenv(REINVOKE)
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
 
-import bzrlib, bzrlib.commands
+try:
+    import bzrlib
+    import bzrlib.commands
+    import bzrlib.trace
+except ImportError, e:
+    sys.stderr.write("bzr: ERROR: "
+                     "Couldn't import bzrlib and dependencies.\n"
+                     "Please check bzrlib is on your PYTHONPATH.\n"
+                     "\n")
+    raise
 
 if __name__ == '__main__':
+    bzrlib.trace.enable_default_logging()
     sys.exit(bzrlib.commands.main(sys.argv))
+else:
+    pass    # should this give an error? - it can't be used as a lib

=== modified file 'bzr-man.py'
--- bzr-man.py	2005-06-06 13:24:18 +0000
+++ bzr-man.py	2005-09-28 04:58:18 +0000
@@ -42,7 +42,8 @@
     print >>sys.stderr, "bzr-man.py: error: cannot find a suitable python interpreter"
     print >>sys.stderr, "  (need %d.%d or later)" % NEED_VERS
     sys.exit(1)
-os.unsetenv(REINVOKE)
+if hasattr(os, "unsetenv"):
+    os.unsetenv(REINVOKE)
 
 import bzrlib, bzrlib.help
 
@@ -90,15 +91,17 @@
 
     def parse_line(self, line):
         m = self.usage_exp.match(line)
+	if line == '':
+		return
         if m:
             if self.state == 0:
                 if self.usage:
                     self.command_usage.append((self.command,self.usage,self.descr))
                     self.all_commands.append(self.command)
-                self.usage = line
+                self.usage = " ".join(line.split(" ")[1:])
                 self.command = m.groups()[0]
             else:
-                raise Error, "matching usage line in state %d" % state
+                raise RuntimeError, "matching usage line in state %d" % state
             self.state = 1
             return
         m = self.descr_exp.match(line)
@@ -106,10 +109,10 @@
             if self.state == 1:
                 self.descr = m.groups()[0]
             else:
-                raise Error, "matching descr line in state %d" % state
+                raise RuntimeError, "matching descr line in state %d" % state
             self.state = 0
             return
-        raise Error, "Cannot parse this line"
+        raise RuntimeError, "Cannot parse this line ('%s')." % line
 
     def end_parse(self):
         if self.state == 0:
@@ -117,7 +120,7 @@
                 self.command_usage.append((self.command,self.usage,self.descr))
                 self.all_commands.append(self.command)
         else:
-            raise Error, "ending parse in state %d" % state
+            raise RuntimeError, "ending parse in state %d" % state
 
     def write_to_manpage(self, outfile):
         bzrcmd = self.params["bzrcmd"]

=== modified file 'bzrlib/__init__.py'
--- bzrlib/__init__.py	2005-07-23 13:52:38 +0000
+++ bzrlib/__init__.py	2005-10-06 12:34:22 +0000
@@ -16,10 +16,6 @@
 
 """bzr library"""
 
-# TODO: Do less imports here
-from branch import Branch, ScratchBranch, find_branch
-from errors import BzrError
-
 BZRDIR = ".bzr"
 
 DEFAULT_IGNORE = ['.bzr.log',
@@ -32,28 +28,35 @@
                   'BitKeeper',
                   '.git',
                   'TAGS', '.make.state', '.sconsign', '.tmp*',
-                  '.del-*']
+                  '.del-*',
+                  '.DS_Store',]
 
 IGNORE_FILENAME = ".bzrignore"
 
+import os
 import locale
 user_encoding = locale.getpreferredencoding() or 'ascii'
 del locale
 
 __copyright__ = "Copyright 2005 Canonical Development Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-__version__ = '0.0.5'
+__version__ = '0.1rc1'
 
 
 def get_bzr_revision():
     """If bzr is run from a branch, return (revno,revid) or None"""
+    import bzrlib.errors
+    from bzrlib.branch import Branch
+    
     try:
-        branch = Branch(__path__[0])
+        branch = Branch.open(os.path.dirname(__path__[0]))
         rh = branch.revision_history()
         if rh:
             return len(rh), rh[-1]
         else:
             return None
-    except BzrError:
+    except bzrlib.errors.BzrError:
         return None
     
+def test_suite():
+    import selftest
+    return selftest.test_suite()

=== modified file 'bzrlib/add.py'
--- bzrlib/add.py	2005-07-23 14:06:37 +0000
+++ bzrlib/add.py	2005-10-08 00:49:13 +0000
@@ -14,11 +14,17 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from trace import mutter, note
+from os.path import dirname
+
+import bzrlib.errors as errors
+from bzrlib.inventory import InventoryEntry
+from bzrlib.trace import mutter, note, warning
+from bzrlib.errors import NotBranchError
+from bzrlib.branch import Branch
+from bzrlib.osutils import quotefn
 
 def glob_expand_for_win32(file_list):
     import glob
-    
     expanded_file_list = []
     for possible_glob in file_list:
         glob_files = glob.glob(possible_glob)
@@ -31,43 +37,72 @@
             expanded_file_list += glob_files
     return expanded_file_list
 
-def smart_add(file_list, verbose=True, recurse=True):
-    """Add files to version, optionally recursing into directories.
-
-    This is designed more towards DWIM for humans than API simplicity.
-    For the specific behaviour see the help for cmd_add().
-    """
-    import os
+
+def add_reporter_null(path, kind, entry):
+    """Absorb add reports and do nothing."""
+    pass
+
+def add_reporter_print(path, kind, entry):
+    """Print a line to stdout for each file that's added."""
+    print "added", quotefn(path)
+    
+def _prepare_file_list(file_list):
+    """Prepare a file list for use by smart_add_*."""
     import sys
-    from bzrlib.osutils import quotefn, kind_marker
-    from bzrlib.errors import BadFileKindError, ForbiddenFileError
-    import bzrlib.branch
-    import bzrlib.osutils
-
     if sys.platform == 'win32':
         file_list = glob_expand_for_win32(file_list)
-        
     if not file_list:
         file_list = ['.']
-    
+    file_list = list(file_list)
+    return file_list
+
+
+def smart_add(file_list, recurse=True, reporter=add_reporter_null):
+    """Add files to version, optionally recursing into directories.
+
+    This is designed more towards DWIM for humans than API simplicity.
+    For the specific behaviour see the help for cmd_add().
+
+    Returns the number of files added.
+    """
+    file_list = _prepare_file_list(file_list)
+    b = Branch.open_containing(file_list[0])
+    return smart_add_branch(b, file_list, recurse, reporter)
+
+        
+def smart_add_branch(branch, file_list, recurse=True, reporter=add_reporter_null):
+    """Add files to version, optionally recursing into directories.
+
+    This is designed more towards DWIM for humans than API simplicity.
+    For the specific behaviour see the help for cmd_add().
+
+    This yields a sequence of (path, kind, file_id) for added files.
+
+    Returns the number of files added.
+    """
+    import os
+    from bzrlib.errors import BadFileKindError, ForbiddenFileError
+    import bzrlib.branch
+
+    assert isinstance(recurse, bool)
+
+    file_list = _prepare_file_list(file_list)
     user_list = file_list[:]
-    assert not isinstance(file_list, basestring)
-    b = bzrlib.branch.Branch(file_list[0], find_root=True)
-    inv = b.read_working_inventory()
-    tree = b.working_tree()
+    inv = branch.read_working_inventory()
+    tree = branch.working_tree()
     count = 0
 
     for f in file_list:
-        rf = b.relpath(f)
-        af = b.abspath(rf)
+        rf = branch.relpath(f)
+        af = branch.abspath(rf)
 
         kind = bzrlib.osutils.file_kind(af)
 
-        if kind != 'file' and kind != 'directory':
+        if not InventoryEntry.versionable_kind(kind):
             if f in user_list:
                 raise BadFileKindError("cannot add %s of type %s" % (f, kind))
             else:
-                print "skipping %s (can't add file of kind '%s')" % (f, kind)
+                warning("skipping %s (can't add file of kind '%s')", f, kind)
                 continue
 
         mutter("smart add of %r, abs=%r" % (f, af))
@@ -77,19 +112,28 @@
             
         versioned = (inv.path2id(rf) != None)
 
+        if kind == 'directory':
+            try:
+                sub_branch = Branch.open(af)
+                sub_tree = True
+            except NotBranchError:
+                sub_tree = False
+            except errors.UnsupportedFormatError:
+                sub_tree = True
+        else:
+            sub_tree = False
+
         if rf == '':
             mutter("branch root doesn't need to be added")
+            sub_tree = False
         elif versioned:
             mutter("%r is already versioned" % f)
+        elif sub_tree:
+            mutter("%r is a bzr tree" %f)
         else:
-            file_id = bzrlib.branch.gen_file_id(rf)
-            inv.add_path(rf, kind=kind, file_id=file_id)
-            mutter("added %r kind %r file_id={%s}" % (rf, kind, file_id))
-            count += 1 
-
-            print 'added', quotefn(f)
-
-        if kind == 'directory' and recurse:
+            count += __add_one(branch, inv, rf, kind, reporter)
+
+        if kind == 'directory' and recurse and not sub_tree:
             for subf in os.listdir(af):
                 subp = os.path.join(rf, subf)
                 if subf == bzrlib.BZRDIR:
@@ -98,12 +142,30 @@
                     mutter("skip ignored sub-file %r" % subp)
                 else:
                     mutter("queue to add sub-file %r" % subp)
-                    file_list.append(b.abspath(subp))
-
+                    file_list.append(branch.abspath(subp))
+
+
+    mutter('added %d entries', count)
+    
     if count > 0:
-        if verbose:
-            note('added %d' % count)
-        b._write_inventory(inv)
-    else:
-        print "nothing new to add"
-        # should this return 1 to the shell?
+        branch._write_inventory(inv)
+
+    return count
+
+def __add_one(branch, inv, path, kind, reporter):
+    """Add a file or directory, automatically add unversioned parents."""
+
+    # Nothing to do if path is already versioned.
+    # This is safe from infinite recursion because the branch root is
+    # always versioned.
+    if inv.path2id(path) != None:
+        return 0
+
+    # add parent
+    count = __add_one(branch, inv, dirname(path), 'directory', reporter)
+
+    entry = inv.add_path(path, kind=kind)
+    mutter("added %r kind %r file_id={%s}" % (path, kind, entry.file_id))
+    reporter(path, kind, entry)
+
+    return count + 1

=== added file 'bzrlib/annotate.py'
--- bzrlib/annotate.py	1970-01-01 00:00:00 +0000
+++ bzrlib/annotate.py	2005-09-30 14:27:41 +0000
@@ -0,0 +1,55 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""File annotate based on weave storage"""
+
+import sys
+import os
+
+import bzrlib.weave
+
+def annotate_file(branch, rev_id, file_id, to_file=None):
+    if to_file is None:
+        to_file = sys.stdout
+    rh = branch.revision_history()
+    w = branch.weave_store.get_weave(file_id)
+    last_origin = None
+    for origin, text in w.annotate_iter(rev_id):
+        text = text.rstrip('\r\n')
+        if origin == last_origin:
+            print '      | %s' % (text)
+        else:
+            last_origin = origin
+            line_rev_id = w.idx_to_name(origin)
+            try:
+                revno = rh.index(line_rev_id) + 1
+                print '%5d | %s' % (revno, text)
+            except ValueError:
+                print 'merge | %s' % (text)
+
+
+
+if __name__ == '__main__':
+    from bzrlib.branch import find_branch
+    from bzrlib.trace import enable_default_logging
+
+    enable_default_logging()
+    b = find_branch(sys.argv[1])
+    rp = b.relpath(sys.argv[1])
+    tree = b.revision_tree(b.last_revision())
+    file_id = tree.inventory.path2id(rp)
+    file_version = tree.inventory[file_id].revision
+    annotate_file(b, file_version, file_id, sys.stdout)

=== modified file 'bzrlib/atomicfile.py'
--- bzrlib/atomicfile.py	2005-05-26 16:09:45 +0000
+++ bzrlib/atomicfile.py	2005-09-29 20:34:25 +0000
@@ -17,7 +17,7 @@
 
 
 from warnings import warn
-
+from osutils import rename
 
 class AtomicFile(object):
     """A file that does an atomic-rename to move into place.
@@ -65,16 +65,7 @@
         self.f.close()
         self.f = None
         
-        if sys.platform == 'win32':
-            # windows cannot rename over an existing file
-            try:
-                os.remove(self.realfilename)
-            except OSError, e:
-                import errno
-                if e.errno != errno.ENOENT:
-                    raise
-                
-        os.rename(self.tmpfilename, self.realfilename)
+        rename(self.tmpfilename, self.realfilename)
 
 
     def abort(self):
@@ -97,6 +88,6 @@
 
 
     def __del__(self):
-        if not self.closed:
+        if hasattr(self, 'closed') and not self.closed:
             warn("%r leaked" % self)
         

=== modified file 'bzrlib/branch.py'
--- bzrlib/branch.py	2005-07-22 22:31:40 +0000
+++ bzrlib/branch.py	2005-10-08 00:49:13 +0000
@@ -15,44 +15,54 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-import sys, os
+import sys
+import os
+import errno
+from warnings import warn
+from cStringIO import StringIO
+
 
 import bzrlib
+from bzrlib.inventory import InventoryEntry
+import bzrlib.inventory as inventory
 from bzrlib.trace import mutter, note
-from bzrlib.osutils import isdir, quotefn, compact_date, rand_bytes, splitpath, \
-     sha_file, appendpath, file_kind
-from bzrlib.errors import BzrError
-
-BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
+from bzrlib.osutils import (isdir, quotefn, compact_date, rand_bytes, 
+                            rename, splitpath, sha_file, appendpath, 
+                            file_kind)
+import bzrlib.errors as errors
+from bzrlib.errors import (BzrError, InvalidRevisionNumber, InvalidRevisionId,
+                           NoSuchRevision, HistoryMissing, NotBranchError,
+                           DivergedBranches, LockError, UnlistableStore,
+                           UnlistableBranch, NoSuchFile)
+from bzrlib.textui import show_status
+from bzrlib.revision import Revision
+from bzrlib.delta import compare_trees
+from bzrlib.tree import EmptyTree, RevisionTree
+from bzrlib.inventory import Inventory
+from bzrlib.store import copy_all
+from bzrlib.store.compressed_text import CompressedTextStore
+from bzrlib.store.text import TextStore
+from bzrlib.store.weave import WeaveStore
+import bzrlib.transactions as transactions
+from bzrlib.transport import Transport, get_transport
+import bzrlib.xml5
+import bzrlib.ui
+
+
+BZR_BRANCH_FORMAT_4 = "Bazaar-NG branch, format 0.0.4\n"
+BZR_BRANCH_FORMAT_5 = "Bazaar-NG branch, format 5\n"
 ## TODO: Maybe include checks for common corruption of newlines, etc?
 
 
-
-def find_branch(f, **args):
-    if f and (f.startswith('http://') or f.startswith('https://')):
-        import remotebranch 
-        return remotebranch.RemoteBranch(f, **args)
-    else:
-        return Branch(f, **args)
-
-
-def find_cached_branch(f, cache_root, **args):
-    from remotebranch import RemoteBranch
-    br = find_branch(f, **args)
-    def cacheify(br, store_name):
-        from meta_store import CachedStore
-        cache_path = os.path.join(cache_root, store_name)
-        os.mkdir(cache_path)
-        new_store = CachedStore(getattr(br, store_name), cache_path)
-        setattr(br, store_name, new_store)
-
-    if isinstance(br, RemoteBranch):
-        cacheify(br, 'inventory_store')
-        cacheify(br, 'text_store')
-        cacheify(br, 'revision_store')
-    return br
-
-
+# TODO: Some operations like log might retrieve the same revisions
+# repeatedly to calculate deltas.  We could perhaps have a weakref
+# cache in memory to make this faster.  In general anything can be
+# cached in memory between lock and unlock operations.
+
+def find_branch(*ignored, **ignored_too):
+    # XXX: leave this here for about one release, then remove it
+    raise NotImplementedError('find_branch() is not supported anymore, '
+                              'please use one of the new branch constructors')
 def _relpath(base, path):
     """Return path relative to base, or raise exception.
 
@@ -75,55 +85,30 @@
         if tail:
             s.insert(0, tail)
     else:
-        from errors import NotBranchError
         raise NotBranchError("path %r is not within branch %r" % (rp, base))
 
     return os.sep.join(s)
         
 
-def find_branch_root(f=None):
-    """Find the branch root enclosing f, or pwd.
-
-    f may be a filename or a URL.
-
-    It is not necessary that f exists.
+def find_branch_root(t):
+    """Find the branch root enclosing the transport's base.
+
+    t is a Transport object.
+
+    It is not necessary that the base of t exists.
 
     Basically we keep looking up until we find the control directory or
-    run into the root."""
-    if f == None:
-        f = os.getcwd()
-    elif hasattr(os.path, 'realpath'):
-        f = os.path.realpath(f)
-    else:
-        f = os.path.abspath(f)
-    if not os.path.exists(f):
-        raise BzrError('%r does not exist' % f)
-        
-
-    orig_f = f
-
+    run into the root.  If there isn't one, raises NotBranchError.
+    """
+    orig_base = t.base
     while True:
-        if os.path.exists(os.path.join(f, bzrlib.BZRDIR)):
-            return f
-        head, tail = os.path.split(f)
-        if head == f:
+        if t.has(bzrlib.BZRDIR):
+            return t
+        new_t = t.clone('..')
+        if new_t.base == t.base:
             # reached the root, whatever that may be
-            raise BzrError('%r is not in a branch' % orig_f)
-        f = head
-    
-class DivergedBranches(Exception):
-    def __init__(self, branch1, branch2):
-        self.branch1 = branch1
-        self.branch2 = branch2
-        Exception.__init__(self, "These branches have diverged.")
-
-
-class NoSuchRevision(BzrError):
-    def __init__(self, branch, revision):
-        self.branch = branch
-        self.revision = revision
-        msg = "Branch %s has no revision %d" % (branch, revision)
-        BzrError.__init__(self, msg)
+            raise NotBranchError('%s is not in a branch' % orig_base)
+        t = new_t
 
 
 ######################################################################
@@ -133,7 +118,56 @@
     """Branch holding a history of revisions.
 
     base
-        Base directory of the branch.
+        Base directory/url of the branch.
+    """
+    base = None
+
+    def __init__(self, *ignored, **ignored_too):
+        raise NotImplementedError('The Branch class is abstract')
+
+    @staticmethod
+    def open_downlevel(base):
+        """Open a branch which may be of an old format.
+        
+        Only local branches are supported."""
+        return _Branch(get_transport(base), relax_version_check=True)
+        
+    @staticmethod
+    def open(base):
+        """Open an existing branch, rooted at 'base' (url)"""
+        t = get_transport(base)
+        mutter("trying to open %r with transport %r", base, t)
+        return _Branch(t)
+
+    @staticmethod
+    def open_containing(url):
+        """Open an existing branch which contains url.
+        
+        This probes for a branch at url, and searches upwards from there.
+        """
+        t = get_transport(url)
+        t = find_branch_root(t)
+        return _Branch(t)
+
+    @staticmethod
+    def initialize(base):
+        """Create a new branch, rooted at 'base' (url)"""
+        t = get_transport(base)
+        return _Branch(t, init=True)
+
+    def setup_caching(self, cache_root):
+        """Subclasses that care about caching should override this, and set
+        up cached stores located under cache_root.
+        """
+        self.cache_root = cache_root
+
+
+class _Branch(Branch):
+    """A branch stored in the actual filesystem.
+
+    Note that it's "local" in the context of the filesystem; it doesn't
+    really matter if it's on an nfs/smb/afs/coda/... share, as long as
+    it's writable, and can be accessed via the normal filesystem API.
 
     _lock_mode
         None, or 'r' or 'w'
@@ -145,53 +179,101 @@
     _lock
         Lock object from bzrlib.lock.
     """
-    base = None
+    # We actually expect this class to be somewhat short-lived; part of its
+    # purpose is to try to isolate what bits of the branch logic are tied to
+    # filesystem access, so that in a later step, we can extricate them to
+    # a separarte ("storage") class.
     _lock_mode = None
     _lock_count = None
     _lock = None
+    _inventory_weave = None
     
     # Map some sort of prefix into a namespace
     # stuff like "revno:10", "revid:", etc.
     # This should match a prefix with a function which accepts
     REVISION_NAMESPACES = {}
 
-    def __init__(self, base, init=False, find_root=True):
+    def push_stores(self, branch_to):
+        """Copy the content of this branches store to branch_to."""
+        if (self._branch_format != branch_to._branch_format
+            or self._branch_format != 4):
+            from bzrlib.fetch import greedy_fetch
+            mutter("falling back to fetch logic to push between %s(%s) and %s(%s)",
+                   self, self._branch_format, branch_to, branch_to._branch_format)
+            greedy_fetch(to_branch=branch_to, from_branch=self,
+                         revision=self.last_revision())
+            return
+
+        store_pairs = ((self.text_store,      branch_to.text_store),
+                       (self.inventory_store, branch_to.inventory_store),
+                       (self.revision_store,  branch_to.revision_store))
+        try:
+            for from_store, to_store in store_pairs: 
+                copy_all(from_store, to_store)
+        except UnlistableStore:
+            raise UnlistableBranch(from_store)
+
+    def __init__(self, transport, init=False,
+                 relax_version_check=False):
         """Create new branch object at a particular location.
 
-        base -- Base directory for the branch.
+        transport -- A Transport object, defining how to access files.
+                (If a string, transport.transport() will be used to
+                create a Transport object)
         
         init -- If True, create new control files in a previously
              unversioned directory.  If False, the branch must already
              be versioned.
 
-        find_root -- If true and init is false, find the root of the
-             existing branch containing base.
+        relax_version_check -- If true, the usual check for the branch
+            version is not applied.  This is intended only for
+            upgrade/recovery type use; it's not guaranteed that
+            all operations will work on old format branches.
 
         In the test suite, creation of new trees is tested using the
         `ScratchBranch` class.
         """
-        from bzrlib.store import ImmutableStore
+        assert isinstance(transport, Transport), \
+            "%r is not a Transport" % transport
+        self._transport = transport
         if init:
-            self.base = os.path.realpath(base)
             self._make_control()
-        elif find_root:
-            self.base = find_branch_root(base)
-        else:
-            self.base = os.path.realpath(base)
-            if not isdir(self.controlfilename('.')):
-                from errors import NotBranchError
-                raise NotBranchError("not a bzr branch: %s" % quotefn(base),
-                                     ['use "bzr init" to initialize a new working tree',
-                                      'current bzr can only operate from top-of-tree'])
-        self._check_format()
-
-        self.text_store = ImmutableStore(self.controlfilename('text-store'))
-        self.revision_store = ImmutableStore(self.controlfilename('revision-store'))
-        self.inventory_store = ImmutableStore(self.controlfilename('inventory-store'))
-
+        self._check_format(relax_version_check)
+
+        def get_store(name, compressed=True):
+            # FIXME: This approach of assuming stores are all entirely compressed
+            # or entirely uncompressed is tidy, but breaks upgrade from 
+            # some existing branches where there's a mixture; we probably 
+            # still want the option to look for both.
+            relpath = self._rel_controlfilename(name)
+            if compressed:
+                store = CompressedTextStore(self._transport.clone(relpath))
+            else:
+                store = TextStore(self._transport.clone(relpath))
+            #if self._transport.should_cache():
+            #    cache_path = os.path.join(self.cache_root, name)
+            #    os.mkdir(cache_path)
+            #    store = bzrlib.store.CachedStore(store, cache_path)
+            return store
+        def get_weave(name):
+            relpath = self._rel_controlfilename(name)
+            ws = WeaveStore(self._transport.clone(relpath))
+            if self._transport.should_cache():
+                ws.enable_cache = True
+            return ws
+
+        if self._branch_format == 4:
+            self.inventory_store = get_store('inventory-store')
+            self.text_store = get_store('text-store')
+            self.revision_store = get_store('revision-store')
+        elif self._branch_format == 5:
+            self.control_weaves = get_weave([])
+            self.weave_store = get_weave('weaves')
+            self.revision_store = get_store('revision-store', compressed=False)
+        self._transaction = None
 
     def __str__(self):
-        return '%s(%r)' % (self.__class__.__name__, self.base)
+        return '%s(%r)' % (self.__class__.__name__, self._transport.base)
 
 
     __repr__ = __str__
@@ -199,26 +281,73 @@
 
     def __del__(self):
         if self._lock_mode or self._lock:
-            from warnings import warn
+            # XXX: This should show something every time, and be suitable for
+            # headless operation and embedding
             warn("branch %r was not explicitly unlocked" % self)
             self._lock.unlock()
 
-
+        # TODO: It might be best to do this somewhere else,
+        # but it is nice for a Branch object to automatically
+        # cache it's information.
+        # Alternatively, we could have the Transport objects cache requests
+        # See the earlier discussion about how major objects (like Branch)
+        # should never expect their __del__ function to run.
+        if hasattr(self, 'cache_root') and self.cache_root is not None:
+            try:
+                import shutil
+                shutil.rmtree(self.cache_root)
+            except:
+                pass
+            self.cache_root = None
+
+    def _get_base(self):
+        if self._transport:
+            return self._transport.base
+        return None
+
+    base = property(_get_base)
+
+    def _finish_transaction(self):
+        """Exit the current transaction."""
+        if self._transaction is None:
+            raise errors.LockError('Branch %s is not in a transaction' %
+                                   self)
+        transaction = self._transaction
+        self._transaction = None
+        transaction.finish()
+
+    def get_transaction(self):
+        """Return the current active transaction.
+
+        If no transaction is active, this returns a passthrough object
+        for which all data is immedaitely flushed and no caching happens.
+        """
+        if self._transaction is None:
+            return transactions.PassThroughTransaction()
+        else:
+            return self._transaction
+
+    def _set_transaction(self, new_transaction):
+        """Set a new active transaction."""
+        if self._transaction is not None:
+            raise errors.LockError('Branch %s is in a transaction already.' %
+                                   self)
+        self._transaction = new_transaction
 
     def lock_write(self):
+        # TODO: Upgrade locking to support using a Transport,
+        # and potentially a remote locking protocol
         if self._lock_mode:
             if self._lock_mode != 'w':
-                from errors import LockError
                 raise LockError("can't upgrade to a write lock from %r" %
                                 self._lock_mode)
             self._lock_count += 1
         else:
-            from bzrlib.lock import WriteLock
-
-            self._lock = WriteLock(self.controlfilename('branch-lock'))
+            self._lock = self._transport.lock_write(
+                    self._rel_controlfilename('branch-lock'))
             self._lock_mode = 'w'
             self._lock_count = 1
-
+            self._set_transaction(transactions.PassThroughTransaction())
 
 
     def lock_read(self):
@@ -227,44 +356,43 @@
                    "invalid lock mode %r" % self._lock_mode
             self._lock_count += 1
         else:
-            from bzrlib.lock import ReadLock
-
-            self._lock = ReadLock(self.controlfilename('branch-lock'))
+            self._lock = self._transport.lock_read(
+                    self._rel_controlfilename('branch-lock'))
             self._lock_mode = 'r'
             self._lock_count = 1
+            self._set_transaction(transactions.ReadOnlyTransaction())
                         
-
-            
     def unlock(self):
         if not self._lock_mode:
-            from errors import LockError
             raise LockError('branch %r is not locked' % (self))
 
         if self._lock_count > 1:
             self._lock_count -= 1
         else:
+            self._finish_transaction()
             self._lock.unlock()
             self._lock = None
             self._lock_mode = self._lock_count = None
 
-
     def abspath(self, name):
         """Return absolute filename for something in the branch"""
-        return os.path.join(self.base, name)
-
+        return self._transport.abspath(name)
 
     def relpath(self, path):
         """Return path relative to this branch of something inside it.
 
         Raises an error if path is not in this branch."""
-        return _relpath(self.base, path)
-
+        return self._transport.relpath(path)
+
+
+    def _rel_controlfilename(self, file_or_path):
+        if isinstance(file_or_path, basestring):
+            file_or_path = [file_or_path]
+        return [bzrlib.BZRDIR] + file_or_path
 
     def controlfilename(self, file_or_path):
         """Return location relative to branch."""
-        if isinstance(file_or_path, basestring):
-            file_or_path = [file_or_path]
-        return os.path.join(self.base, bzrlib.BZRDIR, *file_or_path)
+        return self._transport.abspath(self._rel_controlfilename(file_or_path))
 
 
     def controlfile(self, file_or_path, mode='r'):
@@ -278,60 +406,111 @@
         Controlfiles should almost never be opened in write mode but
         rather should be atomically copied and replaced using atomicfile.
         """
-
-        fn = self.controlfilename(file_or_path)
-
-        if mode == 'rb' or mode == 'wb':
-            return file(fn, mode)
-        elif mode == 'r' or mode == 'w':
-            # open in binary mode anyhow so there's no newline translation;
-            # codecs uses line buffering by default; don't want that.
-            import codecs
-            return codecs.open(fn, mode + 'b', 'utf-8',
-                               buffering=60000)
+        import codecs
+
+        relpath = self._rel_controlfilename(file_or_path)
+        #TODO: codecs.open() buffers linewise, so it was overloaded with
+        # a much larger buffer, do we need to do the same for getreader/getwriter?
+        if mode == 'rb': 
+            return self._transport.get(relpath)
+        elif mode == 'wb':
+            raise BzrError("Branch.controlfile(mode='wb') is not supported, use put_controlfiles")
+        elif mode == 'r':
+            return codecs.getreader('utf-8')(self._transport.get(relpath), errors='replace')
+        elif mode == 'w':
+            raise BzrError("Branch.controlfile(mode='w') is not supported, use put_controlfiles")
         else:
             raise BzrError("invalid controlfile mode %r" % mode)
 
-
+    def put_controlfile(self, path, f, encode=True):
+        """Write an entry as a controlfile.
+
+        :param path: The path to put the file, relative to the .bzr control
+                     directory
+        :param f: A file-like or string object whose contents should be copied.
+        :param encode:  If true, encode the contents as utf-8
+        """
+        self.put_controlfiles([(path, f)], encode=encode)
+
+    def put_controlfiles(self, files, encode=True):
+        """Write several entries as controlfiles.
+
+        :param files: A list of [(path, file)] pairs, where the path is the directory
+                      underneath the bzr control directory
+        :param encode:  If true, encode the contents as utf-8
+        """
+        import codecs
+        ctrl_files = []
+        for path, f in files:
+            if encode:
+                if isinstance(f, basestring):
+                    f = f.encode('utf-8', 'replace')
+                else:
+                    f = codecs.getwriter('utf-8')(f, errors='replace')
+            path = self._rel_controlfilename(path)
+            ctrl_files.append((path, f))
+        self._transport.put_multi(ctrl_files)
 
     def _make_control(self):
         from bzrlib.inventory import Inventory
-        from bzrlib.xml import pack_xml
+        from bzrlib.weavefile import write_weave_v5
+        from bzrlib.weave import Weave
         
-        os.mkdir(self.controlfilename([]))
-        self.controlfile('README', 'w').write(
+        # Create an empty inventory
+        sio = StringIO()
+        # if we want per-tree root ids then this is the place to set
+        # them; they're not needed for now and so ommitted for
+        # simplicity.
+        bzrlib.xml5.serializer_v5.write_inventory(Inventory(), sio)
+        empty_inv = sio.getvalue()
+        sio = StringIO()
+        bzrlib.weavefile.write_weave_v5(Weave(), sio)
+        empty_weave = sio.getvalue()
+
+        dirs = [[], 'revision-store', 'weaves']
+        files = [('README', 
             "This is a Bazaar-NG control directory.\n"
-            "Do not change any files in this directory.\n")
-        self.controlfile('branch-format', 'w').write(BZR_BRANCH_FORMAT)
-        for d in ('text-store', 'inventory-store', 'revision-store'):
-            os.mkdir(self.controlfilename(d))
-        for f in ('revision-history', 'merged-patches',
-                  'pending-merged-patches', 'branch-name',
-                  'branch-lock',
-                  'pending-merges'):
-            self.controlfile(f, 'w').write('')
-        mutter('created control directory in ' + self.base)
-
-        pack_xml(Inventory(gen_root_id()), self.controlfile('inventory','w'))
-
-
-    def _check_format(self):
+            "Do not change any files in this directory.\n"),
+            ('branch-format', BZR_BRANCH_FORMAT_5),
+            ('revision-history', ''),
+            ('branch-name', ''),
+            ('branch-lock', ''),
+            ('pending-merges', ''),
+            ('inventory', empty_inv),
+            ('inventory.weave', empty_weave),
+            ('ancestry.weave', empty_weave)
+        ]
+        cfn = self._rel_controlfilename
+        self._transport.mkdir_multi([cfn(d) for d in dirs])
+        self.put_controlfiles(files)
+        mutter('created control directory in ' + self._transport.base)
+
+    def _check_format(self, relax_version_check):
         """Check this branch format is supported.
 
-        The current tool only supports the current unstable format.
+        The format level is stored, as an integer, in
+        self._branch_format for code that needs to check it later.
 
         In the future, we might need different in-memory Branch
         classes to support downlevel branches.  But not yet.
         """
-        # This ignores newlines so that we can open branches created
-        # on Windows from Linux and so on.  I think it might be better
-        # to always make all internal files in unix format.
-        fmt = self.controlfile('branch-format', 'r').read()
-        fmt.replace('\r\n', '')
-        if fmt != BZR_BRANCH_FORMAT:
-            raise BzrError('sorry, branch format %r not supported' % fmt,
+        try:
+            fmt = self.controlfile('branch-format', 'r').read()
+        except NoSuchFile:
+            raise NotBranchError(self.base)
+        mutter("got branch format %r", fmt)
+        if fmt == BZR_BRANCH_FORMAT_5:
+            self._branch_format = 5
+        elif fmt == BZR_BRANCH_FORMAT_4:
+            self._branch_format = 4
+
+        if (not relax_version_check
+            and self._branch_format != 5):
+            raise errors.UnsupportedFormatError(
+                           'sorry, branch format %r not supported' % fmt,
                            ['use a different bzr version',
-                            'or remove the .bzr directory and "bzr init" again'])
+                            'or remove the .bzr directory'
+                            ' and "bzr init" again'])
 
     def get_root_id(self):
         """Return the id of this branches root"""
@@ -352,19 +531,12 @@
 
     def read_working_inventory(self):
         """Read the working inventory."""
-        from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-        from time import time
-        before = time()
         self.lock_read()
         try:
             # ElementTree does its own conversion from UTF-8, so open in
             # binary.
-            inv = unpack_xml(Inventory,
-                             self.controlfile('inventory', 'rb'))
-            mutter("loaded inventory of %d items in %f"
-                   % (len(inv), time() - before))
-            return inv
+            f = self.controlfile('inventory', 'rb')
+            return bzrlib.xml5.serializer_v5.read_inventory(f)
         finally:
             self.unlock()
             
@@ -375,31 +547,27 @@
         That is to say, the inventory describing changes underway, that
         will be committed to the next revision.
         """
-        from bzrlib.atomicfile import AtomicFile
-        from bzrlib.xml import pack_xml
-        
+        from cStringIO import StringIO
         self.lock_write()
         try:
-            f = AtomicFile(self.controlfilename('inventory'), 'wb')
-            try:
-                pack_xml(inv, f)
-                f.commit()
-            finally:
-                f.close()
+            sio = StringIO()
+            bzrlib.xml5.serializer_v5.write_inventory(inv, sio)
+            sio.seek(0)
+            # Transport handles atomicity
+            self.put_controlfile('inventory', sio)
         finally:
             self.unlock()
         
         mutter('wrote working inventory')
             
-
     inventory = property(read_working_inventory, _write_inventory, None,
                          """Inventory for the working copy.""")
 
-
-    def add(self, files, verbose=False, ids=None):
+    def add(self, files, ids=None):
         """Make files versioned.
 
-        Note that the command line normally calls smart_add instead.
+        Note that the command line normally calls smart_add instead,
+        which can automatically recurse.
 
         This puts the files in the Added state, so that they will be
         recorded by the next commit.
@@ -415,14 +583,8 @@
         TODO: Perhaps have an option to add the ids even if the files do
               not (yet) exist.
 
-        TODO: Perhaps return the ids of the files?  But then again it
-              is easy to retrieve them if they're needed.
-
-        TODO: Adding a directory should optionally recurse down and
-              add all non-ignored children.  Perhaps do that in a
-              higher-level method.
+        TODO: Perhaps yield the ids and paths as they're added.
         """
-        from bzrlib.textui import show_status
         # TODO: Re-adding a file that is removed in the working copy
         # should probably put it back with the previous ID.
         if isinstance(files, basestring):
@@ -454,18 +616,16 @@
                     kind = file_kind(fullpath)
                 except OSError:
                     # maybe something better?
-                    raise BzrError('cannot add: not a regular file or directory: %s' % quotefn(f))
+                    raise BzrError('cannot add: not a regular file, symlink or directory: %s' % quotefn(f))
 
-                if kind != 'file' and kind != 'directory':
-                    raise BzrError('cannot add: not a regular file or directory: %s' % quotefn(f))
+                if not InventoryEntry.versionable_kind(kind):
+                    raise BzrError('cannot add: not a versionable file ('
+                                   'i.e. regular file, symlink or directory): %s' % quotefn(f))
 
                 if file_id is None:
                     file_id = gen_file_id(f)
                 inv.add_path(f, kind=kind, file_id=file_id)
 
-                if verbose:
-                    print 'added', quotefn(f)
-
                 mutter("add file %s file_id:{%s} kind=%r" % (f, file_id, kind))
 
             self._write_inventory(inv)
@@ -477,7 +637,7 @@
         """Print `file` to stdout."""
         self.lock_read()
         try:
-            tree = self.revision_tree(self.lookup_revision(revno))
+            tree = self.revision_tree(self.get_rev_id(revno))
             # use inventory as it was in that revision
             file_id = tree.inventory.path2id(file)
             if not file_id:
@@ -501,7 +661,6 @@
         is the opposite of add.  Removing it is consistent with most
         other tools.  Maybe an option.
         """
-        from bzrlib.textui import show_status
         ## TODO: Normalize names
         ## TODO: Remove nested loops; better scalability
         if isinstance(files, basestring):
@@ -532,7 +691,6 @@
         finally:
             self.unlock()
 
-
     # FIXME: this doesn't need to be a branch method
     def set_inventory(self, new_inventory_list):
         from bzrlib.inventory import Inventory, InventoryEntry
@@ -541,10 +699,17 @@
             name = os.path.basename(path)
             if name == "":
                 continue
-            inv.add(InventoryEntry(file_id, name, kind, parent))
+            # fixme, there should be a factory function inv,add_?? 
+            if kind == 'directory':
+                inv.add(inventory.InventoryDirectory(file_id, name, parent))
+            elif kind == 'file':
+                inv.add(inventory.InventoryFile(file_id, name, parent))
+            elif kind == 'symlink':
+                inv.add(inventory.InventoryLink(file_id, name, parent))
+            else:
+                raise BzrError("unknown kind %r" % kind)
         self._write_inventory(inv)
 
-
     def unknowns(self):
         """Return all unknown files.
 
@@ -565,39 +730,79 @@
 
 
     def append_revision(self, *revision_ids):
-        from bzrlib.atomicfile import AtomicFile
-
         for revision_id in revision_ids:
             mutter("add {%s} to revision-history" % revision_id)
-
-        rev_history = self.revision_history()
-        rev_history.extend(revision_ids)
-
-        f = AtomicFile(self.controlfilename('revision-history'))
-        try:
-            for rev_id in rev_history:
-                print >>f, rev_id
-            f.commit()
-        finally:
-            f.close()
+        self.lock_write()
+        try:
+            rev_history = self.revision_history()
+            rev_history.extend(revision_ids)
+            self.put_controlfile('revision-history', '\n'.join(rev_history))
+        finally:
+            self.unlock()
+
+    def has_revision(self, revision_id):
+        """True if this branch has a copy of the revision.
+
+        This does not necessarily imply the revision is merge
+        or on the mainline."""
+        return (revision_id is None
+                or revision_id in self.revision_store)
+
+    def get_revision_xml_file(self, revision_id):
+        """Return XML file object for revision object."""
+        if not revision_id or not isinstance(revision_id, basestring):
+            raise InvalidRevisionId(revision_id)
+
+        self.lock_read()
+        try:
+            try:
+                return self.revision_store[revision_id]
+            except (IndexError, KeyError):
+                raise bzrlib.errors.NoSuchRevision(self, revision_id)
+        finally:
+            self.unlock()
+
+    #deprecated
+    get_revision_xml = get_revision_xml_file
+
+    def get_revision_xml(self, revision_id):
+        return self.get_revision_xml_file(revision_id).read()
 
 
     def get_revision(self, revision_id):
         """Return the Revision object for a named revision"""
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
+        xml_file = self.get_revision_xml_file(revision_id)
 
-        self.lock_read()
         try:
-            if not revision_id or not isinstance(revision_id, basestring):
-                raise ValueError('invalid revision-id: %r' % revision_id)
-            r = unpack_xml(Revision, self.revision_store[revision_id])
-        finally:
-            self.unlock()
+            r = bzrlib.xml5.serializer_v5.read_revision(xml_file)
+        except SyntaxError, e:
+            raise bzrlib.errors.BzrError('failed to unpack revision_xml',
+                                         [revision_id,
+                                          str(e)])
             
         assert r.revision_id == revision_id
         return r
-        
+
+    def get_revision_delta(self, revno):
+        """Return the delta for one revision.
+
+        The delta is relative to its mainline predecessor, or the
+        empty tree for revision 1.
+        """
+        assert isinstance(revno, int)
+        rh = self.revision_history()
+        if not (1 <= revno <= len(rh)):
+            raise InvalidRevisionNumber(revno)
+
+        # revno is 1-based; list is 0-based
+
+        new_tree = self.revision_tree(rh[revno-1])
+        if revno == 1:
+            old_tree = EmptyTree()
+        else:
+            old_tree = self.revision_tree(rh[revno-2])
+
+        return compare_trees(old_tree, new_tree)
 
     def get_revision_sha1(self, revision_id):
         """Hash the stored value of a revision, and return it."""
@@ -607,44 +812,55 @@
         # the revision, (add signatures/remove signatures) and still
         # have all hash pointers stay consistent.
         # But for now, just hash the contents.
-        return sha_file(self.revision_store[revision_id])
-
-
-    def get_inventory(self, inventory_id):
-        """Get Inventory object by hash.
-
-        TODO: Perhaps for this and similar methods, take a revision
-               parameter which can be either an integer revno or a
-               string hash."""
-        from bzrlib.inventory import Inventory
-        from bzrlib.xml import unpack_xml
-
-        return unpack_xml(Inventory, self.inventory_store[inventory_id])
-            
-
-    def get_inventory_sha1(self, inventory_id):
+        return bzrlib.osutils.sha_file(self.get_revision_xml_file(revision_id))
+
+    def get_ancestry(self, revision_id):
+        """Return a list of revision-ids integrated by a revision.
+        
+        This currently returns a list, but the ordering is not guaranteed:
+        treat it as a set.
+        """
+        if revision_id is None:
+            return [None]
+        w = self.get_inventory_weave()
+        return [None] + map(w.idx_to_name,
+                            w.inclusions([w.lookup(revision_id)]))
+
+    def get_inventory_weave(self):
+        return self.control_weaves.get_weave('inventory',
+                                             self.get_transaction())
+
+    def get_inventory(self, revision_id):
+        """Get Inventory object by hash."""
+        xml = self.get_inventory_xml(revision_id)
+        return bzrlib.xml5.serializer_v5.read_inventory_from_string(xml)
+
+    def get_inventory_xml(self, revision_id):
+        """Get inventory XML as a file object."""
+        try:
+            assert isinstance(revision_id, basestring), type(revision_id)
+            iw = self.get_inventory_weave()
+            return iw.get_text(iw.lookup(revision_id))
+        except IndexError:
+            raise bzrlib.errors.HistoryMissing(self, 'inventory', revision_id)
+
+    def get_inventory_sha1(self, revision_id):
         """Return the sha1 hash of the inventory entry
         """
-        return sha_file(self.inventory_store[inventory_id])
-
+        return self.get_revision(revision_id).inventory_sha1
 
     def get_revision_inventory(self, revision_id):
         """Return inventory of a past revision."""
-        # bzr 0.0.6 imposes the constraint that the inventory_id
+        # TODO: Unify this with get_inventory()
+        # bzr 0.0.6 and later imposes the constraint that the inventory_id
         # must be the same as its revision, so this is trivial.
         if revision_id == None:
-            from bzrlib.inventory import Inventory
             return Inventory(self.get_root_id())
         else:
             return self.get_inventory(revision_id)
 
-
     def revision_history(self):
-        """Return sequence of revision hashes on to this branch.
-
-        >>> ScratchBranch().revision_history()
-        []
-        """
+        """Return sequence of revision hashes on to this branch."""
         self.lock_read()
         try:
             return [l.rstrip('\r\n') for l in
@@ -652,23 +868,22 @@
         finally:
             self.unlock()
 
-
     def common_ancestor(self, other, self_revno=None, other_revno=None):
         """
-        >>> import commit
+        >>> from bzrlib.commit import commit
         >>> sb = ScratchBranch(files=['foo', 'foo~'])
         >>> sb.common_ancestor(sb) == (None, None)
         True
-        >>> commit.commit(sb, "Committing first revision", verbose=False)
+        >>> commit(sb, "Committing first revision", verbose=False)
         >>> sb.common_ancestor(sb)[0]
         1
         >>> clone = sb.clone()
-        >>> commit.commit(sb, "Committing second revision", verbose=False)
+        >>> commit(sb, "Committing second revision", verbose=False)
         >>> sb.common_ancestor(sb)[0]
         2
         >>> sb.common_ancestor(clone)[0]
         1
-        >>> commit.commit(clone, "Committing divergent second revision", 
+        >>> commit(clone, "Committing divergent second revision", 
         ...               verbose=False)
         >>> sb.common_ancestor(clone)[0]
         1
@@ -697,27 +912,6 @@
                 return r+1, my_history[r]
         return None, None
 
-    def enum_history(self, direction):
-        """Return (revno, revision_id) for history of branch.
-
-        direction
-            'forward' is from earliest to latest
-            'reverse' is from latest to earliest
-        """
-        rh = self.revision_history()
-        if direction == 'forward':
-            i = 1
-            for rid in rh:
-                yield i, rid
-                i += 1
-        elif direction == 'reverse':
-            i = len(rh)
-            while i > 0:
-                yield i, rh[i-1]
-                i -= 1
-        else:
-            raise ValueError('invalid history direction', direction)
-
 
     def revno(self):
         """Return current revision number for this branch.
@@ -728,7 +922,7 @@
         return len(self.revision_history())
 
 
-    def last_patch(self):
+    def last_revision(self):
         """Return last patch hash, or None if no history.
         """
         ph = self.revision_history()
@@ -738,8 +932,9 @@
             return None
 
 
-    def missing_revisions(self, other, stop_revision=None):
-        """
+    def missing_revisions(self, other, stop_revision=None, diverged_ok=False):
+        """Return a list of new revisions that would perfectly fit.
+        
         If self and other have not diverged, return a list of the revisions
         present in other, but missing from self.
 
@@ -765,6 +960,9 @@
         Traceback (most recent call last):
         DivergedBranches: These branches have diverged.
         """
+        # FIXME: If the branches have diverged, but the latest
+        # revision in this branch is completely merged into the other,
+        # then we should still be able to pull.
         self_history = self.revision_history()
         self_len = len(self_history)
         other_history = other.revision_history()
@@ -776,262 +974,76 @@
 
         if stop_revision is None:
             stop_revision = other_len
-        elif stop_revision > other_len:
-            raise NoSuchRevision(self, stop_revision)
-        
+        else:
+            assert isinstance(stop_revision, int)
+            if stop_revision > other_len:
+                raise bzrlib.errors.NoSuchRevision(self, stop_revision)
         return other_history[self_len:stop_revision]
 
-
     def update_revisions(self, other, stop_revision=None):
-        """Pull in all new revisions from other branch.
-        
-        >>> from bzrlib.commit import commit
-        >>> bzrlib.trace.silent = True
-        >>> br1 = ScratchBranch(files=['foo', 'bar'])
-        >>> br1.add('foo')
-        >>> br1.add('bar')
-        >>> commit(br1, "lala!", rev_id="REVISION-ID-1", verbose=False)
-        >>> br2 = ScratchBranch()
-        >>> br2.update_revisions(br1)
-        Added 2 texts.
-        Added 1 inventories.
-        Added 1 revisions.
-        >>> br2.revision_history()
-        [u'REVISION-ID-1']
-        >>> br2.update_revisions(br1)
-        Added 0 texts.
-        Added 0 inventories.
-        Added 0 revisions.
-        >>> br1.text_store.total_size() == br2.text_store.total_size()
-        True
-        """
-        from bzrlib.progress import ProgressBar
-
-        pb = ProgressBar()
-
-        pb.update('comparing histories')
-        revision_ids = self.missing_revisions(other, stop_revision)
-
-        if hasattr(other.revision_store, "prefetch"):
-            other.revision_store.prefetch(revision_ids)
-        if hasattr(other.inventory_store, "prefetch"):
-            inventory_ids = [other.get_revision(r).inventory_id
-                             for r in revision_ids]
-            other.inventory_store.prefetch(inventory_ids)
-                
-        revisions = []
-        needed_texts = set()
-        i = 0
-        for rev_id in revision_ids:
-            i += 1
-            pb.update('fetching revision', i, len(revision_ids))
-            rev = other.get_revision(rev_id)
-            revisions.append(rev)
-            inv = other.get_inventory(str(rev.inventory_id))
-            for key, entry in inv.iter_entries():
-                if entry.text_id is None:
-                    continue
-                if entry.text_id not in self.text_store:
-                    needed_texts.add(entry.text_id)
-
-        pb.clear()
-                    
-        count = self.text_store.copy_multi(other.text_store, needed_texts)
-        print "Added %d texts." % count 
-        inventory_ids = [ f.inventory_id for f in revisions ]
-        count = self.inventory_store.copy_multi(other.inventory_store, 
-                                                inventory_ids)
-        print "Added %d inventories." % count 
-        revision_ids = [ f.revision_id for f in revisions]
-        count = self.revision_store.copy_multi(other.revision_store, 
-                                               revision_ids)
-        for revision_id in revision_ids:
-            self.append_revision(revision_id)
-        print "Added %d revisions." % count
-                    
-        
+        """Pull in new perfect-fit revisions."""
+        from bzrlib.fetch import greedy_fetch
+        from bzrlib.revision import get_intervening_revisions
+        if stop_revision is None:
+            stop_revision = other.last_revision()
+        greedy_fetch(to_branch=self, from_branch=other,
+                     revision=stop_revision)
+        pullable_revs = self.missing_revisions(
+            other, other.revision_id_to_revno(stop_revision))
+        if pullable_revs:
+            greedy_fetch(to_branch=self,
+                         from_branch=other,
+                         revision=pullable_revs[-1])
+            self.append_revision(*pullable_revs)
+    
+
     def commit(self, *args, **kw):
-        from bzrlib.commit import commit
-        commit(self, *args, **kw)
-        
-
-    def lookup_revision(self, revision):
-        """Return the revision identifier for a given revision information."""
-        revno, info = self.get_revision_info(revision)
-        return info
-
-    def get_revision_info(self, revision):
-        """Return (revno, revision id) for revision identifier.
-
-        revision can be an integer, in which case it is assumed to be revno (though
-            this will translate negative values into positive ones)
-        revision can also be a string, in which case it is parsed for something like
-            'date:' or 'revid:' etc.
-        """
-        if revision is None:
-            return 0, None
-        revno = None
-        try:# Convert to int if possible
-            revision = int(revision)
-        except ValueError:
-            pass
-        revs = self.revision_history()
-        if isinstance(revision, int):
-            if revision == 0:
-                return 0, None
-            # Mabye we should do this first, but we don't need it if revision == 0
-            if revision < 0:
-                revno = len(revs) + revision + 1
-            else:
-                revno = revision
-        elif isinstance(revision, basestring):
-            for prefix, func in Branch.REVISION_NAMESPACES.iteritems():
-                if revision.startswith(prefix):
-                    revno = func(self, revs, revision)
-                    break
-            else:
-                raise BzrError('No namespace registered for string: %r' % revision)
-
-        if revno is None or revno <= 0 or revno > len(revs):
-            raise BzrError("no such revision %s" % revision)
-        return revno, revs[revno-1]
-
-    def _namespace_revno(self, revs, revision):
-        """Lookup a revision by revision number"""
-        assert revision.startswith('revno:')
-        try:
-            return int(revision[6:])
-        except ValueError:
-            return None
-    REVISION_NAMESPACES['revno:'] = _namespace_revno
-
-    def _namespace_revid(self, revs, revision):
-        assert revision.startswith('revid:')
-        try:
-            return revs.index(revision[6:]) + 1
-        except ValueError:
-            return None
-    REVISION_NAMESPACES['revid:'] = _namespace_revid
-
-    def _namespace_last(self, revs, revision):
-        assert revision.startswith('last:')
-        try:
-            offset = int(revision[5:])
-        except ValueError:
-            return None
-        else:
-            if offset <= 0:
-                raise BzrError('You must supply a positive value for --revision last:XXX')
-            return len(revs) - offset + 1
-    REVISION_NAMESPACES['last:'] = _namespace_last
-
-    def _namespace_tag(self, revs, revision):
-        assert revision.startswith('tag:')
-        raise BzrError('tag: namespace registered, but not implemented.')
-    REVISION_NAMESPACES['tag:'] = _namespace_tag
-
-    def _namespace_date(self, revs, revision):
-        assert revision.startswith('date:')
-        import datetime
-        # Spec for date revisions:
-        #   date:value
-        #   value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
-        #   it can also start with a '+/-/='. '+' says match the first
-        #   entry after the given date. '-' is match the first entry before the date
-        #   '=' is match the first entry after, but still on the given date.
-        #
-        #   +2005-05-12 says find the first matching entry after May 12th, 2005 at 0:00
-        #   -2005-05-12 says find the first matching entry before May 12th, 2005 at 0:00
-        #   =2005-05-12 says find the first match after May 12th, 2005 at 0:00 but before
-        #       May 13th, 2005 at 0:00
-        #
-        #   So the proper way of saying 'give me all entries for today' is:
-        #       -r {date:+today}:{date:-tomorrow}
-        #   The default is '=' when not supplied
-        val = revision[5:]
-        match_style = '='
-        if val[:1] in ('+', '-', '='):
-            match_style = val[:1]
-            val = val[1:]
-
-        today = datetime.datetime.today().replace(hour=0,minute=0,second=0,microsecond=0)
-        if val.lower() == 'yesterday':
-            dt = today - datetime.timedelta(days=1)
-        elif val.lower() == 'today':
-            dt = today
-        elif val.lower() == 'tomorrow':
-            dt = today + datetime.timedelta(days=1)
-        else:
-            import re
-            # This should be done outside the function to avoid recompiling it.
-            _date_re = re.compile(
-                    r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
-                    r'(,|T)?\s*'
-                    r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
-                )
-            m = _date_re.match(val)
-            if not m or (not m.group('date') and not m.group('time')):
-                raise BzrError('Invalid revision date %r' % revision)
-
-            if m.group('date'):
-                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
-            else:
-                year, month, day = today.year, today.month, today.day
-            if m.group('time'):
-                hour = int(m.group('hour'))
-                minute = int(m.group('minute'))
-                if m.group('second'):
-                    second = int(m.group('second'))
-                else:
-                    second = 0
-            else:
-                hour, minute, second = 0,0,0
-
-            dt = datetime.datetime(year=year, month=month, day=day,
-                    hour=hour, minute=minute, second=second)
-        first = dt
-        last = None
-        reversed = False
-        if match_style == '-':
-            reversed = True
-        elif match_style == '=':
-            last = dt + datetime.timedelta(days=1)
-
-        if reversed:
-            for i in range(len(revs)-1, -1, -1):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first >= dt and (last is None or dt >= last):
-                    return i+1
-        else:
-            for i in range(len(revs)):
-                r = self.get_revision(revs[i])
-                # TODO: Handle timezone.
-                dt = datetime.datetime.fromtimestamp(r.timestamp)
-                if first <= dt and (last is None or dt <= last):
-                    return i+1
-    REVISION_NAMESPACES['date:'] = _namespace_date
+        from bzrlib.commit import Commit
+        Commit().commit(self, *args, **kw)
+    
+    def revision_id_to_revno(self, revision_id):
+        """Given a revision id, return its revno"""
+        if revision_id is None:
+            return 0
+        history = self.revision_history()
+        try:
+            return history.index(revision_id) + 1
+        except ValueError:
+            raise bzrlib.errors.NoSuchRevision(self, revision_id)
+
+    def get_rev_id(self, revno, history=None):
+        """Find the revision id of the specified revno."""
+        if revno == 0:
+            return None
+        if history is None:
+            history = self.revision_history()
+        elif revno <= 0 or revno > len(history):
+            raise bzrlib.errors.NoSuchRevision(self, revno)
+        return history[revno - 1]
 
     def revision_tree(self, revision_id):
         """Return Tree for a revision on this branch.
 
         `revision_id` may be None for the null revision, in which case
         an `EmptyTree` is returned."""
-        from bzrlib.tree import EmptyTree, RevisionTree
         # TODO: refactor this to use an existing revision object
         # so we don't need to read it in twice.
         if revision_id == None:
-            return EmptyTree(self.get_root_id())
+            return EmptyTree()
         else:
             inv = self.get_revision_inventory(revision_id)
-            return RevisionTree(self.text_store, inv)
+            return RevisionTree(self.weave_store, inv, revision_id)
 
 
     def working_tree(self):
         """Return a `Tree` for the working copy."""
-        from workingtree import WorkingTree
-        return WorkingTree(self.base, self.read_working_inventory())
+        from bzrlib.workingtree import WorkingTree
+        # TODO: In the future, WorkingTree should utilize Transport
+        # RobertCollins 20051003 - I don't think it should - working trees are
+        # much more complex to keep consistent than our careful .bzr subset.
+        # instead, we should say that working trees are local only, and optimise
+        # for that.
+        return WorkingTree(self._transport.base, self.read_working_inventory())
 
 
     def basis_tree(self):
@@ -1039,13 +1051,7 @@
 
         If there are no revisions yet, return an `EmptyTree`.
         """
-        from bzrlib.tree import EmptyTree, RevisionTree
-        r = self.last_patch()
-        if r == None:
-            return EmptyTree(self.get_root_id())
-        else:
-            return RevisionTree(self.text_store, self.get_revision_inventory(r))
-
+        return self.revision_tree(self.last_revision())
 
 
     def rename_one(self, from_rel, to_rel):
@@ -1083,12 +1089,10 @@
 
             inv.rename(file_id, to_dir_id, to_tail)
 
-            print "%s => %s" % (from_rel, to_rel)
-
             from_abs = self.abspath(from_rel)
             to_abs = self.abspath(to_rel)
             try:
-                os.rename(from_abs, to_abs)
+                rename(from_abs, to_abs)
             except OSError, e:
                 raise BzrError("failed to rename %r to %r: %s"
                         % (from_abs, to_abs, e[1]),
@@ -1109,7 +1113,11 @@
 
         Note that to_name is only the last component of the new name;
         this doesn't change the directory.
+
+        This returns a list of (from_path, to_path) pairs for each
+        entry that is moved.
         """
+        result = []
         self.lock_write()
         try:
             ## TODO: Option to move IDs only
@@ -1150,10 +1158,10 @@
             for f in from_paths:
                 name_tail = splitpath(f)[-1]
                 dest_path = appendpath(to_name, name_tail)
-                print "%s => %s" % (f, dest_path)
+                result.append((f, dest_path))
                 inv.rename(inv.path2id(f), to_dir_id, name_tail)
                 try:
-                    os.rename(self.abspath(f), self.abspath(dest_path))
+                    rename(self.abspath(f), self.abspath(dest_path))
                 except OSError, e:
                     raise BzrError("failed to rename %r to %r: %s" % (f, dest_path, e[1]),
                             ["rename rolled back"])
@@ -1162,6 +1170,8 @@
         finally:
             self.unlock()
 
+        return result
+
 
     def revert(self, filenames, old_tree=None, backups=True):
         """Restore selected files to the versions from a previous tree.
@@ -1213,8 +1223,8 @@
         These are revisions that have been merged into the working
         directory but not yet committed.
         """
-        cfn = self.controlfilename('pending-merges')
-        if not os.path.exists(cfn):
+        cfn = self._rel_controlfilename('pending-merges')
+        if not self._transport.has(cfn):
             return []
         p = []
         for l in self.controlfile('pending-merges', 'r').readlines():
@@ -1222,35 +1232,80 @@
         return p
 
 
-    def add_pending_merge(self, revision_id):
-        from bzrlib.revision import validate_revision_id
-
-        validate_revision_id(revision_id)
-
+    def add_pending_merge(self, *revision_ids):
+        # TODO: Perhaps should check at this point that the
+        # history of the revision is actually present?
         p = self.pending_merges()
-        if revision_id in p:
-            return
-        p.append(revision_id)
-        self.set_pending_merges(p)
-
+        updated = False
+        for rev_id in revision_ids:
+            if rev_id in p:
+                continue
+            p.append(rev_id)
+            updated = True
+        if updated:
+            self.set_pending_merges(p)
 
     def set_pending_merges(self, rev_list):
+        self.lock_write()
+        try:
+            self.put_controlfile('pending-merges', '\n'.join(rev_list))
+        finally:
+            self.unlock()
+
+
+    def get_parent(self):
+        """Return the parent location of the branch.
+
+        This is the default location for push/pull/missing.  The usual
+        pattern is that the user can override it by specifying a
+        location.
+        """
+        import errno
+        _locs = ['parent', 'pull', 'x-pull']
+        for l in _locs:
+            try:
+                return self.controlfile(l, 'r').read().strip('\n')
+            except IOError, e:
+                if e.errno != errno.ENOENT:
+                    raise
+        return None
+
+
+    def set_parent(self, url):
+        # TODO: Maybe delete old location files?
         from bzrlib.atomicfile import AtomicFile
         self.lock_write()
         try:
-            f = AtomicFile(self.controlfilename('pending-merges'))
+            f = AtomicFile(self.controlfilename('parent'))
             try:
-                for l in rev_list:
-                    print >>f, l
+                f.write(url + '\n')
                 f.commit()
             finally:
                 f.close()
         finally:
             self.unlock()
 
-
-
-class ScratchBranch(Branch):
+    def check_revno(self, revno):
+        """\
+        Check whether a revno corresponds to any revision.
+        Zero (the NULL revision) is considered valid.
+        """
+        if revno != 0:
+            self.check_real_revno(revno)
+            
+    def check_real_revno(self, revno):
+        """\
+        Check whether a revno corresponds to a real revision.
+        Zero (the NULL revision) is considered invalid
+        """
+        if revno < 1 or revno > self.revno():
+            raise InvalidRevisionNumber(revno)
+        
+        
+        
+
+
+class ScratchBranch(_Branch):
     """Special test class: a branch that cleans up after itself.
 
     >>> b = ScratchBranch()
@@ -1273,19 +1328,25 @@
         if base is None:
             base = mkdtemp()
             init = True
-        Branch.__init__(self, base, init=init)
+        if isinstance(base, basestring):
+            base = get_transport(base)
+        _Branch.__init__(self, base, init=init)
         for d in dirs:
-            os.mkdir(self.abspath(d))
+            self._transport.mkdir(d)
             
         for f in files:
-            file(os.path.join(self.base, f), 'w').write('content of %s' % f)
+            self._transport.put(f, 'content of %s' % f)
 
 
     def clone(self):
         """
         >>> orig = ScratchBranch(files=["file1", "file2"])
         >>> clone = orig.clone()
-        >>> os.path.samefile(orig.base, clone.base)
+        >>> if os.name != 'nt':
+        ...   os.path.samefile(orig.base, clone.base)
+        ... else:
+        ...   orig.base == clone.base
+        ...
         False
         >>> os.path.isfile(os.path.join(clone.base, "file1"))
         True
@@ -1296,7 +1357,7 @@
         os.rmdir(base)
         copytree(self.base, base, symlinks=True)
         return ScratchBranch(base=base)
-        
+
     def __del__(self):
         self.destroy()
 
@@ -1315,7 +1376,7 @@
                 for name in files:
                     os.chmod(os.path.join(root, name), 0700)
             rmtree(self.base)
-        self.base = None
+        self._transport = None
 
     
 
@@ -1371,3 +1432,4 @@
     """Return a new tree-root file id."""
     return gen_file_id('TREE_ROOT')
 
+

=== added file 'bzrlib/builtins.py'
--- bzrlib/builtins.py	1970-01-01 00:00:00 +0000
+++ bzrlib/builtins.py	2005-10-08 08:45:42 +0000
@@ -0,0 +1,1422 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import sys
+import os
+
+import bzrlib
+import bzrlib.trace
+from bzrlib.trace import mutter, note, log_error, warning
+from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
+from bzrlib.branch import Branch
+from bzrlib import BZRDIR
+from bzrlib.commands import Command
+
+
+class cmd_status(Command):
+    """Display status summary.
+
+    This reports on versioned and unknown files, reporting them
+    grouped by state.  Possible states are:
+
+    added
+        Versioned in the working copy but not in the previous revision.
+
+    removed
+        Versioned in the previous revision but removed or deleted
+        in the working copy.
+
+    renamed
+        Path of this file changed from the previous revision;
+        the text may also have changed.  This includes files whose
+        parent directory was renamed.
+
+    modified
+        Text has changed since the previous revision.
+
+    unchanged
+        Nothing about this file has changed since the previous revision.
+        Only shown with --all.
+
+    unknown
+        Not versioned and not matching an ignore pattern.
+
+    To see ignored files use 'bzr ignored'.  For details in the
+    changes to file texts, use 'bzr diff'.
+
+    If no arguments are specified, the status of the entire working
+    directory is shown.  Otherwise, only the status of the specified
+    files or directories is reported.  If a directory is given, status
+    is reported for everything inside that directory.
+
+    If a revision argument is given, the status is calculated against
+    that revision, or between two revisions if two are provided.
+    """
+    # XXX: FIXME: bzr status should accept a -r option to show changes
+    # relative to a revision, or between revisions
+
+    takes_args = ['file*']
+    takes_options = ['all', 'show-ids']
+    aliases = ['st', 'stat']
+    
+    def run(self, all=False, show_ids=False, file_list=None, revision=None):
+        if file_list:
+            b = Branch.open_containing(file_list[0])
+            file_list = [b.relpath(x) for x in file_list]
+            # special case: only one path was given and it's the root
+            # of the branch
+            if file_list == ['']:
+                file_list = None
+        else:
+            b = Branch.open_containing('.')
+            
+        from bzrlib.status import show_status
+        show_status(b, show_unchanged=all, show_ids=show_ids,
+                    specific_files=file_list, revision=revision)
+
+
+class cmd_cat_revision(Command):
+    """Write out metadata for a revision.
+    
+    The revision to print can either be specified by a specific
+    revision identifier, or you can use --revision.
+    """
+
+    hidden = True
+    takes_args = ['revision_id?']
+    takes_options = ['revision']
+    
+    def run(self, revision_id=None, revision=None):
+        from bzrlib.revisionspec import RevisionSpec
+
+        if revision_id is not None and revision is not None:
+            raise BzrCommandError('You can only supply one of revision_id or --revision')
+        if revision_id is None and revision is None:
+            raise BzrCommandError('You must supply either --revision or a revision_id')
+        b = Branch.open_containing('.')
+        if revision_id is not None:
+            sys.stdout.write(b.get_revision_xml_file(revision_id).read())
+        elif revision is not None:
+            for rev in revision:
+                if rev is None:
+                    raise BzrCommandError('You cannot specify a NULL revision.')
+                revno, rev_id = rev.in_history(b)
+                sys.stdout.write(b.get_revision_xml_file(rev_id).read())
+    
+
+class cmd_revno(Command):
+    """Show current revision number.
+
+    This is equal to the number of revisions on this branch."""
+    def run(self):
+        print Branch.open_containing('.').revno()
+
+
+class cmd_revision_info(Command):
+    """Show revision number and revision id for a given revision identifier.
+    """
+    hidden = True
+    takes_args = ['revision_info*']
+    takes_options = ['revision']
+    def run(self, revision=None, revision_info_list=[]):
+        from bzrlib.revisionspec import RevisionSpec
+
+        revs = []
+        if revision is not None:
+            revs.extend(revision)
+        if revision_info_list is not None:
+            for rev in revision_info_list:
+                revs.append(RevisionSpec(rev))
+        if len(revs) == 0:
+            raise BzrCommandError('You must supply a revision identifier')
+
+        b = Branch.open_containing('.')
+
+        for rev in revs:
+            revinfo = rev.in_history(b)
+            if revinfo.revno is None:
+                print '     %s' % revinfo.rev_id
+            else:
+                print '%4d %s' % (revinfo.revno, revinfo.rev_id)
+
+    
+class cmd_add(Command):
+    """Add specified files or directories.
+
+    In non-recursive mode, all the named items are added, regardless
+    of whether they were previously ignored.  A warning is given if
+    any of the named files are already versioned.
+
+    In recursive mode (the default), files are treated the same way
+    but the behaviour for directories is different.  Directories that
+    are already versioned do not give a warning.  All directories,
+    whether already versioned or not, are searched for files or
+    subdirectories that are neither versioned or ignored, and these
+    are added.  This search proceeds recursively into versioned
+    directories.  If no names are given '.' is assumed.
+
+    Therefore simply saying 'bzr add' will version all files that
+    are currently unknown.
+
+    Adding a file whose parent directory is not versioned will
+    implicitly add the parent, and so on up to the root. This means
+    you should never need to explictly add a directory, they'll just
+    get added when you add a file in the directory.
+    """
+    takes_args = ['file*']
+    takes_options = ['verbose', 'no-recurse']
+    
+    def run(self, file_list, verbose=False, no_recurse=False):
+        # verbose currently has no effect
+        from bzrlib.add import smart_add, add_reporter_print
+        smart_add(file_list, not no_recurse, add_reporter_print)
+
+
+
+class cmd_mkdir(Command):
+    """Create a new versioned directory.
+
+    This is equivalent to creating the directory and then adding it.
+    """
+    takes_args = ['dir+']
+
+    def run(self, dir_list):
+        b = None
+        
+        for d in dir_list:
+            os.mkdir(d)
+            if not b:
+                b = Branch.open_containing(d)
+            b.add([d])
+            print 'added', d
+
+
+class cmd_relpath(Command):
+    """Show path of a file relative to root"""
+    takes_args = ['filename']
+    hidden = True
+    
+    def run(self, filename):
+        print Branch.open_containing(filename).relpath(filename)
+
+
+
+class cmd_inventory(Command):
+    """Show inventory of the current working copy or a revision."""
+    takes_options = ['revision', 'show-ids']
+    
+    def run(self, revision=None, show_ids=False):
+        b = Branch.open_containing('.')
+        if revision is None:
+            inv = b.read_working_inventory()
+        else:
+            if len(revision) > 1:
+                raise BzrCommandError('bzr inventory --revision takes'
+                    ' exactly one revision identifier')
+            inv = b.get_revision_inventory(revision[0].in_history(b).rev_id)
+
+        for path, entry in inv.entries():
+            if show_ids:
+                print '%-50s %s' % (path, entry.file_id)
+            else:
+                print path
+
+
+class cmd_move(Command):
+    """Move files to a different directory.
+
+    examples:
+        bzr move *.txt doc
+
+    The destination must be a versioned directory in the same branch.
+    """
+    takes_args = ['source$', 'dest']
+    def run(self, source_list, dest):
+        b = Branch.open_containing('.')
+
+        # TODO: glob expansion on windows?
+        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
+
+
+class cmd_rename(Command):
+    """Change the name of an entry.
+
+    examples:
+      bzr rename frob.c frobber.c
+      bzr rename src/frob.c lib/frob.c
+
+    It is an error if the destination name exists.
+
+    See also the 'move' command, which moves files into a different
+    directory without changing their name.
+
+    TODO: Some way to rename multiple files without invoking bzr for each
+    one?"""
+    takes_args = ['from_name', 'to_name']
+    
+    def run(self, from_name, to_name):
+        b = Branch.open_containing('.')
+        b.rename_one(b.relpath(from_name), b.relpath(to_name))
+
+
+
+class cmd_mv(Command):
+    """Move or rename a file.
+
+    usage:
+        bzr mv OLDNAME NEWNAME
+        bzr mv SOURCE... DESTINATION
+
+    If the last argument is a versioned directory, all the other names
+    are moved into it.  Otherwise, there must be exactly two arguments
+    and the file is changed to a new name, which must not already exist.
+
+    Files cannot be moved between branches.
+    """
+    takes_args = ['names*']
+    def run(self, names_list):
+        if len(names_list) < 2:
+            raise BzrCommandError("missing file argument")
+        b = Branch.open_containing(names_list[0])
+
+        rel_names = [b.relpath(x) for x in names_list]
+        
+        if os.path.isdir(names_list[-1]):
+            # move into existing directory
+            for pair in b.move(rel_names[:-1], rel_names[-1]):
+                print "%s => %s" % pair
+        else:
+            if len(names_list) != 2:
+                raise BzrCommandError('to mv multiple files the destination '
+                                      'must be a versioned directory')
+            b.rename_one(rel_names[0], rel_names[1])
+            print "%s => %s" % (rel_names[0], rel_names[1])
+            
+    
+
+
+class cmd_pull(Command):
+    """Pull any changes from another branch into the current one.
+
+    If the location is omitted, the last-used location will be used.
+    Both the revision history and the working directory will be
+    updated.
+
+    This command only works on branches that have not diverged.  Branches are
+    considered diverged if both branches have had commits without first
+    pulling from the other.
+
+    If branches have diverged, you can use 'bzr merge' to pull the text changes
+    from one into the other.
+    """
+    takes_options = ['remember']
+    takes_args = ['location?']
+
+    def run(self, location=None, remember=False):
+        from bzrlib.merge import merge
+        import tempfile
+        from shutil import rmtree
+        import errno
+        
+        br_to = Branch.open_containing('.')
+        stored_loc = br_to.get_parent()
+        if location is None:
+            if stored_loc is None:
+                raise BzrCommandError("No pull location known or specified.")
+            else:
+                print "Using saved location: %s" % stored_loc
+                location = stored_loc
+        cache_root = tempfile.mkdtemp()
+        from bzrlib.errors import DivergedBranches
+        br_from = Branch.open_containing(location)
+        location = br_from.base
+        old_revno = br_to.revno()
+        try:
+            from bzrlib.errors import DivergedBranches
+            br_from = Branch.open(location)
+            br_from.setup_caching(cache_root)
+            location = br_from.base
+            old_revno = br_to.revno()
+            try:
+                br_to.update_revisions(br_from)
+            except DivergedBranches:
+                raise BzrCommandError("These branches have diverged."
+                    "  Try merge.")
+                
+            merge(('.', -1), ('.', old_revno), check_clean=False)
+            if stored_loc is None or remember:
+                br_to.set_parent(location)
+        finally:
+            rmtree(cache_root)
+
+
+
+class cmd_branch(Command):
+    """Create a new copy of a branch.
+
+    If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will
+    be used.  In other words, "branch ../foo/bar" will attempt to create ./bar.
+
+    To retrieve the branch as of a particular revision, supply the --revision
+    parameter, as in "branch foo/bar -r 5".
+
+    --basis is to speed up branching from remote branches.  When specified, it
+    copies all the file-contents, inventory and revision data from the basis
+    branch before copying anything from the remote branch.
+    """
+    takes_args = ['from_location', 'to_location?']
+    takes_options = ['revision', 'basis']
+    aliases = ['get', 'clone']
+
+    def run(self, from_location, to_location=None, revision=None, basis=None):
+        from bzrlib.clone import copy_branch
+        import tempfile
+        import errno
+        from shutil import rmtree
+        cache_root = tempfile.mkdtemp()
+        try:
+            if revision is None:
+                revision = [None]
+            elif len(revision) > 1:
+                raise BzrCommandError(
+                    'bzr branch --revision takes exactly 1 revision value')
+            try:
+                br_from = Branch.open(from_location)
+            except OSError, e:
+                if e.errno == errno.ENOENT:
+                    raise BzrCommandError('Source location "%s" does not'
+                                          ' exist.' % to_location)
+                else:
+                    raise
+            br_from.setup_caching(cache_root)
+            if basis is not None:
+                basis_branch = Branch.open_containing(basis)
+            else:
+                basis_branch = None
+            if len(revision) == 1 and revision[0] is not None:
+                revision_id = revision[0].in_history(br_from)[1]
+            else:
+                revision_id = None
+            if to_location is None:
+                to_location = os.path.basename(from_location.rstrip("/\\"))
+            try:
+                os.mkdir(to_location)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    raise BzrCommandError('Target directory "%s" already'
+                                          ' exists.' % to_location)
+                if e.errno == errno.ENOENT:
+                    raise BzrCommandError('Parent of "%s" does not exist.' %
+                                          to_location)
+                else:
+                    raise
+            try:
+                copy_branch(br_from, to_location, revision_id, basis_branch)
+            except bzrlib.errors.NoSuchRevision:
+                rmtree(to_location)
+                msg = "The branch %s has no revision %d." % (from_location, revision[0])
+                raise BzrCommandError(msg)
+            except bzrlib.errors.UnlistableBranch:
+                msg = "The branch %s cannot be used as a --basis"
+        finally:
+            rmtree(cache_root)
+
+
+class cmd_renames(Command):
+    """Show list of renamed files.
+
+    TODO: Option to show renames between two historical versions.
+
+    TODO: Only show renames under dir, rather than in the whole branch.
+    """
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        b = Branch.open_containing(dir)
+        old_inv = b.basis_tree().inventory
+        new_inv = b.read_working_inventory()
+
+        renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
+        renames.sort()
+        for old_name, new_name in renames:
+            print "%s => %s" % (old_name, new_name)        
+
+
+class cmd_info(Command):
+    """Show statistical information about a branch."""
+    takes_args = ['branch?']
+    
+    def run(self, branch=None):
+        import info
+
+        b = Branch.open_containing(branch)
+        info.show_info(b)
+
+
+class cmd_remove(Command):
+    """Make a file unversioned.
+
+    This makes bzr stop tracking changes to a versioned file.  It does
+    not delete the working copy.
+    """
+    takes_args = ['file+']
+    takes_options = ['verbose']
+    aliases = ['rm']
+    
+    def run(self, file_list, verbose=False):
+        b = Branch.open_containing(file_list[0])
+        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
+
+
+class cmd_file_id(Command):
+    """Print file_id of a particular file or directory.
+
+    The file_id is assigned when the file is first added and remains the
+    same through all revisions where the file exists, even when it is
+    moved or renamed.
+    """
+    hidden = True
+    takes_args = ['filename']
+    def run(self, filename):
+        b = Branch.open_containing(filename)
+        i = b.inventory.path2id(b.relpath(filename))
+        if i == None:
+            raise BzrError("%r is not a versioned file" % filename)
+        else:
+            print i
+
+
+class cmd_file_path(Command):
+    """Print path of file_ids to a file or directory.
+
+    This prints one line for each directory down to the target,
+    starting at the branch root."""
+    hidden = True
+    takes_args = ['filename']
+    def run(self, filename):
+        b = Branch.open_containing(filename)
+        inv = b.inventory
+        fid = inv.path2id(b.relpath(filename))
+        if fid == None:
+            raise BzrError("%r is not a versioned file" % filename)
+        for fip in inv.get_idpath(fid):
+            print fip
+
+
+class cmd_revision_history(Command):
+    """Display list of revision ids on this branch."""
+    hidden = True
+    def run(self):
+        for patchid in Branch.open_containing('.').revision_history():
+            print patchid
+
+
+class cmd_ancestry(Command):
+    """List all revisions merged into this branch."""
+    hidden = True
+    def run(self):
+        b = find_branch('.')
+        for revision_id in b.get_ancestry(b.last_revision()):
+            print revision_id
+
+
+class cmd_directories(Command):
+    """Display list of versioned directories in this branch."""
+    def run(self):
+        for name, ie in Branch.open_containing('.').read_working_inventory().directories():
+            if name == '':
+                print '.'
+            else:
+                print name
+
+
+class cmd_init(Command):
+    """Make a directory into a versioned branch.
+
+    Use this to create an empty branch, or before importing an
+    existing project.
+
+    Recipe for importing a tree of files:
+        cd ~/project
+        bzr init
+        bzr add -v .
+        bzr status
+        bzr commit -m 'imported project'
+    """
+    def run(self):
+        Branch.initialize('.')
+
+
+class cmd_diff(Command):
+    """Show differences in working tree.
+    
+    If files are listed, only the changes in those files are listed.
+    Otherwise, all changes for the tree are listed.
+
+    TODO: Allow diff across branches.
+
+    TODO: Option to use external diff command; could be GNU diff, wdiff,
+          or a graphical diff.
+
+    TODO: Python difflib is not exactly the same as unidiff; should
+          either fix it up or prefer to use an external diff.
+
+    TODO: If a directory is given, diff everything under that.
+
+    TODO: Selected-file diff is inefficient and doesn't show you
+          deleted files.
+
+    TODO: This probably handles non-Unix newlines poorly.
+
+    examples:
+        bzr diff
+        bzr diff -r1
+        bzr diff -r1..2
+    """
+    
+    takes_args = ['file*']
+    takes_options = ['revision', 'diff-options']
+    aliases = ['di', 'dif']
+
+    def run(self, revision=None, file_list=None, diff_options=None):
+        from bzrlib.diff import show_diff
+
+        if file_list:
+            b = Branch.open_containing(file_list[0])
+            file_list = [b.relpath(f) for f in file_list]
+            if file_list == ['']:
+                # just pointing to top-of-tree
+                file_list = None
+        else:
+            b = Branch.open_containing('.')
+
+        if revision is not None:
+            if len(revision) == 1:
+                show_diff(b, revision[0], specific_files=file_list,
+                          external_diff_options=diff_options)
+            elif len(revision) == 2:
+                show_diff(b, revision[0], specific_files=file_list,
+                          external_diff_options=diff_options,
+                          revision2=revision[1])
+            else:
+                raise BzrCommandError('bzr diff --revision takes exactly one or two revision identifiers')
+        else:
+            show_diff(b, None, specific_files=file_list,
+                      external_diff_options=diff_options)
+
+        
+
+
+class cmd_deleted(Command):
+    """List files deleted in the working tree.
+
+    TODO: Show files deleted since a previous revision, or between two revisions.
+    """
+    def run(self, show_ids=False):
+        b = Branch.open_containing('.')
+        old = b.basis_tree()
+        new = b.working_tree()
+
+        ## TODO: Much more efficient way to do this: read in new
+        ## directories with readdir, rather than stating each one.  Same
+        ## level of effort but possibly much less IO.  (Or possibly not,
+        ## if the directories are very large...)
+
+        for path, ie in old.inventory.iter_entries():
+            if not new.has_id(ie.file_id):
+                if show_ids:
+                    print '%-50s %s' % (path, ie.file_id)
+                else:
+                    print path
+
+
+class cmd_modified(Command):
+    """List files modified in working tree."""
+    hidden = True
+    def run(self):
+        from bzrlib.delta import compare_trees
+
+        b = Branch.open_containing('.')
+        td = compare_trees(b.basis_tree(), b.working_tree())
+
+        for path, id, kind, text_modified, meta_modified in td.modified:
+            print path
+
+
+
+class cmd_added(Command):
+    """List files added in working tree."""
+    hidden = True
+    def run(self):
+        b = Branch.open_containing('.')
+        wt = b.working_tree()
+        basis_inv = b.basis_tree().inventory
+        inv = wt.inventory
+        for file_id in inv:
+            if file_id in basis_inv:
+                continue
+            path = inv.id2path(file_id)
+            if not os.access(b.abspath(path), os.F_OK):
+                continue
+            print path
+                
+        
+
+class cmd_root(Command):
+    """Show the tree root directory.
+
+    The root is the nearest enclosing directory with a .bzr control
+    directory."""
+    takes_args = ['filename?']
+    def run(self, filename=None):
+        """Print the branch root."""
+        b = Branch.open_containing(filename)
+        print b.base
+
+
+class cmd_log(Command):
+    """Show log of this branch.
+
+    To request a range of logs, you can use the command -r begin:end
+    -r revision requests a specific revision, -r :end or -r begin: are
+    also valid.
+
+    --message allows you to give a regular expression, which will be evaluated
+    so that only matching entries will be displayed.
+    """
+
+    # TODO: Make --revision support uuid: and hash: [future tag:] notation.
+
+    takes_args = ['filename?']
+    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision',
+                     'long', 'message', 'short',]
+    
+    def run(self, filename=None, timezone='original',
+            verbose=False,
+            show_ids=False,
+            forward=False,
+            revision=None,
+            message=None,
+            long=False,
+            short=False):
+        from bzrlib.log import log_formatter, show_log
+        import codecs
+
+        direction = (forward and 'forward') or 'reverse'
+        
+        if filename:
+            b = Branch.open_containing(filename)
+            fp = b.relpath(filename)
+            if fp:
+                file_id = b.read_working_inventory().path2id(fp)
+            else:
+                file_id = None  # points to branch root
+        else:
+            b = Branch.open_containing('.')
+            file_id = None
+
+        if revision is None:
+            rev1 = None
+            rev2 = None
+        elif len(revision) == 1:
+            rev1 = rev2 = revision[0].in_history(b).revno
+        elif len(revision) == 2:
+            rev1 = revision[0].in_history(b).revno
+            rev2 = revision[1].in_history(b).revno
+        else:
+            raise BzrCommandError('bzr log --revision takes one or two values.')
+
+        if rev1 == 0:
+            rev1 = None
+        if rev2 == 0:
+            rev2 = None
+
+        mutter('encoding log as %r' % bzrlib.user_encoding)
+
+        # use 'replace' so that we don't abort if trying to write out
+        # in e.g. the default C locale.
+        outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
+
+        if not short:
+            log_format = 'long'
+        else:
+            log_format = 'short'
+        lf = log_formatter(log_format,
+                           show_ids=show_ids,
+                           to_file=outf,
+                           show_timezone=timezone)
+
+        show_log(b,
+                 lf,
+                 file_id,
+                 verbose=verbose,
+                 direction=direction,
+                 start_revision=rev1,
+                 end_revision=rev2,
+                 search=message)
+
+
+
+class cmd_touching_revisions(Command):
+    """Return revision-ids which affected a particular file.
+
+    A more user-friendly interface is "bzr log FILE"."""
+    hidden = True
+    takes_args = ["filename"]
+    def run(self, filename):
+        b = Branch.open_containing(filename)
+        inv = b.read_working_inventory()
+        file_id = inv.path2id(b.relpath(filename))
+        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
+            print "%6d %s" % (revno, what)
+
+
+class cmd_ls(Command):
+    """List files in a tree.
+
+    TODO: Take a revision or remote path and list that tree instead.
+    """
+    hidden = True
+    def run(self, revision=None, verbose=False):
+        b = Branch.open_containing('.')
+        if revision == None:
+            tree = b.working_tree()
+        else:
+            tree = b.revision_tree(revision.in_history(b).rev_id)
+        for fp, fc, kind, fid, entry in tree.list_files():
+            if verbose:
+                kindch = entry.kind_character()
+                print '%-8s %s%s' % (fc, fp, kindch)
+            else:
+                print fp
+
+
+
+class cmd_unknowns(Command):
+    """List unknown files."""
+    def run(self):
+        from bzrlib.osutils import quotefn
+        for f in Branch.open_containing('.').unknowns():
+            print quotefn(f)
+
+
+
+class cmd_ignore(Command):
+    """Ignore a command or pattern.
+
+    To remove patterns from the ignore list, edit the .bzrignore file.
+
+    If the pattern contains a slash, it is compared to the whole path
+    from the branch root.  Otherwise, it is comapred to only the last
+    component of the path.
+
+    Ignore patterns are case-insensitive on case-insensitive systems.
+
+    Note: wildcards must be quoted from the shell on Unix.
+
+    examples:
+        bzr ignore ./Makefile
+        bzr ignore '*.class'
+    """
+    takes_args = ['name_pattern']
+    
+    def run(self, name_pattern):
+        from bzrlib.atomicfile import AtomicFile
+        import os.path
+
+        b = Branch.open_containing('.')
+        ifn = b.abspath('.bzrignore')
+
+        if os.path.exists(ifn):
+            f = open(ifn, 'rt')
+            try:
+                igns = f.read().decode('utf-8')
+            finally:
+                f.close()
+        else:
+            igns = ''
+
+        # TODO: If the file already uses crlf-style termination, maybe
+        # we should use that for the newly added lines?
+
+        if igns and igns[-1] != '\n':
+            igns += '\n'
+        igns += name_pattern + '\n'
+
+        try:
+            f = AtomicFile(ifn, 'wt')
+            f.write(igns.encode('utf-8'))
+            f.commit()
+        finally:
+            f.close()
+
+        inv = b.working_tree().inventory
+        if inv.path2id('.bzrignore'):
+            mutter('.bzrignore is already versioned')
+        else:
+            mutter('need to make new .bzrignore file versioned')
+            b.add(['.bzrignore'])
+
+
+
+class cmd_ignored(Command):
+    """List ignored files and the patterns that matched them.
+
+    See also: bzr ignore"""
+    def run(self):
+        tree = Branch.open_containing('.').working_tree()
+        for path, file_class, kind, file_id, entry in tree.list_files():
+            if file_class != 'I':
+                continue
+            ## XXX: Slightly inefficient since this was already calculated
+            pat = tree.is_ignored(path)
+            print '%-50s %s' % (path, pat)
+
+
+class cmd_lookup_revision(Command):
+    """Lookup the revision-id from a revision-number
+
+    example:
+        bzr lookup-revision 33
+    """
+    hidden = True
+    takes_args = ['revno']
+    
+    def run(self, revno):
+        try:
+            revno = int(revno)
+        except ValueError:
+            raise BzrCommandError("not a valid revision-number: %r" % revno)
+
+        print Branch.open_containing('.').get_rev_id(revno)
+
+
+class cmd_export(Command):
+    """Export past revision to destination directory.
+
+    If no revision is specified this exports the last committed revision.
+
+    Format may be an "exporter" name, such as tar, tgz, tbz2.  If none is
+    given, try to find the format with the extension. If no extension
+    is found exports to a directory (equivalent to --format=dir).
+
+    Root may be the top directory for tar, tgz and tbz2 formats. If none
+    is given, the top directory will be the root name of the file."""
+    # TODO: list known exporters
+    takes_args = ['dest']
+    takes_options = ['revision', 'format', 'root']
+    def run(self, dest, revision=None, format=None, root=None):
+        import os.path
+        b = Branch.open_containing('.')
+        if revision is None:
+            rev_id = b.last_revision()
+        else:
+            if len(revision) != 1:
+                raise BzrError('bzr export --revision takes exactly 1 argument')
+            rev_id = revision[0].in_history(b).rev_id
+        t = b.revision_tree(rev_id)
+        arg_root, ext = os.path.splitext(os.path.basename(dest))
+        if ext in ('.gz', '.bz2'):
+            new_root, new_ext = os.path.splitext(arg_root)
+            if new_ext == '.tar':
+                arg_root = new_root
+                ext = new_ext + ext
+        if root is None:
+            root = arg_root
+        if not format:
+            if ext in (".tar",):
+                format = "tar"
+            elif ext in (".tar.gz", ".tgz"):
+                format = "tgz"
+            elif ext in (".tar.bz2", ".tbz2"):
+                format = "tbz2"
+            else:
+                format = "dir"
+        t.export(dest, format, root)
+
+
+class cmd_cat(Command):
+    """Write a file's text from a previous revision."""
+
+    takes_options = ['revision']
+    takes_args = ['filename']
+
+    def run(self, filename, revision=None):
+        if revision is None:
+            raise BzrCommandError("bzr cat requires a revision number")
+        elif len(revision) != 1:
+            raise BzrCommandError("bzr cat --revision takes exactly one number")
+        b = Branch.open_containing('.')
+        b.print_file(b.relpath(filename), revision[0].in_history(b).revno)
+
+
+class cmd_local_time_offset(Command):
+    """Show the offset in seconds from GMT to local time."""
+    hidden = True    
+    def run(self):
+        print bzrlib.osutils.local_time_offset()
+
+
+
+class cmd_commit(Command):
+    """Commit changes into a new revision.
+    
+    If no arguments are given, the entire tree is committed.
+
+    If selected files are specified, only changes to those files are
+    committed.  If a directory is specified then the directory and everything 
+    within it is committed.
+
+    A selected-file commit may fail in some cases where the committed
+    tree would be invalid, such as trying to commit a file in a
+    newly-added directory that is not itself committed.
+
+    TODO: Run hooks on tree to-be-committed, and after commit.
+
+    TODO: Strict commit that fails if there are unknown or deleted files.
+    """
+    takes_args = ['selected*']
+    takes_options = ['message', 'file', 'verbose', 'unchanged']
+    aliases = ['ci', 'checkin']
+
+    # TODO: Give better message for -s, --summary, used by tla people
+
+    # XXX: verbose currently does nothing
+    
+    def run(self, message=None, file=None, verbose=True, selected_list=None,
+            unchanged=False):
+        from bzrlib.errors import PointlessCommit, ConflictsInTree
+        from bzrlib.msgeditor import edit_commit_message
+        from bzrlib.status import show_status
+        from cStringIO import StringIO
+
+        b = Branch.open_containing('.')
+        if selected_list:
+            selected_list = [b.relpath(s) for s in selected_list]
+
+            
+        if message is None and not file:
+            catcher = StringIO()
+            show_status(b, specific_files=selected_list,
+                        to_file=catcher)
+            message = edit_commit_message(catcher.getvalue())
+
+            if message is None:
+                raise BzrCommandError("please specify a commit message"
+                                      " with either --message or --file")
+        elif message and file:
+            raise BzrCommandError("please specify either --message or --file")
+        
+        if file:
+            import codecs
+            message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
+
+        if message == "":
+                raise BzrCommandError("empty commit message specified")
+            
+        try:
+            b.commit(message,
+                     specific_files=selected_list,
+                     allow_pointless=unchanged)
+        except PointlessCommit:
+            # FIXME: This should really happen before the file is read in;
+            # perhaps prepare the commit; get the message; then actually commit
+            raise BzrCommandError("no changes to commit",
+                                  ["use --unchanged to commit anyhow"])
+        except ConflictsInTree:
+            raise BzrCommandError("Conflicts detected in working tree.  "
+                'Use "bzr conflicts" to list, "bzr resolve FILE" to resolve.')
+
+
+class cmd_check(Command):
+    """Validate consistency of branch history.
+
+    This command checks various invariants about the branch storage to
+    detect data corruption or bzr bugs.
+    """
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        from bzrlib.check import check
+
+        check(Branch.open_containing(dir))
+
+
+class cmd_scan_cache(Command):
+    hidden = True
+    def run(self):
+        from bzrlib.hashcache import HashCache
+
+        c = HashCache('.')
+        c.read()
+        c.scan()
+            
+        print '%6d stats' % c.stat_count
+        print '%6d in hashcache' % len(c._cache)
+        print '%6d files removed from cache' % c.removed_count
+        print '%6d hashes updated' % c.update_count
+        print '%6d files changed too recently to cache' % c.danger_count
+
+        if c.needs_write:
+            c.write()
+            
+
+
+class cmd_upgrade(Command):
+    """Upgrade branch storage to current format.
+
+    The check command or bzr developers may sometimes advise you to run
+    this command.
+
+    This version of this command upgrades from the full-text storage
+    used by bzr 0.0.8 and earlier to the weave format (v5).
+    """
+    takes_args = ['dir?']
+
+    def run(self, dir='.'):
+        from bzrlib.upgrade import upgrade
+        upgrade(dir)
+
+
+class cmd_whoami(Command):
+    """Show bzr user id."""
+    takes_options = ['email']
+    
+    def run(self, email=False):
+        try:
+            b = bzrlib.branch.Branch.open_containing('.')
+        except NotBranchError:
+            b = None
+        
+        if email:
+            print bzrlib.osutils.user_email(b)
+        else:
+            print bzrlib.osutils.username(b)
+
+
+class cmd_selftest(Command):
+    """Run internal test suite.
+    
+    This creates temporary test directories in the working directory,
+    but not existing data is affected.  These directories are deleted
+    if the tests pass, or left behind to help in debugging if they
+    fail.
+    
+    If arguments are given, they are regular expressions that say
+    which tests should run."""
+    # TODO: --list should give a list of all available tests
+    hidden = True
+    takes_args = ['testnames*']
+    takes_options = ['verbose', 'pattern']
+    def run(self, testnames_list=None, verbose=False, pattern=".*"):
+        import bzrlib.ui
+        from bzrlib.selftest import selftest
+        # we don't want progress meters from the tests to go to the
+        # real output; and we don't want log messages cluttering up
+        # the real logs.
+        save_ui = bzrlib.ui.ui_factory
+        bzrlib.trace.info('running tests...')
+        try:
+            bzrlib.ui.ui_factory = bzrlib.ui.SilentUIFactory()
+            result = selftest(verbose=verbose, 
+                              pattern=pattern,
+                              testnames=testnames_list)
+            if result:
+                bzrlib.trace.info('tests passed')
+            else:
+                bzrlib.trace.info('tests failed')
+            return int(not result)
+        finally:
+            bzrlib.ui.ui_factory = save_ui
+
+
+def show_version():
+    print "bzr (bazaar-ng) %s" % bzrlib.__version__
+    # is bzrlib itself in a branch?
+    bzrrev = bzrlib.get_bzr_revision()
+    if bzrrev:
+        print "  (bzr checkout, revision %d {%s})" % bzrrev
+    print bzrlib.__copyright__
+    print "http://bazaar-ng.org/"
+    print
+    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
+    print "you may use, modify and redistribute it under the terms of the GNU"
+    print "General Public License version 2 or later."
+
+
+class cmd_version(Command):
+    """Show version of bzr."""
+    def run(self):
+        show_version()
+
+class cmd_rocks(Command):
+    """Statement of optimism."""
+    hidden = True
+    def run(self):
+        print "it sure does!"
+
+
+class cmd_find_merge_base(Command):
+    """Find and print a base revision for merging two branches.
+
+    TODO: Options to specify revisions on either side, as if
+          merging only part of the history.
+    """
+    takes_args = ['branch', 'other']
+    hidden = True
+    
+    def run(self, branch, other):
+        from bzrlib.revision import common_ancestor, MultipleRevisionSources
+        
+        branch1 = Branch.open_containing(branch)
+        branch2 = Branch.open_containing(other)
+
+        history_1 = branch1.revision_history()
+        history_2 = branch2.revision_history()
+
+        last1 = branch1.last_revision()
+        last2 = branch2.last_revision()
+
+        source = MultipleRevisionSources(branch1, branch2)
+        
+        base_rev_id = common_ancestor(last1, last2, source)
+
+        print 'merge base is revision %s' % base_rev_id
+        
+        return
+
+        if base_revno is None:
+            raise bzrlib.errors.UnrelatedBranches()
+
+        print ' r%-6d in %s' % (base_revno, branch)
+
+        other_revno = branch2.revision_id_to_revno(base_revid)
+        
+        print ' r%-6d in %s' % (other_revno, other)
+
+
+
+class cmd_merge(Command):
+    """Perform a three-way merge.
+    
+    The branch is the branch you will merge from.  By default, it will
+    merge the latest revision.  If you specify a revision, that
+    revision will be merged.  If you specify two revisions, the first
+    will be used as a BASE, and the second one as OTHER.  Revision
+    numbers are always relative to the specified branch.
+
+    By default bzr will try to merge in all new work from the other
+    branch, automatically determining an appropriate base.  If this
+    fails, you may need to give an explicit base.
+    
+    Examples:
+
+    To merge the latest revision from bzr.dev
+    bzr merge ../bzr.dev
+
+    To merge changes up to and including revision 82 from bzr.dev
+    bzr merge -r 82 ../bzr.dev
+
+    To merge the changes introduced by 82, without previous changes:
+    bzr merge -r 81..82 ../bzr.dev
+    
+    merge refuses to run if there are any uncommitted changes, unless
+    --force is given.
+    """
+    takes_args = ['branch?']
+    takes_options = ['revision', 'force', 'merge-type']
+
+    def run(self, branch=None, revision=None, force=False, 
+            merge_type=None):
+        from bzrlib.merge import merge
+        from bzrlib.merge_core import ApplyMerge3
+        if merge_type is None:
+            merge_type = ApplyMerge3
+        if branch is None:
+            branch = Branch.open_containing('.').get_parent()
+            if branch is None:
+                raise BzrCommandError("No merge location known or specified.")
+            else:
+                print "Using saved location: %s" % branch 
+        if revision is None or len(revision) < 1:
+            base = [None, None]
+            other = [branch, -1]
+        else:
+            if len(revision) == 1:
+                base = [None, None]
+                other = [branch, revision[0].in_history(branch).revno]
+            else:
+                assert len(revision) == 2
+                if None in revision:
+                    raise BzrCommandError(
+                        "Merge doesn't permit that revision specifier.")
+                b = Branch.open(branch)
+
+                base = [branch, revision[0].in_history(b).revno]
+                other = [branch, revision[1].in_history(b).revno]
+
+        try:
+            merge(other, base, check_clean=(not force), merge_type=merge_type)
+        except bzrlib.errors.AmbiguousBase, e:
+            m = ("sorry, bzr can't determine the right merge base yet\n"
+                 "candidates are:\n  "
+                 + "\n  ".join(e.bases)
+                 + "\n"
+                 "please specify an explicit base with -r,\n"
+                 "and (if you want) report this to the bzr developers\n")
+            log_error(m)
+
+
+class cmd_revert(Command):
+    """Reverse all changes since the last commit.
+
+    Only versioned files are affected.  Specify filenames to revert only 
+    those files.  By default, any files that are changed will be backed up
+    first.  Backup files have a '~' appended to their name.
+    """
+    takes_options = ['revision', 'no-backup']
+    takes_args = ['file*']
+    aliases = ['merge-revert']
+
+    def run(self, revision=None, no_backup=False, file_list=None):
+        from bzrlib.merge import merge
+        from bzrlib.commands import parse_spec
+
+        if file_list is not None:
+            if len(file_list) == 0:
+                raise BzrCommandError("No files specified")
+        if revision is None:
+            revno = -1
+        elif len(revision) != 1:
+            raise BzrCommandError('bzr revert --revision takes exactly 1 argument')
+        else:
+            b = Branch.open_containing('.')
+            revno = revision[0].in_history(b).revno
+        merge(('.', revno), parse_spec('.'),
+              check_clean=False,
+              ignore_zero=True,
+              backup_files=not no_backup,
+              file_list=file_list)
+        if not file_list:
+            Branch.open_containing('.').set_pending_merges([])
+
+
+class cmd_assert_fail(Command):
+    """Test reporting of assertion failures"""
+    hidden = True
+    def run(self):
+        assert False, "always fails"
+
+
+class cmd_help(Command):
+    """Show help on a command or other topic.
+
+    For a list of all available commands, say 'bzr help commands'."""
+    takes_options = ['long']
+    takes_args = ['topic?']
+    aliases = ['?']
+    
+    def run(self, topic=None, long=False):
+        import help
+        if topic is None and long:
+            topic = "commands"
+        help.help(topic)
+
+
+class cmd_shell_complete(Command):
+    """Show appropriate completions for context.
+
+    For a list of all available commands, say 'bzr shell-complete'."""
+    takes_args = ['context?']
+    aliases = ['s-c']
+    hidden = True
+    
+    def run(self, context=None):
+        import shellcomplete
+        shellcomplete.shellcomplete(context)
+
+
+class cmd_fetch(Command):
+    """Copy in history from another branch but don't merge it.
+
+    This is an internal method used for pull and merge."""
+    hidden = True
+    takes_args = ['from_branch', 'to_branch']
+    def run(self, from_branch, to_branch):
+        from bzrlib.fetch import Fetcher
+        from bzrlib.branch import Branch
+        from_b = Branch(from_branch)
+        to_b = Branch(to_branch)
+        Fetcher(to_b, from_b)
+        
+
+
+class cmd_missing(Command):
+    """What is missing in this branch relative to other branch.
+    """
+    # TODO: rewrite this in terms of ancestry so that it shows only
+    # unmerged things
+    
+    takes_args = ['remote?']
+    aliases = ['mis', 'miss']
+    # We don't have to add quiet to the list, because 
+    # unknown options are parsed as booleans
+    takes_options = ['verbose', 'quiet']
+
+    def run(self, remote=None, verbose=False, quiet=False):
+        from bzrlib.errors import BzrCommandError
+        from bzrlib.missing import show_missing
+
+        if verbose and quiet:
+            raise BzrCommandError('Cannot pass both quiet and verbose')
+
+        b = Branch.open_containing('.')
+        parent = b.get_parent()
+        if remote is None:
+            if parent is None:
+                raise BzrCommandError("No missing location known or specified.")
+            else:
+                if not quiet:
+                    print "Using last location: %s" % parent
+                remote = parent
+        elif parent is None:
+            # We only update parent if it did not exist, missing
+            # should not change the parent
+            b.set_parent(remote)
+        br_remote = Branch.open_containing(remote)
+        return show_missing(b, br_remote, verbose=verbose, quiet=quiet)
+
+
+class cmd_plugins(Command):
+    """List plugins"""
+    hidden = True
+    def run(self):
+        import bzrlib.plugin
+        from inspect import getdoc
+        for plugin in bzrlib.plugin.all_plugins:
+            if hasattr(plugin, '__path__'):
+                print plugin.__path__[0]
+            elif hasattr(plugin, '__file__'):
+                print plugin.__file__
+            else:
+                print `plugin`
+                
+            d = getdoc(plugin)
+            if d:
+                print '\t', d.split('\n')[0]
+
+

=== modified file 'bzrlib/changeset.py'
--- bzrlib/changeset.py	2005-07-22 19:54:37 +0000
+++ bzrlib/changeset.py	2005-10-03 19:50:36 +0000
@@ -18,9 +18,18 @@
 import patch
 import stat
 from bzrlib.trace import mutter
-"""
-Represent and apply a changeset
-"""
+from bzrlib.osutils import rename
+import bzrlib
+
+# XXX: mbp: I'm not totally convinced that we should handle conflicts
+# as part of changeset application, rather than only in the merge
+# operation.
+
+"""Represent and apply a changeset
+
+Conflicts in applying a changeset are represented as exceptions.
+"""
+
 __docformat__ = "restructuredtext"
 
 NULL_ID = "!NULL"
@@ -35,56 +44,7 @@
         newdict[value] = key
     return newdict
 
-
-class PatchApply(object):
-    """Patch application as a kind of content change"""
-    def __init__(self, contents):
-        """Constructor.
-
-        :param contents: The text of the patch to apply
-        :type contents: str"""
-        self.contents = contents
-
-    def __eq__(self, other):
-        if not isinstance(other, PatchApply):
-            return False
-        elif self.contents != other.contents:
-            return False
-        else:
-            return True
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def apply(self, filename, conflict_handler, reverse=False):
-        """Applies the patch to the specified file.
-
-        :param filename: the file to apply the patch to
-        :type filename: str
-        :param reverse: If true, apply the patch in reverse
-        :type reverse: bool
-        """
-        input_name = filename+".orig"
-        try:
-            os.rename(filename, input_name)
-        except OSError, e:
-            if e.errno != errno.ENOENT:
-                raise
-            if conflict_handler.patch_target_missing(filename, self.contents)\
-                == "skip":
-                return
-            os.rename(filename, input_name)
-            
-
-        status = patch.patch(self.contents, input_name, filename, 
-                                    reverse)
-        os.chmod(filename, os.stat(input_name).st_mode)
-        if status == 0:
-            os.unlink(input_name)
-        elif status == 1:
-            conflict_handler.failed_hunks(filename)
-
-        
+       
 class ChangeUnixPermissions(object):
     """This is two-way change, suitable for file modification, creation,
     deletion"""
@@ -133,6 +93,7 @@
     def __ne__(self, other):
         return not (self == other)
 
+
 def dir_create(filename, conflict_handler, reverse):
     """Creates the directory, or deletes it if reverse is true.  Intended to be
     used with ReplaceContents.
@@ -158,14 +119,12 @@
         try:
             os.rmdir(filename)
         except OSError, e:
-            if e.errno != 39:
+            if e.errno != errno.ENOTEMPTY:
                 raise
             if conflict_handler.rmdir_non_empty(filename) == "skip":
                 return
             os.rmdir(filename)
 
-                
-            
 
 class SymlinkCreate(object):
     """Creates or deletes a symlink (for use with ReplaceContents)"""
@@ -367,35 +326,46 @@
 
 
 class Diff3Merge(object):
-    def __init__(self, base_file, other_file):
-        self.base_file = base_file
-        self.other_file = other_file
+    def __init__(self, file_id, base, other):
+        self.file_id = file_id
+        self.base = base
+        self.other = other
 
     def __eq__(self, other):
         if not isinstance(other, Diff3Merge):
             return False
-        return (self.base_file == other.base_file and 
-                self.other_file == other.other_file)
+        return (self.base == other.base and 
+                self.other == other.other and self.file_id == other.file_id)
 
     def __ne__(self, other):
         return not (self == other)
 
     def apply(self, filename, conflict_handler, reverse=False):
-        new_file = filename+".new" 
+        new_file = filename+".new"
+        base_file = self.base.readonly_path(self.file_id)
+        other_file = self.other.readonly_path(self.file_id)
         if not reverse:
-            base = self.base_file
-            other = self.other_file
+            base = base_file
+            other = other_file
         else:
-            base = self.other_file
-            other = self.base_file
+            base = other_file
+            other = base_file
         status = patch.diff3(new_file, filename, base, other)
         if status == 0:
             os.chmod(new_file, os.stat(filename).st_mode)
-            os.rename(new_file, filename)
+            rename(new_file, filename)
             return
         else:
             assert(status == 1)
-            conflict_handler.merge_conflict(new_file, filename, base, other)
+            def get_lines(filename):
+                my_file = file(filename, "rb")
+                lines = my_file.readlines()
+                my_file.close()
+                return lines
+            base_lines = get_lines(base)
+            other_lines = get_lines(other)
+            conflict_handler.merge_conflict(new_file, filename, base_lines, 
+                                            other_lines)
 
 
 def CreateDir():
@@ -657,7 +627,7 @@
                 return None
             return self.path
 
-    def summarize_name(self, changeset, reverse=False):
+    def summarize_name(self, reverse=False):
         """Produce a one-line summary of the filename.  Indicates renames as
         old => new, indicates creation as None => new, indicates deletion as
         old => None.
@@ -694,7 +664,7 @@
         :type reverse: bool
         :rtype: str
         """
-        mutter("Finding new path for %s" % self.summarize_name(changeset))
+        mutter("Finding new path for %s" % self.summarize_name())
         if reverse:
             parent = self.parent
             to_dir = self.dir
@@ -862,7 +832,7 @@
             if src_path is not None:
                 src_path = os.path.join(dir, src_path)
                 try:
-                    os.rename(src_path, to_name)
+                    rename(src_path, to_name)
                     temp_name[entry.id] = to_name
                 except OSError, e:
                     if e.errno != errno.ENOENT:
@@ -894,7 +864,7 @@
             continue
         new_path = os.path.join(dir, new_tree_path)
         old_path = changed_inventory.get(entry.id)
-        if os.path.exists(new_path):
+        if bzrlib.osutils.lexists(new_path):
             if conflict_handler.target_exists(entry, new_path, old_path) == \
                 "skip":
                 continue
@@ -905,7 +875,7 @@
             if old_path is None:
                 continue
             try:
-                os.rename(old_path, new_path)
+                rename(old_path, new_path)
                 changed_inventory[entry.id] = new_tree_path
             except OSError, e:
                 raise Exception ("%s is missing" % new_path)
@@ -1028,9 +998,12 @@
 
 
 class ExceptionConflictHandler(object):
-    def __init__(self, dir):
-        self.dir = dir
-    
+    """Default handler for merge exceptions.
+
+    This throws an error on any kind of conflict.  Conflict handlers can
+    descend from this class if they have a better way to handle some or
+    all types of conflict.
+    """
     def missing_parent(self, pathname):
         parent = os.path.dirname(pathname)
         raise Exception("Parent directory missing for %s" % pathname)
@@ -1047,13 +1020,10 @@
     def rename_conflict(self, id, this_name, base_name, other_name):
         raise RenameConflict(id, this_name, base_name, other_name)
 
-    def move_conflict(self, id, inventory):
-        this_dir = inventory.this.get_dir(id)
-        base_dir = inventory.base.get_dir(id)
-        other_dir = inventory.other.get_dir(id)
+    def move_conflict(self, id, this_dir, base_dir, other_dir):
         raise MoveConflict(id, this_dir, base_dir, other_dir)
 
-    def merge_conflict(self, new_file, this_path, base_path, other_path):
+    def merge_conflict(self, new_file, this_path, base_lines, other_lines):
         os.unlink(new_file)
         raise MergeConflict(this_path)
 
@@ -1087,13 +1057,13 @@
     def missing_for_rename(self, filename):
         raise MissingForRename(filename)
 
-    def missing_for_merge(self, file_id, inventory):
-        raise MissingForMerge(inventory.other.get_path(file_id))
+    def missing_for_merge(self, file_id, other_path):
+        raise MissingForMerge(other_path)
 
     def new_contents_conflict(self, filename, other_contents):
         raise NewContentsConflict(filename)
 
-    def finalize():
+    def finalize(self):
         pass
 
 def apply_changeset(changeset, inventory, dir, conflict_handler=None, 
@@ -1112,7 +1082,7 @@
     :rtype: Dictionary
     """
     if conflict_handler is None:
-        conflict_handler = ExceptionConflictHandler(dir)
+        conflict_handler = ExceptionConflictHandler()
     temp_dir = os.path.join(dir, "bzr-tree-change")
     try:
         os.mkdir(temp_dir)
@@ -1315,49 +1285,30 @@
         self.full_path = full_path
         self.stat_result = stat_result
 
-def generate_changeset(tree_a, tree_b, inventory_a=None, inventory_b=None):
-    return ChangesetGenerator(tree_a, tree_b, inventory_a, inventory_b)()
+def generate_changeset(tree_a, tree_b, interesting_ids=None):
+    return ChangesetGenerator(tree_a, tree_b, interesting_ids)()
 
 class ChangesetGenerator(object):
-    def __init__(self, tree_a, tree_b, inventory_a=None, inventory_b=None):
+    def __init__(self, tree_a, tree_b, interesting_ids=None):
         object.__init__(self)
         self.tree_a = tree_a
         self.tree_b = tree_b
-        if inventory_a is not None:
-            self.inventory_a = inventory_a
-        else:
-            self.inventory_a = tree_a.inventory()
-        if inventory_b is not None:
-            self.inventory_b = inventory_b
-        else:
-            self.inventory_b = tree_b.inventory()
-        self.r_inventory_a = self.reverse_inventory(self.inventory_a)
-        self.r_inventory_b = self.reverse_inventory(self.inventory_b)
+        self._interesting_ids = interesting_ids
 
-    def reverse_inventory(self, inventory):
-        r_inventory = {}
-        for entry in inventory.itervalues():
-            if entry.id is None:
-                continue
-            r_inventory[entry.id] = entry
-        return r_inventory
+    def iter_both_tree_ids(self):
+        for file_id in self.tree_a:
+            yield file_id
+        for file_id in self.tree_b:
+            if file_id not in self.tree_a:
+                yield file_id
 
     def __call__(self):
         cset = Changeset()
-        for entry in self.inventory_a.itervalues():
-            if entry.id is None:
-                continue
-            cs_entry = self.make_entry(entry.id)
+        for file_id in self.iter_both_tree_ids():
+            cs_entry = self.make_entry(file_id)
             if cs_entry is not None and not cs_entry.is_boring():
                 cset.add_entry(cs_entry)
 
-        for entry in self.inventory_b.itervalues():
-            if entry.id is None:
-                continue
-            if not self.r_inventory_a.has_key(entry.id):
-                cs_entry = self.make_entry(entry.id)
-                if cs_entry is not None and not cs_entry.is_boring():
-                    cset.add_entry(cs_entry)
         for entry in list(cset.entries.itervalues()):
             if entry.parent != entry.new_parent:
                 if not cset.entries.has_key(entry.parent) and\
@@ -1371,54 +1322,58 @@
                     cset.add_entry(parent_entry)
         return cset
 
-    def get_entry_parent(self, entry, inventory):
-        if entry is None:
-            return None
-        if entry.path == "./.":
-            return NULL_ID
-        dirname = os.path.dirname(entry.path)
-        if dirname == ".":
-            dirname = "./."
-        parent = inventory[dirname]
-        return parent.id
-
-    def get_paths(self, entry, tree):
-        if entry is None:
-            return (None, None)
-        full_path = tree.readonly_path(entry.id)
-        if entry.path == ".":
-            return ("", full_path)
-        return (entry.path, full_path)
-
-    def make_basic_entry(self, id, only_interesting):
-        entry_a = self.r_inventory_a.get(id)
-        entry_b = self.r_inventory_b.get(id)
+    def iter_inventory(self, tree):
+        for file_id in tree:
+            yield self.get_entry(file_id, tree)
+
+    def get_entry(self, file_id, tree):
+        if not tree.has_or_had_id(file_id):
+            return None
+        return tree.tree.inventory[file_id]
+
+    def get_entry_parent(self, entry):
+        if entry is None:
+            return None
+        return entry.parent_id
+
+    def get_path(self, file_id, tree):
+        if not tree.has_or_had_id(file_id):
+            return None
+        path = tree.id2path(file_id)
+        if path == '':
+            return './.'
+        else:
+            return path
+
+    def make_basic_entry(self, file_id, only_interesting):
+        entry_a = self.get_entry(file_id, self.tree_a)
+        entry_b = self.get_entry(file_id, self.tree_b)
         if only_interesting and not self.is_interesting(entry_a, entry_b):
-            return (None, None, None)
-        parent = self.get_entry_parent(entry_a, self.inventory_a)
-        (path, full_path_a) = self.get_paths(entry_a, self.tree_a)
-        cs_entry = ChangesetEntry(id, parent, path)
-        new_parent = self.get_entry_parent(entry_b, self.inventory_b)
-
-
-        (new_path, full_path_b) = self.get_paths(entry_b, self.tree_b)
+            return None
+        parent = self.get_entry_parent(entry_a)
+        path = self.get_path(file_id, self.tree_a)
+        cs_entry = ChangesetEntry(file_id, parent, path)
+        new_parent = self.get_entry_parent(entry_b)
+
+        new_path = self.get_path(file_id, self.tree_b)
 
         cs_entry.new_path = new_path
         cs_entry.new_parent = new_parent
-        return (cs_entry, full_path_a, full_path_b)
+        return cs_entry
 
     def is_interesting(self, entry_a, entry_b):
+        if self._interesting_ids is None:
+            return True
         if entry_a is not None:
-            if entry_a.interesting:
-                return True
-        if entry_b is not None:
-            if entry_b.interesting:
-                return True
-        return False
+            file_id = entry_a.file_id
+        elif entry_b is not None:
+            file_id = entry_b.file_id
+        else:
+            return False
+        return file_id in self._interesting_ids
 
     def make_boring_entry(self, id):
-        (cs_entry, full_path_a, full_path_b) = \
-            self.make_basic_entry(id, only_interesting=False)
+        cs_entry = self.make_basic_entry(id, only_interesting=False)
         if cs_entry.is_creation_or_deletion():
             return self.make_entry(id, only_interesting=False)
         else:
@@ -1426,19 +1381,24 @@
         
 
     def make_entry(self, id, only_interesting=True):
-        (cs_entry, full_path_a, full_path_b) = \
-            self.make_basic_entry(id, only_interesting)
+        cs_entry = self.make_basic_entry(id, only_interesting)
 
         if cs_entry is None:
             return None
-       
+
+        full_path_a = self.tree_a.readonly_path(id)
+        full_path_b = self.tree_b.readonly_path(id)
         stat_a = self.lstat(full_path_a)
         stat_b = self.lstat(full_path_b)
-        if stat_b is None:
-            cs_entry.new_parent = None
-            cs_entry.new_path = None
-        
+
         cs_entry.metadata_change = self.make_mode_change(stat_a, stat_b)
+
+        if id in self.tree_a and id in self.tree_b:
+            a_sha1 = self.tree_a.get_file_sha1(id)
+            b_sha1 = self.tree_b.get_file_sha1(id)
+            if None not in (a_sha1, b_sha1) and a_sha1 == b_sha1:
+                return cs_entry
+
         cs_entry.contents_change = self.make_contents_change(full_path_a,
                                                              stat_a, 
                                                              full_path_b, 
@@ -1467,15 +1427,6 @@
             if stat_a.st_ino == stat_b.st_ino and \
                 stat_a.st_dev == stat_b.st_dev:
                 return None
-            if file(full_path_a, "rb").read() == \
-                file(full_path_b, "rb").read():
-                return None
-
-            patch_contents = patch.diff(full_path_a, 
-                                        file(full_path_b, "rb").read())
-            if patch_contents is None:
-                return None
-            return PatchApply(patch_contents)
 
         a_contents = self.get_contents(stat_a, full_path_a)
         b_contents = self.get_contents(stat_b, full_path_b)

=== modified file 'bzrlib/check.py'
--- bzrlib/check.py	2005-07-11 07:33:45 +0000
+++ bzrlib/check.py	2005-09-30 02:54:51 +0000
@@ -15,220 +15,150 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-def _update_store_entry(obj, obj_id, branch, store_name, store):
-    """This is just a meta-function, which handles both revision entries
-    and inventory entries.
-    """
-    from bzrlib.trace import mutter
-    import tempfile, os, errno
-    from osutils import rename
-    obj_tmp = tempfile.TemporaryFile()
-    obj.write_xml(obj_tmp)
-    obj_tmp.seek(0)
-
-    tmpfd, tmp_path = tempfile.mkstemp(prefix=obj_id, suffix='.gz',
-        dir=branch.controlfilename(store_name))
-    os.close(tmpfd)
-    try:
-        orig_obj_path = branch.controlfilename([store_name, obj_id+'.gz'])
-        # Remove the old entry out of the way
-        rename(orig_obj_path, tmp_path)
+# TODO: Check ancestries are correct for every revision: includes
+# every committed so far, and in a reasonable order.
+
+# TODO: Also check non-mainline revisions mentioned as parents.
+
+# TODO: Check for extra files in the control directory.
+
+# TODO: Check revision, inventory and entry objects have all 
+# required fields.
+
+
+import bzrlib.ui
+from bzrlib.trace import note, warning
+from bzrlib.osutils import rename, sha_string, fingerprint_file
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrCheckError, NoSuchRevision
+from bzrlib.inventory import ROOT_ID
+from bzrlib.branch import gen_root_id
+
+
+class Check(object):
+    """Check a branch"""
+    def __init__(self, branch):
+        self.branch = branch
+        branch.lock_read()
         try:
-            # TODO: We may need to handle the case where the old
-            # entry was not compressed (and thus did not end with .gz)
-
-            store.add(obj_tmp, obj_id) # Add the new one
-            os.remove(tmp_path) # Remove the old name
-            mutter('    Updated %s entry {%s}' % (store_name, obj_id))
-        except:
-            # On any exception, restore the old entry
-            rename(tmp_path, orig_obj_path)
-            raise
-    finally:
-        if os.path.exists(tmp_path):
-            # Unfortunately, the next command might throw
-            # an exception, which will mask a previous exception.
-            os.remove(tmp_path)
-        obj_tmp.close()
-
-def _update_revision_entry(rev, branch):
-    """After updating the values in a revision, make sure to
-    write out the data, but try to do it in an atomic manner.
-
-    :param rev:    The Revision object to store
-    :param branch: The Branch object where this Revision is to be stored.
-    """
-    _update_store_entry(rev, rev.revision_id, branch,
-            'revision-store', branch.revision_store)
-
-def _update_inventory_entry(inv, inv_id, branch):
-    """When an inventory has been modified (such as by adding a unique tree root)
-    this atomically re-generates the file.
-
-    :param inv:     The Inventory
-    :param inv_id:  The inventory id for this inventory
-    :param branch:  The Branch where this entry will be stored.
-    """
-    _update_store_entry(inv, inv_id, branch,
-            'inventory-store', branch.inventory_store)
-
-def check(branch):
-    """Run consistency checks on a branch.
-
-    TODO: Also check non-mainline revisions mentioned as parents.
-
-    TODO: Check for extra files in the control directory.
-    """
-    from bzrlib.trace import mutter
-    from bzrlib.errors import BzrCheckError
-    from bzrlib.osutils import fingerprint_file
-    from bzrlib.progress import ProgressBar
-    from bzrlib.inventory import ROOT_ID
-    from bzrlib.branch import gen_root_id
-
-    branch.lock_read()
-
-    try:
-        pb = ProgressBar(show_spinner=True)
-        last_rev_id = None
-
-        missing_inventory_sha_cnt = 0
-        missing_revision_sha_cnt = 0
+            branch.weave_store.enable_cache = True
+            branch.control_weaves.enable_cache = True
+            self.run()
+        finally:
+            branch.unlock()
+            branch.weave_store.enable_cache = False
+            branch.control_weaves.enable_cache = False
+
+
+    def run(self):
+        branch = self.branch
+
+
+        self.checked_text_cnt = 0
+        self.checked_rev_cnt = 0
+        self.repeated_text_cnt = 0
+        self.missing_inventory_sha_cnt = 0
+        self.missing_revision_cnt = 0
+        # maps (file-id, version) -> sha1
+        self.checked_texts = {}
 
         history = branch.revision_history()
         revno = 0
         revcount = len(history)
-        mismatch_inv_id = []
-
-        # for all texts checked, text_id -> sha1
-        checked_texts = {}
-
+
+        last_rev_id = None
+        self.progress = bzrlib.ui.ui_factory.progress_bar()
         for rev_id in history:
+            self.progress.update('checking revision', revno, revcount)
             revno += 1
-            pb.update('checking revision', revno, revcount)
-            mutter('    revision {%s}' % rev_id)
-            rev = branch.get_revision(rev_id)
-            if rev.revision_id != rev_id:
-                raise BzrCheckError('wrong internal revision id in revision {%s}'
-                                    % rev_id)
-
-            # check the previous history entry is a parent of this entry
-            if rev.parents:
-                if last_rev_id is None:
-                    raise BzrCheckError("revision {%s} has %d parents, but is the "
-                                        "start of the branch"
-                                        % (rev_id, len(rev.parents)))
-                for prr in rev.parents:
-                    if prr.revision_id == last_rev_id:
-                        break
-                else:
-                    raise BzrCheckError("previous revision {%s} not listed among "
-                                        "parents of {%s}"
-                                        % (last_rev_id, rev_id))
-
-                for prr in rev.parents:
-                    if prr.revision_sha1 is None:
-                        missing_revision_sha_cnt += 1
-                        continue
-                    prid = prr.revision_id
-                    actual_sha = branch.get_revision_sha1(prid)
-                    if prr.revision_sha1 != actual_sha:
-                        raise BzrCheckError("mismatched revision sha1 for "
-                                            "parent {%s} of {%s}: %s vs %s"
-                                            % (prid, rev_id,
-                                               prr.revision_sha1, actual_sha))
-            elif last_rev_id:
-                raise BzrCheckError("revision {%s} has no parents listed but preceded "
-                                    "by {%s}"
-                                    % (rev_id, last_rev_id))
-
-            if rev.inventory_id != rev_id:
-                mismatch_inv_id.append(rev_id)
-
-            ## TODO: Check all the required fields are present on the revision.
-
-            if rev.inventory_sha1:
-                inv_sha1 = branch.get_inventory_sha1(rev.inventory_id)
-                if inv_sha1 != rev.inventory_sha1:
-                    raise BzrCheckError('Inventory sha1 hash doesn\'t match'
-                        ' value in revision {%s}' % rev_id)
+            self.check_one_rev(rev_id, last_rev_id)
+            last_rev_id = rev_id
+        self.progress.clear()
+        self.report_results()
+
+
+    def report_results(self):
+        note('checked branch %s format %d',
+             self.branch.base, 
+             self.branch._branch_format)
+
+        note('%6d revisions', self.checked_rev_cnt)
+        note('%6d unique file texts', self.checked_text_cnt)
+        note('%6d repeated file texts', self.repeated_text_cnt)
+        if self.missing_inventory_sha_cnt:
+            note('%d revisions are missing inventory_sha1',
+                 self.missing_inventory_sha_cnt)
+        if self.missing_revision_cnt:
+            note('%d revisions are mentioned but not present',
+                 self.missing_revision_cnt)
+
+
+    def check_one_rev(self, rev_id, last_rev_id):
+        """Check one revision.
+
+        rev_id - the one to check
+
+        last_rev_id - the previous one on the mainline, if any.
+        """
+
+        # mutter('    revision {%s}' % rev_id)
+        branch = self.branch
+        rev = branch.get_revision(rev_id)
+        if rev.revision_id != rev_id:
+            raise BzrCheckError('wrong internal revision id in revision {%s}'
+                                % rev_id)
+
+        # check the previous history entry is a parent of this entry
+        if rev.parent_ids:
+            if last_rev_id is None:
+                raise BzrCheckError("revision {%s} has %d parents, but is the "
+                                    "start of the branch"
+                                    % (rev_id, len(rev.parent_ids)))
+            for parent_id in rev.parent_ids:
+                if parent_id == last_rev_id:
+                    break
             else:
-                missing_inventory_sha_cnt += 1
-                mutter("no inventory_sha1 on revision {%s}" % rev_id)
-
-            inv = branch.get_inventory(rev.inventory_id)
-            seen_ids = {}
-            seen_names = {}
-
-            ## p('revision %d/%d file ids' % (revno, revcount))
-            for file_id in inv:
-                if file_id in seen_ids:
-                    raise BzrCheckError('duplicated file_id {%s} '
-                                        'in inventory for revision {%s}'
-                                        % (file_id, rev_id))
-                seen_ids[file_id] = True
-
-            i = 0
-            for file_id in inv:
-                i += 1
-                if i & 31 == 0:
-                    pb.tick()
-
-                ie = inv[file_id]
-
-                if ie.parent_id != None:
-                    if ie.parent_id not in seen_ids:
-                        raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
-                                % (ie.parent_id, rev_id))
-
-                if ie.kind == 'file':
-                    if ie.text_id in checked_texts:
-                        fp = checked_texts[ie.text_id]
-                    else:
-                        if not ie.text_id in branch.text_store:
-                            raise BzrCheckError('text {%s} not in text_store' % ie.text_id)
-
-                        tf = branch.text_store[ie.text_id]
-                        fp = fingerprint_file(tf)
-                        checked_texts[ie.text_id] = fp
-
-                    if ie.text_size != fp['size']:
-                        raise BzrCheckError('text {%s} wrong size' % ie.text_id)
-                    if ie.text_sha1 != fp['sha1']:
-                        raise BzrCheckError('text {%s} wrong sha1' % ie.text_id)
-                elif ie.kind == 'directory':
-                    if ie.text_sha1 != None or ie.text_size != None or ie.text_id != None:
-                        raise BzrCheckError('directory {%s} has text in revision {%s}'
-                                % (file_id, rev_id))
-
-            pb.tick()
-            for path, ie in inv.iter_entries():
-                if path in seen_names:
-                    raise BzrCheckError('duplicated path %s '
-                                        'in inventory for revision {%s}'
-                                        % (path, rev_id))
+                raise BzrCheckError("previous revision {%s} not listed among "
+                                    "parents of {%s}"
+                                    % (last_rev_id, rev_id))
+        elif last_rev_id:
+            raise BzrCheckError("revision {%s} has no parents listed "
+                                "but preceded by {%s}"
+                                % (rev_id, last_rev_id))
+
+        if rev.inventory_sha1:
+            inv_sha1 = branch.get_inventory_sha1(rev_id)
+            if inv_sha1 != rev.inventory_sha1:
+                raise BzrCheckError('Inventory sha1 hash doesn\'t match'
+                    ' value in revision {%s}' % rev_id)
+        else:
+            missing_inventory_sha_cnt += 1
+            mutter("no inventory_sha1 on revision {%s}" % rev_id)
+        self._check_revision_tree(rev_id)
+        self.checked_rev_cnt += 1
+
+    def _check_revision_tree(self, rev_id):
+        tree = self.branch.revision_tree(rev_id)
+        inv = tree.inventory
+        seen_ids = {}
+        for file_id in inv:
+            if file_id in seen_ids:
+                raise BzrCheckError('duplicated file_id {%s} '
+                                    'in inventory for revision {%s}'
+                                    % (file_id, rev_id))
+            seen_ids[file_id] = True
+        for file_id in inv:
+            ie = inv[file_id]
+            ie.check(self, rev_id, inv, tree)
+        seen_names = {}
+        for path, ie in inv.iter_entries():
+            if path in seen_names:
+                raise BzrCheckError('duplicated path %s '
+                                    'in inventory for revision {%s}'
+                                    % (path, rev_id))
             seen_names[path] = True
-            last_rev_id = rev_id
-
-    finally:
-        branch.unlock()
-
-    pb.clear()
-
-    print 'checked %d revisions, %d file texts' % (revcount, len(checked_texts))
-    
-    if missing_inventory_sha_cnt:
-        print '%d revisions are missing inventory_sha1' % missing_inventory_sha_cnt
-
-    if missing_revision_sha_cnt:
-        print '%d parent links are missing revision_sha1' % missing_revision_sha_cnt
-
-    if (missing_inventory_sha_cnt
-        or missing_revision_sha_cnt):
-        print '  (use "bzr upgrade" to fix them)'
-
-    if mismatch_inv_id:
-        print '%d revisions have mismatched inventory ids:' % len(mismatch_inv_id)
-        for rev_id in mismatch_inv_id:
-            print '  ', rev_id
+
+
+def check(branch):
+    """Run consistency checks on a branch."""
+    Check(branch)

=== added file 'bzrlib/clone.py'
--- bzrlib/clone.py	1970-01-01 00:00:00 +0000
+++ bzrlib/clone.py	2005-10-06 06:31:33 +0000
@@ -0,0 +1,157 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Make a copy of an entire branch and all its history.
+
+This is the underlying function for the branch/get/clone commands."""
+
+# TODO: This could be done *much* more efficiently by just copying
+# all the whole weaves and revisions, rather than getting one
+# revision at a time.
+
+# TODO: Optionally, after copying, discard any irrelevant information from
+# the destination, such as revisions committed after the last one we're interested 
+# in.  This needs to apply a weave prune operation (not written yet) to each
+# weave one by one.
+
+# Copying must be done in a way that supports http transports, where we
+# can't list a directory, and therefore have to rely on information
+# retrieved from top-level objects whose names we do know.
+#
+# In practice this means we first fetch the revision history and ancestry.
+# These give us a list of all the revisions that need to be fetched.  We 
+# also get the inventory weave.  We then just need to get a list of all 
+# file-ids ever referenced by this tree.  (It might be nice to keep a list
+# of them directly.)  This is done by walking over the inventories of all
+# copied revisions and accumulating a list of file ids.
+#
+# For local branches it is possible to optimize this considerably in two
+# ways.  One is to hardlink the files (if possible and requested), rather
+# than copying them.  Another is to simply list the directory rather than
+# walking through the inventories to find out what files are present -- but
+# there it may be better to just be consistent with remote branches.
+
+import os
+import sys
+
+import bzrlib
+from bzrlib.merge import build_working_dir
+from bzrlib.branch import Branch
+from bzrlib.trace import mutter, note
+from bzrlib.store import copy_all
+
+def copy_branch(branch_from, to_location, revision=None, basis_branch=None):
+    """Copy branch_from into the existing directory to_location.
+
+    Returns the newly created branch object.
+
+    revision
+        If not None, only revisions up to this point will be copied.
+        The head of the new branch will be that revision.  Must be a
+        revid or None.
+
+    to_location -- The destination directory; must either exist and be 
+        empty, or not exist, in which case it is created.
+
+    basis_branch
+        A local branch to copy revisions from, related to branch_from. 
+        This is used when branching from a remote (slow) branch, and we have
+        a local branch that might contain some relevant revisions.
+    """
+    assert isinstance(branch_from, Branch)
+    assert isinstance(to_location, basestring)
+    if basis_branch is not None:
+        note("basis_branch is not supported for fast weave copy yet.")
+    if not (branch_from.weave_store.listable()
+            and branch_from.revision_store.listable()):
+        return copy_branch_slower(branch_from, to_location, revision,
+                                  basis_branch)
+    history = _get_truncated_history(branch_from, revision)
+    if not bzrlib.osutils.lexists(to_location):
+        os.mkdir(to_location)
+    branch_to = Branch.initialize(to_location)
+    mutter("copy branch from %s to %s", branch_from, branch_to)
+    branch_to.set_root_id(branch_from.get_root_id())
+    branch_to.append_revision(*history)
+    _copy_control_weaves(branch_from, branch_to)
+    _copy_text_weaves(branch_from, branch_to)
+    _copy_revision_store(branch_from, branch_to)
+    build_working_dir(to_location)
+    branch_to.set_parent(branch_from.base)
+    mutter("copied")
+    return branch_to
+
+
+def _get_truncated_history(branch_from, revision):
+    history = branch_from.revision_history()
+    if revision is None:
+        return history
+    try:
+        idx = history.index(revision)
+    except ValueError:
+        raise InvalidRevisionId('revision {%s} is not on the mainline of %s' 
+                                % (revision, branch_from))
+    return history[:idx+1]
+
+def _copy_text_weaves(branch_from, branch_to):
+    copy_all(branch_from.weave_store, branch_to.weave_store)
+
+
+def _copy_revision_store(branch_from, branch_to):
+    copy_all(branch_from.revision_store, branch_to.revision_store)
+
+
+def _copy_control_weaves(branch_from, branch_to):
+    to_control = branch_to.control_weaves
+    from_control = branch_from.control_weaves
+    to_control.copy_multi(from_control, ['inventory'])
+
+    
+def copy_branch_slower(branch_from, to_location, revision=None, basis_branch=None):
+    """Copy branch_from into the existing directory to_location.
+
+    revision
+        If not None, only revisions up to this point will be copied.
+        The head of the new branch will be that revision.  Must be a
+        revid or None.
+
+    to_location -- The destination directory; must either exist and be 
+        empty, or not exist, in which case it is created.
+
+    revno
+        The revision to copy up to
+
+    basis_branch
+        A local branch to copy revisions from, related to branch_from. 
+        This is used when branching from a remote (slow) branch, and we have
+        a local branch that might contain some relevant revisions.
+    """
+    assert isinstance(branch_from, Branch)
+    assert isinstance(to_location, basestring)
+    if not bzrlib.osutils.lexists(to_location):
+        os.mkdir(to_location)
+    br_to = Branch.initialize(to_location)
+    mutter("copy branch from %s to %s", branch_from, br_to)
+    if basis_branch is not None:
+        basis_branch.push_stores(br_to)
+    br_to.set_root_id(branch_from.get_root_id())
+    if revision is None:
+        revision = branch_from.last_revision()
+    br_to.update_revisions(branch_from, stop_revision=revision)
+    build_working_dir(to_location)
+    br_to.set_parent(branch_from.base)
+    mutter("copied")
+    return br_to

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2005-07-23 14:06:37 +0000
+++ bzrlib/commands.py	2005-10-06 13:08:14 +0000
@@ -15,16 +15,30 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-import sys, os
+# TODO: probably should say which arguments are candidates for glob
+# expansion on windows and do that at the command level.
+
+# TODO: Help messages for options.
+
+# TODO: Define arguments by objects, rather than just using names.
+# Those objects can specify the expected type of the argument, which
+# would help with validation and shell completion.
+
+# TODO: "--profile=cum", to change sort order.  Is there any value in leaving
+# the profile output behind so it can be interactively examined?
+
+import sys
+import os
+from warnings import warn
+from inspect import getdoc
 
 import bzrlib
+import bzrlib.trace
 from bzrlib.trace import mutter, note, log_error, warning
-from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError
-from bzrlib.branch import find_branch
+from bzrlib.errors import BzrError, BzrCheckError, BzrCommandError, NotBranchError
+from bzrlib.revisionspec import RevisionSpec
 from bzrlib import BZRDIR
 
-
 plugin_cmds = {}
 
 
@@ -38,6 +52,7 @@
         k_unsquished = k
     if not plugin_cmds.has_key(k_unsquished):
         plugin_cmds[k_unsquished] = cmd
+        mutter('registered plugin command %s', k_unsquished)      
     else:
         log_error('Two plugins defined the same command: %r' % k)
         log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
@@ -55,86 +70,97 @@
 def _parse_revision_str(revstr):
     """This handles a revision string -> revno.
 
-    This always returns a list.  The list will have one element for 
-
-    It supports integers directly, but everything else it
-    defers for passing to Branch.get_revision_info()
+    This always returns a list.  The list will have one element for
+    each revision.
 
     >>> _parse_revision_str('234')
-    [234]
+    [<RevisionSpec_int 234>]
     >>> _parse_revision_str('234..567')
-    [234, 567]
+    [<RevisionSpec_int 234>, <RevisionSpec_int 567>]
     >>> _parse_revision_str('..')
-    [None, None]
+    [<RevisionSpec None>, <RevisionSpec None>]
     >>> _parse_revision_str('..234')
-    [None, 234]
+    [<RevisionSpec None>, <RevisionSpec_int 234>]
     >>> _parse_revision_str('234..')
-    [234, None]
+    [<RevisionSpec_int 234>, <RevisionSpec None>]
     >>> _parse_revision_str('234..456..789') # Maybe this should be an error
-    [234, 456, 789]
+    [<RevisionSpec_int 234>, <RevisionSpec_int 456>, <RevisionSpec_int 789>]
     >>> _parse_revision_str('234....789') # Error?
-    [234, None, 789]
+    [<RevisionSpec_int 234>, <RevisionSpec None>, <RevisionSpec_int 789>]
     >>> _parse_revision_str('revid:test@other.com-234234')
-    ['revid:test@other.com-234234']
+    [<RevisionSpec_revid revid:test@other.com-234234>]
     >>> _parse_revision_str('revid:test@other.com-234234..revid:test@other.com-234235')
-    ['revid:test@other.com-234234', 'revid:test@other.com-234235']
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_revid revid:test@other.com-234235>]
     >>> _parse_revision_str('revid:test@other.com-234234..23')
-    ['revid:test@other.com-234234', 23]
+    [<RevisionSpec_revid revid:test@other.com-234234>, <RevisionSpec_int 23>]
     >>> _parse_revision_str('date:2005-04-12')
-    ['date:2005-04-12']
+    [<RevisionSpec_date date:2005-04-12>]
     >>> _parse_revision_str('date:2005-04-12 12:24:33')
-    ['date:2005-04-12 12:24:33']
+    [<RevisionSpec_date date:2005-04-12 12:24:33>]
     >>> _parse_revision_str('date:2005-04-12T12:24:33')
-    ['date:2005-04-12T12:24:33']
+    [<RevisionSpec_date date:2005-04-12T12:24:33>]
     >>> _parse_revision_str('date:2005-04-12,12:24:33')
-    ['date:2005-04-12,12:24:33']
+    [<RevisionSpec_date date:2005-04-12,12:24:33>]
     >>> _parse_revision_str('-5..23')
-    [-5, 23]
+    [<RevisionSpec_int -5>, <RevisionSpec_int 23>]
     >>> _parse_revision_str('-5')
-    [-5]
+    [<RevisionSpec_int -5>]
     >>> _parse_revision_str('123a')
-    ['123a']
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: '123a'
     >>> _parse_revision_str('abc')
-    ['abc']
+    Traceback (most recent call last):
+      ...
+    BzrError: No namespace registered for string: 'abc'
     """
     import re
     old_format_re = re.compile('\d*:\d*')
     m = old_format_re.match(revstr)
+    revs = []
     if m:
         warning('Colon separator for revision numbers is deprecated.'
                 ' Use .. instead')
-        revs = []
         for rev in revstr.split(':'):
             if rev:
-                revs.append(int(rev))
-            else:
-                revs.append(None)
-        return revs
-    revs = []
-    for x in revstr.split('..'):
-        if not x:
-            revs.append(None)
-        else:
-            try:
-                revs.append(int(x))
-            except ValueError:
-                revs.append(x)
+                revs.append(RevisionSpec(int(rev)))
+            else:
+                revs.append(RevisionSpec(None))
+    else:
+        for x in revstr.split('..'):
+            if not x:
+                revs.append(RevisionSpec(None))
+            else:
+                revs.append(RevisionSpec(x))
     return revs
 
 
+def _builtin_commands():
+    import bzrlib.builtins
+    r = {}
+    builtins = bzrlib.builtins.__dict__
+    for name in builtins:
+        if name.startswith("cmd_"):
+            real_name = _unsquish_command_name(name)        
+            r[real_name] = builtins[name]
+    return r
+
+            
+
+def builtin_command_names():
+    """Return list of builtin command names."""
+    return _builtin_commands().keys()
+    
+
+def plugin_command_names():
+    return plugin_cmds.keys()
+
 
 def _get_cmd_dict(plugins_override=True):
-    d = {}
-    for k, v in globals().iteritems():
-        if k.startswith("cmd_"):
-            d[_unsquish_command_name(k)] = v
-    # If we didn't load plugins, the plugin_cmds dict will be empty
+    """Return name->class mapping for all commands."""
+    d = _builtin_commands()
     if plugins_override:
         d.update(plugin_cmds)
-    else:
-        d2 = plugin_cmds.copy()
-        d2.update(d)
-        d = d2
     return d
 
     
@@ -144,37 +170,56 @@
         yield k,v
 
 
-def get_cmd_class(cmd, plugins_override=True):
+def get_cmd_object(cmd_name, plugins_override=True):
     """Return the canonical name and command class for a command.
+
+    plugins_override
+        If true, plugin commands can override builtins.
     """
-    cmd = str(cmd)                      # not unicode
+    from bzrlib.externalcommand import ExternalCommand
+
+    cmd_name = str(cmd_name)            # not unicode
 
     # first look up this command under the specified name
     cmds = _get_cmd_dict(plugins_override=plugins_override)
     try:
-        return cmd, cmds[cmd]
+        return cmds[cmd_name]()
     except KeyError:
         pass
 
     # look for any command which claims this as an alias
-    for cmdname, cmdclass in cmds.iteritems():
-        if cmd in cmdclass.aliases:
-            return cmdname, cmdclass
-
-    cmdclass = ExternalCommand.find_command(cmd)
-    if cmdclass:
-        return cmd, cmdclass
-
-    raise BzrCommandError("unknown command %r" % cmd)
+    for real_cmd_name, cmd_class in cmds.iteritems():
+        if cmd_name in cmd_class.aliases:
+            return cmd_class()
+
+    cmd_obj = ExternalCommand.find_command(cmd_name)
+    if cmd_obj:
+        return cmd_obj
+
+    raise BzrCommandError("unknown command %r" % cmd_name)
 
 
 class Command(object):
     """Base class for commands.
 
+    Commands are the heart of the command-line bzr interface.
+
+    The command object mostly handles the mapping of command-line
+    parameters into one or more bzrlib operations, and of the results
+    into textual output.
+
+    Commands normally don't have any state.  All their arguments are
+    passed in to the run method.  (Subclasses may take a different
+    policy if the behaviour of the instance needs to depend on e.g. a
+    shell plugin and not just its Python class.)
+
     The docstring for an actual command should give a single-line
     summary, then a complete description of the command.  A grammar
     description will be inserted.
 
+    aliases
+        Other accepted names for this command.
+
     takes_args
         List of argument forms, marked with whether they are optional,
         repeated, etc.
@@ -183,7 +228,8 @@
         List of options that may be given for this command.
 
     hidden
-        If true, this command isn't advertised.
+        If true, this command isn't advertised.  This is typically
+        for commands intended for expert users.
     """
     aliases = []
     
@@ -192,1150 +238,62 @@
 
     hidden = False
     
-    def __init__(self, options, arguments):
-        """Construct and run the command.
-
-        Sets self.status to the return value of run()."""
-        assert isinstance(options, dict)
-        assert isinstance(arguments, dict)
-        cmdargs = options.copy()
-        cmdargs.update(arguments)
+    def __init__(self):
+        """Construct an instance of this command."""
         if self.__doc__ == Command.__doc__:
-            from warnings import warn
             warn("No help message set for %r" % self)
-        self.status = self.run(**cmdargs)
-        if self.status is None:
-            self.status = 0
+
+
+    def run_argv(self, argv):
+        """Parse command line and run."""
+        args, opts = parse_args(argv)
+
+        if 'help' in opts:  # e.g. bzr add --help
+            from bzrlib.help import help_on_command
+            help_on_command(self.name())
+            return 0
+
+        # check options are reasonable
+        allowed = self.takes_options
+        for oname in opts:
+            if oname not in allowed:
+                raise BzrCommandError("option '--%s' is not allowed for command %r"
+                                      % (oname, self.name()))
+
+        # mix arguments and options into one dictionary
+        cmdargs = _match_argform(self.name(), self.takes_args, args)
+        cmdopts = {}
+        for k, v in opts.items():
+            cmdopts[k.replace('-', '_')] = v
+
+        all_cmd_args = cmdargs.copy()
+        all_cmd_args.update(cmdopts)
+
+        return self.run(**all_cmd_args)
 
     
     def run(self):
-        """Override this in sub-classes.
+        """Actually run the command.
 
         This is invoked with the options and arguments bound to
         keyword parameters.
 
-        Return 0 or None if the command was successful, or a shell
-        error code if not.
+        Return 0 or None if the command was successful, or a non-zero
+        shell error code if not.  It's OK for this method to allow
+        an exception to raise up.
         """
-        return 0
-
-
-class ExternalCommand(Command):
-    """Class to wrap external commands.
-
-    We cheat a little here, when get_cmd_class() calls us we actually give it back
-    an object we construct that has the appropriate path, help, options etc for the
-    specified command.
-
-    When run_bzr() tries to instantiate that 'class' it gets caught by the __call__
-    method, which we override to call the Command.__init__ method. That then calls
-    our run method which is pretty straight forward.
-
-    The only wrinkle is that we have to map bzr's dictionary of options and arguments
-    back into command line options and arguments for the script.
-    """
-
-    def find_command(cls, cmd):
-        import os.path
-        bzrpath = os.environ.get('BZRPATH', '')
-
-        for dir in bzrpath.split(os.pathsep):
-            path = os.path.join(dir, cmd)
-            if os.path.isfile(path):
-                return ExternalCommand(path)
-
-        return None
-
-    find_command = classmethod(find_command)
-
-    def __init__(self, path):
-        self.path = path
-
-        pipe = os.popen('%s --bzr-usage' % path, 'r')
-        self.takes_options = pipe.readline().split()
-
-        for opt in self.takes_options:
-            if not opt in OPTIONS:
-                raise BzrError("Unknown option '%s' returned by external command %s"
-                               % (opt, path))
-
-        # TODO: Is there any way to check takes_args is valid here?
-        self.takes_args = pipe.readline().split()
-
-        if pipe.close() is not None:
-            raise BzrError("Failed funning '%s --bzr-usage'" % path)
-
-        pipe = os.popen('%s --bzr-help' % path, 'r')
-        self.__doc__ = pipe.read()
-        if pipe.close() is not None:
-            raise BzrError("Failed funning '%s --bzr-help'" % path)
-
-    def __call__(self, options, arguments):
-        Command.__init__(self, options, arguments)
-        return self
-
-    def run(self, **kargs):
-        opts = []
-        args = []
-
-        keys = kargs.keys()
-        keys.sort()
-        for name in keys:
-            optname = name.replace('_','-')
-            value = kargs[name]
-            if OPTIONS.has_key(optname):
-                # it's an option
-                opts.append('--%s' % optname)
-                if value is not None and value is not True:
-                    opts.append(str(value))
-            else:
-                # it's an arg, or arg list
-                if type(value) is not list:
-                    value = [value]
-                for v in value:
-                    if v is not None:
-                        args.append(str(v))
-
-        self.status = os.spawnv(os.P_WAIT, self.path, [self.path] + opts + args)
-        return self.status
-
-
-class cmd_status(Command):
-    """Display status summary.
-
-    This reports on versioned and unknown files, reporting them
-    grouped by state.  Possible states are:
-
-    added
-        Versioned in the working copy but not in the previous revision.
-
-    removed
-        Versioned in the previous revision but removed or deleted
-        in the working copy.
-
-    renamed
-        Path of this file changed from the previous revision;
-        the text may also have changed.  This includes files whose
-        parent directory was renamed.
-
-    modified
-        Text has changed since the previous revision.
-
-    unchanged
-        Nothing about this file has changed since the previous revision.
-        Only shown with --all.
-
-    unknown
-        Not versioned and not matching an ignore pattern.
-
-    To see ignored files use 'bzr ignored'.  For details in the
-    changes to file texts, use 'bzr diff'.
-
-    If no arguments are specified, the status of the entire working
-    directory is shown.  Otherwise, only the status of the specified
-    files or directories is reported.  If a directory is given, status
-    is reported for everything inside that directory.
-
-    If a revision is specified, the changes since that revision are shown.
-    """
-    takes_args = ['file*']
-    takes_options = ['all', 'show-ids', 'revision']
-    aliases = ['st', 'stat']
-    
-    def run(self, all=False, show_ids=False, file_list=None):
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(x) for x in file_list]
-            # special case: only one path was given and it's the root
-            # of the branch
-            if file_list == ['']:
-                file_list = None
-        else:
-            b = find_branch('.')
-            
-        from bzrlib.status import show_status
-        show_status(b, show_unchanged=all, show_ids=show_ids,
-                    specific_files=file_list)
-
-
-class cmd_cat_revision(Command):
-    """Write out metadata for a revision."""
-
-    hidden = True
-    takes_args = ['revision_id']
-    
-    def run(self, revision_id):
-        from bzrlib.xml import pack_xml
-        pack_xml(find_branch('.').get_revision(revision_id), sys.stdout)
-
-
-class cmd_revno(Command):
-    """Show current revision number.
-
-    This is equal to the number of revisions on this branch."""
-    def run(self):
-        print find_branch('.').revno()
-
-class cmd_revision_info(Command):
-    """Show revision number and revision id for a given revision identifier.
-    """
-    hidden = True
-    takes_args = ['revision_info*']
-    takes_options = ['revision']
-    def run(self, revision=None, revision_info_list=None):
-        from bzrlib.branch import find_branch
-
-        revs = []
-        if revision is not None:
-            revs.extend(revision)
-        if revision_info_list is not None:
-            revs.extend(revision_info_list)
-        if len(revs) == 0:
-            raise BzrCommandError('You must supply a revision identifier')
-
-        b = find_branch('.')
-
-        for rev in revs:
-            print '%4d %s' % b.get_revision_info(rev)
-
-    
-class cmd_add(Command):
-    """Add specified files or directories.
-
-    In non-recursive mode, all the named items are added, regardless
-    of whether they were previously ignored.  A warning is given if
-    any of the named files are already versioned.
-
-    In recursive mode (the default), files are treated the same way
-    but the behaviour for directories is different.  Directories that
-    are already versioned do not give a warning.  All directories,
-    whether already versioned or not, are searched for files or
-    subdirectories that are neither versioned or ignored, and these
-    are added.  This search proceeds recursively into versioned
-    directories.  If no names are given '.' is assumed.
-
-    Therefore simply saying 'bzr add' will version all files that
-    are currently unknown.
-
-    TODO: Perhaps adding a file whose directly is not versioned should
-    recursively add that parent, rather than giving an error?
-    """
-    takes_args = ['file*']
-    takes_options = ['verbose', 'no-recurse']
-    
-    def run(self, file_list, verbose=False, no_recurse=False):
-        from bzrlib.add import smart_add
-        smart_add(file_list, verbose, not no_recurse)
-
-
-
-class cmd_mkdir(Command):
-    """Create a new versioned directory.
-
-    This is equivalent to creating the directory and then adding it.
-    """
-    takes_args = ['dir+']
-
-    def run(self, dir_list):
-        b = None
-        
-        for d in dir_list:
-            os.mkdir(d)
-            if not b:
-                b = find_branch(d)
-            b.add([d], verbose=True)
-
-
-class cmd_relpath(Command):
-    """Show path of a file relative to root"""
-    takes_args = ['filename']
-    hidden = True
-    
-    def run(self, filename):
-        print find_branch(filename).relpath(filename)
-
-
-
-class cmd_inventory(Command):
-    """Show inventory of the current working copy or a revision."""
-    takes_options = ['revision', 'show-ids']
-    
-    def run(self, revision=None, show_ids=False):
-        b = find_branch('.')
-        if revision == None:
-            inv = b.read_working_inventory()
-        else:
-            if len(revision) > 1:
-                raise BzrCommandError('bzr inventory --revision takes'
-                    ' exactly one revision identifier')
-            inv = b.get_revision_inventory(b.lookup_revision(revision[0]))
-
-        for path, entry in inv.entries():
-            if show_ids:
-                print '%-50s %s' % (path, entry.file_id)
-            else:
-                print path
-
-
-class cmd_move(Command):
-    """Move files to a different directory.
-
-    examples:
-        bzr move *.txt doc
-
-    The destination must be a versioned directory in the same branch.
-    """
-    takes_args = ['source$', 'dest']
-    def run(self, source_list, dest):
-        b = find_branch('.')
-
-        b.move([b.relpath(s) for s in source_list], b.relpath(dest))
-
-
-class cmd_rename(Command):
-    """Change the name of an entry.
-
-    examples:
-      bzr rename frob.c frobber.c
-      bzr rename src/frob.c lib/frob.c
-
-    It is an error if the destination name exists.
-
-    See also the 'move' command, which moves files into a different
-    directory without changing their name.
-
-    TODO: Some way to rename multiple files without invoking bzr for each
-    one?"""
-    takes_args = ['from_name', 'to_name']
-    
-    def run(self, from_name, to_name):
-        b = find_branch('.')
-        b.rename_one(b.relpath(from_name), b.relpath(to_name))
-
-
-
-
-
-class cmd_pull(Command):
-    """Pull any changes from another branch into the current one.
-
-    If the location is omitted, the last-used location will be used.
-    Both the revision history and the working directory will be
-    updated.
-
-    This command only works on branches that have not diverged.  Branches are
-    considered diverged if both branches have had commits without first
-    pulling from the other.
-
-    If branches have diverged, you can use 'bzr merge' to pull the text changes
-    from one into the other.
-    """
-    takes_args = ['location?']
-
-    def run(self, location=None):
-        from bzrlib.merge import merge
-        import tempfile
-        from shutil import rmtree
-        import errno
-        
-        br_to = find_branch('.')
-        stored_loc = None
-        try:
-            stored_loc = br_to.controlfile("x-pull", "rb").read().rstrip('\n')
-        except IOError, e:
-            if e.errno != errno.ENOENT:
-                raise
-        if location is None:
-            if stored_loc is None:
-                raise BzrCommandError("No pull location known or specified.")
-            else:
-                print "Using last location: %s" % stored_loc
-                location = stored_loc
-        cache_root = tempfile.mkdtemp()
-        from bzrlib.branch import DivergedBranches
-        br_from = find_branch(location)
-        location = pull_loc(br_from)
-        old_revno = br_to.revno()
-        try:
-            from branch import find_cached_branch, DivergedBranches
-            br_from = find_cached_branch(location, cache_root)
-            location = pull_loc(br_from)
-            old_revno = br_to.revno()
-            try:
-                br_to.update_revisions(br_from)
-            except DivergedBranches:
-                raise BzrCommandError("These branches have diverged."
-                    "  Try merge.")
-                
-            merge(('.', -1), ('.', old_revno), check_clean=False)
-            if location != stored_loc:
-                br_to.controlfile("x-pull", "wb").write(location + "\n")
-        finally:
-            rmtree(cache_root)
-
-
-
-class cmd_branch(Command):
-    """Create a new copy of a branch.
-
-    If the TO_LOCATION is omitted, the last component of the FROM_LOCATION will
-    be used.  In other words, "branch ../foo/bar" will attempt to create ./bar.
-
-    To retrieve the branch as of a particular revision, supply the --revision
-    parameter, as in "branch foo/bar -r 5".
-    """
-    takes_args = ['from_location', 'to_location?']
-    takes_options = ['revision']
-
-    def run(self, from_location, to_location=None, revision=None):
-        import errno
-        from bzrlib.merge import merge
-        from bzrlib.branch import DivergedBranches, NoSuchRevision, \
-             find_cached_branch, Branch
-        from shutil import rmtree
-        from meta_store import CachedStore
-        import tempfile
-        cache_root = tempfile.mkdtemp()
-
-        if revision is None:
-            revision = [None]
-        elif len(revision) > 1:
-            raise BzrCommandError('bzr branch --revision takes exactly 1 revision value')
-
-        try:
-            try:
-                br_from = find_cached_branch(from_location, cache_root)
-            except OSError, e:
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Source location "%s" does not'
-                                          ' exist.' % to_location)
-                else:
-                    raise
-
-            if to_location is None:
-                to_location = os.path.basename(from_location.rstrip("/\\"))
-
-            try:
-                os.mkdir(to_location)
-            except OSError, e:
-                if e.errno == errno.EEXIST:
-                    raise BzrCommandError('Target directory "%s" already'
-                                          ' exists.' % to_location)
-                if e.errno == errno.ENOENT:
-                    raise BzrCommandError('Parent of "%s" does not exist.' %
-                                          to_location)
-                else:
-                    raise
-            br_to = Branch(to_location, init=True)
-
-            br_to.set_root_id(br_from.get_root_id())
-
-            if revision:
-                if revision[0] is None:
-                    revno = br_from.revno()
-                else:
-                    revno, rev_id = br_from.get_revision_info(revision[0])
-                try:
-                    br_to.update_revisions(br_from, stop_revision=revno)
-                except NoSuchRevision:
-                    rmtree(to_location)
-                    msg = "The branch %s has no revision %d." % (from_location,
-                                                                 revno)
-                    raise BzrCommandError(msg)
-            
-            merge((to_location, -1), (to_location, 0), this_dir=to_location,
-                  check_clean=False, ignore_zero=True)
-            from_location = pull_loc(br_from)
-            br_to.controlfile("x-pull", "wb").write(from_location + "\n")
-        finally:
-            rmtree(cache_root)
-
-
-def pull_loc(branch):
-    # TODO: Should perhaps just make attribute be 'base' in
-    # RemoteBranch and Branch?
-    if hasattr(branch, "baseurl"):
-        return branch.baseurl
-    else:
-        return branch.base
-
-
-
-class cmd_renames(Command):
-    """Show list of renamed files.
-
-    TODO: Option to show renames between two historical versions.
-
-    TODO: Only show renames under dir, rather than in the whole branch.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        b = find_branch(dir)
-        old_inv = b.basis_tree().inventory
-        new_inv = b.read_working_inventory()
-
-        renames = list(bzrlib.tree.find_renames(old_inv, new_inv))
-        renames.sort()
-        for old_name, new_name in renames:
-            print "%s => %s" % (old_name, new_name)        
-
-
-class cmd_info(Command):
-    """Show statistical information about a branch."""
-    takes_args = ['branch?']
-    
-    def run(self, branch=None):
-        import info
-
-        b = find_branch(branch)
-        info.show_info(b)
-
-
-class cmd_remove(Command):
-    """Make a file unversioned.
-
-    This makes bzr stop tracking changes to a versioned file.  It does
-    not delete the working copy.
-    """
-    takes_args = ['file+']
-    takes_options = ['verbose']
-    
-    def run(self, file_list, verbose=False):
-        b = find_branch(file_list[0])
-        b.remove([b.relpath(f) for f in file_list], verbose=verbose)
-
-
-class cmd_file_id(Command):
-    """Print file_id of a particular file or directory.
-
-    The file_id is assigned when the file is first added and remains the
-    same through all revisions where the file exists, even when it is
-    moved or renamed.
-    """
-    hidden = True
-    takes_args = ['filename']
-    def run(self, filename):
-        b = find_branch(filename)
-        i = b.inventory.path2id(b.relpath(filename))
-        if i == None:
-            raise BzrError("%r is not a versioned file" % filename)
-        else:
-            print i
-
-
-class cmd_file_path(Command):
-    """Print path of file_ids to a file or directory.
-
-    This prints one line for each directory down to the target,
-    starting at the branch root."""
-    hidden = True
-    takes_args = ['filename']
-    def run(self, filename):
-        b = find_branch(filename)
-        inv = b.inventory
-        fid = inv.path2id(b.relpath(filename))
-        if fid == None:
-            raise BzrError("%r is not a versioned file" % filename)
-        for fip in inv.get_idpath(fid):
-            print fip
-
-
-class cmd_revision_history(Command):
-    """Display list of revision ids on this branch."""
-    hidden = True
-    def run(self):
-        for patchid in find_branch('.').revision_history():
-            print patchid
-
-
-class cmd_directories(Command):
-    """Display list of versioned directories in this branch."""
-    def run(self):
-        for name, ie in find_branch('.').read_working_inventory().directories():
-            if name == '':
-                print '.'
-            else:
-                print name
-
-
-class cmd_init(Command):
-    """Make a directory into a versioned branch.
-
-    Use this to create an empty branch, or before importing an
-    existing project.
-
-    Recipe for importing a tree of files:
-        cd ~/project
-        bzr init
-        bzr add -v .
-        bzr status
-        bzr commit -m 'imported project'
-    """
-    def run(self):
-        from bzrlib.branch import Branch
-        Branch('.', init=True)
-
-
-class cmd_diff(Command):
-    """Show differences in working tree.
-    
-    If files are listed, only the changes in those files are listed.
-    Otherwise, all changes for the tree are listed.
-
-    TODO: Given two revision arguments, show the difference between them.
-
-    TODO: Allow diff across branches.
-
-    TODO: Option to use external diff command; could be GNU diff, wdiff,
-          or a graphical diff.
-
-    TODO: Python difflib is not exactly the same as unidiff; should
-          either fix it up or prefer to use an external diff.
-
-    TODO: If a directory is given, diff everything under that.
-
-    TODO: Selected-file diff is inefficient and doesn't show you
-          deleted files.
-
-    TODO: This probably handles non-Unix newlines poorly.
-    """
-    
-    takes_args = ['file*']
-    takes_options = ['revision', 'diff-options']
-    aliases = ['di', 'dif']
-
-    def run(self, revision=None, file_list=None, diff_options=None):
-        from bzrlib.diff import show_diff
-
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(f) for f in file_list]
-            if file_list == ['']:
-                # just pointing to top-of-tree
-                file_list = None
-        else:
-            b = find_branch('.')
-
-        # TODO: Make show_diff support taking 2 arguments
-        base_rev = None
-        if revision is not None:
-            if len(revision) != 1:
-                raise BzrCommandError('bzr diff --revision takes exactly one revision identifier')
-            base_rev = revision[0]
-    
-        show_diff(b, base_rev, specific_files=file_list,
-                  external_diff_options=diff_options)
-
-
-        
-
-
-class cmd_deleted(Command):
-    """List files deleted in the working tree.
-
-    TODO: Show files deleted since a previous revision, or between two revisions.
-    """
-    def run(self, show_ids=False):
-        b = find_branch('.')
-        old = b.basis_tree()
-        new = b.working_tree()
-
-        ## TODO: Much more efficient way to do this: read in new
-        ## directories with readdir, rather than stating each one.  Same
-        ## level of effort but possibly much less IO.  (Or possibly not,
-        ## if the directories are very large...)
-
-        for path, ie in old.inventory.iter_entries():
-            if not new.has_id(ie.file_id):
-                if show_ids:
-                    print '%-50s %s' % (path, ie.file_id)
-                else:
-                    print path
-
-
-class cmd_modified(Command):
-    """List files modified in working tree."""
-    hidden = True
-    def run(self):
-        from bzrlib.diff import compare_trees
-
-        b = find_branch('.')
-        td = compare_trees(b.basis_tree(), b.working_tree())
-
-        for path, id, kind in td.modified:
-            print path
-
-
-
-class cmd_added(Command):
-    """List files added in working tree."""
-    hidden = True
-    def run(self):
-        b = find_branch('.')
-        wt = b.working_tree()
-        basis_inv = b.basis_tree().inventory
-        inv = wt.inventory
-        for file_id in inv:
-            if file_id in basis_inv:
-                continue
-            path = inv.id2path(file_id)
-            if not os.access(b.abspath(path), os.F_OK):
-                continue
-            print path
-                
-        
-
-class cmd_root(Command):
-    """Show the tree root directory.
-
-    The root is the nearest enclosing directory with a .bzr control
-    directory."""
-    takes_args = ['filename?']
-    def run(self, filename=None):
-        """Print the branch root."""
-        b = find_branch(filename)
-        print getattr(b, 'base', None) or getattr(b, 'baseurl')
-
-
-class cmd_log(Command):
-    """Show log of this branch.
-
-    To request a range of logs, you can use the command -r begin:end
-    -r revision requests a specific revision, -r :end or -r begin: are
-    also valid.
-
-    --message allows you to give a regular expression, which will be evaluated
-    so that only matching entries will be displayed.
-
-    TODO: Make --revision support uuid: and hash: [future tag:] notation.
-  
-    """
-
-    takes_args = ['filename?']
-    takes_options = ['forward', 'timezone', 'verbose', 'show-ids', 'revision','long', 'message']
-    
-    def run(self, filename=None, timezone='original',
-            verbose=False,
-            show_ids=False,
-            forward=False,
-            revision=None,
-            message=None,
-            long=False):
-        from bzrlib.branch import find_branch
-        from bzrlib.log import log_formatter, show_log
-        import codecs
-
-        direction = (forward and 'forward') or 'reverse'
-        
-        if filename:
-            b = find_branch(filename)
-            fp = b.relpath(filename)
-            if fp:
-                file_id = b.read_working_inventory().path2id(fp)
-            else:
-                file_id = None  # points to branch root
-        else:
-            b = find_branch('.')
-            file_id = None
-
-        if revision is None:
-            rev1 = None
-            rev2 = None
-        elif len(revision) == 1:
-            rev1 = rev2 = b.get_revision_info(revision[0])[0]
-        elif len(revision) == 2:
-            rev1 = b.get_revision_info(revision[0])[0]
-            rev2 = b.get_revision_info(revision[1])[0]
-        else:
-            raise BzrCommandError('bzr log --revision takes one or two values.')
-
-        if rev1 == 0:
-            rev1 = None
-        if rev2 == 0:
-            rev2 = None
-
-        mutter('encoding log as %r' % bzrlib.user_encoding)
-
-        # use 'replace' so that we don't abort if trying to write out
-        # in e.g. the default C locale.
-        outf = codecs.getwriter(bzrlib.user_encoding)(sys.stdout, errors='replace')
-
-        if long:
-            log_format = 'long'
-        else:
-            log_format = 'short'
-        lf = log_formatter(log_format,
-                           show_ids=show_ids,
-                           to_file=outf,
-                           show_timezone=timezone)
-
-        show_log(b,
-                 lf,
-                 file_id,
-                 verbose=verbose,
-                 direction=direction,
-                 start_revision=rev1,
-                 end_revision=rev2,
-                 search=message)
-
-
-
-class cmd_touching_revisions(Command):
-    """Return revision-ids which affected a particular file.
-
-    A more user-friendly interface is "bzr log FILE"."""
-    hidden = True
-    takes_args = ["filename"]
-    def run(self, filename):
-        b = find_branch(filename)
-        inv = b.read_working_inventory()
-        file_id = inv.path2id(b.relpath(filename))
-        for revno, revision_id, what in bzrlib.log.find_touching_revisions(b, file_id):
-            print "%6d %s" % (revno, what)
-
-
-class cmd_ls(Command):
-    """List files in a tree.
-
-    TODO: Take a revision or remote path and list that tree instead.
-    """
-    hidden = True
-    def run(self, revision=None, verbose=False):
-        b = find_branch('.')
-        if revision == None:
-            tree = b.working_tree()
-        else:
-            tree = b.revision_tree(b.lookup_revision(revision))
-
-        for fp, fc, kind, fid in tree.list_files():
-            if verbose:
-                if kind == 'directory':
-                    kindch = '/'
-                elif kind == 'file':
-                    kindch = ''
-                else:
-                    kindch = '???'
-
-                print '%-8s %s%s' % (fc, fp, kindch)
-            else:
-                print fp
-
-
-
-class cmd_unknowns(Command):
-    """List unknown files."""
-    def run(self):
-        from bzrlib.osutils import quotefn
-        for f in find_branch('.').unknowns():
-            print quotefn(f)
-
-
-
-class cmd_ignore(Command):
-    """Ignore a command or pattern.
-
-    To remove patterns from the ignore list, edit the .bzrignore file.
-
-    If the pattern contains a slash, it is compared to the whole path
-    from the branch root.  Otherwise, it is comapred to only the last
-    component of the path.
-
-    Ignore patterns are case-insensitive on case-insensitive systems.
-
-    Note: wildcards must be quoted from the shell on Unix.
-
-    examples:
-        bzr ignore ./Makefile
-        bzr ignore '*.class'
-    """
-    takes_args = ['name_pattern']
-    
-    def run(self, name_pattern):
-        from bzrlib.atomicfile import AtomicFile
-        import os.path
-
-        b = find_branch('.')
-        ifn = b.abspath('.bzrignore')
-
-        if os.path.exists(ifn):
-            f = open(ifn, 'rt')
-            try:
-                igns = f.read().decode('utf-8')
-            finally:
-                f.close()
-        else:
-            igns = ''
-
-        # TODO: If the file already uses crlf-style termination, maybe
-        # we should use that for the newly added lines?
-
-        if igns and igns[-1] != '\n':
-            igns += '\n'
-        igns += name_pattern + '\n'
-
-        try:
-            f = AtomicFile(ifn, 'wt')
-            f.write(igns.encode('utf-8'))
-            f.commit()
-        finally:
-            f.close()
-
-        inv = b.working_tree().inventory
-        if inv.path2id('.bzrignore'):
-            mutter('.bzrignore is already versioned')
-        else:
-            mutter('need to make new .bzrignore file versioned')
-            b.add(['.bzrignore'])
-
-
-
-class cmd_ignored(Command):
-    """List ignored files and the patterns that matched them.
-
-    See also: bzr ignore"""
-    def run(self):
-        tree = find_branch('.').working_tree()
-        for path, file_class, kind, file_id in tree.list_files():
-            if file_class != 'I':
-                continue
-            ## XXX: Slightly inefficient since this was already calculated
-            pat = tree.is_ignored(path)
-            print '%-50s %s' % (path, pat)
-
-
-class cmd_lookup_revision(Command):
-    """Lookup the revision-id from a revision-number
-
-    example:
-        bzr lookup-revision 33
-    """
-    hidden = True
-    takes_args = ['revno']
-    
-    def run(self, revno):
-        try:
-            revno = int(revno)
-        except ValueError:
-            raise BzrCommandError("not a valid revision-number: %r" % revno)
-
-        print find_branch('.').lookup_revision(revno)
-
-
-class cmd_export(Command):
-    """Export past revision to destination directory.
-
-    If no revision is specified this exports the last committed revision.
-
-    Format may be an "exporter" name, such as tar, tgz, tbz2.  If none is
-    given, try to find the format with the extension. If no extension
-    is found exports to a directory (equivalent to --format=dir).
-
-    Root may be the top directory for tar, tgz and tbz2 formats. If none
-    is given, the top directory will be the root name of the file."""
-    # TODO: list known exporters
-    takes_args = ['dest']
-    takes_options = ['revision', 'format', 'root']
-    def run(self, dest, revision=None, format=None, root=None):
-        import os.path
-        b = find_branch('.')
-        if revision is None:
-            rev_id = b.last_patch()
-        else:
-            if len(revision) != 1:
-                raise BzrError('bzr export --revision takes exactly 1 argument')
-            revno, rev_id = b.get_revision_info(revision[0])
-        t = b.revision_tree(rev_id)
-        root, ext = os.path.splitext(dest)
-        if not format:
-            if ext in (".tar",):
-                format = "tar"
-            elif ext in (".gz", ".tgz"):
-                format = "tgz"
-            elif ext in (".bz2", ".tbz2"):
-                format = "tbz2"
-            else:
-                format = "dir"
-        t.export(dest, format, root)
-
-
-class cmd_cat(Command):
-    """Write a file's text from a previous revision."""
-
-    takes_options = ['revision']
-    takes_args = ['filename']
-
-    def run(self, filename, revision=None):
-        if revision == None:
-            raise BzrCommandError("bzr cat requires a revision number")
-        elif len(revision) != 1:
-            raise BzrCommandError("bzr cat --revision takes exactly one number")
-        b = find_branch('.')
-        b.print_file(b.relpath(filename), revision[0])
-
-
-class cmd_local_time_offset(Command):
-    """Show the offset in seconds from GMT to local time."""
-    hidden = True    
-    def run(self):
-        print bzrlib.osutils.local_time_offset()
-
-
-
-class cmd_commit(Command):
-    """Commit changes into a new revision.
-
-    If selected files are specified, only changes to those files are
-    committed.  If a directory is specified then its contents are also
-    committed.
-
-    A selected-file commit may fail in some cases where the committed
-    tree would be invalid, such as trying to commit a file in a
-    newly-added directory that is not itself committed.
-
-    TODO: Run hooks on tree to-be-committed, and after commit.
-
-    TODO: Strict commit that fails if there are unknown or deleted files.
-    """
-    takes_args = ['selected*']
-    takes_options = ['message', 'file', 'verbose', 'unchanged']
-    aliases = ['ci', 'checkin']
-
-    def run(self, message=None, file=None, verbose=True, selected_list=None,
-            unchanged=False):
-        from bzrlib.errors import PointlessCommit
-        from bzrlib.osutils import get_text_message
-
-        ## Warning: shadows builtin file()
-        if not message and not file:
-            import cStringIO
-            stdout = sys.stdout
-            catcher = cStringIO.StringIO()
-            sys.stdout = catcher
-            cmd_status({"file_list":selected_list}, {})
-            info = catcher.getvalue()
-            sys.stdout = stdout
-            message = get_text_message(info)
-            
-            if message is None:
-                raise BzrCommandError("please specify a commit message",
-                                      ["use either --message or --file"])
-        elif message and file:
-            raise BzrCommandError("please specify either --message or --file")
-        
-        if file:
-            import codecs
-            message = codecs.open(file, 'rt', bzrlib.user_encoding).read()
-
-        b = find_branch('.')
-
-        try:
-            b.commit(message, verbose=verbose,
-                     specific_files=selected_list,
-                     allow_pointless=unchanged)
-        except PointlessCommit:
-            # FIXME: This should really happen before the file is read in;
-            # perhaps prepare the commit; get the message; then actually commit
-            raise BzrCommandError("no changes to commit",
-                                  ["use --unchanged to commit anyhow"])
-
-
-class cmd_check(Command):
-    """Validate consistency of branch history.
-
-    This command checks various invariants about the branch storage to
-    detect data corruption or bzr bugs.
-
-    If given the --update flag, it will update some optional fields
-    to help ensure data consistency.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        from bzrlib.check import check
-        check(find_branch(dir))
-
-
-
-class cmd_scan_cache(Command):
-    hidden = True
-    def run(self):
-        from bzrlib.hashcache import HashCache
-        import os
-
-        c = HashCache('.')
-        c.read()
-        c.scan()
-            
-        print '%6d stats' % c.stat_count
-        print '%6d in hashcache' % len(c._cache)
-        print '%6d files removed from cache' % c.removed_count
-        print '%6d hashes updated' % c.update_count
-        print '%6d files changed too recently to cache' % c.danger_count
-
-        if c.needs_write:
-            c.write()
-            
-
-
-class cmd_upgrade(Command):
-    """Upgrade branch storage to current format.
-
-    This should normally be used only after the check command tells
-    you to run it.
-    """
-    takes_args = ['dir?']
-
-    def run(self, dir='.'):
-        from bzrlib.upgrade import upgrade
-        upgrade(find_branch(dir))
-
-
-
-class cmd_whoami(Command):
-    """Show bzr user id."""
-    takes_options = ['email']
-    
-    def run(self, email=False):
-        if email:
-            print bzrlib.osutils.user_email()
-        else:
-            print bzrlib.osutils.username()
-
-
-class cmd_selftest(Command):
-    """Run internal test suite"""
-    hidden = True
-    takes_options = ['verbose']
-    def run(self, verbose=False):
-        from bzrlib.selftest import selftest
-        return int(not selftest(verbose=verbose))
-
-
-class cmd_version(Command):
-    """Show version of bzr."""
-    def run(self):
-        show_version()
-
-def show_version():
-    print "bzr (bazaar-ng) %s" % bzrlib.__version__
-    # is bzrlib itself in a branch?
-    bzrrev = bzrlib.get_bzr_revision()
-    if bzrrev:
-        print "  (bzr checkout, revision %d {%s})" % bzrrev
-    print bzrlib.__copyright__
-    print "http://bazaar-ng.org/"
-    print
-    print "bzr comes with ABSOLUTELY NO WARRANTY.  bzr is free software, and"
-    print "you may use, modify and redistribute it under the terms of the GNU"
-    print "General Public License version 2 or later."
-
-
-class cmd_rocks(Command):
-    """Statement of optimism."""
-    hidden = True
-    def run(self):
-        print "it sure does!"
+        raise NotImplementedError()
+
+
+    def help(self):
+        """Return help message for this class."""
+        if self.__doc__ is Command.__doc__:
+            return None
+        return getdoc(self)
+
+    def name(self):
+        return _unsquish_command_name(self.__class__.__name__)
+
 
 def parse_spec(spec):
     """
@@ -1369,118 +327,12 @@
     return parsed
 
 
-
-class cmd_merge(Command):
-    """Perform a three-way merge of trees.
-    
-    The SPEC parameters are working tree or revision specifiers.  Working trees
-    are specified using standard paths or urls.  No component of a directory
-    path may begin with '@'.
-    
-    Working tree examples: '.', '..', 'foo@', but NOT 'foo/@bar'
-
-    Revisions are specified using a dirname/@revno pair, where dirname is the
-    branch directory and revno is the revision within that branch.  If no revno
-    is specified, the latest revision is used.
-
-    Revision examples: './@127', 'foo/@', '../@1'
-
-    The OTHER_SPEC parameter is required.  If the BASE_SPEC parameter is
-    not supplied, the common ancestor of OTHER_SPEC the current branch is used
-    as the BASE.
-
-    merge refuses to run if there are any uncommitted changes, unless
-    --force is given.
-    """
-    takes_args = ['other_spec', 'base_spec?']
-    takes_options = ['force']
-
-    def run(self, other_spec, base_spec=None, force=False):
-        from bzrlib.merge import merge
-        merge(parse_spec(other_spec), parse_spec(base_spec),
-              check_clean=(not force))
-
-
-
-class cmd_revert(Command):
-    """Restore selected files from a previous revision.
-    """
-    takes_args = ['file+']
-    def run(self, file_list):
-        from bzrlib.branch import find_branch
-        
-        if not file_list:
-            file_list = ['.']
-            
-        b = find_branch(file_list[0])
-
-        b.revert([b.relpath(f) for f in file_list])
-
-
-class cmd_merge_revert(Command):
-    """Reverse all changes since the last commit.
-
-    Only versioned files are affected.
-
-    TODO: Store backups of any files that will be reverted, so
-          that the revert can be undone.          
-    """
-    takes_options = ['revision']
-
-    def run(self, revision=None):
-        from bzrlib.merge import merge
-        if revision is None:
-            revision = [-1]
-        elif len(revision) != 1:
-            raise BzrCommandError('bzr merge-revert --revision takes exactly 1 argument')
-        merge(('.', revision[0]), parse_spec('.'),
-              check_clean=False,
-              ignore_zero=True)
-
-
-class cmd_assert_fail(Command):
-    """Test reporting of assertion failures"""
-    hidden = True
-    def run(self):
-        assert False, "always fails"
-
-
-class cmd_help(Command):
-    """Show help on a command or other topic.
-
-    For a list of all available commands, say 'bzr help commands'."""
-    takes_args = ['topic?']
-    aliases = ['?']
-    
-    def run(self, topic=None):
-        import help
-        help.help(topic)
-
-
-
-
-class cmd_plugins(Command):
-    """List plugins"""
-    hidden = True
-    def run(self):
-        import bzrlib.plugin
-        from inspect import getdoc
-        from pprint import pprint
-        for plugin in bzrlib.plugin.all_plugins:
-            print plugin.__path__[0]
-            d = getdoc(plugin)
-            if d:
-                print '\t', d.split('\n')[0]
-
-        #pprint(bzrlib.plugin.all_plugins)
-
-
-
 # list of all available options; the rhs can be either None for an
 # option that takes no argument, or a constructor function that checks
 # the type.
 OPTIONS = {
     'all':                    None,
+    'basis':                  str,
     'diff-options':           str,
     'help':                   None,
     'file':                   unicode,
@@ -1491,6 +343,7 @@
     'no-recurse':             None,
     'profile':                None,
     'revision':               _parse_revision_str,
+    'short':                  None,
     'show-ids':               None,
     'timezone':               str,
     'verbose':                None,
@@ -1500,6 +353,9 @@
     'update':                 None,
     'long':                   None,
     'root':                   str,
+    'no-backup':              None,
+    'pattern':                str,
+    'remember':               None,
     }
 
 SHORT_OPTIONS = {
@@ -1522,6 +378,8 @@
 
     >>> parse_args('--help'.split())
     ([], {'help': True})
+    >>> parse_args('help -- --invalidcmd'.split())
+    (['help', '--invalidcmd'], {})
     >>> parse_args('--version'.split())
     ([], {'version': True})
     >>> parse_args('status --all'.split())
@@ -1529,26 +387,29 @@
     >>> parse_args('commit --message=biter'.split())
     (['commit'], {'message': u'biter'})
     >>> parse_args('log -r 500'.split())
-    (['log'], {'revision': [500]})
+    (['log'], {'revision': [<RevisionSpec_int 500>]})
     >>> parse_args('log -r500..600'.split())
-    (['log'], {'revision': [500, 600]})
+    (['log'], {'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
     >>> parse_args('log -vr500..600'.split())
-    (['log'], {'verbose': True, 'revision': [500, 600]})
-    >>> parse_args('log -rv500..600'.split()) #the r takes an argument
-    (['log'], {'revision': ['v500', 600]})
+    (['log'], {'verbose': True, 'revision': [<RevisionSpec_int 500>, <RevisionSpec_int 600>]})
+    >>> parse_args('log -rrevno:500..600'.split()) #the r takes an argument
+    (['log'], {'revision': [<RevisionSpec_revno revno:500>, <RevisionSpec_int 600>]})
     """
     args = []
     opts = {}
 
-    # TODO: Maybe handle '--' to end options?
-
+    argsover = False
     while argv:
         a = argv.pop(0)
-        if a[0] == '-':
+        if not argsover and a[0] == '-':
             # option names must not be unicode
             a = str(a)
             optarg = None
             if a[1] == '-':
+                if a == '--':
+                    # We've received a standalone -- No more flags
+                    argsover = True
+                    continue
                 mutter("  got option %r" % a)
                 if '=' in a:
                     optname, optarg = a[2:].split('=', 1)
@@ -1655,72 +516,28 @@
     return argdict
 
 
-def _parse_master_args(argv):
-    """Parse the arguments that always go with the original command.
-    These are things like bzr --no-plugins, etc.
-
-    There are now 2 types of option flags. Ones that come *before* the command,
-    and ones that come *after* the command.
-    Ones coming *before* the command are applied against all possible commands.
-    And are generally applied before plugins are loaded.
-
-    The current list are:
-        --builtin   Allow plugins to load, but don't let them override builtin commands,
-                    they will still be allowed if they do not override a builtin.
-        --no-plugins    Don't load any plugins. This lets you get back to official source
-                        behavior.
-        --profile   Enable the hotspot profile before running the command.
-                    For backwards compatibility, this is also a non-master option.
-        --version   Spit out the version of bzr that is running and exit.
-                    This is also a non-master option.
-        --help      Run help and exit, also a non-master option (I think that should stay, though)
-
-    >>> argv, opts = _parse_master_args(['--test'])
-    Traceback (most recent call last):
-    ...
-    BzrCommandError: Invalid master option: 'test'
-    >>> argv, opts = _parse_master_args(['--version', 'command'])
-    >>> print argv
-    ['command']
-    >>> print opts['version']
-    True
-    >>> argv, opts = _parse_master_args(['--profile', 'command', '--more-options'])
-    >>> print argv
-    ['command', '--more-options']
-    >>> print opts['profile']
-    True
-    >>> argv, opts = _parse_master_args(['--no-plugins', 'command'])
-    >>> print argv
-    ['command']
-    >>> print opts['no-plugins']
-    True
-    >>> print opts['profile']
-    False
-    >>> argv, opts = _parse_master_args(['command', '--profile'])
-    >>> print argv
-    ['command', '--profile']
-    >>> print opts['profile']
-    False
-    """
-    master_opts = {'builtin':False,
-        'no-plugins':False,
-        'version':False,
-        'profile':False,
-        'help':False
-    }
-
-    for arg in argv[:]:
-        if arg[:2] != '--': # at the first non-option, we return the rest
-            break
-        arg = arg[2:] # Remove '--'
-        if arg not in master_opts:
-            # We could say that this is not an error, that we should
-            # just let it be handled by the main section instead
-            raise BzrCommandError('Invalid master option: %r' % arg)
-        argv.pop(0) # We are consuming this entry
-        master_opts[arg] = True
-    return argv, master_opts
-
+
+def apply_profiled(the_callable, *args, **kwargs):
+    import hotshot
+    import tempfile
+    import hotshot.stats
+    pffileno, pfname = tempfile.mkstemp()
+    try:
+        prof = hotshot.Profile(pfname)
+        try:
+            ret = prof.runcall(the_callable, *args, **kwargs) or 0
+        finally:
+            prof.close()
+        stats = hotshot.stats.load(pfname)
+        stats.strip_dirs()
+        stats.sort_stats('cum')   # 'time'
+        ## XXX: Might like to write to stderr or the trace file instead but
+        ## print_stats seems hardcoded to stdout
+        stats.print_stats(20)
+        return ret
+    finally:
+        os.close(pffileno)
+        os.remove(pfname)
 
 
 def run_bzr(argv):
@@ -1730,147 +547,113 @@
     logging and error handling.  
     
     argv
-       The command-line arguments, without the program name.
+       The command-line arguments, without the program name from argv[0]
     
     Returns a command status or raises an exception.
+
+    Special master options: these must come before the command because
+    they control how the command is interpreted.
+
+    --no-plugins
+        Do not load plugin modules at all
+
+    --builtin
+        Only use builtin commands.  (Plugins are still allowed to change
+        other behaviour.)
+
+    --profile
+        Run under the Python profiler.
     """
+    # Load all of the transport methods
+    import bzrlib.transport.local, bzrlib.transport.http
+    
     argv = [a.decode(bzrlib.user_encoding) for a in argv]
 
-    # some options like --builtin and --no-plugins have special effects
-    argv, master_opts = _parse_master_args(argv)
-    if not master_opts['no-plugins']:
+    opt_profile = opt_no_plugins = opt_builtin = False
+
+    # --no-plugins is handled specially at a very early stage. We need
+    # to load plugins before doing other command parsing so that they
+    # can override commands, but this needs to happen first.
+
+    for a in argv:
+        if a == '--profile':
+            opt_profile = True
+        elif a == '--no-plugins':
+            opt_no_plugins = True
+        elif a == '--builtin':
+            opt_builtin = True
+        else:
+            break
+        argv.remove(a)
+
+    if (not argv) or (argv[0] == '--help'):
+        from bzrlib.help import help
+        if len(argv) > 1:
+            help(argv[1])
+        else:
+            help()
+        return 0
+
+    if argv[0] == '--version':
+        from bzrlib.builtins import show_version
+        show_version()
+        return 0
+        
+    if not opt_no_plugins:
         from bzrlib.plugin import load_plugins
         load_plugins()
 
-    args, opts = parse_args(argv)
-
-    if master_opts.get('help') or 'help' in opts:
-        from bzrlib.help import help
-        if argv:
-            help(argv[0])
-        else:
-            help()
-        return 0            
-        
-    if 'version' in opts:
-        show_version()
-        return 0
-    
-    if args and args[0] == 'builtin':
-        include_plugins=False
-        args = args[1:]
-    
-    try:
-        cmd = str(args.pop(0))
-    except IndexError:
-        print >>sys.stderr, "please try 'bzr help' for help"
-        return 1
-
-    plugins_override = not (master_opts['builtin'])
-    canonical_cmd, cmd_class = get_cmd_class(cmd, plugins_override=plugins_override)
-
-    profile = master_opts['profile']
-    # For backwards compatibility, I would rather stick with --profile being a
-    # master/global option
-    if 'profile' in opts:
-        profile = True
-        del opts['profile']
-
-    # check options are reasonable
-    allowed = cmd_class.takes_options
-    for oname in opts:
-        if oname not in allowed:
-            raise BzrCommandError("option '--%s' is not allowed for command %r"
-                                  % (oname, cmd))
-
-    # mix arguments and options into one dictionary
-    cmdargs = _match_argform(cmd, cmd_class.takes_args, args)
-    cmdopts = {}
-    for k, v in opts.items():
-        cmdopts[k.replace('-', '_')] = v
-
-    if profile:
-        import hotshot, tempfile
-        pffileno, pfname = tempfile.mkstemp()
-        try:
-            prof = hotshot.Profile(pfname)
-            ret = prof.runcall(cmd_class, cmdopts, cmdargs) or 0
-            prof.close()
-
-            import hotshot.stats
-            stats = hotshot.stats.load(pfname)
-            #stats.strip_dirs()
-            stats.sort_stats('time')
-            ## XXX: Might like to write to stderr or the trace file instead but
-            ## print_stats seems hardcoded to stdout
-            stats.print_stats(20)
-            
-            return ret.status
-
-        finally:
-            os.close(pffileno)
-            os.remove(pfname)
+    cmd = str(argv.pop(0))
+
+    cmd_obj = get_cmd_object(cmd, plugins_override=not opt_builtin)
+
+    if opt_profile:
+        ret = apply_profiled(cmd_obj.run_argv, argv)
     else:
-        return cmd_class(cmdopts, cmdargs).status 
-
-
-def _report_exception(summary, quiet=False):
-    import traceback
-    log_error('bzr: ' + summary)
-    bzrlib.trace.log_exception()
-
-    if not quiet:
-        tb = sys.exc_info()[2]
-        exinfo = traceback.extract_tb(tb)
-        if exinfo:
-            sys.stderr.write('  at %s:%d in %s()\n' % exinfo[-1][:3])
-        sys.stderr.write('  see ~/.bzr.log for debug information\n')
-
+        ret = cmd_obj.run_argv(argv)
+    return ret or 0
 
 
 def main(argv):
-    
-    bzrlib.trace.open_tracefile(argv)
-
+    import bzrlib.ui
+    bzrlib.trace.log_startup(argv)
+    bzrlib.ui.ui_factory = bzrlib.ui.TextUIFactory()
+
+    return run_bzr_catch_errors(argv[1:])
+
+
+def run_bzr_catch_errors(argv):
     try:
         try:
-            try:
-                return run_bzr(argv[1:])
-            finally:
-                # do this here inside the exception wrappers to catch EPIPE
-                sys.stdout.flush()
-        except BzrError, e:
-            quiet = isinstance(e, (BzrCommandError))
-            _report_exception('error: ' + e.args[0], quiet=quiet)
-            if len(e.args) > 1:
-                for h in e.args[1]:
-                    # some explanation or hints
-                    log_error('  ' + h)
-            return 1
-        except AssertionError, e:
-            msg = 'assertion failed'
-            if str(e):
-                msg += ': ' + str(e)
-            _report_exception(msg)
-            return 2
-        except KeyboardInterrupt, e:
-            _report_exception('interrupted', quiet=True)
-            return 2
-        except Exception, e:
-            import errno
-            quiet = False
-            if (isinstance(e, IOError) 
-                and hasattr(e, 'errno')
-                and e.errno == errno.EPIPE):
-                quiet = True
-                msg = 'broken pipe'
-            else:
-                msg = str(e).rstrip('\n')
-            _report_exception(msg, quiet)
-            return 2
-    finally:
-        bzrlib.trace.close_trace()
-
+            return run_bzr(argv)
+        finally:
+            # do this here inside the exception wrappers to catch EPIPE
+            sys.stdout.flush()
+    except BzrCommandError, e:
+        # command line syntax error, etc
+        log_error(str(e))
+        return 1
+    except BzrError, e:
+        bzrlib.trace.log_exception()
+        return 1
+    except AssertionError, e:
+        bzrlib.trace.log_exception('assertion failed: ' + str(e))
+        return 3
+    except KeyboardInterrupt, e:
+        bzrlib.trace.log_exception('interrupted')
+        return 2
+    except Exception, e:
+        import errno
+        if (isinstance(e, IOError) 
+            and hasattr(e, 'errno')
+            and e.errno == errno.EPIPE):
+            bzrlib.trace.note('broken pipe')
+            return 2
+        else:
+            ## import pdb
+            ## pdb.pm()
+            bzrlib.trace.log_exception()
+            return 2
 
 if __name__ == '__main__':
     sys.exit(main(sys.argv))

=== modified file 'bzrlib/commit.py'
--- bzrlib/commit.py	2005-07-22 19:44:09 +0000
+++ bzrlib/commit.py	2005-10-07 06:13:14 +0000
@@ -1,312 +1,420 @@
 # Copyright (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-# FIXME: "bzr commit doc/format" commits doc/format.txt!
-
-def commit(branch, message,
-           timestamp=None,
-           timezone=None,
-           committer=None,
-           verbose=True,
-           specific_files=None,
-           rev_id=None,
-           allow_pointless=True):
-    """Commit working copy as a new revision.
-
-    The basic approach is to add all the file texts into the
-    store, then the inventory, then make a new revision pointing
-    to that inventory and store that.
-
-    This is not quite safe if the working copy changes during the
-    commit; for the moment that is simply not allowed.  A better
-    approach is to make a temporary copy of the files before
-    computing their hashes, and then add those hashes in turn to
-    the inventory.  This should mean at least that there are no
-    broken hash pointers.  There is no way we can get a snapshot
-    of the whole directory at an instant.  This would also have to
-    be robust against files disappearing, moving, etc.  So the
-    whole thing is a bit hard.
-
-    This raises PointlessCommit if there are no changes, no new merges,
-    and allow_pointless  is false.
-
-    timestamp -- if not None, seconds-since-epoch for a
-         postdated/predated commit.
-
-    specific_files
-        If true, commit only those files.
-
-    rev_id
-        If set, use this as the new revision id.
-        Useful for test or import commands that need to tightly
-        control what revisions are assigned.  If you duplicate
-        a revision id that exists elsewhere it is your own fault.
-        If null (default), a time/random revision id is generated.
-    """
-
-    import time, tempfile
-
-    from bzrlib.osutils import local_time_offset, username
-    from bzrlib.branch import gen_file_id
-    from bzrlib.errors import BzrError, PointlessCommit
-    from bzrlib.revision import Revision, RevisionReference
-    from bzrlib.trace import mutter, note
-    from bzrlib.xml import pack_xml
-
-    branch.lock_write()
-
-    try:
-        # First walk over the working inventory; and both update that
-        # and also build a new revision inventory.  The revision
-        # inventory needs to hold the text-id, sha1 and size of the
-        # actual file versions committed in the revision.  (These are
-        # not present in the working inventory.)  We also need to
-        # detect missing/deleted files, and remove them from the
-        # working inventory.
-
-        work_tree = branch.working_tree()
-        work_inv = work_tree.inventory
-        basis = branch.basis_tree()
-        basis_inv = basis.inventory
-
-        if verbose:
-            note('looking for changes...')
-
-        pending_merges = branch.pending_merges()
-
-        missing_ids, new_inv, any_changes = \
-                     _gather_commit(branch,
-                                    work_tree,
-                                    work_inv,
-                                    basis_inv,
-                                    specific_files,
-                                    verbose)
-
-        if not (any_changes or allow_pointless or pending_merges):
-            raise PointlessCommit()
-
-        for file_id in missing_ids:
-            # Any files that have been deleted are now removed from the
-            # working inventory.  Files that were not selected for commit
-            # are left as they were in the working inventory and ommitted
-            # from the revision inventory.
-
-            # have to do this later so we don't mess up the iterator.
-            # since parents may be removed before their children we
-            # have to test.
-
-            # FIXME: There's probably a better way to do this; perhaps
-            # the workingtree should know how to filter itbranch.
-            if work_inv.has_id(file_id):
-                del work_inv[file_id]
-
-
+# XXX: Can we do any better about making interrupted commits change
+# nothing?  Perhaps the best approach is to integrate commit of
+# AtomicFiles with releasing the lock on the Branch.
+
+# TODO: Separate 'prepare' phase where we find a list of potentially
+# committed files.  We then can then pause the commit to prompt for a
+# commit message, knowing the summary will be the same as what's
+# actually used for the commit.  (But perhaps simpler to simply get
+# the tree status, then use that for a selective commit?)
+
+# The newly committed revision is going to have a shape corresponding
+# to that of the working inventory.  Files that are not in the
+# working tree and that were in the predecessor are reported as
+# removed --- this can include files that were either removed from the
+# inventory or deleted in the working tree.  If they were only
+# deleted from disk, they are removed from the working inventory.
+
+# We then consider the remaining entries, which will be in the new
+# version.  Directory entries are simply copied across.  File entries
+# must be checked to see if a new version of the file should be
+# recorded.  For each parent revision inventory, we check to see what
+# version of the file was present.  If the file was present in at
+# least one tree, and if it was the same version in all the trees,
+# then we can just refer to that version.  Otherwise, a new version
+# representing the merger of the file versions must be added.
+
+# TODO: Update hashcache before and after - or does the WorkingTree
+# look after that?
+
+# TODO: Rather than mashing together the ancestry and storing it back,
+# perhaps the weave should have single method which does it all in one
+# go, avoiding a lot of redundant work.
+
+# TODO: Perhaps give a warning if one of the revisions marked as
+# merged is already in the ancestry, and then don't record it as a
+# distinct parent.
+
+# TODO: If the file is newly merged but unchanged from the version it
+# merges from, then it should still be reported as newly added
+# relative to the basis revision.
+
+
+import os
+import re
+import sys
+import time
+import pdb
+
+from binascii import hexlify
+from cStringIO import StringIO
+
+from bzrlib.osutils import (local_time_offset, username,
+                            rand_bytes, compact_date, user_email,
+                            kind_marker, is_inside_any, quotefn,
+                            sha_string, sha_strings, sha_file, isdir, isfile,
+                            split_lines)
+from bzrlib.branch import gen_file_id
+from bzrlib.errors import (BzrError, PointlessCommit,
+                           HistoryMissing,
+                           ConflictsInTree
+                           )
+from bzrlib.revision import Revision
+from bzrlib.trace import mutter, note, warning
+from bzrlib.xml5 import serializer_v5
+from bzrlib.inventory import Inventory, ROOT_ID
+from bzrlib.weave import Weave
+from bzrlib.weavefile import read_weave, write_weave_v5
+from bzrlib.atomicfile import AtomicFile
+
+
+def commit(*args, **kwargs):
+    """Commit a new revision to a branch.
+
+    Function-style interface for convenience of old callers.
+
+    New code should use the Commit class instead.
+    """
+    ## XXX: Remove this in favor of Branch.commit?
+    Commit().commit(*args, **kwargs)
+
+
+class NullCommitReporter(object):
+    """I report on progress of a commit."""
+
+    def snapshot_change(self, change, path):
+        pass
+
+    def completed(self, revno, rev_id):
+        pass
+
+    def deleted(self, file_id):
+        pass
+
+    def escaped(self, escape_count, message):
+        pass
+
+    def missing(self, path):
+        pass
+
+class ReportCommitToLog(NullCommitReporter):
+
+    def snapshot_change(self, change, path):
+        note("%s %s", change, path)
+
+    def completed(self, revno, rev_id):
+        note('committed r%d {%s}', revno, rev_id)
+    
+    def deleted(self, file_id):
+        note('deleted %s', file_id)
+
+    def escaped(self, escape_count, message):
+        note("replaced %d control characters in message", escape_count)
+
+    def missing(self, path):
+        note('missing %s', path)
+
+class Commit(object):
+    """Task of committing a new revision.
+
+    This is a MethodObject: it accumulates state as the commit is
+    prepared, and then it is discarded.  It doesn't represent
+    historical revisions, just the act of recording a new one.
+
+            missing_ids
+            Modified to hold a list of files that have been deleted from
+            the working directory; these should be removed from the
+            working inventory.
+    """
+    def __init__(self,
+                 reporter=None):
+        if reporter is not None:
+            self.reporter = reporter
+        else:
+            self.reporter = NullCommitReporter()
+
+        
+    def commit(self,
+               branch, message,
+               timestamp=None,
+               timezone=None,
+               committer=None,
+               specific_files=None,
+               rev_id=None,
+               allow_pointless=True,
+               verbose=False):
+        """Commit working copy as a new revision.
+
+        timestamp -- if not None, seconds-since-epoch for a
+             postdated/predated commit.
+
+        specific_files -- If true, commit only those files.
+
+        rev_id -- If set, use this as the new revision id.
+            Useful for test or import commands that need to tightly
+            control what revisions are assigned.  If you duplicate
+            a revision id that exists elsewhere it is your own fault.
+            If null (default), a time/random revision id is generated.
+
+        allow_pointless -- If true (default), commit even if nothing
+            has changed and no merges are recorded.
+        """
+        mutter('preparing to commit')
+
+        self.branch = branch
+        self.weave_store = branch.weave_store
+        self.rev_id = rev_id
+        self.specific_files = specific_files
+        self.allow_pointless = allow_pointless
+
+        if timestamp is None:
+            self.timestamp = time.time()
+        else:
+            self.timestamp = long(timestamp)
+            
         if rev_id is None:
-            rev_id = _gen_revision_id(time.time())
-        inv_id = rev_id
-
-        inv_tmp = tempfile.TemporaryFile()
-        pack_xml(new_inv, inv_tmp)
-        inv_tmp.seek(0)
-        branch.inventory_store.add(inv_tmp, inv_id)
-        mutter('new inventory_id is {%s}' % inv_id)
-
-        # We could also just sha hash the inv_tmp file
-        # however, in the case that branch.inventory_store.add()
-        # ever actually does anything special
-        inv_sha1 = branch.get_inventory_sha1(inv_id)
-
-        branch._write_inventory(work_inv)
-
-        if timestamp == None:
-            timestamp = time.time()
-
-        if committer == None:
-            committer = username()
-
-        if timezone == None:
-            timezone = local_time_offset()
-
-        mutter("building commit log message")
-        rev = Revision(timestamp=timestamp,
-                       timezone=timezone,
-                       committer=committer,
-                       message = message,
-                       inventory_id=inv_id,
-                       inventory_sha1=inv_sha1,
-                       revision_id=rev_id)
-
-        rev.parents = []
-        precursor_id = branch.last_patch()
+            self.rev_id = _gen_revision_id(self.branch, self.timestamp)
+        else:
+            self.rev_id = rev_id
+
+        if committer is None:
+            self.committer = username(self.branch)
+        else:
+            assert isinstance(committer, basestring), type(committer)
+            self.committer = committer
+
+        if timezone is None:
+            self.timezone = local_time_offset()
+        else:
+            self.timezone = int(timezone)
+
+        assert isinstance(message, basestring), type(message)
+        self.message = message
+        self._escape_commit_message()
+
+        self.branch.lock_write()
+        try:
+            self.work_tree = self.branch.working_tree()
+            self.work_inv = self.work_tree.inventory
+            self.basis_tree = self.branch.basis_tree()
+            self.basis_inv = self.basis_tree.inventory
+
+            self._gather_parents()
+            if len(self.parents) > 1 and self.specific_files:
+                raise NotImplementedError('selected-file commit of merges is not supported yet')
+            self._check_parents_present()
+            
+            self._remove_deleted()
+            self._populate_new_inv()
+            self._store_snapshot()
+            self._report_deletes()
+
+            if not (self.allow_pointless
+                    or len(self.parents) > 1
+                    or self.new_inv != self.basis_inv):
+                raise PointlessCommit()
+
+            if len(list(self.work_tree.iter_conflicts()))>0:
+                raise ConflictsInTree
+
+            self._record_inventory()
+            self._make_revision()
+            self.reporter.completed(self.branch.revno()+1, self.rev_id)
+            self.branch.append_revision(self.rev_id)
+            self.branch.set_pending_merges([])
+        finally:
+            self.branch.unlock()
+
+    def _record_inventory(self):
+        """Store the inventory for the new revision."""
+        inv_text = serializer_v5.write_inventory_to_string(self.new_inv)
+        self.inv_sha1 = sha_string(inv_text)
+        s = self.branch.control_weaves
+        s.add_text('inventory', self.rev_id,
+                   split_lines(inv_text), self.present_parents,
+                   self.branch.get_transaction())
+
+    def _escape_commit_message(self):
+        """Replace xml-incompatible control characters."""
+        # Python strings can include characters that can't be
+        # represented in well-formed XML; escape characters that
+        # aren't listed in the XML specification
+        # (http://www.w3.org/TR/REC-xml/#NT-Char).
+        if isinstance(self.message, unicode):
+            char_pattern = u'[^\x09\x0A\x0D\u0020-\uD7FF\uE000-\uFFFD]'
+        else:
+            # Use a regular 'str' as pattern to avoid having re.subn
+            # return 'unicode' results.
+            char_pattern = '[^x09\x0A\x0D\x20-\xFF]'
+        self.message, escape_count = re.subn(
+            char_pattern,
+            lambda match: match.group(0).encode('unicode_escape'),
+            self.message)
+        if escape_count:
+            self.reporter.escaped(escape_count, self.message)
+
+    def _gather_parents(self):
+        """Record the parents of a merge for merge detection."""
+        pending_merges = self.branch.pending_merges()
+        self.parents = []
+        self.parent_invs = []
+        self.present_parents = []
+        precursor_id = self.branch.last_revision()
         if precursor_id:
-            precursor_sha1 = branch.get_revision_sha1(precursor_id)
-            rev.parents.append(RevisionReference(precursor_id, precursor_sha1))
-        for merge_rev in pending_merges:
-            rev.parents.append(RevisionReference(merge_rev))            
+            self.parents.append(precursor_id)
+        self.parents += pending_merges
+        for revision in self.parents:
+            if self.branch.has_revision(revision):
+                self.parent_invs.append(self.branch.get_inventory(revision))
+                self.present_parents.append(revision)
 
-        rev_tmp = tempfile.TemporaryFile()
-        pack_xml(rev, rev_tmp)
+    def _check_parents_present(self):
+        for parent_id in self.parents:
+            mutter('commit parent revision {%s}', parent_id)
+            if not self.branch.has_revision(parent_id):
+                if parent_id == self.branch.last_revision():
+                    warning("parent is missing %r", parent_id)
+                    raise HistoryMissing(self.branch, 'revision', parent_id)
+                else:
+                    mutter("commit will ghost revision %r", parent_id)
+            
+    def _make_revision(self):
+        """Record a new revision object for this commit."""
+        self.rev = Revision(timestamp=self.timestamp,
+                            timezone=self.timezone,
+                            committer=self.committer,
+                            message=self.message,
+                            inventory_sha1=self.inv_sha1,
+                            revision_id=self.rev_id)
+        self.rev.parent_ids = self.parents
+        rev_tmp = StringIO()
+        serializer_v5.write_revision(self.rev, rev_tmp)
         rev_tmp.seek(0)
-        branch.revision_store.add(rev_tmp, rev_id)
-        mutter("new revision_id is {%s}" % rev_id)
-
-        ## XXX: Everything up to here can simply be orphaned if we abort
-        ## the commit; it will leave junk files behind but that doesn't
-        ## matter.
-
-        ## TODO: Read back the just-generated changeset, and make sure it
-        ## applies and recreates the right state.
-
-        ## TODO: Also calculate and store the inventory SHA1
-        mutter("committing patch r%d" % (branch.revno() + 1))
-
-        branch.append_revision(rev_id)
-
-        branch.set_pending_merges([])
-
-        if verbose:
-            note("commited r%d" % branch.revno())
-    finally:
-        branch.unlock()
-
-
-
-def _gen_revision_id(when):
+        self.branch.revision_store.add(rev_tmp, self.rev_id)
+        mutter('new revision_id is {%s}', self.rev_id)
+
+    def _remove_deleted(self):
+        """Remove deleted files from the working inventories.
+
+        This is done prior to taking the working inventory as the
+        basis for the new committed inventory.
+
+        This returns true if any files
+        *that existed in the basis inventory* were deleted.
+        Files that were added and deleted
+        in the working copy don't matter.
+        """
+        specific = self.specific_files
+        deleted_ids = []
+        for path, ie in self.work_inv.iter_entries():
+            if specific and not is_inside_any(specific, path):
+                continue
+            if not self.work_tree.has_filename(path):
+                self.reporter.missing(path)
+                deleted_ids.append((path, ie.file_id))
+        if deleted_ids:
+            deleted_ids.sort(reverse=True)
+            for path, file_id in deleted_ids:
+                del self.work_inv[file_id]
+            self.branch._write_inventory(self.work_inv)
+
+    def _store_snapshot(self):
+        """Pass over inventory and record a snapshot.
+
+        Entries get a new revision when they are modified in 
+        any way, which includes a merge with a new set of
+        parents that have the same entry. 
+        """
+        # XXX: Need to think more here about when the user has
+        # made a specific decision on a particular value -- c.f.
+        # mark-merge.  
+        for path, ie in self.new_inv.iter_entries():
+            previous_entries = ie.find_previous_heads(
+                self.parent_invs, 
+                self.weave_store.get_weave_or_empty(ie.file_id,
+                    self.branch.get_transaction()))
+            if ie.revision is None:
+                change = ie.snapshot(self.rev_id, path, previous_entries,
+                                     self.work_tree, self.weave_store,
+                                     self.branch.get_transaction())
+            else:
+                change = "unchanged"
+            self.reporter.snapshot_change(change, path)
+
+    def _populate_new_inv(self):
+        """Build revision inventory.
+
+        This creates a new empty inventory. Depending on
+        which files are selected for commit, and what is present in the
+        current tree, the new inventory is populated. inventory entries 
+        which are candidates for modification have their revision set to
+        None; inventory entries that are carried over untouched have their
+        revision set to their prior value.
+        """
+        mutter("Selecting files for commit with filter %s", self.specific_files)
+        self.new_inv = Inventory()
+        for path, new_ie in self.work_inv.iter_entries():
+            file_id = new_ie.file_id
+            mutter('check %s {%s}', path, new_ie.file_id)
+            if self.specific_files:
+                if not is_inside_any(self.specific_files, path):
+                    mutter('%s not selected for commit', path)
+                    self._carry_entry(file_id)
+                    continue
+                else:
+                    # this is selected, ensure its parents are too.
+                    parent_id = new_ie.parent_id
+                    while parent_id != ROOT_ID:
+                        if not self.new_inv.has_id(parent_id):
+                            ie = self._select_entry(self.work_inv[parent_id])
+                            mutter('%s selected for commit because of %s',
+                                   self.new_inv.id2path(parent_id), path)
+
+                        ie = self.new_inv[parent_id]
+                        if ie.revision is not None:
+                            ie.revision = None
+                            mutter('%s selected for commit because of %s',
+                                   self.new_inv.id2path(parent_id), path)
+                        parent_id = ie.parent_id
+            mutter('%s selected for commit', path)
+            self._select_entry(new_ie)
+
+    def _select_entry(self, new_ie):
+        """Make new_ie be considered for committing."""
+        ie = new_ie.copy()
+        ie.revision = None
+        self.new_inv.add(ie)
+        return ie
+
+    def _carry_entry(self, file_id):
+        """Carry the file unchanged from the basis revision."""
+        if self.basis_inv.has_id(file_id):
+            self.new_inv.add(self.basis_inv[file_id].copy())
+
+    def _report_deletes(self):
+        for file_id in self.basis_inv:
+            if file_id not in self.new_inv:
+                self.reporter.deleted(self.basis_inv.id2path(file_id))
+
+def _gen_revision_id(branch, when):
     """Return new revision-id."""
-    from binascii import hexlify
-    from osutils import rand_bytes, compact_date, user_email
-
-    s = '%s-%s-' % (user_email(), compact_date(when))
+    s = '%s-%s-' % (user_email(branch), compact_date(when))
     s += hexlify(rand_bytes(8))
     return s
-
-
-def _gather_commit(branch, work_tree, work_inv, basis_inv, specific_files,
-                   verbose):
-    """Build inventory preparatory to commit.
-
-    Returns missing_ids, new_inv, any_changes.
-
-    This adds any changed files into the text store, and sets their
-    test-id, sha and size in the returned inventory appropriately.
-
-    missing_ids
-        Modified to hold a list of files that have been deleted from
-        the working directory; these should be removed from the
-        working inventory.
-    """
-    from bzrlib.inventory import Inventory
-    from bzrlib.osutils import isdir, isfile, sha_string, quotefn, \
-         local_time_offset, username, kind_marker, is_inside_any
-    
-    from bzrlib.branch import gen_file_id
-    from bzrlib.errors import BzrError
-    from bzrlib.revision import Revision
-    from bzrlib.trace import mutter, note
-
-    any_changes = False
-    inv = Inventory(work_inv.root.file_id)
-    missing_ids = []
-    
-    for path, entry in work_inv.iter_entries():
-        ## TODO: Check that the file kind has not changed from the previous
-        ## revision of this file (if any).
-
-        p = branch.abspath(path)
-        file_id = entry.file_id
-        mutter('commit prep file %s, id %r ' % (p, file_id))
-
-        if specific_files and not is_inside_any(specific_files, path):
-            mutter('  skipping file excluded from commit')
-            if basis_inv.has_id(file_id):
-                # carry over with previous state
-                inv.add(basis_inv[file_id].copy())
-            else:
-                # omit this from committed inventory
-                pass
-            continue
-
-        if not work_tree.has_id(file_id):
-            if verbose:
-                print('deleted %s%s' % (path, kind_marker(entry.kind)))
-                any_changes = True
-            mutter("    file is missing, removing from inventory")
-            missing_ids.append(file_id)
-            continue
-
-        # this is present in the new inventory; may be new, modified or
-        # unchanged.
-        old_ie = basis_inv.has_id(file_id) and basis_inv[file_id]
-        
-        entry = entry.copy()
-        inv.add(entry)
-
-        if old_ie:
-            old_kind = old_ie.kind
-            if old_kind != entry.kind:
-                raise BzrError("entry %r changed kind from %r to %r"
-                        % (file_id, old_kind, entry.kind))
-
-        if entry.kind == 'directory':
-            if not isdir(p):
-                raise BzrError("%s is entered as directory but not a directory"
-                               % quotefn(p))
-        elif entry.kind == 'file':
-            if not isfile(p):
-                raise BzrError("%s is entered as file but is not a file" % quotefn(p))
-
-            new_sha1 = work_tree.get_file_sha1(file_id)
-
-            if (old_ie
-                and old_ie.text_sha1 == new_sha1):
-                ## assert content == basis.get_file(file_id).read()
-                entry.text_id = old_ie.text_id
-                entry.text_sha1 = new_sha1
-                entry.text_size = old_ie.text_size
-                mutter('    unchanged from previous text_id {%s}' %
-                       entry.text_id)
-            else:
-                content = file(p, 'rb').read()
-
-                # calculate the sha again, just in case the file contents
-                # changed since we updated the cache
-                entry.text_sha1 = sha_string(content)
-                entry.text_size = len(content)
-
-                entry.text_id = gen_file_id(entry.name)
-                branch.text_store.add(content, entry.text_id)
-                mutter('    stored with text_id {%s}' % entry.text_id)
-
-        if verbose:
-            marked = path + kind_marker(entry.kind)
-            if not old_ie:
-                print 'added', marked
-                any_changes = True
-            elif old_ie == entry:
-                pass                    # unchanged
-            elif (old_ie.name == entry.name
-                  and old_ie.parent_id == entry.parent_id):
-                print 'modified', marked
-                any_changes = True
-            else:
-                print 'renamed', marked
-                any_changes = True
-                        
-    return missing_ids, inv, any_changes
-
-

=== added file 'bzrlib/delta.py'
--- bzrlib/delta.py	1970-01-01 00:00:00 +0000
+++ bzrlib/delta.py	2005-10-03 03:30:02 +0000
@@ -0,0 +1,239 @@
+# -*- coding: UTF-8 -*-
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from bzrlib.inventory import InventoryEntry
+from bzrlib.trace import mutter
+
+class TreeDelta(object):
+    """Describes changes from one tree to another.
+
+    Contains four lists:
+
+    added
+        (path, id, kind)
+    removed
+        (path, id, kind)
+    renamed
+        (oldpath, newpath, id, kind, text_modified, meta_modified)
+    modified
+        (path, id, kind, text_modified, meta_modified)
+    unchanged
+        (path, id, kind)
+
+    Each id is listed only once.
+
+    Files that are both modified and renamed are listed only in
+    renamed, with the text_modified flag true. The text_modified
+    applies either to the the content of the file or the target of the
+    symbolic link, depending of the kind of file.
+
+    Files are only considered renamed if their name has changed or
+    their parent directory has changed.  Renaming a directory
+    does not count as renaming all its contents.
+
+    The lists are normally sorted when the delta is created.
+    """
+    def __init__(self):
+        self.added = []
+        self.removed = []
+        self.renamed = []
+        self.modified = []
+        self.unchanged = []
+
+    def __eq__(self, other):
+        if not isinstance(other, TreeDelta):
+            return False
+        return self.added == other.added \
+               and self.removed == other.removed \
+               and self.renamed == other.renamed \
+               and self.modified == other.modified \
+               and self.unchanged == other.unchanged
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def __repr__(self):
+        return "TreeDelta(added=%r, removed=%r, renamed=%r, modified=%r," \
+            " unchanged=%r)" % (self.added, self.removed, self.renamed,
+            self.modified, self.unchanged)
+
+    def has_changed(self):
+        return bool(self.modified
+                    or self.added
+                    or self.removed
+                    or self.renamed)
+
+    def touches_file_id(self, file_id):
+        """Return True if file_id is modified by this delta."""
+        for l in self.added, self.removed, self.modified:
+            for v in l:
+                if v[1] == file_id:
+                    return True
+        for v in self.renamed:
+            if v[2] == file_id:
+                return True
+        return False
+            
+
+    def show(self, to_file, show_ids=False, show_unchanged=False):
+        def show_list(files):
+            for item in files:
+                path, fid, kind = item[:3]
+
+                if kind == 'directory':
+                    path += '/'
+                elif kind == 'symlink':
+                    path += '@'
+
+                if len(item) == 5 and item[4]:
+                    path += '*'
+
+                if show_ids:
+                    print >>to_file, '  %-30s %s' % (path, fid)
+                else:
+                    print >>to_file, ' ', path
+            
+        if self.removed:
+            print >>to_file, 'removed:'
+            show_list(self.removed)
+                
+        if self.added:
+            print >>to_file, 'added:'
+            show_list(self.added)
+
+        if self.renamed:
+            print >>to_file, 'renamed:'
+            for (oldpath, newpath, fid, kind,
+                 text_modified, meta_modified) in self.renamed:
+                if meta_modified:
+                    newpath += '*'
+                if show_ids:
+                    print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
+                else:
+                    print >>to_file, '  %s => %s' % (oldpath, newpath)
+                    
+        if self.modified:
+            print >>to_file, 'modified:'
+            show_list(self.modified)
+            
+        if show_unchanged and self.unchanged:
+            print >>to_file, 'unchanged:'
+            show_list(self.unchanged)
+
+
+
+def compare_trees(old_tree, new_tree, want_unchanged=False, specific_files=None):
+    """Describe changes from one tree to another.
+
+    Returns a TreeDelta with details of added, modified, renamed, and
+    deleted entries.
+
+    The root entry is specifically exempt.
+
+    This only considers versioned files.
+
+    want_unchanged
+        If true, also list files unchanged from one version to
+        the next.
+
+    specific_files
+        If true, only check for changes to specified names or
+        files within them.
+    """
+
+    from osutils import is_inside_any
+    
+    old_inv = old_tree.inventory
+    new_inv = new_tree.inventory
+    delta = TreeDelta()
+    mutter('start compare_trees')
+
+    # TODO: match for specific files can be rather smarter by finding
+    # the IDs of those files up front and then considering only that.
+
+    for file_id in old_tree:
+        if file_id in new_tree:
+            old_ie = old_inv[file_id]
+            new_ie = new_inv[file_id]
+
+            kind = old_ie.kind
+            assert kind == new_ie.kind
+            
+            assert kind in InventoryEntry.known_kinds, \
+                   'invalid file kind %r' % kind
+
+            if kind == 'root_directory':
+                continue
+            
+            if specific_files:
+                if (not is_inside_any(specific_files, old_inv.id2path(file_id)) 
+                    and not is_inside_any(specific_files, new_inv.id2path(file_id))):
+                    continue
+
+            # temporary hack until all entries are populated before clients 
+            # get them
+            old_path = old_inv.id2path(file_id)
+            new_path = new_inv.id2path(file_id)
+            old_ie._read_tree_state(old_path, old_tree)
+            new_ie._read_tree_state(new_path, new_tree)
+            text_modified, meta_modified = new_ie.detect_changes(old_ie)
+
+            # TODO: Can possibly avoid calculating path strings if the
+            # two files are unchanged and their names and parents are
+            # the same and the parents are unchanged all the way up.
+            # May not be worthwhile.
+            
+            if (old_ie.name != new_ie.name
+                or old_ie.parent_id != new_ie.parent_id):
+                delta.renamed.append((old_path,
+                                      new_path,
+                                      file_id, kind,
+                                      text_modified, meta_modified))
+            elif text_modified or meta_modified:
+                delta.modified.append((new_path, file_id, kind,
+                                       text_modified, meta_modified))
+            elif want_unchanged:
+                delta.unchanged.append((new_path, file_id, kind))
+        else:
+            kind = old_inv.get_file_kind(file_id)
+            if kind == 'root_directory':
+                continue
+            old_path = old_inv.id2path(file_id)
+            if specific_files:
+                if not is_inside_any(specific_files, old_path):
+                    continue
+            delta.removed.append((old_path, file_id, kind))
+
+    mutter('start looking for new files')
+    for file_id in new_inv:
+        if file_id in old_inv:
+            continue
+        kind = new_inv.get_file_kind(file_id)
+        if kind == 'root_directory':
+            continue
+        new_path = new_inv.id2path(file_id)
+        if specific_files:
+            if not is_inside_any(specific_files, new_path):
+                continue
+        delta.added.append((new_path, file_id, kind))
+            
+    delta.removed.sort()
+    delta.added.sort()
+    delta.renamed.sort()
+    delta.modified.sort()
+    delta.unchanged.sort()
+
+    return delta

=== modified file 'bzrlib/diff.py'
--- bzrlib/diff.py	2005-07-22 19:33:37 +0000
+++ bzrlib/diff.py	2005-10-03 05:16:47 +0000
@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
@@ -15,9 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from trace import mutter
-from errors import BzrError
-
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrError
+from bzrlib.delta import compare_trees
 
 # TODO: Rather than building a changeset object, we should probably
 # invoke callbacks on an object.  That object can either accumulate a
@@ -42,36 +41,27 @@
     if not oldlines and not newlines:
         return
 
-    nonl = False
-
-    if oldlines and (oldlines[-1][-1] != '\n'):
-        oldlines[-1] += '\n'
-        nonl = True
-    if newlines and (newlines[-1][-1] != '\n'):
-        newlines[-1] += '\n'
-        nonl = True
-
     ud = difflib.unified_diff(oldlines, newlines,
                               fromfile=old_label, tofile=new_label)
 
+    ud = list(ud)
     # work-around for difflib being too smart for its own good
     # if /dev/null is "1,0", patch won't recognize it as /dev/null
     if not oldlines:
-        ud = list(ud)
         ud[2] = ud[2].replace('-1,0', '-0,0')
     elif not newlines:
-        ud = list(ud)
         ud[2] = ud[2].replace('+1,0', '+0,0')
+    # work around for difflib emitting random spaces after the label
+    ud[0] = ud[0][:-2] + '\n'
+    ud[1] = ud[1][:-2] + '\n'
 
     for line in ud:
         to_file.write(line)
-    if nonl:
-        print >>to_file, "\\ No newline at end of file"
+        if not line.endswith('\n'):
+            to_file.write("\n\\ No newline at end of file\n")
     print >>to_file
 
 
-
-
 def external_diff(old_label, oldlines, new_label, newlines, to_file,
                   diff_opts):
     """Display a diff by calling out to the external diff program."""
@@ -149,10 +139,9 @@
     finally:
         oldtmpf.close()                 # and delete
         newtmpf.close()
-    
-
-
-def show_diff(b, revision, specific_files, external_diff_options=None):
+
+def show_diff(b, revision, specific_files, external_diff_options=None,
+              revision2=None, output=None):
     """Shortcut for showing the diff to the working tree.
 
     b
@@ -164,16 +153,21 @@
     The more general form is show_diff_trees(), where the caller
     supplies any two trees.
     """
-    import sys
+    if output is None:
+        import sys
+        output = sys.stdout
 
-    if revision == None:
+    if revision is None:
         old_tree = b.basis_tree()
     else:
-        old_tree = b.revision_tree(b.lookup_revision(revision))
-        
-    new_tree = b.working_tree()
-
-    show_diff_trees(old_tree, new_tree, sys.stdout, specific_files,
+        old_tree = b.revision_tree(revision.in_history(b).rev_id)
+
+    if revision2 is None:
+        new_tree = b.working_tree()
+    else:
+        new_tree = b.revision_tree(revision2.in_history(b).rev_id)
+
+    show_diff_trees(old_tree, new_tree, output, specific_files,
                     external_diff_options)
 
 
@@ -214,244 +208,44 @@
                           specific_files=specific_files)
 
     for path, file_id, kind in delta.removed:
-        print >>to_file, '*** removed %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(old_label + path,
-                      old_tree.get_file(file_id).readlines(),
-                      DEVNULL, 
-                      [],
-                      to_file)
-
+        print >>to_file, '=== removed %s %r' % (kind, path)
+        old_tree.inventory[file_id].diff(diff_file, old_label + path, old_tree,
+                                         DEVNULL, None, None, to_file)
     for path, file_id, kind in delta.added:
-        print >>to_file, '*** added %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(DEVNULL,
-                      [],
-                      new_label + path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-    for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-        print >>to_file, '*** renamed %s %r => %r' % (kind, old_path, new_path)
+        print >>to_file, '=== added %s %r' % (kind, path)
+        new_tree.inventory[file_id].diff(diff_file, new_label + path, new_tree,
+                                         DEVNULL, None, None, to_file, 
+                                         reverse=True)
+    for (old_path, new_path, file_id, kind,
+         text_modified, meta_modified) in delta.renamed:
+        prop_str = get_prop_change(meta_modified)
+        print >>to_file, '=== renamed %s %r => %r%s' % (
+                          kind, old_path, new_path, prop_str)
+        _maybe_diff_file_or_symlink(old_label, old_path, old_tree, file_id,
+                                    new_label, new_path, new_tree,
+                                    text_modified, kind, to_file, diff_file)
+    for path, file_id, kind, text_modified, meta_modified in delta.modified:
+        prop_str = get_prop_change(meta_modified)
+        print >>to_file, '=== modified %s %r%s' % (kind, path, prop_str)
         if text_modified:
-            diff_file(old_label + old_path,
-                      old_tree.get_file(file_id).readlines(),
-                      new_label + new_path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-    for path, file_id, kind in delta.modified:
-        print >>to_file, '*** modified %s %r' % (kind, path)
-        if kind == 'file':
-            diff_file(old_label + path,
-                      old_tree.get_file(file_id).readlines(),
-                      new_label + path,
-                      new_tree.get_file(file_id).readlines(),
-                      to_file)
-
-
-
-class TreeDelta(object):
-    """Describes changes from one tree to another.
-
-    Contains four lists:
-
-    added
-        (path, id, kind)
-    removed
-        (path, id, kind)
-    renamed
-        (oldpath, newpath, id, kind, text_modified)
-    modified
-        (path, id, kind)
-    unchanged
-        (path, id, kind)
-
-    Each id is listed only once.
-
-    Files that are both modified and renamed are listed only in
-    renamed, with the text_modified flag true.
-
-    Files are only considered renamed if their name has changed or
-    their parent directory has changed.  Renaming a directory
-    does not count as renaming all its contents.
-
-    The lists are normally sorted when the delta is created.
-    """
-    def __init__(self):
-        self.added = []
-        self.removed = []
-        self.renamed = []
-        self.modified = []
-        self.unchanged = []
-
-    def __eq__(self, other):
-        if not isinstance(other, TreeDelta):
-            return False
-        return self.added == other.added \
-               and self.removed == other.removed \
-               and self.renamed == other.renamed \
-               and self.modified == other.modified \
-               and self.unchanged == other.unchanged
-
-    def __ne__(self, other):
-        return not (self == other)
-
-    def __repr__(self):
-        return "TreeDelta(added=%r, removed=%r, renamed=%r, modified=%r," \
-            " unchanged=%r)" % (self.added, self.removed, self.renamed,
-            self.modified, self.unchanged)
-
-    def has_changed(self):
-        changes = len(self.added) + len(self.removed) + len(self.renamed)
-        changes += len(self.modified) 
-        return (changes != 0)
-
-    def touches_file_id(self, file_id):
-        """Return True if file_id is modified by this delta."""
-        for l in self.added, self.removed, self.modified:
-            for v in l:
-                if v[1] == file_id:
-                    return True
-        for v in self.renamed:
-            if v[2] == file_id:
-                return True
-        return False
-            
-
-    def show(self, to_file, show_ids=False, show_unchanged=False):
-        def show_list(files):
-            for path, fid, kind in files:
-                if kind == 'directory':
-                    path += '/'
-                elif kind == 'symlink':
-                    path += '@'
-                    
-                if show_ids:
-                    print >>to_file, '  %-30s %s' % (path, fid)
-                else:
-                    print >>to_file, ' ', path
-            
-        if self.removed:
-            print >>to_file, 'removed:'
-            show_list(self.removed)
-                
-        if self.added:
-            print >>to_file, 'added:'
-            show_list(self.added)
-
-        if self.renamed:
-            print >>to_file, 'renamed:'
-            for oldpath, newpath, fid, kind, text_modified in self.renamed:
-                if show_ids:
-                    print >>to_file, '  %s => %s %s' % (oldpath, newpath, fid)
-                else:
-                    print >>to_file, '  %s => %s' % (oldpath, newpath)
-                    
-        if self.modified:
-            print >>to_file, 'modified:'
-            show_list(self.modified)
-            
-        if show_unchanged and self.unchanged:
-            print >>to_file, 'unchanged:'
-            show_list(self.unchanged)
-
-
-
-def compare_trees(old_tree, new_tree, want_unchanged=False, specific_files=None):
-    """Describe changes from one tree to another.
-
-    Returns a TreeDelta with details of added, modified, renamed, and
-    deleted entries.
-
-    The root entry is specifically exempt.
-
-    This only considers versioned files.
-
-    want_unchanged
-        If true, also list files unchanged from one version to
-        the next.
-
-    specific_files
-        If true, only check for changes to specified names or
-        files within them.
-    """
-
-    from osutils import is_inside_any
+            _maybe_diff_file_or_symlink(old_label, path, old_tree, file_id,
+                                        new_label, path, new_tree,
+                                        True, kind, to_file, diff_file)
     
-    old_inv = old_tree.inventory
-    new_inv = new_tree.inventory
-    delta = TreeDelta()
-    mutter('start compare_trees')
-
-    # TODO: match for specific files can be rather smarter by finding
-    # the IDs of those files up front and then considering only that.
-
-    for file_id in old_tree:
-        if file_id in new_tree:
-            old_ie = old_inv[file_id]
-            new_ie = new_inv[file_id]
-
-            kind = old_ie.kind
-            assert kind == new_ie.kind
-            
-            assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
-                   'invalid file kind %r' % kind
-
-            if kind == 'root_directory':
-                continue
-            
-            if specific_files:
-                if (not is_inside_any(specific_files, old_inv.id2path(file_id)) 
-                    and not is_inside_any(specific_files, new_inv.id2path(file_id))):
-                    continue
-
-            if kind == 'file':
-                old_sha1 = old_tree.get_file_sha1(file_id)
-                new_sha1 = new_tree.get_file_sha1(file_id)
-                text_modified = (old_sha1 != new_sha1)
-            else:
-                ## mutter("no text to check for %r %r" % (file_id, kind))
-                text_modified = False
-
-            # TODO: Can possibly avoid calculating path strings if the
-            # two files are unchanged and their names and parents are
-            # the same and the parents are unchanged all the way up.
-            # May not be worthwhile.
-            
-            if (old_ie.name != new_ie.name
-                or old_ie.parent_id != new_ie.parent_id):
-                delta.renamed.append((old_inv.id2path(file_id),
-                                      new_inv.id2path(file_id),
-                                      file_id, kind,
-                                      text_modified))
-            elif text_modified:
-                delta.modified.append((new_inv.id2path(file_id), file_id, kind))
-            elif want_unchanged:
-                delta.unchanged.append((new_inv.id2path(file_id), file_id, kind))
-        else:
-            kind = old_inv.get_file_kind(file_id)
-            old_path = old_inv.id2path(file_id)
-            if specific_files:
-                if not is_inside_any(specific_files, old_path):
-                    continue
-            delta.removed.append((old_path, file_id, kind))
-
-    mutter('start looking for new files')
-    for file_id in new_inv:
-        if file_id in old_inv:
-            continue
-        new_path = new_inv.id2path(file_id)
-        if specific_files:
-            if not is_inside_any(specific_files, new_path):
-                continue
-        kind = new_inv.get_file_kind(file_id)
-        delta.added.append((new_path, file_id, kind))
-            
-    delta.removed.sort()
-    delta.added.sort()
-    delta.renamed.sort()
-    delta.modified.sort()
-    delta.unchanged.sort()
-
-    return delta
+
+def get_prop_change(meta_modified):
+    if meta_modified:
+        return " (properties changed)"
+    else:
+        return  ""
+
+
+def _maybe_diff_file_or_symlink(old_label, old_path, old_tree, file_id,
+                                new_label, new_path, new_tree, text_modified,
+                                kind, to_file, diff_file):
+    if text_modified:
+        new_entry = new_tree.inventory[file_id]
+        old_tree.inventory[file_id].diff(diff_file,
+                                         old_label + old_path, old_tree,
+                                         new_label + new_path, new_entry, 
+                                         new_tree, to_file)

=== modified file 'bzrlib/errors.py'
--- bzrlib/errors.py	2005-07-11 03:40:02 +0000
+++ bzrlib/errors.py	2005-10-08 00:49:13 +0000
@@ -1,4 +1,3 @@
-#! /usr/bin/env python
 # -*- coding: UTF-8 -*-
 
 # This program is free software; you can redistribute it and/or modify
@@ -23,20 +22,53 @@
 ######################################################################
 # exceptions 
 class BzrError(StandardError):
-    pass
+    def __str__(self):
+        # XXX: Should we show the exception class in 
+        # exceptions that don't provide their own message?  
+        # maybe it should be done at a higher level
+        ## n = self.__class__.__name__ + ': '
+        n = ''
+        if len(self.args) == 1:
+            return n + self.args[0]
+        elif len(self.args) == 2:
+            # further explanation or suggestions
+            try:
+                return n + '\n  '.join([self.args[0]] + self.args[1])
+            except TypeError:
+                return n + "%r" % self
+        else:
+            return n + `self.args`
+
 
 class BzrCheckError(BzrError):
     pass
 
 
+class InvalidRevisionNumber(BzrError):
+    def __str__(self):
+        return 'invalid revision number: %r' % self.args[0]
+
+
+class InvalidRevisionId(BzrError):
+    pass
+
+
 class BzrCommandError(BzrError):
     # Error from malformed user command
-    pass
+    def __str__(self):
+        return self.args[0]
 
 
 class NotBranchError(BzrError):
     """Specified path is not in a branch"""
-    pass
+    def __str__(self):
+        return 'not a branch: %s' % self.args[0]
+
+
+class UnsupportedFormatError(BzrError):
+    """Specified path is a bzr branch that we cannot read."""
+    def __str__(self):
+        return 'unsupported branch format: %s' % self.args[0]
 
 
 class NotVersionedError(BzrError):
@@ -68,5 +100,168 @@
             Exception.__init__(self)
 
 
+class CommitNotPossible(LockError):
+    """A commit was attempted but we do not have a write lock open."""
+
+
+class AlreadyCommitted(LockError):
+    """A rollback was requested, but is not able to be accomplished."""
+
+
+class ReadOnlyError(LockError):
+    """A write attempt was made in a read only transaction."""
+
+
 class PointlessCommit(Exception):
     """Commit failed because nothing was changed."""
+
+
+class NoSuchRevision(BzrError):
+    def __init__(self, branch, revision):
+        self.branch = branch
+        self.revision = revision
+        msg = "Branch %s has no revision %s" % (branch, revision)
+        BzrError.__init__(self, msg)
+
+
+class HistoryMissing(BzrError):
+    def __init__(self, branch, object_type, object_id):
+        self.branch = branch
+        BzrError.__init__(self,
+                          '%s is missing %s {%s}'
+                          % (branch, object_type, object_id))
+
+
+class DivergedBranches(BzrError):
+    def __init__(self, branch1, branch2):
+        BzrError.__init__(self, "These branches have diverged.")
+        self.branch1 = branch1
+        self.branch2 = branch2
+
+
+class UnrelatedBranches(BzrCommandError):
+    def __init__(self):
+        msg = "Branches have no common ancestor, and no base revision"\
+            " specified."
+        BzrCommandError.__init__(self, msg)
+
+class NoCommonAncestor(BzrError):
+    def __init__(self, revision_a, revision_b):
+        msg = "Revisions have no common ancestor: %s %s." \
+            % (revision_a, revision_b) 
+        BzrError.__init__(self, msg)
+
+class NoCommonRoot(BzrError):
+    def __init__(self, revision_a, revision_b):
+        msg = "Revisions are not derived from the same root: %s %s." \
+            % (revision_a, revision_b) 
+        BzrError.__init__(self, msg)
+
+class NotAncestor(BzrError):
+    def __init__(self, rev_id, not_ancestor_id):
+        msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
+                                                        rev_id)
+        BzrError.__init__(self, msg)
+        self.rev_id = rev_id
+        self.not_ancestor_id = not_ancestor_id
+
+
+class NotAncestor(BzrError):
+    def __init__(self, rev_id, not_ancestor_id):
+        self.rev_id = rev_id
+        self.not_ancestor_id = not_ancestor_id
+        msg = "Revision %s is not an ancestor of %s" % (not_ancestor_id, 
+                                                        rev_id)
+        BzrError.__init__(self, msg)
+
+
+class InstallFailed(BzrError):
+    def __init__(self, revisions):
+        msg = "Could not install revisions:\n%s" % " ,".join(revisions)
+        BzrError.__init__(self, msg)
+        self.revisions = revisions
+
+
+class AmbiguousBase(BzrError):
+    def __init__(self, bases):
+        msg = "The correct base is unclear, becase %s are all equally close" %\
+            ", ".join(bases)
+        BzrError.__init__(self, msg)
+        self.bases = bases
+
+class NoCommits(BzrError):
+    def __init__(self, branch):
+        msg = "Branch %s has no commits." % branch
+        BzrError.__init__(self, msg)
+
+class UnlistableStore(BzrError):
+    def __init__(self, store):
+        BzrError.__init__(self, "Store %s is not listable" % store)
+
+class UnlistableBranch(BzrError):
+    def __init__(self, br):
+        BzrError.__init__(self, "Stores for branch %s are not listable" % br)
+
+
+from bzrlib.weave import WeaveError, WeaveParentMismatch
+
+class TransportError(BzrError):
+    """All errors thrown by Transport implementations should derive
+    from this class.
+    """
+    def __init__(self, msg=None, orig_error=None):
+        if msg is None and orig_error is not None:
+            msg = str(orig_error)
+        BzrError.__init__(self, msg)
+        self.msg = msg
+        self.orig_error = orig_error
+
+# A set of semi-meaningful errors which can be thrown
+class TransportNotPossible(TransportError):
+    """This is for transports where a specific function is explicitly not
+    possible. Such as pushing files to an HTTP server.
+    """
+    pass
+
+class NonRelativePath(TransportError):
+    """An absolute path was supplied, that could not be decoded into
+    a relative path.
+    """
+    pass
+
+class NoSuchFile(TransportError, IOError):
+    """A get() was issued for a file that doesn't exist."""
+
+    # XXX: Is multiple inheritance for exceptions really needed?
+
+    def __str__(self):
+        return 'no such file: ' + self.msg
+
+    def __init__(self, msg=None, orig_error=None):
+        import errno
+        TransportError.__init__(self, msg=msg, orig_error=orig_error)
+        IOError.__init__(self, errno.ENOENT, self.msg)
+
+class FileExists(TransportError, OSError):
+    """An operation was attempted, which would overwrite an entry,
+    but overwritting is not supported.
+
+    mkdir() can throw this, but put() just overwites existing files.
+    """
+    # XXX: Is multiple inheritance for exceptions really needed?
+    def __init__(self, msg=None, orig_error=None):
+        import errno
+        TransportError.__init__(self, msg=msg, orig_error=orig_error)
+        OSError.__init__(self, errno.EEXIST, self.msg)
+
+class PermissionDenied(TransportError):
+    """An operation cannot succeed because of a lack of permissions."""
+    pass
+
+class ConnectionReset(TransportError):
+    """The connection has been closed."""
+    pass
+
+class ConflictsInTree(BzrError):
+    def __init__(self):
+        BzrError.__init__(self, "Working tree has conflicts.")

=== added file 'bzrlib/externalcommand.py'
--- bzrlib/externalcommand.py	1970-01-01 00:00:00 +0000
+++ bzrlib/externalcommand.py	2005-09-26 08:56:15 +0000
@@ -0,0 +1,63 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# TODO: Perhaps rather than mapping options and arguments back and
+# forth, we should just pass in the whole argv, and allow
+# ExternalCommands to handle it differently to internal commands?
+
+
+import os
+import sys
+
+from bzrlib.commands import Command
+
+
+class ExternalCommand(Command):
+    """Class to wrap external commands."""
+
+    @classmethod
+    def find_command(cls, cmd):
+        import os.path
+        bzrpath = os.environ.get('BZRPATH', '')
+
+        for dir in bzrpath.split(os.pathsep):
+            path = os.path.join(dir, cmd)
+            if os.path.isfile(path):
+                return ExternalCommand(path)
+
+        return None
+
+
+    def __init__(self, path):
+        self.path = path
+
+
+    def name(self):
+        return self.path.split(os.sep)[-1]
+
+
+    def run(self, *args, **kwargs):
+        raise NotImplementedError('should not be called on %r' % self)
+
+
+    def run_argv(self, argv):
+        return os.spawnv(os.P_WAIT, self.path, [self.path] + argv)
+
+
+    def help(self):
+        m = 'external command from %s\n\n' % self.path
+        pipe = os.popen('%s --help' % self.path)
+        return m + pipe.read()

=== added file 'bzrlib/fetch.py'
--- bzrlib/fetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/fetch.py	2005-10-08 00:32:39 +0000
@@ -0,0 +1,237 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from cStringIO import StringIO
+
+import bzrlib
+import bzrlib.errors as errors
+from bzrlib.errors import InstallFailed, NoSuchRevision, WeaveError
+from bzrlib.trace import mutter, note, warning
+from bzrlib.branch import Branch
+from bzrlib.progress import ProgressBar
+from bzrlib.xml5 import serializer_v5
+from bzrlib.osutils import sha_string, split_lines
+
+"""Copying of history from one branch to another.
+
+The basic plan is that every branch knows the history of everything
+that has merged into it.  As the first step of a merge, pull, or
+branch operation we copy history from the source into the destination
+branch.
+
+The copying is done in a slightly complicated order.  We don't want to
+add a revision to the store until everything it refers to is also
+stored, so that if a revision is present we can totally recreate it.
+However, we can't know what files are included in a revision until we
+read its inventory.  Therefore, we first pull the XML and hold it in
+memory until we've updated all of the files referenced.
+"""
+
+# TODO: Avoid repeatedly opening weaves so many times.
+
+# XXX: This doesn't handle ghost (not present in branch) revisions at
+# all yet.  I'm not sure they really should be supported.
+
+# NOTE: This doesn't copy revisions which may be present but not
+# merged into the last revision.  I'm not sure we want to do that.
+
+# - get a list of revisions that need to be pulled in
+# - for each one, pull in that revision file
+#   and get the inventory, and store the inventory with right
+#   parents.
+# - and get the ancestry, and store that with right parents too
+# - and keep a note of all file ids and version seen
+# - then go through all files; for each one get the weave,
+#   and add in all file versions
+
+
+
+def greedy_fetch(to_branch, from_branch, revision=None, pb=None):
+    f = Fetcher(to_branch, from_branch, revision, pb)
+    return f.count_copied, f.failed_revisions
+
+
+
+class Fetcher(object):
+    """Pull revisions and texts from one branch to another.
+
+    This doesn't update the destination's history; that can be done
+    separately if desired.  
+
+    revision_limit
+        If set, pull only up to this revision_id.
+
+    After running:
+
+    last_revision -- if last_revision
+        is given it will be that, otherwise the last revision of
+        from_branch
+
+    count_copied -- number of revisions copied
+
+    count_weaves -- number of file weaves copied
+    """
+    def __init__(self, to_branch, from_branch, last_revision=None, pb=None):
+        if to_branch == from_branch:
+            raise Exception("can't fetch from a branch to itself")
+        self.to_branch = to_branch
+        self.to_weaves = to_branch.weave_store
+        self.to_control = to_branch.control_weaves
+        self.from_branch = from_branch
+        self.from_weaves = from_branch.weave_store
+        self.from_control = from_branch.control_weaves
+        self.failed_revisions = []
+        self.count_copied = 0
+        self.count_total = 0
+        self.count_weaves = 0
+        self.copied_file_ids = set()
+        if pb is None:
+            self.pb = bzrlib.ui.ui_factory.progress_bar()
+        else:
+            self.pb = pb
+        try:
+            self.last_revision = self._find_last_revision(last_revision)
+        except NoSuchRevision, e:
+            mutter('failed getting last revision: %s', e)
+            raise InstallFailed([last_revision])
+        mutter('fetch up to rev {%s}', self.last_revision)
+        try:
+            revs_to_fetch = self._compare_ancestries()
+        except WeaveError:
+            raise InstallFailed([self.last_revision])
+        self._copy_revisions(revs_to_fetch)
+        self.new_ancestry = revs_to_fetch
+
+
+    def _find_last_revision(self, last_revision):
+        """Find the limiting source revision.
+
+        Every ancestor of that revision will be merged across.
+
+        Returns the revision_id, or returns None if there's no history
+        in the source branch."""
+        self.pb.update('get source history')
+        from_history = self.from_branch.revision_history()
+        self.pb.update('get destination history')
+        if last_revision:
+            self.from_branch.get_revision(last_revision)
+            return last_revision
+        elif from_history:
+            return from_history[-1]
+        else:
+            return None                 # no history in the source branch
+            
+
+    def _compare_ancestries(self):
+        """Get a list of revisions that must be copied.
+
+        That is, every revision that's in the ancestry of the source
+        branch and not in the destination branch."""
+        self.pb.update('get source ancestry')
+        self.from_ancestry = self.from_branch.get_ancestry(self.last_revision)
+
+        dest_last_rev = self.to_branch.last_revision()
+        self.pb.update('get destination ancestry')
+        if dest_last_rev:
+            dest_ancestry = self.to_branch.get_ancestry(dest_last_rev)
+        else:
+            dest_ancestry = []
+        ss = set(dest_ancestry)
+        to_fetch = []
+        for rev_id in self.from_ancestry:
+            if rev_id not in ss:
+                to_fetch.append(rev_id)
+                mutter('need to get revision {%s}', rev_id)
+        mutter('need to get %d revisions in total', len(to_fetch))
+        self.count_total = len(to_fetch)
+        return to_fetch
+
+    def _copy_revisions(self, revs_to_fetch):
+        i = 0
+        for rev_id in revs_to_fetch:
+            i += 1
+            if rev_id is None:
+                continue
+            if self.to_branch.has_revision(rev_id):
+                continue
+            self.pb.update('fetch revision', i, self.count_total)
+            self._copy_one_revision(rev_id)
+            self.count_copied += 1
+
+
+    def _copy_one_revision(self, rev_id):
+        """Copy revision and everything referenced by it."""
+        mutter('copying revision {%s}', rev_id)
+        rev_xml = self.from_branch.get_revision_xml(rev_id)
+        inv_xml = self.from_branch.get_inventory_xml(rev_id)
+        rev = serializer_v5.read_revision_from_string(rev_xml)
+        inv = serializer_v5.read_inventory_from_string(inv_xml)
+        assert rev.revision_id == rev_id
+        assert rev.inventory_sha1 == sha_string(inv_xml)
+        mutter('  commiter %s, %d parents',
+               rev.committer,
+               len(rev.parent_ids))
+        self._copy_new_texts(rev_id, inv)
+        parents = rev.parent_ids
+        for parent in parents:
+            if not self.to_branch.has_revision(parent):
+                parents.pop(parents.index(parent))
+        self._copy_inventory(rev_id, inv_xml, parents)
+        self.to_branch.revision_store.add(StringIO(rev_xml), rev_id)
+        mutter('copied revision %s', rev_id)
+
+
+    def _copy_inventory(self, rev_id, inv_xml, parent_ids):
+        self.to_control.add_text('inventory', rev_id,
+                                split_lines(inv_xml), parent_ids,
+                                self.to_branch.get_transaction())
+
+    def _copy_new_texts(self, rev_id, inv):
+        """Copy any new texts occuring in this revision."""
+        # TODO: Rather than writing out weaves every time, hold them
+        # in memory until everything's done?  But this way is nicer
+        # if it's interrupted.
+        for path, ie in inv.iter_entries():
+            if ie.revision != rev_id:
+                continue
+            mutter('%s {%s} is changed in this revision',
+                   path, ie.file_id)
+            self._copy_one_weave(rev_id, ie.file_id)
+
+
+    def _copy_one_weave(self, rev_id, file_id):
+        """Copy one file weave."""
+        mutter('copy file {%s} modified in {%s}', file_id, rev_id)
+        if file_id in self.copied_file_ids:
+            mutter('file {%s} already copied', file_id)
+            return
+        from_weave = self.from_weaves.get_weave(file_id,
+            self.from_branch.get_transaction())
+        to_weave = self.to_weaves.get_weave_or_empty(file_id,
+            self.to_branch.get_transaction())
+        try:
+            to_weave.join(from_weave)
+        except errors.WeaveParentMismatch:
+            to_weave.reweave(from_weave)
+        self.to_weaves.put_weave(file_id, to_weave,
+            self.to_branch.get_transaction())
+        self.count_weaves += 1
+        self.copied_file_ids.add(file_id)
+        mutter('copied file {%s}', file_id)
+
+
+fetch = Fetcher

=== added file 'bzrlib/graph.py'
--- bzrlib/graph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/graph.py	2005-09-21 15:33:23 +0000
@@ -0,0 +1,104 @@
+# (C) 2005 Canonical
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+def max_distance(node, ancestors, distances, root_descendants):
+    """Calculate the max distance to an ancestor.  
+    Return None if not all possible ancestors have known distances"""
+    best = None
+    if node in distances:
+        best = distances[node]
+    for ancestor in ancestors[node]:
+        # skip ancestors we will never traverse:
+        if root_descendants is not None and ancestor not in root_descendants:
+            continue
+        # An ancestor which is not listed in ancestors will never be in
+        # distances, so we pretend it never existed.
+        if ancestor not in ancestors:
+            continue
+        if ancestor not in distances:
+            return None
+        if best is None or distances[ancestor]+1 > best:
+            best = distances[ancestor] + 1
+    return best
+
+    
+def node_distances(graph, ancestors, start, root_descendants=None):
+    """Produce a list of nodes, sorted by distance from a start node.
+    This is an algorithm devised by Aaron Bentley, because applying Dijkstra
+    backwards seemed too complicated.
+
+    For each node, we walk its descendants.  If all the descendant's ancestors
+    have a max-distance-to-start, (excluding ones that can never reach start),
+    we calculate their max-distance-to-start, and schedule their descendants.
+
+    So when a node's last parent acquires a distance, it will acquire a
+    distance on the next iteration.
+
+    Once we know the max distances for all nodes, we can return a list sorted
+    by distance, farthest first.
+    """
+    distances = {start: 0}
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            line_descendants = graph[line]
+            assert line not in line_descendants, "%s refers to itself" % line
+            for descendant in line_descendants:
+                distance = max_distance(descendant, ancestors, distances,
+                                        root_descendants)
+                if distance is None:
+                    continue
+                distances[descendant] = distance
+                new_lines.add(descendant)
+        lines = new_lines
+    return distances
+
+def nodes_by_distance(distances):
+    """Return a list of nodes sorted by distance"""
+    def by_distance(n):
+        return distances[n],n
+
+    node_list = distances.keys()
+    node_list.sort(key=by_distance, reverse=True)
+    return node_list
+
+def select_farthest(distances, common):
+    """Return the farthest common node, or None if no node qualifies."""
+    node_list = nodes_by_distance(distances)
+    for node in node_list:
+        if node in common:
+            return node
+
+def all_descendants(descendants, start):
+    """Produce a set of all descendants of the start node.
+    The input is a map of node->list of descendants for a graph encompassing
+    start.
+    """
+    result = set()
+    lines = set([start])
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            if line not in descendants:
+                continue
+            for descendant in descendants[line]:
+                if descendant in result:
+                    continue
+                result.add(descendant)
+                new_lines.add(descendant)
+        lines = new_lines
+    return result

=== modified file 'bzrlib/hashcache.py'
--- bzrlib/hashcache.py	2005-07-22 18:49:46 +0000
+++ bzrlib/hashcache.py	2005-09-30 02:54:51 +0000
@@ -23,16 +23,21 @@
 # TODO: Perhaps return more details on the file to avoid statting it
 # again: nonexistent, file type, size, etc
 
+# TODO: Perhaps use a Python pickle instead of a text file; might be faster.
+
 
 
 CACHE_HEADER = "### bzr hashcache v5\n"
 
 import os, stat, time
+import sha
 
 from bzrlib.osutils import sha_file
 from bzrlib.trace import mutter, warning
-
-
+from bzrlib.atomicfile import AtomicFile
+
+
+FP_MODE_COLUMN = 5
 
 def _fingerprint(abspath):
     try:
@@ -47,7 +52,7 @@
     # we discard any high precision because it's not reliable; perhaps we
     # could do better on some systems?
     return (fs.st_size, long(fs.st_mtime),
-            long(fs.st_ctime), fs.st_ino, fs.st_dev)
+            long(fs.st_ctime), fs.st_ino, fs.st_dev, fs.st_mode)
 
 
 class HashCache(object):
@@ -135,7 +140,6 @@
                 del self._cache[path]
 
 
-
     def get_sha1(self, path):
         """Return the sha1 of a file.
         """
@@ -161,7 +165,16 @@
             return cache_sha1
         
         self.miss_count += 1
-        digest = sha_file(file(abspath, 'rb', buffering=65000))
+
+
+        mode = file_fp[FP_MODE_COLUMN]
+        if stat.S_ISREG(mode):
+            digest = sha_file(file(abspath, 'rb', buffering=65000))
+        elif stat.S_ISLNK(mode):
+            link_target = os.readlink(abspath)
+            digest = sha.new(os.readlink(abspath)).hexdigest()
+        else:
+            raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
 
         now = int(time.time())
         if file_fp[1] >= now or file_fp[2] >= now:
@@ -177,16 +190,10 @@
             self.update_count += 1
             self.needs_write = True
             self._cache[path] = (digest, file_fp)
-        
         return digest
         
-
-
-
     def write(self):
         """Write contents of cache to file."""
-        from atomicfile import AtomicFile
-
         outf = AtomicFile(self.cache_file_name(), 'wb')
         try:
             print >>outf, CACHE_HEADER,
@@ -222,6 +229,8 @@
             inf = file(fn, 'rb', buffering=65000)
         except IOError, e:
             mutter("failed to open %s: %s" % (fn, e))
+            # better write it now so it is valid
+            self.needs_write = True
             return
 
 
@@ -229,6 +238,7 @@
         if hdr != CACHE_HEADER:
             mutter('cache header marker not found at top of %s; discarding cache'
                    % fn)
+            self.needs_write = True
             return
 
         for l in inf:
@@ -240,7 +250,7 @@
 
             pos += 3
             fields = l[pos:].split(' ')
-            if len(fields) != 6:
+            if len(fields) != 7:
                 warning("bad line in hashcache: %r" % l)
                 continue
 

=== modified file 'bzrlib/help.py'
--- bzrlib/help.py	2005-06-11 01:55:29 +0000
+++ bzrlib/help.py	2005-09-16 14:57:59 +0000
@@ -14,45 +14,40 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Some way to get a list of external commands (defined by shell
+# scripts) so that they can be included in the help listing as well.
+# It should be enough to just list the plugin directory and look for
+# executable files with reasonable names.
+
+# TODO: `help commands --all` should show hidden commands
+
 global_help = \
 """Bazaar-NG -- a free distributed version-control tool
 http://bazaar-ng.org/
 
-**WARNING: THIS IS AN UNSTABLE DEVELOPMENT VERSION**
-
-* Metadata format is not stable yet -- you may need to
-  discard history in the future.
-
-* Many commands unimplemented or partially implemented.
-
-* Space-inefficient storage.
-
-* No merge operators yet.
-
-
-To make a branch, use 'bzr init' in an existing directory, then 'bzr
-add' to make files versioned.  'bzr add .' will recursively add all
-non-ignored files.
-
-'bzr status' describes files that are unknown, ignored, or modified.
-'bzr diff' shows the text changes to the tree or named files.
-'bzr commit -m <MESSAGE>' commits all changes in that branch.
-
-'bzr move' and 'bzr rename' allow you to rename files or directories.
-'bzr remove' makes a file unversioned but keeps the working copy;
-to delete that too simply delete the file.
-
-'bzr log' shows a history of changes, and
-'bzr info' gives summary statistical information.
-'bzr check' validates all files are stored safely.
-
-Files can be ignored by giving a path or a glob in .bzrignore at the
-top of the tree.  Use 'bzr ignored' to see what files are ignored and
-why, and 'bzr unknowns' to see files that are neither versioned or
-ignored.
-
-For more help on any command, type 'bzr help COMMAND', or 'bzr help
-commands' for a list.
+WARNING: This is an unstable development version.
+         Please keep backups.
+
+Basic commands:
+
+  bzr init           makes this directory a versioned branch
+  bzr branch         make a copy of another branch
+
+  bzr add            make files or directories versioned
+  bzr ignore         ignore a file or pattern
+  bzr mv             move or rename a versioned file
+
+  bzr status         summarize changes in working copy
+  bzr diff           show detailed diffs
+
+  bzr merge          pull in changes from another branch
+  bzr commit         save some or all changes
+
+  bzr log            show history of changes
+  bzr check          validate storage
+
+  bzr help init      more help on e.g. init command
+  bzr help commands  list all commands
 """
 
 
@@ -70,13 +65,13 @@
         help_on_command(topic, outfile = outfile)
 
 
-def command_usage(cmdname, cmdclass):
+def command_usage(cmd_object):
     """Return single-line grammar for command.
 
     Only describes arguments, not options.
     """
-    s = cmdname + ' '
-    for aname in cmdclass.takes_args:
+    s = 'bzr ' + cmd_object.name() + ' '
+    for aname in cmd_object.takes_args:
         aname = aname.upper()
         if aname[-1] in ['$', '+']:
             aname = aname[:-1] + '...'
@@ -92,34 +87,37 @@
     return s
 
 
-def help_on_command(cmdname, outfile = None):
+def help_on_command(cmdname, outfile=None):
+    from bzrlib.commands import get_cmd_object
+
     cmdname = str(cmdname)
 
     if outfile == None:
         outfile = sys.stdout
 
-    from inspect import getdoc
-    import commands
-    topic, cmdclass = commands.get_cmd_class(cmdname)
+    cmd_object = get_cmd_object(cmdname)
 
-    doc = getdoc(cmdclass)
+    doc = cmd_object.help()
     if doc == None:
         raise NotImplementedError("sorry, no detailed help yet for %r" % cmdname)
 
-    outfile.write('usage: ' + command_usage(topic, cmdclass) + '\n')
-
-    if cmdclass.aliases:
-        outfile.write('aliases: ' + ', '.join(cmdclass.aliases) + '\n')
-    
+    print >>outfile, 'usage:', command_usage(cmd_object) 
+
+    if cmd_object.aliases:
+        print >>outfile, 'aliases:',
+        print >>outfile, ', '.join(cmd_object.aliases)
+
+    print >>outfile
+
     outfile.write(doc)
     if doc[-1] != '\n':
         outfile.write('\n')
     
-    help_on_option(cmdclass.takes_options, outfile = None)
-
-
-def help_on_option(options, outfile = None):
-    import commands
+    help_on_options(cmd_object.takes_options, outfile=None)
+
+
+def help_on_options(options, outfile=None):
+    from bzrlib.commands import SHORT_OPTIONS
     
     if not options:
         return
@@ -130,32 +128,35 @@
     outfile.write('\noptions:\n')
     for on in options:
         l = '    --' + on
-        for shortname, longname in commands.SHORT_OPTIONS.items():
+        for shortname, longname in SHORT_OPTIONS.items():
             if longname == on:
                 l += ', -' + shortname
                 break
         outfile.write(l + '\n')
 
 
-def help_commands(outfile = None):
+def help_commands(outfile=None):
     """List all commands"""
-    import inspect
-    import commands
+    from bzrlib.commands import (builtin_command_names,
+                                 plugin_command_names,
+                                 get_cmd_object)
 
     if outfile == None:
         outfile = sys.stdout
-    
-    accu = []
-    for cmdname, cmdclass in commands.get_all_cmds():
-        accu.append((cmdname, cmdclass))
-    accu.sort()
-    for cmdname, cmdclass in accu:
-        if cmdclass.hidden:
+
+    names = set()                       # to eliminate duplicates
+    names.update(builtin_command_names())
+    names.update(plugin_command_names())
+    names = list(names)
+    names.sort()
+
+    for cmd_name in names:
+        cmd_object = get_cmd_object(cmd_name)
+        if cmd_object.hidden:
             continue
-        outfile.write(command_usage(cmdname, cmdclass) + '\n')
-        help = inspect.getdoc(cmdclass)
-        if help:
-            outfile.write("    " + help.split('\n', 1)[0] + '\n')
-
-            
-
+        print >>outfile, command_usage(cmd_object)
+        cmd_help = cmd_object.help()
+        if cmd_help:
+            firstline = cmd_help.split('\n', 1)[0]
+            print >>outfile, '    ' + firstline
+        

=== added file 'bzrlib/identitymap.py'
--- bzrlib/identitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/identitymap.py	2005-10-07 04:10:46 +0000
@@ -0,0 +1,63 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides an IdentityMap."""
+
+
+import bzrlib.errors as errors
+
+
+class IdentityMap(object):
+    """An in memory map from object id to instance.
+    
+    An IdentityMap maps from keys to single instances of objects in memory.
+    We have explicit calls on the map for the root of each inheritance tree
+    that is store in the map. Look for find_CLASS and add_CLASS methods.
+    """
+
+    def add_weave(self, id, weave):
+        """Add weave to the map with a given id."""
+        if self._weave_key(id) in self._map:
+            raise errors.BzrError('weave %s already in the identity map' % id)
+        self._map[self._weave_key(id)] = weave
+
+    def find_weave(self, id):
+        """Return the weave for 'id', or None if it is not present."""
+        return self._map.get(self._weave_key(id), None)
+
+    def __init__(self):
+        super(IdentityMap, self).__init__()
+        self._map = {}
+
+    def _weave_key(self, id):
+        """Return the key for a weaves id."""
+        return "weave-" + id
+
+        
+class NullIdentityMap(object):
+    """A pretend in memory map from object id to instance.
+    
+    A NullIdentityMap is an Identity map that does not store anything in it.
+    """
+
+    def add_weave(self, id, weave):
+        """See IdentityMap.add_weave."""
+
+    def find_weave(self, id):
+        """See IdentityMap.find_weave."""
+        return None
+

=== modified file 'bzrlib/info.py'
--- bzrlib/info.py	2005-05-19 08:31:06 +0000
+++ bzrlib/info.py	2005-09-27 07:24:40 +0000
@@ -18,7 +18,7 @@
 
 import time
 
-from osutils import format_date
+from bzrlib.osutils import format_date
 
 
 def _countiter(it):
@@ -96,11 +96,11 @@
         print '  latest revision: %s' % format_date(lastrev.timestamp,
                                                     lastrev.timezone)
 
-    print
-    print 'text store:'
-    c, t = b.text_store.total_size()
-    print '  %8d file texts' % c
-    print '  %8d kB' % (t/1024)
+#     print
+#     print 'text store:'
+#     c, t = b.text_store.total_size()
+#     print '  %8d file texts' % c
+#     print '  %8d kB' % (t/1024)
 
     print
     print 'revision store:'
@@ -109,9 +109,9 @@
     print '  %8d kB' % (t/1024)
 
 
-    print
-    print 'inventory store:'
-    c, t = b.inventory_store.total_size()
-    print '  %8d inventories' % c
-    print '  %8d kB' % (t/1024)
+#     print
+#     print 'inventory store:'
+#     c, t = b.inventory_store.total_size()
+#     print '  %8d inventories' % c
+#     print '  %8d kB' % (t/1024)
 

=== modified file 'bzrlib/intset.py'
--- bzrlib/intset.py	2005-07-17 18:26:45 +0000
+++ bzrlib/intset.py	2005-09-15 13:32:36 +0000
@@ -1,5 +1,3 @@
-#! /usr/bin/python
-
 # Copyright (C) 2005 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
@@ -210,12 +208,3 @@
         if not self._val & m:
             raise KeyError(to_remove)
         self._val ^= m
-        
-        
-            
-    
-
-if __name__ == '__main__':
-    import doctest
-    doctest.testmod()
-    

=== modified file 'bzrlib/inventory.py'
--- bzrlib/inventory.py	2005-07-22 19:32:46 +0000
+++ bzrlib/inventory.py	2005-10-08 00:34:15 +0000
@@ -15,18 +15,29 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
+# TODO: Maybe also keep the full path of the entry, and the children?
+# But those depend on its position within a particular inventory, and
+# it would be nice not to need to hold the backpointer here.
+
 # This should really be an id randomly assigned when the tree is
 # created, but it's not for now.
 ROOT_ID = "TREE_ROOT"
 
 
-import sys, os.path, types, re
+import os.path
+import re
+import sys
+import tarfile
+import types
 
 import bzrlib
 from bzrlib.errors import BzrError, BzrCheckError
 
-from bzrlib.osutils import uuid, quotefn, splitpath, joinpath, appendpath
+from bzrlib.osutils import (pumpfile, quotefn, splitpath, joinpath,
+                            appendpath, sha_strings)
 from bzrlib.trace import mutter
+from bzrlib.errors import NotVersionedError
+
 
 class InventoryEntry(object):
     """Description of a versioned file.
@@ -34,193 +45,332 @@
     An InventoryEntry has the following fields, which are also
     present in the XML inventory-entry element:
 
-    * *file_id*
-    * *name*: (only the basename within the directory, must not
-      contain slashes)
-    * *kind*: "directory" or "file"
-    * *directory_id*: (if absent/null means the branch root directory)
-    * *text_sha1*: only for files
-    * *text_size*: in bytes, only for files 
-    * *text_id*: identifier for the text version, only for files
-
-    InventoryEntries can also exist inside a WorkingTree
-    inventory, in which case they are not yet bound to a
-    particular revision of the file.  In that case the text_sha1,
-    text_size and text_id are absent.
-
+    file_id
+
+    name
+        (within the parent directory)
+
+    parent_id
+        file_id of the parent directory, or ROOT_ID
+
+    revision
+        the revision_id in which this variation of this file was 
+        introduced.
+
+    executable
+        Indicates that this file should be executable on systems
+        that support it.
+
+    text_sha1
+        sha-1 of the text of the file
+        
+    text_size
+        size in bytes of the text of the file
+        
+    (reading a version 4 tree created a text_id field.)
 
     >>> i = Inventory()
     >>> i.path2id('')
     'TREE_ROOT'
-    >>> i.add(InventoryEntry('123', 'src', 'directory', ROOT_ID))
-    >>> i.add(InventoryEntry('2323', 'hello.c', 'file', parent_id='123'))
+    >>> i.add(InventoryDirectory('123', 'src', ROOT_ID))
+    InventoryDirectory('123', 'src', parent_id='TREE_ROOT')
+    >>> i.add(InventoryFile('2323', 'hello.c', parent_id='123'))
+    InventoryFile('2323', 'hello.c', parent_id='123')
     >>> for j in i.iter_entries():
     ...   print j
     ... 
-    ('src', InventoryEntry('123', 'src', kind='directory', parent_id='TREE_ROOT'))
-    ('src/hello.c', InventoryEntry('2323', 'hello.c', kind='file', parent_id='123'))
-    >>> i.add(InventoryEntry('2323', 'bye.c', 'file', '123'))
+    ('src', InventoryDirectory('123', 'src', parent_id='TREE_ROOT'))
+    ('src/hello.c', InventoryFile('2323', 'hello.c', parent_id='123'))
+    >>> i.add(InventoryFile('2323', 'bye.c', '123'))
     Traceback (most recent call last):
     ...
     BzrError: inventory already contains entry with id {2323}
-    >>> i.add(InventoryEntry('2324', 'bye.c', 'file', '123'))
-    >>> i.add(InventoryEntry('2325', 'wibble', 'directory', '123'))
+    >>> i.add(InventoryFile('2324', 'bye.c', '123'))
+    InventoryFile('2324', 'bye.c', parent_id='123')
+    >>> i.add(InventoryDirectory('2325', 'wibble', '123'))
+    InventoryDirectory('2325', 'wibble', parent_id='123')
     >>> i.path2id('src/wibble')
     '2325'
     >>> '2325' in i
     True
-    >>> i.add(InventoryEntry('2326', 'wibble.c', 'file', '2325'))
+    >>> i.add(InventoryFile('2326', 'wibble.c', '2325'))
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
     >>> i['2326']
-    InventoryEntry('2326', 'wibble.c', kind='file', parent_id='2325')
-    >>> for j in i.iter_entries():
-    ...     print j[0]
-    ...     assert i.path2id(j[0])
+    InventoryFile('2326', 'wibble.c', parent_id='2325')
+    >>> for path, entry in i.iter_entries():
+    ...     print path.replace('\\\\', '/')     # for win32 os.sep
+    ...     assert i.path2id(path)
     ... 
     src
     src/bye.c
     src/hello.c
     src/wibble
     src/wibble/wibble.c
-    >>> i.id2path('2326')
+    >>> i.id2path('2326').replace('\\\\', '/')
     'src/wibble/wibble.c'
-
-    TODO: Maybe also keep the full path of the entry, and the children?
-           But those depend on its position within a particular inventory, and
-           it would be nice not to need to hold the backpointer here.
     """
-
-    # TODO: split InventoryEntry into subclasses for files,
-    # directories, etc etc.
-
+    
     __slots__ = ['text_sha1', 'text_size', 'file_id', 'name', 'kind',
-                 'text_id', 'parent_id', 'children', ]
-
-    def __init__(self, file_id, name, kind, parent_id, text_id=None):
+                 'text_id', 'parent_id', 'children', 'executable', 
+                 'revision']
+
+    def _add_text_to_weave(self, new_lines, parents, weave_store, transaction):
+        weave_store.add_text(self.file_id, self.revision, new_lines, parents,
+                             transaction)
+
+    def detect_changes(self, old_entry):
+        """Return a (text_modified, meta_modified) from this to old_entry.
+        
+        _read_tree_state must have been called on self and old_entry prior to 
+        calling detect_changes.
+        """
+        return False, False
+
+    def diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """Perform a diff from this to to_entry.
+
+        text_diff will be used for textual difference calculation.
+        This is a template method, override _diff in child classes.
+        """
+        self._read_tree_state(tree.id2path(self.file_id), tree)
+        if to_entry:
+            # cannot diff from one kind to another - you must do a removal
+            # and an addif they do not match.
+            assert self.kind == to_entry.kind
+            to_entry._read_tree_state(to_tree.id2path(to_entry.file_id),
+                                      to_tree)
+        self._diff(text_diff, from_label, tree, to_label, to_entry, to_tree,
+                   output_to, reverse)
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """Perform a diff between two entries of the same kind."""
+
+    def find_previous_heads(self, previous_inventories, entry_weave):
+        """Return the revisions and entries that directly preceed this.
+
+        Returned as a map from revision to inventory entry.
+
+        This is a map containing the file revisions in all parents
+        for which the file exists, and its revision is not a parent of
+        any other. If the file is new, the set will be empty.
+        """
+        def get_ancestors(weave, entry):
+            return set(map(weave.idx_to_name,
+                           weave.inclusions([weave.lookup(entry.revision)])))
+        heads = {}
+        head_ancestors = {}
+        for inv in previous_inventories:
+            if self.file_id in inv:
+                ie = inv[self.file_id]
+                assert ie.file_id == self.file_id
+                if ie.revision in heads:
+                    # fixup logic, there was a bug in revision updates.
+                    # with x bit support.
+                    try:
+                        if heads[ie.revision].executable != ie.executable:
+                            heads[ie.revision].executable = False
+                            ie.executable = False
+                    except AttributeError:
+                        pass
+                    assert heads[ie.revision] == ie
+                else:
+                    # may want to add it.
+                    # may already be covered:
+                    already_present = 0 != len(
+                        [head for head in heads 
+                         if ie.revision in head_ancestors[head]])
+                    if already_present:
+                        # an ancestor of a known head.
+                        continue
+                    # definately a head:
+                    ancestors = get_ancestors(entry_weave, ie)
+                    # may knock something else out:
+                    check_heads = list(heads.keys())
+                    for head in check_heads:
+                        if head in ancestors:
+                            # this head is not really a head
+                            heads.pop(head)
+                    head_ancestors[ie.revision] = ancestors
+                    heads[ie.revision] = ie
+        return heads
+
+    def get_tar_item(self, root, dp, now, tree):
+        """Get a tarfile item and a file stream for its content."""
+        item = tarfile.TarInfo(os.path.join(root, dp))
+        # TODO: would be cool to actually set it to the timestamp of the
+        # revision it was last changed
+        item.mtime = now
+        fileobj = self._put_in_tar(item, tree)
+        return item, fileobj
+
+    def has_text(self):
+        """Return true if the object this entry represents has textual data.
+
+        Note that textual data includes binary content.
+
+        Also note that all entries get weave files created for them.
+        This attribute is primarily used when upgrading from old trees that
+        did not have the weave index for all inventory entries.
+        """
+        return False
+
+    def __init__(self, file_id, name, parent_id, text_id=None):
         """Create an InventoryEntry
         
         The filename must be a single component, relative to the
         parent directory; it cannot be a whole path or relative name.
 
-        >>> e = InventoryEntry('123', 'hello.c', 'file', ROOT_ID)
+        >>> e = InventoryFile('123', 'hello.c', ROOT_ID)
         >>> e.name
         'hello.c'
         >>> e.file_id
         '123'
-        >>> e = InventoryEntry('123', 'src/hello.c', 'file', ROOT_ID)
+        >>> e = InventoryFile('123', 'src/hello.c', ROOT_ID)
         Traceback (most recent call last):
         BzrCheckError: InventoryEntry name 'src/hello.c' is invalid
         """
+        assert isinstance(name, basestring), name
         if '/' in name or '\\' in name:
             raise BzrCheckError('InventoryEntry name %r is invalid' % name)
         
+        self.executable = False
+        self.revision = None
         self.text_sha1 = None
         self.text_size = None
-    
         self.file_id = file_id
         self.name = name
-        self.kind = kind
         self.text_id = text_id
         self.parent_id = parent_id
-        if kind == 'directory':
-            self.children = {}
-        elif kind == 'file':
-            pass
-        else:
-            raise BzrError("unhandled entry kind %r" % kind)
-
-
+        self.symlink_target = None
+
+    def kind_character(self):
+        """Return a short kind indicator useful for appending to names."""
+        raise BzrError('unknown kind %r' % self.kind)
+
+    known_kinds = ('file', 'directory', 'symlink', 'root_directory')
+
+    def _put_in_tar(self, item, tree):
+        """populate item for stashing in a tar, and return the content stream.
+
+        If no content is available, return None.
+        """
+        raise BzrError("don't know how to export {%s} of kind %r" %
+                       (self.file_id, self.kind))
+
+    def put_on_disk(self, dest, dp, tree):
+        """Create a representation of self on disk in the prefix dest.
+        
+        This is a template method - implement _put_on_disk in subclasses.
+        """
+        fullpath = appendpath(dest, dp)
+        self._put_on_disk(fullpath, tree)
+        mutter("  export {%s} kind %s to %s" % (self.file_id, self.kind, fullpath))
+
+    def _put_on_disk(self, fullpath, tree):
+        """Put this entry onto disk at fullpath, from tree tree."""
+        raise BzrError("don't know how to export {%s} of kind %r" % (self.file_id, self.kind))
 
     def sorted_children(self):
         l = self.children.items()
         l.sort()
         return l
 
+    @staticmethod
+    def versionable_kind(kind):
+        return kind in ('file', 'directory', 'symlink')
+
+    def check(self, checker, rev_id, inv, tree):
+        """Check this inventory entry is intact.
+
+        This is a template method, override _check for kind specific
+        tests.
+        """
+        if self.parent_id != None:
+            if not inv.has_id(self.parent_id):
+                raise BzrCheckError('missing parent {%s} in inventory for revision {%s}'
+                        % (self.parent_id, rev_id))
+        self._check(checker, rev_id, tree)
+
+    def _check(self, checker, rev_id, tree):
+        """Check this inventory entry for kind specific errors."""
+        raise BzrCheckError('unknown entry kind %r in revision {%s}' % 
+                            (self.kind, rev_id))
+
 
     def copy(self):
-        other = InventoryEntry(self.file_id, self.name, self.kind,
-                               self.parent_id, text_id=self.text_id)
-        other.text_sha1 = self.text_sha1
-        other.text_size = self.text_size
-        # note that children are *not* copied; they're pulled across when
-        # others are added
-        return other
+        """Clone this inventory entry."""
+        raise NotImplementedError
 
+    def _get_snapshot_change(self, previous_entries):
+        if len(previous_entries) > 1:
+            return 'merged'
+        elif len(previous_entries) == 0:
+            return 'added'
+        else:
+            return 'modified/renamed/reparented'
 
     def __repr__(self):
-        return ("%s(%r, %r, kind=%r, parent_id=%r)"
+        return ("%s(%r, %r, parent_id=%r)"
                 % (self.__class__.__name__,
                    self.file_id,
                    self.name,
-                   self.kind,
                    self.parent_id))
 
-    
-    def to_element(self):
-        """Convert to XML element"""
-        from bzrlib.xml import Element
-        
-        e = Element('entry')
-
-        e.set('name', self.name)
-        e.set('file_id', self.file_id)
-        e.set('kind', self.kind)
-
-        if self.text_size != None:
-            e.set('text_size', '%d' % self.text_size)
-            
-        for f in ['text_id', 'text_sha1']:
-            v = getattr(self, f)
-            if v != None:
-                e.set(f, v)
-
-        # to be conservative, we don't externalize the root pointers
-        # for now, leaving them as null in the xml form.  in a future
-        # version it will be implied by nested elements.
-        if self.parent_id != ROOT_ID:
-            assert isinstance(self.parent_id, basestring)
-            e.set('parent_id', self.parent_id)
-
-        e.tail = '\n'
-            
-        return e
-
-
-    def from_element(cls, elt):
-        assert elt.tag == 'entry'
-
-        ## original format inventories don't have a parent_id for
-        ## nodes in the root directory, but it's cleaner to use one
-        ## internally.
-        parent_id = elt.get('parent_id')
-        if parent_id == None:
-            parent_id = ROOT_ID
-
-        self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-        self.text_id = elt.get('text_id')
-        self.text_sha1 = elt.get('text_sha1')
-        
-        ## mutter("read inventoryentry: %r" % (elt.attrib))
-
-        v = elt.get('text_size')
-        self.text_size = v and int(v)
-
-        return self
-            
-
-    from_element = classmethod(from_element)
+    def snapshot(self, revision, path, previous_entries,
+                 work_tree, weave_store, transaction):
+        """Make a snapshot of this entry which may or may not have changed.
+        
+        This means that all its fields are populated, that it has its
+        text stored in the text store or weave.
+        """
+        mutter('new parents of %s are %r', path, previous_entries)
+        self._read_tree_state(path, work_tree)
+        if len(previous_entries) == 1:
+            # cannot be unchanged unless there is only one parent file rev.
+            parent_ie = previous_entries.values()[0]
+            if self._unchanged(parent_ie):
+                mutter("found unchanged entry")
+                self.revision = parent_ie.revision
+                return "unchanged"
+        return self.snapshot_revision(revision, previous_entries, 
+                                      work_tree, weave_store, transaction)
+
+    def snapshot_revision(self, revision, previous_entries, work_tree,
+                          weave_store, transaction):
+        """Record this revision unconditionally."""
+        mutter('new revision for {%s}', self.file_id)
+        self.revision = revision
+        change = self._get_snapshot_change(previous_entries)
+        self._snapshot_text(previous_entries, work_tree, weave_store,
+                            transaction)
+        return change
+
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction): 
+        """Record the 'text' of this entry, whatever form that takes.
+        
+        This default implementation simply adds an empty text.
+        """
+        mutter('storing file {%s} in revision {%s}',
+               self.file_id, self.revision)
+        self._add_text_to_weave([], file_parents, weave_store, transaction)
 
     def __eq__(self, other):
         if not isinstance(other, InventoryEntry):
             return NotImplemented
 
-        return (self.file_id == other.file_id) \
-               and (self.name == other.name) \
-               and (self.text_sha1 == other.text_sha1) \
-               and (self.text_size == other.text_size) \
-               and (self.text_id == other.text_id) \
-               and (self.parent_id == other.parent_id) \
-               and (self.kind == other.kind)
-
+        return ((self.file_id == other.file_id)
+                and (self.name == other.name)
+                and (other.symlink_target == self.symlink_target)
+                and (self.text_sha1 == other.text_sha1)
+                and (self.text_size == other.text_size)
+                and (self.text_id == other.text_id)
+                and (self.parent_id == other.parent_id)
+                and (self.kind == other.kind)
+                and (self.revision == other.revision)
+                and (self.executable == other.executable)
+                )
 
     def __ne__(self, other):
         return not (self == other)
@@ -228,9 +378,33 @@
     def __hash__(self):
         raise ValueError('not hashable')
 
+    def _unchanged(self, previous_ie):
+        """Has this entry changed relative to previous_ie.
+
+        This method should be overriden in child classes.
+        """
+        compatible = True
+        # different inv parent
+        if previous_ie.parent_id != self.parent_id:
+            compatible = False
+        # renamed
+        elif previous_ie.name != self.name:
+            compatible = False
+        return compatible
+
+    def _read_tree_state(self, path, work_tree):
+        """Populate fields in the inventory entry from the given tree.
+        
+        Note that this should be modified to be a noop on virtual trees
+        as all entries created there are prepopulated.
+        """
 
 
 class RootEntry(InventoryEntry):
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+
     def __init__(self, file_id):
         self.file_id = file_id
         self.children = {}
@@ -246,6 +420,251 @@
                and (self.children == other.children)
 
 
+class InventoryDirectory(InventoryEntry):
+    """A directory in an inventory."""
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+            raise BzrCheckError('directory {%s} has text in revision {%s}'
+                                % (self.file_id, rev_id))
+
+    def copy(self):
+        other = InventoryDirectory(self.file_id, self.name, self.parent_id)
+        other.revision = self.revision
+        # note that children are *not* copied; they're pulled across when
+        # others are added
+        return other
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryDirectory, self).__init__(file_id, name, parent_id)
+        self.children = {}
+        self.kind = 'directory'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return '/'
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.DIRTYPE
+        fileobj = None
+        item.name += '/'
+        item.size = 0
+        item.mode = 0755
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        os.mkdir(fullpath)
+
+
+class InventoryFile(InventoryEntry):
+    """A file in an inventory."""
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        revision = self.revision
+        t = (self.file_id, revision)
+        if t in checker.checked_texts:
+            prev_sha = checker.checked_texts[t] 
+            if prev_sha != self.text_sha1:
+                raise BzrCheckError('mismatched sha1 on {%s} in {%s}' %
+                                    (self.file_id, rev_id))
+            else:
+                checker.repeated_text_cnt += 1
+                return
+        mutter('check version {%s} of {%s}', rev_id, self.file_id)
+        file_lines = tree.get_file_lines(self.file_id)
+        checker.checked_text_cnt += 1 
+        if self.text_size != sum(map(len, file_lines)):
+            raise BzrCheckError('text {%s} wrong size' % self.text_id)
+        if self.text_sha1 != sha_strings(file_lines):
+            raise BzrCheckError('text {%s} wrong sha1' % self.text_id)
+        checker.checked_texts[t] = self.text_sha1
+
+    def copy(self):
+        other = InventoryFile(self.file_id, self.name, self.parent_id)
+        other.executable = self.executable
+        other.text_id = self.text_id
+        other.text_sha1 = self.text_sha1
+        other.text_size = self.text_size
+        other.revision = self.revision
+        return other
+
+    def detect_changes(self, old_entry):
+        """See InventoryEntry.detect_changes."""
+        assert self.text_sha1 != None
+        assert old_entry.text_sha1 != None
+        text_modified = (self.text_sha1 != old_entry.text_sha1)
+        meta_modified = (self.executable != old_entry.executable)
+        return text_modified, meta_modified
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """See InventoryEntry._diff."""
+        from_text = tree.get_file(self.file_id).readlines()
+        if to_entry:
+            to_text = to_tree.get_file(to_entry.file_id).readlines()
+        else:
+            to_text = []
+        if not reverse:
+            text_diff(from_label, from_text,
+                      to_label, to_text, output_to)
+        else:
+            text_diff(to_label, to_text,
+                      from_label, from_text, output_to)
+
+    def has_text(self):
+        """See InventoryEntry.has_text."""
+        return True
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryFile, self).__init__(file_id, name, parent_id)
+        self.kind = 'file'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return ''
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        item.type = tarfile.REGTYPE
+        fileobj = tree.get_file(self.file_id)
+        item.size = self.text_size
+        if tree.is_executable(self.file_id):
+            item.mode = 0755
+        else:
+            item.mode = 0644
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        pumpfile(tree.get_file(self.file_id), file(fullpath, 'wb'))
+        if tree.is_executable(self.file_id):
+            os.chmod(fullpath, 0755)
+
+    def _read_tree_state(self, path, work_tree):
+        """See InventoryEntry._read_tree_state."""
+        self.text_sha1 = work_tree.get_file_sha1(self.file_id)
+        self.executable = work_tree.is_executable(self.file_id)
+
+    def _snapshot_text(self, file_parents, work_tree, weave_store, transaction):
+        """See InventoryEntry._snapshot_text."""
+        mutter('storing file {%s} in revision {%s}',
+               self.file_id, self.revision)
+        # special case to avoid diffing on renames or 
+        # reparenting
+        if (len(file_parents) == 1
+            and self.text_sha1 == file_parents.values()[0].text_sha1
+            and self.text_size == file_parents.values()[0].text_size):
+            previous_ie = file_parents.values()[0]
+            weave_store.add_identical_text(
+                self.file_id, previous_ie.revision, 
+                self.revision, file_parents, transaction)
+        else:
+            new_lines = work_tree.get_file(self.file_id).readlines()
+            self._add_text_to_weave(new_lines, file_parents, weave_store,
+                                    transaction)
+            self.text_sha1 = sha_strings(new_lines)
+            self.text_size = sum(map(len, new_lines))
+
+
+    def _unchanged(self, previous_ie):
+        """See InventoryEntry._unchanged."""
+        compatible = super(InventoryFile, self)._unchanged(previous_ie)
+        if self.text_sha1 != previous_ie.text_sha1:
+            compatible = False
+        else:
+            # FIXME: 20050930 probe for the text size when getting sha1
+            # in _read_tree_state
+            self.text_size = previous_ie.text_size
+        if self.executable != previous_ie.executable:
+            compatible = False
+        return compatible
+
+
+class InventoryLink(InventoryEntry):
+    """A file in an inventory."""
+
+    __slots__ = ['symlink_target']
+
+    def _check(self, checker, rev_id, tree):
+        """See InventoryEntry._check"""
+        if self.text_sha1 != None or self.text_size != None or self.text_id != None:
+            raise BzrCheckError('symlink {%s} has text in revision {%s}'
+                    % (self.file_id, rev_id))
+        if self.symlink_target == None:
+            raise BzrCheckError('symlink {%s} has no target in revision {%s}'
+                    % (self.file_id, rev_id))
+
+    def copy(self):
+        other = InventoryLink(self.file_id, self.name, self.parent_id)
+        other.symlink_target = self.symlink_target
+        other.revision = self.revision
+        return other
+
+    def detect_changes(self, old_entry):
+        """See InventoryEntry.detect_changes."""
+        # FIXME: which _modified field should we use ? RBC 20051003
+        text_modified = (self.symlink_target != old_entry.symlink_target)
+        if text_modified:
+            mutter("    symlink target changed")
+        meta_modified = False
+        return text_modified, meta_modified
+
+    def _diff(self, text_diff, from_label, tree, to_label, to_entry, to_tree,
+             output_to, reverse=False):
+        """See InventoryEntry._diff."""
+        from_text = self.symlink_target
+        if to_entry is not None:
+            to_text = to_entry.symlink_target
+            if reverse:
+                temp = from_text
+                from_text = to_text
+                to_text = temp
+            print >>output_to, '=== target changed %r => %r' % (from_text, to_text)
+        else:
+            if not reverse:
+                print >>output_to, '=== target was %r' % self.symlink_target
+            else:
+                print >>output_to, '=== target is %r' % self.symlink_target
+
+    def __init__(self, file_id, name, parent_id):
+        super(InventoryLink, self).__init__(file_id, name, parent_id)
+        self.kind = 'symlink'
+
+    def kind_character(self):
+        """See InventoryEntry.kind_character."""
+        return ''
+
+    def _put_in_tar(self, item, tree):
+        """See InventoryEntry._put_in_tar."""
+        iterm.type = tarfile.SYMTYPE
+        fileobj = None
+        item.size = 0
+        item.mode = 0755
+        item.linkname = self.symlink_target
+        return fileobj
+
+    def _put_on_disk(self, fullpath, tree):
+        """See InventoryEntry._put_on_disk."""
+        try:
+            os.symlink(self.symlink_target, fullpath)
+        except OSError,e:
+            raise BzrError("Failed to create symlink %r -> %r, error: %s" % (fullpath, self.symlink_target, e))
+
+    def _read_tree_state(self, path, work_tree):
+        """See InventoryEntry._read_tree_state."""
+        self.symlink_target = work_tree.get_symlink_target(self.file_id)
+
+    def _unchanged(self, previous_ie):
+        """See InventoryEntry._unchanged."""
+        compatible = super(InventoryLink, self)._unchanged(previous_ie)
+        if self.symlink_target != previous_ie.symlink_target:
+            compatible = False
+        return compatible
+
 
 class Inventory(object):
     """Inventory of versioned files in a tree.
@@ -265,7 +684,8 @@
     inserted, other than through the Inventory API.
 
     >>> inv = Inventory()
-    >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
+    >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT')
     >>> inv['123-123'].name
     'hello.c'
 
@@ -281,7 +701,8 @@
     >>> [x[0] for x in inv.iter_entries()]
     ['hello.c']
     >>> inv = Inventory('TREE_ROOT-12345678-12345678')
-    >>> inv.add(InventoryEntry('123-123', 'hello.c', 'file', ROOT_ID))
+    >>> inv.add(InventoryFile('123-123', 'hello.c', ROOT_ID))
+    InventoryFile('123-123', 'hello.c', parent_id='TREE_ROOT-12345678-12345678')
     """
     def __init__(self, root_id=ROOT_ID):
         """Create or read an inventory.
@@ -293,7 +714,7 @@
         The inventory is created with a default root directory, with
         an id of None.
         """
-        # We are letting Branch(init=True) create a unique inventory
+        # We are letting Branch.initialize() create a unique inventory
         # root id. Rather than generating a random one here.
         #if root_id is None:
         #    root_id = bzrlib.branch.gen_file_id('TREE_ROOT')
@@ -301,6 +722,17 @@
         self._byid = {self.root.file_id: self.root}
 
 
+    def copy(self):
+        other = Inventory(self.root.file_id)
+        # copy recursively so we know directories will be added before
+        # their children.  There are more efficient ways than this...
+        for path, entry in self.iter_entries():
+            if entry == self.root:
+                continue
+            other.add(entry.copy())
+        return other
+
+
     def __iter__(self):
         return iter(self._byid)
 
@@ -368,7 +800,8 @@
         """True if this entry contains a file with given id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
+        >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> '456' in inv
@@ -381,7 +814,8 @@
         """Return the entry for given file_id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123123', 'hello.c', 'file', ROOT_ID))
+        >>> inv.add(InventoryFile('123123', 'hello.c', ROOT_ID))
+        InventoryFile('123123', 'hello.c', parent_id='TREE_ROOT')
         >>> inv['123123'].name
         'hello.c'
         """
@@ -405,7 +839,10 @@
         """Add entry to inventory.
 
         To add  a file to a branch ready to be committed, use Branch.add,
-        which calls this."""
+        which calls this.
+
+        Returns the new entry object.
+        """
         if entry.file_id in self._byid:
             raise BzrError("inventory already contains entry with id {%s}" % entry.file_id)
 
@@ -423,20 +860,22 @@
 
         self._byid[entry.file_id] = entry
         parent.children[entry.name] = entry
+        return entry
 
 
     def add_path(self, relpath, kind, file_id=None):
         """Add entry from a path.
 
-        The immediate parent must already be versioned"""
-        from bzrlib.errors import NotVersionedError
+        The immediate parent must already be versioned.
+
+        Returns the new entry object."""
+        from bzrlib.branch import gen_file_id
         
         parts = bzrlib.osutils.splitpath(relpath)
         if len(parts) == 0:
             raise BzrError("cannot re-add root of inventory")
 
         if file_id == None:
-            from bzrlib.branch import gen_file_id
             file_id = gen_file_id(relpath)
 
         parent_path = parts[:-1]
@@ -444,8 +883,14 @@
         if parent_id == None:
             raise NotVersionedError(parent_path)
 
-        ie = InventoryEntry(file_id, parts[-1],
-                            kind=kind, parent_id=parent_id)
+        if kind == 'directory':
+            ie = InventoryDirectory(file_id, parts[-1], parent_id)
+        elif kind == 'file':
+            ie = InventoryFile(file_id, parts[-1], parent_id)
+        elif kind == 'symlink':
+            ie = InventoryLink(file_id, parts[-1], parent_id)
+        else:
+            raise BzrError("unknown kind %r" % kind)
         return self.add(ie)
 
 
@@ -453,7 +898,8 @@
         """Remove entry by id.
 
         >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('123', 'foo.c', 'file', ROOT_ID))
+        >>> inv.add(InventoryFile('123', 'foo.c', ROOT_ID))
+        InventoryFile('123', 'foo.c', parent_id='TREE_ROOT')
         >>> '123' in inv
         True
         >>> del inv['123']
@@ -475,43 +921,6 @@
         del self[ie.parent_id].children[ie.name]
 
 
-    def to_element(self):
-        """Convert to XML Element"""
-        from bzrlib.xml import Element
-        
-        e = Element('inventory')
-        e.text = '\n'
-        if self.root.file_id not in (None, ROOT_ID):
-            e.set('file_id', self.root.file_id)
-        for path, ie in self.iter_entries():
-            e.append(ie.to_element())
-        return e
-    
-
-    def from_element(cls, elt):
-        """Construct from XML Element
-        
-        >>> inv = Inventory()
-        >>> inv.add(InventoryEntry('foo.c-123981239', 'foo.c', 'file', ROOT_ID))
-        >>> elt = inv.to_element()
-        >>> inv2 = Inventory.from_element(elt)
-        >>> inv2 == inv
-        True
-        """
-        # XXXX: doctest doesn't run this properly under python2.3
-        assert elt.tag == 'inventory'
-        root_id = elt.get('file_id') or ROOT_ID
-        o = cls(root_id)
-        for e in elt:
-            ie = InventoryEntry.from_element(e)
-            if ie.parent_id == ROOT_ID:
-                ie.parent_id = root_id
-            o.add(ie)
-        return o
-        
-    from_element = classmethod(from_element)
-
-
     def __eq__(self, other):
         """Compare two sets by comparing their contents.
 
@@ -519,10 +928,12 @@
         >>> i2 = Inventory()
         >>> i1 == i2
         True
-        >>> i1.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
+        >>> i1.add(InventoryFile('123', 'foo', ROOT_ID))
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
         >>> i1 == i2
         False
-        >>> i2.add(InventoryEntry('123', 'foo', 'file', ROOT_ID))
+        >>> i2.add(InventoryFile('123', 'foo', ROOT_ID))
+        InventoryFile('123', 'foo', parent_id='TREE_ROOT')
         >>> i1 == i2
         True
         """
@@ -537,14 +948,13 @@
 
 
     def __ne__(self, other):
-        return not (self == other)
+        return not self.__eq__(other)
 
 
     def __hash__(self):
         raise ValueError('not hashable')
 
 
-
     def get_idpath(self, file_id):
         """Return a list of file_ids for the path to an entry.
 
@@ -565,8 +975,14 @@
 
 
     def id2path(self, file_id):
-        """Return as a list the path to file_id."""
-
+        """Return as a list the path to file_id.
+        
+        >>> i = Inventory()
+        >>> e = i.add(InventoryDirectory('src-id', 'src', ROOT_ID))
+        >>> e = i.add(InventoryFile('foo-id', 'foo.c', parent_id='src-id'))
+        >>> print i.id2path('foo-id').replace(os.sep, '/')
+        src/foo.c
+        """
         # get all names, skipping root
         p = [self._byid[fid].name for fid in self.get_idpath(file_id)[1:]]
         return os.sep.join(p)
@@ -643,7 +1059,11 @@
 
 
 
-_NAME_RE = re.compile(r'^[^/\\]+$')
+_NAME_RE = None
 
 def is_valid_name(name):
+    global _NAME_RE
+    if _NAME_RE == None:
+        _NAME_RE = re.compile(r'^[^/\\]+$')
+        
     return bool(_NAME_RE.match(name))

=== modified file 'bzrlib/lock.py'
--- bzrlib/lock.py	2005-07-21 12:01:14 +0000
+++ bzrlib/lock.py	2005-09-23 12:49:00 +0000
@@ -84,7 +84,7 @@
         f = None
 
         def unlock(self):
-            fcntl.flock(self.f, fcntl.LOCK_UN)
+            fcntl.lockf(self.f, fcntl.LOCK_UN)
             self.f.close()
             del self.f 
 
@@ -92,7 +92,7 @@
     class _fcntl_WriteLock(_fcntl_FileLock):
         def __init__(self, filename):
             try:
-                fcntl.flock(self._open(filename, 'wb'), fcntl.LOCK_EX)
+                fcntl.lockf(self._open(filename, 'wb'), fcntl.LOCK_EX)
             except Exception, e:
                 raise LockError(e)
 
@@ -100,7 +100,7 @@
     class _fcntl_ReadLock(_fcntl_FileLock):
         def __init__(self, filename):
             try:
-                fcntl.flock(self._open(filename, 'rb'), fcntl.LOCK_SH)
+                fcntl.lockf(self._open(filename, 'rb'), fcntl.LOCK_SH)
             except Exception, e:
                 raise LockError(e)
 
@@ -168,6 +168,8 @@
                 LOCK_NB = 4
                 def unlock(self):
                     _msvc_unlock(self.f)
+                    self.f.close()
+                    self.f = None
 
 
             class _msvc_ReadLock(_msvc_FileLock):

=== modified file 'bzrlib/log.py'
--- bzrlib/log.py	2005-07-16 00:07:40 +0000
+++ bzrlib/log.py	2005-10-07 04:25:33 +0000
@@ -28,15 +28,32 @@
 
 * with file-ids and revision-ids shown
 
-* from last to first or (not anymore) from first to last;
-  the default is "reversed" because it shows the likely most
-  relevant and interesting information first
-
-* (not yet) in XML format
+Logs are actually written out through an abstract LogFormatter
+interface, which allows for different preferred formats.  Plugins can
+register formats too.
+
+Logs can be produced in either forward (oldest->newest) or reverse
+(newest->oldest) order.
+
+Logs can be filtered to show only revisions matching a particular
+search string, or within a particular range of revisions.  The range
+can be given as date/times, which are reduced to revisions before
+calling in here.
+
+In verbose mode we show a summary of what changed in each particular
+revision.  Note that this is the delta for changes in that revision
+relative to its mainline parent, not the delta relative to the last
+logged revision.  So for example if you ask for a verbose log of
+changes touching hello.c you will get a list of those revisions also
+listing other things that were changed in the same revision, but not
+all the changes since the previous revision that touched hello.c.
 """
 
 
-from trace import mutter
+from bzrlib.tree import EmptyTree
+from bzrlib.delta import compare_trees
+from bzrlib.trace import mutter
+
 
 def find_touching_revisions(branch, file_id):
     """Yield a description of revisions which affect the file_id.
@@ -83,6 +100,26 @@
 
 
 
+def _enumerate_history(branch):
+    rh = []
+    revno = 1
+    for rev_id in branch.revision_history():
+        rh.append((revno, rev_id))
+        revno += 1
+    return rh
+
+
+def _get_revision_delta(branch, revno):
+    """Return the delta for a mainline revision.
+    
+    This is used to show summaries in verbose logs, and also for finding 
+    revisions which touch a given file."""
+    # XXX: What are we supposed to do when showing a summary for something 
+    # other than a mainline revision.  The delta to it's first parent, or
+    # (more useful) the delta to a nominated other revision.
+    return branch.get_revision_delta(revno)
+
+
 def show_log(branch,
              lf,
              specific_fileid=None,
@@ -113,6 +150,22 @@
     end_revision
         If not None, only show revisions <= end_revision
     """
+    branch.lock_read()
+    try:
+        _show_log(branch, lf, specific_fileid, verbose, direction,
+                  start_revision, end_revision, search)
+    finally:
+        branch.unlock()
+    
+def _show_log(branch,
+             lf,
+             specific_fileid=None,
+             verbose=False,
+             direction='reverse',
+             start_revision=None,
+             end_revision=None,
+             search=None):
+    """Worker function for show_log - see show_log."""
     from bzrlib.osutils import format_date
     from bzrlib.errors import BzrCheckError
     from bzrlib.textui import show_status
@@ -131,20 +184,32 @@
     else:
         searchRE = None
 
-    which_revs = branch.enum_history(direction)
-    which_revs = [x for x in which_revs if (
-            (start_revision is None or x[0] >= start_revision)
-            and (end_revision is None or x[0] <= end_revision))]
-
-    if not (verbose or specific_fileid):
-        # no need to know what changed between revisions
-        with_deltas = deltas_for_log_dummy(branch, which_revs)
-    elif direction == 'reverse':
-        with_deltas = deltas_for_log_reverse(branch, which_revs)
-    else:        
-        with_deltas = deltas_for_log_forward(branch, which_revs)
-
-    for revno, rev, delta in with_deltas:
+    which_revs = _enumerate_history(branch)
+    
+    if start_revision is None:
+        start_revision = 1
+    else:
+        branch.check_real_revno(start_revision)
+    
+    if end_revision is None:
+        end_revision = len(which_revs)
+    else:
+        branch.check_real_revno(end_revision)
+
+    # list indexes are 0-based; revisions are 1-based
+    cut_revs = which_revs[(start_revision-1):(end_revision)]
+
+    if direction == 'reverse':
+        cut_revs.reverse()
+    elif direction == 'forward':
+        pass
+    else:
+        raise ValueError('invalid direction %r' % direction)
+
+    for revno, rev_id in cut_revs:
+        if verbose or specific_fileid:
+            delta = _get_revision_delta(branch, revno)
+            
         if specific_fileid:
             if not delta.touches_file_id(specific_fileid):
                 continue
@@ -153,29 +218,42 @@
             # although we calculated it, throw it away without display
             delta = None
 
-        if searchRE is None or searchRE.search(rev.message):
-            lf.show(revno, rev, delta)
+        rev = branch.get_revision(rev_id)
+
+        if searchRE:
+            if not searchRE.search(rev.message):
+                continue
+
+        lf.show(revno, rev, delta)
 
 
 
 def deltas_for_log_dummy(branch, which_revs):
+    """Return all the revisions without intermediate deltas.
+
+    Useful for log commands that won't need the delta information.
+    """
+    
     for revno, revision_id in which_revs:
         yield revno, branch.get_revision(revision_id), None
 
 
 def deltas_for_log_reverse(branch, which_revs):
-    """Compute deltas for display in reverse log.
-
-    Given a sequence of (revno, revision_id) pairs, return
-    (revno, rev, delta).
+    """Compute deltas for display in latest-to-earliest order.
+
+    branch
+        Branch to traverse
+
+    which_revs
+        Sequence of (revno, revision_id) for the subset of history to examine
+
+    returns 
+        Sequence of (revno, rev, delta)
 
     The delta is from the given revision to the next one in the
     sequence, which makes sense if the log is being displayed from
     newest to oldest.
     """
-    from tree import EmptyTree
-    from diff import compare_trees
-    
     last_revno = last_revision_id = last_tree = None
     for revno, revision_id in which_revs:
         this_tree = branch.revision_tree(revision_id)
@@ -210,9 +288,6 @@
     sequence, which makes sense if the log is being displayed from
     newest to oldest.
     """
-    from tree import EmptyTree
-    from diff import compare_trees
-
     last_revno = last_revision_id = last_tree = None
     prev_tree = EmptyTree(branch.get_root_id())
 
@@ -237,16 +312,16 @@
 
 class LogFormatter(object):
     """Abstract class to display log messages."""
-    def __init__(self, to_file, show_ids=False, show_timezone=False):
+    def __init__(self, to_file, show_ids=False, show_timezone='original'):
         self.to_file = to_file
         self.show_ids = show_ids
         self.show_timezone = show_timezone
-        
-
-
-
-
-
+
+
+    def show(self, revno, rev, delta):
+        raise NotImplementedError('not implemented in abstract base')
+
+    
 class LongLogFormatter(LogFormatter):
     def show(self, revno, rev, delta):
         from osutils import format_date
@@ -257,9 +332,16 @@
         print >>to_file,  'revno:', revno
         if self.show_ids:
             print >>to_file,  'revision-id:', rev.revision_id
+
+            for parent_id in rev.parent_ids:
+                print >>to_file, 'parent:', parent_id
+            
         print >>to_file,  'committer:', rev.committer
-        print >>to_file,  'timestamp: %s' % (format_date(rev.timestamp, rev.timezone or 0,
-                                             self.show_timezone))
+
+        date_str = format_date(rev.timestamp,
+                               rev.timezone or 0,
+                               self.show_timezone)
+        print >>to_file,  'timestamp: %s' % date_str
 
         print >>to_file,  'message:'
         if not rev.message:
@@ -290,6 +372,8 @@
             for l in rev.message.split('\n'):
                 print >>to_file,  '      ' + l
 
+        # TODO: Why not show the modified files in a shorter form as
+        # well? rewrap them single lines of appropriate length
         if delta != None:
             delta.show(to_file, self.show_ids)
         print
@@ -302,9 +386,18 @@
 
 
 def log_formatter(name, *args, **kwargs):
+    """Construct a formatter from arguments.
+
+    name -- Name of the formatter to construct; currently 'long' and
+        'short' are supported.
+    """
     from bzrlib.errors import BzrCommandError
-    
     try:
         return FORMATTERS[name](*args, **kwargs)
     except IndexError:
         raise BzrCommandError("unknown log formatter: %r" % name)
+
+def show_one_log(revno, rev, delta, verbose, to_file, show_timezone):
+    # deprecated; for compatability
+    lf = LongLogFormatter(to_file=to_file, show_timezone=show_timezone)
+    lf.show(revno, rev, delta)

=== modified file 'bzrlib/mdiff.py'
--- bzrlib/mdiff.py	2005-04-09 06:21:44 +0000
+++ bzrlib/mdiff.py	2005-09-26 08:56:15 +0000
@@ -25,12 +25,59 @@
 
 # TODO: maybe work on files not strings?
 
+# FIXME: doesn't work properly on files without trailing newlines
 
-import difflib, sys, struct
+import difflib
+import struct
+import sys
+import unittest
 from cStringIO import StringIO
 
+
+def linesplit(a):
+    """Split into two lists: content and line positions.
+
+    This returns (al, ap).
+
+    al[i] is the string content of line i of the file, including its
+    newline (if any).
+
+    ap[i] is the byte position in the file where that line starts.
+
+    ap[-1] is the byte position of the end of the file (i.e. the
+    length of the file.)
+
+    This transformation allows us to do a line-based diff and then map
+    back to byte positions.
+    """
+
+    al, ap = [], []
+    last = 0
+
+    n = a.find("\n") + 1
+    while n > 0:
+        ap.append(last)
+        al.append(a[last:n])
+        last = n
+        n = a.find("\n", n) + 1
+
+    if last < len(a):
+        al.append(a[last:])
+        ap.append(last)
+
+    # position at the end
+    ap.append(len(a))
+
+    return (al, ap)
+
+
 def diff(a, b):
-    d = difflib.SequenceMatcher(None, a, b)
+    # TODO: Use different splits, perhaps rsync-like, for binary files?
+    
+    (al, ap) = linesplit(a)
+    (bl, bp) = linesplit(b)
+
+    d = difflib.SequenceMatcher(None, al, bl)
     
     ## sys.stderr.write('  ~ real_quick_ratio: %.4f\n' % d.real_quick_ratio())
     
@@ -38,9 +85,9 @@
         if o == 'equal': continue
         # a[m:n] should be replaced by b[s:t]
         if s == t:
-            yield m, n, ''
+            yield ap[m], ap[n], ''
         else:
-            yield m, n, b[s:t]
+            yield ap[m], ap[n], ''.join(bl[s:t])
 
 
 def tobinary(ops):
@@ -94,3 +141,26 @@
 
 
 
+class TestDiffPatch(unittest.TestCase):
+    def doDiffPatch(self, old, new):
+        diff = bdiff(old, new)
+        result = bpatch(old, diff)
+        self.assertEquals(new, result)
+
+
+    def testSimpleDiff(self):
+        """Simply add a line at the end"""
+        self.doDiffPatch('a\nb\n', 'a\nb\nc\n')
+        
+
+        
+    def testTrailingLine(self):
+        """Test diff that adds an unterminated line.
+
+        (Old versions didn't do this properly.)"""
+        self.doDiffPatch('a\nb\nc\n',
+                         'a\nb\nc\nd')
+
+
+if __name__ == '__main__':
+    unittest.main()

=== modified file 'bzrlib/merge.py'
--- bzrlib/merge.py	2005-07-07 08:02:16 +0000
+++ bzrlib/merge.py	2005-10-08 00:02:33 +0000
@@ -1,29 +1,65 @@
-from merge_core import merge_flex
-from changeset import generate_changeset, ExceptionConflictHandler
-from changeset import Inventory
-from bzrlib import find_branch
-import bzrlib.osutils
-from bzrlib.errors import BzrCommandError
-from bzrlib.diff import compare_trees
-from trace import mutter, warning
-import os.path
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
 import tempfile
 import shutil
 import errno
 
-class UnrelatedBranches(BzrCommandError):
-    def __init__(self):
-        msg = "Branches have no common ancestor, and no base revision"\
-            " specified."
-        BzrCommandError.__init__(self, msg)
-
+import bzrlib.osutils
+import bzrlib.revision
+from bzrlib.merge_core import merge_flex, ApplyMerge3, BackupBeforeChange
+from bzrlib.changeset import generate_changeset, ExceptionConflictHandler
+from bzrlib.changeset import Inventory, Diff3Merge, ReplaceContents
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrCommandError, UnrelatedBranches, NoCommonAncestor
+from bzrlib.errors import NoCommits
+from bzrlib.delta import compare_trees
+from bzrlib.trace import mutter, warning
+from bzrlib.fetch import greedy_fetch, fetch
+from bzrlib.revision import is_ancestor
+from bzrlib.osutils import rename
+from bzrlib.revision import common_ancestor, MultipleRevisionSources
+from bzrlib.errors import NoSuchRevision
+
+# TODO: build_working_dir can be built on something simpler than merge()
+
+# FIXME: merge() parameters seem oriented towards the command line
+
+# comments from abentley on irc: merge happens in two stages, each
+# of which generates a changeset object
+
+# stage 1: generate OLD->OTHER,
+# stage 2: use MINE and OLD->OTHER to generate MINE -> RESULT
 
 class MergeConflictHandler(ExceptionConflictHandler):
-    """Handle conflicts encountered while merging"""
-    def __init__(self, dir, ignore_zero=False):
-        ExceptionConflictHandler.__init__(self, dir)
+    """Handle conflicts encountered while merging.
+
+    This subclasses ExceptionConflictHandler, so that any types of
+    conflict that are not explicitly handled cause an exception and
+    terminate the merge.
+    """
+    def __init__(self, this_tree, base_tree, other_tree, ignore_zero=False):
+        ExceptionConflictHandler.__init__(self)
         self.conflicts = 0
         self.ignore_zero = ignore_zero
+        self.this_tree = this_tree
+        self.base_tree = base_tree
+        self.other_tree = other_tree
 
     def copy(self, source, dest):
         """Copy the text and mode of a file
@@ -36,6 +72,15 @@
             d_file.write(line)
         os.chmod(dest, 0777 & os.stat(source).st_mode)
 
+    def dump(self, lines, dest):
+        """Copy the text and mode of a file
+        :param source: The path of the file to copy
+        :param dest: The distination file to create
+        """
+        d_file = file(dest, "wb")
+        for line in lines:
+            d_file.write(line)
+
     def add_suffix(self, name, suffix, last_new_name=None):
         """Rename a file to append a suffix.  If the new name exists, the
         suffix is added repeatedly until a non-existant name is found
@@ -48,7 +93,7 @@
             last_new_name = name
         new_name = last_new_name+suffix
         try:
-            os.rename(name, new_name)
+            rename(name, new_name)
             return new_name
         except OSError, e:
             if e.errno != errno.EEXIST and e.errno != errno.ENOTEMPTY:
@@ -60,7 +105,7 @@
         self.conflicts += 1
         
 
-    def merge_conflict(self, new_file, this_path, base_path, other_path):
+    def merge_conflict(self, new_file, this_path, base_lines, other_lines):
         """
         Handle diff3 conflicts by producing a .THIS, .BASE and .OTHER.  The
         main file will be a version with diff3 conflicts.
@@ -70,11 +115,17 @@
         :param other_path: Path to the file text for the OTHER tree
         """
         self.add_suffix(this_path, ".THIS")
-        self.copy(base_path, this_path+".BASE")
-        self.copy(other_path, this_path+".OTHER")
-        os.rename(new_file, this_path)
+        self.dump(base_lines, this_path+".BASE")
+        self.dump(other_lines, this_path+".OTHER")
+        rename(new_file, this_path)
         self.conflict("Diff3 conflict encountered in %s" % this_path)
 
+    def new_contents_conflict(self, filename, other_contents):
+        """Conflicting contents for newly added file."""
+        self.copy(other_contents, filename + ".OTHER")
+        self.conflict("Conflict in newly added file %s" % filename)
+    
+
     def target_exists(self, entry, target, old_path):
         """Handle the case when the target file or dir exists"""
         moved_path = self.add_suffix(target, ".moved")
@@ -86,51 +137,90 @@
             % filename)
         return "skip"
 
+    def rem_contents_conflict(self, filename, this_contents, base_contents):
+        base_contents(filename+".BASE", self, False)
+        this_contents(filename+".THIS", self, False)
+        return ReplaceContents(this_contents, None)
+
+    def rem_contents_conflict(self, filename, this_contents, base_contents):
+        base_contents(filename+".BASE", self, False)
+        this_contents(filename+".THIS", self, False)
+        self.conflict("Other branch deleted locally modified file %s" %
+                      filename)
+        return ReplaceContents(this_contents, None)
+
+    def abs_this_path(self, file_id):
+        """Return the absolute path for a file_id in the this tree."""
+        relpath = self.this_tree.id2path(file_id)
+        return self.this_tree.tree.abspath(relpath)
+
+    def add_missing_parents(self, file_id, tree):
+        """If some of the parents for file_id are missing, add them."""
+        entry = tree.tree.inventory[file_id]
+        if entry.parent_id not in self.this_tree:
+            return self.create_all_missing(entry.parent_id, tree)
+        else:
+            return self.abs_this_path(entry.parent_id)
+
+    def create_all_missing(self, file_id, tree):
+        """Add contents for a file_id and all its parents to a tree."""
+        entry = tree.tree.inventory[file_id]
+        if entry.parent_id is not None and entry.parent_id not in self.this_tree:
+            abspath = self.create_all_missing(entry.parent_id, tree)
+        else:
+            abspath = self.abs_this_path(entry.parent_id)
+        entry_path = os.path.join(abspath, entry.name)
+        if not os.path.isdir(entry_path):
+            self.create(file_id, entry_path, tree)
+        return entry_path
+
+    def create(self, file_id, path, tree, reverse=False):
+        """Uses tree data to create a filesystem object for the file_id"""
+        from merge_core import get_id_contents
+        get_id_contents(file_id, tree)(path, self, reverse)
+
+    def missing_for_merge(self, file_id, other_path):
+        """The file_id doesn't exist in THIS, but does in OTHER and BASE"""
+        self.conflict("Other branch modified locally deleted file %s" %
+                      other_path)
+        parent_dir = self.add_missing_parents(file_id, self.other_tree)
+        stem = os.path.join(parent_dir, os.path.basename(other_path))
+        self.create(file_id, stem+".OTHER", self.other_tree)
+        self.create(file_id, stem+".BASE", self.base_tree)
+
     def finalize(self):
         if not self.ignore_zero:
             print "%d conflicts encountered.\n" % self.conflicts
             
-class SourceFile(object):
-    def __init__(self, path, id, present=None, isdir=None):
-        self.path = path
-        self.id = id
-        self.present = present
-        self.isdir = isdir
-        self.interesting = True
-
-    def __repr__(self):
-        return "SourceFile(%s, %s)" % (self.path, self.id)
-
-def get_tree(treespec, temp_root, label):
+def get_tree(treespec, temp_root, label, local_branch=None):
     location, revno = treespec
-    branch = find_branch(location)
+    branch = Branch.open_containing(location)
     if revno is None:
+        revision = None
+    elif revno == -1:
+        revision = branch.last_revision()
+    else:
+        revision = branch.get_rev_id(revno)
+    return branch, get_revid_tree(branch, revision, temp_root, label,
+                                  local_branch)
+
+def get_revid_tree(branch, revision, temp_root, label, local_branch):
+    if revision is None:
         base_tree = branch.working_tree()
-    elif revno == -1:
-        base_tree = branch.basis_tree()
     else:
-        base_tree = branch.revision_tree(branch.lookup_revision(revno))
+        if local_branch is not None:
+            greedy_fetch(local_branch, branch, revision)
+            base_tree = local_branch.revision_tree(revision)
+        else:
+            base_tree = branch.revision_tree(revision)
     temp_path = os.path.join(temp_root, label)
     os.mkdir(temp_path)
-    return branch, MergeTree(base_tree, temp_path)
-
-
-def abspath(tree, file_id):
-    path = tree.inventory.id2path(file_id)
-    if path == "":
-        return "./."
-    return "./" + path
+    return MergeTree(base_tree, temp_path)
+
 
 def file_exists(tree, file_id):
     return tree.has_filename(tree.id2path(file_id))
     
-def inventory_map(tree):
-    inventory = {}
-    for file_id in tree.inventory:
-        path = abspath(tree, file_id)
-        inventory[path] = SourceFile(path, file_id)
-    return inventory
-
 
 class MergeTree(object):
     def __init__(self, tree, tempdir):
@@ -139,108 +229,218 @@
             self.root = tree.basedir
         else:
             self.root = None
-        self.inventory = inventory_map(tree)
         self.tree = tree
         self.tempdir = tempdir
         os.mkdir(os.path.join(self.tempdir, "texts"))
+        os.mkdir(os.path.join(self.tempdir, "symlinks"))
         self.cached = {}
 
+    def __iter__(self):
+        return self.tree.__iter__()
+
+    def __contains__(self, file_id):
+        return file_id in self.tree
+
+    def get_file(self, file_id):
+        return self.tree.get_file(file_id)
+
+    def get_file_sha1(self, id):
+        return self.tree.get_file_sha1(id)
+
+    def is_executable(self, id):
+        return self.tree.is_executable(id)
+
+    def id2path(self, file_id):
+        return self.tree.id2path(file_id)
+
+    def has_id(self, file_id):
+        return self.tree.has_id(file_id)
+
+    def has_or_had_id(self, file_id):
+        if file_id == self.tree.inventory.root.file_id:
+            return True
+        return self.tree.inventory.has_id(file_id)
+
+    def has_or_had_id(self, file_id):
+        if file_id == self.tree.inventory.root.file_id:
+            return True
+        return self.tree.inventory.has_id(file_id)
+
     def readonly_path(self, id):
         if id not in self.tree:
             return None
         if self.root is not None:
             return self.tree.abspath(self.tree.id2path(id))
         else:
-            if self.tree.inventory[id].kind in ("directory", "root_directory"):
+            kind = self.tree.inventory[id].kind
+            if kind in ("directory", "root_directory"):
                 return self.tempdir
             if not self.cached.has_key(id):
-                path = os.path.join(self.tempdir, "texts", id)
-                outfile = file(path, "wb")
-                outfile.write(self.tree.get_file(id).read())
-                assert(os.path.exists(path))
+                if kind == "file":
+                    path = os.path.join(self.tempdir, "texts", id)
+                    outfile = file(path, "wb")
+                    outfile.write(self.tree.get_file(id).read())
+                    assert(bzrlib.osutils.lexists(path))
+                    if self.tree.is_executable(id):
+                        os.chmod(path, 0755)
+                else:
+                    assert kind == "symlink"
+                    path = os.path.join(self.tempdir, "symlinks", id)
+                    target = self.tree.get_symlink_target(id)
+                    os.symlink(target, path)
                 self.cached[id] = path
             return self.cached[id]
 
 
+def build_working_dir(to_dir):
+    """Build a working directory in an empty directory.
+
+    to_dir is a directory containing branch metadata but no working files,
+    typically constructed by cloning an existing branch. 
+
+    This is split out as a special idiomatic case of merge.  It could
+    eventually be done by just building the tree directly calling into 
+    lower-level code (e.g. constructing a changeset).
+    """
+    merge((to_dir, -1), (to_dir, 0), this_dir=to_dir,
+          check_clean=False, ignore_zero=True)
+
 
 def merge(other_revision, base_revision,
           check_clean=True, ignore_zero=False,
-          this_dir=None):
+          this_dir=None, backup_files=False, merge_type=ApplyMerge3,
+          file_list=None):
     """Merge changes into a tree.
 
     base_revision
-        Base for three-way merge.
+        tuple(path, revision) Base for three-way merge.
     other_revision
-        Other revision for three-way merge.
+        tuple(path, revision) Other revision for three-way merge.
     this_dir
         Directory to merge changes into; '.' by default.
     check_clean
         If true, this_dir must have no uncommitted changes before the
         merge begins.
+    ignore_zero - If true, suppress the "zero conflicts" message when 
+        there are no conflicts; should be set when doing something we expect
+        to complete perfectly.
+
+    All available ancestors of other_revision and base_revision are
+    automatically pulled into the branch.
     """
     tempdir = tempfile.mkdtemp(prefix="bzr-")
     try:
         if this_dir is None:
             this_dir = '.'
-        this_branch = find_branch(this_dir)
+        this_branch = Branch.open_containing(this_dir)
+        this_rev_id = this_branch.last_revision()
+        if this_rev_id is None:
+            raise BzrCommandError("This branch has no commits")
         if check_clean:
             changes = compare_trees(this_branch.working_tree(), 
                                     this_branch.basis_tree(), False)
             if changes.has_changed():
                 raise BzrCommandError("Working tree has uncommitted changes.")
-        other_branch, other_tree = get_tree(other_revision, tempdir, "other")
+        other_branch, other_tree = get_tree(other_revision, tempdir, "other",
+                                            this_branch)
+        if other_revision[1] == -1:
+            other_rev_id = other_branch.last_revision()
+            if other_rev_id is None:
+                raise NoCommits(other_branch)
+            other_basis = other_rev_id
+        elif other_revision[1] is not None:
+            other_rev_id = other_branch.get_rev_id(other_revision[1])
+            other_basis = other_rev_id
+        else:
+            other_rev_id = None
+            other_basis = other_branch.last_revision()
+            if other_basis is None:
+                raise NoCommits(other_branch)
         if base_revision == [None, None]:
-            if other_revision[1] == -1:
-                o_revno = None
-            else:
-                o_revno = other_revision[1]
-            base_revno = this_branch.common_ancestor(other_branch, 
-                                                     other_revno=o_revno)[0]
-            if base_revno is None:
+            try:
+                base_rev_id = common_ancestor(this_rev_id, other_basis, 
+                                              this_branch)
+            except NoCommonAncestor:
                 raise UnrelatedBranches()
-            base_revision = ['.', base_revno]
-        base_branch, base_tree = get_tree(base_revision, tempdir, "base")
+            base_tree = get_revid_tree(this_branch, base_rev_id, tempdir, 
+                                       "base", None)
+            base_is_ancestor = True
+        else:
+            base_branch, base_tree = get_tree(base_revision, tempdir, "base")
+            if base_revision[1] == -1:
+                base_rev_id = base_branch.last_revision()
+            elif base_revision[1] is None:
+                base_rev_id = None
+            else:
+                base_rev_id = base_branch.get_rev_id(base_revision[1])
+            fetch(from_branch=base_branch, to_branch=this_branch)
+            base_is_ancestor = is_ancestor(this_rev_id, base_rev_id,
+                                           this_branch)
+        if file_list is None:
+            interesting_ids = None
+        else:
+            interesting_ids = set()
+            this_tree = this_branch.working_tree()
+            for fname in file_list:
+                path = this_branch.relpath(fname)
+                found_id = False
+                for tree in (this_tree, base_tree.tree, other_tree.tree):
+                    file_id = tree.inventory.path2id(path)
+                    if file_id is not None:
+                        interesting_ids.add(file_id)
+                        found_id = True
+                if not found_id:
+                    raise BzrCommandError("%s is not a source file in any"
+                                          " tree." % fname)
         merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                    ignore_zero=ignore_zero)
+                    ignore_zero=ignore_zero, backup_files=backup_files, 
+                    merge_type=merge_type, interesting_ids=interesting_ids)
+        if base_is_ancestor and other_rev_id is not None\
+            and other_rev_id not in this_branch.revision_history():
+            this_branch.add_pending_merge(other_rev_id)
     finally:
         shutil.rmtree(tempdir)
 
 
-def generate_cset_optimized(tree_a, tree_b, inventory_a, inventory_b):
-    """Generate a changeset, using the text_id to mark really-changed files.
-    This permits blazing comparisons when text_ids are present.  It also
-    disables metadata comparison for files with identical texts.
+def set_interesting(inventory_a, inventory_b, interesting_ids):
+    """Mark files whose ids are in interesting_ids as interesting
+    """
+    for inventory in (inventory_a, inventory_b):
+        for path, source_file in inventory.iteritems():
+             source_file.interesting = source_file.id in interesting_ids
+
+
+def generate_cset_optimized(tree_a, tree_b, interesting_ids=None):
+    """Generate a changeset.  If interesting_ids is supplied, only changes
+    to those files will be shown.  Metadata changes are stripped.
     """ 
-    for file_id in tree_a.tree.inventory:
-        if file_id not in tree_b.tree.inventory:
-            continue
-        entry_a = tree_a.tree.inventory[file_id]
-        entry_b = tree_b.tree.inventory[file_id]
-        if (entry_a.kind, entry_b.kind) != ("file", "file"):
-            continue
-        if None in (entry_a.text_id, entry_b.text_id):
-            continue
-        if entry_a.text_id != entry_b.text_id:
-            continue
-        inventory_a[abspath(tree_a.tree, file_id)].interesting = False
-        inventory_b[abspath(tree_b.tree, file_id)].interesting = False
-    cset =  generate_changeset(tree_a, tree_b, inventory_a, inventory_b)
+    cset =  generate_changeset(tree_a, tree_b, interesting_ids)
     for entry in cset.entries.itervalues():
         entry.metadata_change = None
     return cset
 
 
 def merge_inner(this_branch, other_tree, base_tree, tempdir, 
-                ignore_zero=False):
+                ignore_zero=False, merge_type=ApplyMerge3, backup_files=False,
+                interesting_ids=None):
+
+    def merge_factory(file_id, base, other):
+        contents_change = merge_type(file_id, base, other)
+        if backup_files:
+            contents_change = BackupBeforeChange(contents_change)
+        return contents_change
+
     this_tree = get_tree((this_branch.base, None), tempdir, "this")[1]
 
     def get_inventory(tree):
-        return tree.inventory
+        return tree.tree.inventory
 
     inv_changes = merge_flex(this_tree, base_tree, other_tree,
                              generate_cset_optimized, get_inventory,
-                             MergeConflictHandler(base_tree.root,
-                                                  ignore_zero=ignore_zero))
+                             MergeConflictHandler(this_tree, base_tree,
+                             other_tree, ignore_zero=ignore_zero),
+                             merge_factory=merge_factory, 
+                             interesting_ids=interesting_ids)
 
     adjust_ids = []
     for id, path in inv_changes.iteritems():
@@ -248,19 +448,36 @@
             if path == '.':
                 path = ''
             else:
-                assert path.startswith('./')
+                assert path.startswith('.' + os.sep), "path is %s" % path
             path = path[2:]
         adjust_ids.append((path, id))
-    this_branch.set_inventory(regen_inventory(this_branch, this_tree.root, adjust_ids))
+    if len(adjust_ids) > 0:
+        this_branch.set_inventory(regen_inventory(this_branch, this_tree.root,
+                                                  adjust_ids))
 
 
 def regen_inventory(this_branch, root, new_entries):
     old_entries = this_branch.read_working_inventory()
     new_inventory = {}
     by_path = {}
+    new_entries_map = {} 
+    for path, file_id in new_entries:
+        if path is None:
+            continue
+        new_entries_map[file_id] = path
+
+    def id2path(file_id):
+        path = new_entries_map.get(file_id)
+        if path is not None:
+            return path
+        entry = old_entries[file_id]
+        if entry.parent_id is None:
+            return entry.name
+        return os.path.join(id2path(entry.parent_id), entry.name)
+        
     for file_id in old_entries:
         entry = old_entries[file_id]
-        path = old_entries.id2path(file_id)
+        path = id2path(file_id)
         new_inventory[file_id] = (path, file_id, entry.parent_id, entry.kind)
         by_path[path] = file_id
     
@@ -294,3 +511,8 @@
     assert len(new_inventory_list) == len(old_entries) + insertions - deletions
     new_inventory_list.sort()
     return new_inventory_list
+
+merge_types = {     "merge3": (ApplyMerge3, "Native diff3-style merge"), 
+                     "diff3": (Diff3Merge,  "Merge using external diff3")
+              }
+

=== modified file 'bzrlib/merge3.py'
--- bzrlib/merge3.py	2005-07-06 04:09:02 +0000
+++ bzrlib/merge3.py	2005-09-04 02:59:56 +0000
@@ -76,9 +76,9 @@
     def merge_lines(self,
                     name_a=None,
                     name_b=None,
-                    start_marker='<<<<<<<<',
-                    mid_marker='========',
-                    end_marker='>>>>>>>>',
+                    start_marker='<<<<<<<',
+                    mid_marker='=======',
+                    end_marker='>>>>>>>',
                     show_base=False):
         """Return merge in cvs-like form.
         """

=== modified file 'bzrlib/merge_core.py'
--- bzrlib/merge_core.py	2005-07-16 00:07:40 +0000
+++ bzrlib/merge_core.py	2005-10-08 00:02:33 +0000
@@ -1,45 +1,112 @@
+import os.path
+
 import changeset
 from changeset import Inventory, apply_changeset, invert_dict
-import os.path
-
-class ThreewayInventory(object):
-    def __init__(self, this_inventory, base_inventory, other_inventory):
-        self.this = this_inventory
-        self.base = base_inventory
-        self.other = other_inventory
+from bzrlib.osutils import backup_file, rename
+from bzrlib.merge3 import Merge3
+import bzrlib
+
+class ApplyMerge3:
+    """Contents-change wrapper around merge3.Merge3"""
+    def __init__(self, file_id, base, other):
+        self.file_id = file_id
+        self.base = base
+        self.other = other
+ 
+    def __eq__(self, other):
+        if not isinstance(other, ApplyMerge3):
+            return False
+        return (self.base == other.base and 
+                self.other == other.other and self.file_id == other.file_id)
+
+    def __ne__(self, other):
+        return not (self == other)
+
+
+    def apply(self, filename, conflict_handler, reverse=False):
+        new_file = filename+".new" 
+        if not reverse:
+            base = self.base
+            other = self.other
+        else:
+            base = self.other
+            other = self.base
+        def get_lines(tree):
+            if self.file_id not in tree:
+                raise Exception("%s not in tree" % self.file_id)
+                return ()
+            return tree.get_file(self.file_id).readlines()
+        base_lines = get_lines(base)
+        other_lines = get_lines(other)
+        m3 = Merge3(base_lines, file(filename, "rb").readlines(), other_lines)
+
+        new_conflicts = False
+        output_file = file(new_file, "wb")
+        start_marker = "!START OF MERGE CONFLICT!" + "I HOPE THIS IS UNIQUE"
+        for line in m3.merge_lines(name_a = "TREE", name_b = "MERGE-SOURCE", 
+                       start_marker=start_marker):
+            if line.startswith(start_marker):
+                new_conflicts = True
+                output_file.write(line.replace(start_marker, '<' * 7))
+            else:
+                output_file.write(line)
+        output_file.close()
+        if not new_conflicts:
+            os.chmod(new_file, os.stat(filename).st_mode)
+            rename(new_file, filename)
+            return
+        else:
+            conflict_handler.merge_conflict(new_file, filename, base_lines,
+                                            other_lines)
+
+
+class BackupBeforeChange:
+    """Contents-change wrapper to back up file first"""
+    def __init__(self, contents_change):
+        self.contents_change = contents_change
+ 
+    def __eq__(self, other):
+        if not isinstance(other, BackupBeforeChange):
+            return False
+        return (self.contents_change == other.contents_change)
+
+    def __ne__(self, other):
+        return not (self == other)
+
+    def apply(self, filename, conflict_handler, reverse=False):
+        backup_file(filename)
+        self.contents_change.apply(filename, conflict_handler, reverse)
+
+
 def invert_invent(inventory):
     invert_invent = {}
-    for key, value in inventory.iteritems():
-        invert_invent[value.id] = key
+    for file_id in inventory:
+        path = inventory.id2path(file_id)
+        if path == '':
+            path = './.'
+        else:
+            path = './' + path
+        invert_invent[file_id] = path
     return invert_invent
 
-def make_inv(inventory):
-    return Inventory(invert_invent(inventory))
-        
 
 def merge_flex(this, base, other, changeset_function, inventory_function,
-               conflict_handler):
-    this_inventory = inventory_function(this)
-    base_inventory = inventory_function(base)
-    other_inventory = inventory_function(other)
-    inventory = ThreewayInventory(make_inv(this_inventory),
-                                  make_inv(base_inventory), 
-                                  make_inv(other_inventory))
-    cset = changeset_function(base, other, base_inventory, other_inventory)
-    new_cset = make_merge_changeset(cset, inventory, this, base, other, 
-                                    conflict_handler)
-    result = apply_changeset(new_cset, invert_invent(this_inventory),
+               conflict_handler, merge_factory, interesting_ids):
+    cset = changeset_function(base, other, interesting_ids)
+    new_cset = make_merge_changeset(cset, this, base, other, 
+                                    conflict_handler, merge_factory)
+    result = apply_changeset(new_cset, invert_invent(this.tree.inventory),
                              this.root, conflict_handler, False)
     conflict_handler.finalize()
     return result
 
     
 
-def make_merge_changeset(cset, inventory, this, base, other, 
-                         conflict_handler=None):
+def make_merge_changeset(cset, this, base, other, 
+                         conflict_handler, merge_factory):
     new_cset = changeset.Changeset()
     def get_this_contents(id):
-        path = os.path.join(this.root, inventory.this.get_path(id))
+        path = this.readonly_path(id)
         if os.path.isdir(path):
             return changeset.dir_create
         else:
@@ -49,100 +116,134 @@
         if entry.is_boring():
             new_cset.add_entry(entry)
         else:
-            new_entry = make_merged_entry(entry, inventory, conflict_handler)
+            new_entry = make_merged_entry(entry, this, base, other, 
+                                          conflict_handler)
             new_contents = make_merged_contents(entry, this, base, other, 
-                                                inventory, conflict_handler)
+                                                conflict_handler,
+                                                merge_factory)
             new_entry.contents_change = new_contents
             new_entry.metadata_change = make_merged_metadata(entry, base, other)
             new_cset.add_entry(new_entry)
 
     return new_cset
 
-def make_merged_entry(entry, inventory, conflict_handler):
+class ThreeWayConflict(Exception):
+    def __init__(self, this, base, other):
+        self.this = this
+        self.base = base
+        self.other = other
+        msg = "Conflict merging %s %s and %s" % (this, base, other)
+        Exception.__init__(self, msg)
+
+def threeway_select(this, base, other):
+    """Returns a value selected by the three-way algorithm.
+    Raises ThreewayConflict if the algorithm yields a conflict"""
+    if base == other:
+        return this
+    elif base == this:
+        return other
+    elif other == this:
+        return this
+    else:
+        raise ThreeWayConflict(this, base, other)
+
+
+def make_merged_entry(entry, this, base, other, conflict_handler):
     from bzrlib.trace import mutter
-    this_name = inventory.this.get_name(entry.id)
-    this_parent = inventory.this.get_parent(entry.id)
-    this_dir = inventory.this.get_dir(entry.id)
-    if this_dir is None:
-        this_dir = ""
-
-    base_name = inventory.base.get_name(entry.id)
-    base_parent = inventory.base.get_parent(entry.id)
-    base_dir = inventory.base.get_dir(entry.id)
-    if base_dir is None:
-        base_dir = ""
-    other_name = inventory.other.get_name(entry.id)
-    other_parent = inventory.other.get_parent(entry.id)
-    other_dir = inventory.base.get_dir(entry.id)
-    if other_dir is None:
-        other_dir = ""
+    def entry_data(file_id, tree):
+        assert hasattr(tree, "__contains__"), "%s" % tree
+        if not tree.has_or_had_id(file_id):
+            return (None, None, "")
+        entry = tree.tree.inventory[file_id]
+        my_dir = tree.id2path(entry.parent_id)
+        if my_dir is None:
+            my_dir = ""
+        return entry.name, entry.parent_id, my_dir 
+    this_name, this_parent, this_dir = entry_data(entry.id, this)
+    base_name, base_parent, base_dir = entry_data(entry.id, base)
+    other_name, other_parent, other_dir = entry_data(entry.id, other)
     mutter("Dirs: this, base, other %r %r %r" % (this_dir, base_dir, other_dir))
     mutter("Names: this, base, other %r %r %r" % (this_name, base_name, other_name))
-    if base_name == other_name:
-        old_name = this_name
-        new_name = this_name
-    else:
-        if this_name != base_name and this_name != other_name:
-            conflict_handler.rename_conflict(entry.id, this_name, base_name,
-                                             other_name)
-        else:
-            old_name = this_name
-            new_name = other_name
-
-    if base_parent == other_parent:
-        old_parent = this_parent
-        new_parent = this_parent
-        old_dir = this_dir
-        new_dir = this_dir
-    else:
-        if this_parent != base_parent and this_parent != other_parent:
-            conflict_handler.move_conflict(entry.id, inventory)
-        else:
-            old_parent = this_parent
-            old_dir = this_dir
-            new_parent = other_parent
-            new_dir = other_dir
-    if old_name is not None and old_parent is not None:
-        old_path = os.path.join(old_dir, old_name)
-    else:
-        old_path = None
+    old_name = this_name
+    try:
+        new_name = threeway_select(this_name, base_name, other_name)
+    except ThreeWayConflict:
+        new_name = conflict_handler.rename_conflict(entry.id, this_name, 
+                                                    base_name, other_name)
+
+    old_parent = this_parent
+    try:
+        new_parent = threeway_select(this_parent, base_parent, other_parent)
+    except ThreeWayConflict:
+        new_parent = conflict_handler.move_conflict(entry.id, this_dir,
+                                                    base_dir, other_dir)
+    def get_path(name, parent):
+        if name is not None:
+            if name == "":
+                assert parent is None
+                return './.'
+            parent_dir = {this_parent: this_dir, other_parent: other_dir, 
+                          base_parent: base_dir}
+            directory = parent_dir[parent]
+            return os.path.join(directory, name)
+        else:
+            assert parent is None
+            return None
+
+    old_path = get_path(old_name, old_parent)
+        
     new_entry = changeset.ChangesetEntry(entry.id, old_parent, old_path)
-    if new_name is not None and new_parent is not None:
-        new_entry.new_path = os.path.join(new_dir, new_name)
-    else:
-        new_entry.new_path = None
+    new_entry.new_path = get_path(new_name, new_parent)
     new_entry.new_parent = new_parent
     mutter(repr(new_entry))
     return new_entry
 
 
-def make_merged_contents(entry, this, base, other, inventory, conflict_handler):
+def get_contents(entry, tree):
+    return get_id_contents(entry.id, tree)
+
+def get_id_contents(file_id, tree):
+    """Get a contents change element suitable for use with ReplaceContents
+    """
+    tree_entry = tree.tree.inventory[file_id]
+    if tree_entry.kind == "file":
+        return changeset.FileCreate(tree.get_file(file_id).read())
+    elif tree_entry.kind == "symlink":
+        return changeset.SymlinkCreate(tree.get_symlink_target(file_id))
+    else:
+        assert tree_entry.kind in ("root_directory", "directory")
+        return changeset.dir_create
+
+
+def make_merged_contents(entry, this, base, other, conflict_handler,
+                         merge_factory):
     contents = entry.contents_change
     if contents is None:
         return None
     this_path = this.readonly_path(entry.id)
-    def make_diff3():
+    def make_merge():
         if this_path is None:
-            return conflict_handler.missing_for_merge(entry.id, inventory)
-        base_path = base.readonly_path(entry.id)
-        other_path = other.readonly_path(entry.id)    
-        return changeset.Diff3Merge(base_path, other_path)
+            return conflict_handler.missing_for_merge(entry.id, 
+                                                      other.id2path(entry.id))
+        return merge_factory(entry.id, base, other)
 
-    if isinstance(contents, changeset.PatchApply):
-        return make_diff3()
     if isinstance(contents, changeset.ReplaceContents):
         if contents.old_contents is None and contents.new_contents is None:
             return None
         if contents.new_contents is None:
-            if this_path is not None and os.path.exists(this_path):
+            this_contents = get_contents(entry, this)
+            if this_path is not None and bzrlib.osutils.lexists(this_path):
+                if this_contents != contents.old_contents:
+                    return conflict_handler.rem_contents_conflict(this_path, 
+                        this_contents, contents.old_contents)
                 return contents
             else:
                 return None
         elif contents.old_contents is None:
-            if this_path is None or not os.path.exists(this_path):
+            if this_path is None or not bzrlib.osutils.lexists(this_path):
                 return contents
             else:
-                this_contents = file(this_path, "rb").read()
+                this_contents = get_contents(entry, this)
                 if this_contents == contents.new_contents:
                     return None
                 else:
@@ -151,23 +252,24 @@
                                                            other_path)
         elif isinstance(contents.old_contents, changeset.FileCreate) and \
             isinstance(contents.new_contents, changeset.FileCreate):
-            return make_diff3()
+            return make_merge()
         else:
             raise Exception("Unhandled merge scenario")
 
 def make_merged_metadata(entry, base, other):
-    if entry.metadata_change is not None:
-        base_path = base.readonly_path(entry.id)
-        other_path = other.readonly_path(entry.id)    
-        return PermissionsMerge(base_path, other_path)
+    metadata = entry.metadata_change
+    if metadata is None:
+        return None
+    if isinstance(metadata, changeset.ChangeUnixPermissions):
+        if metadata.new_mode is None:
+            return None
+        elif metadata.old_mode is None:
+            return metadata
+        else:
+            base_path = base.readonly_path(entry.id)
+            other_path = other.readonly_path(entry.id)    
+            return PermissionsMerge(base_path, other_path)
     
-def get_merge_entry(entry, inventory, base, other, conflict_handler):
-    if entry.contents_change is not None:
-        new_entry.contents_change = changeset.Diff3Merge(base_path, other_path)
-    if entry.metadata_change is not None:
-        new_entry.metadata_change = PermissionsMerge(base_path, other_path)
-
-    return new_entry
 
 class PermissionsMerge(object):
     def __init__(self, base_path, other_path):
@@ -192,390 +294,3 @@
             os.chmod(filename, other_stat)
         else:
             conflict_handler.permission_conflict(filename, base, other)
-
-
-import unittest
-import tempfile
-import shutil
-class MergeTree(object):
-    def __init__(self, dir):
-        self.dir = dir;
-        os.mkdir(dir)
-        self.inventory = {'0': ""}
-    
-    def child_path(self, parent, name):
-        return os.path.join(self.inventory[parent], name)
-
-    def add_file(self, id, parent, name, contents, mode):
-        path = self.child_path(parent, name)
-        full_path = self.abs_path(path)
-        assert not os.path.exists(full_path)
-        file(full_path, "wb").write(contents)
-        os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
-
-    def remove_file(self, id):
-        os.unlink(self.full_path(id))
-        del self.inventory[id]
-
-    def add_dir(self, id, parent, name, mode):
-        path = self.child_path(parent, name)
-        full_path = self.abs_path(path)
-        assert not os.path.exists(full_path)
-        os.mkdir(self.abs_path(path))
-        os.chmod(self.abs_path(path), mode)
-        self.inventory[id] = path
-
-    def abs_path(self, path):
-        return os.path.join(self.dir, path)
-
-    def full_path(self, id):
-        try:
-            tree_path = self.inventory[id]
-        except KeyError:
-            return None
-        return self.abs_path(tree_path)
-
-    def readonly_path(self, id):
-        return self.full_path(id)
-
-    def change_path(self, id, path):
-        new = os.path.join(self.dir, self.inventory[id])
-        os.rename(self.abs_path(self.inventory[id]), self.abs_path(path))
-        self.inventory[id] = path
-
-class MergeBuilder(object):
-    def __init__(self):
-        self.dir = tempfile.mkdtemp(prefix="BaZing")
-        self.base = MergeTree(os.path.join(self.dir, "base"))
-        self.this = MergeTree(os.path.join(self.dir, "this"))
-        self.other = MergeTree(os.path.join(self.dir, "other"))
-        
-        self.cset = changeset.Changeset()
-        self.cset.add_entry(changeset.ChangesetEntry("0", 
-                                                     changeset.NULL_ID, "./."))
-    def get_cset_path(self, parent, name):
-        if name is None:
-            assert (parent is None)
-            return None
-        return os.path.join(self.cset.entries[parent].path, name)
-
-    def add_file(self, id, parent, name, contents, mode):
-        self.base.add_file(id, parent, name, contents, mode)
-        self.this.add_file(id, parent, name, contents, mode)
-        self.other.add_file(id, parent, name, contents, mode)
-        path = self.get_cset_path(parent, name)
-        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
-
-    def remove_file(self, id, base=False, this=False, other=False):
-        for option, tree in ((base, self.base), (this, self.this), 
-                             (other, self.other)):
-            if option:
-                tree.remove_file(id)
-            if other or base:
-                change = self.cset.entries[id].contents_change
-                assert isinstance(change, changeset.ReplaceContents)
-                if other:
-                    change.new_contents=None
-                if base:
-                    change.old_contents=None
-                if change.old_contents is None and change.new_contents is None:
-                    change = None
-
-
-    def add_dir(self, id, parent, name, mode):
-        path = self.get_cset_path(parent, name)
-        self.base.add_dir(id, parent, name, mode)
-        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
-        self.this.add_dir(id, parent, name, mode)
-        self.other.add_dir(id, parent, name, mode)
-
-
-    def change_name(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_name_tree(id, self.base, base)
-            self.cset.entries[id].name = base
-
-        if this is not None:
-            self.change_name_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_name_tree(id, self.other, other)
-            self.cset.entries[id].new_name = other
-
-    def change_parent(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_parent_tree(id, self.base, base)
-            self.cset.entries[id].parent = base
-            self.cset.entries[id].dir = self.cset.entries[base].path
-
-        if this is not None:
-            self.change_parent_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_parent_tree(id, self.other, other)
-            self.cset.entries[id].new_parent = other
-            self.cset.entries[id].new_dir = \
-                self.cset.entries[other].new_path
-
-    def change_contents(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_contents_tree(id, self.base, base)
-
-        if this is not None:
-            self.change_contents_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_contents_tree(id, self.other, other)
-
-        if base is not None or other is not None:
-            old_contents = file(self.base.full_path(id)).read()
-            new_contents = file(self.other.full_path(id)).read()
-            contents = changeset.ReplaceFileContents(old_contents, 
-                                                     new_contents)
-            self.cset.entries[id].contents_change = contents
-
-    def change_perms(self, id, base=None, this=None, other=None):
-        if base is not None:
-            self.change_perms_tree(id, self.base, base)
-
-        if this is not None:
-            self.change_perms_tree(id, self.this, this)
-
-        if other is not None:
-            self.change_perms_tree(id, self.other, other)
-
-        if base is not None or other is not None:
-            old_perms = os.stat(self.base.full_path(id)).st_mode &077
-            new_perms = os.stat(self.other.full_path(id)).st_mode &077
-            contents = changeset.ChangeUnixPermissions(old_perms, 
-                                                       new_perms)
-            self.cset.entries[id].metadata_change = contents
-
-    def change_name_tree(self, id, tree, name):
-        new_path = tree.child_path(self.cset.entries[id].parent, name)
-        tree.change_path(id, new_path)
-
-    def change_parent_tree(self, id, tree, parent):
-        new_path = tree.child_path(parent, self.cset.entries[id].name)
-        tree.change_path(id, new_path)
-
-    def change_contents_tree(self, id, tree, contents):
-        path = tree.full_path(id)
-        mode = os.stat(path).st_mode
-        file(path, "w").write(contents)
-        os.chmod(path, mode)
-
-    def change_perms_tree(self, id, tree, mode):
-        os.chmod(tree.full_path(id), mode)
-
-    def merge_changeset(self):
-        all_inventory = ThreewayInventory(Inventory(self.this.inventory),
-                                          Inventory(self.base.inventory), 
-                                          Inventory(self.other.inventory))
-        conflict_handler = changeset.ExceptionConflictHandler(self.this.dir)
-        return make_merge_changeset(self.cset, all_inventory, self.this,
-                                    self.base, self.other, conflict_handler)
-
-    def apply_inv_change(self, inventory_change, orig_inventory):
-        orig_inventory_by_path = {}
-        for file_id, path in orig_inventory.iteritems():
-            orig_inventory_by_path[path] = file_id
-
-        def parent_id(file_id):
-            try:
-                parent_dir = os.path.dirname(orig_inventory[file_id])
-            except:
-                print file_id
-                raise
-            if parent_dir == "":
-                return None
-            return orig_inventory_by_path[parent_dir]
-        
-        def new_path(file_id):
-            if inventory_change.has_key(file_id):
-                return inventory_change[file_id]
-            else:
-                parent = parent_id(file_id)
-                if parent is None:
-                    return orig_inventory[file_id]
-                dirname = new_path(parent)
-                return os.path.join(dirname, orig_inventory[file_id])
-
-        new_inventory = {}
-        for file_id in orig_inventory.iterkeys():
-            path = new_path(file_id)
-            if path is None:
-                continue
-            new_inventory[file_id] = path
-
-        for file_id, path in inventory_change.iteritems():
-            if orig_inventory.has_key(file_id):
-                continue
-            new_inventory[file_id] = path
-        return new_inventory
-
-        
-
-    def apply_changeset(self, cset, conflict_handler=None, reverse=False):
-        inventory_change = changeset.apply_changeset(cset,
-                                                     self.this.inventory,
-                                                     self.this.dir,
-                                                     conflict_handler, reverse)
-        self.this.inventory =  self.apply_inv_change(inventory_change, 
-                                                     self.this.inventory)
-
-                    
-        
-
-        
-    def cleanup(self):
-        shutil.rmtree(self.dir)
-
-
-class MergeTest(unittest.TestCase):
-    def test_change_name(self):
-        """Test renames"""
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "hello1", 0755)
-        builder.change_name("1", other="name2")
-        builder.add_file("2", "0", "name3", "hello2", 0755)
-        builder.change_name("2", base="name4")
-        builder.add_file("3", "0", "name5", "hello3", 0755)
-        builder.change_name("3", this="name6")
-        cset = builder.merge_changeset()
-        assert(cset.entries["2"].is_boring())
-        assert(cset.entries["1"].name == "name1")
-        assert(cset.entries["1"].new_name == "name2")
-        assert(cset.entries["3"].is_boring())
-        for tree in (builder.this, builder.other, builder.base):
-            assert(tree.dir != builder.dir and 
-                   tree.dir.startswith(builder.dir))
-            for path in tree.inventory.itervalues():
-                fullpath = tree.abs_path(path)
-                assert(fullpath.startswith(tree.dir))
-                assert(not path.startswith(tree.dir))
-                assert os.path.exists(fullpath)
-        builder.apply_changeset(cset)
-        builder.cleanup()
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "hello1", 0644)
-        builder.change_name("1", other="name2", this="name3")
-        self.assertRaises(changeset.RenameConflict, 
-                          builder.merge_changeset)
-        builder.cleanup()
-        
-    def test_file_moves(self):
-        """Test moves"""
-        builder = MergeBuilder()
-        builder.add_dir("1", "0", "dir1", 0755)
-        builder.add_dir("2", "0", "dir2", 0755)
-        builder.add_file("3", "1", "file1", "hello1", 0644)
-        builder.add_file("4", "1", "file2", "hello2", 0644)
-        builder.add_file("5", "1", "file3", "hello3", 0644)
-        builder.change_parent("3", other="2")
-        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
-        builder.change_parent("4", this="2")
-        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
-        builder.change_parent("5", base="2")
-        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
-        cset = builder.merge_changeset()
-        for id in ("1", "2", "4", "5"):
-            assert(cset.entries[id].is_boring())
-        assert(cset.entries["3"].parent == "1")
-        assert(cset.entries["3"].new_parent == "2")
-        builder.apply_changeset(cset)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_dir("1", "0", "dir1", 0755)
-        builder.add_dir("2", "0", "dir2", 0755)
-        builder.add_dir("3", "0", "dir3", 0755)
-        builder.add_file("4", "1", "file1", "hello1", 0644)
-        builder.change_parent("4", other="2", this="3")
-        self.assertRaises(changeset.MoveConflict, 
-                          builder.merge_changeset)
-        builder.cleanup()
-
-    def test_contents_merge(self):
-        """Test diff3 merging"""
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4")
-        builder.add_file("2", "0", "name3", "text2", 0655)
-        builder.change_contents("2", base="text5")
-        builder.add_file("3", "0", "name5", "text3", 0744)
-        builder.change_contents("3", this="text6")
-        cset = builder.merge_changeset()
-        assert(cset.entries["1"].contents_change is not None)
-        assert(isinstance(cset.entries["1"].contents_change,
-                          changeset.Diff3Merge))
-        assert(isinstance(cset.entries["2"].contents_change,
-                          changeset.Diff3Merge))
-        assert(cset.entries["3"].is_boring())
-        builder.apply_changeset(cset)
-        assert(file(builder.this.full_path("1"), "rb").read() == "text4" )
-        assert(file(builder.this.full_path("2"), "rb").read() == "text2" )
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", this="text3")
-        cset = builder.merge_changeset()
-        self.assertRaises(changeset.MergeConflict, builder.apply_changeset,
-                          cset)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", base=True)
-        self.assertRaises(changeset.NewContentsConflict,
-                          builder.merge_changeset)
-        builder.cleanup()
-
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_contents("1", other="text4", base="text3")
-        builder.remove_file("1", this=True)
-        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset)
-        builder.cleanup()
-
-    def test_perms_merge(self):
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655)
-        builder.add_file("2", "0", "name2", "text2", 0755)
-        builder.change_perms("2", base=0655)
-        builder.add_file("3", "0", "name3", "text3", 0755)
-        builder.change_perms("3", this=0655)
-        cset = builder.merge_changeset()
-        assert(cset.entries["1"].metadata_change is not None)
-        assert(isinstance(cset.entries["1"].metadata_change,
-                          PermissionsMerge))
-        assert(isinstance(cset.entries["2"].metadata_change,
-                          PermissionsMerge))
-        assert(cset.entries["3"].is_boring())
-        builder.apply_changeset(cset)
-        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0655)
-        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
-        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0655)
-        builder.cleanup();
-        builder = MergeBuilder()
-        builder.add_file("1", "0", "name1", "text1", 0755)
-        builder.change_perms("1", other=0655, base=0555)
-        cset = builder.merge_changeset()
-        self.assertRaises(changeset.MergePermissionConflict, 
-                     builder.apply_changeset, cset)
-        builder.cleanup()
-
-def test():        
-    changeset_suite = unittest.makeSuite(MergeTest, 'test_')
-    runner = unittest.TextTestRunner()
-    runner.run(changeset_suite)
-        
-if __name__ == "__main__":
-    test()

=== removed file 'bzrlib/meta_store.py'
--- bzrlib/meta_store.py	2005-06-27 01:36:22 +0000
+++ bzrlib/meta_store.py	1970-01-01 00:00:00 +0000
@@ -1,41 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-from trace import mutter
-from bzrlib.store import ImmutableStore
-
-class CachedStore:
-    """A store that caches data locally, to avoid repeated downloads.
-    The precacache method should be used to avoid server round-trips for
-    every piece of data.
-    """
-    def __init__(self, store, cache_dir):
-        self.source_store = store
-        self.cache_store = ImmutableStore(cache_dir)
-
-    def __getitem__(self, id):
-        mutter("Cache add %s" % id)
-        if id not in self.cache_store:
-            self.cache_store.add(self.source_store[id], id)
-        return self.cache_store[id]
-
-    def prefetch(self, ids):
-        """Copy a series of ids into the cache, before they are used.
-        For remote stores that support pipelining or async downloads, this can
-        increase speed considerably.
-        """
-        mutter("Prefetch of ids %s" % ",".join(ids))
-        self.cache_store.copy_multi(self.source_store, ids)

=== added file 'bzrlib/missing.py'
--- bzrlib/missing.py	1970-01-01 00:00:00 +0000
+++ bzrlib/missing.py	2005-09-29 12:49:59 +0000
@@ -0,0 +1,77 @@
+"""\
+A plugin for displaying what revisions are in 'other' but not in local.
+"""
+
+def show_missing(br_local, br_remote, verbose=False, quiet=False):
+    """Show the revisions which exist in br_remote, that 
+    do not exist in br_local.
+    """
+    from bzrlib.log import show_one_log
+    import sys
+    local_history = br_local.revision_history()
+    remote_history = br_remote.revision_history()
+    if local_history == remote_history:
+        if not quiet:
+            print 'Trees are identical.'
+        return 0
+    if local_history[:len(remote_history)] == remote_history:
+        # Local is not missing anything from remote, so consider it
+        # up-to-date
+        if not quiet:
+            print 'Local tree has all of remote revisions (remote is missing local)'
+        return 0
+    if quiet:
+        return 1
+
+    # Check for divergence
+    common_idx = min(len(local_history), len(remote_history)) - 1
+    if common_idx >= 0 and local_history[common_idx] != remote_history[common_idx]:
+        print 'Trees have diverged'
+
+    local_rev_set = set(local_history)
+
+    # Find the last common revision between the two trees
+    revno = 0
+    for revno, (local_rev, remote_rev) in enumerate(zip(local_history, remote_history)):
+        if local_rev != remote_rev:
+            break
+
+    missing_remote = []
+    for rno, rev_id in enumerate(remote_history[revno:]):
+        # This assumes that you can have a revision in the
+        # local history, which does not have the same ancestry
+        # as the remote ancestry.
+        # This may or may not be possible.
+        # In the future this should also checked for merged revisions.
+        if rev_id not in local_rev_set:
+            missing_remote.append((rno+revno+1, rev_id))
+
+    print 'Missing %d revisions' %  len(missing_remote)
+    print
+
+    if verbose:
+        from bzrlib.diff import compare_trees
+        from bzrlib.tree import EmptyTree
+        show_ids = True
+        last_tree = EmptyTree
+        last_rev_id = None
+    else:
+        show_ids = False
+    for revno, rev_id in missing_remote:
+        rev = br_remote.get_revision(rev_id)
+        if verbose:
+            parent_rev_id = rev.parents[0].revision_id
+            if last_rev_id == parent_rev_id:
+                parent_tree = last_tree
+            else:
+                parent_tree = br_remote.revision_tree(parent_rev_id)
+            revision_tree = br_remote.revision_tree(rev_id)
+            last_rev_id = rev_id
+            last_tree = revision_tree
+            delta = compare_trees(revision_tree, parent_tree)
+        else:
+            delta = None
+
+        show_one_log(revno, rev, delta, verbose, sys.stdout, 'original')
+    return 1
+

=== added file 'bzrlib/msgeditor.py'
--- bzrlib/msgeditor.py	1970-01-01 00:00:00 +0000
+++ bzrlib/msgeditor.py	2005-10-02 13:14:12 +0000
@@ -0,0 +1,129 @@
+# Bazaar-NG -- distributed version control
+
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Commit message editor support."""
+
+import os
+from subprocess import call
+from bzrlib.errors import BzrError
+
+def _get_editor():
+    """Return a sequence of possible editor binaries for the current platform"""
+    from bzrlib.osutils import _read_config_value
+    
+    try:
+        yield os.environ["BZR_EDITOR"]
+    except KeyError:
+        pass
+
+    e = _read_config_value("editor")
+    if e is not None:
+        yield e
+        
+    try:
+        yield os.environ["EDITOR"]
+    except KeyError:
+        pass
+
+    if os.name == "nt":
+        yield "notepad.exe"
+    elif os.name == "posix":
+        yield "/usr/bin/vi"
+
+
+def _run_editor(filename):
+    """Try to execute an editor to edit the commit message."""
+    for e in _get_editor():
+        edargs = e.split(' ')
+        x = call(edargs + [filename])
+        if x == 0:
+            return True
+        elif x == 127:
+            continue
+        else:
+            break
+    raise BzrError("Could not start any editor. "
+                   "Please specify $EDITOR or use ~/.bzr.conf/editor")
+                          
+
+def edit_commit_message(infotext, ignoreline=None):
+    """Let the user edit a commit message in a temp file.
+
+    This is run if they don't give a message or
+    message-containing file on the command line.
+
+    infotext:
+        Text to be displayed at bottom of message for
+        the user's reference; currently similar to
+        'bzr status'.
+    """
+    import tempfile
+    
+    if ignoreline is None:
+        ignoreline = "-- This line and the following will be ignored --"
+        
+    try:
+        tmp_fileno, msgfilename = tempfile.mkstemp()
+        msgfile = os.close(tmp_fileno)
+        if infotext is not None and infotext != "":
+            hasinfo = True
+            msgfile = file(msgfilename, "w")
+            msgfile.write("\n\n%s\n\n%s" % (ignoreline, infotext))
+            msgfile.close()
+        else:
+            hasinfo = False
+
+        if not _run_editor(msgfilename):
+            return None
+        
+        started = False
+        msg = []
+        lastline, nlines = 0, 0
+        for line in file(msgfilename, "r"):
+            stripped_line = line.strip()
+            # strip empty line before the log message starts
+            if not started:
+                if stripped_line != "":
+                    started = True
+                else:
+                    continue
+            # check for the ignore line only if there
+            # is additional information at the end
+            if hasinfo and stripped_line == ignoreline:
+                break
+            nlines += 1
+            # keep track of the last line that had some content
+            if stripped_line != "":
+                lastline = nlines
+            msg.append(line)
+            
+        if len(msg) == 0:
+            return ""
+        # delete empty lines at the end
+        del msg[lastline:]
+        # add a newline at the end, if needed
+        if not msg[-1].endswith("\n"):
+            return "%s%s" % ("".join(msg), "\n")
+        else:
+            return "".join(msg)
+    finally:
+        # delete the msg file in any case
+        try: os.unlink(msgfilename)
+        except IOError: pass
+

=== modified file 'bzrlib/osutils.py'
--- bzrlib/osutils.py	2005-07-23 13:59:30 +0000
+++ bzrlib/osutils.py	2005-10-03 05:16:47 +0000
@@ -1,31 +1,40 @@
 # Bazaar-NG -- distributed version control
-
+#
 # Copyright (C) 2005 by Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-import os, types, re, time, errno, sys
-from stat import S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE
+from shutil import copyfile
+from stat import (S_ISREG, S_ISDIR, S_ISLNK, ST_MODE, ST_SIZE,
+                  S_ISCHR, S_ISBLK, S_ISFIFO, S_ISSOCK)
+from cStringIO import StringIO
+import errno
+import os
+import re
+import sha
+import sys
+import time
+import types
 
+import bzrlib
 from bzrlib.errors import BzrError
 from bzrlib.trace import mutter
-import bzrlib
+
 
 def make_readonly(filename):
     """Make a filename read-only."""
-    # TODO: probably needs to be fixed for windows
     mod = os.stat(filename).st_mode
     mod = mod & 0777555
     os.chmod(filename, mod)
@@ -37,9 +46,8 @@
     os.chmod(filename, mod)
 
 
-_QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/_~-])')
+_QUOTE_RE = None
 
-_SLASH_RE = re.compile(r'[\\/]+')
 
 def quotefn(f):
     """Return a quoted filename filename
@@ -47,6 +55,10 @@
     This previously used backslash quoting, but that works poorly on
     Windows."""
     # TODO: I'm not really sure this is the best format either.x
+    global _QUOTE_RE
+    if _QUOTE_RE == None:
+        _QUOTE_RE = re.compile(r'([^a-zA-Z0-9.,:/\\_~-])')
+        
     if _QUOTE_RE.search(f):
         return '"' + f + '"'
     else:
@@ -61,8 +73,16 @@
         return 'directory'
     elif S_ISLNK(mode):
         return 'symlink'
+    elif S_ISCHR(mode):
+        return 'chardev'
+    elif S_ISBLK(mode):
+        return 'block'
+    elif S_ISFIFO(mode):
+        return 'fifo'
+    elif S_ISSOCK(mode):
+        return 'socket'
     else:
-        raise BzrError("can't handle file kind with mode %o of %r" % (mode, f))
+        return 'unknown'
 
 
 def kind_marker(kind):
@@ -75,7 +95,30 @@
     else:
         raise BzrError('invalid file kind %r' % kind)
 
+def lexists(f):
+    try:
+        if hasattr(os, 'lstat'):
+            os.lstat(f)
+        else:
+            os.stat(f)
+        return True
+    except OSError,e:
+        if e.errno == errno.ENOENT:
+            return False;
+        else:
+            raise BzrError("lstat/stat of (%r): %r" % (f, e))
 
+def normalizepath(f):
+    if hasattr(os.path, 'realpath'):
+        F = os.path.realpath
+    else:
+        F = os.path.abspath
+    [p,e] = os.path.split(f)
+    if e == "" or e == "." or e == "..":
+        return F(f)
+    else:
+        return os.path.join(F(p), e)
+    
 
 def backup_file(fn):
     """Copy a file to a backup.
@@ -84,7 +127,6 @@
 
     If the file is already a backup, it's not copied.
     """
-    import os
     if fn[-1] == '~':
         return
     bfn = fn + '~'
@@ -101,18 +143,11 @@
     finally:
         outf.close()
 
-def rename(path_from, path_to):
-    """Basically the same as os.rename() just special for win32"""
-    if sys.platform == 'win32':
-        try:
-            os.remove(path_to)
-        except OSError, e:
-            if e.errno != e.ENOENT:
-                raise
-    os.rename(path_from, path_to)
-
-
-
+if os.name == 'nt':
+    import shutil
+    rename = shutil.move
+else:
+    rename = os.rename
 
 
 def isdir(f):
@@ -123,7 +158,6 @@
         return False
 
 
-
 def isfile(f):
     """True if f is a regular file."""
     try:
@@ -131,6 +165,12 @@
     except OSError:
         return False
 
+def islink(f):
+    """True if f is a symlink."""
+    try:
+        return S_ISLNK(os.lstat(f)[ST_MODE])
+    except OSError:
+        return False
 
 def is_inside(dir, fname):
     """True if fname is inside dir.
@@ -139,24 +179,33 @@
     that . and .. and repeated slashes are eliminated, and the separators
     are canonical for the platform.
     
-    >>> is_inside('src', 'src/foo.c')
+    The empty string as a dir name is taken as top-of-tree and matches 
+    everything.
+    
+    >>> is_inside('src', os.path.join('src', 'foo.c'))
     True
     >>> is_inside('src', 'srccontrol')
     False
-    >>> is_inside('src', 'src/a/a/a/foo.c')
+    >>> is_inside('src', os.path.join('src', 'a', 'a', 'a', 'foo.c'))
     True
     >>> is_inside('foo.c', 'foo.c')
     True
+    >>> is_inside('foo.c', '')
+    False
+    >>> is_inside('', 'foo.c')
+    True
     """
     # XXX: Most callers of this can actually do something smarter by 
     # looking at the inventory
-
     if dir == fname:
         return True
     
+    if dir == '':
+        return True
+
     if dir[-1] != os.sep:
         dir += os.sep
-    
+
     return fname.startswith(dir)
 
 
@@ -174,16 +223,7 @@
     tofile.write(fromfile.read())
 
 
-def uuid():
-    """Return a new UUID"""
-    try:
-        return file('/proc/sys/kernel/random/uuid').readline().rstrip('\n')
-    except IOError:
-        return chomp(os.popen('uuidgen').readline())
-
-
 def sha_file(f):
-    import sha
     if hasattr(f, 'tell'):
         assert f.tell() == 0
     s = sha.new()
@@ -196,16 +236,21 @@
     return s.hexdigest()
 
 
+
+def sha_strings(strings):
+    """Return the sha-1 of concatenation of strings"""
+    s = sha.new()
+    map(s.update, strings)
+    return s.hexdigest()
+
+
 def sha_string(f):
-    import sha
     s = sha.new()
     s.update(f)
     return s.hexdigest()
 
 
-
 def fingerprint_file(f):
-    import sha
     s = sha.new()
     b = f.read()
     s.update(b)
@@ -221,7 +266,7 @@
     
     TODO: Global option --config-dir to override this.
     """
-    return os.path.expanduser("~/.bzr.conf")
+    return os.path.join(os.path.expanduser("~"), ".bzr.conf")
 
 
 def _auto_user_id():
@@ -260,14 +305,36 @@
     return realname, (username + '@' + socket.gethostname())
 
 
-def _get_user_id():
+def _get_user_id(branch):
     """Return the full user id from a file or environment variable.
 
-    TODO: Allow taking this from a file in the branch directory too
-    for per-branch ids."""
+    e.g. "John Hacker <jhacker@foo.org>"
+
+    branch
+        A branch to use for a per-branch configuration, or None.
+
+    The following are searched in order:
+
+    1. $BZREMAIL
+    2. .bzr/email for this branch.
+    3. ~/.bzr.conf/email
+    4. $EMAIL
+    """
     v = os.environ.get('BZREMAIL')
     if v:
         return v.decode(bzrlib.user_encoding)
+
+    if branch:
+        try:
+            return (branch.controlfile("email", "r") 
+                    .read()
+                    .decode(bzrlib.user_encoding)
+                    .rstrip("\r\n"))
+        except IOError, e:
+            if e.errno != errno.ENOENT:
+                raise
+        except BzrError, e:
+            pass
     
     try:
         return (open(os.path.join(config_dir(), "email"))
@@ -285,14 +352,14 @@
         return None
 
 
-def username():
+def username(branch):
     """Return email-style username.
 
     Something similar to 'Martin Pool <mbp@sourcefrog.net>'
 
     TODO: Check it's reasonably well-formed.
     """
-    v = _get_user_id()
+    v = _get_user_id(branch)
     if v:
         return v
     
@@ -303,18 +370,17 @@
         return email
 
 
-_EMAIL_RE = re.compile(r'[\w+.-]+@[\w+.-]+')
-def user_email():
+def user_email(branch):
     """Return just the email component of a username."""
-    e = _get_user_id()
+    e = _get_user_id(branch)
     if e:
-        m = _EMAIL_RE.search(e)
+        m = re.search(r'[\w+.-]+@[\w+.-]+', e)
         if not m:
-            raise BzrError("%r doesn't seem to contain a reasonable email address" % e)
+            raise BzrError("%r doesn't seem to contain "
+                           "a reasonable email address" % e)
         return m.group(0)
 
     return _auto_user_id()[1]
-    
 
 
 def compare_files(a, b):
@@ -329,7 +395,6 @@
             return True
 
 
-
 def local_time_offset(t=None):
     """Return offset of local zone from GMT, either at present or at time t."""
     # python2.3 localtime() can't take None
@@ -358,8 +423,8 @@
         tt = time.localtime(t)
         offset = local_time_offset(t)
     else:
-        raise BzrError("unsupported timezone format %r",
-                ['options are "utc", "original", "local"'])
+        raise BzrError("unsupported timezone format %r" % timezone,
+                       ['options are "utc", "original", "local"'])
 
     return (time.strftime("%a %Y-%m-%d %H:%M:%S", tt)
             + ' %+03d%02d' % (offset / 3600, (offset / 60) % 60))
@@ -374,21 +439,27 @@
     """Return size of given open file."""
     return os.fstat(f.fileno())[ST_SIZE]
 
-
-if hasattr(os, 'urandom'): # python 2.4 and later
+# Define rand_bytes based on platform.
+try:
+    # Python 2.4 and later have os.urandom,
+    # but it doesn't work on some arches
+    os.urandom(1)
     rand_bytes = os.urandom
-elif sys.platform == 'linux2':
-    rand_bytes = file('/dev/urandom', 'rb').read
-else:
-    # not well seeded, but better than nothing
-    def rand_bytes(n):
-        import random
-        s = ''
-        while n:
-            s += chr(random.randint(0, 255))
-            n -= 1
-        return s
-
+except (NotImplementedError, AttributeError):
+    # If python doesn't have os.urandom, or it doesn't work,
+    # then try to first pull random data from /dev/urandom
+    if os.path.exists("/dev/urandom"):
+        rand_bytes = file('/dev/urandom', 'rb').read
+    # Otherwise, use this hack as a last resort
+    else:
+        # not well seeded, but better than nothing
+        def rand_bytes(n):
+            import random
+            s = ''
+            while n:
+                s += chr(random.randint(0, 255))
+                n -= 1
+            return s
 
 ## TODO: We could later have path objects that remember their list
 ## decomposition (might be too tricksy though.)
@@ -440,13 +511,6 @@
         return os.path.join(p1, p2)
     
 
-def extern_command(cmd, ignore_errors = False):
-    mutter('external command: %s' % `cmd`)
-    if os.system(cmd):
-        if not ignore_errors:
-            raise BzrError('command failed')
-
-
 def _read_config_value(name):
     """Read a config value from the file ~/.bzr.conf/<name>
     Return None if the file does not exist"""
@@ -459,87 +523,30 @@
         raise
 
 
-def _get_editor():
-    """Return a sequence of possible editor binaries for the current platform"""
-    e = _read_config_value("editor")
-    if e is not None:
-        yield e
-        
-    if os.name == "windows":
-        yield "notepad.exe"
-    elif os.name == "posix":
-        try:
-            yield os.environ["EDITOR"]
-        except KeyError:
-            yield "/usr/bin/vi"
-
-
-def _run_editor(filename):
-    """Try to execute an editor to edit the commit message. Returns True on success,
-    False on failure"""
-    for e in _get_editor():
-        x = os.spawnvp(os.P_WAIT, e, (e, filename))
-        if x == 0:
-            return True
-        elif x == 127:
-            continue
-        else:
-            break
-    raise BzrError("Could not start any editor. Please specify $EDITOR or use ~/.bzr.conf/editor")
-    return False
-                          
-
-def get_text_message(infotext, ignoreline = "default"):
-    import tempfile
-    
-    if ignoreline == "default":
-        ignoreline = "-- This line and the following will be ignored --"
-        
+def split_lines(s):
+    """Split s into lines, but without removing the newline characters."""
+    return StringIO(s).readlines()
+
+
+def hardlinks_good():
+    return sys.platform not in ('win32', 'cygwin', 'darwin')
+
+
+def link_or_copy(src, dest):
+    """Hardlink a file, or copy it if it can't be hardlinked."""
+    if not hardlinks_good():
+        copyfile(src, dest)
+        return
     try:
-        tmp_fileno, msgfilename = tempfile.mkstemp()
-        msgfile = os.close(tmp_fileno)
-        if infotext is not None and infotext != "":
-            hasinfo = True
-            msgfile = file(msgfilename, "w")
-            msgfile.write("\n\n%s\n\n%s" % (ignoreline, infotext))
-            msgfile.close()
-        else:
-            hasinfo = False
-
-        if not _run_editor(msgfilename):
-            return None
-        
-        started = False
-        msg = []
-        lastline, nlines = 0, 0
-        for line in file(msgfilename, "r"):
-            stripped_line = line.strip()
-            # strip empty line before the log message starts
-            if not started:
-                if stripped_line != "":
-                    started = True
-                else:
-                    continue
-            # check for the ignore line only if there
-            # is additional information at the end
-            if hasinfo and stripped_line == ignoreline:
-                break
-            nlines += 1
-            # keep track of the last line that had some content
-            if stripped_line != "":
-                lastline = nlines
-            msg.append(line)
-            
-        if len(msg) == 0:
-            return None
-        # delete empty lines at the end
-        del msg[lastline:]
-        # add a newline at the end, if needed
-        if not msg[-1].endswith("\n"):
-            return "%s%s" % ("".join(msg), "\n")
-        else:
-            return "".join(msg)
-    finally:
-        # delete the msg file in any case
-        try: os.unlink(msgfilename)
-        except IOError: pass
+        os.link(src, dest)
+    except (OSError, IOError), e:
+        if e.errno != errno.EXDEV:
+            raise
+        copyfile(src, dest)
+
+
+def has_symlinks():
+    if hasattr(os, 'symlink'):
+        return True
+    else:
+        return False

=== modified file 'bzrlib/patch.py'
--- bzrlib/patch.py	2005-05-15 02:36:04 +0000
+++ bzrlib/patch.py	2005-09-23 09:25:16 +0000
@@ -1,10 +1,25 @@
 import os
-import popen2
+from subprocess import Popen, PIPE
 """
 Diff and patch functionality
 """
 __docformat__ = "restructuredtext"
 
+def write_to_cmd(args, input=""):
+    if os.name != 'nt':
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE, close_fds=True)
+    else:
+        process = Popen(args, bufsize=len(input), stdin=PIPE, stdout=PIPE,
+                        stderr=PIPE)
+
+    stdout, stderr = process.communicate(input)
+    status = process.wait()
+    if status < 0:
+        raise Exception("%s killed by signal %i" (args[0], -status))
+    return stdout, stderr, status
+    
+
 def patch(patch_contents, filename, output_filename=None, reverse=False):
     """Apply a patch to a file, to produce another output file.  This is should
     be suitable for our limited purposes.
@@ -26,39 +41,10 @@
     if output_filename is not None:
         args.extend(("-o", output_filename))
     args.append(filename)
-    process = popen2.Popen3(args, bufsize=len(patch_contents))
-    process.tochild.write(patch_contents)
-    process.tochild.close()
-    status = os.WEXITSTATUS(process.wait())
+    stdout, stderr, status = write_to_cmd(args, patch_contents)
     return status 
 
 
-def diff(orig_file, mod_str, orig_label=None, mod_label=None):
-    """Compare two files, and produce a patch.
-
-    :param orig_file: path to the old file
-    :type orig_file: str
-    :param mod_str: Contents of the new file
-    :type mod_str: str
-    :param orig_label: The label to use for the old file
-    :type orig_label: str
-    :param mod_label: The label to use for the new file
-    :type mod_label: str
-    """
-    args = ["diff", "-u" ]
-    if orig_label is not None and mod_label is not None:
-        args.extend(("-L", orig_label, "-L", mod_label))
-    args.extend(("--", orig_file, "-"))
-    process = popen2.Popen3(args, bufsize=len(mod_str))
-    process.tochild.write(mod_str)
-    process.tochild.close()
-    patch = process.fromchild.read()
-    status = os.WEXITSTATUS(process.wait())
-    if status == 0:
-        return None
-    else:
-        return patch
-
 def diff3(out_file, mine_path, older_path, yours_path):
     def add_label(args, label):
         args.extend(("-L", label))
@@ -67,11 +53,8 @@
     add_label(args, "ANCESTOR")
     add_label(args, "MERGE-SOURCE")
     args.extend((mine_path, older_path, yours_path))
-    process = popen2.Popen4(args)
-    process.tochild.close()
-    output = process.fromchild.read()
-    status = os.WEXITSTATUS(process.wait())
+    output, stderr, status = write_to_cmd(args)
     if status not in (0, 1):
-        raise Exception(output)
+        raise Exception(stderr)
     file(out_file, "wb").write(output)
     return status

=== modified file 'bzrlib/plugin.py'
--- bzrlib/plugin.py	2005-07-22 22:31:40 +0000
+++ bzrlib/plugin.py	2005-09-15 21:35:53 +0000
@@ -59,8 +59,10 @@
     
     from bzrlib.trace import log_error, mutter, log_exception
     from bzrlib.errors import BzrError
+    from bzrlib import plugins
 
-    bzrpath = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH)
+    dirs = os.environ.get('BZR_PLUGIN_PATH', DEFAULT_PLUGIN_PATH).split(":")
+    dirs.insert(0, os.path.dirname(plugins.__file__))
 
     # The problem with imp.get_suffixes() is that it doesn't include
     # .pyo which is technically valid
@@ -70,7 +72,7 @@
     suffixes = imp.get_suffixes()
     suffixes.append(('.pyo', 'rb', imp.PY_COMPILED))
     package_entries = ['__init__.py', '__init__.pyc', '__init__.pyo']
-    for d in bzrpath.split(os.pathsep):
+    for d in dirs:
         # going through them one by one allows different plugins with the same
         # filename in different directories in the path
         mutter('looking for plugins in %s' % d)
@@ -114,8 +116,9 @@
                 finally:
                     if plugin_info[0] is not None:
                         plugin_info[0].close()
-            except Exception, e:
+
+                mutter('loaded succesfully')
+            except:
                 log_error('Unable to load plugin %r from %r' % (name, d))
-                log_error(str(e))
                 log_exception()
 

=== renamed directory 'plugins' => 'bzrlib/plugins'
=== added file 'bzrlib/plugins/__init__.py'
--- bzrlib/plugins/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/__init__.py	2005-08-30 03:29:32 +0000
@@ -0,0 +1,1 @@
+"""Null placeholder plugin"""

=== added file 'bzrlib/plugins/conflicts.py'
--- bzrlib/plugins/conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/plugins/conflicts.py	2005-10-06 04:20:44 +0000
@@ -0,0 +1,68 @@
+# Copyright (C) 2005 by Aaron Bentley
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import bzrlib.status
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.commands import register_command
+from bzrlib.workingtree import CONFLICT_SUFFIXES
+import os
+import errno
+
+class cmd_conflicts(bzrlib.commands.Command):
+    """List files with conflicts.
+    (conflicts are determined by the presence of .BASE .TREE, and .OTHER 
+    files.)
+    """
+    def run(self):
+        for path in Branch.open_containing('.').working_tree().iter_conflicts():
+            print path
+
+register_command(cmd_conflicts)
+
+class cmd_resolve(bzrlib.commands.Command):
+    """Mark a conflict as resolved.
+    """
+    takes_args = ['file*']
+    takes_options = ['all']
+    def run(self, file_list=None, all=False):
+        if file_list is None:
+            if not all:
+                raise BzrCommandError(
+                    "command 'resolve' needs one or more FILE, or --all")
+            tree = Branch.open_containing('.').working_tree()
+            file_list = list(tree.abspath(f) for f in tree.iter_conflicts())
+        else:
+            if all:
+                raise BzrCommandError(
+                    "If --all is specified, no FILE may be provided")
+        for filename in file_list:
+            failures = 0
+            for suffix in CONFLICT_SUFFIXES:
+                try:
+                    os.unlink(filename+suffix)
+                except OSError, e:
+                    if e.errno != errno.ENOENT:
+                        raise
+                    else:
+                        failures += 1
+            if failures == len(CONFLICT_SUFFIXES):
+                if not os.path.exists(filename):
+                    print "%s does not exist" % filename
+                else:
+                    print "%s is not conflicted" % filename
+                        
+register_command(cmd_resolve)

=== modified file 'bzrlib/progress.py'
--- bzrlib/progress.py	2005-07-22 23:06:25 +0000
+++ bzrlib/progress.py	2005-09-16 09:56:24 +0000
@@ -94,7 +94,22 @@
         self.show_eta = show_eta
         self.show_bar = show_bar
         self.show_count = show_count
-        
+
+
+
+class DummyProgress(_BaseProgressBar):
+    """Progress-bar standin that does nothing.
+
+    This can be used as the default argument for methods that
+    take an optional progress indicator."""
+    def tick(self):
+        pass
+
+    def update(self, msg=None, current=None, total=None):
+        pass
+
+    def clear(self):
+        pass
         
     
 class DotsProgressBar(_BaseProgressBar):
@@ -176,6 +191,12 @@
     def update(self, msg, current_cnt=None, total_cnt=None):
         """Update and redraw progress bar."""
 
+        if current_cnt < 0:
+            current_cnt = 0
+            
+        if current_cnt > total_cnt:
+            total_cnt = current_cnt
+
         # save these for the tick() function
         self.last_msg = msg
         self.last_cnt = current_cnt
@@ -184,11 +205,6 @@
         if self.throttle():
             return 
         
-        if total_cnt:
-            assert current_cnt <= total_cnt
-        if current_cnt:
-            assert current_cnt >= 0
-        
         if self.show_eta and self.start_time and total_cnt:
             eta = get_eta(self.start_time, current_cnt, total_cnt)
             eta_str = " " + str_tdelta(eta)

=== removed file 'bzrlib/remotebranch.py'
--- bzrlib/remotebranch.py	2005-06-28 11:14:41 +0000
+++ bzrlib/remotebranch.py	1970-01-01 00:00:00 +0000
@@ -1,173 +0,0 @@
-#! /usr/bin/env python
-
-# Copyright (C) 2005 Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Proxy object for access to remote branches.
-
-At the moment remote branches are only for HTTP and only for read
-access.
-"""
-
-
-import gzip
-from cStringIO import StringIO
-import urllib2
-
-from errors import BzrError, BzrCheckError
-from branch import Branch, BZR_BRANCH_FORMAT
-from trace import mutter
-
-# velocitynet.com.au transparently proxies connections and thereby
-# breaks keep-alive -- sucks!
-
-
-ENABLE_URLGRABBER = True
-
-
-if ENABLE_URLGRABBER:
-    import urlgrabber
-    import urlgrabber.keepalive
-    urlgrabber.keepalive.DEBUG = 0
-    def get_url(path, compressed=False):
-        try:
-            url = path
-            if compressed:
-                url += '.gz'
-            mutter("grab url %s" % url)
-            url_f = urlgrabber.urlopen(url, keepalive=1, close_connection=0)
-            if not compressed:
-                return url_f
-            else:
-                return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        except urllib2.URLError, e:
-            raise BzrError("remote fetch failed: %r: %s" % (url, e))
-else:
-    def get_url(url, compressed=False):
-        import urllib2
-        if compressed:
-            url += '.gz'
-        mutter("get_url %s" % url)
-        url_f = urllib2.urlopen(url)
-        if compressed:
-            return gzip.GzipFile(fileobj=StringIO(url_f.read()))
-        else:
-            return url_f
-
-
-
-def _find_remote_root(url):
-    """Return the prefix URL that corresponds to the branch root."""
-    orig_url = url
-    while True:
-        try:
-            ff = get_url(url + '/.bzr/branch-format')
-
-            fmt = ff.read()
-            ff.close()
-
-            fmt = fmt.rstrip('\r\n')
-            if fmt != BZR_BRANCH_FORMAT.rstrip('\r\n'):
-                raise BzrError("sorry, branch format %r not supported at url %s"
-                               % (fmt, url))
-            
-            return url
-        except urllib2.URLError:
-            pass
-
-        try:
-            idx = url.rindex('/')
-        except ValueError:
-            raise BzrError('no branch root found for URL %s' % orig_url)
-
-        url = url[:idx]        
-        
-
-
-class RemoteBranch(Branch):
-    def __init__(self, baseurl, find_root=True):
-        """Create new proxy for a remote branch."""
-        if find_root:
-            self.baseurl = _find_remote_root(baseurl)
-        else:
-            self.baseurl = baseurl
-            self._check_format()
-
-        self.inventory_store = RemoteStore(baseurl + '/.bzr/inventory-store/')
-        self.text_store = RemoteStore(baseurl + '/.bzr/text-store/')
-        self.revision_store = RemoteStore(baseurl + '/.bzr/revision-store/')
-
-    def __str__(self):
-        b = getattr(self, 'baseurl', 'undefined')
-        return '%s(%r)' % (self.__class__.__name__, b)
-
-    __repr__ = __str__
-
-    def controlfile(self, filename, mode):
-        if mode not in ('rb', 'rt', 'r'):
-            raise BzrError("file mode %r not supported for remote branches" % mode)
-        return get_url(self.baseurl + '/.bzr/' + filename, False)
-
-
-    def lock_read(self):
-        # no locking for remote branches yet
-        pass
-
-    def lock_write(self):
-        from errors import LockError
-        raise LockError("write lock not supported for remote branch %s"
-                        % self.baseurl)
-
-    def unlock(self):
-        pass
-    
-
-    def relpath(self, path):
-        if not path.startswith(self.baseurl):
-            raise BzrError('path %r is not under base URL %r'
-                           % (path, self.baseurl))
-        pl = len(self.baseurl)
-        return path[pl:].lstrip('/')
-
-
-    def get_revision(self, revision_id):
-        from bzrlib.revision import Revision
-        from bzrlib.xml import unpack_xml
-        revf = self.revision_store[revision_id]
-        r = unpack_xml(Revision, revf)
-        if r.revision_id != revision_id:
-            raise BzrCheckError('revision stored as {%s} actually contains {%s}'
-                                % (revision_id, r.revision_id))
-        return r
-
-
-class RemoteStore(object):
-    def __init__(self, baseurl):
-        self._baseurl = baseurl
-        
-
-    def _path(self, name):
-        if '/' in name:
-            raise ValueError('invalid store id', name)
-        return self._baseurl + '/' + name
-        
-    def __getitem__(self, fileid):
-        p = self._path(fileid)
-        return get_url(p, compressed=True)
-    
-
-    

=== modified file 'bzrlib/revfile.py'
--- bzrlib/revfile.py	2005-05-26 02:13:57 +0000
+++ bzrlib/revfile.py	2005-09-26 08:56:15 +0000
@@ -52,6 +52,22 @@
 is that sequence numbers are stable references.  But not every
 repository in the world will assign the same sequence numbers,
 therefore the SHA-1 is the only universally unique reference.
+
+This is meant to scale to hold 100,000 revisions of a single file, by
+which time the index file will be ~4.8MB and a bit big to read
+sequentially.
+
+Some of the reserved fields could be used to implement a (semi?)
+balanced tree indexed by SHA1 so we can much more efficiently find the
+index associated with a particular hash.  For 100,000 revs we would be
+able to find it in about 17 random reads, which is not too bad.
+
+This performs pretty well except when trying to calculate deltas of
+really large files.  For that the main thing would be to plug in
+something faster than difflib, which is after all pure Python.
+Another approach is to just store the gzipped full text of big files,
+though perhaps that's too perverse?
+
 The iter method here will generally read through the whole index file
 in one go.  With readahead in the kernel and python/libc (typically
 128kB) this means that there should be no seeks and often only one
@@ -73,12 +89,20 @@
 # if there are gaps and that can happen if we're interrupted while
 # writing to the datafile.  Overlapping would be very bad though.
 
-
-
-import sys, zlib, struct, mdiff, stat, os, sha
+# TODO: Shouldn't need to lock if we always write in append mode and
+# then ftell after writing to see where it went.  In any case we
+# assume the whole branch is protected by a lock.
+
+import os
+import sha
+import stat
+import struct
+import sys
+import zlib
 from binascii import hexlify, unhexlify
 
-factor = 10
+import bzrlib.mdiff as mdiff
+
 
 _RECORDSIZE = 48
 
@@ -96,7 +120,7 @@
 FL_GZIP = 1
 
 # maximum number of patches in a row before recording a whole text.
-CHAIN_LIMIT = 50
+CHAIN_LIMIT = 10
 
 
 class RevfileError(Exception):
@@ -133,7 +157,6 @@
             self.idxfile = open(idxname, 'w+b')
             self.datafile = open(dataname, 'w+b')
             
-            print 'init empty file'
             self.idxfile.write(_HEADER)
             self.idxfile.flush()
         else:
@@ -227,16 +250,38 @@
         return self._add_compressed(text_sha, text, _NO_RECORD, compress)
 
 
+    # NOT USED
+    def _choose_base(self, seed, base):
+        while seed & 3 == 3:
+            if base == _NO_RECORD:
+                return _NO_RECORD
+            idxrec = self[base]
+            if idxrec[I_BASE] == _NO_RECORD:
+                return base
+
+            base = idxrec[I_BASE]
+            seed >>= 2
+                
+        return base        # relative to this full text
+        
+
+
     def _add_delta(self, text, text_sha, base, compress):
         """Add a text stored relative to a previous text."""
         self._check_index(base)
-        
+
         try:
-            base_text = self.get(base, recursion_limit=CHAIN_LIMIT)
+            base_text = self.get(base, CHAIN_LIMIT)
         except LimitHitException:
             return self._add_full_text(text, text_sha, compress)
         
         data = mdiff.bdiff(base_text, text)
+
+
+        if True: # paranoid early check for bad diff
+            result = mdiff.bpatch(base_text, data)
+            assert result == text
+            
         
         # If the delta is larger than the text, we might as well just
         # store the text.  (OK, the delta might be more compressible,
@@ -248,7 +293,7 @@
             return self._add_compressed(text_sha, data, base, compress)
 
 
-    def add(self, text, base=_NO_RECORD, compress=True):
+    def add(self, text, base=None, compress=True):
         """Add a new text to the revfile.
 
         If the text is already present them its existing id is
@@ -262,6 +307,9 @@
         only be used if it would be a size win and if the existing
         base is not at too long of a delta chain already.
         """
+        if base == None:
+            base = _NO_RECORD
+        
         self._check_write()
         
         text_sha = sha.new(text).digest()
@@ -272,6 +320,8 @@
             # it's the same, in case someone ever breaks SHA-1.
             return idx                  # already present
         
+        # base = self._choose_base(ord(text_sha[0]), base)
+
         if base == _NO_RECORD:
             return self._add_full_text(text, text_sha, compress)
         else:
@@ -296,8 +346,8 @@
             text = self._get_patched(idx, idxrec, recursion_limit)
 
         if sha.new(text).digest() != idxrec[I_SHA]:
-            raise RevfileError("corrupt SHA-1 digest on record %d"
-                               % idx)
+            raise RevfileError("corrupt SHA-1 digest on record %d in %s"
+                               % (idx, self.basename))
 
         return text
 
@@ -372,7 +422,7 @@
         self._seek_index(idx)
         idxrec = self._read_next_index()
         if idxrec == None:
-            raise IndexError()
+            raise IndexError("no index %d" % idx)
         else:
             return idxrec
 
@@ -388,7 +438,7 @@
         """Read back all index records.
 
         Do not seek the index file while this is underway!"""
-        sys.stderr.write(" ** iter called ** \n")
+        ## sys.stderr.write(" ** iter called ** \n")
         self._seek_index(0)
         while True:
             idxrec = self._read_next_index()
@@ -436,51 +486,76 @@
         for idx in range(len(self)):
             t += len(self.get(idx))
         return t
+
+
+    def check(self, pb=None):
+        """Extract every version and check its hash."""
+        total = len(self)
+        for i in range(total):
+            if pb:
+                pb.update("check revision", i, total)
+            # the get method implicitly checks the SHA-1
+            self.get(i)
+        if pb:
+            pb.clear()
         
 
 
 def main(argv):
     try:
         cmd = argv[1]
+        filename = argv[2]
     except IndexError:
-        sys.stderr.write("usage: revfile dump\n"
-                         "       revfile add\n"
-                         "       revfile add-delta BASE\n"
-                         "       revfile get IDX\n"
-                         "       revfile find-sha HEX\n"
-                         "       revfile total-text-size\n"
-                         "       revfile last\n")
+        sys.stderr.write("usage: revfile dump REVFILE\n"
+                         "       revfile add REVFILE < INPUT\n"
+                         "       revfile add-delta REVFILE BASE < INPUT\n"
+                         "       revfile add-series REVFILE BASE FILE...\n"
+                         "       revfile get REVFILE IDX\n"
+                         "       revfile find-sha REVFILE HEX\n"
+                         "       revfile total-text-size REVFILE\n"
+                         "       revfile last REVFILE\n")
         return 1
 
+    if filename.endswith('.drev') or filename.endswith('.irev'):
+        filename = filename[:-5]
+
     def rw():
-        return Revfile('testrev', 'w')
+        return Revfile(filename, 'w')
 
     def ro():
-        return Revfile('testrev', 'r')
+        return Revfile(filename, 'r')
 
     if cmd == 'add':
         print rw().add(sys.stdin.read())
     elif cmd == 'add-delta':
-        print rw().add(sys.stdin.read(), int(argv[2]))
+        print rw().add(sys.stdin.read(), int(argv[3]))
+    elif cmd == 'add-series':
+        r = rw()
+        rev = int(argv[3])
+        for fn in argv[4:]:
+            print rev
+            rev = r.add(file(fn).read(), rev)
     elif cmd == 'dump':
         ro().dump()
     elif cmd == 'get':
         try:
-            idx = int(argv[2])
+            idx = int(argv[3])
         except IndexError:
-            sys.stderr.write("usage: revfile get IDX\n")
+            sys.stderr.write("usage: revfile get FILE IDX\n")
             return 1
 
+        r = ro()
+
         if idx < 0 or idx >= len(r):
             sys.stderr.write("invalid index %r\n" % idx)
             return 1
 
-        sys.stdout.write(ro().get(idx))
+        sys.stdout.write(r.get(idx))
     elif cmd == 'find-sha':
         try:
-            s = unhexlify(argv[2])
+            s = unhexlify(argv[3])
         except IndexError:
-            sys.stderr.write("usage: revfile find-sha HEX\n")
+            sys.stderr.write("usage: revfile find-sha FILE HEX\n")
             return 1
 
         idx = ro().find_sha(s)
@@ -493,6 +568,10 @@
         print ro().total_text_size()
     elif cmd == 'last':
         print len(ro())-1
+    elif cmd == 'check':
+        import bzrlib.progress
+        pb = bzrlib.progress.ProgressBar()
+        ro().check(pb)
     else:
         sys.stderr.write("unknown command %r\n" % cmd)
         return 1

=== modified file 'bzrlib/revision.py'
--- bzrlib/revision.py	2005-07-04 11:57:18 +0000
+++ bzrlib/revision.py	2005-10-02 22:47:02 +0000
@@ -15,31 +15,10 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-
-class RevisionReference(object):
-    """
-    Reference to a stored revision.
-
-    Includes the revision_id and revision_sha1.
-    """
-    revision_id = None
-    revision_sha1 = None
-    def __init__(self, revision_id, revision_sha1=None):
-        if revision_id == None \
-           or isinstance(revision_id, basestring):
-            self.revision_id = revision_id
-        else:
-            raise ValueError('bad revision_id %r' % revision_id)
-
-        if revision_sha1 != None:
-            if isinstance(revision_sha1, basestring) \
-               and len(revision_sha1) == 40:
-                self.revision_sha1 = revision_sha1
-            else:
-                raise ValueError('bad revision_sha1 %r' % revision_sha1)
-                
-
+import bzrlib.errors
+from bzrlib.graph import node_distances, select_farthest, all_descendants
+
+NULL_REVISION="null:"
 
 class Revision(object):
     """Single revision on a branch.
@@ -50,112 +29,35 @@
 
     After bzr 0.0.5 revisions are allowed to have multiple parents.
 
-    parents
-        List of parent revisions, each is a RevisionReference.
+    parent_ids
+        List of parent revision_ids
     """
-    inventory_id = None
-    inventory_sha1 = None
-    revision_id = None
-    timestamp = None
-    message = None
-    timezone = None
-    committer = None
     
-    def __init__(self, **args):
+    def __init__(self, revision_id, **args):
+        self.revision_id = revision_id
         self.__dict__.update(args)
-        self.parents = []
-
+        self.parent_ids = []
+        self.parent_sha1s = []
 
     def __repr__(self):
         return "<Revision id %s>" % self.revision_id
 
-        
-    def to_element(self):
-        from bzrlib.xml import Element, SubElement
-        
-        root = Element('revision',
-                       committer = self.committer,
-                       timestamp = '%.9f' % self.timestamp,
-                       revision_id = self.revision_id,
-                       inventory_id = self.inventory_id,
-                       inventory_sha1 = self.inventory_sha1,
-                       )
-        if self.timezone:
-            root.set('timezone', str(self.timezone))
-        root.text = '\n'
-        
-        msg = SubElement(root, 'message')
-        msg.text = self.message
-        msg.tail = '\n'
-
-        if self.parents:
-            pelts = SubElement(root, 'parents')
-            pelts.tail = pelts.text = '\n'
-            for rr in self.parents:
-                assert isinstance(rr, RevisionReference)
-                p = SubElement(pelts, 'revision_ref')
-                p.tail = '\n'
-                assert rr.revision_id
-                p.set('revision_id', rr.revision_id)
-                if rr.revision_sha1:
-                    p.set('revision_sha1', rr.revision_sha1)
-
-        return root
-
-
-    def from_element(cls, elt):
-        return unpack_revision(elt)
-
-    from_element = classmethod(from_element)
-
-
-
-def unpack_revision(elt):
-    """Convert XML element into Revision object."""
-    # <changeset> is deprecated...
-    from bzrlib.errors import BzrError
-    
-    if elt.tag not in ('revision', 'changeset'):
-        raise BzrError("unexpected tag in revision file: %r" % elt)
-
-    rev = Revision(committer = elt.get('committer'),
-                   timestamp = float(elt.get('timestamp')),
-                   revision_id = elt.get('revision_id'),
-                   inventory_id = elt.get('inventory_id'),
-                   inventory_sha1 = elt.get('inventory_sha1')
-                   )
-
-    precursor = elt.get('precursor')
-    precursor_sha1 = elt.get('precursor_sha1')
-
-    pelts = elt.find('parents')
-
-    if pelts:
-        for p in pelts:
-            assert p.tag == 'revision_ref', \
-                   "bad parent node tag %r" % p.tag
-            rev_ref = RevisionReference(p.get('revision_id'),
-                                        p.get('revision_sha1'))
-            rev.parents.append(rev_ref)
-
-        if precursor:
-            # must be consistent
-            prec_parent = rev.parents[0].revision_id
-            assert prec_parent == precursor
-    elif precursor:
-        # revisions written prior to 0.0.5 have a single precursor
-        # give as an attribute
-        rev_ref = RevisionReference(precursor, precursor_sha1)
-        rev.parents.append(rev_ref)
-
-    v = elt.get('timezone')
-    rev.timezone = v and int(v)
-
-    rev.message = elt.findtext('message') # text of <message>
-    return rev
-
-
-
+    def __eq__(self, other):
+        if not isinstance(other, Revision):
+            return False
+        # FIXME: rbc 20050930 parent_ids are not being compared
+        return (
+                self.inventory_sha1 == other.inventory_sha1
+                and self.revision_id == other.revision_id
+                and self.timestamp == other.timestamp
+                and self.message == other.message
+                and self.timezone == other.timezone
+                and self.committer == other.committer)
+
+    def __ne__(self, other):
+        return not self.__eq__(other)
+
+        
 REVISION_ID_RE = None
 
 def validate_revision_id(rid):
@@ -163,8 +65,236 @@
     global REVISION_ID_RE
     if not REVISION_ID_RE:
         import re
-        REVISION_ID_RE = re.compile('[\w.-]+@[\w.-]+--?\d+--?[0-9a-f]+\Z')
+        REVISION_ID_RE = re.compile('[\w:.-]+@[\w%.-]+--?[\w]+--?[0-9a-f]+\Z')
 
     if not REVISION_ID_RE.match(rid):
         raise ValueError("malformed revision-id %r" % rid)
-    
+
+
+def is_ancestor(revision_id, candidate_id, branch):
+    """Return true if candidate_id is an ancestor of revision_id.
+
+    A false negative will be returned if any intermediate descendent of
+    candidate_id is not present in any of the revision_sources.
+    
+    revisions_source is an object supporting a get_revision operation that
+    behaves like Branch's.
+    """
+    return candidate_id in branch.get_ancestry(revision_id)
+
+
+def iter_ancestors(revision_id, revision_source, only_present=False):
+    ancestors = (revision_id,)
+    distance = 0
+    while len(ancestors) > 0:
+        new_ancestors = []
+        for ancestor in ancestors:
+            if not only_present:
+                yield ancestor, distance
+            try:
+                revision = revision_source.get_revision(ancestor)
+            except bzrlib.errors.NoSuchRevision, e:
+                if e.revision == revision_id:
+                    raise 
+                else:
+                    continue
+            if only_present:
+                yield ancestor, distance
+            new_ancestors.extend(revision.parent_ids)
+        ancestors = new_ancestors
+        distance += 1
+
+
+def find_present_ancestors(revision_id, revision_source):
+    """Return the ancestors of a revision present in a branch.
+
+    It's possible that a branch won't have the complete ancestry of
+    one of its revisions.  
+
+    """
+    found_ancestors = {}
+    anc_iter = enumerate(iter_ancestors(revision_id, revision_source,
+                         only_present=True))
+    for anc_order, (anc_id, anc_distance) in anc_iter:
+        if not found_ancestors.has_key(anc_id):
+            found_ancestors[anc_id] = (anc_order, anc_distance)
+    return found_ancestors
+    
+
+def __get_closest(intersection):
+    intersection.sort()
+    matches = [] 
+    for entry in intersection:
+        if entry[0] == intersection[0][0]:
+            matches.append(entry[2])
+    return matches
+
+
+def old_common_ancestor(revision_a, revision_b, revision_source):
+    """Find the ancestor common to both revisions that is closest to both.
+    """
+    from bzrlib.trace import mutter
+    a_ancestors = find_present_ancestors(revision_a, revision_source)
+    b_ancestors = find_present_ancestors(revision_b, revision_source)
+    a_intersection = []
+    b_intersection = []
+    # a_order is used as a tie-breaker when two equally-good bases are found
+    for revision, (a_order, a_distance) in a_ancestors.iteritems():
+        if b_ancestors.has_key(revision):
+            a_intersection.append((a_distance, a_order, revision))
+            b_intersection.append((b_ancestors[revision][1], a_order, revision))
+    mutter("a intersection: %r" % a_intersection)
+    mutter("b intersection: %r" % b_intersection)
+
+    a_closest = __get_closest(a_intersection)
+    if len(a_closest) == 0:
+        return None
+    b_closest = __get_closest(b_intersection)
+    assert len(b_closest) != 0
+    mutter ("a_closest %r" % a_closest)
+    mutter ("b_closest %r" % b_closest)
+    if a_closest[0] in b_closest:
+        return a_closest[0]
+    elif b_closest[0] in a_closest:
+        return b_closest[0]
+    else:
+        raise bzrlib.errors.AmbiguousBase((a_closest[0], b_closest[0]))
+    return a_closest[0]
+
+def revision_graph(revision, revision_source):
+    """Produce a graph of the ancestry of the specified revision.
+    Return root, ancestors map, descendants map
+
+    TODO: Produce graphs with the NULL revision as root, so that we can find
+    a common even when trees are not branches don't represent a single line
+    of descent.
+    """
+    ancestors = {}
+    descendants = {}
+    lines = [revision]
+    root = None
+    descendants[revision] = {}
+    while len(lines) > 0:
+        new_lines = set()
+        for line in lines:
+            if line == NULL_REVISION:
+                parents = []
+                root = NULL_REVISION
+            else:
+                try:
+                    rev = revision_source.get_revision(line)
+                    parents = list(rev.parent_ids)
+                    if len(parents) == 0:
+                        parents = [NULL_REVISION]
+                except bzrlib.errors.NoSuchRevision:
+                    if line == revision:
+                        raise
+                    parents = None
+            if parents is not None:
+                for parent in parents:
+                    if parent not in ancestors:
+                        new_lines.add(parent)
+                    if parent not in descendants:
+                        descendants[parent] = {}
+                    descendants[parent][line] = 1
+            if parents is not None:
+                ancestors[line] = set(parents)
+        lines = new_lines
+    assert root not in descendants[root]
+    assert root not in ancestors[root]
+    return root, ancestors, descendants
+
+
+def combined_graph(revision_a, revision_b, revision_source):
+    """Produce a combined ancestry graph.
+    Return graph root, ancestors map, descendants map, set of common nodes"""
+    root, ancestors, descendants = revision_graph(revision_a, revision_source)
+    root_b, ancestors_b, descendants_b = revision_graph(revision_b, 
+                                                        revision_source)
+    if root != root_b:
+        raise bzrlib.errors.NoCommonRoot(revision_a, revision_b)
+    common = set()
+    for node, node_anc in ancestors_b.iteritems():
+        if node in ancestors:
+            common.add(node)
+        else:
+            ancestors[node] = set()
+        ancestors[node].update(node_anc)
+    for node, node_dec in descendants_b.iteritems():
+        if node not in descendants:
+            descendants[node] = {}
+        descendants[node].update(node_dec)
+    return root, ancestors, descendants, common
+
+
+def common_ancestor(revision_a, revision_b, revision_source):
+    try:
+        root, ancestors, descendants, common = \
+            combined_graph(revision_a, revision_b, revision_source)
+    except bzrlib.errors.NoCommonRoot:
+        raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+        
+    distances = node_distances (descendants, ancestors, root)
+    farthest = select_farthest(distances, common)
+    if farthest is None or farthest == NULL_REVISION:
+        raise bzrlib.errors.NoCommonAncestor(revision_a, revision_b)
+    return farthest
+
+
+class MultipleRevisionSources(object):
+    """Proxy that looks in multiple branches for revisions."""
+    def __init__(self, *args):
+        object.__init__(self)
+        assert len(args) != 0
+        self._revision_sources = args
+
+    def get_revision(self, revision_id):
+        for source in self._revision_sources:
+            try:
+                return source.get_revision(revision_id)
+            except bzrlib.errors.NoSuchRevision, e:
+                pass
+        raise e
+
+def get_intervening_revisions(ancestor_id, rev_id, rev_source, 
+                              revision_history=None):
+    """Find the longest line of descent from maybe_ancestor to revision.
+    Revision history is followed where possible.
+
+    If ancestor_id == rev_id, list will be empty.
+    Otherwise, rev_id will be the last entry.  ancestor_id will never appear.
+    If ancestor_id is not an ancestor, NotAncestor will be thrown
+    """
+    root, ancestors, descendants = revision_graph(rev_id, rev_source)
+    if len(descendants) == 0:
+        raise NoSuchRevision(rev_source, rev_id)
+    if ancestor_id not in descendants:
+        rev_source.get_revision(ancestor_id)
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    root_descendants = all_descendants(descendants, ancestor_id)
+    root_descendants.add(ancestor_id)
+    if rev_id not in root_descendants:
+        raise bzrlib.errors.NotAncestor(rev_id, ancestor_id)
+    distances = node_distances(descendants, ancestors, ancestor_id,
+                               root_descendants=root_descendants)
+
+    def best_ancestor(rev_id):
+        best = None
+        for anc_id in ancestors[rev_id]:
+            try:
+                distance = distances[anc_id]
+            except KeyError:
+                continue
+            if revision_history is not None and anc_id in revision_history:
+                return anc_id
+            elif best is None or distance > best[1]:
+                best = (anc_id, distance)
+        return best[0]
+
+    next = rev_id
+    path = []
+    while next != ancestor_id:
+        path.append(next)
+        next = best_ancestor(next)
+    path.reverse()
+    return path

=== added file 'bzrlib/revisionspec.py'
--- bzrlib/revisionspec.py	1970-01-01 00:00:00 +0000
+++ bzrlib/revisionspec.py	2005-09-29 20:34:25 +0000
@@ -0,0 +1,320 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import datetime
+import re
+from bzrlib.errors import BzrError, NoSuchRevision, NoCommits
+
+_marker = []
+
+class RevisionInfo(object):
+    """The results of applying a revision specification to a branch.
+
+    An instance has two useful attributes: revno, and rev_id.
+
+    They can also be accessed as spec[0] and spec[1] respectively,
+    so that you can write code like:
+    revno, rev_id = RevisionSpec(branch, spec)
+    although this is probably going to be deprecated later.
+
+    This class exists mostly to be the return value of a RevisionSpec,
+    so that you can access the member you're interested in (number or id)
+    or treat the result as a tuple.
+    """
+
+    def __init__(self, branch, revno, rev_id=_marker):
+        self.branch = branch
+        self.revno = revno
+        if rev_id is _marker:
+            # allow caller to be lazy
+            if self.revno is None:
+                self.rev_id = None
+            else:
+                self.rev_id = branch.get_rev_id(self.revno)
+        else:
+            self.rev_id = rev_id
+
+    def __nonzero__(self):
+        # first the easy ones...
+        if self.rev_id is None:
+            return False
+        if self.revno is not None:
+            return True
+        # TODO: otherwise, it should depend on how I was built -
+        # if it's in_history(branch), then check revision_history(),
+        # if it's in_store(branch), do the check below
+        return self.rev_id in self.branch.revision_store
+
+    def __len__(self):
+        return 2
+
+    def __getitem__(self, index):
+        if index == 0: return self.revno
+        if index == 1: return self.rev_id
+        raise IndexError(index)
+
+    def get(self):
+        return self.branch.get_revision(self.rev_id)
+
+    def __eq__(self, other):
+        if type(other) not in (tuple, list, type(self)):
+            return False
+        if type(other) is type(self) and self.branch is not other.branch:
+            return False
+        return tuple(self) == tuple(other)
+
+    def __repr__(self):
+        return '<bzrlib.revisionspec.RevisionInfo object %s, %s for %r>' % (
+            self.revno, self.rev_id, self.branch)
+
+# classes in this list should have a "prefix" attribute, against which
+# string specs are matched
+SPEC_TYPES = []
+
+class RevisionSpec(object):
+    """A parsed revision specification.
+
+    A revision specification can be an integer, in which case it is
+    assumed to be a revno (though this will translate negative values
+    into positive ones); or it can be a string, in which case it is
+    parsed for something like 'date:' or 'revid:' etc.
+
+    Revision specs are an UI element, and they have been moved out
+    of the branch class to leave "back-end" classes unaware of such
+    details.  Code that gets a revno or rev_id from other code should
+    not be using revision specs - revnos and revision ids are the
+    accepted ways to refer to revisions internally.
+
+    (Equivalent to the old Branch method get_revision_info())
+    """
+
+    prefix = None
+
+    def __new__(cls, spec, foo=_marker):
+        """Parse a revision specifier.
+        """
+        if spec is None:
+            return object.__new__(RevisionSpec, spec)
+
+        try:
+            spec = int(spec)
+        except ValueError:
+            pass
+
+        if isinstance(spec, int):
+            return object.__new__(RevisionSpec_int, spec)
+        elif isinstance(spec, basestring):
+            for spectype in SPEC_TYPES:
+                if spec.startswith(spectype.prefix):
+                    return object.__new__(spectype, spec)
+            else:
+                raise BzrError('No namespace registered for string: %r' %
+                               spec)
+        else:
+            raise TypeError('Unhandled revision type %s' % spec)
+
+    def __init__(self, spec):
+        if self.prefix and spec.startswith(self.prefix):
+            spec = spec[len(self.prefix):]
+        self.spec = spec
+
+    def _match_on(self, branch, revs):
+        return RevisionInfo(branch, 0, None)
+
+    def _match_on_and_check(self, branch, revs):
+        info = self._match_on(branch, revs)
+        if info:
+            return info
+        elif info == (0, None):
+            # special case - the empty tree
+            return info
+        elif self.prefix:
+            raise NoSuchRevision(branch, self.prefix + str(self.spec))
+        else:
+            raise NoSuchRevision(branch, str(self.spec))
+
+    def in_history(self, branch):
+        revs = branch.revision_history()
+        return self._match_on_and_check(branch, revs)
+
+    def __repr__(self):
+        # this is mostly for helping with testing
+        return '<%s %s%s>' % (self.__class__.__name__,
+                              self.prefix or '',
+                              self.spec)
+
+
+# private API
+
+class RevisionSpec_int(RevisionSpec):
+    """Spec is a number.  Special case."""
+    def __init__(self, spec):
+        self.spec = int(spec)
+
+    def _match_on(self, branch, revs):
+        if self.spec < 0:
+            revno = len(revs) + self.spec + 1
+        else:
+            revno = self.spec
+        rev_id = branch.get_rev_id(revno, revs)
+        return RevisionInfo(branch, revno, rev_id)
+
+
+class RevisionSpec_revno(RevisionSpec):
+    prefix = 'revno:'
+
+    def _match_on(self, branch, revs):
+        """Lookup a revision by revision number"""
+        try:
+            return RevisionInfo(branch, int(self.spec))
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revno)
+
+
+class RevisionSpec_revid(RevisionSpec):
+    prefix = 'revid:'
+
+    def _match_on(self, branch, revs):
+        try:
+            return RevisionInfo(branch, revs.index(self.spec) + 1, self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_revid)
+
+
+class RevisionSpec_last(RevisionSpec):
+
+    prefix = 'last:'
+
+    def _match_on(self, branch, revs):
+        try:
+            offset = int(self.spec)
+        except ValueError:
+            return RevisionInfo(branch, None)
+        else:
+            if offset <= 0:
+                raise BzrError('You must supply a positive value for --revision last:XXX')
+            return RevisionInfo(branch, len(revs) - offset + 1)
+
+SPEC_TYPES.append(RevisionSpec_last)
+
+
+class RevisionSpec_before(RevisionSpec):
+
+    prefix = 'before:'
+    
+    def _match_on(self, branch, revs):
+        r = RevisionSpec(self.spec)._match_on(branch, revs)
+        if (r.revno is None) or (r.revno == 0):
+            return r
+        return RevisionInfo(branch, r.revno - 1)
+
+SPEC_TYPES.append(RevisionSpec_before)
+
+
+class RevisionSpec_tag(RevisionSpec):
+    prefix = 'tag:'
+
+    def _match_on(self, branch, revs):
+        raise BzrError('tag: namespace registered, but not implemented.')
+
+SPEC_TYPES.append(RevisionSpec_tag)
+
+
+class RevisionSpec_date(RevisionSpec):
+    prefix = 'date:'
+    _date_re = re.compile(
+            r'(?P<date>(?P<year>\d\d\d\d)-(?P<month>\d\d)-(?P<day>\d\d))?'
+            r'(,|T)?\s*'
+            r'(?P<time>(?P<hour>\d\d):(?P<minute>\d\d)(:(?P<second>\d\d))?)?'
+        )
+
+    def _match_on(self, branch, revs):
+        """
+        Spec for date revisions:
+          date:value
+          value can be 'yesterday', 'today', 'tomorrow' or a YYYY-MM-DD string.
+          matches the first entry after a given date (either at midnight or
+          at a specified time).
+
+          So the proper way of saying 'give me all entries for today' is:
+              -r date:today..date:tomorrow
+        """
+        today = datetime.datetime.fromordinal(datetime.date.today().toordinal())
+        if self.spec.lower() == 'yesterday':
+            dt = today - datetime.timedelta(days=1)
+        elif self.spec.lower() == 'today':
+            dt = today
+        elif self.spec.lower() == 'tomorrow':
+            dt = today + datetime.timedelta(days=1)
+        else:
+            m = self._date_re.match(self.spec)
+            if not m or (not m.group('date') and not m.group('time')):
+                raise BzrError('Invalid revision date %r' % self.spec)
+
+            if m.group('date'):
+                year, month, day = int(m.group('year')), int(m.group('month')), int(m.group('day'))
+            else:
+                year, month, day = today.year, today.month, today.day
+            if m.group('time'):
+                hour = int(m.group('hour'))
+                minute = int(m.group('minute'))
+                if m.group('second'):
+                    second = int(m.group('second'))
+                else:
+                    second = 0
+            else:
+                hour, minute, second = 0,0,0
+
+            dt = datetime.datetime(year=year, month=month, day=day,
+                    hour=hour, minute=minute, second=second)
+        first = dt
+        for i in range(len(revs)):
+            r = branch.get_revision(revs[i])
+            # TODO: Handle timezone.
+            dt = datetime.datetime.fromtimestamp(r.timestamp)
+            if first <= dt:
+                return RevisionInfo(branch, i+1)
+        return RevisionInfo(branch, None)
+
+SPEC_TYPES.append(RevisionSpec_date)
+
+
+class RevisionSpec_ancestor(RevisionSpec):
+    prefix = 'ancestor:'
+
+    def _match_on(self, branch, revs):
+        from branch import Branch
+        from revision import common_ancestor, MultipleRevisionSources
+        other_branch = Branch.open_containing(self.spec)
+        revision_a = branch.last_revision()
+        revision_b = other_branch.last_revision()
+        for r, b in ((revision_a, branch), (revision_b, other_branch)):
+            if r is None:
+                raise NoCommits(b)
+        revision_source = MultipleRevisionSources(branch, other_branch)
+        rev_id = common_ancestor(revision_a, revision_b, revision_source)
+        try:
+            revno = branch.revision_id_to_revno(rev_id)
+        except NoSuchRevision:
+            revno = None
+        return RevisionInfo(branch, revno, rev_id)
+        
+SPEC_TYPES.append(RevisionSpec_ancestor)

=== added file 'bzrlib/selftest/HTTPTestUtil.py'
--- bzrlib/selftest/HTTPTestUtil.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/HTTPTestUtil.py	2005-09-29 20:34:25 +0000
@@ -0,0 +1,145 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import BaseHTTPServer, SimpleHTTPServer, socket, errno, time
+from bzrlib.selftest import TestCaseInTempDir
+
+
+class WebserverNotAvailable(Exception):
+    pass
+
+class BadWebserverPath(ValueError):
+    def __str__(self):
+        return 'path %s is not in %s' % self.args
+
+class TestingHTTPRequestHandler(SimpleHTTPServer.SimpleHTTPRequestHandler):
+    def log_message(self, format, *args):
+        self.server.test_case.log("webserver - %s - - [%s] %s" %
+                                  (self.address_string(),
+                                   self.log_date_time_string(),
+                                   format%args))
+
+    def handle_one_request(self):
+        """Handle a single HTTP request.
+
+        You normally don't need to override this method; see the class
+        __doc__ string for information on how to handle specific HTTP
+        commands such as GET and POST.
+
+        """
+        for i in xrange(1,11): # Don't try more than 10 times
+            try:
+                self.raw_requestline = self.rfile.readline()
+            except socket.error, e:
+                if e.args[0] == errno.EAGAIN:
+                    self.log_message('EAGAIN (%d) while reading from raw_requestline' % i)
+                    time.sleep(0.01)
+                    continue
+                raise
+            else:
+                break
+        if not self.raw_requestline:
+            self.close_connection = 1
+            return
+        if not self.parse_request(): # An error code has been sent, just exit
+            return
+        mname = 'do_' + self.command
+        if not hasattr(self, mname):
+            self.send_error(501, "Unsupported method (%r)" % self.command)
+            return
+        method = getattr(self, mname)
+        method()
+
+class TestingHTTPServer(BaseHTTPServer.HTTPServer):
+    def __init__(self, server_address, RequestHandlerClass, test_case):
+        BaseHTTPServer.HTTPServer.__init__(self, server_address,
+                                                RequestHandlerClass)
+        self.test_case = test_case
+
+
+class TestCaseWithWebserver(TestCaseInTempDir):
+    """Derived class that starts a localhost-only webserver
+    (in addition to what TestCaseInTempDir does).
+
+    This is useful for testing RemoteBranch.
+    """
+
+    _HTTP_PORTS = range(13000, 0x8000)
+
+    def _http_start(self):
+        import SimpleHTTPServer, BaseHTTPServer, socket, errno
+        httpd = None
+        for port in self._HTTP_PORTS:
+            try:
+                httpd = TestingHTTPServer(('localhost', port),
+                                          TestingHTTPRequestHandler,
+                                          self)
+            except socket.error, e:
+                if e.args[0] == errno.EADDRINUSE:
+                    continue
+                print >>sys.stderr, "Cannot run webserver :-("
+                raise
+            else:
+                break
+
+        if httpd is None:
+            raise WebserverNotAvailable("Cannot run webserver :-( "
+                                        "no free ports in range %s..%s" %
+                                        (_HTTP_PORTS[0], _HTTP_PORTS[-1]))
+
+        self._http_base_url = 'http://localhost:%s/' % port
+        self._http_starting.release()
+        httpd.socket.settimeout(0.1)
+
+        while self._http_running:
+            try:
+                httpd.handle_request()
+            except socket.timeout:
+                pass
+
+    def get_remote_url(self, path):
+        import os
+
+        path_parts = path.split(os.path.sep)
+        if os.path.isabs(path):
+            if path_parts[:len(self._local_path_parts)] != \
+                   self._local_path_parts:
+                raise BadWebserverPath(path, self.test_dir)
+            remote_path = '/'.join(path_parts[len(self._local_path_parts):])
+        else:
+            remote_path = '/'.join(path_parts)
+
+        self._http_starting.acquire()
+        self._http_starting.release()
+        return self._http_base_url + remote_path
+
+    def setUp(self):
+        TestCaseInTempDir.setUp(self)
+        import threading, os
+        self._local_path_parts = self.test_dir.split(os.path.sep)
+        self._http_starting = threading.Lock()
+        self._http_starting.acquire()
+        self._http_running = True
+        self._http_base_url = None
+        self._http_thread = threading.Thread(target=self._http_start)
+        self._http_thread.setDaemon(True)
+        self._http_thread.start()
+
+    def tearDown(self):
+        self._http_running = False
+        self._http_thread.join()
+        TestCaseInTempDir.tearDown(self)
+

=== added file 'bzrlib/selftest/TestUtil.py'
--- bzrlib/selftest/TestUtil.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/TestUtil.py	2005-08-24 08:34:10 +0000
@@ -0,0 +1,80 @@
+# Copyright (c) 2004 Canonical Limited
+#       Author: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+#
+
+import sys
+import logging
+import unittest
+
+
+class LogCollector(logging.Handler):
+    def __init__(self):
+        logging.Handler.__init__(self)
+        self.records=[]
+    def emit(self, record):
+        self.records.append(record.getMessage())
+
+
+def makeCollectingLogger():
+    """I make a logger instance that collects its logs for programmatic analysis
+    -> (logger, collector)"""
+    logger=logging.Logger("collector")
+    handler=LogCollector()
+    handler.setFormatter(logging.Formatter("%(levelname)s: %(message)s"))
+    logger.addHandler(handler)
+    return logger, handler
+
+
+def visitTests(suite, visitor):
+    """A foreign method for visiting the tests in a test suite."""
+    for test in suite._tests:
+        #Abusing types to avoid monkey patching unittest.TestCase. 
+        # Maybe that would be better?
+        try:
+            test.visit(visitor)
+        except AttributeError:
+            if isinstance(test, unittest.TestCase):
+                visitor.visitCase(test)
+            elif isinstance(test, unittest.TestSuite):
+                visitor.visitSuite(test)
+                visitTests(test, visitor)
+            else:
+                print "unvisitable non-unittest.TestCase element %r (%r)" % (test, test.__class__)
+    
+
+class TestSuite(unittest.TestSuite):
+    """I am an extended TestSuite with a visitor interface.
+    This is primarily to allow filtering of tests - and suites or
+    more in the future. An iterator of just tests wouldn't scale..."""
+
+    def visit(self, visitor):
+        """visit the composite. Visiting is depth-first.
+        current callbacks are visitSuite and visitCase."""
+        visitor.visitSuite(self)
+        visitTests(self, visitor)
+
+
+class TestLoader(unittest.TestLoader):
+    """Custome TestLoader to set the right TestSuite class."""
+    suiteClass = TestSuite
+
+class TestVisitor(object):
+    """A visitor for Tests"""
+    def visitSuite(self, aTestSuite):
+        pass
+    def visitCase(self, aTestCase):
+        pass

=== modified file 'bzrlib/selftest/__init__.py'
--- bzrlib/selftest/__init__.py	2005-07-23 00:18:14 +0000
+++ bzrlib/selftest/__init__.py	2005-10-07 04:10:46 +0000
@@ -15,81 +15,500 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from testsweet import TestBase, run_suite, InTempDir
+from cStringIO import StringIO
+import logging
+import unittest
+import tempfile
+import os
+import sys
+import errno
+import subprocess
+import shutil
+import re
+
+import bzrlib.commands
+import bzrlib.trace
+import bzrlib.fetch
+from bzrlib.selftest import TestUtil
+from bzrlib.selftest.TestUtil import TestLoader, TestSuite
+
 
 MODULES_TO_TEST = []
 MODULES_TO_DOCTEST = []
 
-def selftest(verbose=False):
-    from unittest import TestLoader, TestSuite
-    import bzrlib, bzrlib.store, bzrlib.inventory, bzrlib.branch
-    import bzrlib.osutils, bzrlib.commands, bzrlib.merge3, bzrlib.plugin
+from logging import debug, warning, error
+
+
+
+class EarlyStoppingTestResultAdapter(object):
+    """An adapter for TestResult to stop at the first first failure or error"""
+
+    def __init__(self, result):
+        self._result = result
+
+    def addError(self, test, err):
+        self._result.addError(test, err)
+        self._result.stop()
+
+    def addFailure(self, test, err):
+        self._result.addFailure(test, err)
+        self._result.stop()
+
+    def __getattr__(self, name):
+        return getattr(self._result, name)
+
+    def __setattr__(self, name, value):
+        if name == '_result':
+            object.__setattr__(self, name, value)
+        return setattr(self._result, name, value)
+
+
+class _MyResult(unittest._TextTestResult):
+    """
+    Custom TestResult.
+
+    No special behaviour for now.
+    """
+
+    def startTest(self, test):
+        unittest.TestResult.startTest(self, test)
+        # TODO: Maybe show test.shortDescription somewhere?
+        what = test.shortDescription() or test.id()        
+        if self.showAll:
+            self.stream.write('%-70.70s' % what)
+        self.stream.flush()
+
+    def addError(self, test, err):
+        super(_MyResult, self).addError(test, err)
+        self.stream.flush()
+
+    def addFailure(self, test, err):
+        super(_MyResult, self).addFailure(test, err)
+        self.stream.flush()
+
+    def addSuccess(self, test):
+        if self.showAll:
+            self.stream.writeln('OK')
+        elif self.dots:
+            self.stream.write('~')
+        self.stream.flush()
+        unittest.TestResult.addSuccess(self, test)
+
+    def printErrorList(self, flavour, errors):
+        for test, err in errors:
+            self.stream.writeln(self.separator1)
+            self.stream.writeln("%s: %s" % (flavour,self.getDescription(test)))
+            if hasattr(test, '_get_log'):
+                self.stream.writeln()
+                self.stream.writeln('log from this test:')
+                print >>self.stream, test._get_log()
+            self.stream.writeln(self.separator2)
+            self.stream.writeln("%s" % err)
+
+
+class TextTestRunner(unittest.TextTestRunner):
+
+    def _makeResult(self):
+        result = _MyResult(self.stream, self.descriptions, self.verbosity)
+        return EarlyStoppingTestResultAdapter(result)
+
+
+def iter_suite_tests(suite):
+    """Return all tests in a suite, recursing through nested suites"""
+    for item in suite._tests:
+        if isinstance(item, unittest.TestCase):
+            yield item
+        elif isinstance(item, unittest.TestSuite):
+            for r in iter_suite_tests(item):
+                yield r
+        else:
+            raise Exception('unknown object %r inside test suite %r'
+                            % (item, suite))
+
+
+class TestSkipped(Exception):
+    """Indicates that a test was intentionally skipped, rather than failing."""
+    # XXX: Not used yet
+
+
+class CommandFailed(Exception):
+    pass
+
+class TestCase(unittest.TestCase):
+    """Base class for bzr unit tests.
+    
+    Tests that need access to disk resources should subclass 
+    TestCaseInTempDir not TestCase.
+
+    Error and debug log messages are redirected from their usual
+    location into a temporary file, the contents of which can be
+    retrieved by _get_log().
+       
+    There are also convenience functions to invoke bzr's command-line
+    routine, and to build and check bzr trees."""
+
+    BZRPATH = 'bzr'
+
+    def setUp(self):
+        unittest.TestCase.setUp(self)
+        bzrlib.trace.disable_default_logging()
+        self._enable_file_logging()
+
+
+    def _enable_file_logging(self):
+        fileno, name = tempfile.mkstemp(suffix='.log', prefix='testbzr')
+
+        self._log_file = os.fdopen(fileno, 'w+')
+
+        hdlr = logging.StreamHandler(self._log_file)
+        hdlr.setLevel(logging.DEBUG)
+        hdlr.setFormatter(logging.Formatter('%(levelname)8s  %(message)s'))
+        logging.getLogger('').addHandler(hdlr)
+        logging.getLogger('').setLevel(logging.DEBUG)
+        self._log_hdlr = hdlr
+        debug('opened log file %s', name)
+        
+        self._log_file_name = name
+
+    def tearDown(self):
+        logging.getLogger('').removeHandler(self._log_hdlr)
+        bzrlib.trace.enable_default_logging()
+        logging.debug('%s teardown', self.id())
+        self._log_file.close()
+        unittest.TestCase.tearDown(self)
+
+    def log(self, *args):
+        logging.debug(*args)
+
+    def _get_log(self):
+        """Return as a string the log for this test"""
+        return open(self._log_file_name).read()
+
+
+    def capture(self, cmd):
+        """Shortcut that splits cmd into words, runs, and returns stdout"""
+        return self.run_bzr_captured(cmd.split())[0]
+
+    def run_bzr_captured(self, argv, retcode=0):
+        """Invoke bzr and return (result, stdout, stderr).
+
+        Useful for code that wants to check the contents of the
+        output, the way error messages are presented, etc.
+
+        This should be the main method for tests that want to exercise the
+        overall behavior of the bzr application (rather than a unit test
+        or a functional test of the library.)
+
+        Much of the old code runs bzr by forking a new copy of Python, but
+        that is slower, harder to debug, and generally not necessary.
+
+        This runs bzr through the interface that catches and reports
+        errors, and with logging set to something approximating the
+        default, so that error reporting can be checked.
+
+        argv -- arguments to invoke bzr
+        retcode -- expected return code, or None for don't-care.
+        """
+        stdout = StringIO()
+        stderr = StringIO()
+        self.log('run bzr: %s', ' '.join(argv))
+        handler = logging.StreamHandler(stderr)
+        handler.setFormatter(bzrlib.trace.QuietFormatter())
+        handler.setLevel(logging.INFO)
+        logger = logging.getLogger('')
+        logger.addHandler(handler)
+        try:
+            result = self.apply_redirected(None, stdout, stderr,
+                                           bzrlib.commands.run_bzr_catch_errors,
+                                           argv)
+        finally:
+            logger.removeHandler(handler)
+        out = stdout.getvalue()
+        err = stderr.getvalue()
+        if out:
+            self.log('output:\n%s', out)
+        if err:
+            self.log('errors:\n%s', err)
+        if retcode is not None:
+            self.assertEquals(result, retcode)
+        return out, err
+
+    def run_bzr(self, *args, **kwargs):
+        """Invoke bzr, as if it were run from the command line.
+
+        This should be the main method for tests that want to exercise the
+        overall behavior of the bzr application (rather than a unit test
+        or a functional test of the library.)
+
+        This sends the stdout/stderr results into the test's log,
+        where it may be useful for debugging.  See also run_captured.
+        """
+        retcode = kwargs.pop('retcode', 0)
+        return self.run_bzr_captured(args, retcode)
+
+    def check_inventory_shape(self, inv, shape):
+        """Compare an inventory to a list of expected names.
+
+        Fail if they are not precisely equal.
+        """
+        extras = []
+        shape = list(shape)             # copy
+        for path, ie in inv.entries():
+            name = path.replace('\\', '/')
+            if ie.kind == 'dir':
+                name = name + '/'
+            if name in shape:
+                shape.remove(name)
+            else:
+                extras.append(name)
+        if shape:
+            self.fail("expected paths not found in inventory: %r" % shape)
+        if extras:
+            self.fail("unexpected paths found in inventory: %r" % extras)
+
+    def apply_redirected(self, stdin=None, stdout=None, stderr=None,
+                         a_callable=None, *args, **kwargs):
+        """Call callable with redirected std io pipes.
+
+        Returns the return code."""
+        if not callable(a_callable):
+            raise ValueError("a_callable must be callable.")
+        if stdin is None:
+            stdin = StringIO("")
+        if stdout is None:
+            if hasattr(self, "_log_file"):
+                stdout = self._log_file
+            else:
+                stdout = StringIO()
+        if stderr is None:
+            if hasattr(self, "_log_file"):
+                stderr = self._log_file
+            else:
+                stderr = StringIO()
+        real_stdin = sys.stdin
+        real_stdout = sys.stdout
+        real_stderr = sys.stderr
+        try:
+            sys.stdout = stdout
+            sys.stderr = stderr
+            sys.stdin = stdin
+            return a_callable(*args, **kwargs)
+        finally:
+            sys.stdout = real_stdout
+            sys.stderr = real_stderr
+            sys.stdin = real_stdin
+
+
+BzrTestBase = TestCase
+
+     
+class TestCaseInTempDir(TestCase):
+    """Derived class that runs a test within a temporary directory.
+
+    This is useful for tests that need to create a branch, etc.
+
+    The directory is created in a slightly complex way: for each
+    Python invocation, a new temporary top-level directory is created.
+    All test cases create their own directory within that.  If the
+    tests complete successfully, the directory is removed.
+
+    InTempDir is an old alias for FunctionalTestCase.
+    """
+
+    TEST_ROOT = None
+    _TEST_NAME = 'test'
+    OVERRIDE_PYTHON = 'python'
+
+    def check_file_contents(self, filename, expect):
+        self.log("check contents of file %s" % filename)
+        contents = file(filename, 'r').read()
+        if contents != expect:
+            self.log("expected: %r" % expect)
+            self.log("actually: %r" % contents)
+            self.fail("contents of %s not as expected" % filename)
+
+    def _make_test_root(self):
+        if TestCaseInTempDir.TEST_ROOT is not None:
+            return
+        i = 0
+        while True:
+            root = 'test%04d.tmp' % i
+            try:
+                os.mkdir(root)
+            except OSError, e:
+                if e.errno == errno.EEXIST:
+                    i += 1
+                    continue
+                else:
+                    raise
+            # successfully created
+            TestCaseInTempDir.TEST_ROOT = os.path.abspath(root)
+            break
+        # make a fake bzr directory there to prevent any tests propagating
+        # up onto the source directory's real branch
+        os.mkdir(os.path.join(TestCaseInTempDir.TEST_ROOT, '.bzr'))
+
+    def setUp(self):
+        super(TestCaseInTempDir, self).setUp()
+        self._make_test_root()
+        self._currentdir = os.getcwdu()
+        short_id = self.id().replace('bzrlib.selftest.', '') \
+                   .replace('__main__.', '')
+        self.test_dir = os.path.join(self.TEST_ROOT, short_id)
+        os.mkdir(self.test_dir)
+        os.chdir(self.test_dir)
+        
+    def tearDown(self):
+        os.chdir(self._currentdir)
+        super(TestCaseInTempDir, self).tearDown()
+
+    def build_tree(self, shape):
+        """Build a test tree according to a pattern.
+
+        shape is a sequence of file specifications.  If the final
+        character is '/', a directory is created.
+
+        This doesn't add anything to a branch.
+        """
+        # XXX: It's OK to just create them using forward slashes on windows?
+        for name in shape:
+            assert isinstance(name, basestring)
+            if name[-1] == '/':
+                os.mkdir(name[:-1])
+            else:
+                f = file(name, 'wt')
+                print >>f, "contents of", name
+                f.close()
+
+    def failUnlessExists(self, path):
+        """Fail unless path, which may be abs or relative, exists."""
+        self.failUnless(os.path.exists(path))
+        
+
+class MetaTestLog(TestCase):
+    def test_logging(self):
+        """Test logs are captured when a test fails."""
+        logging.info('an info message')
+        warning('something looks dodgy...')
+        logging.debug('hello, test is running')
+        ##assert 0
+
+
+def filter_suite_by_re(suite, pattern):
+    result = TestUtil.TestSuite()
+    filter_re = re.compile(pattern)
+    for test in iter_suite_tests(suite):
+        if filter_re.match(test.id()):
+            result.addTest(test)
+    return result
+
+
+def filter_suite_by_names(suite, wanted_names):
+    """Return a new suite containing only selected tests.
+    
+    Names are considered to match if any name is a substring of the 
+    fully-qualified test id (i.e. the class ."""
+    result = TestSuite()
+    for test in iter_suite_tests(suite):
+        this_id = test.id()
+        for p in wanted_names:
+            if this_id.find(p) != -1:
+                result.addTest(test)
+    return result
+
+
+def run_suite(suite, name='test', verbose=False, pattern=".*", testnames=None):
+    TestCaseInTempDir._TEST_NAME = name
+    if verbose:
+        verbosity = 2
+    else:
+        verbosity = 1
+    runner = TextTestRunner(stream=sys.stdout,
+                            descriptions=0,
+                            verbosity=verbosity)
+    if testnames:
+        suite = filter_suite_by_names(suite, testnames)
+    if pattern != '.*':
+        suite = filter_suite_by_re(suite, pattern)
+    result = runner.run(suite)
+    # This is still a little bogus, 
+    # but only a little. Folk not using our testrunner will
+    # have to delete their temp directories themselves.
+    if result.wasSuccessful():
+        if TestCaseInTempDir.TEST_ROOT is not None:
+            shutil.rmtree(TestCaseInTempDir.TEST_ROOT) 
+    else:
+        print "Failed tests working directories are in '%s'\n" % TestCaseInTempDir.TEST_ROOT
+    return result.wasSuccessful()
+
+
+def selftest(verbose=False, pattern=".*", testnames=None):
+    """Run the whole test suite under the enhanced runner"""
+    return run_suite(test_suite(), 'testbzr', verbose=verbose, pattern=pattern,
+                     testnames=testnames)
+
+
+def test_suite():
+    """Build and return TestSuite for the whole program."""
+    import bzrlib.store, bzrlib.inventory, bzrlib.branch
+    import bzrlib.osutils, bzrlib.merge3, bzrlib.plugin
+    from doctest import DocTestSuite
+
     global MODULES_TO_TEST, MODULES_TO_DOCTEST
 
-    import bzrlib.selftest.whitebox
-    import bzrlib.selftest.blackbox
-    import bzrlib.selftest.versioning
-    import bzrlib.selftest.testmerge3
-    import bzrlib.selftest.testhashcache
-    import bzrlib.selftest.testrevisionnamespaces
-    import bzrlib.selftest.testbranch
-    import bzrlib.selftest.teststatus
-    import bzrlib.selftest.testinv
-    import bzrlib.merge_core
-    from doctest import DocTestSuite
-    import os
-    import shutil
-    import time
-    import sys
-    import unittest
+    testmod_names = \
+                  ['bzrlib.selftest.MetaTestLog',
+                   'bzrlib.selftest.testidentitymap',
+                   'bzrlib.selftest.testinv',
+                   'bzrlib.selftest.test_ancestry',
+                   'bzrlib.selftest.test_commit',
+                   'bzrlib.selftest.test_commit_merge',
+                   'bzrlib.selftest.versioning',
+                   'bzrlib.selftest.testmerge3',
+                   'bzrlib.selftest.testmerge',
+                   'bzrlib.selftest.testhashcache',
+                   'bzrlib.selftest.teststatus',
+                   'bzrlib.selftest.testlog',
+                   'bzrlib.selftest.testrevisionnamespaces',
+                   'bzrlib.selftest.testbranch',
+                   'bzrlib.selftest.testrevision',
+                   'bzrlib.selftest.test_revision_info',
+                   'bzrlib.selftest.test_merge_core',
+                   'bzrlib.selftest.test_smart_add',
+                   'bzrlib.selftest.test_bad_files',
+                   'bzrlib.selftest.testdiff',
+                   'bzrlib.selftest.test_parent',
+                   'bzrlib.selftest.test_xml',
+                   'bzrlib.selftest.test_weave',
+                   'bzrlib.selftest.testfetch',
+                   'bzrlib.selftest.whitebox',
+                   'bzrlib.selftest.teststore',
+                   'bzrlib.selftest.blackbox',
+                   'bzrlib.selftest.testsampler',
+                   'bzrlib.selftest.testtransactions',
+                   'bzrlib.selftest.testtransport',
+                   'bzrlib.selftest.testgraph',
+                   'bzrlib.selftest.testworkingtree',
+                   'bzrlib.selftest.test_upgrade',
+                   'bzrlib.selftest.test_conflicts',
+                   ]
 
     for m in (bzrlib.store, bzrlib.inventory, bzrlib.branch,
               bzrlib.osutils, bzrlib.commands, bzrlib.merge3):
         if m not in MODULES_TO_DOCTEST:
             MODULES_TO_DOCTEST.append(m)
-    
-    # ugly ugly!
-    for m in (bzrlib.selftest.whitebox,
-              bzrlib.selftest.versioning,
-              bzrlib.selftest.testinv,
-              bzrlib.selftest.testmerge3,
-              bzrlib.selftest.testhashcache,
-              bzrlib.selftest.teststatus,
-              bzrlib.selftest.blackbox,
-              bzrlib.selftest.testhashcache,
-              bzrlib.selftest.testrevisionnamespaces,
-              bzrlib.selftest.testbranch,
-              ):
-        if m not in MODULES_TO_TEST:
-            MODULES_TO_TEST.append(m)
-
-
-    TestBase.BZRPATH = os.path.join(os.path.realpath(os.path.dirname(bzrlib.__path__[0])), 'bzr')
-    print '%-30s %s' % ('bzr binary', TestBase.BZRPATH)
-
+
+    TestCase.BZRPATH = os.path.join(os.path.realpath(os.path.dirname(bzrlib.__path__[0])), 'bzr')
+    print '%-30s %s' % ('bzr binary', TestCase.BZRPATH)
     print
-
     suite = TestSuite()
-
-    # should also test bzrlib.merge_core, but they seem to be out of date with
-    # the code.
-
-
-    # XXX: python2.3's TestLoader() doesn't seem to find all the
-    # tests; don't know why
+    suite.addTest(TestLoader().loadTestsFromNames(testmod_names))
     for m in MODULES_TO_TEST:
          suite.addTest(TestLoader().loadTestsFromModule(m))
-
     for m in (MODULES_TO_DOCTEST):
         suite.addTest(DocTestSuite(m))
-
     for p in bzrlib.plugin.all_plugins:
         if hasattr(p, 'test_suite'):
             suite.addTest(p.test_suite())
-
-    suite.addTest(unittest.makeSuite(bzrlib.merge_core.MergeTest, 'test_'))
-
-    return run_suite(suite, 'testbzr', verbose=verbose)
-
-
+    return suite
 

=== modified file 'bzrlib/selftest/blackbox.py'
--- bzrlib/selftest/blackbox.py	2005-07-15 13:43:54 +0000
+++ bzrlib/selftest/blackbox.py	2005-10-08 08:45:42 +0000
@@ -19,133 +19,478 @@
 """Black-box tests for bzr.
 
 These check that it behaves properly when it's invoked through the regular
-command-line interface.
-
-This always reinvokes bzr through a new Python interpreter, which is a
-bit inefficient but arguably tests in a way more representative of how
-it's normally invoked.
+command-line interface. This doesn't actually run a new interpreter but 
+rather starts again from the run_bzr function.
 """
 
-# this code was previously in testbzr
-
-from unittest import TestCase
-from bzrlib.selftest import TestBase, InTempDir
-
-
-
-class ExternalBase(InTempDir):
-    def runbzr(self, args, retcode=0):
-        try:
-            import shutil
-            from subprocess import call
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
+
+from cStringIO import StringIO
+import os
+import shutil
+import sys
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.errors import BzrCommandError
+from bzrlib.osutils import has_symlinks
+from bzrlib.selftest import TestCaseInTempDir, BzrTestBase
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
+
+
+class ExternalBase(TestCaseInTempDir):
+
+    def runbzr(self, args, retcode=0, backtick=False):
         if isinstance(args, basestring):
             args = args.split()
-            
-        return self.runcmd(['python', self.BZRPATH,] + args,
-                           retcode=retcode)
-
-
-
-class TestVersion(ExternalBase):
-    def runTest(self):
-        # output is intentionally passed through to stdout so that we
-        # can see the version being tested
-        self.runbzr(['version'])
-
-
-
-class HelpCommands(ExternalBase):
-    def runTest(self):
+
+        if backtick:
+            return self.run_bzr_captured(args, retcode=retcode)[0]
+        else:
+            return self.run_bzr_captured(args, retcode=retcode)
+
+
+class TestCommands(ExternalBase):
+
+    def test_help_commands(self):
         self.runbzr('--help')
         self.runbzr('help')
         self.runbzr('help commands')
         self.runbzr('help help')
         self.runbzr('commit -h')
 
-
-class InitBranch(ExternalBase):
-    def runTest(self):
-        import os
+    def test_init_branch(self):
         self.runbzr(['init'])
 
-
-
-class UserIdentity(ExternalBase):
-    def runTest(self):
+    def test_whoami(self):
         # this should always identify something, if only "john@localhost"
         self.runbzr("whoami")
         self.runbzr("whoami --email")
-        self.assertEquals(self.backtick("bzr whoami --email").count('@'),
-                          1)
-
-
-class InvalidCommands(ExternalBase):
-    def runTest(self):
+
+        self.assertEquals(self.runbzr("whoami --email",
+                                      backtick=True).count('@'), 1)
+        
+    def test_whoami_branch(self):
+        """branch specific user identity works."""
+        self.runbzr('init')
+        f = file('.bzr/email', 'wt')
+        f.write('Branch Identity <branch@identi.ty>')
+        f.close()
+        bzr_email = os.environ.get('BZREMAIL')
+        if bzr_email is not None:
+            del os.environ['BZREMAIL']
+        whoami = self.runbzr("whoami",backtick=True)
+        whoami_email = self.runbzr("whoami --email",backtick=True)
+        self.assertTrue(whoami.startswith('Branch Identity <branch@identi.ty>'))
+        self.assertTrue(whoami_email.startswith('branch@identi.ty'))
+        # Verify that the environment variable overrides the value 
+        # in the file
+        os.environ['BZREMAIL'] = 'Different ID <other@environ.ment>'
+        whoami = self.runbzr("whoami",backtick=True)
+        whoami_email = self.runbzr("whoami --email",backtick=True)
+        self.assertTrue(whoami.startswith('Different ID <other@environ.ment>'))
+        self.assertTrue(whoami_email.startswith('other@environ.ment'))
+        if bzr_email is not None:
+            os.environ['BZREMAIL'] = bzr_email
+
+    def test_invalid_commands(self):
         self.runbzr("pants", retcode=1)
         self.runbzr("--pants off", retcode=1)
         self.runbzr("diff --message foo", retcode=1)
 
-
-
-class EmptyCommit(ExternalBase):
-    def runTest(self):
+    def test_empty_commit(self):
         self.runbzr("init")
         self.build_tree(['hello.txt'])
         self.runbzr("commit -m empty", retcode=1)
         self.runbzr("add hello.txt")
         self.runbzr("commit -m added")
 
-
-
-class IgnorePatterns(ExternalBase):
-    def runTest(self):
+    def test_empty_commit_message(self):
+        self.runbzr("init")
+        file('foo.c', 'wt').write('int main() {}')
+        self.runbzr(['add', 'foo.c'])
+        self.runbzr(["commit", "-m", ""] , retcode=1) 
+
+    def test_ignore_patterns(self):
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
         self.assertEquals(list(b.unknowns()), [])
 
         file('foo.tmp', 'wt').write('tmp files are ignored')
         self.assertEquals(list(b.unknowns()), [])
-        assert self.backtick('bzr unknowns') == ''
+        assert self.capture('unknowns') == ''
 
         file('foo.c', 'wt').write('int main() {}')
         self.assertEquals(list(b.unknowns()), ['foo.c'])
-        assert self.backtick('bzr unknowns') == 'foo.c\n'
+        assert self.capture('unknowns') == 'foo.c\n'
 
         self.runbzr(['add', 'foo.c'])
-        assert self.backtick('bzr unknowns') == ''
+        assert self.capture('unknowns') == ''
 
         # 'ignore' works when creating the .bzignore file
         file('foo.blah', 'wt').write('blah')
         self.assertEquals(list(b.unknowns()), ['foo.blah'])
         self.runbzr('ignore *.blah')
         self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\n'
+        assert file('.bzrignore', 'rU').read() == '*.blah\n'
 
         # 'ignore' works when then .bzrignore file already exists
         file('garh', 'wt').write('garh')
         self.assertEquals(list(b.unknowns()), ['garh'])
-        assert self.backtick('bzr unknowns') == 'garh\n'
+        assert self.capture('unknowns') == 'garh\n'
         self.runbzr('ignore garh')
         self.assertEquals(list(b.unknowns()), [])
-        assert file('.bzrignore', 'rb').read() == '*.blah\ngarh\n'
-        
-
+        assert file('.bzrignore', 'rU').read() == '*.blah\ngarh\n'
+
+    def test_revert(self):
+        self.runbzr('init')
+
+        file('hello', 'wt').write('foo')
+        self.runbzr('add hello')
+        self.runbzr('commit -m setup hello')
+
+        file('goodbye', 'wt').write('baz')
+        self.runbzr('add goodbye')
+        self.runbzr('commit -m setup goodbye')
+
+        file('hello', 'wt').write('bar')
+        file('goodbye', 'wt').write('qux')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'foo')
+        self.check_file_contents('goodbye', 'qux')
+        self.runbzr('revert')
+        self.check_file_contents('goodbye', 'baz')
+
+        os.mkdir('revertdir')
+        self.runbzr('add revertdir')
+        self.runbzr('commit -m f')
+        os.rmdir('revertdir')
+        self.runbzr('revert')
+
+        os.symlink('/unlikely/to/exist', 'symlink')
+        self.runbzr('add symlink')
+        self.runbzr('commit -m f')
+        os.unlink('symlink')
+        self.runbzr('revert')
+        
+        file('hello', 'wt').write('xyz')
+        self.runbzr('commit -m xyz hello')
+        self.runbzr('revert -r 1 hello')
+        self.check_file_contents('hello', 'foo')
+        self.runbzr('revert hello')
+        self.check_file_contents('hello', 'xyz')
+        os.chdir('revertdir')
+        self.runbzr('revert')
+        os.chdir('..')
+
+
+    def test_mv_modes(self):
+        """Test two modes of operation for mv"""
+        from bzrlib.branch import Branch
+        b = Branch.initialize('.')
+        self.build_tree(['a', 'c', 'subdir/'])
+        self.run_bzr_captured(['add', self.test_dir])
+        self.run_bzr_captured(['mv', 'a', 'b'])
+        self.run_bzr_captured(['mv', 'b', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/b', 'a'])
+        self.run_bzr_captured(['mv', 'a', 'c', 'subdir'])
+        self.run_bzr_captured(['mv', 'subdir/a', 'subdir/newa'])
+
+    def test_main_version(self):
+        """Check output from version command and master option is reasonable"""
+        # output is intentionally passed through to stdout so that we
+        # can see the version being tested
+        output = self.runbzr('version', backtick=1)
+        self.log('bzr version output:')
+        self.log(output)
+        self.assert_(output.startswith('bzr (bazaar-ng) '))
+        self.assertNotEqual(output.index('Canonical'), -1)
+        # make sure --version is consistent
+        tmp_output = self.runbzr('--version', backtick=1)
+        self.log('bzr --version output:')
+        self.log(tmp_output)
+        self.assertEquals(output, tmp_output)
+
+    def example_branch(test):
+        test.runbzr('init')
+        file('hello', 'wt').write('foo')
+        test.runbzr('add hello')
+        test.runbzr('commit -m setup hello')
+        file('goodbye', 'wt').write('baz')
+        test.runbzr('add goodbye')
+        test.runbzr('commit -m setup goodbye')
+
+    def test_export(self):
+        os.mkdir('branch')
+        os.chdir('branch')
+        self.example_branch()
+        self.runbzr('export ../latest')
+        self.assertEqual(file('../latest/goodbye', 'rt').read(), 'baz')
+        self.runbzr('export ../first -r 1')
+        assert not os.path.exists('../first/goodbye')
+        self.assertEqual(file('../first/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.gz -r 1')
+        self.assertEqual(file('../first.gz/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.bz2 -r 1')
+        self.assertEqual(file('../first.bz2/hello', 'rt').read(), 'foo')
+        self.runbzr('export ../first.tar -r 1')
+        assert os.path.isfile('../first.tar')
+        from tarfile import TarFile
+        tf = TarFile('../first.tar')
+        assert 'first/hello' in tf.getnames(), tf.getnames()
+        self.assertEqual(tf.extractfile('first/hello').read(), 'foo')
+        self.runbzr('export ../first.tar.gz -r 1')
+        assert os.path.isfile('../first.tar.gz')
+        self.runbzr('export ../first.tbz2 -r 1')
+        assert os.path.isfile('../first.tbz2')
+        self.runbzr('export ../first.tar.bz2 -r 1')
+        assert os.path.isfile('../first.tar.bz2')
+        self.runbzr('export ../first.tar.tbz2 -r 1')
+        assert os.path.isfile('../first.tar.tbz2')
+        from bz2 import BZ2File
+        tf = TarFile('../first.tar.tbz2', 
+                     fileobj=BZ2File('../first.tar.tbz2', 'r'))
+        assert 'first.tar/hello' in tf.getnames(), tf.getnames()
+        self.assertEqual(tf.extractfile('first.tar/hello').read(), 'foo')
+        self.runbzr('export ../first2.tar -r 1 --root pizza')
+        tf = TarFile('../first2.tar')
+        assert 'pizza/hello' in tf.getnames(), tf.getnames()
+
+    def test_diff(self):
+        self.example_branch()
+        file('hello', 'wt').write('hello world!')
+        self.runbzr('commit -m fixing hello')
+        output = self.runbzr('diff -r 2..3', backtick=1)
+        self.assert_('\n+hello world!' in output)
+        output = self.runbzr('diff -r last:3..last:1', backtick=1)
+        self.assert_('\n+baz' in output)
+
+    def test_branch(self):
+        """Branch from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        self.runbzr('branch a c -r 1')
+        os.chdir('b')
+        self.runbzr('commit -m foo --unchanged')
+        os.chdir('..')
+        # naughty - abstraction violations RBC 20050928  
+        print "test_branch used to delete the stores, how is this meant to work ?"
+        #shutil.rmtree('a/.bzr/revision-store')
+        #shutil.rmtree('a/.bzr/inventory-store', ignore_errors=True)
+        #shutil.rmtree('a/.bzr/text-store', ignore_errors=True)
+        self.runbzr('branch a d --basis b')
+
+    def test_merge(self):
+        from bzrlib.branch import Branch
+        
+        os.mkdir('a')
+        os.chdir('a')
+        self.example_branch()
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        file('goodbye', 'wt').write('quux')
+        self.runbzr(['commit',  '-m',  "more u's are always good"])
+
+        os.chdir('../a')
+        file('hello', 'wt').write('quuux')
+        # We can't merge when there are in-tree changes
+        self.runbzr('merge ../b', retcode=1)
+        self.runbzr(['commit', '-m', "Like an epidemic of u's"])
+        self.runbzr('merge ../b')
+        self.check_file_contents('goodbye', 'quux')
+        # Merging a branch pulls its revision into the tree
+        a = Branch.open('.')
+        b = Branch.open('../b')
+        a.get_revision_xml(b.last_revision())
+        self.log('pending merges: %s', a.pending_merges())
+        #        assert a.pending_merges() == [b.last_revision()], "Assertion %s %s" \
+        #        % (a.pending_merges(), b.last_patch())
+
+    def test_merge_with_missing_file(self):
+        """Merge handles missing file conflicts"""
+        os.mkdir('a')
+        os.chdir('a')
+        os.mkdir('sub')
+        print >> file('sub/a.txt', 'wb'), "hello"
+        print >> file('b.txt', 'wb'), "hello"
+        print >> file('sub/c.txt', 'wb'), "hello"
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr(('commit', '-m', 'added a'))
+        self.runbzr('branch . ../b')
+        print >> file('sub/a.txt', 'ab'), "there"
+        print >> file('b.txt', 'ab'), "there"
+        print >> file('sub/c.txt', 'ab'), "there"
+        self.runbzr(('commit', '-m', 'Added there'))
+        os.unlink('sub/a.txt')
+        os.unlink('sub/c.txt')
+        os.rmdir('sub')
+        os.unlink('b.txt')
+        self.runbzr(('commit', '-m', 'Removed a.txt'))
+        os.chdir('../b')
+        print >> file('sub/a.txt', 'ab'), "something"
+        print >> file('b.txt', 'ab'), "something"
+        print >> file('sub/c.txt', 'ab'), "something"
+        self.runbzr(('commit', '-m', 'Modified a.txt'))
+        self.runbzr('merge ../a/')
+        assert os.path.exists('sub/a.txt.THIS')
+        assert os.path.exists('sub/a.txt.BASE')
+        os.chdir('../a')
+        self.runbzr('merge ../b/')
+        assert os.path.exists('sub/a.txt.OTHER')
+        assert os.path.exists('sub/a.txt.BASE')
+
+    def test_pull(self):
+        """Pull changes from one branch to another."""
+        os.mkdir('a')
+        os.chdir('a')
+
+        self.example_branch()
+        self.runbzr('pull', retcode=1)
+        self.runbzr('missing', retcode=1)
+        self.runbzr('missing .')
+        self.runbzr('missing')
+        self.runbzr('pull')
+        self.runbzr('pull /', retcode=1)
+        self.runbzr('pull')
+
+        os.chdir('..')
+        self.runbzr('branch a b')
+        os.chdir('b')
+        self.runbzr('pull')
+        os.mkdir('subdir')
+        self.runbzr('add subdir')
+        self.runbzr('commit -m blah --unchanged')
+        os.chdir('../a')
+        a = Branch.open('.')
+        b = Branch.open('../b')
+        assert a.revision_history() == b.revision_history()[:-1]
+        self.runbzr('pull ../b')
+        assert a.revision_history() == b.revision_history()
+        self.runbzr('commit -m blah2 --unchanged')
+        os.chdir('../b')
+        self.runbzr('commit -m blah3 --unchanged')
+        self.runbzr('pull ../a', retcode=1)
+        print "DECIDE IF PULL CAN CONVERGE, blackbox.py"
+        return
+        os.chdir('../a')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah4 --unchanged')
+        os.chdir('../b/subdir')
+        self.runbzr('pull ../../a')
+        assert a.revision_history()[-1] == b.revision_history()[-1]
+        self.runbzr('commit -m blah5 --unchanged')
+        self.runbzr('commit -m blah6 --unchanged')
+        os.chdir('..')
+        self.runbzr('pull ../a')
+        os.chdir('../a')
+        self.runbzr('commit -m blah7 --unchanged')
+        self.runbzr('merge ../b')
+        self.runbzr('commit -m blah8 --unchanged')
+        self.runbzr('pull ../b')
+        self.runbzr('pull ../b')
+
+    def test_locations(self):
+        """Using and remembering different locations"""
+        os.mkdir('a')
+        os.chdir('a')
+        self.runbzr('init')
+        self.runbzr('commit -m unchanged --unchanged')
+        self.runbzr('pull', retcode=1)
+        self.runbzr('merge', retcode=1)
+        self.runbzr('branch . ../b')
+        os.chdir('../b')
+        self.runbzr('pull')
+        self.runbzr('branch . ../c')
+        self.runbzr('pull ../c')
+        self.runbzr('merge')
+        os.chdir('../a')
+        self.runbzr('pull ../b')
+        self.runbzr('pull')
+        self.runbzr('pull ../c')
+        self.runbzr('branch ../c ../d')
+        shutil.rmtree('../c')
+        self.runbzr('pull')
+        os.chdir('../b')
+        self.runbzr('pull')
+        os.chdir('../d')
+        self.runbzr('pull', retcode=1)
+        self.runbzr('pull ../a --remember')
+        self.runbzr('pull')
+        
+    def test_add_reports(self):
+        """add command prints the names of added files."""
+        b = Branch.initialize('.')
+        self.build_tree(['top.txt', 'dir/', 'dir/sub.txt'])
+        out = self.run_bzr_captured(['add'], retcode = 0)[0]
+        # the ordering is not defined at the moment
+        results = sorted(out.rstrip('\n').split('\n'))
+        self.assertEquals(['added dir',
+                           'added dir'+os.sep+'sub.txt',
+                           'added top.txt',],
+                          results)
+
+    def test_unknown_command(self):
+        """Handling of unknown command."""
+        out, err = self.run_bzr_captured(['fluffy-badger'],
+                                         retcode=1)
+        self.assertEquals(out, '')
+        err.index('unknown command')
+
+    def test_conflicts(self):
+        """Handling of merge conflicts"""
+        os.mkdir('base')
+        os.chdir('base')
+        file('hello', 'wb').write("hi world")
+        file('answer', 'wb').write("42")
+        self.runbzr('init')
+        self.runbzr('add')
+        self.runbzr('commit -m base')
+        self.runbzr('branch . ../other')
+        self.runbzr('branch . ../this')
+        os.chdir('../other')
+        file('hello', 'wb').write("Hello.")
+        file('answer', 'wb').write("Is anyone there?")
+        self.runbzr('commit -m other')
+        os.chdir('../this')
+        file('hello', 'wb').write("Hello, world")
+        self.runbzr('mv answer question')
+        file('question', 'wb').write("What do you get when you multiply six"
+                                   "times nine?")
+        self.runbzr('commit -m this')
+        self.runbzr('merge ../other')
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "hello\nquestion\n")
+        result = self.runbzr('status', backtick=1)
+        assert "conflicts:\n  hello\n  question\n" in result, result
+        self.runbzr('resolve hello')
+        result = self.runbzr('conflicts', backtick=1)
+        self.assertEquals(result, "question\n")
+        self.runbzr('commit -m conflicts', retcode=1)
+        self.runbzr('resolve --all')
+        result = self.runbzr('conflicts', backtick=1)
+        self.runbzr('commit -m conflicts')
+        self.assertEquals(result, "")
+
+def listdir_sorted(dir):
+    L = os.listdir(dir)
+    L.sort()
+    return L
 
 
 class OldTests(ExternalBase):
-    # old tests moved from ./testbzr
-    def runTest(self):
+    """old tests moved from ./testbzr."""
+
+    def test_bzr(self):
         from os import chdir, mkdir
         from os.path import exists
-        import os
 
         runbzr = self.runbzr
-        backtick = self.backtick
+        capture = self.capture
         progress = self.log
 
         progress("basic branch creation")
@@ -153,7 +498,7 @@
         chdir('branch1')
         runbzr('init')
 
-        self.assertEquals(backtick('bzr root').rstrip(),
+        self.assertEquals(capture('root').rstrip(),
                           os.path.join(self.test_dir, 'branch1'))
 
         progress("status of new file")
@@ -162,26 +507,25 @@
         f.write('hello world!\n')
         f.close()
 
-        out = backtick("bzr unknowns")
-        self.assertEquals(out, 'test.txt\n')
+        self.assertEquals(capture('unknowns'), 'test.txt\n')
 
-        out = backtick("bzr status")
+        out = capture("status")
         assert out == 'unknown:\n  test.txt\n'
 
-        out = backtick("bzr status --all")
+        out = capture("status --all")
         assert out == "unknown:\n  test.txt\n"
 
-        out = backtick("bzr status test.txt --all")
+        out = capture("status test.txt --all")
         assert out == "unknown:\n  test.txt\n"
 
         f = file('test2.txt', 'wt')
         f.write('goodbye cruel world...\n')
         f.close()
 
-        out = backtick("bzr status test.txt")
+        out = capture("status test.txt")
         assert out == "unknown:\n  test.txt\n"
 
-        out = backtick("bzr status")
+        out = capture("status")
         assert out == ("unknown:\n"
                        "  test.txt\n"
                        "  test2.txt\n")
@@ -189,11 +533,11 @@
         os.unlink('test2.txt')
 
         progress("command aliases")
-        out = backtick("bzr st --all")
+        out = capture("st --all")
         assert out == ("unknown:\n"
                        "  test.txt\n")
 
-        out = backtick("bzr stat")
+        out = capture("stat")
         assert out == ("unknown:\n"
                        "  test.txt\n")
 
@@ -203,7 +547,7 @@
         runbzr("help commands")
         runbzr("help slartibartfast", 1)
 
-        out = backtick("bzr help ci")
+        out = capture("help ci")
         out.index('aliases: ')
 
         progress("can't rename unversioned file")
@@ -212,8 +556,8 @@
         progress("adding a file")
 
         runbzr("add test.txt")
-        assert backtick("bzr unknowns") == ''
-        assert backtick("bzr status --all") == ("added:\n"
+        assert capture("unknowns") == ''
+        assert capture("status --all") == ("added:\n"
                                                 "  test.txt\n")
 
         progress("rename newly-added file")
@@ -221,7 +565,7 @@
         assert os.path.exists("hello.txt")
         assert not os.path.exists("test.txt")
 
-        assert backtick("bzr revno") == '0\n'
+        assert capture("revno") == '0\n'
 
         progress("add first revision")
         runbzr(['commit', '-m', 'add first revision'])
@@ -235,7 +579,7 @@
         runbzr("add sub1")
         runbzr("rename sub1 sub2")
         runbzr("move hello.txt sub2")
-        assert backtick("bzr relpath sub2/hello.txt") == os.path.join("sub2", "hello.txt\n")
+        assert capture("relpath sub2/hello.txt") == os.path.join("sub2", "hello.txt\n")
 
         assert exists("sub2")
         assert exists("sub2/hello.txt")
@@ -256,15 +600,16 @@
         runbzr(['commit', '-m', 'rename nested subdirectories'])
 
         chdir('sub1/sub2')
-        self.assertEquals(backtick('bzr root')[:-1],
+        self.assertEquals(capture('root')[:-1],
                           os.path.join(self.test_dir, 'branch1'))
         runbzr('move ../hello.txt .')
         assert exists('./hello.txt')
-        assert backtick('bzr relpath hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
-        assert backtick('bzr relpath ../../sub1/sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        self.assertEquals(capture('relpath hello.txt'),
+                          os.path.join('sub1', 'sub2', 'hello.txt') + '\n')
+        assert capture('relpath ../../sub1/sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
         runbzr(['commit', '-m', 'move to parent directory'])
         chdir('..')
-        assert backtick('bzr relpath sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
+        assert capture('relpath sub2/hello.txt') == os.path.join('sub1', 'sub2', 'hello.txt\n')
 
         runbzr('move sub2/hello.txt .')
         assert exists('hello.txt')
@@ -279,7 +624,7 @@
 
         runbzr('commit -F msg.tmp')
 
-        assert backtick('bzr revno') == '5\n'
+        assert capture('revno') == '5\n'
         runbzr('export -r 5 export-5.tmp')
         runbzr('export export.tmp')
 
@@ -287,11 +632,11 @@
         runbzr('log -v')
         runbzr('log -v --forward')
         runbzr('log -m', retcode=1)
-        log_out = backtick('bzr log -m commit')
+        log_out = capture('log -m commit')
         assert "this is my new commit" in log_out
         assert "rename nested" not in log_out
         assert 'revision-id' not in log_out
-        assert 'revision-id' in backtick('bzr log --show-ids -m commit')
+        assert 'revision-id' in capture('log --show-ids -m commit')
 
 
         progress("file with spaces in name")
@@ -307,102 +652,116 @@
 
         runbzr('info')
 
-
-
-
-
-
-        chdir('..')
-        chdir('..')
-        progress('branch')
-        assert os.path.exists('branch1')
-        assert not os.path.exists('branch2')
-        # Can't create a branch if it already exists
-        runbzr('branch branch1', retcode=1)
-        # Can't create a branch if its parent doesn't exist
-        runbzr('branch /unlikely/to/exist', retcode=1)
-        runbzr('branch branch1 branch2')
-        assert exists('branch2')
-        assert exists('branch2/sub1')
-        assert exists('branch2/sub1/hello.txt')
-        
-        runbzr('branch --revision 0 branch1 branch3')
-        assert not exists('branch3/sub1/hello.txt')
-        runbzr('branch --revision 0..3 branch1 branch4', retcode=1)
-
-        progress("pull")
-        chdir('branch1')
-        runbzr('pull', retcode=1)
-        runbzr('pull ../branch2')
-        chdir('.bzr')
-        runbzr('pull')
-        runbzr('commit --unchanged -m empty')
-        runbzr('pull')
-        chdir('../../branch2')
-        runbzr('pull')
-        runbzr('commit --unchanged -m empty')
-        chdir('../branch1')
-        runbzr('commit --unchanged -m empty')
-        runbzr('pull', retcode=1)
-        chdir ('..')
-
-        progress('status after remove')
-        mkdir('status-after-remove')
-        # see mail from William Dodé, 2005-05-25
-        # $ bzr init; touch a; bzr add a; bzr commit -m "add a"
-        #     * looking for changes...
-        #     added a
-        #     * commited r1
-        #     $ bzr remove a
-        #     $ bzr status
-        #     bzr: local variable 'kind' referenced before assignment
-        #     at /vrac/python/bazaar-ng/bzrlib/diff.py:286 in compare_trees()
-        #     see ~/.bzr.log for debug information
-        chdir('status-after-remove')
-        runbzr('init')
-        file('a', 'w').write('foo')
-        runbzr('add a')
-        runbzr(['commit', '-m', 'add a'])
-        runbzr('remove a')
-        runbzr('status')
-
-        chdir('..')
-
-
-
-        progress("recursive and non-recursive add")
-        mkdir('no-recurse')
-        chdir('no-recurse')
-        runbzr('init')
-        mkdir('foo')
-        fp = os.path.join('foo', 'test.txt')
-        f = file(fp, 'w')
-        f.write('hello!\n')
-        f.close()
-        runbzr('add --no-recurse foo')
-        runbzr('file-id foo')
-        runbzr('file-id ' + fp, 1)      # not versioned yet
-        runbzr('commit -m add-dir-only')
-
-        self.runbzr('file-id ' + fp, 1)      # still not versioned 
-
-        self.runbzr('add foo')
-        self.runbzr('file-id ' + fp)
-        self.runbzr('commit -m add-sub-file')
-
-        chdir('..')
-
-
-
-class RevertCommand(ExternalBase):
-    def runTest(self):
-        self.runbzr('init')
-
-        file('hello', 'wt').write('foo')
-        self.runbzr('add hello')
-        self.runbzr('commit -m setup hello')
-        
-        file('hello', 'wt').write('bar')
-        self.runbzr('revert hello')
-        self.check_file_contents('hello', 'foo')
-
+        if has_symlinks():
+            progress("symlinks")
+            mkdir('symlinks')
+            chdir('symlinks')
+            runbzr('init')
+            os.symlink("NOWHERE1", "link1")
+            runbzr('add link1')
+            assert self.capture('unknowns') == ''
+            runbzr(['commit', '-m', '1: added symlink link1'])
+    
+            mkdir('d1')
+            runbzr('add d1')
+            assert self.capture('unknowns') == ''
+            os.symlink("NOWHERE2", "d1/link2")
+            assert self.capture('unknowns') == 'd1/link2\n'
+            # is d1/link2 found when adding d1
+            runbzr('add d1')
+            assert self.capture('unknowns') == ''
+            os.symlink("NOWHERE3", "d1/link3")
+            assert self.capture('unknowns') == 'd1/link3\n'
+            runbzr(['commit', '-m', '2: added dir, symlink'])
+    
+            runbzr('rename d1 d2')
+            runbzr('move d2/link2 .')
+            runbzr('move link1 d2')
+            assert os.readlink("./link2") == "NOWHERE2"
+            assert os.readlink("d2/link1") == "NOWHERE1"
+            runbzr('add d2/link3')
+            runbzr('diff')
+            runbzr(['commit', '-m', '3: rename of dir, move symlinks, add link3'])
+    
+            os.unlink("link2")
+            os.symlink("TARGET 2", "link2")
+            os.unlink("d2/link1")
+            os.symlink("TARGET 1", "d2/link1")
+            runbzr('diff')
+            assert self.capture("relpath d2/link1") == "d2/link1\n"
+            runbzr(['commit', '-m', '4: retarget of two links'])
+    
+            runbzr('remove d2/link1')
+            assert self.capture('unknowns') == 'd2/link1\n'
+            runbzr(['commit', '-m', '5: remove d2/link1'])
+            # try with the rm alias
+            runbzr('add d2/link1')
+            runbzr(['commit', '-m', '6: add d2/link1'])
+            runbzr('rm d2/link1')
+            assert self.capture('unknowns') == 'd2/link1\n'
+            runbzr(['commit', '-m', '7: remove d2/link1'])
+    
+            os.mkdir("d1")
+            runbzr('add d1')
+            runbzr('rename d2/link3 d1/link3new')
+            assert self.capture('unknowns') == 'd2/link1\n'
+            runbzr(['commit', '-m', '8: remove d2/link1, move/rename link3'])
+            
+            runbzr(['check'])
+            
+            runbzr(['export', '-r', '1', 'exp1.tmp'])
+            chdir("exp1.tmp")
+            assert listdir_sorted(".") == [ "link1" ]
+            assert os.readlink("link1") == "NOWHERE1"
+            chdir("..")
+            
+            runbzr(['export', '-r', '2', 'exp2.tmp'])
+            chdir("exp2.tmp")
+            assert listdir_sorted(".") == [ "d1", "link1" ]
+            chdir("..")
+            
+            runbzr(['export', '-r', '3', 'exp3.tmp'])
+            chdir("exp3.tmp")
+            assert listdir_sorted(".") == [ "d2", "link2" ]
+            assert listdir_sorted("d2") == [ "link1", "link3" ]
+            assert os.readlink("d2/link1") == "NOWHERE1"
+            assert os.readlink("link2")    == "NOWHERE2"
+            chdir("..")
+            
+            runbzr(['export', '-r', '4', 'exp4.tmp'])
+            chdir("exp4.tmp")
+            assert listdir_sorted(".") == [ "d2", "link2" ]
+            assert os.readlink("d2/link1") == "TARGET 1"
+            assert os.readlink("link2")    == "TARGET 2"
+            assert listdir_sorted("d2") == [ "link1", "link3" ]
+            chdir("..")
+            
+            runbzr(['export', '-r', '5', 'exp5.tmp'])
+            chdir("exp5.tmp")
+            assert listdir_sorted(".") == [ "d2", "link2" ]
+            assert os.path.islink("link2")
+            assert listdir_sorted("d2")== [ "link3" ]
+            chdir("..")
+            
+            runbzr(['export', '-r', '8', 'exp6.tmp'])
+            chdir("exp6.tmp")
+            self.assertEqual(listdir_sorted("."), [ "d1", "d2", "link2"])
+            assert listdir_sorted("d1") == [ "link3new" ]
+            assert listdir_sorted("d2") == []
+            assert os.readlink("d1/link3new") == "NOWHERE3"
+            chdir("..")
+        else:
+            progress("skipping symlink tests")
+
+
+class HttpTests(TestCaseWithWebserver):
+    """Test bzr ui commands against remote branches."""
+
+    def test_branch(self):
+        os.mkdir('from')
+        branch = Branch.initialize('from')
+        branch.commit('empty commit for nonsense', allow_pointless=True)
+        url = self.get_remote_url('from')
+        self.run_bzr('branch', url, 'to')
+        branch = Branch.open('to')
+        self.assertEqual(1, len(branch.revision_history()))

=== modified file 'bzrlib/selftest/plugins.py'
--- bzrlib/selftest/plugins.py	2005-06-22 07:57:56 +0000
+++ bzrlib/selftest/plugins.py	2005-08-30 03:10:32 +0000
@@ -17,20 +17,25 @@
 
 """Tests for plugins"""
 
+
+
+# **************************************************
 # NOT RUN YET
-
-
-
-
-
-
-
-from bzrlib.selftest import InTempDir
-
-
-def PluginTest(InTempDir):
+# **************************************************
+
+
+
+
+
+
+
+
+from bzrlib.selftest import TestCaseInTempDir
+
+
+class PluginTest(TestCaseInTempDir):
     """Create an external plugin and test loading."""
-    def runTest(self):
+    def test_plugin_loading(self):
         import os
         
         orig_help = self.backtick('bzr help commands') # No plugins yet
@@ -51,36 +56,36 @@
 
 
 
-PLUGIN_TEXT = \
-"""import bzrlib, bzrlib.commands
-class cmd_myplug(bzrlib.commands.Command):
-    '''Just a simple test plugin.'''
-    aliases = ['mplg']
-    def run(self):
-        print 'Hello from my plugin'
-""")
-    f.close()
-
-    os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
-    help = backtick('bzr help commands')
-    assert help.find('myplug') != -1
-    assert help.find('Just a simple test plugin.') != -1
-
-
-    assert backtick('bzr myplug') == 'Hello from my plugin\n'
-    assert backtick('bzr mplg') == 'Hello from my plugin\n'
-
-    f = open(os.path.join('plugin_test', 'override.py'), 'wb')
-    f.write("""import bzrlib, bzrlib.commands
-class cmd_commit(bzrlib.commands.cmd_commit):
-    '''Commit changes into a new revision.'''
-    def run(self, *args, **kwargs):
-        print "I'm sorry dave, you can't do that"
-
-class cmd_help(bzrlib.commands.cmd_help):
-    '''Show help on a command or other topic.'''
-    def run(self, *args, **kwargs):
-        print "You have been overridden"
-        bzrlib.commands.cmd_help.run(self, *args, **kwargs)
-
-    """
+#         PLUGIN_TEXT = \
+#         """import bzrlib, bzrlib.commands
+#         class cmd_myplug(bzrlib.commands.Command):
+#             '''Just a simple test plugin.'''
+#             aliases = ['mplg']
+#             def run(self):
+#                 print 'Hello from my plugin'
+#         """
+#         f.close()
+
+#         os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
+#         help = backtick('bzr help commands')
+#         assert help.find('myplug') != -1
+#         assert help.find('Just a simple test plugin.') != -1
+
+
+#         assert backtick('bzr myplug') == 'Hello from my plugin\n'
+#         assert backtick('bzr mplg') == 'Hello from my plugin\n'
+
+#         f = open(os.path.join('plugin_test', 'override.py'), 'wb')
+#         f.write("""import bzrlib, bzrlib.commands
+#     class cmd_commit(bzrlib.commands.cmd_commit):
+#         '''Commit changes into a new revision.'''
+#         def run(self, *args, **kwargs):
+#             print "I'm sorry dave, you can't do that"
+
+#     class cmd_help(bzrlib.commands.cmd_help):
+#         '''Show help on a command or other topic.'''
+#         def run(self, *args, **kwargs):
+#             print "You have been overridden"
+#             bzrlib.commands.cmd_help.run(self, *args, **kwargs)
+
+#         """

=== added file 'bzrlib/selftest/test_ancestry.py'
--- bzrlib/selftest/test_ancestry.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_ancestry.py	2005-10-06 06:31:33 +0000
@@ -0,0 +1,58 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import sys
+import os
+
+from cStringIO import StringIO
+
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.revision import is_ancestor
+
+
+class TestAncestry(TestCaseInTempDir):
+
+    def test_straightline_ancestry(self):
+        """Test ancestry file when just committing."""
+        b = Branch.initialize('.')
+
+        b.commit(message='one',
+                 allow_pointless=True,
+                 rev_id='tester@foo--1')
+
+        b.commit(message='two',
+                 allow_pointless=True,
+                 rev_id='tester@foo--2')
+
+        ancs = b.get_ancestry('tester@foo--2')
+        self.assertEqual([None, 'tester@foo--1', 'tester@foo--2'], ancs)
+        self.assertEqual([None, 'tester@foo--1'], 
+                         b.get_ancestry('tester@foo--1'))
+
+    def test_none_is_always_an_ancestor(self):
+        b = Branch.initialize('.')
+        # note this is tested before any commits are done.
+        self.assertEqual(True, is_ancestor(None, None, b))
+        b.commit(message='one',
+                 allow_pointless=True,
+                 rev_id='tester@foo--1')
+        self.assertEqual(True, is_ancestor(None, None, b))
+        self.assertEqual(True, is_ancestor('tester@foo--1', None, b))
+        self.assertEqual(False, is_ancestor(None, 'tester@foo--1', b))
+
+
+# TODO: check that ancestry is updated to include indirectly merged revisions

=== added file 'bzrlib/selftest/test_bad_files.py'
--- bzrlib/selftest/test_bad_files.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_bad_files.py	2005-09-18 16:09:41 +0000
@@ -0,0 +1,93 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Tests being able to ignore mad filetypes.
+"""
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import BadFileKindError
+import os
+
+def verify_status(tester, branch, value):
+    from bzrlib.status import show_status
+    from cStringIO import StringIO
+
+    tof = StringIO()
+    show_status(branch, to_file=tof)
+    tof.seek(0)
+    tester.assertEquals(tof.readlines(), value)
+
+
+class TestBadFiles(TestCaseInTempDir):
+    
+    def test_bad_files(self): 
+        """Test that bzr will ignore files it doesn't like"""
+        from bzrlib.commit import commit
+        from bzrlib.add import smart_add
+        from bzrlib.branch import Branch
+
+        b = Branch.initialize('.')
+
+        self.build_tree(['one', 'two', 'three'])
+        smart_add('.')
+        commit(b, "Commit one", rev_id="a@u-0-0")
+        self.build_tree(['four'])
+        smart_add('.')
+        commit(b, "Commit two", rev_id="a@u-0-1")
+        self.build_tree(['five'])
+        smart_add('.')
+        commit(b, "Commit three", rev_id="a@u-0-2")
+
+        # We should now have a few files, lets try to
+        # put some bogus stuff in the tree
+
+        # We can only continue if we have mkfifo
+        if not hasattr(os, 'mkfifo'):
+            return
+
+        # status with nothing
+        verify_status(self, b, [])
+
+        os.mkfifo('a-fifo')
+        self.build_tree(['six'])
+
+        verify_status(self, b,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           '  six\n'
+                           ])
+        
+        # Make sure smart_add can handle having a bogus
+        # file in the way
+        smart_add('.')
+        verify_status(self, b,
+                          ['added:\n',
+                           '  six\n',
+                           'unknown:\n',
+                           '  a-fifo\n',
+                           ])
+        commit(b, "Commit four", rev_id="a@u-0-3")
+
+        verify_status(self, b,
+                          ['unknown:\n',
+                           '  a-fifo\n',
+                           ])
+
+        # We should raise an error if we are adding a bogus file
+        # Is there a way to test the actual error that should be raised?
+        self.run_bzr('add', 'a-fifo', retcode=1)
+

=== added file 'bzrlib/selftest/test_commit.py'
--- bzrlib/selftest/test_commit.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_commit.py	2005-10-06 07:03:33 +0000
@@ -0,0 +1,248 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.commit import Commit
+from bzrlib.errors import PointlessCommit, BzrError
+
+
+# TODO: Test commit with some added, and added-but-missing files
+
+class TestCommit(TestCaseInTempDir):
+
+    def test_simple_commit(self):
+        """Commit and check two versions of a single file."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add('hello')
+        b.commit(message='add hello')
+        file_id = b.working_tree().path2id('hello')
+
+        file('hello', 'w').write('version 2')
+        b.commit(message='commit 2')
+
+        eq = self.assertEquals
+        eq(b.revno(), 2)
+        rh = b.revision_history()
+        rev = b.get_revision(rh[0])
+        eq(rev.message, 'add hello')
+
+        tree1 = b.revision_tree(rh[0])
+        text = tree1.get_file_text(file_id)
+        eq(text, 'hello world')
+
+        tree2 = b.revision_tree(rh[1])
+        eq(tree2.get_file_text(file_id), 'version 2')
+
+
+    def test_delete_commit(self):
+        """Test a commit with a deleted file"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add(['hello'], ['hello-id'])
+        b.commit(message='add hello')
+
+        os.remove('hello')
+        b.commit('removed hello', rev_id='rev2')
+
+        tree = b.revision_tree('rev2')
+        self.assertFalse(tree.has_id('hello-id'))
+
+
+    def test_pointless_commit(self):
+        """Commit refuses unless there are changes or it's forced."""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello')
+        b.add(['hello'])
+        b.commit(message='add hello')
+        self.assertEquals(b.revno(), 1)
+        self.assertRaises(PointlessCommit,
+                          b.commit,
+                          message='fails',
+                          allow_pointless=False)
+        self.assertEquals(b.revno(), 1)
+        
+
+
+    def test_commit_empty(self):
+        """Commiting an empty tree works."""
+        b = Branch.initialize('.')
+        b.commit(message='empty tree', allow_pointless=True)
+        self.assertRaises(PointlessCommit,
+                          b.commit,
+                          message='empty tree',
+                          allow_pointless=False)
+        b.commit(message='empty tree', allow_pointless=True)
+        self.assertEquals(b.revno(), 2)
+
+
+    def test_selective_delete(self):
+        """Selective commit in tree with deletions"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello')
+        file('buongia', 'w').write('buongia')
+        b.add(['hello', 'buongia'],
+              ['hello-id', 'buongia-id'])
+        b.commit(message='add files',
+                 rev_id='test@rev-1')
+        
+        os.remove('hello')
+        file('buongia', 'w').write('new text')
+        b.commit(message='update text',
+                 specific_files=['buongia'],
+                 allow_pointless=False,
+                 rev_id='test@rev-2')
+
+        b.commit(message='remove hello',
+                 specific_files=['hello'],
+                 allow_pointless=False,
+                 rev_id='test@rev-3')
+
+        eq = self.assertEquals
+        eq(b.revno(), 3)
+
+        tree2 = b.revision_tree('test@rev-2')
+        self.assertTrue(tree2.has_filename('hello'))
+        self.assertEquals(tree2.get_file_text('hello-id'), 'hello')
+        self.assertEquals(tree2.get_file_text('buongia-id'), 'new text')
+        
+        tree3 = b.revision_tree('test@rev-3')
+        self.assertFalse(tree3.has_filename('hello'))
+        self.assertEquals(tree3.get_file_text('buongia-id'), 'new text')
+
+
+    def test_commit_rename(self):
+        """Test commit of a revision where a file is renamed."""
+        b = Branch.initialize('.')
+        self.build_tree(['hello'])
+        b.add(['hello'], ['hello-id'])
+        b.commit(message='one', rev_id='test@rev-1', allow_pointless=False)
+
+        b.rename_one('hello', 'fruity')
+        b.commit(message='renamed', rev_id='test@rev-2', allow_pointless=False)
+
+        eq = self.assertEquals
+        tree1 = b.revision_tree('test@rev-1')
+        eq(tree1.id2path('hello-id'), 'hello')
+        eq(tree1.get_file_text('hello-id'), 'contents of hello\n')
+        self.assertFalse(tree1.has_filename('fruity'))
+        self.check_inventory_shape(tree1.inventory, ['hello'])
+        ie = tree1.inventory['hello-id']
+        eq(ie.revision, 'test@rev-1')
+
+        tree2 = b.revision_tree('test@rev-2')
+        eq(tree2.id2path('hello-id'), 'fruity')
+        eq(tree2.get_file_text('hello-id'), 'contents of hello\n')
+        self.check_inventory_shape(tree2.inventory, ['fruity'])
+        ie = tree2.inventory['hello-id']
+        eq(ie.revision, 'test@rev-2')
+
+
+    def test_reused_rev_id(self):
+        """Test that a revision id cannot be reused in a branch"""
+        b = Branch.initialize('.')
+        b.commit('initial', rev_id='test@rev-1', allow_pointless=True)
+        self.assertRaises(Exception,
+                          b.commit,
+                          message='reused id',
+                          rev_id='test@rev-1',
+                          allow_pointless=True)
+                          
+
+
+    def test_commit_move(self):
+        """Test commit of revisions with moved files and directories"""
+        eq = self.assertEquals
+        b = Branch.initialize('.')
+        r1 = 'test@rev-1'
+        self.build_tree(['hello', 'a/', 'b/'])
+        b.add(['hello', 'a', 'b'], ['hello-id', 'a-id', 'b-id'])
+        b.commit('initial', rev_id=r1, allow_pointless=False)
+
+        b.move(['hello'], 'a')
+        r2 = 'test@rev-2'
+        b.commit('two', rev_id=r2, allow_pointless=False)
+        self.check_inventory_shape(b.inventory,
+                                   ['a', 'a/hello', 'b'])
+
+        b.move(['b'], 'a')
+        r3 = 'test@rev-3'
+        b.commit('three', rev_id=r3, allow_pointless=False)
+        self.check_inventory_shape(b.inventory,
+                                   ['a', 'a/hello', 'a/b'])
+        self.check_inventory_shape(b.get_revision_inventory(r3),
+                                   ['a', 'a/hello', 'a/b'])
+
+        b.move([os.sep.join(['a', 'hello'])],
+               os.sep.join(['a', 'b']))
+        r4 = 'test@rev-4'
+        b.commit('four', rev_id=r4, allow_pointless=False)
+        self.check_inventory_shape(b.inventory,
+                                   ['a', 'a/b/hello', 'a/b'])
+
+        inv = b.get_revision_inventory(r4)
+        eq(inv['hello-id'].revision, r4)
+        eq(inv['a-id'].revision, r1)
+        eq(inv['b-id'].revision, r3)
+
+        
+    def test_removed_commit(self):
+        """Test a commit with a removed file"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        b.add(['hello'], ['hello-id'])
+        b.commit(message='add hello')
+
+        b.remove('hello')
+        b.commit('removed hello', rev_id='rev2')
+
+        tree = b.revision_tree('rev2')
+        self.assertFalse(tree.has_id('hello-id'))
+
+
+    def test_committed_ancestry(self):
+        """Test commit appends revisions to ancestry."""
+        b = Branch.initialize('.')
+        rev_ids = []
+        for i in range(4):
+            file('hello', 'w').write((str(i) * 4) + '\n')
+            if i == 0:
+                b.add(['hello'], ['hello-id'])
+            rev_id = 'test@rev-%d' % (i+1)
+            rev_ids.append(rev_id)
+            b.commit(message='rev %d' % (i+1),
+                     rev_id=rev_id)
+        eq = self.assertEquals
+        eq(b.revision_history(), rev_ids)
+        for i in range(4):
+            anc = b.get_ancestry(rev_ids[i])
+            eq(anc, [None] + rev_ids[:i+1])
+
+    def test_commit_new_subdir_child_selective(self):
+        b = Branch.initialize('.')
+        self.build_tree(['dir/', 'dir/file1', 'dir/file2'])
+        b.add(['dir', 'dir/file1', 'dir/file2'],
+              ['dirid', 'file1id', 'file2id'])
+        b.commit('dir/file1', specific_files=['dir/file1'], rev_id='1')
+        inv = b.get_inventory('1')
+        self.assertEqual('1', inv['dirid'].revision)
+        self.assertEqual('1', inv['file1id'].revision)
+        # FIXME: This should raise a KeyError I think, rbc20051006
+        self.assertRaises(BzrError, inv.__getitem__, 'file2id')

=== added file 'bzrlib/selftest/test_commit_merge.py'
--- bzrlib/selftest/test_commit_merge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_commit_merge.py	2005-09-30 14:27:41 +0000
@@ -0,0 +1,98 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+import shutil
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.errors import PointlessCommit, BzrError, PointlessCommit
+from bzrlib.selftest.testrevision import make_branches
+from bzrlib.fetch import fetch
+from bzrlib.check import check
+
+
+class TestCommitMerge(TestCaseInTempDir):
+    """Tests for committing the results of a merge.
+
+    These don't currently test the merge code, which is intentional to
+    reduce the scope of testing.  We just mark the revision as merged
+    without bothering about the contents much."""
+
+    def test_merge_commit_empty(self):
+        """Simple commit of two-way merge of empty trees."""
+        os.mkdir('x')
+        os.mkdir('y')
+        bx = Branch.initialize('x')
+        by = Branch.initialize('y')
+
+        commit(bx, 'commit one', rev_id='x@u-0-1', allow_pointless=True)
+        commit(by, 'commit two', rev_id='y@u-0-1', allow_pointless=True)
+
+        fetch(from_branch=bx, to_branch=by)
+        # just having the history there does nothing
+        self.assertRaises(PointlessCommit,
+                          commit,
+                          by, 'no changes yet', rev_id='y@u-0-2',
+                          allow_pointless=False)
+        by.add_pending_merge('x@u-0-1')
+        commit(by, 'merge from x', rev_id='y@u-0-2', allow_pointless=False)
+
+        self.assertEquals(by.revno(), 2)
+        self.assertEquals(list(by.revision_history()),
+                          ['y@u-0-1', 'y@u-0-2'])
+        rev = by.get_revision('y@u-0-2')
+        self.assertEquals(rev.parent_ids,
+                          ['y@u-0-1', 'x@u-0-1'])
+
+
+
+    def test_merge_new_file(self):
+        """Commit merge of two trees with no overlapping files."""
+        self.build_tree(['x/', 'x/ecks', 'y/', 'y/why'])
+
+        bx = Branch.initialize('x')
+        by = Branch.initialize('y')
+        bx.add(['ecks'], ['ecks-id'])
+        by.add(['why'], ['why-id'])
+
+        commit(bx, 'commit one', rev_id='x@u-0-1', allow_pointless=True)
+        commit(by, 'commit two', rev_id='y@u-0-1', allow_pointless=True)
+
+        fetch(from_branch=bx, to_branch=by)
+        # we haven't merged the texts, but let's fake it
+        shutil.copyfile('x/ecks', 'y/ecks')
+        by.add(['ecks'], ['ecks-id'])
+        by.add_pending_merge('x@u-0-1')
+
+        # partial commit of merges is currently not allowed, because
+        # it would give different merge graphs for each file which
+        # might be complex.  it can be allowed in the future.
+        self.assertRaises(Exception,
+                          commit,
+                          by, 'partial commit', allow_pointless=False,
+                          specific_files=['ecks'])
+        
+        commit(by, 'merge from x', rev_id='y@u-0-2', allow_pointless=False)
+        tree = by.revision_tree('y@u-0-2')
+        inv = tree.inventory
+        self.assertEquals(inv['ecks-id'].revision, 'x@u-0-1')
+        self.assertEquals(inv['why-id'].revision, 'y@u-0-1')
+
+        check(bx)
+        check(by)

=== added file 'bzrlib/selftest/test_conflicts.py'
--- bzrlib/selftest/test_conflicts.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_conflicts.py	2005-10-06 03:15:08 +0000
@@ -0,0 +1,44 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.commit import Commit
+
+
+# TODO: Test commit with some added, and added-but-missing files
+
+class TestConflicts(TestCaseInTempDir):
+
+    def test_conflicts(self):
+        """Conflicts are detected properly"""
+        b = Branch.initialize('.')
+        file('hello', 'w').write('hello world')
+        file('hello.BASE', 'w').write('hello world')
+        file('hello.sploo.BASE', 'w').write('yellow world')
+        tree = b.working_tree()
+        self.assertEqual(len(list(tree.list_files())), 3)
+        conflicts = list(tree.iter_conflicts())
+        self.assertEqual(len(conflicts), 2)
+        assert 'hello' in conflicts
+        assert 'hello.sploo' in conflicts
+        os.unlink('hello.BASE')
+        os.unlink('hello.sploo.BASE')
+        self.assertEqual(len(list(tree.iter_conflicts())), 0)
+

=== added file 'bzrlib/selftest/test_merge_core.py'
--- bzrlib/selftest/test_merge_core.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_merge_core.py	2005-10-03 15:19:25 +0000
@@ -0,0 +1,508 @@
+import os
+import shutil
+import tempfile
+import unittest
+
+from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.branch import ScratchBranch, Branch
+from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.inventory import RootEntry
+import bzrlib.inventory as inventory
+from bzrlib.osutils import file_kind, rename
+from bzrlib import changeset
+from bzrlib.merge_core import (ApplyMerge3, make_merge_changeset,
+                                BackupBeforeChange, PermissionsMerge)
+from bzrlib.changeset import Inventory, apply_changeset, invert_dict
+
+
+class FalseTree(object):
+    def __init__(self, realtree):
+        self._realtree = realtree
+        self.inventory = self
+
+    def __getitem__(self, file_id):
+        entry = self.make_inventory_entry(file_id)
+        if entry is None:
+            raise KeyError(file_id)
+        return entry
+        
+    def make_inventory_entry(self, file_id):
+        path = self._realtree.inventory.get(file_id)
+        if path is None:
+            return None
+        if path == "":
+            return RootEntry(file_id)
+        dir, name = os.path.split(path)
+        kind = file_kind(self._realtree.abs_path(path))
+        for parent_id, path in self._realtree.inventory.iteritems():
+            if path == dir:
+                break
+        if path != dir:
+            raise Exception("Can't find parent for %s" % name)
+        if kind not in ('directory', 'file', 'symlink'):
+            raise ValueError('unknown kind %r' % kind)
+        if kind == 'directory':
+            return inventory.InventoryDirectory(file_id, name, parent_id)
+        elif kind == 'file':
+            return inventory.InventoryFile(file_id, name, parent_id)
+        else:
+            return inventory.InventoryLink(file_id, name, parent_id)
+
+
+class MergeTree(object):
+    def __init__(self, dir):
+        self.dir = dir;
+        os.mkdir(dir)
+        self.inventory = {'0': ""}
+        self.tree = FalseTree(self)
+    
+    def child_path(self, parent, name):
+        return os.path.join(self.inventory[parent], name)
+
+    def add_file(self, id, parent, name, contents, mode):
+        path = self.child_path(parent, name)
+        full_path = self.abs_path(path)
+        assert not os.path.exists(full_path)
+        file(full_path, "wb").write(contents)
+        os.chmod(self.abs_path(path), mode)
+        self.inventory[id] = path
+
+    def remove_file(self, id):
+        os.unlink(self.full_path(id))
+        del self.inventory[id]
+
+    def add_dir(self, id, parent, name, mode):
+        path = self.child_path(parent, name)
+        full_path = self.abs_path(path)
+        assert not os.path.exists(full_path)
+        os.mkdir(self.abs_path(path))
+        os.chmod(self.abs_path(path), mode)
+        self.inventory[id] = path
+
+    def abs_path(self, path):
+        return os.path.join(self.dir, path)
+
+    def full_path(self, id):
+        try:
+            tree_path = self.inventory[id]
+        except KeyError:
+            return None
+        return self.abs_path(tree_path)
+
+    def readonly_path(self, id):
+        return self.full_path(id)
+
+    def __contains__(self, file_id):
+        return file_id in self.inventory
+
+    def has_or_had_id(self, file_id):
+        return file_id in self
+
+    def get_file(self, file_id):
+        path = self.readonly_path(file_id)
+        return file(path, "rb")
+
+    def id2path(self, file_id):
+        return self.inventory[file_id]
+
+    def change_path(self, id, path):
+        old_path = os.path.join(self.dir, self.inventory[id])
+        rename(old_path, self.abs_path(path))
+        self.inventory[id] = path
+
+
+class MergeBuilder(object):
+    def __init__(self):
+        self.dir = tempfile.mkdtemp(prefix="BaZing")
+        self.base = MergeTree(os.path.join(self.dir, "base"))
+        self.this = MergeTree(os.path.join(self.dir, "this"))
+        self.other = MergeTree(os.path.join(self.dir, "other"))
+        
+        self.cset = changeset.Changeset()
+        self.cset.add_entry(changeset.ChangesetEntry("0", 
+                                                     changeset.NULL_ID, "./."))
+    def get_cset_path(self, parent, name):
+        if name is None:
+            assert (parent is None)
+            return None
+        return os.path.join(self.cset.entries[parent].path, name)
+
+    def add_file(self, id, parent, name, contents, mode):
+        self.base.add_file(id, parent, name, contents, mode)
+        self.this.add_file(id, parent, name, contents, mode)
+        self.other.add_file(id, parent, name, contents, mode)
+        path = self.get_cset_path(parent, name)
+        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
+
+    def remove_file(self, id, base=False, this=False, other=False):
+        for option, tree in ((base, self.base), (this, self.this), 
+                             (other, self.other)):
+            if option:
+                tree.remove_file(id)
+            if other or base:
+                change = self.cset.entries[id].contents_change
+                if change is None:
+                    change = changeset.ReplaceContents(None, None)
+                    self.cset.entries[id].contents_change = change
+                    def create_file(tree):
+                        return changeset.FileCreate(tree.get_file(id).read())
+                    if not other:
+                        change.new_contents = create_file(self.other)
+                    if not base:
+                        change.old_contents = create_file(self.base)
+                else:
+                    assert isinstance(change, changeset.ReplaceContents)
+                if other:
+                    change.new_contents=None
+                if base:
+                    change.old_contents=None
+                if change.old_contents is None and change.new_contents is None:
+                    change = None
+
+
+    def add_dir(self, id, parent, name, mode):
+        path = self.get_cset_path(parent, name)
+        self.base.add_dir(id, parent, name, mode)
+        self.cset.add_entry(changeset.ChangesetEntry(id, parent, path))
+        self.this.add_dir(id, parent, name, mode)
+        self.other.add_dir(id, parent, name, mode)
+
+
+    def change_name(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_name_tree(id, self.base, base)
+            self.cset.entries[id].name = base
+
+        if this is not None:
+            self.change_name_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_name_tree(id, self.other, other)
+            self.cset.entries[id].new_name = other
+
+    def change_parent(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_parent_tree(id, self.base, base)
+            self.cset.entries[id].parent = base
+            self.cset.entries[id].dir = self.cset.entries[base].path
+
+        if this is not None:
+            self.change_parent_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_parent_tree(id, self.other, other)
+            self.cset.entries[id].new_parent = other
+            self.cset.entries[id].new_dir = \
+                self.cset.entries[other].new_path
+
+    def change_contents(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_contents_tree(id, self.base, base)
+
+        if this is not None:
+            self.change_contents_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_contents_tree(id, self.other, other)
+
+        if base is not None or other is not None:
+            old_contents = file(self.base.full_path(id)).read()
+            new_contents = file(self.other.full_path(id)).read()
+            contents = changeset.ReplaceFileContents(old_contents, 
+                                                     new_contents)
+            self.cset.entries[id].contents_change = contents
+
+    def change_perms(self, id, base=None, this=None, other=None):
+        if base is not None:
+            self.change_perms_tree(id, self.base, base)
+
+        if this is not None:
+            self.change_perms_tree(id, self.this, this)
+
+        if other is not None:
+            self.change_perms_tree(id, self.other, other)
+
+        if base is not None or other is not None:
+            old_perms = os.stat(self.base.full_path(id)).st_mode &077
+            new_perms = os.stat(self.other.full_path(id)).st_mode &077
+            contents = changeset.ChangeUnixPermissions(old_perms, 
+                                                       new_perms)
+            self.cset.entries[id].metadata_change = contents
+
+    def change_name_tree(self, id, tree, name):
+        new_path = tree.child_path(self.cset.entries[id].parent, name)
+        tree.change_path(id, new_path)
+
+    def change_parent_tree(self, id, tree, parent):
+        new_path = tree.child_path(parent, self.cset.entries[id].name)
+        tree.change_path(id, new_path)
+
+    def change_contents_tree(self, id, tree, contents):
+        path = tree.full_path(id)
+        mode = os.stat(path).st_mode
+        file(path, "w").write(contents)
+        os.chmod(path, mode)
+
+    def change_perms_tree(self, id, tree, mode):
+        os.chmod(tree.full_path(id), mode)
+
+    def merge_changeset(self, merge_factory):
+        conflict_handler = changeset.ExceptionConflictHandler()
+        return make_merge_changeset(self.cset, self.this, self.base,
+                                    self.other, conflict_handler,
+                                    merge_factory)
+
+    def apply_inv_change(self, inventory_change, orig_inventory):
+        orig_inventory_by_path = {}
+        for file_id, path in orig_inventory.iteritems():
+            orig_inventory_by_path[path] = file_id
+
+        def parent_id(file_id):
+            try:
+                parent_dir = os.path.dirname(orig_inventory[file_id])
+            except:
+                print file_id
+                raise
+            if parent_dir == "":
+                return None
+            return orig_inventory_by_path[parent_dir]
+        
+        def new_path(file_id):
+            if inventory_change.has_key(file_id):
+                return inventory_change[file_id]
+            else:
+                parent = parent_id(file_id)
+                if parent is None:
+                    return orig_inventory[file_id]
+                dirname = new_path(parent)
+                return os.path.join(dirname, orig_inventory[file_id])
+
+        new_inventory = {}
+        for file_id in orig_inventory.iterkeys():
+            path = new_path(file_id)
+            if path is None:
+                continue
+            new_inventory[file_id] = path
+
+        for file_id, path in inventory_change.iteritems():
+            if orig_inventory.has_key(file_id):
+                continue
+            new_inventory[file_id] = path
+        return new_inventory
+
+    def apply_changeset(self, cset, conflict_handler=None, reverse=False):
+        inventory_change = changeset.apply_changeset(cset,
+                                                     self.this.inventory,
+                                                     self.this.dir,
+                                                     conflict_handler, reverse)
+        self.this.inventory =  self.apply_inv_change(inventory_change, 
+                                                     self.this.inventory)
+
+    def cleanup(self):
+        shutil.rmtree(self.dir)
+
+class MergeTest(unittest.TestCase):
+    def test_change_name(self):
+        """Test renames"""
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "hello1", 0755)
+        builder.change_name("1", other="name2")
+        builder.add_file("2", "0", "name3", "hello2", 0755)
+        builder.change_name("2", base="name4")
+        builder.add_file("3", "0", "name5", "hello3", 0755)
+        builder.change_name("3", this="name6")
+        cset = builder.merge_changeset(ApplyMerge3)
+        assert(cset.entries["2"].is_boring())
+        assert(cset.entries["1"].name == "name1")
+        assert(cset.entries["1"].new_name == "name2")
+        assert(cset.entries["3"].is_boring())
+        for tree in (builder.this, builder.other, builder.base):
+            assert(tree.dir != builder.dir and 
+                   tree.dir.startswith(builder.dir))
+            for path in tree.inventory.itervalues():
+                fullpath = tree.abs_path(path)
+                assert(fullpath.startswith(tree.dir))
+                assert(not path.startswith(tree.dir))
+                assert os.path.exists(fullpath)
+        builder.apply_changeset(cset)
+        builder.cleanup()
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "hello1", 0644)
+        builder.change_name("1", other="name2", this="name3")
+        self.assertRaises(changeset.RenameConflict, 
+                          builder.merge_changeset, ApplyMerge3)
+        builder.cleanup()
+        
+    def test_file_moves(self):
+        """Test moves"""
+        builder = MergeBuilder()
+        builder.add_dir("1", "0", "dir1", 0755)
+        builder.add_dir("2", "0", "dir2", 0755)
+        builder.add_file("3", "1", "file1", "hello1", 0644)
+        builder.add_file("4", "1", "file2", "hello2", 0644)
+        builder.add_file("5", "1", "file3", "hello3", 0644)
+        builder.change_parent("3", other="2")
+        assert(Inventory(builder.other.inventory).get_parent("3") == "2")
+        builder.change_parent("4", this="2")
+        assert(Inventory(builder.this.inventory).get_parent("4") == "2")
+        builder.change_parent("5", base="2")
+        assert(Inventory(builder.base.inventory).get_parent("5") == "2")
+        cset = builder.merge_changeset(ApplyMerge3)
+        for id in ("1", "2", "4", "5"):
+            assert(cset.entries[id].is_boring())
+        assert(cset.entries["3"].parent == "1")
+        assert(cset.entries["3"].new_parent == "2")
+        builder.apply_changeset(cset)
+        builder.cleanup()
+
+        builder = MergeBuilder()
+        builder.add_dir("1", "0", "dir1", 0755)
+        builder.add_dir("2", "0", "dir2", 0755)
+        builder.add_dir("3", "0", "dir3", 0755)
+        builder.add_file("4", "1", "file1", "hello1", 0644)
+        builder.change_parent("4", other="2", this="3")
+        self.assertRaises(changeset.MoveConflict, 
+                          builder.merge_changeset, ApplyMerge3)
+        builder.cleanup()
+
+    def test_contents_merge(self):
+        """Test merge3 merging"""
+        self.do_contents_test(ApplyMerge3)
+
+    def test_contents_merge2(self):
+        """Test diff3 merging"""
+        self.do_contents_test(changeset.Diff3Merge)
+
+    def test_contents_merge3(self):
+        """Test diff3 merging"""
+        def backup_merge(file_id, base, other):
+            return BackupBeforeChange(ApplyMerge3(file_id, base, other))
+        builder = self.contents_test_success(backup_merge)
+        def backup_exists(file_id):
+            return os.path.exists(builder.this.full_path(file_id)+"~")
+        assert backup_exists("1")
+        assert backup_exists("2")
+        assert not backup_exists("3")
+        builder.cleanup()
+
+    def do_contents_test(self, merge_factory):
+        """Test merging with specified ContentsChange factory"""
+        builder = self.contents_test_success(merge_factory)
+        builder.cleanup()
+        self.contents_test_conflicts(merge_factory)
+
+    def contents_test_success(self, merge_factory):
+        from inspect import isclass
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4")
+        builder.add_file("2", "0", "name3", "text2", 0655)
+        builder.change_contents("2", base="text5")
+        builder.add_file("3", "0", "name5", "text3", 0744)
+        builder.add_file("4", "0", "name6", "text4", 0744)
+        builder.remove_file("4", base=True)
+        assert not builder.cset.entries["4"].is_boring()
+        builder.change_contents("3", this="text6")
+        cset = builder.merge_changeset(merge_factory)
+        assert(cset.entries["1"].contents_change is not None)
+        if isclass(merge_factory):
+            assert(isinstance(cset.entries["1"].contents_change,
+                          merge_factory))
+            assert(isinstance(cset.entries["2"].contents_change,
+                          merge_factory))
+        assert(cset.entries["3"].is_boring())
+        assert(cset.entries["4"].is_boring())
+        builder.apply_changeset(cset)
+        assert(file(builder.this.full_path("1"), "rb").read() == "text4" )
+        assert(file(builder.this.full_path("2"), "rb").read() == "text2" )
+        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0755)
+        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0655)
+        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0744)
+        return builder
+
+    def contents_test_conflicts(self, merge_factory):
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4", this="text3")
+        cset = builder.merge_changeset(merge_factory)
+        self.assertRaises(changeset.MergeConflict, builder.apply_changeset,
+                          cset)
+        builder.cleanup()
+
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4", base="text3")
+        builder.remove_file("1", base=True)
+        self.assertRaises(changeset.NewContentsConflict,
+                          builder.merge_changeset, merge_factory)
+        builder.cleanup()
+
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_contents("1", other="text4", base="text3")
+        builder.remove_file("1", this=True)
+        self.assertRaises(changeset.MissingForMerge, builder.merge_changeset, 
+                          merge_factory)
+        builder.cleanup()
+
+    def test_perms_merge(self):
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_perms("1", other=0655)
+        builder.add_file("2", "0", "name2", "text2", 0755)
+        builder.change_perms("2", base=0655)
+        builder.add_file("3", "0", "name3", "text3", 0755)
+        builder.change_perms("3", this=0655)
+        cset = builder.merge_changeset(ApplyMerge3)
+        assert(cset.entries["1"].metadata_change is not None)
+        assert(isinstance(cset.entries["1"].metadata_change,
+                          PermissionsMerge))
+        assert(isinstance(cset.entries["2"].metadata_change,
+                          PermissionsMerge))
+        assert(cset.entries["3"].is_boring())
+        builder.apply_changeset(cset)
+        assert(os.stat(builder.this.full_path("1")).st_mode &0777 == 0655)
+        assert(os.stat(builder.this.full_path("2")).st_mode &0777 == 0755)
+        assert(os.stat(builder.this.full_path("3")).st_mode &0777 == 0655)
+        builder.cleanup();
+        builder = MergeBuilder()
+        builder.add_file("1", "0", "name1", "text1", 0755)
+        builder.change_perms("1", other=0655, base=0555)
+        cset = builder.merge_changeset(ApplyMerge3)
+        self.assertRaises(changeset.MergePermissionConflict, 
+                     builder.apply_changeset, cset)
+        builder.cleanup()
+
+class FunctionalMergeTest(TestCaseInTempDir):
+
+    def test_trivial_star_merge(self):
+        """Test that merges in a star shape Just Work.""" 
+        from bzrlib.add import smart_add_branch, add_reporter_null
+        from bzrlib.clone import copy_branch
+        from bzrlib.merge import merge
+        # John starts a branch
+        self.build_tree(("original/", "original/file1", "original/file2"))
+        branch = Branch.initialize("original")
+        smart_add_branch(branch, ["original"], True, add_reporter_null)
+        branch.commit("start branch.", verbose=False)
+        # Mary branches it.
+        self.build_tree(("mary/",))
+        copy_branch(branch, "mary")
+        # Now John commits a change
+        file = open("original/file1", "wt")
+        file.write("John\n")
+        file.close()
+        branch.commit("change file1")
+        # Mary does too
+        mary_branch = Branch.open("mary")
+        file = open("mary/file2", "wt")
+        file.write("Mary\n")
+        file.close()
+        mary_branch.commit("change file2")
+        # john should be able to merge with no conflicts.
+        merge_type = ApplyMerge3
+        base = [None, None]
+        other = ("mary", -1)
+        merge(other, base, check_clean=True, merge_type=merge_type, this_dir="original")
+        self.assertEqual("John\n", open("original/file1", "rt").read())
+        self.assertEqual("Mary\n", open("original/file2", "rt").read())

=== added file 'bzrlib/selftest/test_parent.py'
--- bzrlib/selftest/test_parent.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_parent.py	2005-09-29 20:34:25 +0000
@@ -0,0 +1,58 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+
+
+"""Tests for Branch parent URL"""
+
+
+class TestParent(TestCaseInTempDir):
+    def test_no_default_parent(self):
+        """Branches should have no parent by default"""
+        b = Branch.initialize('.')
+        self.assertEquals(b.get_parent(), None)
+        
+    
+    def test_set_get_parent(self):
+        """Set and then re-get the parent"""
+        b = Branch.initialize('.')
+        url = 'http://bazaar-ng.org/bzr/bzr.dev'
+        b.set_parent(url)
+        self.assertEquals(b.get_parent(), url)
+
+    def test_branch_sets_parent(self):
+        """The branch command should set the new branch's parent"""
+        from bzrlib.commands import run_bzr
+
+        os.mkdir('from')
+        branch_from = Branch.initialize('from')
+        file('from/foo', 'wt').write('contents of foo')
+        branch_from.add('foo')
+        branch_from.commit('initial commit')
+        
+        os.mkdir('to')
+        copy_branch(branch_from, 'to', None)
+
+        branch_to = Branch.open('to')
+        abspath = os.path.abspath('from')
+        self.assertEquals(branch_to.get_parent(), abspath)
+        
+

=== added file 'bzrlib/selftest/test_revision_info.py'
--- bzrlib/selftest/test_revision_info.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_revision_info.py	2005-09-19 06:05:19 +0000
@@ -0,0 +1,110 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import BzrCommandError, NoSuchRevision
+from bzrlib.branch import Branch
+from bzrlib.revisionspec import RevisionSpec
+
+class TestRevisionInfo(TestCaseInTempDir):
+    
+    def check_error(self, output, *args):
+        """Verify that the expected error matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertEquals(self.run_bzr_captured(args, retcode=1)[1], output)
+
+    def check_output(self, output, *args):
+        """Verify that the expected output matches what bzr says.
+        
+        The output is supplied first, so that you can supply a variable
+        number of arguments to bzr.
+        """
+        self.assertEquals(self.run_bzr_captured(args)[0], output)
+
+    def test_revision_info(self):
+        """Test that 'bzr revision-info' reports the correct thing.
+        """
+
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        # Make sure revision-info without any arguments throws an exception
+        self.check_error('bzr: ERROR: You must supply a revision identifier\n',
+                         'revision-info')
+
+        values = {
+            1:'   1 a@r-0-1\n',
+            2:'   2 a@r-0-2\n',
+            3:'   3 a@r-0-3\n'
+        }
+
+        # Check the results of just specifying a numeric revision
+        self.check_output(values[1], 'revision-info', '1')
+        self.check_output(values[2], 'revision-info', '2')
+        self.check_output(values[3], 'revision-info', '3')
+        self.check_output(values[1]+values[2], 'revision-info', '1', '2')
+        self.check_output(values[1]+values[2]+values[3], 'revision-info', '1', '2', '3')
+        self.check_output(values[2]+values[1], 'revision-info', '2', '1')
+
+        # Check as above, only using the '--revision' syntax
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', '1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', '2')
+        self.check_output('   3 a@r-0-3\n', 'revision-info', '-r', '3')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n', 'revision-info', '-r', '1..2')
+        self.check_output('   1 a@r-0-1\n   2 a@r-0-2\n   3 a@r-0-3\n'
+                , 'revision-info', '-r', '1..2..3')
+        self.check_output('   2 a@r-0-2\n   1 a@r-0-1\n', 'revision-info', '-r', '2..1')
+
+        # Now try some more advanced revision specifications
+        
+        self.check_output('   1 a@r-0-1\n', 'revision-info', '-r', 'revid:a@r-0-1')
+        self.check_output('   2 a@r-0-2\n', 'revision-info', '--revision', 'revid:a@r-0-2')
+
+    def test_cat_revision(self):
+        """Test bzr cat-revision.
+        """
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1')
+        b.commit('Commit two', rev_id='a@r-0-2')
+        b.commit('Commit three', rev_id='a@r-0-3')
+
+        revs = {
+            1:b.get_revision_xml_file('a@r-0-1').read(),
+            2:b.get_revision_xml_file('a@r-0-2').read(),
+            3:b.get_revision_xml_file('a@r-0-3').read()
+        }
+
+        self.check_output(revs[1], 'cat-revision', 'a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', 'a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', 'a@r-0-3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', '1')
+        self.check_output(revs[2], 'cat-revision', '-r', '2')
+        self.check_output(revs[3], 'cat-revision', '-r', '3')
+
+        self.check_output(revs[1], 'cat-revision', '-r', 'revid:a@r-0-1')
+        self.check_output(revs[2], 'cat-revision', '-r', 'revid:a@r-0-2')
+        self.check_output(revs[3], 'cat-revision', '-r', 'revid:a@r-0-3')
+

=== added file 'bzrlib/selftest/test_smart_add.py'
--- bzrlib/selftest/test_smart_add.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_smart_add.py	2005-10-03 15:19:25 +0000
@@ -0,0 +1,160 @@
+import os
+import unittest
+
+from bzrlib.selftest import TestCaseInTempDir, TestCase
+from bzrlib.branch import Branch
+from bzrlib.errors import NotBranchError, NotVersionedError
+from bzrlib.inventory import InventoryFile
+
+class TestSmartAdd(TestCaseInTempDir):
+
+    def test_add_dot_from_root(self):
+        """Test adding . from the root of the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch.initialize(".")
+        smart_add((".",), recurse=True)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_dot_from_subdir(self):
+        """Test adding . from a subdir of the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch.initialize(".")
+        os.chdir("original")
+        smart_add((".",), recurse=True)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_tree_from_above_tree(self):
+        """Test adding a tree from above the tree.""" 
+        from bzrlib.add import smart_add
+        paths = ("original/", "original/file1", "original/file2")
+        branch_paths = ("branch/", "branch/original/", "branch/original/file1",
+                        "branch/original/file2")
+        self.build_tree(branch_paths)
+        branch = Branch.initialize("branch")
+        smart_add(("branch",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_above_tree_preserves_tree(self):
+        """Test nested trees are not affect by an add above them."""
+        from bzrlib.add import smart_add, add_reporter_null
+        
+        paths = ("original/", "original/file1", "original/file2")
+        child_paths = ("path",)
+        full_child_paths = ("original/child", "original/child/path")
+        build_paths = ("original/", "original/file1", "original/file2", 
+                       "original/child/", "original/child/path")
+        
+        self.build_tree(build_paths)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
+        smart_add((".",), True, add_reporter_null)
+        for path in paths:
+            self.assertNotEqual((path, branch.inventory.path2id(path)),
+                                (path, None))
+        for path in full_child_paths:
+            self.assertEqual((path, branch.inventory.path2id(path)),
+                             (path, None))
+        for path in child_paths:
+            self.assertEqual(child_branch.inventory.path2id(path), None)
+
+    def test_add_paths(self):
+        """Test smart-adding a list of paths."""
+        from bzrlib.add import smart_add
+        paths = ("file1", "file2")
+        self.build_tree(paths)
+        branch = Branch.initialize(".")
+        smart_add(paths)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+            
+class TestSmartAddBranch(TestCaseInTempDir):
+    """Test smart adds with a specified branch."""
+
+    def test_add_dot_from_root(self):
+        """Test adding . from the root of the tree.""" 
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch.initialize(".")
+        smart_add_branch(branch, (".",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_dot_from_subdir(self):
+        """Test adding . from a subdir of the tree.""" 
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        self.build_tree(paths)
+        branch = Branch.initialize(".")
+        os.chdir("original")
+        smart_add_branch(branch, (".",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_tree_from_above_tree(self):
+        """Test adding a tree from above the tree.""" 
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        branch_paths = ("branch/", "branch/original/", "branch/original/file1",
+                        "branch/original/file2")
+        self.build_tree(branch_paths)
+        branch = Branch.initialize("branch")
+        smart_add_branch(branch, ("branch",))
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+    def test_add_above_tree_preserves_tree(self):
+        """Test nested trees are not affect by an add above them."""
+        from bzrlib.add import smart_add_branch
+        paths = ("original/", "original/file1", "original/file2")
+        child_paths = ("path")
+        full_child_paths = ("original/child", "original/child/path")
+        build_paths = ("original/", "original/file1", "original/file2", 
+                       "original/child/", "original/child/path")
+        self.build_tree(build_paths)
+        branch = Branch.initialize(".")
+        child_branch = Branch.initialize("original/child")
+        smart_add_branch(branch, (".",))
+        for path in paths:
+            self.assertNotEqual((path, branch.inventory.path2id(path)),
+                                (path, None))
+        for path in full_child_paths:
+            self.assertEqual((path, branch.inventory.path2id(path)),
+                             (path, None))
+        for path in child_paths:
+            self.assertEqual(child_branch.inventory.path2id(path), None)
+
+    def test_add_paths(self):
+        """Test smart-adding a list of paths."""
+        from bzrlib.add import smart_add_branch
+        paths = ("file1", "file2")
+        self.build_tree(paths)
+        branch = Branch.initialize(".")
+        smart_add_branch(branch, paths)
+        for path in paths:
+            self.assertNotEqual(branch.inventory.path2id(path), None)
+
+class TestAddCallbacks(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestAddCallbacks, self).setUp()
+        self.entry = InventoryFile("id", "name", None)
+
+    def test_null_callback(self):
+        from bzrlib.add import add_reporter_null
+        add_reporter_null('path', 'file', self.entry)
+
+    def test_print_callback(self):
+        from bzrlib.add import add_reporter_print
+        from StringIO import StringIO
+        stdout = StringIO()
+        self.apply_redirected(None, stdout, None, add_reporter_print,
+                              'path', 'file', self.entry)
+        self.assertEqual(stdout.getvalue(), "added path\n")

=== added file 'bzrlib/selftest/test_upgrade.py'
--- bzrlib/selftest/test_upgrade.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_upgrade.py	2005-10-06 11:26:51 +0000
@@ -0,0 +1,240 @@
+# Copyright (C) 2005 by Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for upgrade of old trees.
+
+This file contains canned versions of some old trees, which are instantiated 
+and then upgraded to the new format."""
+
+import base64
+import os
+import sys
+
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.revision import is_ancestor
+from bzrlib.upgrade import upgrade
+from bzrlib.selftest.treeshape import build_tree_contents
+
+
+class TestUpgrade(TestCaseInTempDir):
+    
+    def test_build_tree(self):
+        """Test tree-building test helper"""
+        build_tree_contents(_upgrade1_template)
+        self.failUnlessExists('foo')
+        self.failUnlessExists('.bzr/README')
+
+    def test_upgrade_simple(self):
+        """Upgrade simple v0.0.4 format to v5"""
+        eq = self.assertEquals
+        build_tree_contents(_upgrade1_template)
+        upgrade('.')
+        b = Branch.open('.')
+        eq(b._branch_format, 5)
+        rh = b.revision_history()
+        eq(rh,
+           ['mbp@sourcefrog.net-20051004035611-176b16534b086b3c',
+            'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd'])
+        t = b.revision_tree(rh[0])
+        foo_id = 'foo-20051004035605-91e788d1875603ae'
+        eq(t.get_file_text(foo_id), 'initial contents\n')
+        t = b.revision_tree(rh[1])
+        eq(t.get_file_text(foo_id), 'new contents\n')
+
+    def test_upgrade_with_ghosts(self):
+        """Upgrade v0.0.4 tree containing ghost references.
+
+        That is, some of the parents of revisions mentioned in the branch
+        aren't present in the branches storage. 
+
+        This shouldn't normally happen in branches created entirely in 
+        bzr but can happen in imports from baz and arch, or from other  
+        systems, where the importer knows about a revision but not 
+        its contents."""
+        eq = self.assertEquals
+        build_tree_contents(_ghost_template)
+        upgrade('.')
+        b = Branch.open('.')
+        revision_id = b.revision_history()[1]
+        rev = b.get_revision(revision_id)
+        eq(len(rev.parent_ids), 2)
+        eq(rev.parent_ids[1], 'wibble@wobble-2')
+
+    def test_upgrade_makes_dir_weaves(self):
+        build_tree_contents(_upgrade_dir_template)
+        upgrade('.')
+        self.failUnlessExists(
+            '.bzr/weaves/dir-20051005095101-da1441ea3fa6917a.weave')
+
+
+_upgrade1_template = \
+     [
+     ('foo', 'new contents\n'),
+     ('.bzr/',),
+     ('.bzr/README',
+      'This is a Bazaar-NG control directory.\nDo not change any files in this directory.\n'),
+     ('.bzr/branch-format', 'Bazaar-NG branch, format 0.0.4\n'),
+     ('.bzr/revision-history',
+      'mbp@sourcefrog.net-20051004035611-176b16534b086b3c\n'
+      'mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd\n'),
+     ('.bzr/merged-patches', ''),
+     ('.bzr/pending-merged-patches', ''),
+     ('.bzr/branch-name', ''),
+     ('.bzr/branch-lock', ''),
+     ('.bzr/pending-merges', ''),
+     ('.bzr/inventory',
+      '<inventory>\n'
+      '<entry file_id="foo-20051004035605-91e788d1875603ae" kind="file" name="foo" />\n'
+      '</inventory>\n'),
+     ('.bzr/stat-cache',
+      '### bzr hashcache v5\n'
+      'foo// be9f309239729f69a6309e970ef24941d31e042c 13 1128398176 1128398176 303464 770\n'),
+     ('.bzr/text-store/',),
+     ('.bzr/text-store/foo-20051004035611-1591048e9dc7c2d4.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\xcb\xcc\xcb,\xc9L\xccQH\xce\xcf+I\xcd+)\xe6\x02\x00\xdd\xcc\xf90\x11\x00\x00\x00'),
+     ('.bzr/text-store/foo-20051004035756-4081373d897c3453.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\xcbK-WH\xce\xcf+I\xcd+)\xe6\x02\x00g\xc3\xdf\xc9\r\x00\x00\x00'),
+     ('.bzr/inventory-store/',),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xffm\x8f\xcd\n\xc20\x10\x84\xef>E\xc8\xbdt7?M\x02\xad\xaf"\xa1\x99`P[\xa8E\xacOo\x14\x05\x0f\xdef\xe1\xfbv\x98\xbeL7L\xeb\xbcl\xfb]_\xc3\xb2\x89\\\xce8\x944\xc8<\xcf\x8d"\xb2LdH\xdb\x8el\x13\x18\xce\xfb\xc4\xde\xd5SGHq*\xd3\x0b\xad\x8e\x14S\xbc\xe0\xadI\xb1\xe2\xbe\xfe}\xc2\xdc\xb0\rL\xc6#\xa4\xd1\x8d*\x99\x0f}=F\x1e$8G\x9d\xa0\x02\xa1rP9\x01c`FV\xda1qg\x98"\x02}\xa5\xf2\xa8\x95\xec\xa4h\xeb\x80\xf6g\xcd\x13\xb3\x01\xcc\x98\xda\x00\x00\x00'),
+     ('.bzr/inventory-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xffm\x8f\xc1\n\xc20\x10D\xef~E\xc8\xbd\xb8\x9bM\x9a,\xb4\xfe\x8a\xc4f\x83Am\xa1\x16\xb1~\xbdQ\x14<x\x9b\x81y3LW\xc6\x9b\x8c\xcb4\xaf\xbbMW\xc5\xbc\xaa\\\xce\xb2/\xa9\xd7y\x9a\x1a\x03\xe0\x10\xc0\x02\xb9\x16\\\xc3(>\x84\x84\xc1WKQ\xb4:\x95\xf1\x15\xad\x8cVc\xbc\xc8\x1b\xd3j\x91\xfb\xf2\xaf\xa4r\x8d\x85\x80\xe4)\x05\xf6\x03YG\x9f\xf4\xf5\x18\xb1\xd7\x07\xe1L\xc0\x86\xd8\x1b\xce-\xc7\xb6:a\x0f\x92\x8de\x8b\x89P\xc0\x9a\xe1\x0b\x95G\x9d\xc4\xda\xb1\xad\x07\xb6?o\x9e\xb5\xff\xf0\xf9\xda\x00\x00\x00'),
+     ('.bzr/revision-store/',),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035611-176b16534b086b3c.gz',
+      '\x1f\x8b\x08\x00[\xfdAC\x02\xff\x9d\x8eKj\xc30\x14E\xe7^\x85\xd0 \xb3$\xefI\xd1\x8f\xd8\xa6\x1b(t\x07E?\xbb\x82H\n\xb2\x1ahW\xdfB1\x14:\xeb\xf4r\xee\xbdgl\xf1\x91\xb6T\x0b\xf15\xe7\xd4{l\x13}\xb6\xad\xa7B^j\xbd\x91\xc3\xad_\xb3\xbb?m\xf5\xbd\xf9\xb8\xb4\xba\x9eJ\xec\x87\xb5_)I\xe5\x11K\xaf\xed\xe35\x85\x89\xfe\xa5\x8e\x0c@ \xc0\x05\xb8\x90\x88GT\xd2\xa1\x14\xfc\xe2@K\xc7\xfd\xef\x85\xed\xcd\xe2D\x95\x8d\x1a\xa47<\x02c2\xb0 \xbc\xd0\x8ay\xa3\xbcp\x8a\x83\x12A3\xb7XJv\xef\x7f_\xf7\x94\xe3\xd6m\xbeO\x14\x91in4*<\x812\x88\xc60\xfc\x01>k\x89\x13\xe5\x12\x00\xe8<\x8c\xdf\x8d\xcd\xaeq\xb6!\x90\xa5\xd6\xf1\xbc\x07\xc3x\xde\x85\xe6\xe1\x0b\xc8\x8a\x98\x03T\x01\x00\x00'),
+     ('.bzr/revision-store/mbp@sourcefrog.net-20051004035756-235f2b7dcdddd8dd.gz',
+      '\x1f\x8b\x08\x00\xc4\xfdAC\x02\xff\x9d\x90Kj\x031\x0c\x86\xf79\xc5\xe0Ev\xe9\xc8o\x9b\xcc\x84^\xa0\xd0\x1b\x14\xbf&5d\xec`\xbb\x81\xf6\xf45\x84\xa4\x81\xaeZ\xa1\x85\x84^\xdf\xaf\xa9\x84K\xac1\xa7\xc1\xe5u\x8d\xad\x852\xa3\x17SZL\xc3k\xce\xa7a{j\xfb\xd5\x9e\x9fk\xfe(.,%\x1f\x9fRh\xdbc\xdb\xa3!\xa6KH-\x97\xcf\xb7\xe8g\xf4\xbbkG\x008\x06`@\xb9\xe4bG(_\x88\x95\xde\xf9n\xca\xfb\xc7\r\xf5\xdd\xe0\x19\xa9\x85)\x81\xf5"\xbd\x04j\xb8\x02b\xa8W\\\x0b\xc9\x14\xf4\xbc\xbb\xd7\xd6H4\xdc\xb8\xff}\xba\xc55\xd4f\xd6\xf3\x8c0&\x8ajE\xa4x\xe2@\xa5\xa6\x9a\xf3k\xc3WNaFT\x00\x00:l\xa6>Q\xcd1\x1cjp9\xf9;\xc34\xde\n\x9b\xe9lJWT{t\',a\xf9\x0b\xae\xc0x\x87\xa5\xb0Xp\xca,(a\xa9{\xd0{}\xd4\x12\x04(\xc5\xbb$\xc5$V\xceaI\x19\x01\xa2\x1dh\xed\x82d\x8c.\xccr@\xc3\xd8Q\xc6\x1f\xaa\xf1\xb6\xe8\xb0\xf9\x06QR\r\xf9\xfc\x01\x00\x00')]
+
+
+_ghost_template = [
+    ( './foo',
+        'hello\n'
+    ),
+    ( './.bzr/', ),
+    ( './.bzr/README',
+        'This is a Bazaar-NG control directory.\n'
+        'Do not change any files in this directory.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="foo-20051004104918-0379cb7c76354cde" kind="file" name="foo" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\n'
+        'mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+        'foo// f572d396fae9206628714fb2ce00f72e94f2258f 6 1128422956 1128422956 306900 770\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/text-store/foo-20051004104921-8de8118a71be45ba.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03foo-20051004104921-8de8118a71be45ba\x00\xcbH\xcd\xc9\xc9\xe7\x02\x00 0:6\x06\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/inventory-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00m\x8f\xcb\n'
+        '\xc20\x10E\xf7~E\xc8\xbe83\xcd\x13\xaa\xbf"yL0\xa8-\xd4"\xd6\xaf7\x8a\x82\x0bw\xb38\xe7\xde;C\x1do<.\xd3\xbc\xee7C;\xe6U\x94z\xe6C\xcd;Y\xa6\xa9#\x00\x8d\x00\n'
+        'Ayt\x1d\xf4\xd6\xa7h\x935\xbdV)\xb3\x14\xa7:\xbe\xd0\xe6H1\x86\x0b\xbf5)\x16\xbe/\x7fC\x08;\x97\xd9!\xba`1\xb2\xd21|\xe8\xeb1`\xe3\xb5\xa5\xdc{S\x02{\x02c\xc8YT%Rb\x80b\x89\xbd*D\xda\x95\xafT\x1f\xad\xd2H\xb1m\xfb\xb7?\xcf<\x01W}\xb5\x8b\xd9\x00\x00\x00'
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b.gz',
+        '\x1f\x8b\x08\x081^BC\x00\x03mbp@sourcefrog.net-20051004104921-a98be2278dd30b7b\x00\x9d\x8eMj\xc30\x14\x84\xf7>\x85\xd0"\xbb$\xef\xc9\xb6,\x11\xdb\xf4\x02\x85\xde\xa0\xe8\xe7\xd9\x11\xc4R\x90\xd4@{\xfa\x06\x8a\xa1\xd0]\x97\x03\xdf\xcc|c\xa6G(!E\xe6\xd2\xb6\x85Z)O\xfc\xd5\xe4\x1a"{K\xe9\xc6\x0e\xb7z\xd9\xec\xfd\xa5\xa4\x8f\xech\xc9i=E\xaa\x87\xb5^8\x0b\xf1A\xb1\xa6\xfc\xf9\x1e\xfc\xc4\xffRG\x01\xd0#@\x87\xd0i\x81G\xa3\x95%!\x06\xe5}\x0bv\xb0\xbf\x17\xca\xd5\xe0\xc4-\xa0\xb1\x8b\xb6`\xc0I\xa4\xc5\xf4\x9el\xef\x95v [\x94\xcf\x8e\xd5\xcay\xe4l\xf7\xfe\xf7u\r'
+        '\x1b\x95j\xb6\xfb\xc4\x11\x85\xea\x84\xd0\x12O\x03t\x83D\xad\xc4\x0f\xf0\x95"M\xbc\x95\x00\xc0\xe7f|6\x8aYi^B.u<\xef\xb1\x19\xcf\xbb\xce\xdc|\x038=\xc7\xe6R\x01\x00\x00'
+    ),
+    ( './.bzr/revision-store/mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d.gz',
+        '\x1f\x8b\x08\x08A^BC\x00\x03mbp@sourcefrog.net-20051004104937-c9b7a7bfcc0bb22d\x00\x9d\x90\xc1j\xc30\x0c\x86\xef}\n'
+        "\xe3Coie'\xb1c\x9a\x94\xbe\xc0`o0,[N\x03M\\\x1c\xafe{\xfae\x94n\x85\xc1`;Y\x88O\xd2\xff\xb9Mt\x19\xe6!N\xcc\xc5q\x1cr\xa6\xd4\xf1'\x9b\xf20\xb1\xe7\x18Ol}\xca\xbb\x11\xcf\x879\xbe&G!\xc5~3Q^\xf7y\xc7\xd90]h\xca1\xbd\xbd\x0c\xbe\xe3?\xa9B\x02\xd4\x02\xa0\x12P\x99R\x17\xce\xa0\xb6\x1a\x83s\x80(\xa5\x7f\xdc0\x1f\xad\xe88\x82\xb0\x18\x0c\x82\x05\xa7\x04\x05[{\xc2\xda7\xc6\x81*\x85B\x8dh\x1a\xe7\x05g\xf7\xdc\xff>\x9d\x87\x91\xe6l\xc7s\xc7\x85\x90M%\xa5\xd1z#\x85\xa8\x9b\x1a\xaa\xfa\x06\xbc\xc7\x89:^*\x00\xe0\xfbU\xbbL\xcc\xb6\xa7\xfdH\xa9'\x16\x03\xeb\x8fq\xce\xed\xf6\xde_\xb5g\x9b\x16\xa1y\xa9\xbe\x02&\n"
+        '\x7fJ+EaM\x83$\xa5n\xbc/a\x91~\xd0\xbd\xfd\x135\n'
+        '\xd0\x9a`\x0c*W\x1aR\xc1\x94du\x08(\t\xb0\x91\xdeZ\xa3\x9cU\x9cm\x7f\x8dr\x1d\x10Ot\xb8\xc6\xcf\xa7\x907|\xfb-\xb1\xbd\xd3\xfb\xd5\x07\xeeD\xee\x08*\x02\x00\x00'
+    ),
+]
+
+_upgrade_dir_template = [
+    ( './.bzr/', ),
+    ( './.bzr/README',
+        'This is a Bazaar-NG control directory.\n'
+        'Do not change any files in this directory.\n'
+    ),
+    ( './.bzr/branch-format',
+        'Bazaar-NG branch, format 0.0.4\n'
+    ),
+    ( './.bzr/branch-lock',
+        ''
+    ),
+    ( './.bzr/branch-name',
+        ''
+    ),
+    ( './.bzr/inventory',
+        '<inventory>\n'
+        '<entry file_id="dir-20051005095101-da1441ea3fa6917a" kind="directory" name="dir" />\n'
+        '</inventory>\n'
+    ),
+    ( './.bzr/merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merged-patches',
+        ''
+    ),
+    ( './.bzr/pending-merges',
+        ''
+    ),
+    ( './.bzr/revision-history',
+        'robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e\n'
+    ),
+    ( './.bzr/stat-cache',
+        '### bzr hashcache v5\n'
+    ),
+    ( './.bzr/text-store/', ),
+    ( './.bzr/inventory-store/', ),
+    ( './.bzr/inventory-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xb3\xc9\xcc+K\xcd+\xc9/\xaa\xb4\xe3\xb2\x012\x8a*\x15\xd22sR\xe33Sl\x95R2\x8bt\x8d\x0c\x0cL\r'
+        "\x81\xd8\xc0\x12H\x19\xea\xa6$\x1a\x9a\x98\x18\xa6&\x1a\xa7%\x9aY\x1a\x9a'*)dg\xe6A\x94\xa6&\x83LQR\xc8K\xccM\x05\x0b()\xe8\x03\xcd\xd4G\xb2\x00\x00\xc2<\x94\xb1m\x00\x00\x00"
+    ),
+    ( './.bzr/revision-store/', ),
+    ( './.bzr/revision-store/robertc@robertcollins.net-20051005095108-6065fbd8e7d8617e.gz',
+        '\x1f\x8b\x08\x00\x0c\xa2CC\x02\xff\xa5OKj\xc30\x14\xdc\xfb\x14B\x8b\xec\x92<I\xd6\xc7\xc42\x85\xde\xa0\x17(\xb6\xf4\x9c\n'
+        'l\xa9H"\x90\x9c\xbe\xa6\xa9\xa1\x9b\xae\xbax\x0c\xcc\xe71\xd3g\xbc\x85\x12R$.\xadk\xa8\x15\xb3\xa5oi\xc2\\\xc9kZ\x96\x10\x0b9,\xf5\x92\xbf)\xf7\xf2\x83O\xe5\x14\xb1\x1e\xae\xf5BI\x887\x8c5\xe5\xfb{\xf0\x96\xfei>r\x00\xc9\xb6\x83n\x03sT\xa0\xe4<y\x83\xda\x1b\xc54\xfe~T>Ff\xe9\xcc:\xdd\x8e\xa6E\xc7@\xa2\x82I\xaaNL\xbas\\313)\x00\xb9\xe6\xe0(\xd9\x87\xfc\xb7A\r'
+        "+\x96:\xae\x9f\x962\xc6\x8d\x04i\x949\x01\x97R\xb7\x1d\x17O\xc3#E\xb4T(\x00\xa0C\xd3o\x892^q\x18\xbd'>\xe4\xfe\xbc\x13M\x7f\xde{\r"
+        '\xcd\x17\x85\xea\xba\x03l\x01\x00\x00'
+    ),
+    ( './dir/', ),
+]

=== renamed file 'tools/testweave.py' => 'bzrlib/selftest/test_weave.py'
--- tools/testweave.py	2005-07-18 14:28:13 +0000
+++ bzrlib/selftest/test_weave.py	2005-10-08 00:03:38 +0000
@@ -17,26 +17,19 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
+# TODO: tests regarding version names
+# TODO: rbc 20050108 test that join does not leave an inconsistent weave 
+#       if it fails.
 
 """test suite for weave algorithm"""
 
+from pprint import pformat
 
-import testsweet
-from bzrlib.weave import Weave, WeaveFormatError
+import bzrlib.errors as errors
+from bzrlib.weave import Weave, WeaveFormatError, WeaveError, reweave
 from bzrlib.weavefile import write_weave, read_weave
-from pprint import pformat
-
-
-try:
-    set
-    frozenset
-except NameError:
-    from sets import Set, ImmutableSet
-    set = Set
-    frozenset = ImmutableSet
-    del Set, ImmutableSet
-
+from bzrlib.selftest import TestCase
+from bzrlib.osutils import sha_string
 
 
 # texts for use in testing
@@ -46,7 +39,7 @@
 
 
 
-class TestBase(testsweet.TestBase):
+class TestBase(TestCase):
     def check_read_write(self, k):
         """Check the weave k can be written & re-read."""
         from tempfile import TemporaryFile
@@ -60,6 +53,14 @@
             tf.seek(0)
             self.log('serialized weave:')
             self.log(tf.read())
+
+            self.log('')
+            self.log('parents: %s' % (k._parents == k2._parents))
+            self.log('         %r' % k._parents)
+            self.log('         %r' % k2._parents)
+            self.log('')
+
+            
             self.fail('read/write check failed')
         
         
@@ -74,7 +75,7 @@
     """Store and retrieve a simple text."""
     def runTest(self):
         k = Weave()
-        idx = k.add([], TEXT_0)
+        idx = k.add('text0', [], TEXT_0)
         self.assertEqual(k.get(idx), TEXT_0)
         self.assertEqual(idx, 0)
 
@@ -83,7 +84,7 @@
 class AnnotateOne(TestBase):
     def runTest(self):
         k = Weave()
-        k.add([], TEXT_0)
+        k.add('text0', [], TEXT_0)
         self.assertEqual(k.annotate(0),
                          [(0, TEXT_0[0])])
 
@@ -92,16 +93,24 @@
     def runTest(self):
         k = Weave()
 
-        idx = k.add([], TEXT_0)
+        idx = k.add('text0', [], TEXT_0)
         self.assertEqual(idx, 0)
 
-        idx = k.add([], TEXT_1)
+        idx = k.add('text1', [], TEXT_1)
         self.assertEqual(idx, 1)
 
         self.assertEqual(k.get(0), TEXT_0)
         self.assertEqual(k.get(1), TEXT_1)
 
-        k.dump(self.TEST_LOG)
+
+
+class AddWithGivenSha(TestBase):
+    def runTest(self):
+        """Add with caller-supplied SHA-1"""
+        k = Weave()
+
+        t = 'text0'
+        k.add('text0', [], [t], sha1=sha_string(t))
 
 
 
@@ -112,10 +121,38 @@
 
         self.assertRaises(IndexError,
                           k.add,
+                          'text0',
                           [69],
                           ['new text!'])
 
 
+class RepeatedAdd(TestBase):
+    """Add the same version twice; harmless."""
+    def runTest(self):
+        k = Weave()
+        idx = k.add('text0', [], TEXT_0)
+        idx2 = k.add('text0', [], TEXT_0)
+        self.assertEqual(idx, idx2)
+
+
+
+class InvalidRepeatedAdd(TestBase):
+    def runTest(self):
+        k = Weave()
+        idx = k.add('text0', [], TEXT_0)
+        self.assertRaises(WeaveError,
+                          k.add,
+                          'text0',
+                          [],
+                          ['not the same text'])
+        self.assertRaises(WeaveError,
+                          k.add,
+                          'text0',
+                          [12],         # not the right parents
+                          TEXT_0)
+        
+
+
 class InsertLines(TestBase):
     """Store a revision that adds one line to the original.
 
@@ -124,8 +161,8 @@
     def runTest(self):
         k = Weave()
 
-        k.add([], ['line 1'])
-        k.add([0], ['line 1', 'line 2'])
+        k.add('text0', [], ['line 1'])
+        k.add('text1', [0], ['line 1', 'line 2'])
 
         self.assertEqual(k.annotate(0),
                          [(0, 'line 1')])
@@ -138,14 +175,15 @@
                          [(0, 'line 1'),
                           (1, 'line 2')])
 
-        k.add([0], ['line 1', 'diverged line'])
+        k.add('text2', [0], ['line 1', 'diverged line'])
 
         self.assertEqual(k.annotate(2),
                          [(0, 'line 1'),
                           (2, 'diverged line')])
 
         text3 = ['line 1', 'middle line', 'line 2']
-        k.add([0, 1],
+        k.add('text3',
+              [0, 1],
               text3)
 
         # self.log("changes to text3: " + pformat(list(k._delta(set([0, 1]), text3))))
@@ -158,7 +196,8 @@
                           (1, 'line 2')])
 
         # now multiple insertions at different places
-        k.add([0, 1, 3],
+        k.add('text4',
+              [0, 1, 3],
               ['line 1', 'aaa', 'middle line', 'bbb', 'line 2', 'ccc'])
 
         self.assertEqual(k.annotate(4), 
@@ -180,7 +219,7 @@
 
         base_text = ['one', 'two', 'three', 'four']
 
-        k.add([], base_text)
+        k.add('text0', [], base_text)
         
         texts = [['one', 'two', 'three'],
                  ['two', 'three', 'four'],
@@ -188,8 +227,11 @@
                  ['one', 'two', 'three', 'four'],
                  ]
 
+        i = 1
         for t in texts:
-            ver = k.add([0], t)
+            ver = k.add('text%d' % i,
+                        [0], t)
+            i += 1
 
         self.log('final weave:')
         self.log('k._weave=' + pformat(k._weave))
@@ -406,14 +448,14 @@
     def runTest(self):
         k = Weave()
 
-        k.add([], ["line the first",
+        k.add('text0', [], ["line the first",
                    "line 2",
                    "line 3",
                    "fine"])
 
         self.assertEqual(len(k.get(0)), 4)
 
-        k.add([0], ["line the first",
+        k.add('text1', [0], ["line the first",
                    "fine"])
 
         self.assertEqual(k.get(1),
@@ -454,8 +496,6 @@
         self.assertEqual(k.get(0),
                          ["first line"])
 
-        k.dump(self.TEST_LOG)
-
 
 class DivergedIncludes(TestBase):
     """Weave with two diverged texts based on version 0.
@@ -501,8 +541,8 @@
         text0 = ['cheddar', 'stilton', 'gruyere']
         text1 = ['cheddar', 'blue vein', 'neufchatel', 'chevre']
         
-        k.add([], text0)
-        k.add([0], text1)
+        k.add('text0', [], text0)
+        k.add('text1', [0], text1)
 
         self.log('k._weave=' + pformat(k._weave))
 
@@ -522,10 +562,10 @@
                  ['header', '', 'line from 1', 'fixup line', 'line from 2'],
                  ]
 
-        k.add([], texts[0])
-        k.add([0], texts[1])
-        k.add([0], texts[2])
-        k.add([0, 1, 2], texts[3])
+        k.add('text0', [], texts[0])
+        k.add('text1', [0], texts[1])
+        k.add('text2', [0], texts[2])
+        k.add('merge', [0, 1, 2], texts[3])
 
         for i, t in enumerate(texts):
             self.assertEqual(k.get(i), t)
@@ -593,9 +633,9 @@
                  ['header', 'aaa', 'bbb', 'line from 2', 'more from 2'],
                  ]
 
-        k.add([], texts[0])
-        k.add([0], texts[1])
-        k.add([0], texts[2])
+        k.add('text0', [], texts[0])
+        k.add('text1', [0], texts[1])
+        k.add('text2', [0], texts[2])
 
         self.log('k._weave=' + pformat(k._weave))
 
@@ -641,9 +681,12 @@
 
         k = Weave()
         parents = set()
+        i = 0
         for t in texts:
-            ver = k.add(list(parents), t)
+            ver = k.add('text%d' % i,
+                        list(parents), t)
             parents.add(ver)
+            i += 1
 
         self.log("k._weave=" + pformat(k._weave))
 
@@ -663,9 +706,9 @@
             return x + '\n'
         
         w = Weave()
-        w.add([], map(addcrlf, base))
-        w.add([0], map(addcrlf, a))
-        w.add([0], map(addcrlf, b))
+        w.add('text0', [], map(addcrlf, base))
+        w.add('text1', [0], map(addcrlf, a))
+        w.add('text2', [0], map(addcrlf, b))
 
         self.log('weave is:')
         tmpf = StringIO()
@@ -749,22 +792,130 @@
                      ['aaa', 'ddd', 'ccc'],
                      ['aaa', 'ccc'],
                      ['<<<<', 'aaa', '====', '>>>>', 'ccc'])
-    
-
-
-def testweave():
-    import testsweet
-    from unittest import TestSuite, TestLoader
-    import testweave
-
-    tl = TestLoader()
-    suite = TestSuite()
-    suite.addTest(tl.loadTestsFromModule(testweave))
-    
-    return int(not testsweet.run_suite(suite)) # for shell 0=true
+
+
+class JoinWeavesTests(TestBase):
+    def setUp(self):
+        super(JoinWeavesTests, self).setUp()
+        self.weave1 = Weave()
+        self.lines1 = ['hello\n']
+        self.lines3 = ['hello\n', 'cruel\n', 'world\n']
+        self.weave1.add('v1', [], self.lines1)
+        self.weave1.add('v2', [0], ['hello\n', 'world\n'])
+        self.weave1.add('v3', [1], self.lines3)
+        
+    def test_join_empty(self):
+        """Join two empty weaves."""
+        eq = self.assertEqual
+        w1 = Weave()
+        w2 = Weave()
+        w1.join(w2)
+        eq(w1.numversions(), 0)
+        
+    def test_join_empty_to_nonempty(self):
+        """Join empty weave onto nonempty."""
+        self.weave1.join(Weave())
+        self.assertEqual(len(self.weave1), 3)
+
+    def test_join_unrelated(self):
+        """Join two weaves with no history in common."""
+        wb = Weave()
+        wb.add('b1', [], ['line from b\n'])
+        w1 = self.weave1
+        w1.join(wb)
+        eq = self.assertEqual
+        eq(len(w1), 4)
+        eq(sorted(list(w1.iter_names())),
+           ['b1', 'v1', 'v2', 'v3'])
+
+    def test_join_related(self):
+        wa = self.weave1.copy()
+        wb = self.weave1.copy()
+        wa.add('a1', ['v3'], ['hello\n', 'sweet\n', 'world\n'])
+        wb.add('b1', ['v3'], ['hello\n', 'pale blue\n', 'world\n'])
+        eq = self.assertEquals
+        eq(len(wa), 4)
+        eq(len(wb), 4)
+        wa.join(wb)
+        eq(len(wa), 5)
+        eq(wa.get_lines('b1'),
+           ['hello\n', 'pale blue\n', 'world\n'])
+
+    def test_join_parent_disagreement(self):
+        """Cannot join weaves with different parents for a version."""
+        wa = Weave()
+        wb = Weave()
+        wa.add('v1', [], ['hello\n'])
+        wb.add('v0', [], [])
+        wb.add('v1', ['v0'], ['hello\n'])
+        self.assertRaises(WeaveError,
+                          wa.join, wb)
+
+    def test_join_text_disagreement(self):
+        """Cannot join weaves with different texts for a version."""
+        wa = Weave()
+        wb = Weave()
+        wa.add('v1', [], ['hello\n'])
+        wb.add('v1', [], ['not\n', 'hello\n'])
+        self.assertRaises(WeaveError,
+                          wa.join, wb)
+
+    def test_join_unordered(self):
+        """Join weaves where indexes differ.
+        
+        The source weave contains a different version at index 0."""
+        wa = self.weave1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1'], ['hello\n', 'world\n'])
+        wa.join(wb)
+        eq = self.assertEquals
+        eq(sorted(wa.iter_names()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wa.get_text('x1'), 'line from x1\n')
+
+    def test_reweave_with_empty(self):
+        wb = Weave()
+        wr = reweave(self.weave1, wb)
+        eq = self.assertEquals
+        eq(sorted(wr.iter_names()), ['v1', 'v2', 'v3'])
+        eq(wr.get_lines('v3'), ['hello\n', 'cruel\n', 'world\n'])
+        self.weave1.reweave(wb)
+        self.assertEquals(wr, self.weave1)
+
+    def test_join_with_ghosts_raises_parent_mismatch(self):
+        wa = self.weave1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        self.assertRaises(errors.WeaveParentMismatch, wa.join, wb)
+
+    def test_reweave_with_ghosts(self):
+        """Join that inserts parents of an existing revision.
+
+        This can happen when merging from another branch who
+        knows about revisions the destination does not.  In 
+        this test the second weave knows of an additional parent of 
+        v2.  Any revisions which are in common still have to have the 
+        same text."""
+        wa = self.weave1.copy()
+        wb = Weave()
+        wb.add('x1', [], ['line from x1\n'])
+        wb.add('v1', [], ['hello\n'])
+        wb.add('v2', ['v1', 'x1'], ['hello\n', 'world\n'])
+        wc = reweave(wa, wb)
+        eq = self.assertEquals
+        eq(sorted(wc.iter_names()), ['v1', 'v2', 'v3', 'x1',])
+        eq(wc.get_text('x1'), 'line from x1\n')
+        eq(wc.get_lines('v2'), ['hello\n', 'world\n'])
+        eq(wc.parent_names('v2'), ['v1', 'x1'])
+        self.weave1.reweave(wb)
+        self.assertEquals(wc, self.weave1)
 
 
 if __name__ == '__main__':
     import sys
-    sys.exit(testweave())
+    import unittest
+    sys.exit(unittest.main())
     

=== added file 'bzrlib/selftest/test_xml.py'
--- bzrlib/selftest/test_xml.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/test_xml.py	2005-09-30 14:27:41 +0000
@@ -0,0 +1,135 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+from cStringIO import StringIO
+
+from bzrlib.selftest import TestCase
+from bzrlib.inventory import Inventory, InventoryEntry
+from bzrlib.xml4 import serializer_v4
+from bzrlib.xml5 import serializer_v5
+
+_working_inventory_v4 = """<inventory file_id="TREE_ROOT">
+<entry file_id="bar-20050901064931-73b4b1138abc9cd2" kind="file" name="bar" parent_id="TREE_ROOT" />
+<entry file_id="foo-20050801201819-4139aa4a272f4250" kind="directory" name="foo" parent_id="TREE_ROOT" />
+<entry file_id="bar-20050824000535-6bc48cfad47ed134" kind="file" name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" />
+</inventory>"""
+
+
+_revision_v4 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92" revision_sha1="7bdf4cc8c5bdac739f8cf9b10b78cf4b68f915ff" />
+</parents>
+</revision>
+"""
+
+_revision_v5 = """<revision committer="Martin Pool &lt;mbp@sourcefrog.net&gt;"
+    inventory_sha1="e79c31c1deb64c163cf660fdedd476dd579ffd41"
+    revision_id="mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9"
+    timestamp="1125907235.211783886"
+    timezone="36000">
+<message>- start splitting code for xml (de)serialization away from objects
+  preparatory to supporting multiple formats by a single library
+</message>
+<parents>
+<revision_ref revision_id="mbp@sourcefrog.net-20050905063503-43948f59fa127d92"/>
+</parents>
+</revision>
+"""
+
+_committed_inv_v5 = """<inventory>
+<file file_id="bar-20050901064931-73b4b1138abc9cd2" 
+      name="bar" parent_id="TREE_ROOT" 
+      revision="mbp@foo-123123"/>
+<directory name="subdir"
+           file_id="foo-20050801201819-4139aa4a272f4250"
+           parent_id="TREE_ROOT" 
+           revision="mbp@foo-00"/>
+<file file_id="bar-20050824000535-6bc48cfad47ed134" 
+      name="bar" parent_id="foo-20050801201819-4139aa4a272f4250" 
+      revision="mbp@foo-00"/>
+</inventory>
+"""
+
+class TestSerializer(TestCase):
+    """Test XML serialization"""
+    def test_canned_inventory(self):
+        """Test unpacked a canned inventory v4 file."""
+        inp = StringIO(_working_inventory_v4)
+        inv = serializer_v4.read_inventory(inp)
+        self.assertEqual(len(inv), 4)
+        self.assert_('bar-20050901064931-73b4b1138abc9cd2' in inv)
+
+    def test_unpack_revision(self):
+        """Test unpacking a canned revision v4"""
+        inp = StringIO(_revision_v4)
+        rev = serializer_v4.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(rev.inventory_id,
+           "mbp@sourcefrog.net-20050905080035-e0439293f8b6b9f9")
+        eq(len(rev.parent_ids), 1)
+        eq(rev.parent_ids[0],
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_revision_5(self):
+        """Test unpacking a canned revision v5"""
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        eq = self.assertEqual
+        eq(rev.committer,
+           "Martin Pool <mbp@sourcefrog.net>")
+        eq(len(rev.parent_ids), 1)
+        eq(rev.timezone, 36000)
+        eq(rev.parent_ids[0],
+           "mbp@sourcefrog.net-20050905063503-43948f59fa127d92")
+
+    def test_unpack_inventory_5(self):
+        """Unpack canned new-style inventory"""
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        eq = self.assertEqual
+        eq(len(inv), 4)
+        ie = inv['bar-20050824000535-6bc48cfad47ed134']
+        eq(ie.kind, 'file')
+        eq(ie.revision, 'mbp@foo-00')
+        eq(ie.name, 'bar')
+        eq(inv[ie.parent_id].kind, 'directory')
+
+    def test_repack_inventory_5(self):
+        inp = StringIO(_committed_inv_v5)
+        inv = serializer_v5.read_inventory(inp)
+        outp = StringIO()
+        serializer_v5.write_inventory(inv, outp)
+        inv2 = serializer_v5.read_inventory(StringIO(outp.getvalue()))
+        self.assertEqual(inv, inv2)
+
+    def test_repack_revision_5(self):
+        inp = StringIO(_revision_v5)
+        rev = serializer_v5.read_revision(inp)
+        outp = StringIO()
+        serializer_v5.write_revision(rev, outp)
+        rev2 = serializer_v5.read_revision(StringIO(outp.getvalue()))
+        self.assertEqual(rev, rev2)
+

=== modified file 'bzrlib/selftest/testbranch.py'
--- bzrlib/selftest/testbranch.py	2005-07-11 07:05:34 +0000
+++ bzrlib/selftest/testbranch.py	2005-10-07 04:10:46 +0000
@@ -14,22 +14,206 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import InTempDir
-
-
-
-class TestAppendRevisions(InTempDir):
-    """Test appending more than one revision"""
-    def runTest(self):
-        from bzrlib.branch import Branch
-        br = Branch(".", init=True)
+import os
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+from bzrlib.commit import commit
+import bzrlib.errors as errors
+from bzrlib.errors import NoSuchRevision, UnlistableBranch
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.trace import mutter
+import bzrlib.transactions as transactions
+
+
+class TestBranch(TestCaseInTempDir):
+
+    def test_append_revisions(self):
+        """Test appending more than one revision"""
+        br = Branch.initialize(".")
         br.append_revision("rev1")
         self.assertEquals(br.revision_history(), ["rev1",])
         br.append_revision("rev2", "rev3")
         self.assertEquals(br.revision_history(), ["rev1", "rev2", "rev3"])
-        
-
-
-TEST_CLASSES = [
-    TestAppendRevisions,
-    ]
+
+    def test_fetch_revisions(self):
+        """Test fetch-revision operation."""
+        from bzrlib.fetch import Fetcher
+        os.mkdir('b1')
+        os.mkdir('b2')
+        b1 = Branch.initialize('b1')
+        b2 = Branch.initialize('b2')
+        file(os.sep.join(['b1', 'foo']), 'w').write('hello')
+        b1.add(['foo'], ['foo-id'])
+        b1.commit('lala!', rev_id='revision-1', allow_pointless=False)
+
+        mutter('start fetch')
+        f = Fetcher(from_branch=b1, to_branch=b2)
+        eq = self.assertEquals
+        eq(f.count_copied, 1)
+        eq(f.last_revision, 'revision-1')
+
+        rev = b2.get_revision('revision-1')
+        tree = b2.revision_tree('revision-1')
+        eq(tree.get_file_text('foo-id'), 'hello')
+
+    def test_push_stores(self):
+        """Copy the stores from one branch to another"""
+        os.mkdir('a')
+        br_a = Branch.initialize("a")
+        file('a/b', 'wb').write('b')
+        br_a.add('b')
+        commit(br_a, "silly commit")
+
+        os.mkdir('b')
+        br_b = Branch.initialize("b")
+        self.assertRaises(NoSuchRevision, br_b.get_revision, 
+                          br_a.revision_history()[0])
+        br_a.push_stores(br_b)
+        rev = br_b.get_revision(br_a.revision_history()[0])
+        tree = br_b.revision_tree(br_a.revision_history()[0])
+        for file_id in tree:
+            if tree.inventory[file_id].kind == "file":
+                tree.get_file(file_id).read()
+        return br_a, br_b
+
+    def test_copy_branch(self):
+        """Copy the stores from one branch to another"""
+        br_a, br_b = self.test_push_stores()
+        commit(br_b, "silly commit")
+        os.mkdir('c')
+        br_c = copy_branch(br_a, 'c', basis_branch=br_b)
+        self.assertEqual(br_a.revision_history(), br_c.revision_history())
+        ## # basis branches currently disabled for weave format
+        ## self.assertFalse(br_b.last_revision() in br_c.revision_history())
+        ## br_c.get_revision(br_b.last_revision())
+
+    def test_copy_partial(self):
+        """Copy only part of the history of a branch."""
+        self.build_tree(['a/', 'a/one'])
+        br_a = Branch.initialize('a')
+        br_a.add(['one'])
+        br_a.commit('commit one', rev_id='u@d-1')
+        self.build_tree(['a/two'])
+        br_a.add(['two'])
+        br_a.commit('commit two', rev_id='u@d-2')
+        br_b = copy_branch(br_a, 'b', revision='u@d-1')
+        self.assertEqual(br_b.last_revision(), 'u@d-1')
+        self.assertTrue(os.path.exists('b/one'))
+        self.assertFalse(os.path.exists('b/two'))
+        
+
+    def test_record_initial_ghost_merge(self):
+        """A pending merge with no revision present is still a merge."""
+        branch = Branch.initialize('.')
+        branch.add_pending_merge('non:existent@rev--ision--0--2')
+        branch.commit('pretend to merge nonexistent-revision', rev_id='first')
+        rev = branch.get_revision(branch.last_revision())
+        self.assertEqual(len(rev.parent_ids), 1)
+        # parent_sha1s is not populated now, WTF. rbc 20051003
+        self.assertEqual(len(rev.parent_sha1s), 0)
+        self.assertEqual(rev.parent_ids[0], 'non:existent@rev--ision--0--2')
+
+# TODO 20051003 RBC:
+# compare the gpg-to-sign info for a commit with a ghost and 
+#     an identical tree without a ghost
+# fetch missing should rewrite the TOC of weaves to list newly available parents.
+        
+    def test_pending_merges(self):
+        """Tracking pending-merged revisions."""
+        b = Branch.initialize('.')
+
+        self.assertEquals(b.pending_merges(), [])
+        b.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        b.add_pending_merge('foo@azkhazan-123123-abcabc')
+        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
+        b.add_pending_merge('wibble@fofof--20050401--1928390812')
+        self.assertEquals(b.pending_merges(),
+                          ['foo@azkhazan-123123-abcabc',
+                           'wibble@fofof--20050401--1928390812'])
+        b.commit("commit from base with two merges")
+        rev = b.get_revision(b.revision_history()[0])
+        self.assertEquals(len(rev.parent_ids), 2)
+        self.assertEquals(rev.parent_ids[0],
+                          'foo@azkhazan-123123-abcabc')
+        self.assertEquals(rev.parent_ids[1],
+                           'wibble@fofof--20050401--1928390812')
+        # list should be cleared when we do a commit
+        self.assertEquals(b.pending_merges(), [])
+ 
+
+# TODO: rewrite this as a regular unittest, without relying on the displayed output        
+#         >>> from bzrlib.commit import commit
+#         >>> bzrlib.trace.silent = True
+#         >>> br1 = ScratchBranch(files=['foo', 'bar'])
+#         >>> br1.add('foo')
+#         >>> br1.add('bar')
+#         >>> commit(br1, "lala!", rev_id="REVISION-ID-1", verbose=False)
+#         >>> br2 = ScratchBranch()
+#         >>> br2.update_revisions(br1)
+#         Added 2 texts.
+#         Added 1 inventories.
+#         Added 1 revisions.
+#         >>> br2.revision_history()
+#         [u'REVISION-ID-1']
+#         >>> br2.update_revisions(br1)
+#         Added 0 revisions.
+#         >>> br1.text_store.total_size() == br2.text_store.total_size()
+#         True
+
+class InstrumentedTransaction(object):
+
+    def finish(self):
+        self.calls.append('finish')
+
+    def __init__(self):
+        self.calls = []
+
+
+class TestBranchTransaction(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestBranchTransaction, self).setUp()
+        self.branch = Branch.initialize('.')
+        
+    def test_default_get_transaction(self):
+        """branch.get_transaction on a new branch should give a PassThrough."""
+        self.failUnless(isinstance(self.branch.get_transaction(),
+                                   transactions.PassThroughTransaction))
+
+    def test__set_new_transaction(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+
+    def test__set_over_existing_transaction_raises(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+        self.assertRaises(errors.LockError,
+                          self.branch._set_transaction,
+                          transactions.ReadOnlyTransaction())
+
+    def test_finish_no_transaction_raises(self):
+        self.assertRaises(errors.LockError, self.branch._finish_transaction)
+
+    def test_finish_readonly_transaction_works(self):
+        self.branch._set_transaction(transactions.ReadOnlyTransaction())
+        self.branch._finish_transaction()
+        self.assertEqual(None, self.branch._transaction)
+
+    def test_unlock_calls_finish(self):
+        self.branch.lock_read()
+        transaction = InstrumentedTransaction()
+        self.branch._transaction = transaction
+        self.branch.unlock()
+        self.assertEqual(['finish'], transaction.calls)
+
+    def test_lock_read_acquires_ro_transaction(self):
+        self.branch.lock_read()
+        self.failUnless(isinstance(self.branch.get_transaction(),
+                                   transactions.ReadOnlyTransaction))
+        self.branch.unlock()
+        
+    def test_lock_write_acquires_passthrough_transaction(self):
+        self.branch.lock_write()
+        # cannot use get_transaction as its magic
+        self.failUnless(isinstance(self.branch._transaction,
+                                   transactions.PassThroughTransaction))
+        self.branch.unlock()

=== added file 'bzrlib/selftest/testdiff.py'
--- bzrlib/selftest/testdiff.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testdiff.py	2005-08-25 12:46:42 +0000
@@ -0,0 +1,48 @@
+from bzrlib.selftest import TestCase
+from bzrlib.diff import internal_diff
+from cStringIO import StringIO
+def udiff_lines(old, new):
+    output = StringIO()
+    internal_diff('old', old, 'new', new, output)
+    output.seek(0, 0)
+    return output.readlines()
+
+def check_patch(lines):
+    assert len(lines) > 1, \
+        "Not enough lines for a file header for patch:\n%s" % "".join(lines)
+    assert lines[0].startswith ('---'), \
+        'No orig line for patch:\n%s' % "".join(lines)
+    assert lines[1].startswith ('+++'), \
+        'No mod line for patch:\n%s' % "".join(lines)
+    assert len(lines) > 2, \
+        "No hunks for patch:\n%s" % "".join(lines)
+    assert lines[2].startswith('@@'),\
+        "No hunk header for patch:\n%s" % "".join(lines)
+    assert '@@' in lines[2][2:], \
+        "Unterminated hunk header for patch:\n%s" % "".join(lines)
+
+class TestDiff(TestCase):
+    def test_add_nl(self):
+        """diff generates a valid diff for patches that add a newline"""
+        lines = udiff_lines(['boo'], ['boo\n'])
+        check_patch(lines)
+        assert lines[4] == '\\ No newline at end of file\n', \
+            "expected no-nl, got %r" % lines[4]
+
+    def test_add_nl_2(self):
+        """diff generates a valid diff for patches that change last line and
+        add a newline.
+        """
+        lines = udiff_lines(['boo'], ['goo\n'])
+        check_patch(lines)
+        assert lines[4] == '\\ No newline at end of file\n', \
+            "expected no-nl, got %r" % lines[4]
+
+    def test_remove_nl(self):
+        """diff generates a valid diff for patches that change last line and
+        add a newline.
+        """
+        lines = udiff_lines(['boo\n'], ['boo'])
+        check_patch(lines)
+        assert lines[5] == '\\ No newline at end of file\n', \
+            "expected no-nl, got %r" % lines[5]

=== added file 'bzrlib/selftest/testfetch.py'
--- bzrlib/selftest/testfetch.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testfetch.py	2005-10-08 00:03:38 +0000
@@ -0,0 +1,140 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+import sys
+
+import bzrlib.errors
+from bzrlib.selftest.testrevision import make_branches
+from bzrlib.trace import mutter
+from bzrlib.branch import Branch
+from bzrlib.fetch import greedy_fetch
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
+
+
+def has_revision(branch, revision_id):
+    try:
+        branch.get_revision_xml_file(revision_id)
+        return True
+    except bzrlib.errors.NoSuchRevision:
+        return False
+
+def fetch_steps(self, br_a, br_b, writable_a):
+    """A foreign test method for testing fetch locally and remotely."""
+    def new_branch(name):
+        os.mkdir(name)
+        return Branch.initialize(name)
+            
+    assert not has_revision(br_b, br_a.revision_history()[3])
+    assert has_revision(br_b, br_a.revision_history()[2])
+    assert len(br_b.revision_history()) == 7
+    assert greedy_fetch(br_b, br_a, br_a.revision_history()[2])[0] == 0
+
+    # greedy_fetch is not supposed to alter the revision history
+    assert len(br_b.revision_history()) == 7
+    assert not has_revision(br_b, br_a.revision_history()[3])
+
+    assert len(br_b.revision_history()) == 7
+    assert greedy_fetch(br_b, br_a, br_a.revision_history()[3])[0] == 1
+    assert has_revision(br_b, br_a.revision_history()[3])
+    assert not has_revision(br_a, br_b.revision_history()[6])
+    assert has_revision(br_a, br_b.revision_history()[5])
+
+    # When a non-branch ancestor is missing, it should be unlisted...
+    # as its not reference from the inventory weave.
+    br_b4 = new_branch('br_4')
+    count, failures = greedy_fetch(br_b4, br_b)
+    self.assertEqual(count, 7)
+    self.assertEqual(failures, [])
+
+    self.assertEqual(greedy_fetch(writable_a, br_b)[0], 1)
+    assert has_revision(br_a, br_b.revision_history()[3])
+    assert has_revision(br_a, br_b.revision_history()[4])
+        
+    br_b2 = new_branch('br_b2')
+    assert greedy_fetch(br_b2, br_b)[0] == 7
+    assert has_revision(br_b2, br_b.revision_history()[4])
+    assert has_revision(br_b2, br_a.revision_history()[2])
+    assert not has_revision(br_b2, br_a.revision_history()[3])
+
+    br_a2 = new_branch('br_a2')
+    assert greedy_fetch(br_a2, br_a)[0] == 9
+    assert has_revision(br_a2, br_b.revision_history()[4])
+    assert has_revision(br_a2, br_a.revision_history()[3])
+    assert has_revision(br_a2, br_a.revision_history()[2])
+
+    br_a3 = new_branch('br_a3')
+    assert greedy_fetch(br_a3, br_a2)[0] == 0
+    for revno in range(4):
+        assert not has_revision(br_a3, br_a.revision_history()[revno])
+    self.assertEqual(greedy_fetch(br_a3, br_a2, br_a.revision_history()[2])[0], 3)
+    fetched = greedy_fetch(br_a3, br_a2, br_a.revision_history()[3])[0]
+    assert fetched == 3, "fetched %d instead of 3" % fetched
+    # InstallFailed should be raised if the branch is missing the revision
+    # that was requested.
+    self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
+                      br_a2, 'pizza')
+    # InstallFailed should be raised if the branch is missing a revision
+    # from its own revision history
+    br_a2.append_revision('a-b-c')
+    self.assertRaises(bzrlib.errors.InstallFailed, greedy_fetch, br_a3,
+                      br_a2)
+
+
+    #TODO: test that fetch correctly does reweaving when needed. RBC 20051008
+
+class TestFetch(TestCaseInTempDir):
+
+    def test_fetch(self):
+        #highest indices a: 5, b: 7
+        br_a, br_b = make_branches()
+        fetch_steps(self, br_a, br_b, br_a)
+
+
+class TestHttpFetch(TestCaseWithWebserver):
+
+    def setUp(self):
+        super(TestHttpFetch, self).setUp()
+        self.weblogs = []
+
+    def test_fetch(self):
+        #highest indices a: 5, b: 7
+        br_a, br_b = make_branches()
+        br_rem_a = Branch.open(self.get_remote_url(br_a._transport.base))
+        fetch_steps(self, br_rem_a, br_b, br_a)
+
+    def log(self, *args):
+        """Capture web server log messages for introspection."""
+        super(TestHttpFetch, self).log(*args)
+        if args[0].startswith("webserver"):
+            self.weblogs.append(args[0])
+
+    def test_weaves_are_retrieved_once(self):
+        self.build_tree(("source/", "source/file", "target/"))
+        branch = Branch.initialize("source")
+        branch.add(["file"], ["id"])
+        branch.commit("added file")
+        print >>open("source/file", 'w'), "blah"
+        branch.commit("changed file")
+        target = Branch.initialize("target/")
+        source = Branch.open(self.get_remote_url("source/"))
+        source.weave_store.enable_cache = False
+        self.assertEqual(greedy_fetch(target, source), (2, []))
+        weave_suffix = 'weaves/id.weave HTTP/1.1" 200 -'
+        self.assertEqual(1,
+            len([log for log in self.weblogs if log.endswith(weave_suffix)]))

=== added file 'bzrlib/selftest/testgraph.py'
--- bzrlib/selftest/testgraph.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testgraph.py	2005-09-21 15:33:23 +0000
@@ -0,0 +1,49 @@
+from bzrlib.selftest import TestCase
+from bzrlib.graph import node_distances, nodes_by_distance
+
+class TestBase(TestCase):
+    def edge_add(self, *args):
+        for start, end in zip(args[:-1], args[1:]):
+            if start not in self.graph:
+                self.graph[start] = {}
+            if end not in self.graph:
+                self.graph[end] = {}
+            self.graph[start][end] = 1
+
+    def setUp(self):
+        TestCase.setUp(self)
+        self.graph = {}
+        self.edge_add('A', 'B', 'C', 'D')
+        self.edge_add('A', 'E', 'F', 'C')
+        self.edge_add('A', 'G', 'H', 'I', 'B')
+        self.edge_add('A', 'J', 'K', 'L', 'M', 'N')
+        self.edge_add('O', 'N')
+
+    def node_descendants(self):
+        descendants = {'A':set()}
+        for node in self.graph:
+            for ancestor in self.graph[node]:
+                if ancestor not in descendants:
+                    descendants[ancestor] = set()
+                descendants[ancestor].add(node)
+        return descendants
+    
+    def test_distances(self):
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        nodes = nodes_by_distance(distances)
+        self.assertEqual(nodes[0], 'D')
+        assert nodes[1] in ('N', 'C')
+        assert nodes[2] in ('N', 'C')
+        assert nodes[3] in ('B', 'M')
+        assert nodes[4] in ('B', 'M')
+
+        #Ensure we don't shortcut through B when there's only a difference of
+        # 1 in distance
+        self.graph = {}
+        self.edge_add('A', 'B', 'C')
+        self.edge_add('A', 'D', 'E', 'C')
+        descendants = self.node_descendants()
+        distances = node_distances(self.graph, descendants, 'A')
+        self.assertEqual(distances['C'], 3)
+

=== modified file 'bzrlib/selftest/testhashcache.py'
--- bzrlib/selftest/testhashcache.py	2005-07-08 06:54:58 +0000
+++ bzrlib/selftest/testhashcache.py	2005-09-27 07:24:40 +0000
@@ -14,7 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import InTempDir
+import os
+import time
+from bzrlib.selftest import TestCaseInTempDir
 
 
 
@@ -24,20 +26,23 @@
 
 
 def pause():
-    import time
+    if False:
+        return
+    if os.name == 'nt':
+        time.sleep(3)
+        return
     # allow it to stabilize
     start = int(time.time())
     while int(time.time()) == start:
         time.sleep(0.2)
     
 
+class TestHashCache(TestCaseInTempDir):
 
-class TestHashCache(InTempDir):
-    """Functional tests for hashcache"""
-    def runTest(self):
+    def test_hashcache(self):
+        """Functional tests for hashcache"""
         from bzrlib.hashcache import HashCache
         import os
-        import time
 
         # make a dummy bzr directory just to hold the cache
         os.mkdir('.bzr')
@@ -55,21 +60,21 @@
         # check we hit without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 1)
+        ##self.assertEquals(hc.miss_count, 1)
+        ##self.assertEquals(hc.hit_count, 1)
 
         # check again without re-reading
         self.assertEquals(hc.get_sha1('foo'),
                           'aaf4c61ddcc5e8a2dabede0f3b482cd9aea9434d')
-        self.assertEquals(hc.miss_count, 1)
-        self.assertEquals(hc.hit_count, 2)
+        ##self.assertEquals(hc.miss_count, 1)
+        ##self.assertEquals(hc.hit_count, 2)
 
         # write new file and make sure it is seen
         file('foo', 'wb').write('goodbye')
         pause()
         self.assertEquals(hc.get_sha1('foo'),
                           '3c8ec4874488f6090a157b014ce3397ca8e06d4f')
-        self.assertEquals(hc.miss_count, 2)
+        ##self.assertEquals(hc.miss_count, 2)
 
         # quickly write new file of same size and make sure it is seen
         # this may rely on detection of timestamps that are too close
@@ -95,9 +100,9 @@
 
         # should now be safe to cache it if we reread them
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(len(hc._cache), 1)
+        ##self.assertEquals(len(hc._cache), 1)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-        self.assertEquals(len(hc._cache), 2)
+        ##self.assertEquals(len(hc._cache), 2)
 
         # write out, read back in and check that we don't need to
         # re-read any files
@@ -107,13 +112,8 @@
         hc = HashCache('.')
         hc.read()
 
-        self.assertEquals(len(hc._cache), 2)
+        ##self.assertEquals(len(hc._cache), 2)
         self.assertEquals(hc.get_sha1('foo'), sha1('g00dbye'))
-        self.assertEquals(hc.hit_count, 1)
-        self.assertEquals(hc.miss_count, 0)
+        ##self.assertEquals(hc.hit_count, 1)
+        ##self.assertEquals(hc.miss_count, 0)
         self.assertEquals(hc.get_sha1('foo2'), sha1('new content'))
-
-        
-
-        
-

=== added file 'bzrlib/selftest/testidentitymap.py'
--- bzrlib/selftest/testidentitymap.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testidentitymap.py	2005-10-07 04:10:46 +0000
@@ -0,0 +1,70 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the IdentityMap class."""
+
+# import system imports here
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.selftest import TestCase
+import bzrlib.identitymap as identitymap
+
+
+class TestIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import IdentityMap
+
+    def test_construct(self):
+        identitymap.IdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.IdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertRaises(errors.BzrError, map.add_weave, "id", weave)
+        self.assertEqual(weave, map.find_weave("id"))
+ 
+ 
+class TestNullIdentityMap(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.identitymap import NullIdentityMap
+
+    def test_construct(self):
+        identitymap.NullIdentityMap()
+
+    def test_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+
+    def test_double_add_weave(self):
+        map = identitymap.NullIdentityMap()
+        weave = "foo"
+        map.add_weave("id", weave)
+        map.add_weave("id", weave)
+        self.assertEqual(None, map.find_weave("id"))
+        

=== modified file 'bzrlib/selftest/testinv.py'
--- bzrlib/selftest/testinv.py	2005-07-23 00:18:14 +0000
+++ bzrlib/selftest/testinv.py	2005-10-07 06:13:14 +0000
@@ -14,17 +14,27 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from bzrlib.selftest import TestBase
-
-from bzrlib.inventory import Inventory, InventoryEntry
-
-
-class TestIsWithin(TestBase):
-    def runTest(self):
+from cStringIO import StringIO
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.clone import copy_branch
+import bzrlib.errors as errors
+from bzrlib.diff import internal_diff
+from bzrlib.inventory import Inventory, ROOT_ID
+import bzrlib.inventory as inventory
+from bzrlib.osutils import has_symlinks, rename
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+
+
+class TestInventory(TestCase):
+
+    def test_is_within(self):
         from bzrlib.osutils import is_inside_any
-        
-        for dirs, fn in [(['src', 'doc'], 'src/foo.c'),
-                         (['src'], 'src/foo.c'),
+
+        SRC_FOO_C = os.path.join('src', 'foo.c')
+        for dirs, fn in [(['src', 'doc'], SRC_FOO_C),
+                         (['src'], SRC_FOO_C),
                          (['src'], 'src'),
                          ]:
             self.assert_(is_inside_any(dirs, fn))
@@ -33,10 +43,7 @@
                          (['src'], 'srccontrol/foo')]:
             self.assertFalse(is_inside_any(dirs, fn))
             
-            
-            
-class TestInventoryIds(TestBase):
-    def runTest(self):
+    def test_ids(self):
         """Test detection of files within selected directories."""
         inv = Inventory()
         
@@ -51,3 +58,309 @@
         self.assertEqual(inv.path2id('src/bye.c'), 'bye-id')
         
         self.assert_('src-id' in inv)
+
+
+    def test_version(self):
+        """Inventory remembers the text's version."""
+        inv = Inventory()
+        ie = inv.add_path('foo.txt', 'file')
+        ## XXX
+
+
+class TestInventoryEntry(TestCase):
+
+    def test_file_kind_character(self):
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        self.assertEqual(file.kind_character(), '')
+
+    def test_dir_kind_character(self):
+        dir = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '/')
+
+    def test_link_kind_character(self):
+        dir = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        self.assertEqual(dir.kind_character(), '')
+
+    def test_dir_detect_changes(self):
+        left = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        left.executable = True
+        left.symlink_target='foo'
+        right = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 321
+        right.symlink_target='bar'
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+
+    def test_file_detect_changes(self):
+        left = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        right = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 123
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+        left.executable = True
+        self.assertEqual((False, True), left.detect_changes(right))
+        self.assertEqual((False, True), right.detect_changes(left))
+        right.text_sha1 = 321
+        self.assertEqual((True, True), left.detect_changes(right))
+        self.assertEqual((True, True), right.detect_changes(left))
+
+    def test_symlink_detect_changes(self):
+        left = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        left.text_sha1 = 123
+        left.executable = True
+        left.symlink_target='foo'
+        right = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        right.text_sha1 = 321
+        right.symlink_target='foo'
+        self.assertEqual((False, False), left.detect_changes(right))
+        self.assertEqual((False, False), right.detect_changes(left))
+        left.symlink_target = 'different'
+        self.assertEqual((True, False), left.detect_changes(right))
+        self.assertEqual((True, False), right.detect_changes(left))
+
+    def test_file_has_text(self):
+        file = inventory.InventoryFile('123', 'hello.c', ROOT_ID)
+        self.failUnless(file.has_text())
+
+    def test_directory_has_text(self):
+        dir = inventory.InventoryDirectory('123', 'hello.c', ROOT_ID)
+        self.failIf(dir.has_text())
+
+    def test_link_has_text(self):
+        link = inventory.InventoryLink('123', 'hello.c', ROOT_ID)
+        self.failIf(link.has_text())
+
+
+class TestEntryDiffing(TestCaseInTempDir):
+
+    def setUp(self):
+        super(TestEntryDiffing, self).setUp()
+        self.branch = Branch.initialize('.')
+        print >> open('file', 'wb'), 'foo'
+        self.branch.add(['file'], ['fileid'])
+        if has_symlinks():
+            os.symlink('target1', 'symlink')
+            self.branch.add(['symlink'], ['linkid'])
+        self.branch.commit('message_1', rev_id = '1')
+        print >> open('file', 'wb'), 'bar'
+        if has_symlinks():
+            os.unlink('symlink')
+            os.symlink('target2', 'symlink')
+        self.tree_1 = self.branch.revision_tree('1')
+        self.inv_1 = self.branch.get_inventory('1')
+        self.file_1 = self.inv_1['fileid']
+        self.tree_2 = self.branch.working_tree()
+        self.inv_2 = self.branch.inventory
+        self.file_2 = self.inv_2['fileid']
+        if has_symlinks():
+            self.link_1 = self.inv_1['linkid']
+            self.link_2 = self.inv_2['linkid']
+
+    def test_file_diff_deleted(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "old_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output)
+        self.assertEqual(output.getvalue(), "--- old_label\n"
+                                            "+++ /dev/null\n"
+                                            "@@ -1,1 +0,0 @@\n"
+                                            "-foo\n"
+                                            "\n")
+
+    def test_file_diff_added(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "new_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output, reverse=True)
+        self.assertEqual(output.getvalue(), "--- /dev/null\n"
+                                            "+++ new_label\n"
+                                            "@@ -0,0 +1,1 @@\n"
+                                            "+foo\n"
+                                            "\n")
+
+    def test_file_diff_changed(self):
+        output = StringIO()
+        self.file_1.diff(internal_diff, 
+                          "/dev/null", self.tree_1, 
+                          "new_label", self.file_2, self.tree_2,
+                          output)
+        self.assertEqual(output.getvalue(), "--- /dev/null\n"
+                                            "+++ new_label\n"
+                                            "@@ -1,1 +1,1 @@\n"
+                                            "-foo\n"
+                                            "+bar\n"
+                                            "\n")
+        
+    def test_link_diff_deleted(self):
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "old_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output)
+        self.assertEqual(output.getvalue(),
+                         "=== target was 'target1'\n")
+
+    def test_link_diff_added(self):
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "new_label", self.tree_1,
+                          "/dev/null", None, None,
+                          output, reverse=True)
+        self.assertEqual(output.getvalue(),
+                         "=== target is 'target1'\n")
+
+    def test_link_diff_changed(self):
+        output = StringIO()
+        self.link_1.diff(internal_diff, 
+                          "/dev/null", self.tree_1, 
+                          "new_label", self.link_2, self.tree_2,
+                          output)
+        self.assertEqual(output.getvalue(),
+                         "=== target changed 'target1' => 'target2'\n")
+
+
+class TestSnapshot(TestCaseInTempDir):
+
+    def setUp(self):
+        # for full testing we'll need a branch
+        # with a subdir to test parent changes.
+        # and a file, link and dir under that.
+        # but right now I only need one attribute
+        # to change, and then test merge patterns
+        # with fake parent entries.
+        super(TestSnapshot, self).setUp()
+        self.branch = Branch.initialize('.')
+        self.build_tree(['subdir/', 'subdir/file'])
+        self.branch.add(['subdir', 'subdir/file'], ['dirid', 'fileid'])
+        if has_symlinks():
+            pass
+        self.branch.commit('message_1', rev_id = '1')
+        self.tree_1 = self.branch.revision_tree('1')
+        self.inv_1 = self.branch.get_inventory('1')
+        self.file_1 = self.inv_1['fileid']
+        self.work_tree = self.branch.working_tree()
+        self.file_active = self.work_tree.inventory['fileid']
+
+    def test_snapshot_new_revision(self):
+        # This tests that a simple commit with no parents makes a new
+        # revision value in the inventory entry
+        self.file_active.snapshot('2', 'subdir/file', {}, self.work_tree, 
+                                  self.branch.weave_store,
+                                  self.branch.get_transaction())
+        # expected outcome - file_1 has a revision id of '2', and we can get
+        # its text of 'file contents' out of the weave.
+        self.assertEqual(self.file_1.revision, '1')
+        self.assertEqual(self.file_active.revision, '2')
+        # this should be a separate test probably, but lets check it once..
+        lines = self.branch.weave_store.get_lines('fileid','2',
+            self.branch.get_transaction())
+        self.assertEqual(lines, ['contents of subdir/file\n'])
+
+    def test_snapshot_unchanged(self):
+        #This tests that a simple commit does not make a new entry for
+        # an unchanged inventory entry
+        self.file_active.snapshot('2', 'subdir/file', {'1':self.file_1},
+                                  self.work_tree, self.branch.weave_store,
+                                  self.branch.get_transaction())
+        self.assertEqual(self.file_1.revision, '1')
+        self.assertEqual(self.file_active.revision, '1')
+        self.assertRaises(errors.WeaveError,
+                          self.branch.weave_store.get_lines, 'fileid', '2',
+                          self.branch.get_transaction())
+
+    def test_snapshot_merge_identical_different_revid(self):
+        # This tests that a commit with two identical parents, one of which has
+        # a different revision id, results in a new revision id in the entry.
+        # 1->other, commit a merge of other against 1, results in 2.
+        other_ie = inventory.InventoryFile('fileid', 'newname', self.file_1.parent_id)
+        other_ie = inventory.InventoryFile('fileid', 'file', self.file_1.parent_id)
+        other_ie.revision = '1'
+        other_ie.text_sha1 = self.file_1.text_sha1
+        other_ie.text_size = self.file_1.text_size
+        self.assertEqual(self.file_1, other_ie)
+        other_ie.revision = 'other'
+        self.assertNotEqual(self.file_1, other_ie)
+        self.branch.weave_store.add_identical_text('fileid', '1', 'other', ['1'],
+            self.branch.get_transaction())
+        self.file_active.snapshot('2', 'subdir/file', 
+                                  {'1':self.file_1, 'other':other_ie},
+                                  self.work_tree, self.branch.weave_store,
+                                  self.branch.get_transaction())
+        self.assertEqual(self.file_active.revision, '2')
+
+    def test_snapshot_changed(self):
+        # This tests that a commit with one different parent results in a new
+        # revision id in the entry.
+        self.file_active.name='newname'
+        rename('subdir/file', 'subdir/newname')
+        self.file_active.snapshot('2', 'subdir/newname', {'1':self.file_1}, 
+                                  self.work_tree, 
+                                  self.branch.weave_store,
+                                  self.branch.get_transaction())
+        # expected outcome - file_1 has a revision id of '2'
+        self.assertEqual(self.file_active.revision, '2')
+
+
+class TestPreviousHeads(TestCaseInTempDir):
+
+    def setUp(self):
+        # we want several inventories, that respectively
+        # give use the following scenarios:
+        # A) fileid not in any inventory (A),
+        # B) fileid present in one inventory (B) and (A,B)
+        # C) fileid present in two inventories, and they
+        #   are not mutual descendents (B, C)
+        # D) fileid present in two inventories and one is
+        #   a descendent of the other. (B, D)
+        super(TestPreviousHeads, self).setUp()
+        self.build_tree(['file'])
+        self.branch = Branch.initialize('.')
+        self.branch.commit('new branch', allow_pointless=True, rev_id='A')
+        self.inv_A = self.branch.get_inventory('A')
+        self.branch.add(['file'], ['fileid'])
+        self.branch.commit('add file', rev_id='B')
+        self.inv_B = self.branch.get_inventory('B')
+        self.branch.put_controlfile('revision-history', 'A\n')
+        self.assertEqual(self.branch.revision_history(), ['A'])
+        self.branch.commit('another add of file', rev_id='C')
+        self.inv_C = self.branch.get_inventory('C')
+        self.branch.add_pending_merge('B')
+        self.branch.commit('merge in B', rev_id='D')
+        self.inv_D = self.branch.get_inventory('D')
+        self.file_active = self.branch.working_tree().inventory['fileid']
+        self.weave = self.branch.weave_store.get_weave('fileid',
+            self.branch.get_transaction())
+        
+    def get_previous_heads(self, inventories):
+        return self.file_active.find_previous_heads(inventories, self.weave)
+        
+    def test_fileid_in_no_inventory(self):
+        self.assertEqual({}, self.get_previous_heads([self.inv_A]))
+
+    def test_fileid_in_one_inventory(self):
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_B]))
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_A, self.inv_B]))
+        self.assertEqual({'B':self.inv_B['fileid']},
+                         self.get_previous_heads([self.inv_B, self.inv_A]))
+
+    def test_fileid_in_two_inventories_gives_both_entries(self):
+        self.assertEqual({'B':self.inv_B['fileid'],
+                          'C':self.inv_C['fileid']},
+                          self.get_previous_heads([self.inv_B, self.inv_C]))
+        self.assertEqual({'B':self.inv_B['fileid'],
+                          'C':self.inv_C['fileid']},
+                          self.get_previous_heads([self.inv_C, self.inv_B]))
+
+    def test_fileid_in_two_inventories_already_merged_gives_head(self):
+        self.assertEqual({'D':self.inv_D['fileid']},
+                         self.get_previous_heads([self.inv_B, self.inv_D]))
+        self.assertEqual({'D':self.inv_D['fileid']},
+                         self.get_previous_heads([self.inv_D, self.inv_B]))
+
+    # TODO: test two inventories with the same file revision 

=== added file 'bzrlib/selftest/testlog.py'
--- bzrlib/selftest/testlog.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testlog.py	2005-10-02 21:51:29 +0000
@@ -0,0 +1,162 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from cStringIO import StringIO
+
+from bzrlib.selftest import BzrTestBase, TestCaseInTempDir
+from bzrlib.log import LogFormatter, show_log, LongLogFormatter
+from bzrlib.branch import Branch
+from bzrlib.errors import InvalidRevisionNumber
+
+class _LogEntry(object):
+    # should probably move into bzrlib.log?
+    pass
+
+
+class LogCatcher(LogFormatter):
+    """Pull log messages into list rather than displaying them.
+
+    For ease of testing we save log messages here rather than actually
+    formatting them, so that we can precisely check the result without
+    being too dependent on the exact formatting.
+
+    We should also test the LogFormatter.
+    """
+    def __init__(self):
+        super(LogCatcher, self).__init__(to_file=None)
+        self.logs = []
+        
+        
+    def show(self, revno, rev, delta):
+        le = _LogEntry()
+        le.revno = revno
+        le.rev = rev
+        le.delta = delta
+        self.logs.append(le)
+
+
+class SimpleLogTest(TestCaseInTempDir):
+
+    def checkDelta(self, delta, **kw):
+        """Check the filenames touched by a delta are as expected."""
+        for n in 'added', 'removed', 'renamed', 'modified', 'unchanged':
+            expected = kw.get(n, [])
+
+            # tests are written with unix paths; fix them up for windows
+            if os.sep != '/':
+                expected = [x.replace('/', os.sep) for x in expected]
+
+            # strip out only the path components
+            got = [x[0] for x in getattr(delta, n)]
+            self.assertEquals(expected, got)
+
+    def test_cur_revno(self):
+        b = Branch('.', init=True)
+
+        lf = LogCatcher()
+        b.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
+    def test_cur_revno(self):
+        b = Branch.initialize('.')
+
+        lf = LogCatcher()
+        b.commit('empty commit')
+        show_log(b, lf, verbose=True, start_revision=1, end_revision=1)
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=2, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=0, end_revision=2) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=0) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=-1, end_revision=1) 
+        self.assertRaises(InvalidRevisionNumber, show_log, b, lf,
+                          start_revision=1, end_revision=-1) 
+
+    def test_simple_log(self):
+        eq = self.assertEquals
+        
+        b = Branch.initialize('.')
+
+        lf = LogCatcher()
+        show_log(b, lf)
+        # no entries yet
+        eq(lf.logs, [])
+
+
+        b.commit('empty commit')
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        eq(len(lf.logs), 1)
+        eq(lf.logs[0].revno, 1)
+        eq(lf.logs[0].rev.message, 'empty commit')
+        d = lf.logs[0].delta
+        self.log('log delta: %r' % d)
+        self.checkDelta(d)
+
+
+        self.build_tree(['hello'])
+        b.add('hello')
+        b.commit('add one file')
+
+        lf = StringIO()
+        # log using regular thing
+        show_log(b, LongLogFormatter(lf))
+        lf.seek(0)
+        for l in lf.readlines():
+            self.log(l)
+
+        # get log as data structure
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        eq(len(lf.logs), 2)
+        self.log('log entries:')
+        for logentry in lf.logs:
+            self.log('%4d %s' % (logentry.revno, logentry.rev.message))
+        
+        # first one is most recent
+        logentry = lf.logs[0]
+        eq(logentry.revno, 2)
+        eq(logentry.rev.message, 'add one file')
+        d = logentry.delta
+        self.log('log 2 delta: %r' % d)
+        # self.checkDelta(d, added=['hello'])
+        
+        # commit a log message with control characters
+        msg = "All 8-bit chars: " +  ''.join([unichr(x) for x in range(256)])
+        b.commit(msg)
+        lf = LogCatcher()
+        show_log(b, lf, verbose=True)
+        committed_msg = lf.logs[0].rev.message
+        self.log("escaped commit message: %r", committed_msg)
+        self.assert_(msg != committed_msg)
+        self.assert_(len(committed_msg) > len(msg))

=== added file 'bzrlib/selftest/testmerge.py'
--- bzrlib/selftest/testmerge.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testmerge.py	2005-09-29 20:34:25 +0000
@@ -0,0 +1,52 @@
+import os
+
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.merge import merge
+from bzrlib.errors import UnrelatedBranches, NoCommits
+from bzrlib.revision import common_ancestor
+from bzrlib.fetch import fetch
+
+
+class TestMerge(TestCaseInTempDir):
+    """Test appending more than one revision"""
+    def test_pending(self):
+        br = Branch.initialize(".")
+        commit(br, "lala!")
+        self.assertEquals(len(br.pending_merges()), 0)
+        merge(['.', -1], [None, None])
+        self.assertEquals(len(br.pending_merges()), 0)
+
+    def test_nocommits(self):
+        self.test_pending()
+        os.mkdir('branch2')
+        br2 = Branch.initialize('branch2')
+        self.assertRaises(NoCommits, merge, ['branch2', -1], 
+                          [None, None])
+        return br2
+
+    def test_unrelated(self):
+        br2 = self.test_nocommits()
+        commit(br2, "blah")
+        self.assertRaises(UnrelatedBranches, merge, ['branch2', -1], 
+                          [None, None])
+        return br2
+
+    def test_pending_with_null(self):
+        """When base is forced to revno 0, pending_merges is set"""
+        br2 = self.test_unrelated()
+        br1 = Branch.open('.')
+        fetch(from_branch=br2, to_branch=br1)
+        # merge all of branch 2 into branch 1 even though they 
+        # are not related.
+        merge(['branch2', -1], ['branch2', 0])
+        self.assertEquals(len(br1.pending_merges()), 1)
+        return (br1, br2)
+
+    def test_two_roots(self):
+        """Merge base is sane when two unrelated branches are merged"""
+        br1, br2 = self.test_pending_with_null()
+        commit(br1, "blah")
+        last = br1.last_revision()
+        self.assertEquals(common_ancestor(last, last, br1), last)

=== modified file 'bzrlib/selftest/testmerge3.py'
--- bzrlib/selftest/testmerge3.py	2005-07-06 05:24:29 +0000
+++ bzrlib/selftest/testmerge3.py	2005-09-04 02:59:56 +0000
@@ -15,16 +15,85 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from bzrlib.selftest import InTempDir, TestBase
+from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.merge3 import Merge3
 
-
-
-
-
-class NoChanges(TestBase):
-    """No conflicts because nothing changed"""
-    def runTest(self):
+def split_lines(t):
+    from cStringIO import StringIO
+    return StringIO(t).readlines()
+
+############################################################
+# test case data from the gnu diffutils manual
+# common base
+TZU = split_lines("""     The Nameless is the origin of Heaven and Earth;
+     The named is the mother of all things.
+     
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their outcome.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+     They both may be called deep and profound.
+     Deeper and more profound,
+     The door of all subtleties!
+""")
+
+LAO = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The Named is the mother of all things.
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their outcome.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+""")
+
+
+TAO = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The named is the mother of all things.
+     
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their result.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+     
+       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
+
+""")
+
+MERGED_RESULT = split_lines("""     The Way that can be told of is not the eternal Way;
+     The name that can be named is not the eternal name.
+     The Nameless is the origin of Heaven and Earth;
+     The Named is the mother of all things.
+     Therefore let there always be non-being,
+       so we may see their subtlety,
+     And let there always be being,
+       so we may see their result.
+     The two are the same,
+     But after they are produced,
+       they have different names.
+<<<<<<< LAO
+=======
+     
+       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
+
+>>>>>>> TAO
+""")
+
+class TestMerge3(TestCase):
+
+    def test_no_changes(self):
+        """No conflicts because nothing changed"""
         m3 = Merge3(['aaa', 'bbb'],
                     ['aaa', 'bbb'],
                     ['aaa', 'bbb'])
@@ -44,9 +113,7 @@
         self.assertEquals(list(m3.merge_groups()),
                           [('unchanged', ['aaa', 'bbb'])])
 
-
-class FrontInsert(TestBase):
-    def runTest(self):
+    def test_front_insert(self):
         m3 = Merge3(['zz'],
                     ['aaa', 'bbb', 'zz'],
                     ['zz'])
@@ -65,14 +132,10 @@
                           [('a', ['aaa', 'bbb']),
                            ('unchanged', ['zz'])])
         
-    
-
-class NullInsert(TestBase):
-    def runTest(self):
+    def test_null_insert(self):
         m3 = Merge3([],
                     ['aaa', 'bbb'],
                     [])
-
         # todo: should use a sentinal at end as from get_matching_blocks
         # to match without zz
         self.assertEquals(list(m3.find_sync_regions()),
@@ -83,12 +146,9 @@
 
         self.assertEquals(list(m3.merge_lines()),
                           ['aaa', 'bbb'])
-        
-    
 
-class NoConflicts(TestBase):
-    """No conflicts because only one side changed"""
-    def runTest(self):
+    def test_no_conflicts(self):
+        """No conflicts because only one side changed"""
         m3 = Merge3(['aaa', 'bbb'],
                     ['aaa', '111', 'bbb'],
                     ['aaa', 'bbb'])
@@ -106,10 +166,7 @@
                            ('a', 1, 2),
                            ('unchanged', 1, 2),])
 
-
-
-class AppendA(TestBase):
-    def runTest(self):
+    def test_append_a(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n'])
@@ -117,8 +174,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendB(TestBase):
-    def runTest(self):
+    def test_append_b(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'])
@@ -126,8 +182,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendAgreement(TestBase):
-    def runTest(self):
+    def test_append_agreement(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n', '222\n'])
@@ -135,8 +190,7 @@
         self.assertEquals(''.join(m3.merge_lines()),
                           'aaa\nbbb\n222\n')
 
-class AppendClash(TestBase):
-    def runTest(self):
+    def test_append_clash(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', 'bbb\n', '222\n'],
                     ['aaa\n', 'bbb\n', '333\n'])
@@ -157,9 +211,7 @@
 >> b
 ''')
 
-
-class InsertAgreement(TestBase):
-    def runTest(self):
+    def test_insert_agreement(self):
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'])
@@ -169,15 +221,11 @@
                             start_marker='<<',
                             mid_marker='--',
                             end_marker='>>')
-        self.assertEquals(''.join(m3.merge_lines()),
-                          'aaa\n222\nbbb\n')
-
-
-
-
-class InsertClash(TestBase):
-    """Both try to insert lines in the same place."""
-    def runTest(self):
+        self.assertEquals(''.join(ml), 'aaa\n222\nbbb\n')
+        
+
+    def test_insert_clash(self):
+        """Both try to insert lines in the same place."""
         m3 = Merge3(['aaa\n', 'bbb\n'],
                     ['aaa\n', '111\n', 'bbb\n'],
                     ['aaa\n', '222\n', 'bbb\n'])
@@ -216,11 +264,8 @@
 bbb
 ''')
 
-
-
-class ReplaceClash(TestBase):
-    """Both try to insert lines in the same place."""
-    def runTest(self):
+    def test_replace_clash(self):
+        """Both try to insert lines in the same place."""
         m3 = Merge3(['aaa', '000', 'bbb'],
                     ['aaa', '111', 'bbb'],
                     ['aaa', '222', 'bbb'])
@@ -233,11 +278,8 @@
                            (2,3, 2,3, 2,3),
                            (3,3, 3,3, 3,3),])
 
-
-
-class ReplaceMulti(TestBase):
-    """Replacement with regions of different size."""
-    def runTest(self):
+    def test_replace_multi(self):
+        """Replacement with regions of different size."""
         m3 = Merge3(['aaa', '000', '000', 'bbb'],
                     ['aaa', '111', '111', '111', 'bbb'],
                     ['aaa', '222', '222', '222', '222', 'bbb'])
@@ -251,112 +293,10 @@
                            (3,4, 4,5, 5,6),
                            (4,4, 5,5, 6,6),])
 
-        
-        
-
-
-
-
-def split_lines(t):
-    from cStringIO import StringIO
-    return StringIO(t).readlines()
-
-
-
-############################################################
-# test case from the gnu diffutils manual
-
-# common base
-TZU = split_lines("""     The Nameless is the origin of Heaven and Earth;
-     The named is the mother of all things.
-     
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their outcome.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-     They both may be called deep and profound.
-     Deeper and more profound,
-     The door of all subtleties!
-""")
-
-LAO = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The Named is the mother of all things.
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their outcome.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-""")
-
-
-TAO = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The named is the mother of all things.
-     
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their result.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-     
-       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
-
-""")
-
-MERGED_RESULT = split_lines("""     The Way that can be told of is not the eternal Way;
-     The name that can be named is not the eternal name.
-     The Nameless is the origin of Heaven and Earth;
-     The Named is the mother of all things.
-     Therefore let there always be non-being,
-       so we may see their subtlety,
-     And let there always be being,
-       so we may see their result.
-     The two are the same,
-     But after they are produced,
-       they have different names.
-<<<<<<<< LAO
-========
-     
-       -- The Way of Lao-Tzu, tr. Wing-tsit Chan
-
->>>>>>>> TAO
-""")
-
-
-
-class MergePoem(TestBase):
-    """Test case from diff3 manual"""
-    def runTest(self):
+    def test_merge_poem(self):
+        """Test case from diff3 manual"""
         m3 = Merge3(TZU, LAO, TAO)
         ml = list(m3.merge_lines('LAO', 'TAO'))
         self.log('merge result:')
         self.log(''.join(ml))
         self.assertEquals(ml, MERGED_RESULT)
-
-
-
-TEST_CLASSES = [
-    NoChanges,
-    FrontInsert,
-    NullInsert,
-    NoConflicts,
-    AppendA,
-    AppendB,
-    AppendAgreement,
-    AppendClash,
-    InsertAgreement,
-    InsertClash,
-    ReplaceClash,
-    ReplaceMulti,
-    MergePoem,
-    ]

=== added file 'bzrlib/selftest/testrevision.py'
--- bzrlib/selftest/testrevision.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testrevision.py	2005-10-02 23:17:00 +0000
@@ -0,0 +1,300 @@
+# (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.branch import Branch
+from bzrlib.commit import commit
+from bzrlib.fetch import fetch
+from bzrlib.revision import (find_present_ancestors, combined_graph,
+                             is_ancestor, MultipleRevisionSources)
+from bzrlib.trace import mutter
+from bzrlib.errors import NoSuchRevision
+
+def make_branches():
+    """Create two branches
+
+    branch 1 has 6 commits, branch 2 has 3 commits
+    commit 10 was a psuedo merge from branch 1
+    but has been disabled until ghost support is
+    implemented.
+
+    the object graph is
+    B:     A:
+    a..0   a..0 
+    a..1   a..1
+    a..2   a..2
+    b..3   a..3 merges b..4
+    b..4   a..4
+    b..5   a..5 merges b..5
+    b..6 merges a4
+
+    so A is missing b6 at the start
+    and B is missing a3, a4, a5
+    """
+    os.mkdir("branch1")
+    br1 = Branch.initialize("branch1")
+    
+    commit(br1, "Commit one", rev_id="a@u-0-0")
+    commit(br1, "Commit two", rev_id="a@u-0-1")
+    commit(br1, "Commit three", rev_id="a@u-0-2")
+
+    os.mkdir("branch2")
+    br2 = Branch.initialize("branch2")
+    br2.update_revisions(br1)
+    commit(br2, "Commit four", rev_id="b@u-0-3")
+    commit(br2, "Commit five", rev_id="b@u-0-4")
+    revisions_2 = br2.revision_history()
+    
+    fetch(from_branch=br2, to_branch=br1)
+    br1.add_pending_merge(revisions_2[4])
+    assert revisions_2[4] == 'b@u-0-4'
+    commit(br1, "Commit six", rev_id="a@u-0-3")
+    commit(br1, "Commit seven", rev_id="a@u-0-4")
+    commit(br2, "Commit eight", rev_id="b@u-0-5")
+    
+    fetch(from_branch=br2, to_branch=br1)
+    br1.add_pending_merge(br2.revision_history()[5])
+    commit(br1, "Commit nine", rev_id="a@u-0-5")
+    # DO NOT FETCH HERE - we WANT a GHOST.
+    #fetch(from_branch=br1, to_branch=br2)
+    br2.add_pending_merge(br1.revision_history()[4])
+    commit(br2, "Commit ten - ghost merge", rev_id="b@u-0-6")
+    
+    return br1, br2
+
+
+class TestIsAncestor(TestCaseInTempDir):
+    def test_recorded_ancestry(self):
+        """Test that commit records all ancestors"""
+        br1, br2 = make_branches()
+        d = [('a@u-0-0', ['a@u-0-0']),
+             ('a@u-0-1', ['a@u-0-0', 'a@u-0-1']),
+             ('a@u-0-2', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2']),
+             ('b@u-0-3', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3']),
+             ('b@u-0-4', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3',
+                          'b@u-0-4']),
+             ('a@u-0-3', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'a@u-0-3']),
+             ('a@u-0-4', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'a@u-0-3', 'a@u-0-4']),
+             ('b@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5']),
+             ('a@u-0-5', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4',
+                          'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5', 'a@u-0-5']),
+             ('b@u-0-6', ['a@u-0-0', 'a@u-0-1', 'a@u-0-2',
+                          'b@u-0-3', 'b@u-0-4',
+                          'b@u-0-5', 'b@u-0-6']),
+             ]
+        br1_only = ('a@u-0-3', 'a@u-0-4', 'a@u-0-5')
+        br2_only = ('b@u-0-6',)
+        for branch in br1, br2:
+            for rev_id, anc in d:
+                if rev_id in br1_only and not branch is br1:
+                    continue
+                if rev_id in br2_only and not branch is br2:
+                    continue
+                mutter('ancestry of {%s}: %r',
+                       rev_id, branch.get_ancestry(rev_id))
+                self.assertEquals(sorted(branch.get_ancestry(rev_id)),
+                                  [None] + sorted(anc))
+    
+    
+    def test_is_ancestor(self):
+        """Test checking whether a revision is an ancestor of another revision"""
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = br1
+
+        assert is_ancestor(revisions[0], revisions[0], br1)
+        assert is_ancestor(revisions[1], revisions[0], sources)
+        assert not is_ancestor(revisions[0], revisions[1], sources)
+        assert is_ancestor(revisions_2[3], revisions[0], sources)
+        # disabled mbp 20050914, doesn't seem to happen anymore
+        ## self.assertRaises(NoSuchRevision, is_ancestor, revisions_2[3],
+        ##                  revisions[0], br1)        
+        assert is_ancestor(revisions[3], revisions_2[4], sources)
+        assert is_ancestor(revisions[3], revisions_2[4], br1)
+        assert is_ancestor(revisions[3], revisions_2[3], sources)
+        ## assert not is_ancestor(revisions[3], revisions_2[3], br1)
+
+
+class TestIntermediateRevisions(TestCaseInTempDir):
+
+    def setUp(self):
+        from bzrlib.commit import commit
+        TestCaseInTempDir.setUp(self)
+        self.br1, self.br2 = make_branches()
+
+        self.br2.commit("Commit eleven", rev_id="b@u-0-7")
+        self.br2.commit("Commit twelve", rev_id="b@u-0-8")
+        self.br2.commit("Commit thirtteen", rev_id="b@u-0-9")
+
+        fetch(from_branch=self.br2, to_branch=self.br1)
+        self.br1.add_pending_merge(self.br2.revision_history()[6])
+        self.br1.commit("Commit fourtten", rev_id="a@u-0-6")
+
+        fetch(from_branch=self.br1, to_branch=self.br2)
+        self.br2.add_pending_merge(self.br1.revision_history()[6])
+        self.br2.commit("Commit fifteen", rev_id="b@u-0-10")
+
+        from bzrlib.revision import MultipleRevisionSources
+        self.sources = MultipleRevisionSources(self.br1, self.br2)
+
+    def intervene(self, ancestor, revision, revision_history=None):
+        from bzrlib.revision import get_intervening_revisions
+        return get_intervening_revisions(ancestor,revision, self.sources, 
+                                         revision_history)
+
+    def test_intervene(self):
+        """Find intermediate revisions, without requiring history"""
+        from bzrlib.errors import NotAncestor, NoSuchRevision
+        assert len(self.intervene('a@u-0-0', 'a@u-0-0')) == 0
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-1'), ['a@u-0-1'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-2'), 
+                         ['a@u-0-1', 'a@u-0-2'])
+        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-3'), 
+                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3'])
+        self.assertEqual(self.intervene('b@u-0-3', 'a@u-0-3'), 
+                         ['b@u-0-4', 'a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-2', 'a@u-0-3', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-3'])
+        self.assertEqual(self.intervene('a@u-0-0', 'a@u-0-5', 
+                                        self.br1.revision_history()), 
+                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+                          'a@u-0-5'])
+        print ("testrevision.py 191 - intervene appears to return b..6 even"
+               "though it is not reachable!")
+#        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-6', 
+#                         self.br1.revision_history()), 
+#                         ['a@u-0-1', 'a@u-0-2', 'a@u-0-3', 'a@u-0-4', 
+#                          'b@u-0-6'])
+#        self.assertEqual(self.intervene('a@u-0-0', 'b@u-0-5'), 
+#                         ['a@u-0-1', 'a@u-0-2', 'b@u-0-3', 'b@u-0-4', 
+#                          'b@u-0-5'])
+        self.assertEqual(self.intervene('b@u-0-3', 'b@u-0-6', 
+                         self.br2.revision_history()), 
+                         ['b@u-0-4', 'b@u-0-5', 'b@u-0-6'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10'), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertEqual(self.intervene('b@u-0-6', 'b@u-0-10', 
+                                        self.br2.revision_history()), 
+                         ['b@u-0-7', 'b@u-0-8', 'b@u-0-9', 'b@u-0-10'])
+        self.assertRaises(NotAncestor, self.intervene, 'b@u-0-10', 'b@u-0-6', 
+                          self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'c@u-0-10', 
+                          'b@u-0-6', self.br2.revision_history())
+        self.assertRaises(NoSuchRevision, self.intervene, 'b@u-0-10', 
+                          'c@u-0-6', self.br2.revision_history())
+
+
+class TestCommonAncestor(TestCaseInTempDir):
+    """Test checking whether a revision is an ancestor of another revision"""
+
+    def test_old_common_ancestor(self):
+        """Pick a resonable merge base using the old functionality"""
+        from bzrlib.revision import old_common_ancestor as common_ancestor
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = br1
+
+        expected_ancestors_list = {revisions[3]:(0, 0), 
+                                   revisions[2]:(1, 1),
+                                   revisions_2[4]:(2, 1), 
+                                   revisions[1]:(3, 2),
+                                   revisions_2[3]:(4, 2),
+                                   revisions[0]:(5, 3) }
+        ancestors_list = find_present_ancestors(revisions[3], sources)
+        assert len(expected_ancestors_list) == len(ancestors_list)
+        for key, value in expected_ancestors_list.iteritems():
+            self.assertEqual(ancestors_list[key], value, 
+                              "key %r, %r != %r" % (key, ancestors_list[key],
+                                                    value))
+
+        self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
+                          revisions[0])
+        self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[1], revisions[1], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[2], revisions_2[4], sources),
+                          revisions[2])
+        self.assertEqual(common_ancestor(revisions[3], revisions_2[4], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
+                          revisions_2[4])
+        fetch(from_branch=br2, to_branch=br1)
+        self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
+                          revisions[4]) # revisions_2[5] is equally valid
+        self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
+                          revisions_2[5])
+
+    def test_common_ancestor(self):
+        """Pick a reasonable merge base"""
+        from bzrlib.revision import common_ancestor
+        br1, br2 = make_branches()
+        revisions = br1.revision_history()
+        revisions_2 = br2.revision_history()
+        sources = MultipleRevisionSources(br1, br2)
+        expected_ancestors_list = {revisions[3]:(0, 0), 
+                                   revisions[2]:(1, 1),
+                                   revisions_2[4]:(2, 1), 
+                                   revisions[1]:(3, 2),
+                                   revisions_2[3]:(4, 2),
+                                   revisions[0]:(5, 3) }
+        ancestors_list = find_present_ancestors(revisions[3], sources)
+        assert len(expected_ancestors_list) == len(ancestors_list)
+        for key, value in expected_ancestors_list.iteritems():
+            self.assertEqual(ancestors_list[key], value, 
+                              "key %r, %r != %r" % (key, ancestors_list[key],
+                                                    value))
+        self.assertEqual(common_ancestor(revisions[0], revisions[0], sources),
+                          revisions[0])
+        self.assertEqual(common_ancestor(revisions[1], revisions[2], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[1], revisions[1], sources),
+                          revisions[1])
+        self.assertEqual(common_ancestor(revisions[2], revisions_2[4], sources),
+                          revisions[2])
+        self.assertEqual(common_ancestor(revisions[3], revisions_2[4], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[4], revisions_2[5], sources),
+                          revisions_2[4])
+        self.assertEqual(common_ancestor(revisions[5], revisions_2[6], sources),
+                          revisions[4]) # revisions_2[5] is equally valid
+        self.assertEqual(common_ancestor(revisions_2[6], revisions[5], sources),
+                          revisions[4]) # revisions_2[5] is equally valid
+
+    def test_combined(self):
+        """combined_graph
+        Ensure it's not order-sensitive
+        """
+        br1, br2 = make_branches()
+        source = MultipleRevisionSources(br1, br2)
+        combined_1 = combined_graph(br1.last_revision(), 
+                                    br2.last_revision(), source)
+        combined_2 = combined_graph(br2.last_revision(),
+                                    br1.last_revision(), source)
+        assert combined_1[1] == combined_2[1]
+        assert combined_1[2] == combined_2[2]
+        assert combined_1[3] == combined_2[3]
+        assert combined_1 == combined_2

=== modified file 'bzrlib/selftest/testrevisionnamespaces.py'
--- bzrlib/selftest/testrevisionnamespaces.py	2005-07-11 06:13:18 +0000
+++ bzrlib/selftest/testrevisionnamespaces.py	2005-09-29 20:34:25 +0000
@@ -14,33 +14,67 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-
-from bzrlib.selftest import InTempDir, TestBase
-
-class TestRevisionNamespaces(InTempDir):
-    """Functional tests for hashcache"""
-    def runTest(self):
-        from bzrlib.errors import BzrError
+import os
+import time
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.errors import NoCommonAncestor, NoCommits
+from bzrlib.clone import copy_branch
+from bzrlib.merge import merge
+
+class TestRevisionNamespaces(TestCaseInTempDir):
+
+    def test_revision_namespaces(self):
+        """Test revision specifiers.
+
+        These identify revisions by date, etc."""
+        from bzrlib.errors import NoSuchRevision
         from bzrlib.branch import Branch
-        import os
-        import time
-
-        b = Branch('.', init=True)
-
-        b.commit('Commit one', rev_id='a@r-0-1')
+        from bzrlib.revisionspec import RevisionSpec
+
+        b = Branch.initialize('.')
+
+        b.commit('Commit one', rev_id='a@r-0-1', timestamp=time.time() - 60*60*24)
         b.commit('Commit two', rev_id='a@r-0-2')
         b.commit('Commit three', rev_id='a@r-0-3')
 
-        self.assertEquals(b.get_revision_info(1), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revno:1'), (1, 'a@r-0-1'))
-        self.assertEquals(b.get_revision_info('revid:a@r-0-1'), (1, 'a@r-0-1'))
-        self.assertRaises(BzrError, b.get_revision_info, 'revid:a@r-0-0')
-
-        self.assertEquals(b.get_revision_info('date:-tomorrow'), (3, 'a@r-0-3'))
-        self.assertEquals(b.get_revision_info('date:+today'), (1, 'a@r-0-1'))
-
-        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
-
-TEST_CLASSES = [
-    TestRevisionNamespaces
-    ]
+        self.assertEquals(RevisionSpec(None).in_history(b), (0, None))
+        self.assertEquals(RevisionSpec(1).in_history(b), (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revno:1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('revid:a@r-0-1').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertRaises(NoSuchRevision,
+                          RevisionSpec('revid:a@r-0-0').in_history, b)
+        self.assertRaises(TypeError, RevisionSpec, object)
+
+        self.assertEquals(RevisionSpec('date:today').in_history(b),
+                          (2, 'a@r-0-2'))
+        self.assertEquals(RevisionSpec('date:yesterday').in_history(b),
+                          (1, 'a@r-0-1'))
+        self.assertEquals(RevisionSpec('before:date:today').in_history(b),
+                          (1, 'a@r-0-1'))
+
+        self.assertEquals(RevisionSpec('last:1').in_history(b),
+                          (3, 'a@r-0-3'))
+        self.assertEquals(RevisionSpec('-1').in_history(b), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('last:1'), (3, 'a@r-0-3'))
+#        self.assertEquals(b.get_revision_info('-1'), (3, 'a@r-0-3'))
+
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b).rev_id,
+                          'a@r-0-3')
+
+        os.mkdir('newbranch')
+        b2 = Branch.initialize('newbranch')
+        self.assertRaises(NoCommits, RevisionSpec('ancestor:.').in_history, b2)
+
+        os.mkdir('copy')
+        b3 = copy_branch(b, 'copy')
+        b3.commit('Commit four', rev_id='b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'a@r-0-3')
+        merge(['copy', -1], [None, None])
+        b.commit('Commit five', rev_id='a@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:copy').in_history(b).rev_id,
+                          'b@r-0-4')
+        self.assertEquals(RevisionSpec('ancestor:.').in_history(b3).rev_id,
+                          'b@r-0-4')

=== added file 'bzrlib/selftest/testsampler.py'
--- bzrlib/selftest/testsampler.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testsampler.py	2005-10-06 11:40:12 +0000
@@ -0,0 +1,39 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Document what this test file is expecting to test here.
+
+If you need more than one line, make the first line a good sentence on its
+own and add more explanation here, like this.
+
+Be sure to register your new test script in bzrlib/selftest/__init__.py -
+search for sampler in there.
+"""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+from bzrlib.selftest import TestCaseInTempDir
+
+
+# Now we need a test script:
+class DemoTest(TestCaseInTempDir):
+
+    def test_nothing(self):
+        self.assertEqual(1,1)

=== modified file 'bzrlib/selftest/teststatus.py'
--- bzrlib/selftest/teststatus.py	2005-07-12 01:44:23 +0000
+++ bzrlib/selftest/teststatus.py	2005-10-03 01:53:47 +0000
@@ -21,16 +21,18 @@
 """
 
 
-from bzrlib.selftest import InTempDir
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.revisionspec import RevisionSpec
 
-class BranchStatus(InTempDir):
-    def runTest(self): 
-        """Basic 'bzr mkdir' operation"""
+class BranchStatus(TestCaseInTempDir):
+    
+    def test_branch_status(self): 
+        """Test basic branch status"""
         from cStringIO import StringIO
         from bzrlib.status import show_status
         from bzrlib.branch import Branch
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         # status with nothing
         tof = StringIO()
@@ -39,11 +41,100 @@
 
         tof = StringIO()
         self.build_tree(['hello.c', 'bye.c'])
+        b.add_pending_merge('pending@pending-0-0')
         show_status(b, to_file=tof)
         tof.seek(0)
         self.assertEquals(tof.readlines(),
                           ['unknown:\n',
                            '  bye.c\n',
                            '  hello.c\n',
-                           ])
-
+                           'pending merges:\n',
+                           '  pending@pending-0-0\n'
+                           ])
+
+    def test_branch_status_revisions(self):
+        """Tests branch status with revisions"""
+        from cStringIO import StringIO
+        from bzrlib.status import show_status
+        from bzrlib.branch import Branch
+        
+        b = Branch.initialize('.')
+
+        tof = StringIO()
+        self.build_tree(['hello.c', 'bye.c'])
+        b.add('hello.c')
+        b.add('bye.c')
+        b.commit('Test message')
+
+        tof = StringIO()
+        revs =[]
+        revs.append(RevisionSpec(0))
+        
+        show_status(b, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+
+        self.build_tree(['more.c'])
+        b.add('more.c')
+        b.commit('Another test message')
+        
+        tof = StringIO()
+        revs.append(RevisionSpec(1))
+        
+        show_status(b, to_file=tof, revision=revs)
+        
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['added:\n',
+                           '  bye.c\n',
+                           '  hello.c\n'])
+
+    def test_branch_status_specific_files(self): 
+        """Tests branch status with given specific files"""
+        from cStringIO import StringIO
+        from bzrlib.status import show_status
+        from bzrlib.branch import Branch
+        
+        b = Branch.initialize('.')
+
+        self.build_tree(['directory/','directory/hello.c', 'bye.c','test.c','dir2/'])
+        b.add('directory')
+        b.add('test.c')
+        b.commit('testing')
+        
+        tof = StringIO()
+        show_status(b, to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  bye.c\n',
+                           '  dir2\n',
+                           '  directory/hello.c\n'
+                           ])
+
+        tof = StringIO()
+        show_status(b, specific_files=['bye.c','test.c','absent.c'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  bye.c\n'
+                           ])
+        
+        tof = StringIO()
+        show_status(b, specific_files=['directory'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  directory/hello.c\n'
+                           ])
+        tof = StringIO()
+        show_status(b, specific_files=['dir2'], to_file=tof)
+        tof.seek(0)
+        self.assertEquals(tof.readlines(),
+                          ['unknown:\n',
+                           '  dir2\n'
+                           ])

=== added file 'bzrlib/selftest/teststore.py'
--- bzrlib/selftest/teststore.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/teststore.py	2005-10-05 06:44:02 +0000
@@ -0,0 +1,208 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Test Store implementation
+"""
+from cStringIO import StringIO
+import os
+
+from bzrlib.store import copy_all
+from bzrlib.transport.local import LocalTransport
+from bzrlib.transport import NoSuchFile
+from bzrlib.store.compressed_text import CompressedTextStore
+from bzrlib.store.text import TextStore
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+from bzrlib.errors import BzrError, UnlistableStore
+import bzrlib.store
+
+
+def fill_store(store):
+    store.add(StringIO('hello'), 'a')
+    store.add(StringIO('other'), 'b')
+    store.add(StringIO('something'), 'c')
+    store.add(StringIO('goodbye'), '123123')
+
+def check_equals(tester, store, files, values, permit_failure=False):
+    files = store.get(files, permit_failure=permit_failure)
+    count = 0
+    for f, v in zip(files, values):
+        count += 1
+        if v is None:
+            tester.assert_(f is None)
+        else:
+            tester.assertEquals(f.read(), v)
+    tester.assertEquals(count, len(values))
+    # We need to check to make sure there are no more
+    # files to be returned, I'm using a cheezy way
+    # Convert to a list, and there shouldn't be any left
+    tester.assertEquals(len(list(files)), 0)
+
+def test_multiple_add(tester, store):
+    fill_store(store)
+    tester.assertRaises(BzrError, store.add, StringIO('goodbye'), '123123')
+
+def test_get(tester, store):
+    fill_store(store)
+
+    check_equals(tester, store, ['a'], ['hello'])
+    check_equals(tester, store, ['b', 'c'], ['other', 'something'])
+
+    # Make sure that requesting a non-existing file fails
+    tester.assertRaises(NoSuchFile, check_equals, tester, store,
+            ['d'], [None])
+    tester.assertRaises(NoSuchFile, check_equals, tester, store,
+            ['a', 'd'], ['hello', None])
+    tester.assertRaises(NoSuchFile, check_equals, tester, store,
+            ['d', 'a'], [None, 'hello'])
+    tester.assertRaises(NoSuchFile, check_equals, tester, store,
+            ['d', 'd', 'd'], [None, None, None])
+    tester.assertRaises(NoSuchFile, check_equals, tester, store,
+            ['a', 'd', 'b'], ['hello', None, 'other'])
+
+def test_ignore_get(tester, store):
+    fill_store(store)
+
+    files = store.get(['d'], permit_failure=True)
+    files = list(files)
+    tester.assertEquals(len(files), 1)
+    tester.assert_(files[0] is None)
+
+    check_equals(tester, store, ['a', 'd'], ['hello', None],
+            permit_failure=True)
+    check_equals(tester, store, ['d', 'a'], [None, 'hello'],
+            permit_failure=True)
+    check_equals(tester, store, ['d', 'd'], [None, None],
+            permit_failure=True)
+    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
+            permit_failure=True)
+    check_equals(tester, store, ['a', 'd', 'b'], ['hello', None, 'other'],
+            permit_failure=True)
+    check_equals(tester, store, ['b', 'd', 'c'], ['other', None, 'something'],
+            permit_failure=True)
+
+
+def get_compressed_store(path='.'):
+    t = LocalTransport(path)
+    return CompressedTextStore(t)
+
+
+def get_text_store(path='.'):
+    t = LocalTransport(path)
+    return TextStore(t)
+
+
+class TestCompressedTextStore(TestCaseInTempDir):
+
+    def test_multiple_add(self):
+        """Multiple add with same ID should raise a BzrError"""
+        store = get_compressed_store()
+        test_multiple_add(self, store)
+
+    def test_get(self):
+        store = get_compressed_store()
+        test_get(self, store)
+
+    def test_ignore_get(self):
+        store = get_compressed_store()
+        test_ignore_get(self, store)
+
+    def test_total_size(self):
+        store = get_compressed_store('.')
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        # these get gzipped - content should be stable
+        self.assertEqual(store.total_size(), (2, 55))
+        
+    def test_copy_all(self):
+        """Test copying"""
+        os.mkdir('a')
+        store_a = get_text_store('a')
+        store_a.add('foo', '1')
+        os.mkdir('b')
+        store_b = get_text_store('b')
+        copy_all(store_a, store_b)
+        self.assertEqual(store_a['1'].read(), 'foo')
+        self.assertEqual(store_b['1'].read(), 'foo')
+
+
+class TestMemoryStore(TestCase):
+    
+    def get_store(self):
+        return bzrlib.store.ImmutableMemoryStore()
+    
+    def test_imports(self):
+        from bzrlib.store import ImmutableMemoryStore
+
+    def test_add_and_retrieve(self):
+        store = self.get_store()
+        store.add(StringIO('hello'), 'aa')
+        self.assertNotEqual(store['aa'], None)
+        self.assertEqual(store['aa'].read(), 'hello')
+        store.add(StringIO('hello world'), 'bb')
+        self.assertNotEqual(store['bb'], None)
+        self.assertEqual(store['bb'].read(), 'hello world')
+
+    def test_missing_is_absent(self):
+        store = self.get_store()
+        self.failIf('aa' in store)
+
+    def test_adding_fails_when_present(self):
+        store = self.get_store()
+        store.add(StringIO('hello'), 'aa')
+        self.assertRaises(bzrlib.store.StoreError,
+                          store.add, StringIO('hello'), 'aa')
+
+    def test_total_size(self):
+        store = self.get_store()
+        store.add(StringIO('goodbye'), '123123')
+        store.add(StringIO('goodbye2'), '123123.dsc')
+        self.assertEqual(store.total_size(), (2, 15))
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)
+
+
+class TestTextStore(TestCaseInTempDir):
+    def test_multiple_add(self):
+        """Multiple add with same ID should raise a BzrError"""
+        store = get_text_store()
+        test_multiple_add(self, store)
+
+    def test_get(self):
+        store = get_text_store()
+        test_get(self, store)
+
+    def test_ignore_get(self):
+        store = get_text_store()
+        test_ignore_get(self, store)
+
+    def test_copy_all(self):
+        """Test copying"""
+        os.mkdir('a')
+        store_a = get_text_store('a')
+        store_a.add('foo', '1')
+        os.mkdir('b')
+        store_b = get_text_store('b')
+        copy_all(store_a, store_b)
+        self.assertEqual(store_a['1'].read(), 'foo')
+        self.assertEqual(store_b['1'].read(), 'foo')
+        # TODO: Switch the exception form UnlistableStore to
+        #       or make Stores throw UnlistableStore if their
+        #       Transport doesn't support listing
+        # store_c = RemoteStore('http://example.com/')
+        # self.assertRaises(UnlistableStore, copy_all, store_c, store_b)

=== added file 'bzrlib/selftest/testtransactions.py'
--- bzrlib/selftest/testtransactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testtransactions.py	2005-10-07 06:13:14 +0000
@@ -0,0 +1,106 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Tests for the behaviour of the Transaction concept in bzr."""
+
+# import system imports here
+import os
+import sys
+
+#import bzrlib specific imports here
+import bzrlib.errors as errors
+from bzrlib.selftest import TestCase, TestCaseInTempDir
+import bzrlib.transactions as transactions
+
+
+class TestReadOnlyTransaction(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.transactions import ReadOnlyTransaction
+
+    def test_construct(self):
+        transactions.ReadOnlyTransaction()
+
+    def test_register_clean(self):
+        transaction = transactions.ReadOnlyTransaction()
+        transaction.register_clean("anobject")
+
+    def test_register_dirty_raises(self):
+        transaction = transactions.ReadOnlyTransaction()
+        self.assertRaises(errors.ReadOnlyError, 
+                          transaction.register_dirty,"anobject")
+    
+    def test_commit_raises(self):
+        transaction = transactions.ReadOnlyTransaction()
+        self.assertRaises(errors.CommitNotPossible, transaction.commit)
+
+    def test_map(self):
+        transaction = transactions.ReadOnlyTransaction()
+        self.assertNotEqual(None, getattr(transaction, "map", None))
+    
+    def test_add_and_get(self):
+        transaction = transactions.ReadOnlyTransaction()
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(weave, transaction.map.find_weave("id"))
+
+    def test_rollback_returns(self):
+        transaction = transactions.ReadOnlyTransaction()
+        transaction.rollback()
+
+    def test_finish_returns(self):
+        transaction = transactions.ReadOnlyTransaction()
+        transaction.finish()
+
+        
+class TestPassThroughTransaction(TestCase):
+
+    def test_symbols(self):
+        from bzrlib.transactions import PassThroughTransaction
+
+    def test_construct(self):
+        transactions.PassThroughTransaction()
+
+    def test_register_clean(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_clean("anobject")
+    
+    def test_register_dirty(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.register_dirty("anobject")
+    
+    def test_commit_nothing_returns(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.commit()
+
+    def test_map(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertNotEqual(None, getattr(transaction, "map", None))
+    
+    def test_add_and_get(self):
+        transaction = transactions.PassThroughTransaction()
+        weave = "a weave"
+        transaction.map.add_weave("id", weave)
+        self.assertEqual(None, transaction.map.find_weave("id"))
+        
+    def test_rollback_asserts(self):
+        transaction = transactions.PassThroughTransaction()
+        self.assertRaises(errors.AlreadyCommitted, transaction.rollback)
+
+    def test_finish_returns(self):
+        transaction = transactions.PassThroughTransaction()
+        transaction.finish()

=== added file 'bzrlib/selftest/testtransport.py'
--- bzrlib/selftest/testtransport.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testtransport.py	2005-09-19 03:26:14 +0000
@@ -0,0 +1,515 @@
+# Copyright (C) 2004, 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+import os
+from cStringIO import StringIO
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.selftest.HTTPTestUtil import TestCaseWithWebserver
+from bzrlib.errors import NoSuchFile, FileExists, TransportNotPossible
+
+def _append(fn, txt):
+    """Append the given text (file-like object) to the supplied filename."""
+    f = open(fn, 'ab')
+    f.write(txt)
+    f.flush()
+    f.close()
+    del f
+
+class TestTransportMixIn(object):
+    """Subclass this, and it will provide a series of tests for a Transport.
+    It assumes that the Transport object is connected to the 
+    current working directory.  So that whatever is done 
+    through the transport, should show up in the working 
+    directory, and vice-versa.
+
+    This also tests to make sure that the functions work with both
+    generators and lists (assuming iter(list) is effectively a generator)
+    """
+    readonly = False
+    def get_transport(self):
+        """Children should override this to return the Transport object.
+        """
+        raise NotImplementedError
+
+    def test_has(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g']
+        self.build_tree(files)
+        self.assertEqual(t.has('a'), True)
+        self.assertEqual(t.has('c'), False)
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h'])),
+                [True, True, False, False, True, False, True, False])
+        self.assertEqual(list(t.has_multi(iter(['a', 'b', 'c', 'd', 'e', 'f', 'g', 'h']))),
+                [True, True, False, False, True, False, True, False])
+
+    def test_get(self):
+        t = self.get_transport()
+
+        files = ['a', 'b', 'e', 'g']
+        self.build_tree(files)
+        self.assertEqual(t.get('a').read(), open('a').read())
+        content_f = t.get_multi(files)
+        for path,f in zip(files, content_f):
+            self.assertEqual(open(path).read(), f.read())
+
+        content_f = t.get_multi(iter(files))
+        for path,f in zip(files, content_f):
+            self.assertEqual(open(path).read(), f.read())
+
+        self.assertRaises(NoSuchFile, t.get, 'c')
+        try:
+            files = list(t.get_multi(['a', 'b', 'c']))
+        except NoSuchFile:
+            pass
+        else:
+            self.fail('Failed to raise NoSuchFile for missing file in get_multi')
+        try:
+            files = list(t.get_multi(iter(['a', 'b', 'c', 'e'])))
+        except NoSuchFile:
+            pass
+        else:
+            self.fail('Failed to raise NoSuchFile for missing file in get_multi')
+
+    def test_put(self):
+        t = self.get_transport()
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.put, 'a', 'some text for a\n')
+            open('a', 'wb').write('some text for a\n')
+        else:
+            t.put('a', 'some text for a\n')
+        self.assert_(os.path.exists('a'))
+        self.check_file_contents('a', 'some text for a\n')
+        self.assertEqual(t.get('a').read(), 'some text for a\n')
+        # Make sure 'has' is updated
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, False, False])
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.put_multi,
+                    [('a', 'new\ncontents for\na\n'),
+                        ('d', 'contents\nfor d\n')])
+            open('a', 'wb').write('new\ncontents for\na\n')
+            open('d', 'wb').write('contents\nfor d\n')
+        else:
+            # Put also replaces contents
+            self.assertEqual(t.put_multi([('a', 'new\ncontents for\na\n'),
+                                          ('d', 'contents\nfor d\n')]),
+                             2)
+        self.assertEqual(list(t.has_multi(['a', 'b', 'c', 'd', 'e'])),
+                [True, False, False, True, False])
+        self.check_file_contents('a', 'new\ncontents for\na\n')
+        self.check_file_contents('d', 'contents\nfor d\n')
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                t.put_multi, iter([('a', 'diff\ncontents for\na\n'),
+                                  ('d', 'another contents\nfor d\n')]))
+            open('a', 'wb').write('diff\ncontents for\na\n')
+            open('d', 'wb').write('another contents\nfor d\n')
+        else:
+            self.assertEqual(
+                t.put_multi(iter([('a', 'diff\ncontents for\na\n'),
+                                  ('d', 'another contents\nfor d\n')]))
+                             , 2)
+        self.check_file_contents('a', 'diff\ncontents for\na\n')
+        self.check_file_contents('d', 'another contents\nfor d\n')
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.put, 'path/doesnt/exist/c', 'contents')
+        else:
+            self.assertRaises(NoSuchFile,
+                    t.put, 'path/doesnt/exist/c', 'contents')
+
+    def test_put_file(self):
+        t = self.get_transport()
+
+        # Test that StringIO can be used as a file-like object with put
+        f1 = StringIO('this is a string\nand some more stuff\n')
+        if self.readonly:
+            open('f1', 'wb').write(f1.read())
+        else:
+            t.put('f1', f1)
+
+        del f1
+
+        self.check_file_contents('f1', 
+                'this is a string\nand some more stuff\n')
+
+        f2 = StringIO('here is some text\nand a bit more\n')
+        f3 = StringIO('some text for the\nthird file created\n')
+
+        if self.readonly:
+            open('f2', 'wb').write(f2.read())
+            open('f3', 'wb').write(f3.read())
+        else:
+            t.put_multi([('f2', f2), ('f3', f3)])
+
+        del f2, f3
+
+        self.check_file_contents('f2', 'here is some text\nand a bit more\n')
+        self.check_file_contents('f3', 'some text for the\nthird file created\n')
+
+        # Test that an actual file object can be used with put
+        f4 = open('f1', 'rb')
+        if self.readonly:
+            open('f4', 'wb').write(f4.read())
+        else:
+            t.put('f4', f4)
+
+        del f4
+
+        self.check_file_contents('f4', 
+                'this is a string\nand some more stuff\n')
+
+        f5 = open('f2', 'rb')
+        f6 = open('f3', 'rb')
+        if self.readonly:
+            open('f5', 'wb').write(f5.read())
+            open('f6', 'wb').write(f6.read())
+        else:
+            t.put_multi([('f5', f5), ('f6', f6)])
+
+        del f5, f6
+
+        self.check_file_contents('f5', 'here is some text\nand a bit more\n')
+        self.check_file_contents('f6', 'some text for the\nthird file created\n')
+
+
+
+    def test_mkdir(self):
+        t = self.get_transport()
+
+        # Test mkdir
+        os.mkdir('dir_a')
+        self.assertEqual(t.has('dir_a'), True)
+        self.assertEqual(t.has('dir_b'), False)
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.mkdir, 'dir_b')
+            os.mkdir('dir_b')
+        else:
+            t.mkdir('dir_b')
+        self.assertEqual(t.has('dir_b'), True)
+        self.assert_(os.path.isdir('dir_b'))
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.mkdir_multi, ['dir_c', 'dir_d'])
+            os.mkdir('dir_c')
+            os.mkdir('dir_d')
+        else:
+            t.mkdir_multi(['dir_c', 'dir_d'])
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.mkdir_multi, iter(['dir_e', 'dir_f']))
+            os.mkdir('dir_e')
+            os.mkdir('dir_f')
+        else:
+            t.mkdir_multi(iter(['dir_e', 'dir_f']))
+        self.assertEqual(list(t.has_multi(
+            ['dir_a', 'dir_b', 'dir_c', 'dir_q',
+             'dir_d', 'dir_e', 'dir_f', 'dir_b'])),
+            [True, True, True, False,
+             True, True, True, True])
+        for d in ['dir_a', 'dir_b', 'dir_c', 'dir_d', 'dir_e', 'dir_f']:
+            self.assert_(os.path.isdir(d))
+
+        if not self.readonly:
+            self.assertRaises(NoSuchFile, t.mkdir, 'path/doesnt/exist')
+            self.assertRaises(FileExists, t.mkdir, 'dir_a') # Creating a directory again should fail
+
+        # Make sure the transport recognizes when a
+        # directory is created by other means
+        # Caching Transports will fail, because dir_e was already seen not
+        # to exist. So instead, we will search for a new directory
+        #os.mkdir('dir_e')
+        #if not self.readonly:
+        #    self.assertRaises(FileExists, t.mkdir, 'dir_e')
+
+        os.mkdir('dir_g')
+        if not self.readonly:
+            self.assertRaises(FileExists, t.mkdir, 'dir_g')
+
+        # Test get/put in sub-directories
+        if self.readonly:
+            open('dir_a/a', 'wb').write('contents of dir_a/a')
+            open('dir_b/b', 'wb').write('contents of dir_b/b')
+        else:
+            self.assertEqual(
+                t.put_multi([('dir_a/a', 'contents of dir_a/a'),
+                             ('dir_b/b', 'contents of dir_b/b')])
+                          , 2)
+        for f in ('dir_a/a', 'dir_b/b'):
+            self.assertEqual(t.get(f).read(), open(f).read())
+
+    def test_copy_to(self):
+        import tempfile
+        from bzrlib.transport.local import LocalTransport
+
+        t = self.get_transport()
+
+        files = ['a', 'b', 'c', 'd']
+        self.build_tree(files)
+
+        dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
+        dtmp_base = os.path.basename(dtmp)
+        local_t = LocalTransport(dtmp)
+
+        t.copy_to(files, local_t)
+        for f in files:
+            self.assertEquals(open(f).read(),
+                    open(os.path.join(dtmp_base, f)).read())
+
+        del dtmp, dtmp_base, local_t
+
+        dtmp = tempfile.mkdtemp(dir='.', prefix='test-transport-')
+        dtmp_base = os.path.basename(dtmp)
+        local_t = LocalTransport(dtmp)
+
+        files = ['a', 'b', 'c', 'd']
+        t.copy_to(iter(files), local_t)
+        for f in files:
+            self.assertEquals(open(f).read(),
+                    open(os.path.join(dtmp_base, f)).read())
+
+        del dtmp, dtmp_base, local_t
+
+    def test_append(self):
+        t = self.get_transport()
+
+        if self.readonly:
+            open('a', 'wb').write('diff\ncontents for\na\n')
+            open('b', 'wb').write('contents\nfor b\n')
+        else:
+            t.put_multi([
+                    ('a', 'diff\ncontents for\na\n'),
+                    ('b', 'contents\nfor b\n')
+                    ])
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.append, 'a', 'add\nsome\nmore\ncontents\n')
+            _append('a', 'add\nsome\nmore\ncontents\n')
+        else:
+            t.append('a', 'add\nsome\nmore\ncontents\n')
+
+        self.check_file_contents('a', 
+            'diff\ncontents for\na\nadd\nsome\nmore\ncontents\n')
+
+        if self.readonly:
+            self.assertRaises(TransportNotPossible,
+                    t.append_multi,
+                        [('a', 'and\nthen\nsome\nmore\n'),
+                         ('b', 'some\nmore\nfor\nb\n')])
+            _append('a', 'and\nthen\nsome\nmore\n')
+            _append('b', 'some\nmore\nfor\nb\n')
+        else:
+            t.append_multi([('a', 'and\nthen\nsome\nmore\n'),
+                    ('b', 'some\nmore\nfor\nb\n')])
+        self.check_file_contents('a', 
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n')
+        self.check_file_contents('b', 
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n')
+
+        if self.readonly:
+            _append('a', 'a little bit more\n')
+            _append('b', 'from an iterator\n')
+        else:
+            t.append_multi(iter([('a', 'a little bit more\n'),
+                    ('b', 'from an iterator\n')]))
+        self.check_file_contents('a', 
+            'diff\ncontents for\na\n'
+            'add\nsome\nmore\ncontents\n'
+            'and\nthen\nsome\nmore\n'
+            'a little bit more\n')
+        self.check_file_contents('b', 
+                'contents\nfor b\n'
+                'some\nmore\nfor\nb\n'
+                'from an iterator\n')
+
+    def test_append_file(self):
+        t = self.get_transport()
+
+        contents = [
+            ('f1', 'this is a string\nand some more stuff\n'),
+            ('f2', 'here is some text\nand a bit more\n'),
+            ('f3', 'some text for the\nthird file created\n'),
+            ('f4', 'this is a string\nand some more stuff\n'),
+            ('f5', 'here is some text\nand a bit more\n'),
+            ('f6', 'some text for the\nthird file created\n')
+        ]
+        
+        if self.readonly:
+            for f, val in contents:
+                open(f, 'wb').write(val)
+        else:
+            t.put_multi(contents)
+
+        a1 = StringIO('appending to\none\n')
+        if self.readonly:
+            _append('f1', a1.read())
+        else:
+            t.append('f1', a1)
+
+        del a1
+
+        self.check_file_contents('f1', 
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n')
+
+        a2 = StringIO('adding more\ntext to two\n')
+        a3 = StringIO('some garbage\nto put in three\n')
+
+        if self.readonly:
+            _append('f2', a2.read())
+            _append('f3', a3.read())
+        else:
+            t.append_multi([('f2', a2), ('f3', a3)])
+
+        del a2, a3
+
+        self.check_file_contents('f2',
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n')
+        self.check_file_contents('f3', 
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n')
+
+        # Test that an actual file object can be used with put
+        a4 = open('f1', 'rb')
+        if self.readonly:
+            _append('f4', a4.read())
+        else:
+            t.append('f4', a4)
+
+        del a4
+
+        self.check_file_contents('f4', 
+                'this is a string\nand some more stuff\n'
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n')
+
+        a5 = open('f2', 'rb')
+        a6 = open('f3', 'rb')
+        if self.readonly:
+            _append('f5', a5.read())
+            _append('f6', a6.read())
+        else:
+            t.append_multi([('f5', a5), ('f6', a6)])
+
+        del a5, a6
+
+        self.check_file_contents('f5',
+                'here is some text\nand a bit more\n'
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n')
+        self.check_file_contents('f6',
+                'some text for the\nthird file created\n'
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n')
+
+    def test_get_partial(self):
+        t = self.get_transport()
+
+        contents = [
+            ('f1', 
+                'here is some text\nand a bit more\n'
+                'adding more\ntext to two\n'),
+            ('f2',
+                'this is a string\nand some more stuff\n'
+                'appending to\none\n'),
+            ('f3',
+                'some text for the\nthird file created\n'
+                'some garbage\nto put in three\n')
+        ]
+        if self.readonly:
+            for f, val in contents:
+                open(f, 'wb').write(val)
+        else:
+            t.put_multi(contents)
+
+        self.assertRaises(NoSuchFile,
+                t.get_partial, 'a-missing-file', 20)
+        self.assertRaises(NoSuchFile,
+                t.get_partial, 'another-missing-file', 20, 30)
+        f = t.get_partial('f1', 33)
+        self.assertEqual(f.read(), 
+                'adding more\ntext to two\n')
+        f = t.get_partial('f1', 33, 10)
+        self.assertEqual(f.read(10), 
+                'adding mor')
+
+        del f
+
+        offsets = [('f2', 37), ('f3', 20, 10), ('f1', 10, 20)]
+        values = ['appending to\none\n',
+                  'ird file c',
+                  'me text\nand a bit mo'
+                 ]
+        contents_f = t.get_partial_multi(offsets)
+        count = 0
+        for f, val in zip(contents_f, values):
+            count += 1
+            self.assertEqual(val, f.read(len(val)))
+        # Make sure we saw all values, and no extra
+        self.assertEqual(len(values), count)
+        self.assertEqual(list(contents_f), [])
+
+        # Do the same thing with an iterator
+        offsets = iter([('f2', 34), ('f3', 18, 10), ('f1', 15, 15)])
+        values = ['ff\nappending to\none\n',
+                  'third file',
+                  'xt\nand a bit mo'
+                 ]
+        contents_f = t.get_partial_multi(offsets)
+        count = 0
+        for f, val in zip(contents_f, values):
+            count += 1
+            self.assertEqual(val, f.read(len(val)))
+        self.assertEqual(len(values), count)
+        self.assertEqual(list(contents_f), [])
+
+
+    def test_delete(self):
+        # TODO: Test Transport.delete
+        pass
+
+    def test_move(self):
+        # TODO: Test Transport.move
+        pass
+
+class LocalTransportTest(TestCaseInTempDir, TestTransportMixIn):
+    def get_transport(self):
+        from bzrlib.transport.local import LocalTransport
+        return LocalTransport('.')
+
+class HttpTransportTest(TestCaseWithWebserver, TestTransportMixIn):
+    readonly = True
+    def get_transport(self):
+        from bzrlib.transport.http import HttpTransport
+        url = self.get_remote_url('.')
+        return HttpTransport(url)
+

=== added file 'bzrlib/selftest/testworkingtree.py'
--- bzrlib/selftest/testworkingtree.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/testworkingtree.py	2005-10-04 02:43:06 +0000
@@ -0,0 +1,59 @@
+# (C) 2005 Canonical Ltd
+# Authors:  Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+import os
+from bzrlib.branch import Branch
+from bzrlib.selftest import TestCaseInTempDir
+from bzrlib.trace import mutter
+from bzrlib.workingtree import TreeEntry, TreeDirectory, TreeFile, TreeLink
+
+class TestTreeDirectory(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeDirectory().kind_character(), '/')
+
+
+class TestTreeEntry(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeEntry().kind_character(), '???')
+
+
+class TestTreeFile(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeFile().kind_character(), '')
+
+
+class TestTreeLink(TestCaseInTempDir):
+
+    def test_kind_character(self):
+        self.assertEqual(TreeLink().kind_character(), '')
+
+
+class TestWorkingTree(TestCaseInTempDir):
+
+    def test_listfiles(self):
+        branch = Branch.initialize('.')
+        os.mkdir('dir')
+        print >> open('file', 'w'), "content"
+        os.symlink('target', 'symlink')
+        tree = branch.working_tree()
+        files = list(tree.list_files())
+        self.assertEqual(files[0], ('dir', '?', 'directory', None, TreeDirectory()))
+        self.assertEqual(files[1], ('file', '?', 'file', None, TreeFile()))
+        self.assertEqual(files[2], ('symlink', '?', 'symlink', None, TreeLink()))

=== added file 'bzrlib/selftest/treeshape.py'
--- bzrlib/selftest/treeshape.py	1970-01-01 00:00:00 +0000
+++ bzrlib/selftest/treeshape.py	2005-10-04 10:10:48 +0000
@@ -0,0 +1,76 @@
+# Copyright (C) 2005 by Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+"""Test helper for constructing and testing directories.
+
+This module transforms filesystem directories to and from Python lists.  
+As a Python list the descriptions can be stored in test cases, compared,
+etc.
+"""
+
+# TODO: Script to write a description of a directory for testing
+# TODO: Helper that compares two structures and raises a helpful error
+# where they differ.  Option to ignore some files or directories in the 
+# comparison.
+
+import os
+import stat
+
+from bzrlib.trace import warning
+
+def build_tree_contents(template):
+    """Reconstitute some files from a text description.
+
+    Each element of template is a tuple.  The first element is a filename,
+    with an optional ending character indicating the type.
+
+    The template is built relative to the Python process's current
+    working directory.
+    """
+    for tt in template:
+        name = tt[0]
+        if name[-1] == '/':
+            os.mkdir(name)
+        elif name[-1] == '@':
+            raise NotImplementedError('symlinks not handled yet')
+        else:
+            f = file(name, 'wb')
+            try:
+                f.write(tt[1])
+            finally:
+                f.close()
+
+
+def capture_tree_contents(top):
+    """Make a Python datastructure description of a tree.
+    
+    If top is an absolute path the descriptions will be absolute."""
+    for dirpath, dirnames, filenames in os.walk(top):
+        yield (dirpath + '/', )
+        filenames.sort()
+        for fn in filenames:
+            fullpath = os.path.join(dirpath, fn)
+            assert fullpath[-1] not in '@/'
+            info = os.lstat(fullpath)
+            if stat.S_ISLNK(info.st_mode):
+                yield (fullpath + '@', os.readlink(fullpath))
+            elif stat.S_ISREG(info.st_mode):
+                yield (fullpath, file(fullpath, 'rb').read())
+            else:
+                warning("can't capture file %s with mode %#o",
+                        fullpath, info.st_mode)
+                pass

=== modified file 'bzrlib/selftest/versioning.py'
--- bzrlib/selftest/versioning.py	2005-07-23 14:06:37 +0000
+++ bzrlib/selftest/versioning.py	2005-09-29 20:34:25 +0000
@@ -17,14 +17,24 @@
 
 """Tests of simple versioning operations"""
 
-
-from bzrlib.selftest import InTempDir
-
-class Mkdir(InTempDir):
-    def runTest(self): 
+# TODO: test trying to commit within a directory that is not yet added
+
+
+import os
+
+from bzrlib.selftest import BzrTestBase, TestCaseInTempDir
+from bzrlib.branch import Branch
+
+import logging
+logger = logging.getLogger('bzr.test.versioning')
+debug = logger.debug
+
+
+class TestVersioning(TestCaseInTempDir):
+    
+    def test_mkdir(self): 
         """Basic 'bzr mkdir' operation"""
         from bzrlib.commands import run_bzr
-        import os
 
         run_bzr(['init'])
         run_bzr(['mkdir', 'foo'])
@@ -32,30 +42,28 @@
 
         self.assertRaises(OSError, run_bzr, ['mkdir', 'foo'])
 
-        from bzrlib.diff import compare_trees, TreeDelta
+        from bzrlib.diff import compare_trees
         from bzrlib.branch import Branch
-        b = Branch('.')
+        b = Branch.open('.')
         
         delta = compare_trees(b.basis_tree(), b.working_tree())
 
+        self.log('delta.added = %r' % delta.added)
+
         self.assertEquals(len(delta.added), 1)
         self.assertEquals(delta.added[0][0], 'foo')
         self.failIf(delta.modified)
 
-
-
-class AddInUnversioned(InTempDir):
-    def runTest(self):
+    def test_branch_add_in_unversioned(self):
         """Try to add a file in an unversioned directory.
 
-        smart_add may eventually add the parent as necessary, but simple
-        branch add doesn't do that.
+        "bzr add" adds the parent as necessary, but simple branch add
+        doesn't do that.
         """
         from bzrlib.branch import Branch
-        import os
         from bzrlib.errors import NotVersionedError
 
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['foo/',
                          'foo/hello'])
@@ -64,29 +72,151 @@
                           b.add,
                           'foo/hello')
         
-        
-class SubdirCommit(InTempDir):
-    def runTest(self):
-        """Various commits from a subdirectory"""
-        
-        from bzrlib.branch import Branch
-        from bzrlib.commands import run_bzr
-        import os
-        
-        b = Branch('.', init=True)
-        self.build_tree(['src/', 'src/foo.c', 'README'])
-        
-        self.assertEqual(sorted(b.unknowns()),
-                         ['README', 'src'])
-        
-        self.assertEqual(run_bzr(['version']), 0)
-        
-        self.assertEqual(run_bzr(['add']), 0)
-        
-        
-
-
-TEST_CLASSES = [
-    Mkdir,
-    AddInUnversioned,
-    ]
+        self.check_branch()
+
+
+    def test_add_in_unversioned(self):
+        """Try to add a file in an unversioned directory.
+
+        "bzr add" should add the parent(s) as necessary.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch.initialize('.')
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
+        # Multiple unversioned parents
+        self.build_tree(['veil/', 'veil/cerpin/', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), ['veil'])
+        run_bzr(['add', 'veil/cerpin/taxt'])
+        eq(list(b.unknowns()), [])
+
+        # Check whacky paths work
+        self.build_tree(['cicatriz/', 'cicatriz/esp'])
+        eq(list(b.unknowns()), ['cicatriz'])
+        run_bzr(['add', 'inertiatic/../cicatriz/esp'])
+        eq(list(b.unknowns()), [])
+
+    def test_add_in_versioned(self):
+        """Try to add a file in a versioned directory.
+
+        "bzr add" should do this happily.
+        """
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        eq = self.assertEqual
+
+        b = Branch.initialize('.')
+
+        self.build_tree(['inertiatic/', 'inertiatic/esp'])
+        eq(list(b.unknowns()), ['inertiatic'])
+        run_bzr(['add', '--no-recurse', 'inertiatic'])
+        eq(list(b.unknowns()), ['inertiatic'+os.sep+'esp'])
+        run_bzr(['add', 'inertiatic/esp'])
+        eq(list(b.unknowns()), [])
+
+    def test_subdir_add(self):
+        """Add in subdirectory should add only things from there down"""
+        
+        from bzrlib.branch import Branch
+        from bzrlib.commands import run_bzr
+        
+        eq = self.assertEqual
+        ass = self.assert_
+        chdir = os.chdir
+        
+        b = Branch.initialize('.')
+        self.build_tree(['src/', 'README'])
+        
+        eq(sorted(b.unknowns()),
+           ['README', 'src'])
+        
+        eq(run_bzr(['add', 'src']), 0)
+        
+        self.build_tree(['src/foo.c'])
+        
+        chdir('src')
+        eq(run_bzr(['add']), 0)
+        
+        eq(sorted(b.unknowns()), 
+           ['README'])
+        eq(len(b.inventory), 3)
+                
+        chdir('..')
+        eq(run_bzr(['add']), 0)
+        eq(list(b.unknowns()), [])
+
+        self.check_branch()
+
+
+    def check_branch(self):
+        """After all the above changes, run the check and upgrade commands.
+
+        The upgrade should be a no-op."""
+        b = Branch.open('.')
+        debug('branch has %d revisions', b.revno())
+        
+        debug('check branch...')
+        from bzrlib.check import check
+        check(b)
+        
+
+        
+class SubdirCommit(TestCaseInTempDir):
+
+    def test_subdir_commit(self):
+        """Test committing a subdirectory, and committing within a directory."""
+        run_bzr = self.run_bzr
+        eq = self.assertEqual
+
+        self.build_tree(['a/', 'b/'])
+        
+        run_bzr('init')
+        b = Branch.open('.')
+        
+        for fn in ('a/one', 'b/two', 'top'):
+            file(fn, 'w').write('old contents')
+            
+        run_bzr('add')
+        run_bzr('commit', '-m', 'first revision')
+        
+        for fn in ('a/one', 'b/two', 'top'):
+            file(fn, 'w').write('new contents')
+            
+        debug('start selective subdir commit')
+        run_bzr('commit', 'a', '-m', 'commit a only')
+        
+        old = b.revision_tree(b.get_rev_id(1))
+        new = b.revision_tree(b.get_rev_id(2))
+        
+        eq(new.get_file_by_path('b/two').read(), 'old contents')
+        eq(new.get_file_by_path('top').read(), 'old contents')
+        eq(new.get_file_by_path('a/one').read(), 'new contents')
+        
+        os.chdir('a')
+        # commit from here should do nothing
+        run_bzr('commit', '.', '-m', 'commit subdir only', '--unchanged')
+        v3 = b.revision_tree(b.get_rev_id(3))
+        eq(v3.get_file_by_path('b/two').read(), 'old contents')
+        eq(v3.get_file_by_path('top').read(), 'old contents')
+        eq(v3.get_file_by_path('a/one').read(), 'new contents')
+                
+        # commit in subdirectory commits whole tree
+        run_bzr('commit', '-m', 'commit whole tree from subdir')
+        v4 = b.revision_tree(b.get_rev_id(4))
+        eq(v4.get_file_by_path('b/two').read(), 'new contents')        
+        eq(v4.get_file_by_path('top').read(), 'new contents')
+        
+        # TODO: factor out some kind of assert_tree_state() method
+        
+
+if __name__ == '__main__':
+    import unittest
+    unittest.main()
+    

=== modified file 'bzrlib/selftest/whitebox.py'
--- bzrlib/selftest/whitebox.py	2005-07-11 22:20:04 +0000
+++ bzrlib/selftest/whitebox.py	2005-10-02 23:23:56 +0000
@@ -1,16 +1,15 @@
-#! /usr/bin/python
-
 import os
 import unittest
 
-from bzrlib.selftest import InTempDir, TestBase
+from bzrlib.selftest import TestCaseInTempDir, TestCase
 from bzrlib.branch import ScratchBranch, Branch
 from bzrlib.errors import NotBranchError, NotVersionedError
 
 
-class Unknowns(InTempDir):
-    def runTest(self):
-        b = Branch('.', init=True)
+class TestBranch(TestCaseInTempDir):
+
+    def test_unknowns(self):
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt',
                          'hello.txt~'])
@@ -18,13 +17,10 @@
         self.assertEquals(list(b.unknowns()),
                           ['hello.txt'])
 
-
-
-class NoChanges(InTempDir):
-    def runTest(self):
+    def test_no_changes(self):
         from bzrlib.errors import PointlessCommit
         
-        b = Branch('.', init=True)
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
 
@@ -49,76 +45,33 @@
         b.commit('commit pointless revision with one file',
                  allow_pointless=True)
 
-        b.add_pending_merge('mbp@892739123-2005-123123')
-        b.commit('commit new merge with no text changes',
-                 allow_pointless=False)
         
 
 
-class ValidateRevisionId(TestBase):
-    def runTest(self):
+class TestRevisionId(TestCase):
+    
+    def test_validate_revision_id(self):
         from bzrlib.revision import validate_revision_id
         validate_revision_id('mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-        
         self.assertRaises(ValueError,
                           validate_revision_id,
                           ' asdkjas')
-
-
         self.assertRaises(ValueError,
                           validate_revision_id,
                           'mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe\n')
-
-
         self.assertRaises(ValueError,
                           validate_revision_id,
                           ' mbp@sourcefrog.net-20050311061123-96a255005c7c9dbe')
-
         self.assertRaises(ValueError,
                           validate_revision_id,
                           'Martin Pool <mbp@sourcefrog.net>-20050311061123-96a255005c7c9dbe')
 
 
-
-class PendingMerges(InTempDir):
-    """Tracking pending-merged revisions."""
-    def runTest(self):
-        b = Branch('.', init=True)
-
-        self.assertEquals(b.pending_merges(), [])
-        
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-    
-        b.add_pending_merge('foo@azkhazan-123123-abcabc')
-        
-        self.assertEquals(b.pending_merges(), ['foo@azkhazan-123123-abcabc'])
-
-        b.add_pending_merge('wibble@fofof--20050401--1928390812')
-        self.assertEquals(b.pending_merges(),
-                          ['foo@azkhazan-123123-abcabc',
-                           'wibble@fofof--20050401--1928390812'])
-
-        b.commit("commit from base with two merges")
-
-        rev = b.get_revision(b.revision_history()[0])
-        self.assertEquals(len(rev.parents), 2)
-        self.assertEquals(rev.parents[0].revision_id,
-                          'foo@azkhazan-123123-abcabc')
-        self.assertEquals(rev.parents[1].revision_id,
-                           'wibble@fofof--20050401--1928390812')
-
-        # list should be cleared when we do a commit
-        self.assertEquals(b.pending_merges(), [])
-        
-        
-        
-
-class Revert(InTempDir):
-    """Test selected-file revert"""
-    def runTest(self):
-        b = Branch('.', init=True)
+class MoreTests(TestCaseInTempDir):
+
+    def test_revert(self):
+        """Test selected-file revert"""
+        b = Branch.initialize('.')
 
         self.build_tree(['hello.txt'])
         file('hello.txt', 'w').write('initial hello')
@@ -143,12 +96,9 @@
         self.check_file_contents('hello.txt', 'initial hello')
         self.check_file_contents('hello.txt~', 'initial hello')
 
-
-
-class RenameDirs(InTempDir):
-    """Test renaming directories and the files within them."""
-    def runTest(self):
-        b = Branch('.', init=True)
+    def test_rename_dirs(self):
+        """Test renaming directories and the files within them."""
+        b = Branch.initialize('.')
         self.build_tree(['dir/', 'dir/sub/', 'dir/sub/file'])
         b.add(['dir', 'dir/sub', 'dir/sub/file'])
 
@@ -176,18 +126,13 @@
                                    ['newdir', 'newdir/newsub',
                                     'newdir/newsub/file'])
 
-        
-
-
-class BranchPathTestCase(TestBase):
-    """test for branch path lookups
-
-    Branch.relpath and bzrlib.branch._relpath do a simple but subtle
-    job: given a path (either relative to cwd or absolute), work out
-    if it is inside a branch and return the path relative to the base.
-    """
-
-    def runTest(self):
+    def test_relpath(self):
+        """test for branch path lookups
+    
+        Branch.relpath and bzrlib.branch._relpath do a simple but subtle
+        job: given a path (either relative to cwd or absolute), work out
+        if it is inside a branch and return the path relative to the base.
+        """
         from bzrlib.branch import _relpath
         import tempfile, shutil
         
@@ -224,7 +169,8 @@
             # directory, or nearby
             os.chdir(dtmp)
 
-            self.assertEqual(rp('foo/bar/quux'), 'foo/bar/quux')
+            FOO_BAR_QUUX = os.path.join('foo', 'bar', 'quux')
+            self.assertEqual(rp('foo/bar/quux'), FOO_BAR_QUUX)
 
             self.assertEqual(rp('foo'), 'foo')
 
@@ -238,14 +184,3 @@
         finally:
             os.chdir(savedir)
             shutil.rmtree(dtmp)
-
-
-
-
-TEST_CLASSES = [Unknowns,
-                ValidateRevisionId,
-                PendingMerges,
-                Revert,
-                RenameDirs,
-                BranchPathTestCase,
-                ]

=== added file 'bzrlib/shellcomplete.py'
--- bzrlib/shellcomplete.py	1970-01-01 00:00:00 +0000
+++ bzrlib/shellcomplete.py	2005-09-09 17:12:26 +0000
@@ -0,0 +1,74 @@
+import sys
+
+
+def shellcomplete(context=None, outfile = None):
+    if outfile == None:
+        outfile = sys.stdout
+    if context == None:
+        shellcomplete_commands(outfile = outfile)
+    else:
+        shellcomplete_on_command(context, outfile = outfile)
+
+def shellcomplete_on_command(cmdname, outfile = None):
+    cmdname = str(cmdname)
+
+    if outfile == None:
+        outfile = sys.stdout
+
+    from inspect import getdoc
+    import commands
+    cmdobj = commands.get_cmd_object(cmdname)
+
+    doc = getdoc(cmdobj)
+    if doc == None:
+        raise NotImplementedError("sorry, no detailed shellcomplete yet for %r" % cmdname)
+
+    shellcomplete_on_option(cmdobj.takes_options, outfile = None)
+    for aname in cmdobj.takes_args:
+        outfile.write(aname + '\n')
+
+
+def shellcomplete_on_option(options, outfile = None):
+    import commands
+    
+    if not options:
+        return
+    
+    if outfile == None:
+        outfile = sys.stdout
+
+    for on in options:
+        for shortname, longname in commands.SHORT_OPTIONS.items():
+            if longname == on:
+                l = '"(--' + on + ' -' + shortname + ')"{--' + on + ',-' + shortname + '}'
+                break
+	    else:
+		l = '--' + on
+        outfile.write(l + '\n')
+
+
+def shellcomplete_commands(outfile = None):
+    """List all commands"""
+    import inspect
+    import commands
+    from inspect import getdoc
+    
+    if outfile == None:
+        outfile = sys.stdout
+    
+    cmds = []
+    for cmdname, cmdclass in commands.get_all_cmds():
+        cmds.append((cmdname, cmdclass))
+	for alias in cmdclass.aliases:
+	    cmds.append((alias, cmdclass))
+    cmds.sort()
+    for cmdname, cmdclass in cmds:
+        if cmdclass.hidden:
+            continue
+        doc = getdoc(cmdclass)
+        if doc == None:
+	    outfile.write(cmdname + '\n')
+        else:
+	    doclines = doc.splitlines()
+	    firstline = doclines[0].lower()
+	    outfile.write(cmdname + ':' + firstline[0:-1] + '\n')

=== modified file 'bzrlib/status.py'
--- bzrlib/status.py	2005-07-12 01:44:23 +0000
+++ bzrlib/status.py	2005-10-06 13:08:14 +0000
@@ -1,47 +1,77 @@
 # (C) 2005 Canonical Ltd
-
+#
 # This program is free software; you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation; either version 2 of the License, or
 # (at your option) any later version.
-
+#
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
-
+#
 # You should have received a copy of the GNU General Public License
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+import sys
+from bzrlib.osutils import is_inside_any
+from bzrlib.delta import compare_trees
 
 
 def show_status(branch, show_unchanged=False,
                 specific_files=None,
                 show_ids=False,
-                to_file=None):
-    """Display single-line status for non-ignored working files.
+                to_file=None,
+                show_pending=True,
+                revision=None):
+    """Display status for non-ignored working files.
 
-    show_all
-        If true, show unmodified files too.
+    show_unchanged
+        If set, includes unchanged files.
 
     specific_files
         If set, only show the status of files in this list.
 
+    show_ids
+        If set, includes each file's id.
+
     to_file
         If set, write to this file (default stdout.)
+
+    show_pending
+        If set, write pending merges.
+
+    revision
+        If None the compare latest revision with working tree
+        If one revision show compared it with working tree.
+        If two revisions show status between first and second.
     """
-    import sys
-    from bzrlib.diff import compare_trees
-
     if to_file == None:
         to_file = sys.stdout
     
     branch.lock_read()
     try:
-
-        old = branch.basis_tree()
-        new = branch.working_tree()
+        new_is_working_tree = True
+        if revision is None:
+            old = branch.basis_tree()
+            new = branch.working_tree()
+        elif len(revision) > 0:
+            try:
+                rev_id = revision[0].in_history(branch).rev_id
+                old = branch.revision_tree(rev_id)
+            except NoSuchRevision, e:
+                raise BzrCommandError(str(e))
+            if len(revision) > 1:
+                try:
+                    rev_id = revision[1].in_history(branch).rev_id
+                    new = branch.revision_tree(rev_id)
+                    new_is_working_tree = False
+                except NoSuchRevision, e:
+                    raise BzrCommandError(str(e))
+            else:
+                new = branch.working_tree()
+                
 
         delta = compare_trees(old, new, want_unchanged=show_unchanged,
                               specific_files=specific_files)
@@ -50,18 +80,24 @@
                    show_ids=show_ids,
                    show_unchanged=show_unchanged)
 
-        unknowns = new.unknowns()
-        done_header = False
-        for path in unknowns:
-            # FIXME: Should also match if the unknown file is within a
-            # specified directory.
-            if specific_files:
-                if path not in specific_files:
-                    continue
-            if not done_header:
-                print >>to_file, 'unknown:'
-                done_header = True
-            print >>to_file, ' ', path
+        if new_is_working_tree:
+            conflicts = new.iter_conflicts()
+            unknowns = new.unknowns()
+            list_paths('unknown', unknowns, specific_files, to_file)
+            list_paths('conflicts', conflicts, specific_files, to_file)
+            if show_pending and len(branch.pending_merges()) > 0:
+                print >>to_file, 'pending merges:'
+                for merge in branch.pending_merges():
+                    print >> to_file, ' ', merge
     finally:
         branch.unlock()
         
+def list_paths(header, paths, specific_files, to_file):
+    done_header = False
+    for path in paths:
+        if specific_files and not is_inside_any(specific_files, path):
+            continue
+        if not done_header:
+            print >>to_file, '%s:' % header
+            done_header = True
+        print >>to_file, ' ', path

=== added directory 'bzrlib/store'
=== renamed file 'bzrlib/store.py' => 'bzrlib/store/__init__.py'
--- bzrlib/store.py	2005-07-11 07:33:45 +0000
+++ bzrlib/store/__init__.py	2005-10-04 04:37:29 +0000
@@ -14,6 +14,9 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
+# TODO: Could remember a bias towards whether a particular store is typically
+# compressed or not.
+
 """
 Stores are the main data-storage mechanism for Bazaar-NG.
 
@@ -21,10 +24,12 @@
 unique ID.
 """
 
-import os, tempfile, types, osutils, gzip, errno
-from stat import ST_SIZE
-from StringIO import StringIO
-from trace import mutter
+from cStringIO import StringIO
+
+from bzrlib.errors import BzrError, UnlistableStore, TransportNotPossible
+from bzrlib.trace import mutter
+import bzrlib.transport
+from bzrlib.transport.local import LocalTransport
 
 ######################################################################
 # stores
@@ -33,190 +38,272 @@
     pass
 
 
-class ImmutableStore(object):
-    """Store that holds files indexed by unique names.
-
+class Store(object):
+    """This class represents the abstract storage layout for saving information.
+    
     Files can be added, but not modified once they are in.  Typically
     the hash is used as the name, or something else known to be unique,
     such as a UUID.
-
-    >>> st = ImmutableScratchStore()
-
-    >>> st.add(StringIO('hello'), 'aa')
-    >>> 'aa' in st
-    True
-    >>> 'foo' in st
-    False
-
-    You are not allowed to add an id that is already present.
-
-    Entries can be retrieved as files, which may then be read.
-
-    >>> st.add(StringIO('goodbye'), '123123')
-    >>> st['123123'].read()
-    'goodbye'
-
-    TODO: Atomic add by writing to a temporary file and renaming.
-
-    In bzr 0.0.5 and earlier, files within the store were marked
-    readonly on disk.  This is no longer done but existing stores need
-    to be accomodated.
     """
 
-    def __init__(self, basedir):
-        self._basedir = basedir
-
-    def _path(self, id):
-        if '\\' in id or '/' in id:
-            raise ValueError("invalid store id %r" % id)
-        return os.path.join(self._basedir, id)
-
-    def __repr__(self):
-        return "%s(%r)" % (self.__class__.__name__, self._basedir)
-
-    def add(self, f, fileid, compressed=True):
-        """Add contents of a file into the store.
-
-        f -- An open file, or file-like object."""
-        # FIXME: Only works on files that will fit in memory
-        
-        from bzrlib.atomicfile import AtomicFile
-        
-        mutter("add store entry %r" % (fileid))
-        if isinstance(f, types.StringTypes):
-            content = f
-        else:
-            content = f.read()
-            
-        p = self._path(fileid)
-        if os.access(p, os.F_OK) or os.access(p + '.gz', os.F_OK):
-            from bzrlib.errors import bailout
-            raise BzrError("store %r already contains id %r" % (self._basedir, fileid))
-
-        fn = p
-        if compressed:
-            fn = fn + '.gz'
-            
-        af = AtomicFile(fn, 'wb')
-        try:
-            if compressed:
-                gf = gzip.GzipFile(mode='wb', fileobj=af)
-                gf.write(content)
-                gf.close()
+    def __len__(self):
+        raise NotImplementedError('Children should define their length')
+
+    def __getitem__(self, fileid):
+        """Returns a file reading from a particular entry."""
+        raise NotImplementedError
+
+    def __contains__(self, fileid):
+        """"""
+        raise NotImplementedError
+
+    def __iter__(self):
+        raise NotImplementedError
+
+    def add(self, f, fileid):
+        """Add a file object f to the store accessible from the given fileid"""
+        raise NotImplementedError('Children of Store must define their method of adding entries.')
+
+    def add_multi(self, entries):
+        """Add a series of file-like or string objects to the store with the given
+        identities.
+        
+        :param entries: A list of tuples of file,id pairs [(file1, id1), (file2, id2), ...]
+                        This could also be a generator yielding (file,id) pairs.
+        """
+        for f, fileid in entries:
+            self.add(f, fileid)
+
+    def has(self, fileids):
+        """Return True/False for each entry in fileids.
+
+        :param fileids: A List or generator yielding file ids.
+        :return: A generator or list returning True/False for each entry.
+        """
+        for fileid in fileids:
+            if fileid in self:
+                yield True
             else:
-                af.write(content)
-            af.commit()
-        finally:
-            af.close()
-
-
-    def copy_multi(self, other, ids):
+                yield False
+
+    def listable(self):
+        """Return True if this store is able to be listed."""
+        return hasattr(self, "__iter__")
+
+    def get(self, fileids, permit_failure=False, pb=None):
+        """Return a set of files, one for each requested entry.
+        
+        :param permit_failure: If true, return None for entries which do not 
+                               exist.
+        :return: A list or generator of file-like objects, one for each id.
+        """
+        for fileid in fileids:
+            try:
+                yield self[fileid]
+            except KeyError:
+                if permit_failure:
+                    yield None
+                else:
+                    raise
+
+    def copy_multi(self, other, ids, pb=None, permit_failure=False):
         """Copy texts for ids from other into self.
 
         If an id is present in self, it is skipped.  A count of copied
         ids is returned, which may be less than len(ids).
+
+        :param other: Another Store object
+        :param ids: A list of entry ids to be copied
+        :param pb: A ProgressBar object, if none is given, the default will be created.
+        :param permit_failure: Allow missing entries to be ignored
+        :return: (n_copied, [failed]) The number of entries copied successfully,
+            followed by a list of entries which could not be copied (because they
+            were missing)
         """
-        from bzrlib.progress import ProgressBar
-        pb = ProgressBar()
+        if pb is None:
+            pb = bzrlib.ui.ui_factory.progress_bar()
+
+        # XXX: Is there any reason why we couldn't make this accept a generator
+        # and build a list as it finds things to copy?
+        ids = list(ids) # Make sure we don't have a generator, since we iterate 2 times
         pb.update('preparing to copy')
-        to_copy = [id for id in ids if id not in self]
-        if isinstance(other, ImmutableStore):
-            return self.copy_multi_immutable(other, to_copy, pb)
-        count = 0
-        for id in to_copy:
-            count += 1
-            pb.update('copy', count, len(to_copy))
-            self.add(other[id], id)
-        assert count == len(to_copy)
-        pb.clear()
-        return count
-
-
-    def copy_multi_immutable(self, other, to_copy, pb):
-        from shutil import copyfile
-        count = 0
-        for id in to_copy:
-            p = self._path(id)
-            other_p = other._path(id)
-            try:
-                copyfile(other_p, p)
-            except IOError, e:
-                if e.errno == errno.ENOENT:
-                    copyfile(other_p+".gz", p+".gz")
-                else:
-                    raise
-            
-            count += 1
-            pb.update('copy', count, len(to_copy))
-        assert count == len(to_copy)
-        pb.clear()
-        return count
-    
+        to_copy = []
+        for file_id, has in zip(ids, self.has(ids)):
+            if not has:
+                to_copy.append(file_id)
+        return self._do_copy(other, to_copy, pb, permit_failure=permit_failure)
+
+    def _do_copy(self, other, to_copy, pb, permit_failure=False):
+        """This is the standard copying mechanism, just get them one at
+        a time from remote, and store them locally.
+
+        :param other: Another Store object
+        :param to_copy: A list of entry ids to copy
+        :param pb: A ProgressBar object to display completion status.
+        :param permit_failure: Allow missing entries to be ignored
+        :return: (n_copied, [failed])
+            The number of entries copied, and a list of failed entries.
+        """
+        # This should be updated to use add_multi() rather than
+        # the current methods of buffering requests.
+        # One question, is it faster to queue up 1-10 and then copy 1-10
+        # then queue up 11-20, copy 11-20
+        # or to queue up 1-10, copy 1, queue 11, copy 2, etc?
+        # sort of pipeline versus batch.
+
+        # We can't use self._transport.copy_to because we don't know
+        # whether the local tree is in the same format as other
+        failed = set()
+        def buffer_requests():
+            count = 0
+            buffered_requests = []
+            for fileid in to_copy:
+                try:
+                    f = other[fileid]
+                except KeyError:
+                    if permit_failure:
+                        failed.add(fileid)
+                        continue
+                    else:
+                        raise
+
+                buffered_requests.append((f, fileid))
+                if len(buffered_requests) > self._max_buffered_requests:
+                    yield buffered_requests.pop(0)
+                    count += 1
+                    pb.update('copy', count, len(to_copy))
+
+            for req in buffered_requests:
+                yield req
+                count += 1
+                pb.update('copy', count, len(to_copy))
+
+            assert count == len(to_copy)
+
+        self.add_multi(buffer_requests())
+
+        pb.clear()
+        return len(to_copy), failed
+
+
+class TransportStore(Store):
+    """A TransportStore is a Store superclass for Stores that use Transports."""
+
+    _max_buffered_requests = 10
+
+    def __init__(self, transport):
+        assert isinstance(transport, bzrlib.transport.Transport)
+        super(TransportStore, self).__init__()
+        self._transport = transport
+
+    def __repr__(self):
+        if self._transport is None:
+            return "%s(None)" % (self.__class__.__name__)
+        else:
+            return "%s(%r)" % (self.__class__.__name__, self._transport.base)
+
+    __str__ = __repr__
+
+    def listable(self):
+        """Return True if this store is able to be listed."""
+        return self._transport.listable()
+
+
+class ImmutableMemoryStore(Store):
+    """A memory only store."""
 
     def __contains__(self, fileid):
-        """"""
-        p = self._path(fileid)
-        return (os.access(p, os.R_OK)
-                or os.access(p + '.gz', os.R_OK))
-
-    # TODO: Guard against the same thing being stored twice, compressed and uncompresse
-
-    def __iter__(self):
-        for f in os.listdir(self._basedir):
-            if f[-3:] == '.gz':
-                # TODO: case-insensitive?
-                yield f[:-3]
-            else:
-                yield f
-
-    def __len__(self):
-        return len(os.listdir(self._basedir))
+        return self._contents.has_key(fileid)
+
+    def __init__(self):
+        super(ImmutableMemoryStore, self).__init__()
+        self._contents = {}
+
+    def add(self, stream, fileid, compressed=True):
+        if self._contents.has_key(fileid):
+            raise StoreError("fileid %s already in the store" % fileid)
+        self._contents[fileid] = stream.read()
 
     def __getitem__(self, fileid):
         """Returns a file reading from a particular entry."""
-        p = self._path(fileid)
-        try:
-            return gzip.GzipFile(p + '.gz', 'rb')
-        except IOError, e:
-            if e.errno == errno.ENOENT:
-                return file(p, 'rb')
-            else:
-                raise e
+        if not self._contents.has_key(fileid):
+            raise IndexError
+        return StringIO(self._contents[fileid])
+
+    def _item_size(self, fileid):
+        return len(self._contents[fileid])
+
+    def __iter__(self):
+        return iter(self._contents.keys())
 
     def total_size(self):
-        """Return (count, bytes)
-
-        This is the (compressed) size stored on disk, not the size of
-        the content."""
-        total = 0
+        result = 0
         count = 0
-        for fid in self:
+        for fileid in self:
             count += 1
-            p = self._path(fid)
-            try:
-                total += os.stat(p)[ST_SIZE]
-            except OSError:
-                total += os.stat(p + '.gz')[ST_SIZE]
-                
-        return count, total
-
-
-
-
-class ImmutableScratchStore(ImmutableStore):
-    """Self-destructing test subclass of ImmutableStore.
-
-    The Store only exists for the lifetime of the Python object.
- Obviously you should not put anything precious in it.
+            result += self._item_size(fileid)
+        return count, result
+        
+
+class CachedStore(Store):
+    """A store that caches data locally, to avoid repeated downloads.
+    The precacache method should be used to avoid server round-trips for
+    every piece of data.
     """
-    def __init__(self):
-        ImmutableStore.__init__(self, tempfile.mkdtemp())
-
-    def __del__(self):
-        for f in os.listdir(self._basedir):
-            fpath = os.path.join(self._basedir, f)
-            # needed on windows, and maybe some other filesystems
-            os.chmod(fpath, 0600)
-            os.remove(fpath)
-        os.rmdir(self._basedir)
-        mutter("%r destroyed" % self)
+
+    def __init__(self, store, cache_dir):
+        super(CachedStore, self).__init__()
+        self.source_store = store
+        # This clones the source store type with a locally bound
+        # transport. FIXME: it assumes a constructor is == cloning.
+        # clonable store - it might be nicer to actually have a clone()
+        # or something. RBC 20051003
+        self.cache_store = store.__class__(LocalTransport(cache_dir))
+
+    def __getitem__(self, id):
+        mutter("Cache add %s" % id)
+        if id not in self.cache_store:
+            self.cache_store.add(self.source_store[id], id)
+        return self.cache_store[id]
+
+    def __contains__(self, fileid):
+        if fileid in self.cache_store:
+            return True
+        if fileid in self.source_store:
+            # We could copy at this time
+            return True
+        return False
+
+    def get(self, fileids, permit_failure=False, pb=None):
+        fileids = list(fileids)
+        hasids = self.cache_store.has(fileids)
+        needs = set()
+        for has, fileid in zip(hasids, fileids):
+            if not has:
+                needs.add(fileid)
+        if needs:
+            self.cache_store.copy_multi(self.source_store, needs,
+                    permit_failure=permit_failure)
+        return self.cache_store.get(fileids,
+                permit_failure=permit_failure, pb=pb)
+
+    def prefetch(self, ids):
+        """Copy a series of ids into the cache, before they are used.
+        For remote stores that support pipelining or async downloads, this can
+        increase speed considerably.
+
+        Failures while prefetching are ignored.
+        """
+        mutter("Prefetch of ids %s" % ",".join(ids))
+        self.cache_store.copy_multi(self.source_store, ids, 
+                                    permit_failure=True)
+
+
+def copy_all(store_from, store_to):
+    """Copy all ids from one store to another."""
+    # TODO: Optional progress indicator
+    if not store_from.listable():
+        raise UnlistableStore(store_from)
+    ids = [f for f in store_from]
+    store_to.copy_multi(store_from, ids)
+

=== added file 'bzrlib/store/compressed_text.py'
--- bzrlib/store/compressed_text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/compressed_text.py	2005-10-02 21:51:29 +0000
@@ -0,0 +1,261 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""
+An implementation the primary storage type CompressedTextStore.
+
+This store keeps compressed versions of the full text. It does not
+do any sort of delta compression.
+"""
+
+import os, tempfile, gzip
+
+import bzrlib.store
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrError
+
+from StringIO import StringIO
+from stat import ST_SIZE
+
+class CompressedTextStore(bzrlib.store.TransportStore):
+    """Store that holds files indexed by unique names.
+
+    Files can be added, but not modified once they are in.  Typically
+    the hash is used as the name, or something else known to be unique,
+    such as a UUID.
+
+    Files are stored gzip compressed, with no delta compression.
+
+    >>> st = ScratchCompressedTextStore()
+
+    >>> st.add(StringIO('hello'), 'aa')
+    >>> 'aa' in st
+    True
+    >>> 'foo' in st
+    False
+
+    You are not allowed to add an id that is already present.
+
+    Entries can be retrieved as files, which may then be read.
+
+    >>> st.add(StringIO('goodbye'), '123123')
+    >>> st['123123'].read()
+    'goodbye'
+    """
+
+    def __init__(self, transport):
+        super(CompressedTextStore, self).__init__(transport)
+
+    def _check_fileid(self, fileid):
+        if '\\' in fileid or '/' in fileid:
+            raise ValueError("invalid store id %r" % fileid)
+
+    def _relpath(self, fileid):
+        self._check_fileid(fileid)
+        return fileid + '.gz'
+
+    def add(self, f, fileid):
+        """Add contents of a file into the store.
+
+        f -- An open file, or file-like object."""
+        # TODO: implement an add_multi which can do some of it's
+        #       own piplelining, and possible take advantage of
+        #       transport.put_multi(). The problem is that
+        #       entries potentially need to be compressed as they
+        #       are received, which implies translation, which
+        #       means it isn't as straightforward as we would like.
+        from cStringIO import StringIO
+        from bzrlib.osutils import pumpfile
+        
+        mutter("add store entry %r" % (fileid))
+        if isinstance(f, basestring):
+            f = StringIO(f)
+            
+        fn = self._relpath(fileid)
+        if self._transport.has(fn):
+            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
+
+
+        sio = StringIO()
+        gf = gzip.GzipFile(mode='wb', fileobj=sio)
+        # if pumpfile handles files that don't fit in ram,
+        # so will this function
+        if isinstance(f, basestring):
+            gf.write(f)
+        else:
+            pumpfile(f, gf)
+        gf.close()
+        sio.seek(0)
+        self._transport.put(fn, sio)
+
+    def _do_copy(self, other, to_copy, pb, permit_failure=False):
+        if isinstance(other, CompressedTextStore):
+            return self._copy_multi_text(other, to_copy, pb,
+                    permit_failure=permit_failure)
+        return super(CompressedTextStore, self)._do_copy(other, to_copy,
+                pb, permit_failure=permit_failure)
+
+    def _copy_multi_text(self, other, to_copy, pb,
+            permit_failure=False):
+        # Because of _transport, we can no longer assume
+        # that they are on the same filesystem, we can, however
+        # assume that we only need to copy the exact bytes,
+        # we don't need to process the files.
+
+        failed = set()
+        if permit_failure:
+            new_to_copy = set()
+            for fileid, has in zip(to_copy, other.has(to_copy)):
+                if has:
+                    new_to_copy.add(fileid)
+                else:
+                    failed.add(fileid)
+            to_copy = new_to_copy
+            #mutter('_copy_multi_text copying %s, failed %s' % (to_copy, failed))
+
+        paths = [self._relpath(fileid) for fileid in to_copy]
+        count = other._transport.copy_to(paths, self._transport, pb=pb)
+        assert count == len(to_copy)
+        return count, failed
+
+    def __contains__(self, fileid):
+        """"""
+        fn = self._relpath(fileid)
+        return self._transport.has(fn)
+
+    def has(self, fileids, pb=None):
+        """Return True/False for each entry in fileids.
+
+        :param fileids: A List or generator yielding file ids.
+        :return: A generator or list returning True/False for each entry.
+        """
+        relpaths = (self._relpath(fid) for fid in fileids)
+        return self._transport.has_multi(relpaths, pb=pb)
+
+    def get(self, fileids, permit_failure=False, pb=None):
+        """Return a set of files, one for each requested entry.
+        
+        TODO: Write some tests to make sure that permit_failure is
+              handled correctly.
+
+        TODO: What should the exception be for a missing file?
+              KeyError, or NoSuchFile?
+        """
+
+        # This next code gets a bit hairy because it can allow
+        # to not request a file which doesn't seem to exist.
+        # Also, the same fileid may be requested twice, so we
+        # can't just build up a map.
+        rel_paths = [self._relpath(fid) for fid in fileids]
+        is_requested = []
+
+        #mutter('CompressedTextStore.get(permit_failure=%s)' % permit_failure)
+        if permit_failure:
+            existing_paths = []
+            for path, has in zip(rel_paths,
+                    self._transport.has_multi(rel_paths)):
+                if has:
+                    existing_paths.append(path)
+                    is_requested.append(True)
+                else:
+                    is_requested.append(False)
+            #mutter('Retrieving %s out of %s' % (existing_paths, rel_paths))
+        else:
+            #mutter('Retrieving all %s' % (rel_paths, ))
+            existing_paths = rel_paths
+            is_requested = [True for x in rel_paths]
+
+        count = 0
+        for f in self._transport.get_multi(existing_paths, pb=pb):
+            assert count < len(is_requested)
+            while not is_requested[count]:
+                yield None
+                count += 1
+            if hasattr(f, 'tell'):
+                yield gzip.GzipFile(mode='rb', fileobj=f)
+            else:
+                from cStringIO import StringIO
+                sio = StringIO(f.read())
+                yield gzip.GzipFile(mode='rb', fileobj=sio)
+            count += 1
+
+        while count < len(is_requested):
+            yield None
+            count += 1
+
+    def __iter__(self):
+        # TODO: case-insensitive?
+        for f in self._transport.list_dir('.'):
+            if f[-3:] == '.gz':
+                yield f[:-3]
+            else:
+                yield f
+
+    def __len__(self):
+        return len([f for f in self._transport.list_dir('.')])
+
+
+    def __getitem__(self, fileid):
+        """Returns a file reading from a particular entry."""
+        fn = self._relpath(fileid)
+        # This will throw if the file doesn't exist.
+        try:
+            f = self._transport.get(fn)
+        except:
+            raise KeyError('This store (%s) does not contain %s' % (self, fileid))
+
+        # gzip.GzipFile.read() requires a tell() function
+        # but some transports return objects that cannot seek
+        # so buffer them in a StringIO instead
+        if hasattr(f, 'tell'):
+            return gzip.GzipFile(mode='rb', fileobj=f)
+        else:
+            from cStringIO import StringIO
+            sio = StringIO(f.read())
+            return gzip.GzipFile(mode='rb', fileobj=sio)
+            
+
+    def total_size(self):
+        """Return (count, bytes)
+
+        This is the (compressed) size stored on disk, not the size of
+        the content."""
+        total = 0
+        count = 0
+        relpaths = [self._relpath(fid) for fid in self]
+        for st in self._transport.stat_multi(relpaths):
+            count += 1
+            total += st[ST_SIZE]
+                
+        return count, total
+
+
+class ScratchCompressedTextStore(CompressedTextStore):
+    """Self-destructing test subclass of CompressedTextStore.
+
+    The Store only exists for the lifetime of the Python object.
+    Obviously you should not put anything precious in it.
+    """
+    def __init__(self):
+        from transport import transport
+        t = transport(tempfile.mkdtemp())
+        super(ScratchCompressedTextStore, self).__init__(t)
+
+    def __del__(self):
+        self._transport.delete_multi(self._transport.list_dir('.'))
+        os.rmdir(self._transport.base)
+        mutter("%r destroyed" % self)
+

=== added file 'bzrlib/store/text.py'
--- bzrlib/store/text.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/text.py	2005-10-02 21:51:29 +0000
@@ -0,0 +1,209 @@
+# Copyright (C) 2005 by Canonical Development Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""
+A store that keeps the full text of every version.
+
+This store keeps uncompressed versions of the full text. It does not
+do any sort of delta compression.
+"""
+
+import os, tempfile
+
+import bzrlib.store
+from bzrlib.trace import mutter
+from bzrlib.errors import BzrError
+
+from cStringIO import StringIO
+from stat import ST_SIZE
+
+
+class TextStore(bzrlib.store.TransportStore):
+    """Store that holds files indexed by unique names.
+
+    Files can be added, but not modified once they are in.  Typically
+    the hash is used as the name, or something else known to be unique,
+    such as a UUID.
+
+    Files are stored uncompressed, with no delta compression.
+    """
+
+    def __init__(self, transport):
+        super(TextStore, self).__init__(transport)
+
+    def _check_fileid(self, fileid):
+        if not isinstance(fileid, basestring):
+            raise TypeError('Fileids should be a string type: %s %r' % (type(fileid), fileid))
+        if '\\' in fileid or '/' in fileid:
+            raise ValueError("invalid store id %r" % fileid)
+
+    def _relpath(self, fileid):
+        self._check_fileid(fileid)
+        return fileid
+
+    def add(self, f, fileid):
+        """Add contents of a file into the store.
+
+        f -- A file-like object, or string
+        """
+        mutter("add store entry %r" % (fileid))
+            
+        fn = self._relpath(fileid)
+        if self._transport.has(fn):
+            raise BzrError("store %r already contains id %r" % (self._transport.base, fileid))
+
+        self._transport.put(fn, f)
+
+    def _do_copy(self, other, to_copy, pb, permit_failure=False):
+        if isinstance(other, TextStore):
+            return self._copy_multi_text(other, to_copy, pb,
+                    permit_failure=permit_failure)
+        return super(TextStore, self)._do_copy(other, to_copy,
+                pb, permit_failure=permit_failure)
+
+    def _copy_multi_text(self, other, to_copy, pb,
+            permit_failure=False):
+        # Because of _transport, we can no longer assume
+        # that they are on the same filesystem, we can, however
+        # assume that we only need to copy the exact bytes,
+        # we don't need to process the files.
+
+        failed = set()
+        if permit_failure:
+            new_to_copy = set()
+            for fileid, has in zip(to_copy, other.has(to_copy)):
+                if has:
+                    new_to_copy.add(fileid)
+                else:
+                    failed.add(fileid)
+            to_copy = new_to_copy
+            #mutter('_copy_multi_text copying %s, failed %s' % (to_copy, failed))
+
+        paths = [self._relpath(fileid) for fileid in to_copy]
+        count = other._transport.copy_to(paths, self._transport, pb=pb)
+        assert count == len(to_copy)
+        return count, failed
+
+    def __contains__(self, fileid):
+        """"""
+        fn = self._relpath(fileid)
+        return self._transport.has(fn)
+
+    def has(self, fileids, pb=None):
+        """Return True/False for each entry in fileids.
+
+        :param fileids: A List or generator yielding file ids.
+        :return: A generator or list returning True/False for each entry.
+        """
+        relpaths = (self._relpath(fid) for fid in fileids)
+        return self._transport.has_multi(relpaths, pb=pb)
+
+    def get(self, fileids, permit_failure=False, pb=None):
+        """Return a set of files, one for each requested entry.
+        
+        TODO: Write some tests to make sure that permit_failure is
+              handled correctly.
+
+        TODO: What should the exception be for a missing file?
+              KeyError, or NoSuchFile?
+        """
+
+        # This next code gets a bit hairy because it can allow
+        # to not request a file which doesn't seem to exist.
+        # Also, the same fileid may be requested twice, so we
+        # can't just build up a map.
+        rel_paths = [self._relpath(fid) for fid in fileids]
+        is_requested = []
+
+        if permit_failure:
+            existing_paths = []
+            for path, has in zip(rel_paths,
+                    self._transport.has_multi(rel_paths)):
+                if has:
+                    existing_paths.append(path)
+                    is_requested.append(True)
+                else:
+                    is_requested.append(False)
+            #mutter('Retrieving %s out of %s' % (existing_paths, rel_paths))
+        else:
+            #mutter('Retrieving all %s' % (rel_paths, ))
+            existing_paths = rel_paths
+            is_requested = [True for x in rel_paths]
+
+        count = 0
+        for f in self._transport.get_multi(existing_paths, pb=pb):
+            assert count < len(is_requested)
+            while not is_requested[count]:
+                yield None
+                count += 1
+            yield f
+            count += 1
+
+        while count < len(is_requested):
+            yield None
+            count += 1
+
+    def __iter__(self):
+        # TODO: case-insensitive?
+        for f in self._transport.list_dir('.'):
+            yield f
+
+    def __len__(self):
+        return len([f for f in self._transport.list_dir('.')])
+
+
+    def __getitem__(self, fileid):
+        """Returns a file reading from a particular entry."""
+        fn = self._relpath(fileid)
+        # This will throw if the file doesn't exist.
+        try:
+            f = self._transport.get(fn)
+        except:
+            raise KeyError('This store (%s) does not contain %s' % (self, fileid))
+
+        return f
+            
+
+    def total_size(self):
+        """Return (count, bytes)
+
+        This is the (compressed) size stored on disk, not the size of
+        the content."""
+        total = 0
+        count = 0
+        relpaths = [self._relpath(fid) for fid in self]
+        for st in self._transport.stat_multi(relpaths):
+            count += 1
+            total += st[ST_SIZE]
+                
+        return count, total
+
+
+class ScratchTextStore(TextStore):
+    """Self-destructing test subclass of TextStore.
+
+    The Store only exists for the lifetime of the Python object.
+    Obviously you should not put anything precious in it.
+    """
+    def __init__(self):
+        from transport import transport
+        super(ScratchTextStore, self).__init__(transport(tempfile.mkdtemp()))
+
+    def __del__(self):
+        self._transport.delete_multi(self._transport.list_dir('.'))
+        os.rmdir(self._transport.base)
+        mutter("%r destroyed" % self)
+

=== added file 'bzrlib/store/weave.py'
--- bzrlib/store/weave.py	1970-01-01 00:00:00 +0000
+++ bzrlib/store/weave.py	2005-10-07 06:13:14 +0000
@@ -0,0 +1,123 @@
+#! /usr/bin/python
+
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+# XXX: Some consideration of the problems that might occur if there are
+# files whose id differs only in case.  That should probably be forbidden.
+
+
+from cStringIO import StringIO
+import os
+import errno
+
+from bzrlib.weavefile import read_weave, write_weave_v5
+from bzrlib.weave import Weave
+from bzrlib.store import Store
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.errors import NoSuchFile
+from bzrlib.trace import mutter
+
+
+class WeaveStore(Store):
+    """Collection of several weave files in a directory.
+
+    This has some shortcuts for reading and writing them.
+    """
+    FILE_SUFFIX = '.weave'
+
+    def __init__(self, transport):
+        self._transport = transport
+
+    def filename(self, file_id):
+        """Return the path relative to the transport root."""
+        return file_id + WeaveStore.FILE_SUFFIX
+
+    def __iter__(self):
+        l = len(WeaveStore.FILE_SUFFIX)
+        for f in self._transport.list_dir('.'):
+            if f.endswith(WeaveStore.FILE_SUFFIX):
+                f = f[:-l]
+                yield f
+
+    def __contains__(self, fileid):
+        """"""
+        return self._transport.has(self.filename(fileid))
+
+    def _get(self, file_id):
+        return self._transport.get(self.filename(file_id))
+
+    def _put(self, file_id, f):
+        return self._transport.put(self.filename(file_id), f)
+
+    def get_weave(self, file_id, transaction):
+        weave = transaction.map.find_weave(file_id)
+        if weave:
+            mutter("cache hit in %s for %s", self, file_id)
+            return weave
+        w = read_weave(self._get(file_id))
+        transaction.map.add_weave(file_id, w)
+        transaction.register_clean(w)
+        return w
+
+
+    def get_lines(self, file_id, rev_id, transaction):
+        """Return text from a particular version of a weave.
+
+        Returned as a list of lines."""
+        w = self.get_weave(file_id, transaction)
+        return w.get(w.lookup(rev_id))
+    
+    def get_weave_or_empty(self, file_id, transaction):
+        """Return a weave, or an empty one if it doesn't exist.""" 
+        try:
+            return self.get_weave(file_id, transaction)
+        except NoSuchFile:
+            weave = Weave(weave_name=file_id)
+            transaction.map.add_weave(file_id, weave)
+            transaction.register_clean(weave)
+            return weave
+
+    def put_weave(self, file_id, weave, transaction):
+        """Write back a modified weave"""
+        transaction.register_dirty(weave)
+        # TODO FOR WRITE TRANSACTIONS: this should be done in a callback
+        # from the transaction, when it decides to save.
+        sio = StringIO()
+        write_weave_v5(weave, sio)
+        sio.seek(0)
+
+        self._put(file_id, sio)
+
+
+    def add_text(self, file_id, rev_id, new_lines, parents, transaction):
+        w = self.get_weave_or_empty(file_id, transaction)
+        parent_idxs = map(w.lookup, parents)
+        w.add(rev_id, parent_idxs, new_lines)
+        self.put_weave(file_id, w, transaction)
+        
+    def add_identical_text(self, file_id, old_rev_id, new_rev_id, parents,
+                           transaction):
+        w = self.get_weave_or_empty(file_id, transaction)
+        parent_idxs = map(w.lookup, parents)
+        w.add_identical(old_rev_id, new_rev_id, parent_idxs)
+        self.put_weave(file_id, w, transaction)
+     
+    def copy_multi(self, from_store, file_ids):
+        assert isinstance(from_store, WeaveStore)
+        for f in file_ids:
+            mutter("copy weave {%s} into %s", f, self)
+            self._put(f, from_store._get(f))

=== modified file 'bzrlib/textinv.py'
--- bzrlib/textinv.py	2005-04-11 02:44:45 +0000
+++ bzrlib/textinv.py	2005-10-03 05:54:35 +0000
@@ -15,8 +15,8 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-from errors import BzrError
-from inventory import InventoryEntry, Inventory
+from bzrlib.errors import BzrError
+from bzrlib.inventory import InventoryEntry, Inventory
 
 
 START_MARK = "# bzr inventory format 3\n"
@@ -69,8 +69,6 @@
     outf.write(END_MARK)
 
 
-
-
 def read_text_inventory(tf):
     """Return an inventory read in from tf"""
     if tf.readline() != START_MARK:

=== modified file 'bzrlib/trace.py'
--- bzrlib/trace.py	2005-07-20 17:31:25 +0000
+++ bzrlib/trace.py	2005-10-06 07:28:41 +0000
@@ -13,69 +13,89 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 
-
-# TODO: Could probably replace this with something based on Python logging
-# module.
-
-
-
-
-__copyright__ = "Copyright (C) 2005 Canonical Ltd."
-__author__ = "Martin Pool <mbp@canonical.com>"
-
-
 """Messages and logging for bazaar-ng
 
-Nothing is actually logged unless you call bzrlib.open_tracefile().
+Messages are sent out through the Python logging library.
+
+They can be sent to two places: to stderr, and to ~/.bzr.log.
+
+~/.bzr.log gets all messages, and tracebacks of all uncaught
+exceptions.
+
+Normally stderr only gets messages of level INFO and higher, and gets
+only a summary of exceptions, not the traceback.
 """
 
 
-import sys, os
-
-######################################################################
-# messages and logging
-
-global _tracefile, _starttime
-_tracefile = None
-
-# used to have % (os.environ['USER'], time.time(), os.getpid()), 'w')
-_starttime = None
-
-# If false, notes also go to stdout; should replace this with --silent
-# at some point.
-silent = False
-
-
-# fix this if we ever fork within python
-_mypid = os.getpid()
-_logprefix = '[%d] ' % _mypid
-
-
-def _write_trace(msg):
-    if 0:
-        if _tracefile:
-            _tracefile.write(_logprefix + msg + '\n')
-
-
-def warning(msg):
-    sys.stderr.write('bzr: warning: ' + msg + '\n')
-    _write_trace('warning: ' + msg)
-
-
-mutter = _write_trace
-
-
-def note(msg):
-    b = '* ' + str(msg) + '\n'
-    if not silent:
-        sys.stderr.write(b)
-    _write_trace('note: ' + msg)
-
-
-def log_error(msg):
-    sys.stderr.write(msg + '\n')
-    _write_trace(msg)
-
+# TODO: in debug mode, stderr should get full tracebacks and also
+# debug messages.  (Is this really needed?)
+
+# TODO: When running the test suites, we should add an additional
+# logger that sends messages into the test log file.
+
+
+import sys
+import os
+import logging
+import traceback
+
+
+_file_handler = None
+_stderr_handler = None
+
+
+class QuietFormatter(logging.Formatter):
+    """Formatter that supresses the details of errors.
+
+    This is used by default on stderr so as not to scare the user.
+    """
+    # At first I tried overriding formatException to suppress the
+    # exception details, but that has global effect: no loggers
+    # can get the exception details is we suppress them here.
+
+    def format(self, record):
+        if record.levelno >= logging.WARNING:
+            s = 'bzr: ' + record.levelname + ': '
+        else:
+            s = ''
+            
+        s += record.getMessage()
+
+        ##import textwrap
+        ##s = textwrap.fill(s)
+            
+        if record.exc_info:
+            # give just a summary of the exception, not the whole thing
+            exinfo = traceback.extract_tb(record.exc_info[2])
+            # the format of this string matches one of the REs
+            s += '\n'
+            s += ('  at %s line %d, in %s()\n' % exinfo[-1][:3])
+            s += '  see ~/.bzr.log for debug information'
+
+        return s
+        
+
+
+
+################
+# configure convenient aliases for output routines
+
+_bzr_logger = logging.getLogger('bzr')
+_bzr_logger.setLevel(logging.DEBUG) 
+
+info = note = _bzr_logger.info
+warning =   _bzr_logger.warning
+log_error = _bzr_logger.error
+error =     _bzr_logger.error
+mutter =    _bzr_logger.debug
+debug =     _bzr_logger.debug
+
+
+
+
+# we do the rollover using this code, rather than the default from python
+# logging, because we only want to rollover at program startup, not on each
+# message.  maybe that's not a good enough reason.
 
 def _rollover_trace_maybe(trace_fname):
     import stat
@@ -85,67 +105,101 @@
             return
         old_fname = trace_fname + '.old'
 
-        try:
-            # must remove before rename on windows
-            os.remove(old_fname)
-        except OSError:
-            pass
+        from osutils import rename
+        rename(trace_fname, old_fname)
 
-        try:
-            # might fail if in use on windows
-            os.rename(trace_fname, old_fname)
-        except OSError:
-            pass
     except OSError:
         return
 
 
 
-def open_tracefile(argv=[], tracefilename='~/.bzr.log'):
+def open_tracefile(tracefilename='~/.bzr.log'):
     # Messages are always written to here, so that we have some
     # information if something goes wrong.  In a future version this
     # file will be removed on successful completion.
-    global _starttime, _tracefile
+    global _file_handler
     import stat, codecs
 
-    _starttime = os.times()[4]
-
     trace_fname = os.path.join(os.path.expanduser(tracefilename))
     _rollover_trace_maybe(trace_fname)
 
     # buffering=1 means line buffered
     try:
-        _tracefile = codecs.open(trace_fname, 'at', 'utf8', buffering=1)
-        t = _tracefile
-
-        if os.fstat(t.fileno())[stat.ST_SIZE] == 0:
-            t.write("\nthis is a debug log for diagnosing/reporting problems in bzr\n")
-            t.write("you can delete or truncate this file, or include sections in\n")
-            t.write("bug reports to bazaar-ng@lists.canonical.com\n\n")
-
-        import bzrlib
-        _write_trace('bzr %s invoked on python %s (%s)'
-                     % (bzrlib.__version__,
-                        '.'.join(map(str, sys.version_info)),
-                        sys.platform))
-
-        _write_trace('  arguments: %r' % argv)
-        _write_trace('  working dir: ' + os.getcwdu())
+        tf = codecs.open(trace_fname, 'at', 'utf8', buffering=1)
+
+        if os.fstat(tf.fileno())[stat.ST_SIZE] == 0:
+            tf.write("\nthis is a debug log for diagnosing/reporting problems in bzr\n")
+            tf.write("you can delete or truncate this file, or include sections in\n")
+            tf.write("bug reports to bazaar-ng@lists.canonical.com\n\n")
+        
+        _file_handler = logging.StreamHandler(tf)
+        fmt = r'[%(process)5d] %(asctime)s.%(msecs)03d %(levelname)s: %(message)s'
+        datefmt = r'%a %H:%M:%S'
+        _file_handler.setFormatter(logging.Formatter(fmt, datefmt))
+        _file_handler.setLevel(logging.DEBUG)
+        logging.getLogger('').addHandler(_file_handler)
+
     except IOError, e:
         warning("failed to open trace file: %s" % (e))
 
-def close_trace():
-    times = os.times()
-    mutter("finished, %.3fu/%.3fs cpu, %.3fu/%.3fs cum, %.3f elapsed"
-           % (times[:4] + ((times[4] - _starttime),)))
-
-
-
-def log_exception():
-    """Log the last exception into the trace file."""
-    import cgitb
-    s = cgitb.text(sys.exc_info())
-    for l in s.split('\n'):
-        _write_trace(l)
-        
-    
+
+def log_startup(argv):
+    import bzrlib
+
+    debug('bzr %s invoked on python %s (%s)',
+          bzrlib.__version__,
+          '.'.join(map(str, sys.version_info)),
+          sys.platform)
+
+    debug('  arguments: %r', argv)
+    debug('  working dir: %s', os.getcwdu())
+
+
+def log_exception(msg=None):
+    """Log the last exception into the trace file.
+
+    The exception string representation is used as the error
+    summary, unless msg is given.
+    """
+    if msg == None:
+        ei = sys.exc_info()
+        msg = str(ei[1])
+    if msg and (msg[-1] == '\n'):
+        msg = msg[:-1]
+    ## msg = "(%s) %s" % (str(type(ei[1])), msg)
+    _bzr_logger.exception(msg)
+
+
+
+def enable_default_logging():
+    """Configure default logging to stderr and .bzr.log"""
+    global _stderr_handler, _file_handler
+
+    _stderr_handler = logging.StreamHandler()
+    _stderr_handler.setFormatter(QuietFormatter())
+
+    if not _file_handler:
+        open_tracefile()
+
+    if os.environ.get('BZR_DEBUG'):
+        level = logging.DEBUG
+    else:
+        level = logging.INFO
+
+    _stderr_handler.setLevel(logging.INFO)
+    _file_handler.setLevel(level)
+
+    logging.getLogger('').addHandler(_stderr_handler)
+
+
+def disable_default_logging():
+    """Turn off default log handlers.
+
+    This is intended to be used by the test framework, which doesn't
+    want leakage from the code-under-test into the main logs.
+    """
+
+    l = logging.getLogger('')
+    l.removeHandler(_stderr_handler)
+    if _file_handler:
+        l.removeHandler(_file_handler)

=== added file 'bzrlib/transactions.py'
--- bzrlib/transactions.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transactions.py	2005-10-07 06:13:14 +0000
@@ -0,0 +1,104 @@
+# Copyright (C) 2005 by Canonical Ltd
+#   Authors: Robert Collins <robert.collins@canonical.com>
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""This module provides a transactional facility.
+
+Transactions provide hooks to allow data objects (i.e. inventory weaves or
+the revision-history file) to be placed in a registry and retrieved later
+during the same transaction.  This allows for repeated read isolation. At
+the end of a transaction, a callback is issued to each registered changed
+item informing it whether it should commit or not. We provide a two layer
+facility - domain objects are notified first, then data objects.
+
+Read only transactions raise an assert when objects are listed as dirty
+against them - preventing unintended writes. Once all the data storage is
+hooked into this facility, it might be nice to have a readonly transaction
+that just excepts on commit, for testing or simulating of things.
+
+Write transactions queue all changes in the transaction (which may in the 
+future involve writing them to uncommitted atomic files in preparation 
+for commit - i.e. on network connections where latency matters) and then
+notify each object of commit or rollback.
+
+Both read and write transactions *may* flush unchanged objects out of 
+memory, unless they are marked as 'preserve' which indicates that 
+repeated reads cannot be obtained if the object is ejected.
+"""
+
+import bzrlib.errors as errors
+from bzrlib.identitymap import IdentityMap, NullIdentityMap
+
+
+class ReadOnlyTransaction(object):
+    """A read only unit of work for data objects."""
+
+    def commit(self):
+        """ReadOnlyTransactions cannot commit."""
+        raise errors.CommitNotPossible('In a read only transaction')
+
+    def finish(self):
+        """Clean up this transaction
+
+        This will rollback on transactions that can if they have nto been
+        committed.
+        """
+
+    def __init__(self):
+        self.map = IdentityMap()
+
+    def register_clean(self, an_object):
+        """Register an_object as being clean."""
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty."""
+        raise errors.ReadOnlyError(
+            "Cannot dirty objects in a read only transaction")
+
+    def rollback(self):
+        """Let people call this even though nothing has to happen."""
+
+        
+class PassThroughTransaction(object):
+    """A pass through transaction
+    
+    - all actions are committed immediately.
+    - rollback is not supported.
+    - commit() is a no-op.
+    """
+
+    def commit(self):
+        """PassThroughTransactions have nothing to do."""
+
+    def finish(self):
+        """Clean up this transaction
+
+        This will rollback on transactions that can if they have nto been
+        committed.
+        """
+
+    def __init__(self):
+        self.map = NullIdentityMap()
+
+    def register_clean(self, an_object):
+        """Register an_object as being clean."""
+
+    def register_dirty(self, an_object):
+        """Register an_object as being dirty."""
+
+    def rollback(self):
+        """Cannot rollback a pass through transaction."""
+        raise errors.AlreadyCommitted

=== added directory 'bzrlib/transport'
=== added file 'bzrlib/transport/__init__.py'
--- bzrlib/transport/__init__.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/__init__.py	2005-10-04 04:37:29 +0000
@@ -0,0 +1,359 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Transport is an abstraction layer to handle file access.
+
+The abstraction is to allow access from the local filesystem, as well
+as remote (such as http or sftp).
+"""
+
+from bzrlib.trace import mutter
+from bzrlib.errors import (BzrError, 
+    TransportError, TransportNotPossible, NonRelativePath,
+    NoSuchFile, FileExists, PermissionDenied,
+    ConnectionReset)
+
+_protocol_handlers = {
+}
+
+def register_transport(prefix, klass, override=True):
+    global _protocol_handlers
+
+    if _protocol_handlers.has_key(prefix):
+        if override:
+            mutter('overriding transport: %s => %s' % (prefix, klass.__name__))
+            _protocol_handlers[prefix] = klass
+    else:
+        mutter('registering transport: %s => %s' % (prefix, klass.__name__))
+        _protocol_handlers[prefix] = klass
+
+class Transport(object):
+    """This class encapsulates methods for retrieving or putting a file
+    from/to a storage location.
+
+    Most functions have a _multi variant, which allows you to queue up
+    multiple requests. They generally have a dumb base implementation 
+    which just iterates over the arguments, but smart Transport
+    implementations can do pipelining.
+    In general implementations should support having a generator or a list
+    as an argument (ie always iterate, never index)
+    """
+
+    def __init__(self, base):
+        super(Transport, self).__init__()
+        self.base = base
+
+    def clone(self, offset=None):
+        """Return a new Transport object, cloned from the current location,
+        using a subdirectory or parent directory. This allows connections 
+        to be pooled, rather than a new one needed for each subdir.
+        """
+        raise NotImplementedError
+
+    def should_cache(self):
+        """Return True if the data pulled across should be cached locally.
+        """
+        return False
+
+    def _pump(self, from_file, to_file):
+        """Most children will need to copy from one file-like 
+        object or string to another one.
+        This just gives them something easy to call.
+        """
+        if isinstance(from_file, basestring):
+            to_file.write(from_file)
+        else:
+            from bzrlib.osutils import pumpfile
+            pumpfile(from_file, to_file)
+
+    def _get_total(self, multi):
+        """Try to figure out how many entries are in multi,
+        but if not possible, return None.
+        """
+        try:
+            return len(multi)
+        except TypeError: # We can't tell how many, because relpaths is a generator
+            return None
+
+    def _update_pb(self, pb, msg, count, total):
+        """Update the progress bar based on the current count
+        and total available, total may be None if it was
+        not possible to determine.
+        """
+        if pb is None:
+            return
+        if total is None:
+            pb.update(msg, count, count+1)
+        else:
+            pb.update(msg, count, total)
+
+    def _iterate_over(self, multi, func, pb, msg, expand=True):
+        """Iterate over all entries in multi, passing them to func,
+        and update the progress bar as you go along.
+
+        :param expand:  If True, the entries will be passed to the function
+                        by expanding the tuple. If False, it will be passed
+                        as a single parameter.
+        """
+        total = self._get_total(multi)
+        count = 0
+        for entry in multi:
+            self._update_pb(pb, msg, count, total)
+            if expand:
+                func(*entry)
+            else:
+                func(entry)
+            count += 1
+        return count
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        This can be supplied with a string or a list
+        """
+        raise NotImplementedError
+
+    def relpath(self, abspath):
+        """Return the local path portion from a given absolute path.
+        """
+        raise NotImplementedError
+
+    def has(self, relpath):
+        """Does the target location exist?"""
+        raise NotImplementedError
+
+    def has_multi(self, relpaths, pb=None):
+        """Return True/False for each entry in relpaths"""
+        total = self._get_total(relpaths)
+        count = 0
+        for relpath in relpaths:
+            self._update_pb(pb, 'has', count, total)
+            yield self.has(relpath)
+            count += 1
+
+    def get(self, relpath):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        raise NotImplementedError
+
+    def get_multi(self, relpaths, pb=None):
+        """Get a list of file-like objects, one for each entry in relpaths.
+
+        :param relpaths: A list of relative paths.
+        :param pb:  An optional ProgressBar for indicating percent done.
+        :return: A list or generator of file-like objects
+        """
+        # TODO: Consider having this actually buffer the requests,
+        # in the default mode, it probably won't give worse performance,
+        # and all children wouldn't have to implement buffering
+        total = self._get_total(relpaths)
+        count = 0
+        for relpath in relpaths:
+            self._update_pb(pb, 'get', count, total)
+            yield self.get(relpath)
+            count += 1
+
+    def get_partial(self, relpath, start, length=None):
+        """Get just part of a file.
+
+        :param relpath: Path to the file, relative to base
+        :param start: The starting position to read from
+        :param length: The length to read. A length of None indicates
+                       read to the end of the file.
+        :return: A file-like object containing at least the specified bytes.
+                 Some implementations may return objects which can be read
+                 past this length, but this is not guaranteed.
+        """
+        raise NotImplementedError
+
+    def get_partial_multi(self, offsets, pb=None):
+        """Put a set of files or strings into the location.
+
+        Requesting multiple portions of the same file can be dangerous.
+
+        :param offsets: A list of tuples of relpath, start, length
+                         [(path1, start1, length1),
+                          (path2, start2, length2),
+                          (path3, start3), ...]
+                         length is optional, defaulting to None
+        :param pb:  An optional ProgressBar for indicating percent done.
+        :return: A generator of file-like objects.
+        """
+        total = self._get_total(offsets)
+        count = 0
+        for offset in offsets:
+            self._update_pb(pb, 'get_partial', count, total)
+            yield self.get_partial(*offset)
+            count += 1
+
+    def put(self, relpath, f):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        """
+        raise NotImplementedError
+
+    def put_multi(self, files, pb=None):
+        """Put a set of files or strings into the location.
+
+        :param files: A list of tuples of relpath, file object [(path1, file1), (path2, file2),...]
+        :param pb:  An optional ProgressBar for indicating percent done.
+        :return: The number of files copied.
+        """
+        return self._iterate_over(files, self.put, pb, 'put', expand=True)
+
+    def mkdir(self, relpath):
+        """Create a directory at the given path."""
+        raise NotImplementedError
+
+    def mkdir_multi(self, relpaths, pb=None):
+        """Create a group of directories"""
+        return self._iterate_over(relpaths, self.mkdir, pb, 'mkdir', expand=False)
+
+    def append(self, relpath, f):
+        """Append the text in the file-like or string object to 
+        the supplied location.
+        """
+        raise NotImplementedError
+
+    def append_multi(self, files, pb=None):
+        """Append the text in each file-like or string object to
+        the supplied location.
+
+        :param files: A set of (path, f) entries
+        :param pb:  An optional ProgressBar for indicating percent done.
+        """
+        return self._iterate_over(files, self.append, pb, 'append', expand=True)
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        raise NotImplementedError
+
+    def copy_multi(self, relpaths, pb=None):
+        """Copy a bunch of entries.
+        
+        :param relpaths: A list of tuples of the form [(from, to), (from, to),...]
+        """
+        # This is the non-pipelined implementation, so that
+        # implementors don't have to implement everything.
+        return self._iterate_over(relpaths, self.copy, pb, 'copy', expand=True)
+
+    def copy_to(self, relpaths, other, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+        """
+        # The dummy implementation just does a simple get + put
+        def copy_entry(path):
+            other.put(path, self.get(path))
+
+        return self._iterate_over(relpaths, copy_entry, pb, 'copy_to', expand=False)
+
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        raise NotImplementedError
+
+    def move_multi(self, relpaths, pb=None):
+        """Move a bunch of entries.
+        
+        :param relpaths: A list of tuples of the form [(from1, to1), (from2, to2),...]
+        """
+        return self._iterate_over(relpaths, self.move, pb, 'move', expand=True)
+
+    def move_multi_to(self, relpaths, rel_to):
+        """Move a bunch of entries to a single location.
+        This differs from move_multi in that you give a list of from, and
+        a single destination, rather than multiple destinations.
+
+        :param relpaths: A list of relative paths [from1, from2, from3, ...]
+        :param rel_to: A directory where each entry should be placed.
+        """
+        # This is not implemented, because you need to do special tricks to
+        # extract the basename, and add it to rel_to
+        raise NotImplementedError
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        raise NotImplementedError
+
+    def delete_multi(self, relpaths, pb=None):
+        """Queue up a bunch of deletes to be done.
+        """
+        return self._iterate_over(relpaths, self.delete, pb, 'delete', expand=False)
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        WARNING: This may not be implementable for all protocols, so use
+        sparingly.
+        """
+        raise NotImplementedError
+
+    def stat_multi(self, relpaths, pb=None):
+        """Stat multiple files and return the information.
+        """
+        #TODO:  Is it worth making this a generator instead of a
+        #       returning a list?
+        stats = []
+        def gather(path):
+            stats.append(self.stat(path))
+
+        count = self._iterate_over(relpaths, gather, pb, 'stat', expand=False)
+        return stats
+
+    def listable(self):
+        """Return True if this store supports listing."""
+        raise NotImplementedError
+
+    def list_dir(self, relpath):
+        """Return a list of all files at the given location.
+        WARNING: many transports do not support this, so trying avoid using
+        it if at all possible.
+        """
+        raise TransportNotPossible("This transport has not "
+                                   "implemented list_dir.")
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should contain an unlock() function.
+        """
+        raise NotImplementedError
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should contain an unlock() function.
+        """
+        raise NotImplementedError
+
+
+def get_transport(base):
+    global _protocol_handlers
+    if base is None:
+        base = '.'
+    for proto, klass in _protocol_handlers.iteritems():
+        if proto is not None and base.startswith(proto):
+            return klass(base)
+    # The default handler is the filesystem handler
+    # which has a lookup of None
+    return _protocol_handlers[None](base)
+
+# Local transport should always be initialized
+import bzrlib.transport.local

=== added file 'bzrlib/transport/http.py'
--- bzrlib/transport/http.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/http.py	2005-10-04 04:37:29 +0000
@@ -0,0 +1,250 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Implementation of Transport over http.
+"""
+
+from bzrlib.transport import Transport, register_transport
+from bzrlib.errors import (TransportNotPossible, NoSuchFile, 
+                           NonRelativePath, TransportError)
+import os, errno
+from cStringIO import StringIO
+import urllib2
+import urlparse
+
+from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib.branch import Branch
+from bzrlib.trace import mutter
+
+# velocitynet.com.au transparently proxies connections and thereby
+# breaks keep-alive -- sucks!
+
+
+def get_url(url):
+    import urllib2
+    mutter("get_url %s" % url)
+    url_f = urllib2.urlopen(url)
+    return url_f
+
+class HttpTransportError(TransportError):
+    pass
+
+class HttpTransport(Transport):
+    """This is the transport agent for http:// access.
+    
+    TODO: Implement pipelined versions of all of the *_multi() functions.
+    """
+
+    def __init__(self, base):
+        """Set the base path where files will be stored."""
+        assert base.startswith('http://') or base.startswith('https://')
+        super(HttpTransport, self).__init__(base)
+        # In the future we might actually connect to the remote host
+        # rather than using get_url
+        # self._connection = None
+        (self._proto, self._host,
+            self._path, self._parameters,
+            self._query, self._fragment) = urlparse.urlparse(self.base)
+
+    def should_cache(self):
+        """Return True if the data pulled across should be cached locally.
+        """
+        return True
+
+    def clone(self, offset=None):
+        """Return a new HttpTransport with root at self.base + offset
+        For now HttpTransport does not actually connect, so just return
+        a new HttpTransport object.
+        """
+        if offset is None:
+            return HttpTransport(self.base)
+        else:
+            return HttpTransport(self.abspath(offset))
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        This can be supplied with a string or a list
+        """
+        if isinstance(relpath, basestring):
+            relpath = [relpath]
+        basepath = self._path.split('/')
+        if len(basepath) > 0 and basepath[-1] == '':
+            basepath = basepath[:-1]
+
+        for p in relpath:
+            if p == '..':
+                if len(basepath) < 0:
+                    # In most filesystems, a request for the parent
+                    # of root, just returns root.
+                    continue
+                basepath.pop()
+            elif p == '.':
+                continue # No-op
+            else:
+                basepath.append(p)
+
+        # Possibly, we could use urlparse.urljoin() here, but
+        # I'm concerned about when it chooses to strip the last
+        # portion of the path, and when it doesn't.
+        path = '/'.join(basepath)
+        return urlparse.urlunparse((self._proto,
+                self._host, path, '', '', ''))
+
+    def relpath(self, abspath):
+        if not abspath.startswith(self.base):
+            raise NonRelativePath('path %r is not under base URL %r'
+                           % (abspath, self.base))
+        pl = len(self.base)
+        return abspath[pl:].lstrip('/')
+
+    def has(self, relpath):
+        """Does the target location exist?
+
+        TODO: HttpTransport.has() should use a HEAD request,
+        not a full GET request.
+
+        TODO: This should be changed so that we don't use
+        urllib2 and get an exception, the code path would be
+        cleaner if we just do an http HEAD request, and parse
+        the return code.
+        """
+        try:
+            f = get_url(self.abspath(relpath))
+            # Without the read and then close()
+            # we tend to have busy sockets.
+            f.read()
+            f.close()
+            return True
+        except BzrError:
+            return False
+        except urllib2.URLError:
+            return False
+        except IOError, e:
+            if e.errno == errno.ENOENT:
+                return False
+            raise HttpTransportError(orig_error=e)
+
+    def get(self, relpath, decode=False):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        try:
+            return get_url(self.abspath(relpath))
+        except (BzrError, urllib2.URLError, IOError), e:
+            raise NoSuchFile(orig_error=e)
+        except Exception,e:
+            raise HttpTransportError(orig_error=e)
+
+    def get_partial(self, relpath, start, length=None):
+        """Get just part of a file.
+
+        :param relpath: Path to the file, relative to base
+        :param start: The starting position to read from
+        :param length: The length to read. A length of None indicates
+                       read to the end of the file.
+        :return: A file-like object containing at least the specified bytes.
+                 Some implementations may return objects which can be read
+                 past this length, but this is not guaranteed.
+        """
+        # TODO: You can make specialized http requests for just
+        # a portion of the file. Figure out how to do that.
+        # For now, urllib2 returns files that cannot seek() so
+        # we just read bytes off the beginning, until we
+        # get to the point that we care about.
+        f = self.get(relpath)
+        # TODO: read in smaller chunks, in case things are
+        # buffered internally.
+        f.read(start)
+        return f
+
+    def put(self, relpath, f):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        """
+        raise TransportNotPossible('http PUT not supported')
+
+    def mkdir(self, relpath):
+        """Create a directory at the given path."""
+        raise TransportNotPossible('http does not support mkdir()')
+
+    def append(self, relpath, f):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        raise TransportNotPossible('http does not support append()')
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        raise TransportNotPossible('http does not support copy()')
+
+    def copy_to(self, relpaths, other, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+
+        TODO: if other is LocalTransport, is it possible to
+              do better than put(get())?
+        """
+        # At this point HttpTransport might be able to check and see if
+        # the remote location is the same, and rather than download, and
+        # then upload, it could just issue a remote copy_this command.
+        if isinstance(other, HttpTransport):
+            raise TransportNotPossible('http cannot be the target of copy_to()')
+        else:
+            return super(HttpTransport, self).copy_to(relpaths, other, pb=pb)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        raise TransportNotPossible('http does not support move()')
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        raise TransportNotPossible('http does not support delete()')
+
+    def listable(self):
+        """See Transport.listable."""
+        return False
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        """
+        raise TransportNotPossible('http does not support stat()')
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        # The old RemoteBranch ignore lock for reading, so we will
+        # continue that tradition and return a bogus lock object.
+        class BogusLock(object):
+            def __init__(self, path):
+                self.path = path
+            def unlock(self):
+                pass
+        return BogusLock(relpath)
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        raise TransportNotPossible('http does not support lock_write()')
+
+register_transport('http://', HttpTransport)
+register_transport('https://', HttpTransport)

=== added file 'bzrlib/transport/local.py'
--- bzrlib/transport/local.py	1970-01-01 00:00:00 +0000
+++ bzrlib/transport/local.py	2005-10-04 04:37:29 +0000
@@ -0,0 +1,232 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+"""Implementation of Transport for the local filesystem.
+"""
+
+from bzrlib.transport import Transport, register_transport, \
+    TransportError, NoSuchFile, FileExists
+import os, errno
+
+class LocalTransportError(TransportError):
+    pass
+
+class LocalTransport(Transport):
+    """This is the transport agent for local filesystem access."""
+
+    def __init__(self, base):
+        """Set the base path where files will be stored."""
+        if base.startswith('file://'):
+            base = base[7:]
+        # realpath is incompatible with symlinks. When we traverse
+        # up we might be able to normpath stuff. RBC 20051003
+        super(LocalTransport, self).__init__(
+            os.path.normpath(os.path.abspath(base)))
+
+    def should_cache(self):
+        return False
+
+    def clone(self, offset=None):
+        """Return a new LocalTransport with root at self.base + offset
+        Because the local filesystem does not require a connection, 
+        we can just return a new object.
+        """
+        if offset is None:
+            return LocalTransport(self.base)
+        else:
+            return LocalTransport(self.abspath(offset))
+
+    def abspath(self, relpath):
+        """Return the full url to the given relative path.
+        This can be supplied with a string or a list
+        """
+        if isinstance(relpath, basestring):
+            relpath = [relpath]
+        return os.path.join(self.base, *relpath)
+
+    def relpath(self, abspath):
+        """Return the local path portion from a given absolute path.
+        """
+        from bzrlib.branch import _relpath
+        return _relpath(self.base, abspath)
+
+    def has(self, relpath):
+        return os.access(self.abspath(relpath), os.F_OK)
+
+    def get(self, relpath):
+        """Get the file at the given relative path.
+
+        :param relpath: The relative path to the file
+        """
+        try:
+            path = self.abspath(relpath)
+            return open(path, 'rb')
+        except IOError,e:
+            if e.errno == errno.ENOENT:
+                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
+            raise LocalTransportError(orig_error=e)
+
+    def get_partial(self, relpath, start, length=None):
+        """Get just part of a file.
+
+        :param relpath: Path to the file, relative to base
+        :param start: The starting position to read from
+        :param length: The length to read. A length of None indicates
+                       read to the end of the file.
+        :return: A file-like object containing at least the specified bytes.
+                 Some implementations may return objects which can be read
+                 past this length, but this is not guaranteed.
+        """
+        # LocalTransport.get_partial() doesn't care about the length
+        # argument, because it is using a local file, and thus just
+        # returns the file seek'ed to the appropriate location.
+        try:
+            path = self.abspath(relpath)
+            f = open(path, 'rb')
+            f.seek(start, 0)
+            return f
+        except IOError,e:
+            if e.errno == errno.ENOENT:
+                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
+            raise LocalTransportError(orig_error=e)
+
+    def put(self, relpath, f):
+        """Copy the file-like or string object into the location.
+
+        :param relpath: Location to put the contents, relative to base.
+        :param f:       File-like or string object.
+        """
+        from bzrlib.atomicfile import AtomicFile
+
+        try:
+            path = self.abspath(relpath)
+            fp = AtomicFile(path, 'wb')
+        except IOError, e:
+            if e.errno == errno.ENOENT:
+                raise NoSuchFile('File %r does not exist' % path, orig_error=e)
+            raise LocalTransportError(orig_error=e)
+        try:
+            self._pump(f, fp)
+            fp.commit()
+        finally:
+            fp.close()
+
+    def mkdir(self, relpath):
+        """Create a directory at the given path."""
+        try:
+            os.mkdir(self.abspath(relpath))
+        except OSError,e:
+            if e.errno == errno.EEXIST:
+                raise FileExists(orig_error=e)
+            elif e.errno == errno.ENOENT:
+                raise NoSuchFile(orig_error=e)
+            raise LocalTransportError(orig_error=e)
+
+    def append(self, relpath, f):
+        """Append the text in the file-like object into the final
+        location.
+        """
+        fp = open(self.abspath(relpath), 'ab')
+        self._pump(f, fp)
+
+    def copy(self, rel_from, rel_to):
+        """Copy the item at rel_from to the location at rel_to"""
+        import shutil
+        path_from = self.abspath(rel_from)
+        path_to = self.abspath(rel_to)
+        try:
+            shutil.copy(path_from, path_to)
+        except OSError,e:
+            raise LocalTransportError(orig_error=e)
+
+    def move(self, rel_from, rel_to):
+        """Move the item at rel_from to the location at rel_to"""
+        path_from = self.abspath(rel_from)
+        path_to = self.abspath(rel_to)
+
+        try:
+            os.rename(path_from, path_to)
+        except OSError,e:
+            raise LocalTransportError(orig_error=e)
+
+    def delete(self, relpath):
+        """Delete the item at relpath"""
+        try:
+            os.remove(self.abspath(relpath))
+        except OSError,e:
+            raise LocalTransportError(orig_error=e)
+
+    def copy_to(self, relpaths, other, pb=None):
+        """Copy a set of entries from self into another Transport.
+
+        :param relpaths: A list/generator of entries to be copied.
+        """
+        if isinstance(other, LocalTransport):
+            # Both from & to are on the local filesystem
+            # Unfortunately, I can't think of anything faster than just
+            # copying them across, one by one :(
+            import shutil
+
+            total = self._get_total(relpaths)
+            count = 0
+            for path in relpaths:
+                self._update_pb(pb, 'copy-to', count, total)
+                shutil.copy(self.abspath(path), other.abspath(path))
+                count += 1
+            return count
+        else:
+            return super(LocalTransport, self).copy_to(relpaths, other, pb=pb)
+
+    def listable(self):
+        """See Transport.listable."""
+        return True
+
+    def list_dir(self, relpath):
+        """Return a list of all files at the given location.
+        WARNING: many transports do not support this, so trying avoid using
+        it if at all possible.
+        """
+        try:
+            return os.listdir(self.abspath(relpath))
+        except OSError,e:
+            raise LocalTransportError(orig_error=e)
+
+    def stat(self, relpath):
+        """Return the stat information for a file.
+        """
+        try:
+            return os.stat(self.abspath(relpath))
+        except OSError,e:
+            raise LocalTransportError(orig_error=e)
+
+    def lock_read(self, relpath):
+        """Lock the given file for shared (read) access.
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        from bzrlib.lock import ReadLock
+        return ReadLock(self.abspath(relpath))
+
+    def lock_write(self, relpath):
+        """Lock the given file for exclusive (write) access.
+        WARNING: many transports do not support this, so trying avoid using it
+
+        :return: A lock object, which should be passed to Transport.unlock()
+        """
+        from bzrlib.lock import WriteLock
+        return WriteLock(self.abspath(relpath))
+
+# If nothing else matches, try the LocalTransport
+register_transport(None, LocalTransport)
+register_transport('file://', LocalTransport)

=== modified file 'bzrlib/tree.py'
--- bzrlib/tree.py	2005-07-11 07:33:45 +0000
+++ bzrlib/tree.py	2005-10-07 06:13:14 +0000
@@ -17,13 +17,15 @@
 """Tree classes, representing directory at point in time.
 """
 
-from osutils import pumpfile, appendpath, fingerprint_file
 import os
+from cStringIO import StringIO
 
+import bzrlib
 from bzrlib.trace import mutter, note
-from bzrlib.errors import BzrError
+from bzrlib.errors import BzrError, BzrCheckError
+from bzrlib.inventory import Inventory
+from bzrlib.osutils import appendpath, fingerprint_file
 
-import bzrlib
 
 exporters = {}
 
@@ -66,11 +68,16 @@
 
     def _get_inventory(self):
         return self._inventory
+    
+    def get_file_by_path(self, path):
+        return self.get_file(self._inventory.path2id(path))
 
     inventory = property(_get_inventory,
                          doc="Inventory of this Tree")
 
     def _check_retrieved(self, ie, f):
+        if not __debug__:
+            return  
         fp = fingerprint_file(f)
         f.seek(0)
         
@@ -88,10 +95,10 @@
                      "store is probably damaged/corrupt"])
 
 
-    def print_file(self, fileid):
-        """Print file with id `fileid` to stdout."""
+    def print_file(self, file_id):
+        """Print file with id `file_id` to stdout."""
         import sys
-        pumpfile(self.get_file(fileid), sys.stdout)
+        sys.stdout.write(self.get_file_text(file_id))
         
         
     def export(self, dest, format='dir', root=None):
@@ -115,23 +122,42 @@
            or at least passing a description to the constructor.
     """
     
-    def __init__(self, store, inv):
-        self._store = store
+    def __init__(self, weave_store, inv, revision_id):
+        self._weave_store = weave_store
         self._inventory = inv
+        self._revision_id = revision_id
+
+    def get_weave(self, file_id):
+        # FIXME: RevisionTree should be given a branch
+        # not a store, or the store should know the branch.
+        import bzrlib.transactions as transactions
+        return self._weave_store.get_weave(file_id,
+            transactions.PassThroughTransaction())
+
+
+    def get_file_lines(self, file_id):
+        ie = self._inventory[file_id]
+        weave = self.get_weave(file_id)
+        return weave.get(ie.revision)
+        
+
+    def get_file_text(self, file_id):
+        return ''.join(self.get_file_lines(file_id))
+
 
     def get_file(self, file_id):
-        ie = self._inventory[file_id]
-        f = self._store[ie.text_id]
-        mutter("  get fileid{%s} from %r" % (file_id, self))
-        self._check_retrieved(ie, f)
-        return f
+        return StringIO(self.get_file_text(file_id))
 
     def get_file_size(self, file_id):
         return self._inventory[file_id].text_size
 
     def get_file_sha1(self, file_id):
         ie = self._inventory[file_id]
-        return ie.text_sha1
+        if ie.kind == "file":
+            return ie.text_sha1
+
+    def is_executable(self, file_id):
+        return self._inventory[file_id].executable
 
     def has_filename(self, filename):
         return bool(self.inventory.path2id(filename))
@@ -139,21 +165,32 @@
     def list_files(self):
         # The only files returned by this are those from the version
         for path, entry in self.inventory.iter_entries():
-            yield path, 'V', entry.kind, entry.file_id
+            yield path, 'V', entry.kind, entry.file_id, entry
+
+    def get_symlink_target(self, file_id):
+        ie = self._inventory[file_id]
+        return ie.symlink_target;
 
 
 class EmptyTree(Tree):
-    def __init__(self, root_id):
-        from bzrlib.inventory import Inventory
-        self._inventory = Inventory(root_id)
+    def __init__(self):
+        self._inventory = Inventory()
+
+    def get_symlink_target(self, file_id):
+        return None
 
     def has_filename(self, filename):
         return False
 
     def list_files(self):
-        if False:  # just to make it a generator
-            yield None
+        return iter([])
     
+    def __contains__(self, file_id):
+        return file_id in self._inventory
+
+    def get_file_sha1(self, file_id):
+        assert self._inventory[file_id].kind == "root_directory"
+        return None
 
 
 ######################################################################
@@ -242,15 +279,8 @@
     mutter('export version %r' % tree)
     inv = tree.inventory
     for dp, ie in inv.iter_entries():
-        kind = ie.kind
-        fullpath = appendpath(dest, dp)
-        if kind == 'directory':
-            os.mkdir(fullpath)
-        elif kind == 'file':
-            pumpfile(tree.get_file(ie.file_id), file(fullpath, 'wb'))
-        else:
-            raise BzrError("don't know how to export {%s} of kind %r" % (ie.file_id, kind))
-        mutter("  export {%s} kind %s to %s" % (ie.file_id, kind, fullpath))
+        ie.put_on_disk(dest, dp, tree)
+
 exporters['dir'] = dir_exporter
 
 try:
@@ -299,27 +329,10 @@
         inv = tree.inventory
         for dp, ie in inv.iter_entries():
             mutter("  export {%s} kind %s to %s" % (ie.file_id, ie.kind, dest))
-            item = tarfile.TarInfo(os.path.join(root, dp))
-            # TODO: would be cool to actually set it to the timestamp of the
-            # revision it was last changed
-            item.mtime = now
-            if ie.kind == 'directory':
-                item.type = tarfile.DIRTYPE
-                fileobj = None
-                item.name += '/'
-                item.size = 0
-                item.mode = 0755
-            elif ie.kind == 'file':
-                item.type = tarfile.REGTYPE
-                fileobj = tree.get_file(ie.file_id)
-                item.size = _find_file_size(fileobj)
-                item.mode = 0644
-            else:
-                raise BzrError("don't know how to export {%s} of kind %r" %
-                        (ie.file_id, ie.kind))
-
+            item, fileobj = ie.get_tar_item(root, dp, now, tree)
             ball.addfile(item, fileobj)
         ball.close()
+
     exporters['tar'] = tar_exporter
 
     def tgz_exporter(tree, dest, root):
@@ -329,21 +342,3 @@
     def tbz_exporter(tree, dest, root):
         tar_exporter(tree, dest, root, compression='bz2')
     exporters['tbz2'] = tbz_exporter
-
-
-def _find_file_size(fileobj):
-    offset = fileobj.tell()
-    try:
-        fileobj.seek(0, 2)
-        size = fileobj.tell()
-    except TypeError:
-        # gzip doesn't accept second argument to seek()
-        fileobj.seek(0)
-        size = 0
-        while True:
-            nread = len(fileobj.read())
-            if nread == 0:
-                break
-            size += nread
-    fileobj.seek(offset)
-    return size

=== added file 'bzrlib/ui.py'
--- bzrlib/ui.py	1970-01-01 00:00:00 +0000
+++ bzrlib/ui.py	2005-08-25 12:46:42 +0000
@@ -0,0 +1,51 @@
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+
+"""UI abstraction.
+
+This tells the library how to display things to the user.  Through this
+layer different applications can choose the style of UI.
+
+At the moment this layer is almost trivial: the application can just
+choose the style of progress bar.
+
+Set the ui_factory member to define the behaviour.  The default
+displays no output.
+"""
+
+
+
+
+
+import bzrlib.progress
+
+
+class TextUIFactory(object):
+    def progress_bar(self):
+
+        # this in turn is abstract, and creates either a tty or dots
+        # bar depending on what we think of the terminal
+        return bzrlib.progress.ProgressBar()
+
+
+class SilentUIFactory(object):
+    def progress_bar(self):
+        return bzrlib.progress.DummyProgress()
+
+
+ui_factory = SilentUIFactory()

=== added file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	1970-01-01 00:00:00 +0000
+++ bzrlib/upgrade.py	2005-10-06 06:31:33 +0000
@@ -0,0 +1,386 @@
+#! /usr/bin/python
+#
+# Copyright (C) 2005 Canonical Ltd
+#
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+#
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+#
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Experiment in converting existing bzr branches to weaves."""
+
+# To make this properly useful
+#
+# 1. assign text version ids, and put those text versions into
+#    the inventory as they're converted.
+#
+# 2. keep track of the previous version of each file, rather than
+#    just using the last one imported
+#
+# 3. assign entry versions when files are added, renamed or moved.
+#
+# 4. when merged-in versions are observed, walk down through them
+#    to discover everything, then commit bottom-up
+#
+# 5. track ancestry as things are merged in, and commit that in each
+#    revision
+#
+# Perhaps it's best to first walk the whole graph and make a plan for
+# what should be imported in what order?  Need a kind of topological
+# sort of all revisions.  (Or do we, can we just before doing a revision
+# see that all its parents have either been converted or abandoned?)
+
+
+# Cannot import a revision until all its parents have been
+# imported.  in other words, we can only import revisions whose
+# parents have all been imported.  the first step must be to
+# import a revision with no parents, of which there must be at
+# least one.  (So perhaps it's useful to store forward pointers
+# from a list of parents to their children?)
+#
+# Another (equivalent?) approach is to build up the ordered
+# ancestry list for the last revision, and walk through that.  We
+# are going to need that.
+#
+# We don't want to have to recurse all the way back down the list.
+#
+# Suppose we keep a queue of the revisions able to be processed at
+# any point.  This starts out with all the revisions having no
+# parents.
+#
+# This seems like a generally useful algorithm...
+#
+# The current algorithm is dumb (O(n**2)?) but will do the job, and
+# takes less than a second on the bzr.dev branch.
+
+# This currently does a kind of lazy conversion of file texts, where a
+# new text is written in every version.  That's unnecessary but for
+# the moment saves us having to worry about when files need new
+# versions.
+
+
+# TODO: Don't create a progress bar here, have it passed by the caller.  
+# At least do it from the UI factory.
+
+if False:
+    try:
+        import psyco
+        psyco.full()
+    except ImportError:
+        pass
+
+
+import os
+import tempfile
+import sys
+import logging
+import shutil
+
+from bzrlib.branch import Branch, find_branch, BZR_BRANCH_FORMAT_5
+from bzrlib.revfile import Revfile
+from bzrlib.weave import Weave
+from bzrlib.weavefile import read_weave, write_weave
+from bzrlib.ui import ui_factory
+from bzrlib.atomicfile import AtomicFile
+from bzrlib.xml4 import serializer_v4
+from bzrlib.xml5 import serializer_v5
+from bzrlib.trace import mutter, note, warning, enable_default_logging
+from bzrlib.osutils import sha_strings, sha_string
+
+
+class Convert(object):
+    def __init__(self, base_dir):
+        self.base = base_dir
+        self.converted_revs = set()
+        self.absent_revisions = set()
+        self.text_count = 0
+        self.revisions = {}
+        self.convert()
+
+
+    def convert(self):
+        if not self._open_branch():
+            return
+        note('starting upgrade of %s', os.path.abspath(self.base))
+        self._backup_control_dir()
+        note('starting upgrade')
+        note('note: upgrade may be faster if all store files are ungzipped first')
+        self.pb = ui_factory.progress_bar()
+        if not os.path.isdir(self.base + '/.bzr/weaves'):
+            os.mkdir(self.base + '/.bzr/weaves')
+        self.inv_weave = Weave('inventory')
+        # holds in-memory weaves for all files
+        self.text_weaves = {}
+        os.remove(self.branch.controlfilename('branch-format'))
+        self._convert_working_inv()
+        rev_history = self.branch.revision_history()
+        # to_read is a stack holding the revisions we still need to process;
+        # appending to it adds new highest-priority revisions
+        self.known_revisions = set(rev_history)
+        self.to_read = [rev_history[-1]]
+        while self.to_read:
+            rev_id = self.to_read.pop()
+            if (rev_id not in self.revisions
+                and rev_id not in self.absent_revisions):
+                self._load_one_rev(rev_id)
+        self.pb.clear()
+        to_import = self._make_order()
+        for i, rev_id in enumerate(to_import):
+            self.pb.update('converting revision', i, len(to_import))
+            self._convert_one_rev(rev_id)
+        self.pb.clear()
+        note('upgraded to weaves:')
+        note('  %6d revisions and inventories' % len(self.revisions))
+        note('  %6d revisions not present' % len(self.absent_revisions))
+        note('  %6d texts' % self.text_count)
+        self._write_all_weaves()
+        self._write_all_revs()
+        self._set_new_format()
+        self._cleanup_spare_files()
+
+
+    def _open_branch(self):
+        self.branch = Branch.open_downlevel(self.base)
+        if self.branch._branch_format == 5:
+            note('this branch is already in the most current format')
+            return False
+        if self.branch._branch_format != 4:
+            raise BzrError("cannot upgrade from branch format %r" %
+                           self.branch._branch_format)
+        return True
+
+
+    def _set_new_format(self):
+        self.branch.put_controlfile('branch-format', BZR_BRANCH_FORMAT_5)
+
+
+    def _cleanup_spare_files(self):
+        for n in 'merged-patches', 'pending-merged-patches':
+            p = self.branch.controlfilename(n)
+            if not os.path.exists(p):
+                continue
+            ## assert os.path.getsize(p) == 0
+            os.remove(p)
+        shutil.rmtree(self.base + '/.bzr/inventory-store')
+        shutil.rmtree(self.base + '/.bzr/text-store')
+
+
+    def _backup_control_dir(self):
+        orig = self.base + '/.bzr'
+        backup = orig + '.backup'
+        note('making backup of tree history')
+        shutil.copytree(orig, backup)
+        note('%s has been backed up to %s', orig, backup)
+        note('if conversion fails, you can move this directory back to .bzr')
+        note('if it succeeds, you can remove this directory if you wish')
+
+
+    def _convert_working_inv(self):
+        branch = self.branch
+        inv = serializer_v4.read_inventory(branch.controlfile('inventory', 'rb'))
+        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        branch.put_controlfile('inventory', new_inv_xml)
+
+
+
+    def _write_all_weaves(self):
+        write_a_weave(self.inv_weave, self.base + '/.bzr/inventory.weave')
+        i = 0
+        try:
+            for file_id, file_weave in self.text_weaves.items():
+                self.pb.update('writing weave', i, len(self.text_weaves))
+                write_a_weave(file_weave, self.base + '/.bzr/weaves/%s.weave' % file_id)
+                i += 1
+        finally:
+            self.pb.clear()
+
+
+    def _write_all_revs(self):
+        """Write all revisions out in new form."""
+        shutil.rmtree(self.base + '/.bzr/revision-store')
+        os.mkdir(self.base + '/.bzr/revision-store')
+        try:
+            for i, rev_id in enumerate(self.converted_revs):
+                self.pb.update('write revision', i, len(self.converted_revs))
+                f = file(self.base + '/.bzr/revision-store/%s' % rev_id, 'wb')
+                try:
+                    serializer_v5.write_revision(self.revisions[rev_id], f)
+                finally:
+                    f.close()
+        finally:
+            self.pb.clear()
+
+            
+    def _load_one_rev(self, rev_id):
+        """Load a revision object into memory.
+
+        Any parents not either loaded or abandoned get queued to be
+        loaded."""
+        self.pb.update('loading revision',
+                       len(self.revisions),
+                       len(self.known_revisions))
+        if rev_id not in self.branch.revision_store:
+            self.pb.clear()
+            note('revision {%s} not present in branch; '
+                 'will be converted as a ghost',
+                 rev_id)
+            self.absent_revisions.add(rev_id)
+        else:
+            rev_xml = self.branch.revision_store[rev_id].read()
+            rev = serializer_v4.read_revision_from_string(rev_xml)
+            for parent_id in rev.parent_ids:
+                self.known_revisions.add(parent_id)
+                self.to_read.append(parent_id)
+            self.revisions[rev_id] = rev
+
+
+    def _load_old_inventory(self, rev_id):
+        assert rev_id not in self.converted_revs
+        old_inv_xml = self.branch.inventory_store[rev_id].read()
+        inv = serializer_v4.read_inventory_from_string(old_inv_xml)
+        rev = self.revisions[rev_id]
+        if rev.inventory_sha1:
+            assert rev.inventory_sha1 == sha_string(old_inv_xml), \
+                'inventory sha mismatch for {%s}' % rev_id
+        return inv
+        
+
+    def _load_updated_inventory(self, rev_id):
+        assert rev_id in self.converted_revs
+        inv_xml = self.inv_weave.get_text(rev_id)
+        inv = serializer_v5.read_inventory_from_string(inv_xml)
+        return inv
+
+
+    def _convert_one_rev(self, rev_id):
+        """Convert revision and all referenced objects to new format."""
+        rev = self.revisions[rev_id]
+        inv = self._load_old_inventory(rev_id)
+        present_parents = [p for p in rev.parent_ids
+                           if p not in self.absent_revisions]
+        self._convert_revision_contents(rev, inv, present_parents)
+        self._store_new_weave(rev, inv, present_parents)
+        self.converted_revs.add(rev_id)
+
+
+    def _store_new_weave(self, rev, inv, present_parents):
+        # the XML is now updated with text versions
+        if __debug__:
+            for file_id in inv:
+                ie = inv[file_id]
+                if ie.kind == 'root_directory':
+                    continue
+                assert hasattr(ie, 'revision'), \
+                    'no revision on {%s} in {%s}' % \
+                    (file_id, rev.revision_id)
+        new_inv_xml = serializer_v5.write_inventory_to_string(inv)
+        new_inv_sha1 = sha_string(new_inv_xml)
+        self.inv_weave.add(rev.revision_id, 
+                           present_parents,
+                           new_inv_xml.splitlines(True),
+                           new_inv_sha1)
+        rev.inventory_sha1 = new_inv_sha1
+
+    def _convert_revision_contents(self, rev, inv, present_parents):
+        """Convert all the files within a revision.
+
+        Also upgrade the inventory to refer to the text revision ids."""
+        rev_id = rev.revision_id
+        mutter('converting texts of revision {%s}',
+               rev_id)
+        parent_invs = map(self._load_updated_inventory, present_parents)
+        for file_id in inv:
+            ie = inv[file_id]
+            self._convert_file_version(rev, ie, parent_invs)
+
+    def _convert_file_version(self, rev, ie, parent_invs):
+        """Convert one version of one file.
+
+        The file needs to be added into the weave if it is a merge
+        of >=2 parents or if it's changed from its parent.
+        """
+        if ie.kind == 'root_directory':
+            return
+        file_id = ie.file_id
+        rev_id = rev.revision_id
+        w = self.text_weaves.get(file_id)
+        if w is None:
+            w = Weave(file_id)
+            self.text_weaves[file_id] = w
+        text_changed = False
+        previous_entries = ie.find_previous_heads(parent_invs, w)
+        for old_revision in previous_entries:
+                # if this fails, its a ghost ?
+                assert old_revision in self.converted_revs 
+        self.snapshot_ie(previous_entries, ie, w, rev_id)
+        del ie.text_id
+        assert getattr(ie, 'revision', None) is not None
+
+    def snapshot_ie(self, previous_revisions, ie, w, rev_id):
+        # TODO: convert this logic, which is ~= snapshot to
+        # a call to:. This needs the path figured out. rather than a work_tree
+        # a v4 revision_tree can be given, or something that looks enough like
+        # one to give the file content to the entry if it needs it.
+        # and we need something that looks like a weave store for snapshot to 
+        # save against.
+        #ie.snapshot(rev, PATH, previous_revisions, REVISION_TREE, InMemoryWeaveStore(self.text_weaves))
+        if len(previous_revisions) == 1:
+            previous_ie = previous_revisions.values()[0]
+            if ie._unchanged(previous_ie):
+                ie.revision = previous_ie.revision
+                return
+        parent_indexes = map(w.lookup, previous_revisions)
+        if ie.has_text():
+            file_lines = self.branch.text_store[ie.text_id].readlines()
+            assert sha_strings(file_lines) == ie.text_sha1
+            assert sum(map(len, file_lines)) == ie.text_size
+            w.add(rev_id, parent_indexes, file_lines, ie.text_sha1)
+            self.text_count += 1
+        else:
+            w.add(rev_id, parent_indexes, [], None)
+        ie.revision = rev_id
+        ##mutter('import text {%s} of {%s}',
+        ##       ie.text_id, file_id)
+
+    def _make_order(self):
+        """Return a suitable order for importing revisions.
+
+        The order must be such that an revision is imported after all
+        its (present) parents.
+        """
+        todo = set(self.revisions.keys())
+        done = self.absent_revisions.copy()
+        o = []
+        while todo:
+            # scan through looking for a revision whose parents
+            # are all done
+            for rev_id in sorted(list(todo)):
+                rev = self.revisions[rev_id]
+                parent_ids = set(rev.parent_ids)
+                if parent_ids.issubset(done):
+                    # can take this one now
+                    o.append(rev_id)
+                    todo.remove(rev_id)
+                    done.add(rev_id)
+        return o
+
+
+def write_a_weave(weave, filename):
+    inv_wf = file(filename, 'wb')
+    try:
+        write_weave(weave, inv_wf)
+    finally:
+        inv_wf.close()
+
+
+def upgrade(base_dir):
+    Convert(base_dir)

=== removed file 'bzrlib/upgrade.py'
--- bzrlib/upgrade.py	2005-06-28 05:33:40 +0000
+++ bzrlib/upgrade.py	1970-01-01 00:00:00 +0000
@@ -1,138 +0,0 @@
-# Copyright (C) 2004, 2005 by Martin Pool
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-
-
-def upgrade(branch):
-    """
-    Upgrade branch to current format.
-
-    This causes objects to be rewritten into the current format.
-
-    If they change, their SHA-1 will of course change, which might
-    break any later signatures, or backreferences that check the
-    SHA-1.
-
-    TODO: Check non-mainline revisions.
-    """
-    import sys
-
-    from bzrlib.trace import mutter
-    from bzrlib.errors import BzrCheckError
-    from bzrlib.progress import ProgressBar
-
-    branch.lock_write()
-
-    try:
-        pb = ProgressBar(show_spinner=True)
-        last_rev_id = None
-
-        history = branch.revision_history()
-        revno = 0
-        revcount = len(history)
-
-        updated_revisions = []
-
-        # Set to True in the case that the previous revision entry
-        # was updated, since this will affect future revision entries
-        updated_previous_revision = False
-
-        for rev_id in history:
-            revno += 1
-            pb.update('upgrading revision', revno, revcount)
-            rev = branch.get_revision(rev_id)
-            if rev.revision_id != rev_id:
-                raise BzrCheckError('wrong internal revision id in revision {%s}'
-                                    % rev_id)
-
-            last_rev_id = rev_id
-
-            # if set to true, revision must be written out
-            updated = False
-
-            if rev.inventory_sha1 is None:
-                rev.inventory_sha1 = branch.get_inventory_sha1(rev.inventory_id)
-                updated = True
-                mutter("  set inventory_sha1 on {%s}" % rev_id)
-
-            for prr in rev.parents:
-                actual_sha1 = branch.get_revision_sha1(prr.revision_id)
-                if (updated_previous_revision
-                    or prr.revision_sha1 is None):
-                    if prr.revision_sha1 != actual_sha1:
-                        prr.revision_sha1 = actual_sha1
-                        updated = True
-                elif actual_sha1 != prr.revision_sha1:
-                    raise BzrCheckError("parent {%s} of {%s} sha1 mismatch: "
-                                        "%s vs %s"
-                                        % (prr.revision_id, rev_id,
-                                           actual_sha1, prr.revision_sha1))
-
-            if updated:
-                updated_previous_revision = True
-                # We had to update this revision entries hashes
-                # Now we need to write out a new value
-                # This is a little bit invasive, since we are *rewriting* a
-                # revision entry. I'm not supremely happy about it, but
-                # there should be *some* way of making old entries have
-                # the full meta information.
-                import tempfile, os, errno
-                from bzrlib.xml import pack_xml
-                
-                rev_tmp = tempfile.TemporaryFile()
-                pack_xml(rev, rev_tmp)
-                rev_tmp.seek(0)
-
-                tmpfd, tmp_path = tempfile.mkstemp(prefix=rev_id, suffix='.gz',
-                    dir=branch.controlfilename('revision-store'))
-                os.close(tmpfd)
-                def special_rename(p1, p2):
-                    if sys.platform == 'win32':
-                        try:
-                            os.remove(p2)
-                        except OSError, e:
-                            if e.errno != errno.ENOENT:
-                                raise
-                    os.rename(p1, p2)
-
-                try:
-                    # TODO: We may need to handle the case where the old revision
-                    # entry was not compressed (and thus did not end with .gz)
-
-                    # Remove the old revision entry out of the way
-                    rev_path = branch.controlfilename(['revision-store', rev_id+'.gz'])
-                    special_rename(rev_path, tmp_path)
-                    branch.revision_store.add(rev_tmp, rev_id) # Add the new one
-                    os.remove(tmp_path) # Remove the old name
-                    mutter('    Updated revision entry {%s}' % rev_id)
-                except:
-                    # On any exception, restore the old entry
-                    special_rename(tmp_path, rev_path)
-                    raise
-                rev_tmp.close()
-                updated_revisions.append(rev_id)
-            else:
-                updated_previous_revision = False
-
-    finally:
-        branch.unlock()
-
-    pb.clear()
-
-    if updated_revisions:
-        print '%d revisions updated to current format' % len(updated_revisions)

=== added directory 'bzrlib/util'
=== added file 'bzrlib/util/__init__.py'
=== renamed directory 'effbot' => 'bzrlib/util/effbot'
=== renamed directory 'elementtree' => 'bzrlib/util/elementtree'
=== renamed directory 'urlgrabber' => 'bzrlib/util/urlgrabber'
=== modified file 'bzrlib/weave.py'
--- bzrlib/weave.py	2005-07-18 18:53:46 +0000
+++ bzrlib/weave.py	2005-10-08 00:03:38 +0000
@@ -21,34 +21,6 @@
 
 """Weave - storage of related text file versions"""
 
-# before intset (r923) 2000 versions in 41.5s
-# with intset (r926) 2000 versions in 93s !!!
-# better to just use plain sets.
-
-# making _extract build and return a list, rather than being a generator
-# takes 37.94s
-
-# with python -O, r923 does 2000 versions in 36.87s
-
-# with optimizations to avoid mutating lists - 35.75!  I guess copying
-# all the elements every time costs more than the small manipulations.
-# a surprisingly small change.
-
-# r931, which avoids using a generator for extract, does 36.98s
-
-# with memoized inclusions, takes 41.49s; not very good
-
-# with slots, takes 37.35s; without takes 39.16, a bit surprising
-
-# with the delta calculation mixed in with the add method, rather than
-# separated, takes 36.78s
-
-# with delta folded in and mutation of the list, 36.13s
-
-# with all this and simplification of add code, 33s 
-
-
-# TODO: Perhaps have copy method for Weave instances?
 
 # XXX: If we do weaves this way, will a merge still behave the same
 # way if it's done in a different order?  That's a pretty desirable
@@ -59,8 +31,6 @@
 # binaries, perhaps by naively splitting on \n or perhaps using
 # something like a rolling checksum.
 
-# TODO: Track version names as well as indexes. 
-
 # TODO: End marker for each version so we can stop reading?
 
 # TODO: Check that no insertion occurs inside a deletion that was
@@ -73,9 +43,38 @@
 
 # TODO: Parallel-extract that passes back each line along with a
 # description of which revisions include it.  Nice for checking all
-# shas in parallel.
-
-
+# shas or calculating stats in parallel.
+
+# TODO: Using a single _extract routine and then processing the output
+# is probably inefficient.  It's simple enough that we can afford to
+# have slight specializations for different ways its used: annotate,
+# basis for add, get, etc.
+
+# TODO: Perhaps the API should work only in names to hide the integer
+# indexes from the user?
+
+# TODO: Is there any potential performance win by having an add()
+# variant that is passed a pre-cooked version of the single basis
+# version?
+
+# TODO: Have a way to go back and insert a revision V1 that is a parent 
+# of an already-stored revision V2.  This means some lines previously 
+# counted as new in V2 will be discovered to have actually come from V1.  
+# It is probably necessary to insert V1, then compute a whole new diff 
+# from the mashed ancestors to V2.  This must be repeated for every 
+# direct child of V1.  The deltas from V2 to its descendents won't change,
+# although their location within the weave may change.  It may be possible
+# to just adjust the location of those instructions rather than 
+# re-weaving the whole thing.  This is expected to be a fairly rare
+# operation, only used when inserting data that was previously a ghost.
+
+
+
+
+import sha
+from difflib import SequenceMatcher
+
+from bzrlib.trace import mutter
 
 
 class WeaveError(Exception):
@@ -84,6 +83,10 @@
 
 class WeaveFormatError(WeaveError):
     """Weave invariant violated"""
+
+
+class WeaveParentMismatch(WeaveError):
+    """Parents are mismatched between two revisions."""
     
 
 class Weave(object):
@@ -118,7 +121,9 @@
     The instruction can be '{' or '}' for an insertion block, and '['
     and ']' for a deletion block respectively.  The version is the
     integer version index.  There is no replace operator, only deletes
-    and inserts.
+    and inserts.  For '}', the end of an insertion, there is no
+    version parameter because it always closes the most recently
+    opened insertion.
 
     Constraints/notes:
 
@@ -160,52 +165,135 @@
         each version; the parent's parents are implied.
 
     _sha1s
-        List of hex SHA-1 of each version, or None if not recorded.
+        List of hex SHA-1 of each version.
+
+    _names
+        List of symbolic names for each version.  Each should be unique.
+
+    _name_map
+        For each name, the version number.
+
+    _weave_name
+        Descriptive name of this weave; typically the filename if known.
+        Set by read_weave.
     """
 
-    __slots__ = ['_weave', '_parents', '_sha1s']
+    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
+                 '_weave_name']
     
-    def __init__(self):
+    def __init__(self, weave_name=None):
         self._weave = []
         self._parents = []
         self._sha1s = []
-
+        self._names = []
+        self._name_map = {}
+        self._weave_name = weave_name
+
+
+    def copy(self):
+        """Return a deep copy of self.
+        
+        The copy can be modified without affecting the original weave."""
+        other = Weave()
+        other._weave = self._weave[:]
+        other._parents = self._parents[:]
+        other._sha1s = self._sha1s[:]
+        other._names = self._names[:]
+        other._name_map = self._name_map.copy()
+        other._weave_name = self._weave_name
+        return other
 
     def __eq__(self, other):
         if not isinstance(other, Weave):
             return False
         return self._parents == other._parents \
-               and self._weave == other._weave
+               and self._weave == other._weave \
+               and self._sha1s == other._sha1s 
+
     
-
     def __ne__(self, other):
         return not self.__eq__(other)
 
-        
-    def add(self, parents, text):
+
+    def maybe_lookup(self, name_or_index):
+        """Convert possible symbolic name to index, or pass through indexes."""
+        if isinstance(name_or_index, (int, long)):
+            return name_or_index
+        else:
+            return self.lookup(name_or_index)
+
+        
+    def lookup(self, name):
+        """Convert symbolic version name to index."""
+        try:
+            return self._name_map[name]
+        except KeyError:
+            raise WeaveError("name %r not present in weave %r" %
+                             (name, self._weave_name))
+
+
+    def iter_names(self):
+        """Yield a list of all names in this weave."""
+        return iter(self._names)
+
+    def idx_to_name(self, version):
+        return self._names[version]
+
+
+    def _check_repeated_add(self, name, parents, text, sha1):
+        """Check that a duplicated add is OK.
+
+        If it is, return the (old) index; otherwise raise an exception.
+        """
+        idx = self.lookup(name)
+        if sorted(self._parents[idx]) != sorted(parents):
+            raise WeaveError("name \"%s\" already present in weave "
+                             "with different parents" % name)
+        if sha1 != self._sha1s[idx]:
+            raise WeaveError("name \"%s\" already present in weave "
+                             "with different text" % name)            
+        return idx
+        
+
+        
+    def add(self, name, parents, text, sha1=None):
         """Add a single text on top of the weave.
   
         Returns the index number of the newly added version.
 
+        name
+            Symbolic name for this version.
+            (Typically the revision-id of the revision that added it.)
+
         parents
             List or set of direct parent version numbers.
             
         text
-            Sequence of lines to be added in the new version."""
-
+            Sequence of lines to be added in the new version.
+
+        sha -- SHA-1 of the file, if known.  This is trusted to be
+            correct if supplied.
+        """
+        from bzrlib.osutils import sha_strings
+
+        assert isinstance(name, basestring)
+        if sha1 is None:
+            sha1 = sha_strings(text)
+        if name in self._name_map:
+            return self._check_repeated_add(name, parents, text, sha1)
+
+        parents = map(self.maybe_lookup, parents)
         self._check_versions(parents)
         ## self._check_lines(text)
         new_version = len(self._parents)
 
-        import sha
-        s = sha.new()
-        map(s.update, text)
-        sha1 = s.hexdigest()
-        del s
 
-        # if we abort after here the weave will be corrupt
-        self._parents.append(frozenset(parents))
+        # if we abort after here the (in-memory) weave will be corrupt because only
+        # some fields are updated
+        self._parents.append(parents[:])
         self._sha1s.append(sha1)
+        self._names.append(name)
+        self._name_map[name] = new_version
 
             
         if not parents:
@@ -216,7 +304,7 @@
             if text:
                 self._weave.append(('{', new_version))
                 self._weave.extend(text)
-                self._weave.append(('}', new_version))
+                self._weave.append(('}', None))
         
             return new_version
 
@@ -238,7 +326,8 @@
             basis_lineno.append(lineno)
             basis_lines.append(line)
 
-        # another small special case: a merge, producing the same text as auto-merge
+        # another small special case: a merge, producing the same text
+        # as auto-merge
         if text == basis_lines:
             return new_version            
 
@@ -252,7 +341,6 @@
         #print 'basis_lines:', basis_lines
         #print 'new_lines:  ', lines
 
-        from difflib import SequenceMatcher
         s = SequenceMatcher(None, basis_lines, text)
 
         # offset gives the number of lines that have been inserted
@@ -287,26 +375,36 @@
                 # we don't destroy ourselves
                 i = i2 + offset
                 self._weave[i:i] = ([('{', new_version)] 
-                                + text[j1:j2] 
-                                + [('}', new_version)])
+                                    + text[j1:j2] 
+                                    + [('}', None)])
                 offset += 2 + (j2 - j1)
 
         return new_version
 
+    def add_identical(self, old_rev_id, new_rev_id, parents):
+        """Add an identical text to old_rev_id as new_rev_id."""
+        old_lines = self.get(self.lookup(old_rev_id))
+        self.add(new_rev_id, parents, old_lines)
 
     def inclusions(self, versions):
         """Return set of all ancestors of given version(s)."""
         i = set(versions)
-        v = max(versions)
-        try:
-            while v >= 0:
-                if v in i:
-                    # include all its parents
-                    i.update(self._parents[v])
-                v -= 1
-            return i
-        except IndexError:
-            raise ValueError("version %d not present in weave" % v)
+        for v in xrange(max(versions), 0, -1):
+            if v in i:
+                # include all its parents
+                i.update(self._parents[v])
+        return i
+        ## except IndexError:
+        ##     raise ValueError("version %d not present in weave" % v)
+
+
+    def parents(self, version):
+        return self._parents[version]
+
+
+    def parent_names(self, version):
+        """Return version names for parents of a version."""
+        return map(self.idx_to_name, self._parents[self.lookup(version)])
 
 
     def minimal_parents(self, version):
@@ -352,15 +450,16 @@
                 raise IndexError("invalid version number %r" % i)
 
     
-    def annotate(self, index):
-        return list(self.annotate_iter(index))
-
-
-    def annotate_iter(self, version):
+    def annotate(self, name_or_index):
+        return list(self.annotate_iter(name_or_index))
+
+
+    def annotate_iter(self, name_or_index):
         """Yield list of (index-id, line) pairs for the specified version.
 
         The index indicates when the line originated in the weave."""
-        for origin, lineno, text in self._extract([version]):
+        incls = [self.maybe_lookup(name_or_index)]
+        for origin, lineno, text in self._extract(incls):
             yield origin, text
 
 
@@ -384,7 +483,7 @@
                 if c == '{':
                     istack.append(v)
                 elif c == '}':
-                    oldv = istack.pop()
+                    istack.pop()
                 elif c == '[':
                     assert v not in dset
                     dset.add(v)
@@ -410,6 +509,10 @@
 
         The set typically but not necessarily corresponds to a version.
         """
+        for i in versions:
+            if not isinstance(i, int):
+                raise ValueError(i)
+            
         included = self.inclusions(versions)
 
         istack = []
@@ -431,8 +534,7 @@
                     assert v not in istack
                     istack.append(v)
                 elif c == '}':
-                    oldv = istack.pop()
-                    assert oldv == v
+                    istack.pop()
                 elif c == '[':
                     if v in included:
                         assert v not in dset
@@ -461,18 +563,28 @@
     
 
 
-    def get_iter(self, version):
+    def get_iter(self, name_or_index):
         """Yield lines for the specified version."""
-        for origin, lineno, line in self._extract([version]):
+        incls = [self.maybe_lookup(name_or_index)]
+        for origin, lineno, line in self._extract(incls):
             yield line
 
 
-    def get(self, index):
-        return list(self.get_iter(index))
+    def get_text(self, name_or_index):
+        return ''.join(self.get_iter(name_or_index))
+        assert isinstance(version, int)
+
+
+    def get_lines(self, name_or_index):
+        return list(self.get_iter(name_or_index))
+
+
+    get = get_lines
 
 
     def mash_iter(self, included):
         """Return composed version of multiple included versions."""
+        included = map(self.maybe_lookup, included)
         for origin, lineno, text in self._extract(included):
             yield text
 
@@ -508,7 +620,6 @@
 
         # try extracting all versions; this is a bit slow and parallel
         # extraction could be used
-        import sha
         nv = self.numversions()
         for version in range(nv):
             if progress_bar:
@@ -539,7 +650,6 @@
         If there is a chunk of the file where there's no diagreement,
         only one alternative is given.
         """
-
         # approach: find the included versions common to all the
         # merged versions
         raise NotImplementedError()
@@ -565,7 +675,7 @@
         If line1=line2, this is a pure insert; if newlines=[] this is a
         pure delete.  (Similar to difflib.)
         """
-
+        raise NotImplementedError()
 
             
     def plan_merge(self, ver_a, ver_b):
@@ -665,29 +775,181 @@
                        state
 
                 
-
-
-
-
-
-def weave_info(w):
-    """Show some text information about the weave."""
-    print '%6s %40s %20s' % ('ver', 'sha1', 'parents')
-    for i in (6, 40, 20):
+    def join(self, other):
+        """Integrate versions from other into this weave.
+
+        The resulting weave contains all the history of both weaves; 
+        any version you could retrieve from either self or other can be 
+        retrieved from self after this call.
+
+        It is illegal for the two weaves to contain different values 
+        or different parents for any version.  See also reweave().
+        """
+        if other.numversions() == 0:
+            return          # nothing to update, easy
+        # two loops so that we do not change ourselves before verifying it
+        # will be ok
+        # work through in index order to make sure we get all dependencies
+        for other_idx, name in enumerate(other._names):
+            if self._check_version_consistent(other, other_idx, name):
+                continue
+        for other_idx, name in enumerate(other._names):
+            # TODO: If all the parents of the other version are already 
+            # present then we can avoid some work by just taking the delta
+            # and adjusting the offsets.
+            new_parents = self._imported_parents(other, other_idx)
+            lines = other.get_lines(other_idx)
+            sha1 = other._sha1s[other_idx]
+            self.add(name, new_parents, lines, sha1)
+
+
+    def _imported_parents(self, other, other_idx):
+        """Return list of parents in self corresponding to indexes in other."""
+        new_parents = []
+        for parent_idx in other._parents[other_idx]:
+            parent_name = other._names[parent_idx]
+            if parent_name not in self._names:
+                # should not be possible
+                raise WeaveError("missing parent {%s} of {%s} in %r" 
+                                 % (parent_name, other._name_map[other_idx], self))
+            new_parents.append(self._name_map[parent_name])
+        return new_parents
+
+    def _check_version_consistent(self, other, other_idx, name):
+        """Check if a version in consistent in this and other.
+
+        To be consistent it must have:
+
+         * the same text
+         * the same direct parents (by name, not index, and disregarding
+           order)
+        
+        If present & correct return True;
+        if not present in self return False; 
+        if inconsistent raise error."""
+        this_idx = self._name_map.get(name, -1)
+        if this_idx != -1:
+            if self._sha1s[this_idx] != other._sha1s[other_idx]:
+                raise WeaveError("inconsistent texts for version {%s} "
+                                 "when joining weaves"
+                                 % (name))
+            self_parents = self._parents[this_idx]
+            other_parents = other._parents[other_idx]
+            n1 = [self._names[i] for i in self_parents]
+            n2 = [other._names[i] for i in other_parents]
+            n1.sort()
+            n2.sort()
+            if n1 != n2:
+                raise WeaveParentMismatch("inconsistent parents "
+                    "for version {%s}: %s vs %s" % (name, n1, n2))
+            else:
+                return True         # ok!
+        else:
+            return False
+
+    def reweave(self, other):
+        """Reweave self with other."""
+        new_weave = reweave(self, other)
+        for attr in self.__slots__:
+            setattr(self, attr, getattr(new_weave, attr))
+
+
+def reweave(wa, wb):
+    """Combine two weaves and return the result.
+
+    This works even if a revision R has different parents in 
+    wa and wb.  In the resulting weave all the parents are given.
+
+    This is done by just building up a new weave, maintaining ordering 
+    of the versions in the two inputs.  More efficient approaches
+    might be possible but it should only be necessary to do 
+    this operation rarely, when a new previously ghost version is 
+    inserted.
+    """
+    wr = Weave()
+    ia = ib = 0
+    queue_a = range(wa.numversions())
+    queue_b = range(wb.numversions())
+    # first determine combined parents of all versions
+    # map from version name -> all parent names
+    combined_parents = _reweave_parent_graphs(wa, wb)
+    mutter("combined parents: %r", combined_parents)
+    order = _make_reweave_order(wa._names, wb._names, combined_parents)
+    mutter("order to reweave: %r", order)
+    for name in order:
+        if name in wa._name_map:
+            lines = wa.get_lines(name)
+            if name in wb._name_map:
+                assert lines == wb.get_lines(name)
+        else:
+            lines = wb.get_lines(name)
+        wr.add(name, combined_parents[name], lines)
+    return wr
+
+
+def _reweave_parent_graphs(wa, wb):
+    """Return combined parent ancestry for two weaves.
+    
+    Returned as a list of (version_name, set(parent_names))"""
+    combined = {}
+    for weave in [wa, wb]:
+        for idx, name in enumerate(weave._names):
+            p = combined.setdefault(name, set())
+            p.update(map(weave.idx_to_name, weave._parents[idx]))
+    return combined
+
+
+def _make_reweave_order(wa_order, wb_order, combined_parents):
+    """Return an order to reweave versions respecting parents."""
+    done = set()
+    result = []
+    ia = ib = 0
+    next_a = next_b = None
+    len_a = len(wa_order)
+    len_b = len(wb_order)
+    while ia < len(wa_order) or ib < len(wb_order):
+        if ia < len_a:
+            next_a = wa_order[ia]
+            if next_a in done:
+                ia += 1
+                continue
+            if combined_parents[next_a].issubset(done):
+                ia += 1
+                result.append(next_a)
+                done.add(next_a)
+                continue
+        if ib < len_b:
+            next_b = wb_order[ib]
+            if next_b in done:
+                ib += 1
+                continue
+            elif combined_parents[next_b].issubset(done):
+                ib += 1
+                result.append(next_b)
+                done.add(next_b)
+                continue
+        raise WeaveError("don't know how to reweave at {%s} and {%s}"
+                         % (next_a, next_b))
+    return result
+
+
+def weave_toc(w):
+    """Show the weave's table-of-contents"""
+    print '%6s %50s %10s %10s' % ('ver', 'name', 'sha1', 'parents')
+    for i in (6, 50, 10, 10):
         print '-' * i,
     print
     for i in range(w.numversions()):
         sha1 = w._sha1s[i]
-        print '%6d %40s %s' % (i, sha1, ' '.join(map(str, w._parents[i])))
-
-
-
-def weave_stats(weave_file):
-    from bzrlib.progress import ProgressBar
+        name = w._names[i]
+        parent_str = ' '.join(map(str, w._parents[i]))
+        print '%6d %-50.50s %10.10s %s' % (i, name, sha1, parent_str)
+
+
+
+def weave_stats(weave_file, pb):
     from bzrlib.weavefile import read_weave
 
-    pb = ProgressBar()
-
     wf = file(weave_file, 'rb')
     w = read_weave(wf)
     # FIXME: doesn't work on pipes
@@ -697,7 +959,7 @@
     vers = len(w)
     for i in range(vers):
         pb.update('checking sizes', i, vers)
-        for line in w.get_iter(i):
+        for origin, lineno, line in w._extract([i]):
             total += len(line)
 
     pb.clear()
@@ -706,7 +968,10 @@
     print 'weave file        %9d bytes' % weave_size
     print 'total contents    %9d bytes' % total
     print 'compression ratio %9.2fx' % (float(total) / float(weave_size))
-
+    if vers:
+        avg = total/vers
+        print 'average size      %9d bytes' % avg
+        print 'relative size     %9.2fx' % (float(weave_size) / float(avg))
 
 
 def usage():
@@ -721,9 +986,9 @@
         Write out specified version.
     weave check WEAVEFILE
         Check consistency of all versions.
-    weave info WEAVEFILE
+    weave toc WEAVEFILE
         Display table of contents.
-    weave add WEAVEFILE [BASE...] < NEWTEXT
+    weave add WEAVEFILE NAME [BASE...] < NEWTEXT
         Add NEWTEXT, with specified parent versions.
     weave annotate WEAVEFILE VERSION
         Display origin of each line.
@@ -731,28 +996,30 @@
         Display composite of all selected versions.
     weave merge WEAVEFILE VERSION1 VERSION2 > OUT
         Auto-merge two versions and display conflicts.
+    weave diff WEAVEFILE VERSION1 VERSION2 
+        Show differences between two versions.
 
 example:
 
     % weave init foo.weave
     % vi foo.txt
-    % weave add foo.weave < foo.txt
+    % weave add foo.weave ver0 < foo.txt
     added version 0
 
     (create updated version)
     % vi foo.txt
     % weave get foo.weave 0 | diff -u - foo.txt
-    % weave add foo.weave 0 < foo.txt
+    % weave add foo.weave ver1 0 < foo.txt
     added version 1
 
     % weave get foo.weave 0 > foo.txt       (create forked version)
     % vi foo.txt
-    % weave add foo.weave 0 < foo.txt
+    % weave add foo.weave ver2 0 < foo.txt
     added version 2
 
     % weave merge foo.weave 1 2 > foo.txt   (merge them)
     % vi foo.txt                            (resolve conflicts)
-    % weave add foo.weave 1 2 < foo.txt     (commit merged version)     
+    % weave add foo.weave merged 1 2 < foo.txt     (commit merged version)     
     
 """
     
@@ -761,11 +1028,24 @@
 def main(argv):
     import sys
     import os
-    from weavefile import write_weave, read_weave
+    try:
+        import bzrlib
+    except ImportError:
+        # in case we're run directly from the subdirectory
+        sys.path.append('..')
+        import bzrlib
+    from bzrlib.weavefile import write_weave, read_weave
     from bzrlib.progress import ProgressBar
 
-    #import psyco
-    #psyco.full()
+    try:
+        import psyco
+        psyco.full()
+    except ImportError:
+        pass
+
+    if len(argv) < 2:
+        usage()
+        return 0
 
     cmd = argv[1]
 
@@ -777,11 +1057,12 @@
     elif cmd == 'add':
         w = readit()
         # at the moment, based on everything in the file
-        parents = map(int, argv[3:])
+        name = argv[3]
+        parents = map(int, argv[4:])
         lines = sys.stdin.readlines()
-        ver = w.add(parents, lines)
+        ver = w.add(name, parents, lines)
         write_weave(w, file(argv[2], 'wb'))
-        print 'added version %d' % ver
+        print 'added version %r %d' % (name, ver)
     elif cmd == 'init':
         fn = argv[2]
         if os.path.exists(fn):
@@ -796,6 +1077,18 @@
         w = readit()
         sys.stdout.writelines(w.mash_iter(map(int, argv[3:])))
 
+    elif cmd == 'diff':
+        from difflib import unified_diff
+        w = readit()
+        fn = argv[2]
+        v1, v2 = map(int, argv[3:5])
+        lines1 = w.get(v1)
+        lines2 = w.get(v2)
+        diff_gen = unified_diff(lines1, lines2,
+                                '%s version %d' % (fn, v1),
+                                '%s version %d' % (fn, v2))
+        sys.stdout.writelines(diff_gen)
+            
     elif cmd == 'annotate':
         w = readit()
         # newline is added to all lines regardless; too hard to get
@@ -809,11 +1102,11 @@
                 print '%5d | %s' % (origin, text)
                 lasto = origin
                 
-    elif cmd == 'info':
-        weave_info(readit())
+    elif cmd == 'toc':
+        weave_toc(readit())
 
     elif cmd == 'stats':
-        weave_stats(argv[2])
+        weave_stats(argv[2], ProgressBar())
         
     elif cmd == 'check':
         w = readit()
@@ -865,6 +1158,33 @@
         raise ValueError('unknown command %r' % cmd)
     
 
+
+def profile_main(argv): 
+    import tempfile, hotshot, hotshot.stats
+
+    prof_f = tempfile.NamedTemporaryFile()
+
+    prof = hotshot.Profile(prof_f.name)
+
+    ret = prof.runcall(main, argv)
+    prof.close()
+
+    stats = hotshot.stats.load(prof_f.name)
+    #stats.strip_dirs()
+    stats.sort_stats('cumulative')
+    ## XXX: Might like to write to stderr or the trace file instead but
+    ## print_stats seems hardcoded to stdout
+    stats.print_stats(20)
+            
+    return ret
+
+
 if __name__ == '__main__':
     import sys
-    sys.exit(main(sys.argv))
+    if '--profile' in sys.argv:
+        args = sys.argv[:]
+        args.remove('--profile')
+        sys.exit(profile_main(args))
+    else:
+        sys.exit(main(sys.argv))
+

=== modified file 'bzrlib/weavefile.py'
--- bzrlib/weavefile.py	2005-07-18 13:38:13 +0000
+++ bzrlib/weavefile.py	2005-09-27 07:24:40 +0000
@@ -1,5 +1,3 @@
-#! /usr/bin/python
-
 # Copyright (C) 2005 Canonical Ltd
 
 # This program is free software; you can redistribute it and/or modify
@@ -26,9 +24,13 @@
 There is one format marker followed by a blank line, followed by a
 series of version headers, followed by the weave itself.
 
-Each version marker has 'i' and the included previous versions, then
-'1' and the SHA-1 of the text, if known.  The inclusions do not need
-to list versions included by a parent.
+Each version marker has
+
+ 'i'   parent version indexes
+ '1'   SHA-1 of text
+ 'n'   name
+
+The inclusions do not need to list versions included by a parent.
 
 The weave is bracketed by 'w' and 'W' lines, and includes the '{}[]'
 processing instructions.  Lines of text are prefixed by '.' if the
@@ -36,19 +38,20 @@
 """
 
 # TODO: When extracting a single version it'd be enough to just pass
-# an iterator returning the weave lines...
+# an iterator returning the weave lines...  We don't really need to
+# deserialize it into memory.
 
-FORMAT_1 = '# bzr weave file v3\n'
+FORMAT_1 = '# bzr weave file v5\n'
 
 
 def write_weave(weave, f, format=None):
     if format == None or format == 1:
-        return write_weave_v1(weave, f)
+        return write_weave_v5(weave, f)
     else:
         raise ValueError("unknown weave format %r" % format)
 
 
-def write_weave_v1(weave, f):
+def write_weave_v5(weave, f):
     """Write weave to file f."""
     print >>f, FORMAT_1,
 
@@ -63,6 +66,7 @@
         else:
             print >>f, 'i'
         print >>f, '1', weave._sha1s[version]
+        print >>f, 'n', weave._names[version]
         print >>f
 
     print >>f, 'w'
@@ -70,7 +74,10 @@
     for l in weave._weave:
         if isinstance(l, tuple):
             assert l[0] in '{}[]'
-            print >>f, '%s %d' % l
+            if l[0] == '}':
+                print >>f, '}'
+            else:
+                print >>f, '%s %d' % l
         else: # text line
             if not l:
                 print >>f, ', '
@@ -86,14 +93,13 @@
 
 
 def read_weave(f):
-    return read_weave_v1(f)
-
-
-def read_weave_v1(f):
+    return read_weave_v5(f)
+
+
+def read_weave_v5(f):
     from weave import Weave, WeaveFormatError
-    w = Weave()
+    w = Weave(getattr(f, 'name', None))
 
-    wfe = WeaveFormatError
     l = f.readline()
     if l != FORMAT_1:
         raise WeaveFormatError('invalid weave file header: %r' % l)
@@ -102,19 +108,26 @@
     while True:
         l = f.readline()
         if l[0] == 'i':
-            ver += 1
-
             if len(l) > 2:
-                w._parents.append(frozenset(map(int, l[2:].split(' '))))
+                w._parents.append(map(int, l[2:].split(' ')))
             else:
-                w._parents.append(frozenset())
+                w._parents.append([])
 
             l = f.readline()[:-1]
             assert l.startswith('1 ')
             w._sha1s.append(l[2:])
                 
             l = f.readline()
+            assert l.startswith('n ')
+            name = l[2:-1]
+            assert name not in w._name_map
+            w._names.append(name)
+            w._name_map[name] = ver
+                
+            l = f.readline()
             assert l == '\n'
+
+            ver += 1
         elif l == 'w\n':
             break
         else:
@@ -128,8 +141,10 @@
             w._weave.append(l[2:])  # include newline
         elif l.startswith(', '):
             w._weave.append(l[2:-1])        # exclude newline
+        elif l == '}\n':
+            w._weave.append(('}', None))
         else:
-            assert l[0] in '{}[]', l
+            assert l[0] in '{[]', l
             assert l[1] == ' ', l
             w._weave.append((intern(l[0]), int(l[2:])))
 

=== modified file 'bzrlib/workingtree.py'
--- bzrlib/workingtree.py	2005-07-22 19:32:52 +0000
+++ bzrlib/workingtree.py	2005-10-06 04:29:10 +0000
@@ -21,10 +21,68 @@
 # it's not predictable when it will be written out.
 
 import os
-    
+import stat
+import fnmatch
+        
 import bzrlib.tree
-from errors import BzrCheckError
-from trace import mutter
+from bzrlib.osutils import appendpath, file_kind, isdir, splitpath
+from bzrlib.errors import BzrCheckError
+from bzrlib.trace import mutter
+
+class TreeEntry(object):
+    """An entry that implements the minium interface used by commands.
+
+    This needs further inspection, it may be better to have 
+    InventoryEntries without ids - though that seems wrong. For now,
+    this is a parallel hierarchy to InventoryEntry, and needs to become
+    one of several things: decorates to that hierarchy, children of, or
+    parents of it.
+    Another note is that these objects are currently only used when there is
+    no InventoryEntry available - i.e. for unversioned objects.
+    Perhaps they should be UnversionedEntry et al. ? - RBC 20051003
+    """
+ 
+    def __eq__(self, other):
+        # yes, this us ugly, TODO: best practice __eq__ style.
+        return (isinstance(other, TreeEntry)
+                and other.__class__ == self.__class__)
+ 
+    def kind_character(self):
+        return "???"
+
+
+class TreeDirectory(TreeEntry):
+    """See TreeEntry. This is a directory in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeDirectory)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return "/"
+
+
+class TreeFile(TreeEntry):
+    """See TreeEntry. This is a regular file in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeFile)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
+
+class TreeLink(TreeEntry):
+    """See TreeEntry. This is a symlink in a working tree."""
+
+    def __eq__(self, other):
+        return (isinstance(other, TreeLink)
+                and other.__class__ == self.__class__)
+
+    def kind_character(self):
+        return ''
+
 
 class WorkingTree(bzrlib.tree.Tree):
     """Working copy tree.
@@ -67,7 +125,7 @@
         """
         inv = self._inventory
         for path, ie in inv.iter_entries():
-            if os.path.exists(self.abspath(path)):
+            if bzrlib.osutils.lexists(self.abspath(path)):
                 yield ie.file_id
 
 
@@ -81,7 +139,7 @@
         return os.path.join(self.basedir, filename)
 
     def has_filename(self, filename):
-        return os.path.exists(self.abspath(filename))
+        return bzrlib.osutils.lexists(self.abspath(filename))
 
     def get_file(self, file_id):
         return self.get_file_byname(self.id2path(file_id))
@@ -93,6 +151,10 @@
         ## XXX: badly named; this isn't in the store at all
         return self.abspath(self.id2path(file_id))
 
+
+    def id2abspath(self, file_id):
+        return self.abspath(self.id2path(file_id))
+
                 
     def has_id(self, file_id):
         # files that have been deleted are excluded
@@ -100,22 +162,31 @@
         if not inv.has_id(file_id):
             return False
         path = inv.id2path(file_id)
-        return os.path.exists(self.abspath(path))
+        return bzrlib.osutils.lexists(self.abspath(path))
 
 
     __contains__ = has_id
     
 
     def get_file_size(self, file_id):
-        # is this still called?
-        raise NotImplementedError()
-
+        return os.path.getsize(self.id2abspath(file_id))
 
     def get_file_sha1(self, file_id):
         path = self._inventory.id2path(file_id)
         return self._hashcache.get_sha1(path)
 
 
+    def is_executable(self, file_id):
+        if os.name == "nt":
+            return self._inventory[file_id].executable
+        else:
+            path = self._inventory.id2path(file_id)
+            mode = os.lstat(self.abspath(path)).st_mode
+            return bool(stat.S_ISREG(mode) and stat.S_IEXEC&mode)
+
+    def get_symlink_target(self, file_id):
+        return os.readlink(self.id2path(file_id))
+
     def file_class(self, filename):
         if self.path2id(filename):
             return 'V'
@@ -135,9 +206,6 @@
 
         Skips the control directory.
         """
-        from osutils import appendpath, file_kind
-        import os
-
         inv = self._inventory
 
         def descend(from_dir_relpath, from_dir_id, dp):
@@ -172,7 +240,20 @@
                                             "now of kind %r"
                                             % (fap, f_ie.kind, f_ie.file_id, fk))
 
-                yield fp, c, fk, (f_ie and f_ie.file_id)
+                # make a last minute entry
+                if f_ie:
+                    entry = f_ie
+                else:
+                    if fk == 'directory':
+                        entry = TreeDirectory()
+                    elif fk == 'file':
+                        entry = TreeFile()
+                    elif fk == 'symlink':
+                        entry = TreeLink()
+                    else:
+                        entry = TreeEntry()
+                
+                yield fp, c, fk, (f_ie and f_ie.file_id), entry
 
                 if fk != 'directory':
                     continue
@@ -194,6 +275,15 @@
             if not self.is_ignored(subp):
                 yield subp
 
+    def iter_conflicts(self):
+        conflicted = set()
+        for path in (s[0] for s in self.list_files()):
+            stem = get_conflicted_stem(path)
+            if stem is None:
+                continue
+            if stem not in conflicted:
+                conflicted.add(stem)
+                yield stem
 
     def extras(self):
         """Yield all unknown files in this WorkingTree.
@@ -205,8 +295,6 @@
         Currently returned depth-first, sorted by name within directories.
         """
         ## TODO: Work from given directory downwards
-        from osutils import isdir, appendpath
-        
         for path, dir_entry in self.inventory.directories():
             mutter("search for unknowns in %r" % path)
             dirabs = self.abspath(path)
@@ -269,9 +357,6 @@
         # Eventually it should be replaced with something more
         # accurate.
         
-        import fnmatch
-        from osutils import splitpath
-        
         for pat in self.get_ignore_list():
             if '/' in pat or '\\' in pat:
                 
@@ -290,4 +375,9 @@
                     return pat
         else:
             return None
-        
\ No newline at end of file
+
+CONFLICT_SUFFIXES = ('.THIS', '.BASE', '.OTHER')
+def get_conflicted_stem(path):
+    for suffix in CONFLICT_SUFFIXES:
+        if path.endswith(suffix):
+            return path[:-len(suffix)]

=== modified file 'bzrlib/xml.py'
--- bzrlib/xml.py	2005-06-28 05:33:40 +0000
+++ bzrlib/xml.py	2005-10-03 14:28:37 +0000
@@ -20,21 +20,55 @@
 # "XML is like violence: if it doesn't solve your problem, you aren't
 # using enough of it." -- various
 
-# importing this module is fairly slow because it has to load several ElementTree bits
+# importing this module is fairly slow because it has to load several
+# ElementTree bits
+
+from bzrlib.trace import mutter, warning
+
 try:
-    from cElementTree import ElementTree, SubElement, Element
+    from cElementTree import (ElementTree, SubElement, Element,
+                              XMLTreeBuilder, fromstring, tostring)
 except ImportError:
-    from elementtree.ElementTree import ElementTree, SubElement, Element
-
-
-def pack_xml(o, f):
-    """Write object o to file f as XML.
-
-    o must provide a to_element method.
-    """
-    ElementTree(o.to_element()).write(f, 'utf-8')
-    f.write('\n')
-
-
-def unpack_xml(cls, f):
-    return cls.from_element(ElementTree().parse(f))
+    ## from warnings import warn
+    ## warn('using slower ElementTree; consider installing cElementTree')
+    from util.elementtree.ElementTree import (ElementTree, SubElement,
+                                              Element, XMLTreeBuilder,
+                                              fromstring, tostring)
+
+from bzrlib.errors import BzrError
+
+
+class Serializer(object):
+    """Abstract object serialize/deserialize"""
+    def write_inventory(self, inv, f):
+        """Write inventory to a file"""
+        elt = self._pack_inventory(inv)
+        self._write_element(elt, f)
+
+    def write_inventory_to_string(self, inv):
+        return tostring(self._pack_inventory(inv))
+
+    def read_inventory_from_string(self, xml_string):
+        return self._unpack_inventory(fromstring(xml_string))
+
+    def read_inventory(self, f):
+        return self._unpack_inventory(self._read_element(f))
+
+    def write_revision(self, rev, f):
+        self._write_element(self._pack_revision(rev), f)
+
+    def write_revision_to_string(self, rev):
+        return tostring(self._pack_revision(rev), f)
+
+    def read_revision(self, f):
+        return self._unpack_revision(self._read_element(f))
+
+    def read_revision_from_string(self, xml_string):
+        return self._unpack_revision(fromstring(xml_string))
+
+    def _write_element(self, elt, f):
+        ElementTree(elt).write(f, 'utf-8')
+        f.write('\n')
+
+    def _read_element(self, f):
+        return ElementTree().parse(f)

=== added file 'bzrlib/xml4.py'
--- bzrlib/xml4.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml4.py	2005-10-03 15:19:25 +0000
@@ -0,0 +1,200 @@
+#! /usr/bin/env python
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.xml import ElementTree, SubElement, Element, Serializer
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+import bzrlib.inventory as inventory
+from bzrlib.revision import Revision        
+from bzrlib.errors import BzrError
+
+
+
+
+
+
+class _Serializer_v4(Serializer):
+    """Version 0.0.4 serializer
+
+    You should use the serialzer_v4 singleton."""
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        e = Element('entry')
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+        e.set('kind', ie.kind)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_id', 'text_sha1', 'symlink_target']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        assert elt.tag == 'entry'
+
+        ## original format inventories don't have a parent_id for
+        ## nodes in the root directory, but it's cleaner to use one
+        ## internally.
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        kind = elt.get('kind')
+        if kind == 'directory':
+            ie = inventory.InventoryDirectory(elt.get('file_id'),
+                                              elt.get('name'),
+                                              parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_id = elt.get('text_id')
+            ie.text_sha1 = elt.get('text_sha1')
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
+        elif kind == 'symlink':
+            ie = inventory.InventoryLink(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.symlink_target = elt.get('symlink_target')
+        else:
+            raise BzrError("unknown kind %r" % kind)
+
+        ## mutter("read inventoryentry: %r" % (elt.attrib))
+
+        return ie
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_id = rev.inventory_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+
+        if rev.parents:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for i, parent_id in enumerate(rev.parents):
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                assert parent_id
+                p.set('revision_id', parent_id)
+                if i < len(rev.parent_sha1s):
+                    p.set('revision_sha1', rev.parent_sha1s[i])
+        return root
+
+    
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        
+        # <changeset> is deprecated...
+        if elt.tag not in ('revision', 'changeset'):
+            raise BzrError("unexpected tag in revision file: %r" % elt)
+
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_id = elt.get('inventory_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+
+        precursor = elt.get('precursor')
+        precursor_sha1 = elt.get('precursor_sha1')
+
+        pelts = elt.find('parents')
+
+        if pelts:
+            for p in pelts:
+                assert p.tag == 'revision_ref', \
+                       "bad parent node tag %r" % p.tag
+                rev.parent_ids.append(p.get('revision_id'))
+                rev.parent_sha1s.append(p.get('revision_sha1'))
+            if precursor:
+                # must be consistent
+                prec_parent = rev.parent_ids[0]
+                assert prec_parent == precursor
+        elif precursor:
+            # revisions written prior to 0.0.5 have a single precursor
+            # give as an attribute
+            rev.parent_ids.append(precursor)
+            rev.parent_sha1s.append(precursor_sha1)
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+
+
+"""singleton instance"""
+serializer_v4 = _Serializer_v4()
+

=== added file 'bzrlib/xml5.py'
--- bzrlib/xml5.py	1970-01-01 00:00:00 +0000
+++ bzrlib/xml5.py	2005-10-06 06:31:16 +0000
@@ -0,0 +1,184 @@
+#! /usr/bin/env python
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+
+from bzrlib.xml import ElementTree, SubElement, Element, Serializer
+from bzrlib.inventory import ROOT_ID, Inventory, InventoryEntry
+import bzrlib.inventory as inventory
+from bzrlib.revision import Revision        
+from bzrlib.errors import BzrError
+
+
+
+
+
+class Serializer_v5(Serializer):
+    """Version 5 serializer
+
+    Packs objects into XML and vice versa.
+    """
+    
+    __slots__ = []
+    
+    def _pack_inventory(self, inv):
+        """Convert to XML Element"""
+        e = Element('inventory',
+                    format='5')
+        e.text = '\n'
+        if inv.root.file_id not in (None, ROOT_ID):
+            e.set('file_id', inv.root.file_id)
+        for path, ie in inv.iter_entries():
+            e.append(self._pack_entry(ie))
+        return e
+
+
+    def _pack_entry(self, ie):
+        """Convert InventoryEntry to XML element"""
+        if not InventoryEntry.versionable_kind(ie.kind):
+            raise AssertionError('unsupported entry kind %s' % ie.kind)
+        e = Element(ie.kind)
+        e.set('name', ie.name)
+        e.set('file_id', ie.file_id)
+
+        if ie.text_size != None:
+            e.set('text_size', '%d' % ie.text_size)
+
+        for f in ['text_sha1', 'revision', 'symlink_target']:
+            v = getattr(ie, f)
+            if v != None:
+                e.set(f, v)
+
+        if ie.executable:
+            e.set('executable', 'yes')
+
+        # to be conservative, we don't externalize the root pointers
+        # for now, leaving them as null in the xml form.  in a future
+        # version it will be implied by nested elements.
+        if ie.parent_id != ROOT_ID:
+            assert isinstance(ie.parent_id, basestring)
+            e.set('parent_id', ie.parent_id)
+
+        e.tail = '\n'
+
+        return e
+
+
+    def _pack_revision(self, rev):
+        """Revision object -> xml tree"""
+        root = Element('revision',
+                       committer = rev.committer,
+                       timestamp = '%.9f' % rev.timestamp,
+                       revision_id = rev.revision_id,
+                       inventory_sha1 = rev.inventory_sha1,
+                       format='5',
+                       )
+        if rev.timezone:
+            root.set('timezone', str(rev.timezone))
+        root.text = '\n'
+        msg = SubElement(root, 'message')
+        msg.text = rev.message
+        msg.tail = '\n'
+        if rev.parent_ids:
+            pelts = SubElement(root, 'parents')
+            pelts.tail = pelts.text = '\n'
+            for parent_id in rev.parent_ids:
+                assert isinstance(parent_id, basestring)
+                p = SubElement(pelts, 'revision_ref')
+                p.tail = '\n'
+                p.set('revision_id', parent_id)
+        return root
+    
+
+    def _unpack_inventory(self, elt):
+        """Construct from XML Element
+        """
+        assert elt.tag == 'inventory'
+        root_id = elt.get('file_id') or ROOT_ID
+        format = elt.get('format')
+        if format is not None:
+            if format != '5':
+                raise BzrError("invalid format version %r on inventory"
+                                % format)
+        inv = Inventory(root_id)
+        for e in elt:
+            ie = self._unpack_entry(e)
+            if ie.parent_id == ROOT_ID:
+                ie.parent_id = root_id
+            inv.add(ie)
+        return inv
+
+
+    def _unpack_entry(self, elt):
+        kind = elt.tag
+        if not InventoryEntry.versionable_kind(kind):
+            raise AssertionError('unsupported entry kind %s' % kind)
+
+        parent_id = elt.get('parent_id')
+        if parent_id == None:
+            parent_id = ROOT_ID
+
+        if kind == 'directory':
+            ie = inventory.InventoryDirectory(elt.get('file_id'),
+                                              elt.get('name'),
+                                              parent_id)
+        elif kind == 'file':
+            ie = inventory.InventoryFile(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.text_sha1 = elt.get('text_sha1')
+            if elt.get('executable') == 'yes':
+                ie.executable = True
+            v = elt.get('text_size')
+            ie.text_size = v and int(v)
+        elif kind == 'symlink':
+            ie = inventory.InventoryLink(elt.get('file_id'),
+                                         elt.get('name'),
+                                         parent_id)
+            ie.symlink_target = elt.get('symlink_target')
+        else:
+            raise BzrError("unknown kind %r" % kind)
+        ie.revision = elt.get('revision')
+
+        return ie
+
+
+    def _unpack_revision(self, elt):
+        """XML Element -> Revision object"""
+        assert elt.tag == 'revision'
+        format = elt.get('format')
+        if format is not None:
+            if format != '5':
+                raise BzrError("invalid format version %r on inventory"
+                                % format)
+        rev = Revision(committer = elt.get('committer'),
+                       timestamp = float(elt.get('timestamp')),
+                       revision_id = elt.get('revision_id'),
+                       inventory_sha1 = elt.get('inventory_sha1')
+                       )
+        parents = elt.find('parents') or []
+        for p in parents:
+            assert p.tag == 'revision_ref', \
+                   "bad parent node tag %r" % p.tag
+            rev.parent_ids.append(p.get('revision_id'))
+
+        v = elt.get('timezone')
+        rev.timezone = v and int(v)
+
+        rev.message = elt.findtext('message') # text of <message>
+        return rev
+
+
+serializer_v5 = Serializer_v5()

=== added directory 'contrib/emacs'
=== added file 'contrib/emacs/bzr-mode.el'
--- contrib/emacs/bzr-mode.el	1970-01-01 00:00:00 +0000
+++ contrib/emacs/bzr-mode.el	2005-08-25 05:58:05 +0000
@@ -0,0 +1,163 @@
+;;; bzr.el -- version control commands for Bazaar-NG.
+;;; Copyright 2005  Luke Gorrie <luke@member.fsf.org>
+;;;
+;;; bzr.el is free software distributed under the terms of the GNU
+;;; General Public Licence, version 2. For details see the file
+;;; COPYING in the GNU Emacs distribution.
+;;;
+;;; This is MAJOR copy & paste job from darcs.el
+
+(eval-when-compile
+  (unless (fboundp 'define-minor-mode)
+    (require 'easy-mmode)
+    (defalias 'define-minor-mode 'easy-mmode-define-minor-mode))
+  (when (featurep 'xemacs)
+    (require 'cl)))
+
+;;;; Configurables
+
+(defvar bzr-command-prefix "\C-cb"
+  ;; This default value breaks the Emacs rules and uses a sequence
+  ;; reserved for the user's own custom bindings. That's not good but
+  ;; I can't think of a decent standard one. -luke (14/Mar/2005)
+  "Prefix sequence for bzr-mode commands.")
+
+(defvar bzr-command "bzr"
+  "*Shell command to execute bzr.")
+
+(defvar bzr-buffer "*bzr-command*"
+  "Buffer for user-visible bzr command output.")
+
+;;;; Minor-mode
+
+(define-minor-mode bzr-mode
+  "\\{bzr-mode-map}"
+  nil
+  " bzr"
+  ;; Coax define-minor-mode into creating a keymap.
+  ;; We'll fill it in manually though because define-minor-mode seems
+  ;; hopeless for changing bindings without restarting Emacs.
+  `((,bzr-command-prefix . fake)))
+
+(defvar bzr-mode-commands-map nil
+  "Keymap for bzr-mode commands.
+This map is bound to a prefix sequence in `bzr-mode-map'.")
+
+(defconst bzr-command-keys '(("l" bzr-log)
+                             ("d" bzr-diff)
+                             ("s" bzr-status)
+                             ("c" bzr-commit))
+  "Keys to bind in `bzr-mode-commands-map'.")
+
+(defun bzr-init-command-keymap ()
+  "Bind the bzr-mode keys.
+This command can be called interactively to redefine the keys from
+`bzr-commands-keys'."
+  (interactive)
+  (setq bzr-mode-commands-map (make-sparse-keymap))
+  (dolist (spec bzr-command-keys)
+    (define-key bzr-mode-commands-map (car spec) (cadr spec)))
+  (define-key bzr-mode-map bzr-command-prefix bzr-mode-commands-map))
+
+(bzr-init-command-keymap)
+
+
+;;;; Commands
+
+(defun bzr-log ()
+  "Run \"bzr log\" in the repository top-level."
+  (interactive)
+  (bzr "log"))
+
+(defun bzr-diff ()
+  "Run \"bzr diff\" in the repository top-level."
+  (interactive)
+  (save-some-buffers)
+  (bzr-run-command (bzr-command "diff") 'diff-mode))
+
+(defun bzr-status ()
+  "Run \"bzr diff\" in the repository top-level."
+  (interactive)
+  (bzr "status"))
+
+(defun bzr-commit (message)
+  "Run \"bzr diff\" in the repository top-level."
+  (interactive "sCommit message: ")
+  (save-some-buffers)
+  (bzr "commit -m %s" (shell-quote-argument message)))
+
+;;;; Utilities
+
+(defun bzr (format &rest args)
+  (bzr-run-command (apply #'bzr-command format args)))
+
+(defun bzr-command (format &rest args)
+  (concat bzr-command " " (apply #'format format args)))
+
+(defun bzr-run-command (command &optional pre-view-hook)
+  "Run COMMAND at the top-level and view the result in another window.
+PRE-VIEW-HOOK is an optional function to call before entering
+view-mode. This is useful to set the major-mode of the result buffer,
+because if you did it afterwards then it would zap view-mode."
+  (bzr-cleanup)
+  (let ((toplevel (bzr-toplevel)))
+    (with-current-buffer (get-buffer-create bzr-buffer)
+      ;; prevent `shell-command' from printing output in a message
+      (let ((max-mini-window-height 0))
+        (let ((default-directory toplevel))
+          (shell-command command t)))
+      (goto-char (point-min))
+      (when pre-view-hook
+        (funcall pre-view-hook))))
+  (if (zerop (buffer-size (get-buffer bzr-buffer)))
+      (message "(bzr command finished with no output.)")
+    (view-buffer-other-window bzr-buffer)
+    ;; Bury the buffer when dismissed.
+    (with-current-buffer (get-buffer bzr-buffer)
+      (setq view-exit-action #'bury-buffer))))
+
+(defun bzr-current-file ()
+  (or (buffer-file-name)
+      (error "Don't know what file to use!")))
+
+(defun bzr-cleanup (&optional buffer-name)
+  "Cleanup before executing a command.
+BUFFER-NAME is the command's output buffer."
+  (let ((name (or buffer-name bzr-buffer)))
+    (when (get-buffer bzr-buffer)
+      (kill-buffer bzr-buffer))))
+
+(defun bzr-toplevel ()
+  "Return the top-level directory of the repository."
+  (let ((dir (bzr-find-repository)))
+    (if dir
+        (file-name-directory dir)
+      (error "Can't find bzr repository top-level."))))
+  
+(defun bzr-find-repository (&optional start-directory)
+  "Return the enclosing \".bzr\" directory, or nil if there isn't one."
+  (when (and (buffer-file-name)
+             (file-directory-p (file-name-directory (buffer-file-name))))
+    (let ((dir (or start-directory
+                   default-directory
+                   (error "No start directory given."))))
+      (or (car (directory-files dir t "^\\.bzr$"))
+          (let ((next-dir (file-name-directory (directory-file-name dir))))
+            (unless (equal dir next-dir)
+              (bzr-find-repository next-dir)))))))
+
+;;;; Hook setup
+;;;
+;;; Automaticaly enter bzr-mode when we open a file that's under bzr
+;;; control, i.e. if the .bzr directory can be found.
+
+(defun bzr-find-file-hook ()
+  "Enable bzr-mode if the file is inside a bzr repository."
+  ;; Note: This function is called for every file that Emacs opens so
+  ;; it mustn't make any mistakes.
+  (when (bzr-find-repository) (bzr-mode 1)))
+
+(add-hook 'find-file-hooks 'bzr-find-file-hook)
+
+(provide 'bzr)
+

=== renamed file 'bzrlib/newinventory.py' => 'contrib/newinventory.py'
--- bzrlib/newinventory.py	2005-07-11 07:33:45 +0000
+++ contrib/newinventory.py	2005-09-27 07:24:40 +0000
@@ -14,13 +14,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
-from xml import ElementTree, Element
+from bzrlib.xml import ElementTree, Element
 
 
 def write_inventory(inv, f):
     el = Element('inventory', {'version': '2'})
     el.text = '\n'
-    
+
     root = Element('root_directory', {'id': inv.root.file_id})
     root.tail = root.text = '\n'
     el.append(root)

=== modified file 'doc/index.txt'
--- doc/index.txt	2005-05-17 00:33:59 +0000
+++ doc/index.txt	2005-09-01 02:39:02 +0000
@@ -14,10 +14,9 @@
 __ http://canonical.com/
 
 
-**Note:** These documents are in a very preliminary state, and so may
-be internally or externally inconsistent or redundant.  Comments are
-still very welcome.  Please send them to <mbp@sourcefrog.net>.
-
+**Note: These documents describe the initial design and are out of
+date in parts.**  For more up-to-date information see the wiki at
+http://bazaar.canonical.com/
 
 For more information, see the homepage at http://bazaar-ng.org/
 

=== added file 'doc/split-join-files.txt'
--- doc/split-join-files.txt	1970-01-01 00:00:00 +0000
+++ doc/split-join-files.txt	2005-08-25 05:58:05 +0000
@@ -0,0 +1,59 @@
+Split and join files
+====================
+
+Version control systems tend to have trouble when files are split or
+joined.  Here is one possibility for handling that.
+
+
+Splitting files
+---------------
+
+This can work like 'svn cp': copy with history::
+
+  % bzr cp a b
+
+This creates a new file b, whose file-history is the same as a, but
+with a different file-id, and present at a different name.
+
+This creates the slightly strange situation that b claims to have text
+versions for revisions where b was not actually present.
+
+
+Joining files
+-------------
+
+We might want to also join files in various cases:
+
+ * Multiple imports of the same file
+
+ * Two source files are being refactored into a single one
+
+ * Two people apply an external patch that creates the same file
+
+I'm not aware of any other system that handles this.
+
+  % bzr join a b
+
+a and b must be versioned files.  b is removed from the working tree
+and inventory.  b's file id is added as a secondary id as a, so that
+later merges between them will detect them as being the same.  
+
+b's weave is (???) incorporated into a.  
+
+This means that we have a file which has multiple history paths.  For
+example, asking to compare that file to its state in a previous
+version has to arbitrarily pick one to use.  That's not necessarily
+fatal if we assign one as primary, but it might be a bit confusing.
+Trying to merge from another tree might can't really choose a good
+basis.
+
+
+
+File aliases
+------------
+
+Rather than trying to join files we might just add notes that files
+are also known by a different file-id in other trees.
+
+
+

=== modified file 'doc/todo-from-arch.txt'
--- doc/todo-from-arch.txt	2005-04-15 01:31:21 +0000
+++ doc/todo-from-arch.txt	2005-09-01 02:34:38 +0000
@@ -2,10 +2,11 @@
 Opportunities for improvement on GNU Arch
 ***************************************** 
 
+[note that this document is rather out of date in 2005-08]
 
-Bazaar-NG is based on the GNU Arch system, and inherits a lot of its
-design from Arch.  However, there are several things we will change in
-Baz to (we hope) improve the user experience.
+GNU Arch is one influence on bazaar-ng.  There are several things we
+would change from Arch in Bazaar to (we hope) improve the user
+experience.
 
 The core design of Arch is good, brilliant even.  It can scale from
 small projects too large ones, and is a good foundation for building

=== added file 'notes/inventory-v2-sample.xml'
--- notes/inventory-v2-sample.xml	1970-01-01 00:00:00 +0000
+++ notes/inventory-v2-sample.xml	2005-08-19 21:34:32 +0000
@@ -0,0 +1,10 @@
+<inventory>
+  <root>
+    <file name="readme.txt" id="readme-123" size="400" sha1="abcdef" 
+          revision="2050808"/>
+    <directory name="src" id="src-123"> 
+      <directory name="libwibble" id="libwibble-123">
+      </directory>
+    </directory>
+  </root>
+</inventory>
\ No newline at end of file

=== added file 'notes/inventory-v2.rnc'
--- notes/inventory-v2.rnc	1970-01-01 00:00:00 +0000
+++ notes/inventory-v2.rnc	2005-08-19 21:34:32 +0000
@@ -0,0 +1,30 @@
+grammar {
+  file = element file {
+    attribute id { text },
+    attribute name { text }, 
+
+    # the following attributes are absent on the working-inventory
+    attribute sha1 { text }?,
+    attribute size { text }?,
+
+    attribute revision { text }?,
+
+    empty
+  }
+
+  directory = element directory {
+    attribute id { text },
+    attribute name { text },
+    
+    entry*
+  }
+
+  entry = 
+    (file | directory )
+
+  start = element inventory {
+    element root {
+      entry*
+    }
+  }
+}

=== added file 'notes/revfile.txt'
--- notes/revfile.txt	1970-01-01 00:00:00 +0000
+++ notes/revfile.txt	2005-08-25 05:58:05 +0000
@@ -0,0 +1,45 @@
+importing all inventories:
+
+with 50% chance of choosing earlier parent:
+
+-rw-r--r--  1 mbp mbp 296961 2005-04-09 22:37 inventory.drev
+-rw-r--r--  1 mbp mbp  11184 2005-04-09 22:37 inventory.irev
+
+imported in 2.490s
+
+
+with no chance:
+
+clearly slows down, 11.163s to import
+
+-rw-r--r--  1 mbp mbp 63590 2005-04-09 22:39 inventory.drev
+-rw-r--r--  1 mbp mbp 11184 2005-04-09 22:39 inventory.irev
+
+
+
+with 25% chance:
+
+revfile.py add-series inventory 0 mbp*  7.86s user 0.12s system 96% cpu 8.247 total
+-rw-r--r--  1 mbp mbp 79535 2005-04-09 22:42 inventory.drev
+-rw-r--r--  1 mbp mbp 11184 2005-04-09 22:42 inventory.irev
+
+
+
+with full text inserted every 20 revisions:
+
+revfile.py add-series inventory 0 mbp*  1.98s user 0.06s system 95% cpu 2.139 total
+
+-rw-r--r--  1 mbp mbp 133719 2005-04-09 22:44 inventory.drev
+-rw-r--r--  1 mbp mbp  11184 2005-04-09 22:44 inventory.irev
+
+So pretty quick, simple, and somewhat easily justified.
+
+
+
+every 50 revisions:
+
+revfile.py add-series inventory 0 mbp*  3.35s user 0.08s system 93% cpu 3.667 total
+
+-rw-r--r--  1 mbp mbp 91895 2005-04-09 22:46 inventory.drev
+-rw-r--r--  1 mbp mbp 11184 2005-04-09 22:46 inventory.irev
+

=== added file 'notes/schemas.xml'
--- notes/schemas.xml	1970-01-01 00:00:00 +0000
+++ notes/schemas.xml	2005-08-19 21:34:32 +0000
@@ -0,0 +1,4 @@
+<?xml version="1.0"?>
+<locatingRules xmlns="http://thaiopensource.com/ns/locating-rules/1.0">
+  <uri resource="inventory-v2-sample.xml" uri="inventory-v2.rnc"/>
+</locatingRules>

=== removed file 'patches/annotate3.patch'
--- patches/annotate3.patch	2005-06-10 06:29:35 +0000
+++ patches/annotate3.patch	1970-01-01 00:00:00 +0000
@@ -1,7263 +0,0 @@
-*** added file 'bzrlib/patches.py'
---- /dev/null 
-+++ bzrlib/patches.py 
-@@ -0,0 +1,497 @@
-+# Copyright (C) 2004, 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+import sys
-+import progress
-+class PatchSyntax(Exception):
-+    def __init__(self, msg):
-+        Exception.__init__(self, msg)
-+
-+
-+class MalformedPatchHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed patch header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedHunkHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed hunk header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedLine(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+def get_patch_names(iter_lines):
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("--- "):
-+            raise MalformedPatchHeader("No orig name", line)
-+        else:
-+            orig_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No orig line", "")
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("+++ "):
-+            raise PatchSyntax("No mod name")
-+        else:
-+            mod_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No mod line", "")
-+    return (orig_name, mod_name)
-+
-+def parse_range(textrange):
-+    """Parse a patch range, handling the "1" special-case
-+
-+    :param textrange: The text to parse
-+    :type textrange: str
-+    :return: the position and range, as a tuple
-+    :rtype: (int, int)
-+    """
-+    tmp = textrange.split(',')
-+    if len(tmp) == 1:
-+        pos = tmp[0]
-+        range = "1"
-+    else:
-+        (pos, range) = tmp
-+    pos = int(pos)
-+    range = int(range)
-+    return (pos, range)
-+
-+ 
-+def hunk_from_header(line):
-+    if not line.startswith("@@") or not line.endswith("@@\n") \
-+        or not len(line) > 4:
-+        raise MalformedHunkHeader("Does not start and end with @@.", line)
-+    try:
-+        (orig, mod) = line[3:-4].split(" ")
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if not orig.startswith('-') or not mod.startswith('+'):
-+        raise MalformedHunkHeader("Positions don't start with + or -.", line)
-+    try:
-+        (orig_pos, orig_range) = parse_range(orig[1:])
-+        (mod_pos, mod_range) = parse_range(mod[1:])
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if mod_range < 0 or orig_range < 0:
-+        raise MalformedHunkHeader("Hunk range is negative", line)
-+    return Hunk(orig_pos, orig_range, mod_pos, mod_range)
-+
-+
-+class HunkLine:
-+    def __init__(self, contents):
-+        self.contents = contents
-+
-+    def get_str(self, leadchar):
-+        if self.contents == "\n" and leadchar == " " and False:
-+            return "\n"
-+        return leadchar + self.contents
-+
-+class ContextLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str(" ")
-+
-+
-+class InsertLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("+")
-+
-+
-+class RemoveLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("-")
-+
-+__pychecker__="no-returnvalues"
-+def parse_line(line):
-+    if line.startswith("\n"):
-+        return ContextLine(line)
-+    elif line.startswith(" "):
-+        return ContextLine(line[1:])
-+    elif line.startswith("+"):
-+        return InsertLine(line[1:])
-+    elif line.startswith("-"):
-+        return RemoveLine(line[1:])
-+    else:
-+        raise MalformedLine("Unknown line type", line)
-+__pychecker__=""
-+
-+
-+class Hunk:
-+    def __init__(self, orig_pos, orig_range, mod_pos, mod_range):
-+        self.orig_pos = orig_pos
-+        self.orig_range = orig_range
-+        self.mod_pos = mod_pos
-+        self.mod_range = mod_range
-+        self.lines = []
-+
-+    def get_header(self):
-+        return "@@ -%s +%s @@\n" % (self.range_str(self.orig_pos, 
-+                                                   self.orig_range),
-+                                    self.range_str(self.mod_pos, 
-+                                                   self.mod_range))
-+
-+    def range_str(self, pos, range):
-+        """Return a file range, special-casing for 1-line files.
-+
-+        :param pos: The position in the file
-+        :type pos: int
-+        :range: The range in the file
-+        :type range: int
-+        :return: a string in the format 1,4 except when range == pos == 1
-+        """
-+        if range == 1:
-+            return "%i" % pos
-+        else:
-+            return "%i,%i" % (pos, range)
-+
-+    def __str__(self):
-+        lines = [self.get_header()]
-+        for line in self.lines:
-+            lines.append(str(line))
-+        return "".join(lines)
-+
-+    def shift_to_mod(self, pos):
-+        if pos < self.orig_pos-1:
-+            return 0
-+        elif pos > self.orig_pos+self.orig_range:
-+            return self.mod_range - self.orig_range
-+        else:
-+            return self.shift_to_mod_lines(pos)
-+
-+    def shift_to_mod_lines(self, pos):
-+        assert (pos >= self.orig_pos-1 and pos <= self.orig_pos+self.orig_range)
-+        position = self.orig_pos-1
-+        shift = 0
-+        for line in self.lines:
-+            if isinstance(line, InsertLine):
-+                shift += 1
-+            elif isinstance(line, RemoveLine):
-+                if position == pos:
-+                    return None
-+                shift -= 1
-+                position += 1
-+            elif isinstance(line, ContextLine):
-+                position += 1
-+            if position > pos:
-+                break
-+        return shift
-+
-+def iter_hunks(iter_lines):
-+    hunk = None
-+    for line in iter_lines:
-+        if line.startswith("@@"):
-+            if hunk is not None:
-+                yield hunk
-+            hunk = hunk_from_header(line)
-+        else:
-+            hunk.lines.append(parse_line(line))
-+
-+    if hunk is not None:
-+        yield hunk
-+
-+class Patch:
-+    def __init__(self, oldname, newname):
-+        self.oldname = oldname
-+        self.newname = newname
-+        self.hunks = []
-+
-+    def __str__(self):
-+        ret =  "--- %s\n+++ %s\n" % (self.oldname, self.newname) 
-+        ret += "".join([str(h) for h in self.hunks])
-+        return ret
-+
-+    def stats_str(self):
-+        """Return a string of patch statistics"""
-+        removes = 0
-+        inserts = 0
-+        for hunk in self.hunks:
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                     inserts+=1;
-+                elif isinstance(line, RemoveLine):
-+                     removes+=1;
-+        return "%i inserts, %i removes in %i hunks" % \
-+            (inserts, removes, len(self.hunks))
-+
-+    def pos_in_mod(self, position):
-+        newpos = position
-+        for hunk in self.hunks:
-+            shift = hunk.shift_to_mod(position)
-+            if shift is None:
-+                return None
-+            newpos += shift
-+        return newpos
-+            
-+    def iter_inserted(self):
-+        """Iteraties through inserted lines
-+        
-+        :return: Pair of line number, line
-+        :rtype: iterator of (int, InsertLine)
-+        """
-+        for hunk in self.hunks:
-+            pos = hunk.mod_pos - 1;
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                    yield (pos, line)
-+                    pos += 1
-+                if isinstance(line, ContextLine):
-+                    pos += 1
-+
-+def parse_patch(iter_lines):
-+    (orig_name, mod_name) = get_patch_names(iter_lines)
-+    patch = Patch(orig_name, mod_name)
-+    for hunk in iter_hunks(iter_lines):
-+        patch.hunks.append(hunk)
-+    return patch
-+
-+
-+class AnnotateLine:
-+    """A line associated with the log that produced it"""
-+    def __init__(self, text, log=None):
-+        self.text = text
-+        self.log = log
-+
-+class CantGetRevisionData(Exception):
-+    def __init__(self, revision):
-+        Exception.__init__(self, "Can't get data for revision %s" % revision)
-+        
-+def annotate_file2(file_lines, anno_iter):
-+    for result in iter_annotate_file(file_lines, anno_iter):
-+        pass
-+    return result
-+
-+        
-+def iter_annotate_file(file_lines, anno_iter):
-+    lines = [AnnotateLine(f) for f in file_lines]
-+    patches = []
-+    try:
-+        for result in anno_iter:
-+            if isinstance(result, progress.Progress):
-+                yield result
-+                continue
-+            log, iter_inserted, patch = result
-+            for (num, line) in iter_inserted:
-+                old_num = num
-+
-+                for cur_patch in patches:
-+                    num = cur_patch.pos_in_mod(num)
-+                    if num == None: 
-+                        break
-+
-+                if num >= len(lines):
-+                    continue
-+                if num is not None and lines[num].log is None:
-+                    lines[num].log = log
-+            patches=[patch]+patches
-+    except CantGetRevisionData:
-+        pass
-+    yield lines
-+
-+
-+def difference_index(atext, btext):
-+    """Find the indext of the first character that differs betweeen two texts
-+
-+    :param atext: The first text
-+    :type atext: str
-+    :param btext: The second text
-+    :type str: str
-+    :return: The index, or None if there are no differences within the range
-+    :rtype: int or NoneType
-+    """
-+    length = len(atext)
-+    if len(btext) < length:
-+        length = len(btext)
-+    for i in range(length):
-+        if atext[i] != btext[i]:
-+            return i;
-+    return None
-+
-+
-+def test():
-+    import unittest
-+    class PatchesTester(unittest.TestCase):
-+        def testValidPatchHeader(self):
-+            """Parse a valid patch header"""
-+            lines = "--- orig/commands.py\n+++ mod/dommands.py\n".split('\n')
-+            (orig, mod) = get_patch_names(lines.__iter__())
-+            assert(orig == "orig/commands.py")
-+            assert(mod == "mod/dommands.py")
-+
-+        def testInvalidPatchHeader(self):
-+            """Parse an invalid patch header"""
-+            lines = "-- orig/commands.py\n+++ mod/dommands.py".split('\n')
-+            self.assertRaises(MalformedPatchHeader, get_patch_names,
-+                              lines.__iter__())
-+
-+        def testValidHunkHeader(self):
-+            """Parse a valid hunk header"""
-+            header = "@@ -34,11 +50,6 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 34)
-+            assert (hunk.orig_range == 11)
-+            assert (hunk.mod_pos == 50)
-+            assert (hunk.mod_range == 6)
-+            assert (str(hunk) == header)
-+
-+        def testValidHunkHeader2(self):
-+            """Parse a tricky, valid hunk header"""
-+            header = "@@ -1 +0,0 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 1)
-+            assert (hunk.orig_range == 1)
-+            assert (hunk.mod_pos == 0)
-+            assert (hunk.mod_range == 0)
-+            assert (str(hunk) == header)
-+
-+        def makeMalformed(self, header):
-+            self.assertRaises(MalformedHunkHeader, hunk_from_header, header)
-+
-+        def testInvalidHeader(self):
-+            """Parse an invalid hunk header"""
-+            self.makeMalformed(" -34,11 +50,6 \n")
-+            self.makeMalformed("@@ +50,6 -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6 @@")
-+            self.makeMalformed("@@ -34.5,11 +50,6 @@\n")
-+            self.makeMalformed("@@-34,11 +50,6@@\n")
-+            self.makeMalformed("@@ 34,11 50,6 @@\n")
-+            self.makeMalformed("@@ -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6.5 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,-6 @@\n")
-+
-+        def lineThing(self,text, type):
-+            line = parse_line(text)
-+            assert(isinstance(line, type))
-+            assert(str(line)==text)
-+
-+        def makeMalformedLine(self, text):
-+            self.assertRaises(MalformedLine, parse_line, text)
-+
-+        def testValidLine(self):
-+            """Parse a valid hunk line"""
-+            self.lineThing(" hello\n", ContextLine)
-+            self.lineThing("+hello\n", InsertLine)
-+            self.lineThing("-hello\n", RemoveLine)
-+        
-+        def testMalformedLine(self):
-+            """Parse invalid valid hunk lines"""
-+            self.makeMalformedLine("hello\n")
-+        
-+        def compare_parsed(self, patchtext):
-+            lines = patchtext.splitlines(True)
-+            patch = parse_patch(lines.__iter__())
-+            pstr = str(patch)
-+            i = difference_index(patchtext, pstr)
-+            if i is not None:
-+                print "%i: \"%s\" != \"%s\"" % (i, patchtext[i], pstr[i])
-+            assert (patchtext == str(patch))
-+
-+        def testAll(self):
-+            """Test parsing a whole patch"""
-+            patchtext = """--- orig/commands.py
-++++ mod/commands.py
-+@@ -1337,7 +1337,8 @@
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree,
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1983,7 +1984,11 @@
-+                                          version)
-+         if len(new_merges) > 0:
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree, comp_version)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+"""
-+            self.compare_parsed(patchtext)
-+
-+        def testInit(self):
-+            """Handle patches missing half the position, range tuple"""
-+            patchtext = \
-+"""--- orig/__init__.py
-++++ mod/__init__.py
-+@@ -1 +1,2 @@
-+ __docformat__ = "restructuredtext en"
-++__doc__ = An alternate Arch commandline interface"""
-+            self.compare_parsed(patchtext)
-+            
-+
-+
-+        def testLineLookup(self):
-+            """Make sure we can accurately look up mod line from orig"""
-+            patch = parse_patch(open("testdata/diff"))
-+            orig = list(open("testdata/orig"))
-+            mod = list(open("testdata/mod"))
-+            removals = []
-+            for i in range(len(orig)):
-+                mod_pos = patch.pos_in_mod(i)
-+                if mod_pos is None:
-+                    removals.append(orig[i])
-+                    continue
-+                assert(mod[mod_pos]==orig[i])
-+            rem_iter = removals.__iter__()
-+            for hunk in patch.hunks:
-+                for line in hunk.lines:
-+                    if isinstance(line, RemoveLine):
-+                        next = rem_iter.next()
-+                        if line.contents != next:
-+                            sys.stdout.write(" orig:%spatch:%s" % (next,
-+                                             line.contents))
-+                        assert(line.contents == next)
-+            self.assertRaises(StopIteration, rem_iter.next)
-+
-+        def testFirstLineRenumber(self):
-+            """Make sure we handle lines at the beginning of the hunk"""
-+            patch = parse_patch(open("testdata/insert_top.patch"))
-+            assert (patch.pos_in_mod(0)==1)
-+    
-+            
-+    patchesTestSuite = unittest.makeSuite(PatchesTester,'test')
-+    runner = unittest.TextTestRunner(verbosity=0)
-+    return runner.run(patchesTestSuite)
-+    
-+
-+if __name__ == "__main__":
-+    test()
-+# arch-tag: d1541a25-eac5-4de9-a476-08a7cecd5683
-
-*** added directory 'testdata'
-*** added file 'testdata/diff'
---- /dev/null 
-+++ testdata/diff 
-@@ -0,0 +1,1154 @@
-+--- orig/commands.py
-++++ mod/commands.py
-+@@ -19,25 +19,31 @@
-+ import arch
-+ import arch.util
-+ import arch.arch
-++
-++import pylon.errors
-++from pylon.errors import *
-++from pylon import errors
-++from pylon import util
-++from pylon import arch_core
-++from pylon import arch_compound
-++from pylon import ancillary
-++from pylon import misc
-++from pylon import paths 
-++
-+ import abacmds
-+ import cmdutil
-+ import shutil
-+ import os
-+ import options
-+-import paths 
-+ import time
-+ import cmd
-+ import readline
-+ import re
-+ import string
-+-import arch_core
-+-from errors import *
-+-import errors
-+ import terminal
-+-import ancillary
-+-import misc
-+ import email
-+ import smtplib
-++import textwrap
-+ 
-+ __docformat__ = "restructuredtext"
-+ __doc__ = "Implementation of user (sub) commands"
-+@@ -257,7 +263,7 @@
-+ 
-+         tree=arch.tree_root()
-+         if len(args) == 0:
-+-            a_spec = cmdutil.comp_revision(tree)
-++            a_spec = ancillary.comp_revision(tree)
-+         else:
-+             a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+         cmdutil.ensure_archive_registered(a_spec.archive)
-+@@ -284,7 +290,7 @@
-+             changeset=options.changeset
-+             tmpdir = None
-+         else:
-+-            tmpdir=cmdutil.tmpdir()
-++            tmpdir=util.tmpdir()
-+             changeset=tmpdir+"/changeset"
-+         try:
-+             delta=arch.iter_delta(a_spec, b_spec, changeset)
-+@@ -304,14 +310,14 @@
-+             if status > 1:
-+                 return
-+             if (options.perform_diff):
-+-                chan = cmdutil.ChangesetMunger(changeset)
-++                chan = arch_compound.ChangesetMunger(changeset)
-+                 chan.read_indices()
-+-                if isinstance(b_spec, arch.Revision):
-+-                    b_dir = b_spec.library_find()
-+-                else:
-+-                    b_dir = b_spec
-+-                a_dir = a_spec.library_find()
-+                 if options.diffopts is not None:
-++                    if isinstance(b_spec, arch.Revision):
-++                        b_dir = b_spec.library_find()
-++                    else:
-++                        b_dir = b_spec
-++                    a_dir = a_spec.library_find()
-+                     diffopts = options.diffopts.split()
-+                     cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                 else:
-+@@ -517,7 +523,7 @@
-+         except arch.errors.TreeRootError, e:
-+             print e
-+             return
-+-        from_revision=cmdutil.tree_latest(tree)
-++        from_revision = arch_compound.tree_latest(tree)
-+         if from_revision==to_revision:
-+             print "Tree is already up to date with:\n"+str(to_revision)+"."
-+             return
-+@@ -592,6 +598,9 @@
-+ 
-+         if len(args) == 0:
-+             args = None
-++        if options.version is None:
-++            return options, tree.tree_version, args
-++
-+         revision=cmdutil.determine_revision_arch(tree, options.version)
-+         return options, revision.get_version(), args
-+ 
-+@@ -601,11 +610,16 @@
-+         """
-+         tree=arch.tree_root()
-+         options, version, files = self.parse_commandline(cmdargs, tree)
-++        ancestor = None
-+         if options.__dict__.has_key("base") and options.base:
-+             base = cmdutil.determine_revision_tree(tree, options.base)
-++            ancestor = base
-+         else:
-+-            base = cmdutil.submit_revision(tree)
-+-        
-++            base = ancillary.submit_revision(tree)
-++            ancestor = base
-++        if ancestor is None:
-++            ancestor = arch_compound.tree_latest(tree, version)
-++
-+         writeversion=version
-+         archive=version.archive
-+         source=cmdutil.get_mirror_source(archive)
-+@@ -625,18 +639,26 @@
-+         try:
-+             last_revision=tree.iter_logs(version, True).next().revision
-+         except StopIteration, e:
-+-            if cmdutil.prompt("Import from commit"):
-+-                return do_import(version)
-+-            else:
-+-                raise NoVersionLogs(version)
-+-        if last_revision!=version.iter_revisions(True).next():
-++            last_revision = None
-++            if ancestor is None:
-++                if cmdutil.prompt("Import from commit"):
-++                    return do_import(version)
-++                else:
-++                    raise NoVersionLogs(version)
-++        try:
-++            arch_last_revision = version.iter_revisions(True).next()
-++        except StopIteration, e:
-++            arch_last_revision = None
-++ 
-++        if last_revision != arch_last_revision:
-++            print "Tree is not up to date with %s" % str(version)
-+             if not cmdutil.prompt("Out of date"):
-+                 raise OutOfDate
-+             else:
-+                 allow_old=True
-+ 
-+         try:
-+-            if not cmdutil.has_changed(version):
-++            if not cmdutil.has_changed(ancestor):
-+                 if not cmdutil.prompt("Empty commit"):
-+                     raise EmptyCommit
-+         except arch.util.ExecProblem, e:
-+@@ -645,15 +667,15 @@
-+                 raise MissingID(e)
-+             else:
-+                 raise
-+-        log = tree.log_message(create=False)
-++        log = tree.log_message(create=False, version=version)
-+         if log is None:
-+             try:
-+                 if cmdutil.prompt("Create log"):
-+-                    edit_log(tree)
-++                    edit_log(tree, version)
-+ 
-+             except cmdutil.NoEditorSpecified, e:
-+                 raise CommandFailed(e)
-+-            log = tree.log_message(create=False)
-++            log = tree.log_message(create=False, version=version)
-+         if log is None: 
-+             raise NoLogMessage
-+         if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+@@ -837,23 +859,24 @@
-+             if spec is not None:
-+                 revision = cmdutil.determine_revision_tree(tree, spec)
-+             else:
-+-                revision = cmdutil.comp_revision(tree)
-++                revision = ancillary.comp_revision(tree)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         munger = None
-+ 
-+         if options.file_contents or options.file_perms or options.deletions\
-+             or options.additions or options.renames or options.hunk_prompt:
-+-            munger = cmdutil.MungeOpts()
-+-            munger.hunk_prompt = options.hunk_prompt
-++            munger = arch_compound.MungeOpts()
-++            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-++                                   options.hunk_prompt)
-+ 
-+         if len(args) > 0 or options.logs or options.pattern_files or \
-+             options.control:
-+             if munger is None:
-+-                munger = cmdutil.MungeOpts(True)
-++                munger = cmdutil.arch_compound.MungeOpts(True)
-+                 munger.all_types(True)
-+         if len(args) > 0:
-+-            t_cwd = cmdutil.tree_cwd(tree)
-++            t_cwd = arch_compound.tree_cwd(tree)
-+             for name in args:
-+                 if len(t_cwd) > 0:
-+                     t_cwd += "/"
-+@@ -878,7 +901,7 @@
-+         if options.pattern_files:
-+             munger.add_keep_pattern(options.pattern_files)
-+                 
-+-        for line in cmdutil.revert(tree, revision, munger, 
-++        for line in arch_compound.revert(tree, revision, munger, 
-+                                    not options.no_output):
-+             cmdutil.colorize(line)
-+ 
-+@@ -1042,18 +1065,13 @@
-+         help_tree_spec()
-+         return
-+ 
-+-def require_version_exists(version, spec):
-+-    if not version.exists():
-+-        raise cmdutil.CantDetermineVersion(spec, 
-+-                                           "The version %s does not exist." \
-+-                                           % version)
-+-
-+ class Revisions(BaseCommand):
-+     """
-+     Print a revision name based on a revision specifier
-+     """
-+     def __init__(self):
-+         self.description="Lists revisions"
-++        self.cl_revisions = []
-+     
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1066,224 +1084,68 @@
-+             self.tree = arch.tree_root()
-+         except arch.errors.TreeRootError:
-+             self.tree = None
-++        if options.type == "default":
-++            options.type = "archive"
-+         try:
-+-            iter = self.get_iterator(options.type, args, options.reverse, 
-+-                                     options.modified)
-++            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-++                                             options.reverse, options.modified,
-++                                             options.shallow)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+-
-++        except cmdutil.CantDetermineVersion, e:
-++            raise CommandFailedWrapper(e)
-+         if options.skip is not None:
-+             iter = cmdutil.iter_skip(iter, int(options.skip))
-+ 
-+-        for revision in iter:
-+-            log = None
-+-            if isinstance(revision, arch.Patchlog):
-+-                log = revision
-+-                revision=revision.revision
-+-            print options.display(revision)
-+-            if log is None and (options.summary or options.creator or 
-+-                                options.date or options.merges):
-+-                log = revision.patchlog
-+-            if options.creator:
-+-                print "    %s" % log.creator
-+-            if options.date:
-+-                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+-            if options.summary:
-+-                print "    %s" % log.summary
-+-            if options.merges:
-+-                showed_title = False
-+-                for revision in log.merged_patches:
-+-                    if not showed_title:
-+-                        print "    Merged:"
-+-                        showed_title = True
-+-                    print "    %s" % revision
-+-
-+-    def get_iterator(self, type, args, reverse, modified):
-+-        if len(args) > 0:
-+-            spec = args[0]
-+-        else:
-+-            spec = None
-+-        if modified is not None:
-+-            iter = cmdutil.modified_iter(modified, self.tree)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-        elif type == "archive":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return version.iter_revisions(reverse)
-+-        elif type == "cacherevs":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return cmdutil.iter_cacherevs(version, reverse)
-+-        elif type == "library":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-            return version.iter_library_revisions(reverse)
-+-        elif type == "logs":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+-                                  self.tree), reverse)
-+-        elif type == "missing" or type == "skip-present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            skip = (type == "skip-present")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_missing(self.tree, version, reverse,
-+-                                        skip_present=skip)
-+-
-+-        elif type == "present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_present(self.tree, version, reverse)
-+-
-+-        elif type == "new-merges" or type == "direct-merges":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+-            if type == "new-merges":
-+-                return iter
-+-            elif type == "direct-merges":
-+-                return cmdutil.direct_merges(iter)
-+-
-+-        elif type == "missing-from":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            libtree = cmdutil.find_or_make_local_revision(revision)
-+-            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+-                                        reverse)
-+-
-+-        elif type == "partner-missing":
-+-            return cmdutil.iter_partner_missing(self.tree, reverse)
-+-
-+-        elif type == "ancestry":
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-
-+-        elif type == "dependencies" or type == "non-dependencies":
-+-            nondeps = (type == "non-dependencies")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+-            if reverse:
-+-                return iter_depends
-+-            else:
-+-                return cmdutil.iter_reverse(iter_depends)
-+-        elif type == "micro":
-+-            return cmdutil.iter_micro(self.tree)
-+-
-+-    
-++        try:
-++            for revision in iter:
-++                log = None
-++                if isinstance(revision, arch.Patchlog):
-++                    log = revision
-++                    revision=revision.revision
-++                out = options.display(revision)
-++                if out is not None:
-++                    print out
-++                if log is None and (options.summary or options.creator or 
-++                                    options.date or options.merges):
-++                    log = revision.patchlog
-++                if options.creator:
-++                    print "    %s" % log.creator
-++                if options.date:
-++                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-++                if options.summary:
-++                    print "    %s" % log.summary
-++                if options.merges:
-++                    showed_title = False
-++                    for revision in log.merged_patches:
-++                        if not showed_title:
-++                            print "    Merged:"
-++                            showed_title = True
-++                        print "    %s" % revision
-++            if len(self.cl_revisions) > 0:
-++                print pylon.changelog_for_merge(self.cl_revisions)
-++        except pylon.errors.TreeRootNone:
-++            raise CommandFailedWrapper(
-++                Exception("This option can only be used in a project tree."))
-++
-++    def changelog_append(self, revision):
-++        if isinstance(revision, arch.Revision):
-++            revision=arch.Patchlog(revision)
-++        self.cl_revisions.append(revision)
-++   
-+     def get_parser(self):
-+         """
-+         Returns the options parser to use for the "revision" command.
-+ 
-+         :rtype: cmdutil.CmdOptionParser
-+         """
-+-        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-++        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+         select = cmdutil.OptionGroup(parser, "Selection options",
-+                           "Control which revisions are listed.  These options"
-+                           " are mutually exclusive.  If more than one is"
-+                           " specified, the last is used.")
-+-        select.add_option("", "--archive", action="store_const", 
-+-                          const="archive", dest="type", default="archive",
-+-                          help="List all revisions in the archive")
-+-        select.add_option("", "--cacherevs", action="store_const", 
-+-                          const="cacherevs", dest="type",
-+-                          help="List all revisions stored in the archive as "
-+-                          "complete copies")
-+-        select.add_option("", "--logs", action="store_const", 
-+-                          const="logs", dest="type",
-+-                          help="List revisions that have a patchlog in the "
-+-                          "tree")
-+-        select.add_option("", "--missing", action="store_const", 
-+-                          const="missing", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree")
-+-        select.add_option("", "--skip-present", action="store_const", 
-+-                          const="skip-present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlogs at all in the tree")
-+-        select.add_option("", "--present", action="store_const", 
-+-                          const="present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree, but can't be merged")
-+-        select.add_option("", "--missing-from", action="store_const", 
-+-                          const="missing-from", dest="type",
-+-                          help="List revisions from the specified revision "
-+-                          "that have no patchlog for the tree version")
-+-        select.add_option("", "--partner-missing", action="store_const", 
-+-                          const="partner-missing", dest="type",
-+-                          help="List revisions in partner versions that are"
-+-                          " missing")
-+-        select.add_option("", "--new-merges", action="store_const", 
-+-                          const="new-merges", dest="type",
-+-                          help="List revisions that have had patchlogs added"
-+-                          " to the tree since the last commit")
-+-        select.add_option("", "--direct-merges", action="store_const", 
-+-                          const="direct-merges", dest="type",
-+-                          help="List revisions that have been directly added"
-+-                          " to tree since the last commit ")
-+-        select.add_option("", "--library", action="store_const", 
-+-                          const="library", dest="type",
-+-                          help="List revisions in the revision library")
-+-        select.add_option("", "--ancestry", action="store_const", 
-+-                          const="ancestry", dest="type",
-+-                          help="List revisions that are ancestors of the "
-+-                          "current tree version")
-+-
-+-        select.add_option("", "--dependencies", action="store_const", 
-+-                          const="dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "depends on")
-+-
-+-        select.add_option("", "--non-dependencies", action="store_const", 
-+-                          const="non-dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "does not depend on")
-+-
-+-        select.add_option("--micro", action="store_const", 
-+-                          const="micro", dest="type",
-+-                          help="List partner revisions aimed for this "
-+-                          "micro-branch")
-+-
-+-        select.add_option("", "--modified", dest="modified", 
-+-                          help="List tree ancestor revisions that modified a "
-+-                          "given file", metavar="FILE[:LINE]")
-+ 
-++        cmdutil.add_revision_iter_options(select)
-+         parser.add_option("", "--skip", dest="skip", 
-+                           help="Skip revisions.  Positive numbers skip from "
-+                           "beginning, negative skip from end.",
-+@@ -1312,6 +1174,9 @@
-+         format.add_option("--cacherev", action="store_const", 
-+                          const=paths.determine_cacherev_path, dest="display",
-+                          help="Show location of cacherev file")
-++        format.add_option("--changelog", action="store_const", 
-++                         const=self.changelog_append, dest="display",
-++                         help="Show location of cacherev file")
-+         parser.add_option_group(format)
-+         display = cmdutil.OptionGroup(parser, "Display format options",
-+                           "These control the display of data")
-+@@ -1448,6 +1313,7 @@
-+         if os.access(self.history_file, os.R_OK) and \
-+             os.path.isfile(self.history_file):
-+             readline.read_history_file(self.history_file)
-++        self.cwd = os.getcwd()
-+ 
-+     def write_history(self):
-+         readline.write_history_file(self.history_file)
-+@@ -1470,16 +1336,21 @@
-+     def set_prompt(self):
-+         if self.tree is not None:
-+             try:
-+-                version = " "+self.tree.tree_version.nonarch
-++                prompt = pylon.alias_or_version(self.tree.tree_version, 
-++                                                self.tree, 
-++                                                full=False)
-++                if prompt is not None:
-++                    prompt = " " + prompt
-+             except:
-+-                version = ""
-++                prompt = ""
-+         else:
-+-            version = ""
-+-        self.prompt = "Fai%s> " % version
-++            prompt = ""
-++        self.prompt = "Fai%s> " % prompt
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1489,8 +1360,15 @@
-+     def do_cd(self, line):
-+         if line == "":
-+             line = "~"
-++        line = os.path.expanduser(line)
-++        if os.path.isabs(line):
-++            newcwd = line
-++        else:
-++            newcwd = self.cwd+'/'+line
-++        newcwd = os.path.normpath(newcwd)
-+         try:
-+-            os.chdir(os.path.expanduser(line))
-++            os.chdir(newcwd)
-++            self.cwd = newcwd
-+         except Exception, e:
-+             print e
-+         try:
-+@@ -1523,7 +1401,7 @@
-+             except cmdutil.CantDetermineRevision, e:
-+                 print e
-+             except Exception, e:
-+-                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-++                print "Unhandled error:\n%s" % errors.exception_str(e)
-+ 
-+         elif suggestions.has_key(args[0]):
-+             print suggestions[args[0]]
-+@@ -1574,7 +1452,7 @@
-+                 arg = line.split()[-1]
-+             else:
-+                 arg = ""
-+-            iter = iter_munged_completions(iter, arg, text)
-++            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+         except Exception, e:
-+             print e
-+         return list(iter)
-+@@ -1604,10 +1482,11 @@
-+                 else:
-+                     arg = ""
-+                 if arg.startswith("-"):
-+-                    return list(iter_munged_completions(iter, arg, text))
-++                    return list(cmdutil.iter_munged_completions(iter, arg, 
-++                                                                text))
-+                 else:
-+-                    return list(iter_munged_completions(
-+-                        iter_file_completions(arg), arg, text))
-++                    return list(cmdutil.iter_munged_completions(
-++                        cmdutil.iter_file_completions(arg), arg, text))
-+ 
-+ 
-+             elif cmd == "cd":
-+@@ -1615,13 +1494,13 @@
-+                     arg = args.split()[-1]
-+                 else:
-+                     arg = ""
-+-                iter = iter_dir_completions(arg)
-+-                iter = iter_munged_completions(iter, arg, text)
-++                iter = cmdutil.iter_dir_completions(arg)
-++                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                 return list(iter)
-+             elif len(args)>0:
-+                 arg = args.split()[-1]
-+-                return list(iter_munged_completions(iter_file_completions(arg),
-+-                                                    arg, text))
-++                iter = cmdutil.iter_file_completions(arg)
-++                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+             else:
-+                 return self.completenames(text, line, begidx, endidx)
-+         except Exception, e:
-+@@ -1636,44 +1515,8 @@
-+             yield entry
-+ 
-+ 
-+-def iter_file_completions(arg, only_dirs = False):
-+-    """Generate an iterator that iterates through filename completions.
-+-
-+-    :param arg: The filename fragment to match
-+-    :type arg: str
-+-    :param only_dirs: If true, match only directories
-+-    :type only_dirs: bool
-+-    """
-+-    cwd = os.getcwd()
-+-    if cwd != "/":
-+-        extras = [".", ".."]
-+-    else:
-+-        extras = []
-+-    (dir, file) = os.path.split(arg)
-+-    if dir != "":
-+-        listingdir = os.path.expanduser(dir)
-+-    else:
-+-        listingdir = cwd
-+-    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+-        if dir != "":
-+-            userfile = dir+'/'+file
-+-        else:
-+-            userfile = file
-+-        if userfile.startswith(arg):
-+-            if os.path.isdir(listingdir+'/'+file):
-+-                userfile+='/'
-+-                yield userfile
-+-            elif not only_dirs:
-+-                yield userfile
-+-
-+-def iter_munged_completions(iter, arg, text):
-+-    for completion in iter:
-+-        completion = str(completion)
-+-        if completion.startswith(arg):
-+-            yield completion[len(arg)-len(text):]
-+-
-+ def iter_source_file_completions(tree, arg):
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1701,7 +1544,7 @@
-+     :return: An iterator of all matching untagged files
-+     :rtype: iterator of str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1743,8 +1586,8 @@
-+     :param arg: The prefix to match
-+     :type arg: str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-+-    tmpdir = cmdutil.tmpdir()
-++    treepath = arch_compound.tree_cwd(tree)
-++    tmpdir = util.tmpdir()
-+     changeset = tmpdir+"/changeset"
-+     completions = []
-+     revision = cmdutil.determine_revision_tree(tree)
-+@@ -1756,14 +1599,6 @@
-+     shutil.rmtree(tmpdir)
-+     return completions
-+ 
-+-def iter_dir_completions(arg):
-+-    """Generate an iterator that iterates through directory name completions.
-+-
-+-    :param arg: The directory name fragment to match
-+-    :type arg: str
-+-    """
-+-    return iter_file_completions(arg, True)
-+-
-+ class Shell(BaseCommand):
-+     def __init__(self):
-+         self.description = "Runs Fai as a shell"
-+@@ -1795,7 +1630,11 @@
-+         parser=self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+ 
-+-        tree = arch.tree_root()
-++        try:
-++            tree = arch.tree_root()
-++        except arch.errors.TreeRootError, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++            
-+ 
-+         if (len(args) == 0) == (options.untagged == False):
-+             raise cmdutil.GetHelp
-+@@ -1809,13 +1648,22 @@
-+         if options.id_type == "tagline":
-+             if method != "tagline":
-+                 if not cmdutil.prompt("Tagline in other tree"):
-+-                    if method == "explicit":
-+-                        options.id_type == explicit
-++                    if method == "explicit" or method == "implicit":
-++                        options.id_type == method
-+                     else:
-+                         print "add-id not supported for \"%s\" tagging method"\
-+                             % method 
-+                         return
-+         
-++        elif options.id_type == "implicit":
-++            if method != "implicit":
-++                if not cmdutil.prompt("Implicit in other tree"):
-++                    if method == "explicit" or method == "tagline":
-++                        options.id_type == method
-++                    else:
-++                        print "add-id not supported for \"%s\" tagging method"\
-++                            % method 
-++                        return
-+         elif options.id_type == "explicit":
-+             if method != "tagline" and method != explicit:
-+                 if not prompt("Explicit in other tree"):
-+@@ -1824,7 +1672,8 @@
-+                     return
-+         
-+         if options.id_type == "auto":
-+-            if method != "tagline" and method != "explicit":
-++            if method != "tagline" and method != "explicit" \
-++                and method !="implicit":
-+                 print "add-id not supported for \"%s\" tagging method" % method
-+                 return
-+             else:
-+@@ -1852,10 +1701,12 @@
-+             previous_files.extend(files)
-+             if id_type == "explicit":
-+                 cmdutil.add_id(files)
-+-            elif id_type == "tagline":
-++            elif id_type == "tagline" or id_type == "implicit":
-+                 for file in files:
-+                     try:
-+-                        cmdutil.add_tagline_or_explicit_id(file)
-++                        implicit = (id_type == "implicit")
-++                        cmdutil.add_tagline_or_explicit_id(file, False,
-++                                                           implicit)
-+                     except cmdutil.AlreadyTagged:
-+                         print "\"%s\" already has a tagline." % file
-+                     except cmdutil.NoCommentSyntax:
-+@@ -1888,6 +1739,9 @@
-+         parser.add_option("--tagline", action="store_const", 
-+                          const="tagline", dest="id_type", 
-+                          help="Use a tagline id")
-++        parser.add_option("--implicit", action="store_const", 
-++                         const="implicit", dest="id_type", 
-++                         help="Use an implicit id (deprecated)")
-+         parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", default=False, 
-+                          help="tag all untagged files")
-+@@ -1926,27 +1780,7 @@
-+     def get_completer(self, arg, index):
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+-        completions = list(ancillary.iter_partners(self.tree, 
-+-                                                   self.tree.tree_version))
-+-        if len(completions) == 0:
-+-            completions = list(self.tree.iter_log_versions())
-+-
-+-        aliases = []
-+-        try:
-+-            for completion in completions:
-+-                alias = ancillary.compact_alias(str(completion), self.tree)
-+-                if alias:
-+-                    aliases.extend(alias)
-+-
-+-            for completion in completions:
-+-                if completion.archive == self.tree.tree_version.archive:
-+-                    aliases.append(completion.nonarch)
-+-
-+-        except Exception, e:
-+-            print e
-+-            
-+-        completions.extend(aliases)
-+-        return completions
-++        return cmdutil.merge_completions(self.tree, arg, index)
-+ 
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1961,7 +1795,7 @@
-+         
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError(os.getcwd())
-+-        if cmdutil.has_changed(self.tree.tree_version):
-++        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+             raise UncommittedChanges(self.tree)
-+ 
-+         if len(args) > 0:
-+@@ -2027,14 +1861,14 @@
-+         :type other_revision: `arch.Revision`
-+         :return: 0 if the merge was skipped, 1 if it was applied
-+         """
-+-        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-++        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+         try:
-+             if action == "native-merge":
-+-                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+-                                                   other_revision)
-++                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-++                                                         other_revision)
-+             elif action == "update":
-+-                ancestor = cmdutil.tree_latest(self.tree, 
-+-                                               other_revision.version)
-++                ancestor = arch_compound.tree_latest(self.tree, 
-++                                                     other_revision.version)
-+         except CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+@@ -2104,7 +1938,10 @@
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+ 
-+-        edit_log(self.tree)
-++        try:
-++            edit_log(self.tree, self.tree.tree_version)
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-+ 
-+     def get_parser(self):
-+         """
-+@@ -2132,7 +1969,7 @@
-+         """
-+         return
-+ 
-+-def edit_log(tree):
-++def edit_log(tree, version):
-+     """Makes and edits the log for a tree.  Does all kinds of fancy things
-+     like log templates and merge summaries and log-for-merge
-+     
-+@@ -2141,28 +1978,29 @@
-+     """
-+     #ensure we have an editor before preparing the log
-+     cmdutil.find_editor()
-+-    log = tree.log_message(create=False)
-++    log = tree.log_message(create=False, version=version)
-+     log_is_new = False
-+     if log is None or cmdutil.prompt("Overwrite log"):
-+         if log is not None:
-+            os.remove(log.name)
-+-        log = tree.log_message(create=True)
-++        log = tree.log_message(create=True, version=version)
-+         log_is_new = True
-+         tmplog = log.name
-+-        template = tree+"/{arch}/=log-template"
-+-        if not os.path.exists(template):
-+-            template = os.path.expanduser("~/.arch-params/=log-template")
-+-            if not os.path.exists(template):
-+-                template = None
-++        template = pylon.log_template_path(tree)
-+         if template:
-+             shutil.copyfile(template, tmplog)
-+-        
-+-        new_merges = list(cmdutil.iter_new_merges(tree, 
-+-                                                  tree.tree_version))
-+-        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-++        comp_version = ancillary.comp_revision(tree).version
-++        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-++        new_merges = cmdutil.direct_merges(new_merges)
-++        log["Summary"] = pylon.merge_summary(new_merges, 
-++                                         version)
-+         if len(new_merges) > 0:   
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+@@ -2172,29 +2010,6 @@
-+             os.remove(log.name)
-+         raise
-+ 
-+-def merge_summary(new_merges, tree_version):
-+-    if len(new_merges) == 0:
-+-        return ""
-+-    if len(new_merges) == 1:
-+-        summary = new_merges[0].summary
-+-    else:
-+-        summary = "Merge"
-+-
-+-    credits = []
-+-    for merge in new_merges:
-+-        if arch.my_id() != merge.creator:
-+-            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+-            if not name in credits:
-+-                credits.append(name)
-+-        else:
-+-            version = merge.revision.version
-+-            if version.archive == tree_version.archive:
-+-                if not version.nonarch in credits:
-+-                    credits.append(version.nonarch)
-+-            elif not str(version) in credits:
-+-                credits.append(str(version))
-+-
-+-    return ("%s (%s)") % (summary, ", ".join(credits))
-+ 
-+ class MirrorArchive(BaseCommand):
-+     """
-+@@ -2268,31 +2083,73 @@
-+ 
-+ Use "alias" to list available (user and automatic) aliases."""
-+ 
-++auto_alias = [
-++"acur", 
-++"The latest revision in the archive of the tree-version.  You can specify \
-++a different version like so: acur:foo--bar--0 (aliases can be used)",
-++"tcur",
-++"""(tree current) The latest revision in the tree of the tree-version. \
-++You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-++used).""",
-++"tprev" , 
-++"""(tree previous) The previous revision in the tree of the tree-version.  To \
-++specify an older revision, use a number, e.g. "tprev:4" """,
-++"tanc" , 
-++"""(tree ancestor) The ancestor revision of the tree To specify an older \
-++revision, use a number, e.g. "tanc:4".""",
-++"tdate" , 
-++"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-++"tmod" , 
-++""" (tree modified) The latest revision to modify a given file, e.g. \
-++"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-++"ttag" , 
-++"""(tree tag) The revision that was tagged into the current tree revision, \
-++according to the tree""",
-++"tagcur", 
-++"""(tag current) The latest revision of the version that the current tree \
-++was tagged from.""",
-++"mergeanc" , 
-++"""The common ancestor of the current tree and the specified revision. \
-++Defaults to the first partner-version's latest revision or to tagcur.""",
-++]
-++
-++
-++def is_auto_alias(name):
-++    """Determine whether a name is an auto alias name
-++
-++    :param name: the name to check
-++    :type name: str
-++    :return: True if the name is an auto alias, false if not
-++    :rtype: bool
-++    """
-++    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-++
-++
-++def display_def(iter, wrap = 80):
-++    """Display a list of definitions
-++
-++    :param iter: iter of name, definition pairs
-++    :type iter: iter of (str, str)
-++    :param wrap: The width for text wrapping
-++    :type wrap: int
-++    """
-++    vals = list(iter)
-++    maxlen = 0
-++    for (key, value) in vals:
-++        if len(key) > maxlen:
-++            maxlen = len(key)
-++    for (key, value) in vals:
-++        tw=textwrap.TextWrapper(width=wrap, 
-++                                initial_indent=key.rjust(maxlen)+" : ",
-++                                subsequent_indent="".rjust(maxlen+3))
-++        print tw.fill(value)
-++
-++
-+ def help_aliases(tree):
-+-    print """Auto-generated aliases
-+- acur : The latest revision in the archive of the tree-version.  You can specfy
-+-        a different version like so: acur:foo--bar--0 (aliases can be used)
-+- tcur : (tree current) The latest revision in the tree of the tree-version.
-+-        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+-        can be used).
-+-tprev : (tree previous) The previous revision in the tree of the tree-version.
-+-        To specify an older revision, use a number, e.g. "tprev:4"
-+- tanc : (tree ancestor) The ancestor revision of the tree
-+-        To specify an older revision, use a number, e.g. "tanc:4"
-+-tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+- tmod : (tree modified) The latest revision to modify a given file 
-+-        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+- ttag : (tree tag) The revision that was tagged into the current tree revision,
-+-        according to the tree.
-+-tagcur: (tag current) The latest revision of the version that the current tree
-+-        was tagged from.
-+-mergeanc : The common ancestor of the current tree and the specified revision.
-+-        Defaults to the first partner-version's latest revision or to tagcur.
-+-   """
-++    print """Auto-generated aliases"""
-++    display_def(pylon.util.iter_pairs(auto_alias))
-+     print "User aliases"
-+-    for parts in ancillary.iter_all_alias(tree):
-+-        print parts[0].rjust(10)+" : "+parts[1]
-+-
-++    display_def(ancillary.iter_all_alias(tree))
-+ 
-+ class Inventory(BaseCommand):
-+     """List the status of files in the tree"""
-+@@ -2428,6 +2285,11 @@
-+         except cmdutil.ForbiddenAliasSyntax, e:
-+             raise CommandFailedWrapper(e)
-+ 
-++    def no_prefix(self, alias):
-++        if alias.startswith("^"):
-++            alias = alias[1:]
-++        return alias
-++        
-+     def arg_dispatch(self, args, options):
-+         """Add, modify, or list aliases, depending on number of arguments
-+ 
-+@@ -2438,15 +2300,20 @@
-+         if len(args) == 0:
-+             help_aliases(self.tree)
-+             return
-+-        elif len(args) == 1:
-+-            self.print_alias(args[0])
-+-        elif (len(args)) == 2:
-+-            self.add(args[0], args[1], options)
-+         else:
-+-            raise cmdutil.GetHelp
-++            alias = self.no_prefix(args[0])
-++            if len(args) == 1:
-++                self.print_alias(alias)
-++            elif (len(args)) == 2:
-++                self.add(alias, args[1], options)
-++            else:
-++                raise cmdutil.GetHelp
-+ 
-+     def print_alias(self, alias):
-+         answer = None
-++        if is_auto_alias(alias):
-++            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-++                "  Use \"revision\" to expand auto aliases." % alias)
-+         for pair in ancillary.iter_all_alias(self.tree):
-+             if pair[0] == alias:
-+                 answer = pair[1]
-+@@ -2464,6 +2331,8 @@
-+         :type expansion: str
-+         :param options: The commandline options
-+         """
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         written = False
-+         new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+@@ -2490,14 +2359,17 @@
-+         deleted = False
-+         if len(args) != 1:
-+             raise cmdutil.GetHelp
-++        alias = self.no_prefix(args[0])
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         for pair in self.get_iterator(options):
-+-            if pair[0] != args[0]:
-++            if pair[0] != alias:
-+                 newlist+="%s=%s\n" % (pair[0], pair[1])
-+             else:
-+                 deleted = True
-+         if not deleted:
-+-            raise errors.NoSuchAlias(args[0])
-++            raise errors.NoSuchAlias(alias)
-+         self.write_aliases(newlist, options)
-+ 
-+     def get_alias_file(self, options):
-+@@ -2526,7 +2398,7 @@
-+         :param options: The commandline options
-+         """
-+         filename = os.path.expanduser(self.get_alias_file(options))
-+-        file = cmdutil.NewFileVersion(filename)
-++        file = util.NewFileVersion(filename)
-+         file.write(newlist)
-+         file.commit()
-+ 
-+@@ -2588,10 +2460,13 @@
-+         :param cmdargs: The commandline arguments
-+         :type cmdargs: list of str
-+         """
-+-        cmdutil.find_editor()
-+         parser = self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+         try:
-++            cmdutil.find_editor()
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++        try:
-+             self.tree=arch.tree_root()
-+         except:
-+             self.tree=None
-+@@ -2655,7 +2530,7 @@
-+             target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                               args[0])
-+         else:
-+-            target_revision = cmdutil.tree_latest(self.tree)
-++            target_revision = arch_compound.tree_latest(self.tree)
-+         if len(args) > 1:
-+             merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                        self.tree, f)) for f in args[1:] ]
-+@@ -2711,7 +2586,7 @@
-+ 
-+         :param message: The message to send
-+         :type message: `email.Message`"""
-+-        server = smtplib.SMTP()
-++        server = smtplib.SMTP("localhost")
-+         server.sendmail(message['From'], message['To'], message.as_string())
-+         server.quit()
-+ 
-+@@ -2763,6 +2638,22 @@
-+ 'alias' : Alias,
-+ 'request-merge': RequestMerge,
-+ }
-++
-++def my_import(mod_name):
-++    module = __import__(mod_name)
-++    components = mod_name.split('.')
-++    for comp in components[1:]:
-++        module = getattr(module, comp)
-++    return module
-++
-++def plugin(mod_name):
-++    module = my_import(mod_name)
-++    module.add_command(commands)
-++
-++for file in os.listdir(sys.path[0]+"/command"):
-++    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-++        plugin("command."+file[:-3])
-++
-+ suggestions = {
-+ 'apply-delta' : "Try \"apply-changes\".",
-+ 'delta' : "To compare two revisions, use \"changes\".",
-+@@ -2784,6 +2675,7 @@
-+ 'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+ 'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+ 'library-revisions' : "Use revisions --library",
-+-'file-revert' : "Use revert FILE"
-++'file-revert' : "Use revert FILE",
-++'join-branch' : "Use replay --logs-only"
-+ }
-+ # arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/insert_top.patch'
---- /dev/null 
-+++ testdata/insert_top.patch 
-@@ -0,0 +1,7 @@
-+--- orig/pylon/patches.py
-++++ mod/pylon/patches.py
-+@@ -1,3 +1,4 @@
-++#test
-+ import util
-+ import sys
-+ class PatchSyntax(Exception):
-
-*** added file 'testdata/mod'
---- /dev/null 
-+++ testdata/mod 
-@@ -0,0 +1,2681 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+
-+import pylon.errors
-+from pylon.errors import *
-+from pylon import errors
-+from pylon import util
-+from pylon import arch_core
-+from pylon import arch_compound
-+from pylon import ancillary
-+from pylon import misc
-+from pylon import paths 
-+
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import terminal
-+import email
-+import smtplib
-+import textwrap
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = ancillary.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=util.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = arch_compound.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if options.diffopts is not None:
-+                    if isinstance(b_spec, arch.Revision):
-+                        b_dir = b_spec.library_find()
-+                    else:
-+                        b_dir = b_spec
-+                    a_dir = a_spec.library_find()
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision = arch_compound.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        if options.version is None:
-+            return options, tree.tree_version, args
-+
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        ancestor = None
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+            ancestor = base
-+        else:
-+            base = ancillary.submit_revision(tree)
-+            ancestor = base
-+        if ancestor is None:
-+            ancestor = arch_compound.tree_latest(tree, version)
-+
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            last_revision = None
-+            if ancestor is None:
-+                if cmdutil.prompt("Import from commit"):
-+                    return do_import(version)
-+                else:
-+                    raise NoVersionLogs(version)
-+        try:
-+            arch_last_revision = version.iter_revisions(True).next()
-+        except StopIteration, e:
-+            arch_last_revision = None
-+ 
-+        if last_revision != arch_last_revision:
-+            print "Tree is not up to date with %s" % str(version)
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(ancestor):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False, version=version)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree, version)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False, version=version)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = ancillary.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = arch_compound.MungeOpts()
-+            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-+                                   options.hunk_prompt)
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.arch_compound.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = arch_compound.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in arch_compound.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+        self.cl_revisions = []
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        if options.type == "default":
-+            options.type = "archive"
-+        try:
-+            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-+                                             options.reverse, options.modified,
-+                                             options.shallow)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        except cmdutil.CantDetermineVersion, e:
-+            raise CommandFailedWrapper(e)
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        try:
-+            for revision in iter:
-+                log = None
-+                if isinstance(revision, arch.Patchlog):
-+                    log = revision
-+                    revision=revision.revision
-+                out = options.display(revision)
-+                if out is not None:
-+                    print out
-+                if log is None and (options.summary or options.creator or 
-+                                    options.date or options.merges):
-+                    log = revision.patchlog
-+                if options.creator:
-+                    print "    %s" % log.creator
-+                if options.date:
-+                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+                if options.summary:
-+                    print "    %s" % log.summary
-+                if options.merges:
-+                    showed_title = False
-+                    for revision in log.merged_patches:
-+                        if not showed_title:
-+                            print "    Merged:"
-+                            showed_title = True
-+                        print "    %s" % revision
-+            if len(self.cl_revisions) > 0:
-+                print pylon.changelog_for_merge(self.cl_revisions)
-+        except pylon.errors.TreeRootNone:
-+            raise CommandFailedWrapper(
-+                Exception("This option can only be used in a project tree."))
-+
-+    def changelog_append(self, revision):
-+        if isinstance(revision, arch.Revision):
-+            revision=arch.Patchlog(revision)
-+        self.cl_revisions.append(revision)
-+   
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+
-+        cmdutil.add_revision_iter_options(select)
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        format.add_option("--changelog", action="store_const", 
-+                         const=self.changelog_append, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+        self.cwd = os.getcwd()
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                prompt = pylon.alias_or_version(self.tree.tree_version, 
-+                                                self.tree, 
-+                                                full=False)
-+                if prompt is not None:
-+                    prompt = " " + prompt
-+            except:
-+                prompt = ""
-+        else:
-+            prompt = ""
-+        self.prompt = "Fai%s> " % prompt
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-+                                             full=False)
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        line = os.path.expanduser(line)
-+        if os.path.isabs(line):
-+            newcwd = line
-+        else:
-+            newcwd = self.cwd+'/'+line
-+        newcwd = os.path.normpath(newcwd)
-+        try:
-+            os.chdir(newcwd)
-+            self.cwd = newcwd
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % errors.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(cmdutil.iter_munged_completions(iter, arg, 
-+                                                                text))
-+                else:
-+                    return list(cmdutil.iter_munged_completions(
-+                        cmdutil.iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = cmdutil.iter_dir_completions(arg)
-+                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                iter = cmdutil.iter_file_completions(arg)
-+                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    tmpdir = util.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+            
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit" or method == "implicit":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "implicit":
-+            if method != "implicit":
-+                if not cmdutil.prompt("Implicit in other tree"):
-+                    if method == "explicit" or method == "tagline":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit" \
-+                and method !="implicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline" or id_type == "implicit":
-+                for file in files:
-+                    try:
-+                        implicit = (id_type == "implicit")
-+                        cmdutil.add_tagline_or_explicit_id(file, False,
-+                                                           implicit)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--implicit", action="store_const", 
-+                         const="implicit", dest="id_type", 
-+                         help="Use an implicit id (deprecated)")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        return cmdutil.merge_completions(self.tree, arg, index)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-+                                                         other_revision)
-+            elif action == "update":
-+                ancestor = arch_compound.tree_latest(self.tree, 
-+                                                     other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        try:
-+            edit_log(self.tree, self.tree.tree_version)
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree, version):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False, version=version)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True, version=version)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = pylon.log_template_path(tree)
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        comp_version = ancillary.comp_revision(tree).version
-+        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-+        new_merges = cmdutil.direct_merges(new_merges)
-+        log["Summary"] = pylon.merge_summary(new_merges, 
-+                                         version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                if cmdutil.prompt("changelog for merge"):
-+                    mergestuff = "Patches applied:\n"
-+                    mergestuff += pylon.changelog_for_merge(new_merges)
-+                else:
-+                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+auto_alias = [
-+"acur", 
-+"The latest revision in the archive of the tree-version.  You can specify \
-+a different version like so: acur:foo--bar--0 (aliases can be used)",
-+"tcur",
-+"""(tree current) The latest revision in the tree of the tree-version. \
-+You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-+used).""",
-+"tprev" , 
-+"""(tree previous) The previous revision in the tree of the tree-version.  To \
-+specify an older revision, use a number, e.g. "tprev:4" """,
-+"tanc" , 
-+"""(tree ancestor) The ancestor revision of the tree To specify an older \
-+revision, use a number, e.g. "tanc:4".""",
-+"tdate" , 
-+"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-+"tmod" , 
-+""" (tree modified) The latest revision to modify a given file, e.g. \
-+"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-+"ttag" , 
-+"""(tree tag) The revision that was tagged into the current tree revision, \
-+according to the tree""",
-+"tagcur", 
-+"""(tag current) The latest revision of the version that the current tree \
-+was tagged from.""",
-+"mergeanc" , 
-+"""The common ancestor of the current tree and the specified revision. \
-+Defaults to the first partner-version's latest revision or to tagcur.""",
-+]
-+
-+
-+def is_auto_alias(name):
-+    """Determine whether a name is an auto alias name
-+
-+    :param name: the name to check
-+    :type name: str
-+    :return: True if the name is an auto alias, false if not
-+    :rtype: bool
-+    """
-+    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-+
-+
-+def display_def(iter, wrap = 80):
-+    """Display a list of definitions
-+
-+    :param iter: iter of name, definition pairs
-+    :type iter: iter of (str, str)
-+    :param wrap: The width for text wrapping
-+    :type wrap: int
-+    """
-+    vals = list(iter)
-+    maxlen = 0
-+    for (key, value) in vals:
-+        if len(key) > maxlen:
-+            maxlen = len(key)
-+    for (key, value) in vals:
-+        tw=textwrap.TextWrapper(width=wrap, 
-+                                initial_indent=key.rjust(maxlen)+" : ",
-+                                subsequent_indent="".rjust(maxlen+3))
-+        print tw.fill(value)
-+
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases"""
-+    display_def(pylon.util.iter_pairs(auto_alias))
-+    print "User aliases"
-+    display_def(ancillary.iter_all_alias(tree))
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def no_prefix(self, alias):
-+        if alias.startswith("^"):
-+            alias = alias[1:]
-+        return alias
-+        
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        else:
-+            alias = self.no_prefix(args[0])
-+            if len(args) == 1:
-+                self.print_alias(alias)
-+            elif (len(args)) == 2:
-+                self.add(alias, args[1], options)
-+            else:
-+                raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        if is_auto_alias(alias):
-+            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-+                "  Use \"revision\" to expand auto aliases." % alias)
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        alias = self.no_prefix(args[0])
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(alias)
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = util.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            cmdutil.find_editor()
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = arch_compound.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP("localhost")
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+
-+def my_import(mod_name):
-+    module = __import__(mod_name)
-+    components = mod_name.split('.')
-+    for comp in components[1:]:
-+        module = getattr(module, comp)
-+    return module
-+
-+def plugin(mod_name):
-+    module = my_import(mod_name)
-+    module.add_command(commands)
-+
-+for file in os.listdir(sys.path[0]+"/command"):
-+    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-+        plugin("command."+file[:-3])
-+
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE",
-+'join-branch' : "Use replay --logs-only"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/orig'
---- /dev/null 
-+++ testdata/orig 
-@@ -0,0 +1,2789 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import paths 
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import arch_core
-+from errors import *
-+import errors
-+import terminal
-+import ancillary
-+import misc
-+import email
-+import smtplib
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = cmdutil.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=cmdutil.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = cmdutil.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if isinstance(b_spec, arch.Revision):
-+                    b_dir = b_spec.library_find()
-+                else:
-+                    b_dir = b_spec
-+                a_dir = a_spec.library_find()
-+                if options.diffopts is not None:
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision=cmdutil.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+        else:
-+            base = cmdutil.submit_revision(tree)
-+        
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            if cmdutil.prompt("Import from commit"):
-+                return do_import(version)
-+            else:
-+                raise NoVersionLogs(version)
-+        if last_revision!=version.iter_revisions(True).next():
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(version):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = cmdutil.MungeOpts()
-+            munger.hunk_prompt = options.hunk_prompt
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = cmdutil.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in cmdutil.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+def require_version_exists(version, spec):
-+    if not version.exists():
-+        raise cmdutil.CantDetermineVersion(spec, 
-+                                           "The version %s does not exist." \
-+                                           % version)
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        try:
-+            iter = self.get_iterator(options.type, args, options.reverse, 
-+                                     options.modified)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        for revision in iter:
-+            log = None
-+            if isinstance(revision, arch.Patchlog):
-+                log = revision
-+                revision=revision.revision
-+            print options.display(revision)
-+            if log is None and (options.summary or options.creator or 
-+                                options.date or options.merges):
-+                log = revision.patchlog
-+            if options.creator:
-+                print "    %s" % log.creator
-+            if options.date:
-+                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+            if options.summary:
-+                print "    %s" % log.summary
-+            if options.merges:
-+                showed_title = False
-+                for revision in log.merged_patches:
-+                    if not showed_title:
-+                        print "    Merged:"
-+                        showed_title = True
-+                    print "    %s" % revision
-+
-+    def get_iterator(self, type, args, reverse, modified):
-+        if len(args) > 0:
-+            spec = args[0]
-+        else:
-+            spec = None
-+        if modified is not None:
-+            iter = cmdutil.modified_iter(modified, self.tree)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+        elif type == "archive":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return version.iter_revisions(reverse)
-+        elif type == "cacherevs":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return cmdutil.iter_cacherevs(version, reverse)
-+        elif type == "library":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+            return version.iter_library_revisions(reverse)
-+        elif type == "logs":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+                                  self.tree), reverse)
-+        elif type == "missing" or type == "skip-present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            skip = (type == "skip-present")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_missing(self.tree, version, reverse,
-+                                        skip_present=skip)
-+
-+        elif type == "present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_present(self.tree, version, reverse)
-+
-+        elif type == "new-merges" or type == "direct-merges":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+            if type == "new-merges":
-+                return iter
-+            elif type == "direct-merges":
-+                return cmdutil.direct_merges(iter)
-+
-+        elif type == "missing-from":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            libtree = cmdutil.find_or_make_local_revision(revision)
-+            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+                                        reverse)
-+
-+        elif type == "partner-missing":
-+            return cmdutil.iter_partner_missing(self.tree, reverse)
-+
-+        elif type == "ancestry":
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            iter = cmdutil._iter_ancestry(self.tree, revision)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+
-+        elif type == "dependencies" or type == "non-dependencies":
-+            nondeps = (type == "non-dependencies")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+            if reverse:
-+                return iter_depends
-+            else:
-+                return cmdutil.iter_reverse(iter_depends)
-+        elif type == "micro":
-+            return cmdutil.iter_micro(self.tree)
-+
-+    
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+        select.add_option("", "--archive", action="store_const", 
-+                          const="archive", dest="type", default="archive",
-+                          help="List all revisions in the archive")
-+        select.add_option("", "--cacherevs", action="store_const", 
-+                          const="cacherevs", dest="type",
-+                          help="List all revisions stored in the archive as "
-+                          "complete copies")
-+        select.add_option("", "--logs", action="store_const", 
-+                          const="logs", dest="type",
-+                          help="List revisions that have a patchlog in the "
-+                          "tree")
-+        select.add_option("", "--missing", action="store_const", 
-+                          const="missing", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree")
-+        select.add_option("", "--skip-present", action="store_const", 
-+                          const="skip-present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlogs at all in the tree")
-+        select.add_option("", "--present", action="store_const", 
-+                          const="present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree, but can't be merged")
-+        select.add_option("", "--missing-from", action="store_const", 
-+                          const="missing-from", dest="type",
-+                          help="List revisions from the specified revision "
-+                          "that have no patchlog for the tree version")
-+        select.add_option("", "--partner-missing", action="store_const", 
-+                          const="partner-missing", dest="type",
-+                          help="List revisions in partner versions that are"
-+                          " missing")
-+        select.add_option("", "--new-merges", action="store_const", 
-+                          const="new-merges", dest="type",
-+                          help="List revisions that have had patchlogs added"
-+                          " to the tree since the last commit")
-+        select.add_option("", "--direct-merges", action="store_const", 
-+                          const="direct-merges", dest="type",
-+                          help="List revisions that have been directly added"
-+                          " to tree since the last commit ")
-+        select.add_option("", "--library", action="store_const", 
-+                          const="library", dest="type",
-+                          help="List revisions in the revision library")
-+        select.add_option("", "--ancestry", action="store_const", 
-+                          const="ancestry", dest="type",
-+                          help="List revisions that are ancestors of the "
-+                          "current tree version")
-+
-+        select.add_option("", "--dependencies", action="store_const", 
-+                          const="dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "depends on")
-+
-+        select.add_option("", "--non-dependencies", action="store_const", 
-+                          const="non-dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "does not depend on")
-+
-+        select.add_option("--micro", action="store_const", 
-+                          const="micro", dest="type",
-+                          help="List partner revisions aimed for this "
-+                          "micro-branch")
-+
-+        select.add_option("", "--modified", dest="modified", 
-+                          help="List tree ancestor revisions that modified a "
-+                          "given file", metavar="FILE[:LINE]")
-+
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                version = " "+self.tree.tree_version.nonarch
-+            except:
-+                version = ""
-+        else:
-+            version = ""
-+        self.prompt = "Fai%s> " % version
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = self.tree.tree_version.nonarch
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        try:
-+            os.chdir(os.path.expanduser(line))
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(iter_munged_completions(iter, arg, text))
-+                else:
-+                    return list(iter_munged_completions(
-+                        iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = iter_dir_completions(arg)
-+                iter = iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                return list(iter_munged_completions(iter_file_completions(arg),
-+                                                    arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_file_completions(arg, only_dirs = False):
-+    """Generate an iterator that iterates through filename completions.
-+
-+    :param arg: The filename fragment to match
-+    :type arg: str
-+    :param only_dirs: If true, match only directories
-+    :type only_dirs: bool
-+    """
-+    cwd = os.getcwd()
-+    if cwd != "/":
-+        extras = [".", ".."]
-+    else:
-+        extras = []
-+    (dir, file) = os.path.split(arg)
-+    if dir != "":
-+        listingdir = os.path.expanduser(dir)
-+    else:
-+        listingdir = cwd
-+    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+        if dir != "":
-+            userfile = dir+'/'+file
-+        else:
-+            userfile = file
-+        if userfile.startswith(arg):
-+            if os.path.isdir(listingdir+'/'+file):
-+                userfile+='/'
-+                yield userfile
-+            elif not only_dirs:
-+                yield userfile
-+
-+def iter_munged_completions(iter, arg, text):
-+    for completion in iter:
-+        completion = str(completion)
-+        if completion.startswith(arg):
-+            yield completion[len(arg)-len(text):]
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    tmpdir = cmdutil.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+def iter_dir_completions(arg):
-+    """Generate an iterator that iterates through directory name completions.
-+
-+    :param arg: The directory name fragment to match
-+    :type arg: str
-+    """
-+    return iter_file_completions(arg, True)
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        tree = arch.tree_root()
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit":
-+                        options.id_type == explicit
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline":
-+                for file in files:
-+                    try:
-+                        cmdutil.add_tagline_or_explicit_id(file)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        completions = list(ancillary.iter_partners(self.tree, 
-+                                                   self.tree.tree_version))
-+        if len(completions) == 0:
-+            completions = list(self.tree.iter_log_versions())
-+
-+        aliases = []
-+        try:
-+            for completion in completions:
-+                alias = ancillary.compact_alias(str(completion), self.tree)
-+                if alias:
-+                    aliases.extend(alias)
-+
-+            for completion in completions:
-+                if completion.archive == self.tree.tree_version.archive:
-+                    aliases.append(completion.nonarch)
-+
-+        except Exception, e:
-+            print e
-+            
-+        completions.extend(aliases)
-+        return completions
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(self.tree.tree_version):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+                                                   other_revision)
-+            elif action == "update":
-+                ancestor = cmdutil.tree_latest(self.tree, 
-+                                               other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        edit_log(self.tree)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = tree+"/{arch}/=log-template"
-+        if not os.path.exists(template):
-+            template = os.path.expanduser("~/.arch-params/=log-template")
-+            if not os.path.exists(template):
-+                template = None
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        
-+        new_merges = list(cmdutil.iter_new_merges(tree, 
-+                                                  tree.tree_version))
-+        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                mergestuff = cmdutil.log_for_merge(tree)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+def merge_summary(new_merges, tree_version):
-+    if len(new_merges) == 0:
-+        return ""
-+    if len(new_merges) == 1:
-+        summary = new_merges[0].summary
-+    else:
-+        summary = "Merge"
-+
-+    credits = []
-+    for merge in new_merges:
-+        if arch.my_id() != merge.creator:
-+            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+            if not name in credits:
-+                credits.append(name)
-+        else:
-+            version = merge.revision.version
-+            if version.archive == tree_version.archive:
-+                if not version.nonarch in credits:
-+                    credits.append(version.nonarch)
-+            elif not str(version) in credits:
-+                credits.append(str(version))
-+
-+    return ("%s (%s)") % (summary, ", ".join(credits))
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases
-+ acur : The latest revision in the archive of the tree-version.  You can specfy
-+        a different version like so: acur:foo--bar--0 (aliases can be used)
-+ tcur : (tree current) The latest revision in the tree of the tree-version.
-+        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+        can be used).
-+tprev : (tree previous) The previous revision in the tree of the tree-version.
-+        To specify an older revision, use a number, e.g. "tprev:4"
-+ tanc : (tree ancestor) The ancestor revision of the tree
-+        To specify an older revision, use a number, e.g. "tanc:4"
-+tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+ tmod : (tree modified) The latest revision to modify a given file 
-+        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+ ttag : (tree tag) The revision that was tagged into the current tree revision,
-+        according to the tree.
-+tagcur: (tag current) The latest revision of the version that the current tree
-+        was tagged from.
-+mergeanc : The common ancestor of the current tree and the specified revision.
-+        Defaults to the first partner-version's latest revision or to tagcur.
-+   """
-+    print "User aliases"
-+    for parts in ancillary.iter_all_alias(tree):
-+        print parts[0].rjust(10)+" : "+parts[1]
-+
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        elif len(args) == 1:
-+            self.print_alias(args[0])
-+        elif (len(args)) == 2:
-+            self.add(args[0], args[1], options)
-+        else:
-+            raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != args[0]:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(args[0])
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = cmdutil.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        cmdutil.find_editor()
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = cmdutil.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP()
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -31,6 +31,8 @@
- from revision import Revision
- from errors import bailout, BzrError
- from textui import show_status
-+import patches
-+from bzrlib import progress
- 
- BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
- ## TODO: Maybe include checks for common corruption of newlines, etc?
-@@ -802,3 +804,36 @@
- 
-     s = hexlify(rand_bytes(8))
-     return '-'.join((name, compact_date(time.time()), s))
-+
-+
-+def iter_anno_data(branch, file_id):
-+    later_revision = branch.revno()
-+    q = range(branch.revno())
-+    q.reverse()
-+    later_text_id = branch.basis_tree().inventory[file_id].text_id
-+    i = 0
-+    for revno in q:
-+        i += 1
-+        cur_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        if file_id not in cur_tree.inventory:
-+            text_id = None
-+        else:
-+            text_id = cur_tree.inventory[file_id].text_id
-+        if text_id != later_text_id:
-+            patch = get_patch(branch, revno, later_revision, file_id)
-+            yield revno, patch.iter_inserted(), patch
-+            later_revision = revno
-+            later_text_id = text_id
-+        yield progress.Progress("revisions", i)
-+
-+def get_patch(branch, old_revno, new_revno, file_id):
-+    old_tree = branch.revision_tree(branch.lookup_revision(old_revno))
-+    new_tree = branch.revision_tree(branch.lookup_revision(new_revno))
-+    if file_id in old_tree.inventory:
-+        old_file = old_tree.get_file(file_id).readlines()
-+    else:
-+        old_file = []
-+    ud = difflib.unified_diff(old_file, new_tree.get_file(file_id).readlines())
-+    return patches.parse_patch(ud)
-+
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -27,6 +27,9 @@
- from bzrlib import Branch, Inventory, InventoryEntry, ScratchBranch, BZRDIR, \
-      format_date
- from bzrlib import merge
-+from bzrlib.branch import iter_anno_data
-+from bzrlib import patches
-+from bzrlib import progress
- 
- 
- def _squish_command_name(cmd):
-@@ -882,7 +885,15 @@
-                 print '%3d FAILED!' % mf
-             else:
-                 print
--
-+        result = bzrlib.patches.test()
-+        resultFailed = len(result.errors) + len(result.failures)
-+        print '%-40s %3d tests' % ('bzrlib.patches', result.testsRun),
-+        if resultFailed:
-+            print '%3d FAILED!' % resultFailed
-+        else:
-+            print
-+        tests += result.testsRun
-+        failures += resultFailed
-         print '%-40s %3d tests' % ('total', tests),
-         if failures:
-             print '%3d FAILED!' % failures
-@@ -897,6 +908,27 @@
-     """Show version of bzr"""
-     def run(self):
-         show_version()
-+
-+class cmd_annotate(Command):
-+    """Show which revision added each line in a file"""
-+
-+    takes_args = ['filename']
-+    def run(self, filename):
-+        branch = (Branch(filename))
-+        file_id = branch.working_tree().path2id(filename)
-+        lines = branch.basis_tree().get_file(file_id)
-+        total = branch.revno()
-+        anno_d_iter = iter_anno_data(branch, file_id)
-+        for result in patches.iter_annotate_file(lines, anno_d_iter):
-+            if isinstance(result, progress.Progress):
-+                result.total = total
-+                progress.progress_bar(result)
-+            else:
-+                progress.clear_progress_bar()
-+                anno_lines = result
-+        for line in anno_lines:
-+            sys.stdout.write("%4s:%s" % (str(line.log), line.text))
-+
- 
- def show_version():
-     print "bzr (bazaar-ng) %s" % bzrlib.__version__
-

=== removed file 'patches/annotate4.patch'
--- patches/annotate4.patch	2005-06-10 06:34:26 +0000
+++ patches/annotate4.patch	1970-01-01 00:00:00 +0000
@@ -1,7270 +0,0 @@
-*** added file 'bzrlib/patches.py'
---- /dev/null 
-+++ bzrlib/patches.py 
-@@ -0,0 +1,497 @@
-+# Copyright (C) 2004, 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+import sys
-+import progress
-+class PatchSyntax(Exception):
-+    def __init__(self, msg):
-+        Exception.__init__(self, msg)
-+
-+
-+class MalformedPatchHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed patch header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedHunkHeader(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed hunk header.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+class MalformedLine(PatchSyntax):
-+    def __init__(self, desc, line):
-+        self.desc = desc
-+        self.line = line
-+        msg = "Malformed line.  %s\n%s" % (self.desc, self.line)
-+        PatchSyntax.__init__(self, msg)
-+
-+def get_patch_names(iter_lines):
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("--- "):
-+            raise MalformedPatchHeader("No orig name", line)
-+        else:
-+            orig_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No orig line", "")
-+    try:
-+        line = iter_lines.next()
-+        if not line.startswith("+++ "):
-+            raise PatchSyntax("No mod name")
-+        else:
-+            mod_name = line[4:].rstrip("\n")
-+    except StopIteration:
-+        raise MalformedPatchHeader("No mod line", "")
-+    return (orig_name, mod_name)
-+
-+def parse_range(textrange):
-+    """Parse a patch range, handling the "1" special-case
-+
-+    :param textrange: The text to parse
-+    :type textrange: str
-+    :return: the position and range, as a tuple
-+    :rtype: (int, int)
-+    """
-+    tmp = textrange.split(',')
-+    if len(tmp) == 1:
-+        pos = tmp[0]
-+        range = "1"
-+    else:
-+        (pos, range) = tmp
-+    pos = int(pos)
-+    range = int(range)
-+    return (pos, range)
-+
-+ 
-+def hunk_from_header(line):
-+    if not line.startswith("@@") or not line.endswith("@@\n") \
-+        or not len(line) > 4:
-+        raise MalformedHunkHeader("Does not start and end with @@.", line)
-+    try:
-+        (orig, mod) = line[3:-4].split(" ")
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if not orig.startswith('-') or not mod.startswith('+'):
-+        raise MalformedHunkHeader("Positions don't start with + or -.", line)
-+    try:
-+        (orig_pos, orig_range) = parse_range(orig[1:])
-+        (mod_pos, mod_range) = parse_range(mod[1:])
-+    except Exception, e:
-+        raise MalformedHunkHeader(str(e), line)
-+    if mod_range < 0 or orig_range < 0:
-+        raise MalformedHunkHeader("Hunk range is negative", line)
-+    return Hunk(orig_pos, orig_range, mod_pos, mod_range)
-+
-+
-+class HunkLine:
-+    def __init__(self, contents):
-+        self.contents = contents
-+
-+    def get_str(self, leadchar):
-+        if self.contents == "\n" and leadchar == " " and False:
-+            return "\n"
-+        return leadchar + self.contents
-+
-+class ContextLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str(" ")
-+
-+
-+class InsertLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("+")
-+
-+
-+class RemoveLine(HunkLine):
-+    def __init__(self, contents):
-+        HunkLine.__init__(self, contents)
-+
-+    def __str__(self):
-+        return self.get_str("-")
-+
-+__pychecker__="no-returnvalues"
-+def parse_line(line):
-+    if line.startswith("\n"):
-+        return ContextLine(line)
-+    elif line.startswith(" "):
-+        return ContextLine(line[1:])
-+    elif line.startswith("+"):
-+        return InsertLine(line[1:])
-+    elif line.startswith("-"):
-+        return RemoveLine(line[1:])
-+    else:
-+        raise MalformedLine("Unknown line type", line)
-+__pychecker__=""
-+
-+
-+class Hunk:
-+    def __init__(self, orig_pos, orig_range, mod_pos, mod_range):
-+        self.orig_pos = orig_pos
-+        self.orig_range = orig_range
-+        self.mod_pos = mod_pos
-+        self.mod_range = mod_range
-+        self.lines = []
-+
-+    def get_header(self):
-+        return "@@ -%s +%s @@\n" % (self.range_str(self.orig_pos, 
-+                                                   self.orig_range),
-+                                    self.range_str(self.mod_pos, 
-+                                                   self.mod_range))
-+
-+    def range_str(self, pos, range):
-+        """Return a file range, special-casing for 1-line files.
-+
-+        :param pos: The position in the file
-+        :type pos: int
-+        :range: The range in the file
-+        :type range: int
-+        :return: a string in the format 1,4 except when range == pos == 1
-+        """
-+        if range == 1:
-+            return "%i" % pos
-+        else:
-+            return "%i,%i" % (pos, range)
-+
-+    def __str__(self):
-+        lines = [self.get_header()]
-+        for line in self.lines:
-+            lines.append(str(line))
-+        return "".join(lines)
-+
-+    def shift_to_mod(self, pos):
-+        if pos < self.orig_pos-1:
-+            return 0
-+        elif pos > self.orig_pos+self.orig_range:
-+            return self.mod_range - self.orig_range
-+        else:
-+            return self.shift_to_mod_lines(pos)
-+
-+    def shift_to_mod_lines(self, pos):
-+        assert (pos >= self.orig_pos-1 and pos <= self.orig_pos+self.orig_range)
-+        position = self.orig_pos-1
-+        shift = 0
-+        for line in self.lines:
-+            if isinstance(line, InsertLine):
-+                shift += 1
-+            elif isinstance(line, RemoveLine):
-+                if position == pos:
-+                    return None
-+                shift -= 1
-+                position += 1
-+            elif isinstance(line, ContextLine):
-+                position += 1
-+            if position > pos:
-+                break
-+        return shift
-+
-+def iter_hunks(iter_lines):
-+    hunk = None
-+    for line in iter_lines:
-+        if line.startswith("@@"):
-+            if hunk is not None:
-+                yield hunk
-+            hunk = hunk_from_header(line)
-+        else:
-+            hunk.lines.append(parse_line(line))
-+
-+    if hunk is not None:
-+        yield hunk
-+
-+class Patch:
-+    def __init__(self, oldname, newname):
-+        self.oldname = oldname
-+        self.newname = newname
-+        self.hunks = []
-+
-+    def __str__(self):
-+        ret =  "--- %s\n+++ %s\n" % (self.oldname, self.newname) 
-+        ret += "".join([str(h) for h in self.hunks])
-+        return ret
-+
-+    def stats_str(self):
-+        """Return a string of patch statistics"""
-+        removes = 0
-+        inserts = 0
-+        for hunk in self.hunks:
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                     inserts+=1;
-+                elif isinstance(line, RemoveLine):
-+                     removes+=1;
-+        return "%i inserts, %i removes in %i hunks" % \
-+            (inserts, removes, len(self.hunks))
-+
-+    def pos_in_mod(self, position):
-+        newpos = position
-+        for hunk in self.hunks:
-+            shift = hunk.shift_to_mod(position)
-+            if shift is None:
-+                return None
-+            newpos += shift
-+        return newpos
-+            
-+    def iter_inserted(self):
-+        """Iteraties through inserted lines
-+        
-+        :return: Pair of line number, line
-+        :rtype: iterator of (int, InsertLine)
-+        """
-+        for hunk in self.hunks:
-+            pos = hunk.mod_pos - 1;
-+            for line in hunk.lines:
-+                if isinstance(line, InsertLine):
-+                    yield (pos, line)
-+                    pos += 1
-+                if isinstance(line, ContextLine):
-+                    pos += 1
-+
-+def parse_patch(iter_lines):
-+    (orig_name, mod_name) = get_patch_names(iter_lines)
-+    patch = Patch(orig_name, mod_name)
-+    for hunk in iter_hunks(iter_lines):
-+        patch.hunks.append(hunk)
-+    return patch
-+
-+
-+class AnnotateLine:
-+    """A line associated with the log that produced it"""
-+    def __init__(self, text, log=None):
-+        self.text = text
-+        self.log = log
-+
-+class CantGetRevisionData(Exception):
-+    def __init__(self, revision):
-+        Exception.__init__(self, "Can't get data for revision %s" % revision)
-+        
-+def annotate_file2(file_lines, anno_iter):
-+    for result in iter_annotate_file(file_lines, anno_iter):
-+        pass
-+    return result
-+
-+        
-+def iter_annotate_file(file_lines, anno_iter):
-+    lines = [AnnotateLine(f) for f in file_lines]
-+    patches = []
-+    try:
-+        for result in anno_iter:
-+            if isinstance(result, progress.Progress):
-+                yield result
-+                continue
-+            log, iter_inserted, patch = result
-+            for (num, line) in iter_inserted:
-+                old_num = num
-+
-+                for cur_patch in patches:
-+                    num = cur_patch.pos_in_mod(num)
-+                    if num == None: 
-+                        break
-+
-+                if num >= len(lines):
-+                    continue
-+                if num is not None and lines[num].log is None:
-+                    lines[num].log = log
-+            patches=[patch]+patches
-+    except CantGetRevisionData:
-+        pass
-+    yield lines
-+
-+
-+def difference_index(atext, btext):
-+    """Find the indext of the first character that differs betweeen two texts
-+
-+    :param atext: The first text
-+    :type atext: str
-+    :param btext: The second text
-+    :type str: str
-+    :return: The index, or None if there are no differences within the range
-+    :rtype: int or NoneType
-+    """
-+    length = len(atext)
-+    if len(btext) < length:
-+        length = len(btext)
-+    for i in range(length):
-+        if atext[i] != btext[i]:
-+            return i;
-+    return None
-+
-+
-+def test():
-+    import unittest
-+    class PatchesTester(unittest.TestCase):
-+        def testValidPatchHeader(self):
-+            """Parse a valid patch header"""
-+            lines = "--- orig/commands.py\n+++ mod/dommands.py\n".split('\n')
-+            (orig, mod) = get_patch_names(lines.__iter__())
-+            assert(orig == "orig/commands.py")
-+            assert(mod == "mod/dommands.py")
-+
-+        def testInvalidPatchHeader(self):
-+            """Parse an invalid patch header"""
-+            lines = "-- orig/commands.py\n+++ mod/dommands.py".split('\n')
-+            self.assertRaises(MalformedPatchHeader, get_patch_names,
-+                              lines.__iter__())
-+
-+        def testValidHunkHeader(self):
-+            """Parse a valid hunk header"""
-+            header = "@@ -34,11 +50,6 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 34)
-+            assert (hunk.orig_range == 11)
-+            assert (hunk.mod_pos == 50)
-+            assert (hunk.mod_range == 6)
-+            assert (str(hunk) == header)
-+
-+        def testValidHunkHeader2(self):
-+            """Parse a tricky, valid hunk header"""
-+            header = "@@ -1 +0,0 @@\n"
-+            hunk = hunk_from_header(header);
-+            assert (hunk.orig_pos == 1)
-+            assert (hunk.orig_range == 1)
-+            assert (hunk.mod_pos == 0)
-+            assert (hunk.mod_range == 0)
-+            assert (str(hunk) == header)
-+
-+        def makeMalformed(self, header):
-+            self.assertRaises(MalformedHunkHeader, hunk_from_header, header)
-+
-+        def testInvalidHeader(self):
-+            """Parse an invalid hunk header"""
-+            self.makeMalformed(" -34,11 +50,6 \n")
-+            self.makeMalformed("@@ +50,6 -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6 @@")
-+            self.makeMalformed("@@ -34.5,11 +50,6 @@\n")
-+            self.makeMalformed("@@-34,11 +50,6@@\n")
-+            self.makeMalformed("@@ 34,11 50,6 @@\n")
-+            self.makeMalformed("@@ -34,11 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,6.5 @@\n")
-+            self.makeMalformed("@@ -34,11 +50,-6 @@\n")
-+
-+        def lineThing(self,text, type):
-+            line = parse_line(text)
-+            assert(isinstance(line, type))
-+            assert(str(line)==text)
-+
-+        def makeMalformedLine(self, text):
-+            self.assertRaises(MalformedLine, parse_line, text)
-+
-+        def testValidLine(self):
-+            """Parse a valid hunk line"""
-+            self.lineThing(" hello\n", ContextLine)
-+            self.lineThing("+hello\n", InsertLine)
-+            self.lineThing("-hello\n", RemoveLine)
-+        
-+        def testMalformedLine(self):
-+            """Parse invalid valid hunk lines"""
-+            self.makeMalformedLine("hello\n")
-+        
-+        def compare_parsed(self, patchtext):
-+            lines = patchtext.splitlines(True)
-+            patch = parse_patch(lines.__iter__())
-+            pstr = str(patch)
-+            i = difference_index(patchtext, pstr)
-+            if i is not None:
-+                print "%i: \"%s\" != \"%s\"" % (i, patchtext[i], pstr[i])
-+            assert (patchtext == str(patch))
-+
-+        def testAll(self):
-+            """Test parsing a whole patch"""
-+            patchtext = """--- orig/commands.py
-++++ mod/commands.py
-+@@ -1337,7 +1337,8 @@
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree,
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1983,7 +1984,11 @@
-+                                          version)
-+         if len(new_merges) > 0:
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree, comp_version)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+"""
-+            self.compare_parsed(patchtext)
-+
-+        def testInit(self):
-+            """Handle patches missing half the position, range tuple"""
-+            patchtext = \
-+"""--- orig/__init__.py
-++++ mod/__init__.py
-+@@ -1 +1,2 @@
-+ __docformat__ = "restructuredtext en"
-++__doc__ = An alternate Arch commandline interface"""
-+            self.compare_parsed(patchtext)
-+            
-+
-+
-+        def testLineLookup(self):
-+            """Make sure we can accurately look up mod line from orig"""
-+            patch = parse_patch(open("testdata/diff"))
-+            orig = list(open("testdata/orig"))
-+            mod = list(open("testdata/mod"))
-+            removals = []
-+            for i in range(len(orig)):
-+                mod_pos = patch.pos_in_mod(i)
-+                if mod_pos is None:
-+                    removals.append(orig[i])
-+                    continue
-+                assert(mod[mod_pos]==orig[i])
-+            rem_iter = removals.__iter__()
-+            for hunk in patch.hunks:
-+                for line in hunk.lines:
-+                    if isinstance(line, RemoveLine):
-+                        next = rem_iter.next()
-+                        if line.contents != next:
-+                            sys.stdout.write(" orig:%spatch:%s" % (next,
-+                                             line.contents))
-+                        assert(line.contents == next)
-+            self.assertRaises(StopIteration, rem_iter.next)
-+
-+        def testFirstLineRenumber(self):
-+            """Make sure we handle lines at the beginning of the hunk"""
-+            patch = parse_patch(open("testdata/insert_top.patch"))
-+            assert (patch.pos_in_mod(0)==1)
-+    
-+            
-+    patchesTestSuite = unittest.makeSuite(PatchesTester,'test')
-+    runner = unittest.TextTestRunner(verbosity=0)
-+    return runner.run(patchesTestSuite)
-+    
-+
-+if __name__ == "__main__":
-+    test()
-+# arch-tag: d1541a25-eac5-4de9-a476-08a7cecd5683
-
-*** added directory 'testdata'
-*** added file 'testdata/diff'
---- /dev/null 
-+++ testdata/diff 
-@@ -0,0 +1,1154 @@
-+--- orig/commands.py
-++++ mod/commands.py
-+@@ -19,25 +19,31 @@
-+ import arch
-+ import arch.util
-+ import arch.arch
-++
-++import pylon.errors
-++from pylon.errors import *
-++from pylon import errors
-++from pylon import util
-++from pylon import arch_core
-++from pylon import arch_compound
-++from pylon import ancillary
-++from pylon import misc
-++from pylon import paths 
-++
-+ import abacmds
-+ import cmdutil
-+ import shutil
-+ import os
-+ import options
-+-import paths 
-+ import time
-+ import cmd
-+ import readline
-+ import re
-+ import string
-+-import arch_core
-+-from errors import *
-+-import errors
-+ import terminal
-+-import ancillary
-+-import misc
-+ import email
-+ import smtplib
-++import textwrap
-+ 
-+ __docformat__ = "restructuredtext"
-+ __doc__ = "Implementation of user (sub) commands"
-+@@ -257,7 +263,7 @@
-+ 
-+         tree=arch.tree_root()
-+         if len(args) == 0:
-+-            a_spec = cmdutil.comp_revision(tree)
-++            a_spec = ancillary.comp_revision(tree)
-+         else:
-+             a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+         cmdutil.ensure_archive_registered(a_spec.archive)
-+@@ -284,7 +290,7 @@
-+             changeset=options.changeset
-+             tmpdir = None
-+         else:
-+-            tmpdir=cmdutil.tmpdir()
-++            tmpdir=util.tmpdir()
-+             changeset=tmpdir+"/changeset"
-+         try:
-+             delta=arch.iter_delta(a_spec, b_spec, changeset)
-+@@ -304,14 +310,14 @@
-+             if status > 1:
-+                 return
-+             if (options.perform_diff):
-+-                chan = cmdutil.ChangesetMunger(changeset)
-++                chan = arch_compound.ChangesetMunger(changeset)
-+                 chan.read_indices()
-+-                if isinstance(b_spec, arch.Revision):
-+-                    b_dir = b_spec.library_find()
-+-                else:
-+-                    b_dir = b_spec
-+-                a_dir = a_spec.library_find()
-+                 if options.diffopts is not None:
-++                    if isinstance(b_spec, arch.Revision):
-++                        b_dir = b_spec.library_find()
-++                    else:
-++                        b_dir = b_spec
-++                    a_dir = a_spec.library_find()
-+                     diffopts = options.diffopts.split()
-+                     cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                 else:
-+@@ -517,7 +523,7 @@
-+         except arch.errors.TreeRootError, e:
-+             print e
-+             return
-+-        from_revision=cmdutil.tree_latest(tree)
-++        from_revision = arch_compound.tree_latest(tree)
-+         if from_revision==to_revision:
-+             print "Tree is already up to date with:\n"+str(to_revision)+"."
-+             return
-+@@ -592,6 +598,9 @@
-+ 
-+         if len(args) == 0:
-+             args = None
-++        if options.version is None:
-++            return options, tree.tree_version, args
-++
-+         revision=cmdutil.determine_revision_arch(tree, options.version)
-+         return options, revision.get_version(), args
-+ 
-+@@ -601,11 +610,16 @@
-+         """
-+         tree=arch.tree_root()
-+         options, version, files = self.parse_commandline(cmdargs, tree)
-++        ancestor = None
-+         if options.__dict__.has_key("base") and options.base:
-+             base = cmdutil.determine_revision_tree(tree, options.base)
-++            ancestor = base
-+         else:
-+-            base = cmdutil.submit_revision(tree)
-+-        
-++            base = ancillary.submit_revision(tree)
-++            ancestor = base
-++        if ancestor is None:
-++            ancestor = arch_compound.tree_latest(tree, version)
-++
-+         writeversion=version
-+         archive=version.archive
-+         source=cmdutil.get_mirror_source(archive)
-+@@ -625,18 +639,26 @@
-+         try:
-+             last_revision=tree.iter_logs(version, True).next().revision
-+         except StopIteration, e:
-+-            if cmdutil.prompt("Import from commit"):
-+-                return do_import(version)
-+-            else:
-+-                raise NoVersionLogs(version)
-+-        if last_revision!=version.iter_revisions(True).next():
-++            last_revision = None
-++            if ancestor is None:
-++                if cmdutil.prompt("Import from commit"):
-++                    return do_import(version)
-++                else:
-++                    raise NoVersionLogs(version)
-++        try:
-++            arch_last_revision = version.iter_revisions(True).next()
-++        except StopIteration, e:
-++            arch_last_revision = None
-++ 
-++        if last_revision != arch_last_revision:
-++            print "Tree is not up to date with %s" % str(version)
-+             if not cmdutil.prompt("Out of date"):
-+                 raise OutOfDate
-+             else:
-+                 allow_old=True
-+ 
-+         try:
-+-            if not cmdutil.has_changed(version):
-++            if not cmdutil.has_changed(ancestor):
-+                 if not cmdutil.prompt("Empty commit"):
-+                     raise EmptyCommit
-+         except arch.util.ExecProblem, e:
-+@@ -645,15 +667,15 @@
-+                 raise MissingID(e)
-+             else:
-+                 raise
-+-        log = tree.log_message(create=False)
-++        log = tree.log_message(create=False, version=version)
-+         if log is None:
-+             try:
-+                 if cmdutil.prompt("Create log"):
-+-                    edit_log(tree)
-++                    edit_log(tree, version)
-+ 
-+             except cmdutil.NoEditorSpecified, e:
-+                 raise CommandFailed(e)
-+-            log = tree.log_message(create=False)
-++            log = tree.log_message(create=False, version=version)
-+         if log is None: 
-+             raise NoLogMessage
-+         if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+@@ -837,23 +859,24 @@
-+             if spec is not None:
-+                 revision = cmdutil.determine_revision_tree(tree, spec)
-+             else:
-+-                revision = cmdutil.comp_revision(tree)
-++                revision = ancillary.comp_revision(tree)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         munger = None
-+ 
-+         if options.file_contents or options.file_perms or options.deletions\
-+             or options.additions or options.renames or options.hunk_prompt:
-+-            munger = cmdutil.MungeOpts()
-+-            munger.hunk_prompt = options.hunk_prompt
-++            munger = arch_compound.MungeOpts()
-++            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-++                                   options.hunk_prompt)
-+ 
-+         if len(args) > 0 or options.logs or options.pattern_files or \
-+             options.control:
-+             if munger is None:
-+-                munger = cmdutil.MungeOpts(True)
-++                munger = cmdutil.arch_compound.MungeOpts(True)
-+                 munger.all_types(True)
-+         if len(args) > 0:
-+-            t_cwd = cmdutil.tree_cwd(tree)
-++            t_cwd = arch_compound.tree_cwd(tree)
-+             for name in args:
-+                 if len(t_cwd) > 0:
-+                     t_cwd += "/"
-+@@ -878,7 +901,7 @@
-+         if options.pattern_files:
-+             munger.add_keep_pattern(options.pattern_files)
-+                 
-+-        for line in cmdutil.revert(tree, revision, munger, 
-++        for line in arch_compound.revert(tree, revision, munger, 
-+                                    not options.no_output):
-+             cmdutil.colorize(line)
-+ 
-+@@ -1042,18 +1065,13 @@
-+         help_tree_spec()
-+         return
-+ 
-+-def require_version_exists(version, spec):
-+-    if not version.exists():
-+-        raise cmdutil.CantDetermineVersion(spec, 
-+-                                           "The version %s does not exist." \
-+-                                           % version)
-+-
-+ class Revisions(BaseCommand):
-+     """
-+     Print a revision name based on a revision specifier
-+     """
-+     def __init__(self):
-+         self.description="Lists revisions"
-++        self.cl_revisions = []
-+     
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1066,224 +1084,68 @@
-+             self.tree = arch.tree_root()
-+         except arch.errors.TreeRootError:
-+             self.tree = None
-++        if options.type == "default":
-++            options.type = "archive"
-+         try:
-+-            iter = self.get_iterator(options.type, args, options.reverse, 
-+-                                     options.modified)
-++            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-++                                             options.reverse, options.modified,
-++                                             options.shallow)
-+         except cmdutil.CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+-
-++        except cmdutil.CantDetermineVersion, e:
-++            raise CommandFailedWrapper(e)
-+         if options.skip is not None:
-+             iter = cmdutil.iter_skip(iter, int(options.skip))
-+ 
-+-        for revision in iter:
-+-            log = None
-+-            if isinstance(revision, arch.Patchlog):
-+-                log = revision
-+-                revision=revision.revision
-+-            print options.display(revision)
-+-            if log is None and (options.summary or options.creator or 
-+-                                options.date or options.merges):
-+-                log = revision.patchlog
-+-            if options.creator:
-+-                print "    %s" % log.creator
-+-            if options.date:
-+-                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+-            if options.summary:
-+-                print "    %s" % log.summary
-+-            if options.merges:
-+-                showed_title = False
-+-                for revision in log.merged_patches:
-+-                    if not showed_title:
-+-                        print "    Merged:"
-+-                        showed_title = True
-+-                    print "    %s" % revision
-+-
-+-    def get_iterator(self, type, args, reverse, modified):
-+-        if len(args) > 0:
-+-            spec = args[0]
-+-        else:
-+-            spec = None
-+-        if modified is not None:
-+-            iter = cmdutil.modified_iter(modified, self.tree)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-        elif type == "archive":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return version.iter_revisions(reverse)
-+-        elif type == "cacherevs":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-                cmdutil.ensure_archive_registered(version.archive)
-+-                require_version_exists(version, spec)
-+-            return cmdutil.iter_cacherevs(version, reverse)
-+-        elif type == "library":
-+-            if spec is None:
-+-                if self.tree is None:
-+-                    raise cmdutil.CantDetermineRevision("", 
-+-                                                        "Not in a project tree")
-+-                version = cmdutil.determine_version_tree(spec, self.tree)
-+-            else:
-+-                version = cmdutil.determine_version_arch(spec, self.tree)
-+-            return version.iter_library_revisions(reverse)
-+-        elif type == "logs":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+-                                  self.tree), reverse)
-+-        elif type == "missing" or type == "skip-present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            skip = (type == "skip-present")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_missing(self.tree, version, reverse,
-+-                                        skip_present=skip)
-+-
-+-        elif type == "present":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            return cmdutil.iter_present(self.tree, version, reverse)
-+-
-+-        elif type == "new-merges" or type == "direct-merges":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            version = cmdutil.determine_version_tree(spec, self.tree)
-+-            cmdutil.ensure_archive_registered(version.archive)
-+-            require_version_exists(version, spec)
-+-            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+-            if type == "new-merges":
-+-                return iter
-+-            elif type == "direct-merges":
-+-                return cmdutil.direct_merges(iter)
-+-
-+-        elif type == "missing-from":
-+-            if self.tree is None:
-+-                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            libtree = cmdutil.find_or_make_local_revision(revision)
-+-            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+-                                        reverse)
-+-
-+-        elif type == "partner-missing":
-+-            return cmdutil.iter_partner_missing(self.tree, reverse)
-+-
-+-        elif type == "ancestry":
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            if reverse:
-+-                return iter
-+-            else:
-+-                return cmdutil.iter_reverse(iter)
-+-
-+-        elif type == "dependencies" or type == "non-dependencies":
-+-            nondeps = (type == "non-dependencies")
-+-            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+-            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+-            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+-            if reverse:
-+-                return iter_depends
-+-            else:
-+-                return cmdutil.iter_reverse(iter_depends)
-+-        elif type == "micro":
-+-            return cmdutil.iter_micro(self.tree)
-+-
-+-    
-++        try:
-++            for revision in iter:
-++                log = None
-++                if isinstance(revision, arch.Patchlog):
-++                    log = revision
-++                    revision=revision.revision
-++                out = options.display(revision)
-++                if out is not None:
-++                    print out
-++                if log is None and (options.summary or options.creator or 
-++                                    options.date or options.merges):
-++                    log = revision.patchlog
-++                if options.creator:
-++                    print "    %s" % log.creator
-++                if options.date:
-++                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-++                if options.summary:
-++                    print "    %s" % log.summary
-++                if options.merges:
-++                    showed_title = False
-++                    for revision in log.merged_patches:
-++                        if not showed_title:
-++                            print "    Merged:"
-++                            showed_title = True
-++                        print "    %s" % revision
-++            if len(self.cl_revisions) > 0:
-++                print pylon.changelog_for_merge(self.cl_revisions)
-++        except pylon.errors.TreeRootNone:
-++            raise CommandFailedWrapper(
-++                Exception("This option can only be used in a project tree."))
-++
-++    def changelog_append(self, revision):
-++        if isinstance(revision, arch.Revision):
-++            revision=arch.Patchlog(revision)
-++        self.cl_revisions.append(revision)
-++   
-+     def get_parser(self):
-+         """
-+         Returns the options parser to use for the "revision" command.
-+ 
-+         :rtype: cmdutil.CmdOptionParser
-+         """
-+-        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-++        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+         select = cmdutil.OptionGroup(parser, "Selection options",
-+                           "Control which revisions are listed.  These options"
-+                           " are mutually exclusive.  If more than one is"
-+                           " specified, the last is used.")
-+-        select.add_option("", "--archive", action="store_const", 
-+-                          const="archive", dest="type", default="archive",
-+-                          help="List all revisions in the archive")
-+-        select.add_option("", "--cacherevs", action="store_const", 
-+-                          const="cacherevs", dest="type",
-+-                          help="List all revisions stored in the archive as "
-+-                          "complete copies")
-+-        select.add_option("", "--logs", action="store_const", 
-+-                          const="logs", dest="type",
-+-                          help="List revisions that have a patchlog in the "
-+-                          "tree")
-+-        select.add_option("", "--missing", action="store_const", 
-+-                          const="missing", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree")
-+-        select.add_option("", "--skip-present", action="store_const", 
-+-                          const="skip-present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlogs at all in the tree")
-+-        select.add_option("", "--present", action="store_const", 
-+-                          const="present", dest="type",
-+-                          help="List revisions from the specified version that"
-+-                          " have no patchlog in the tree, but can't be merged")
-+-        select.add_option("", "--missing-from", action="store_const", 
-+-                          const="missing-from", dest="type",
-+-                          help="List revisions from the specified revision "
-+-                          "that have no patchlog for the tree version")
-+-        select.add_option("", "--partner-missing", action="store_const", 
-+-                          const="partner-missing", dest="type",
-+-                          help="List revisions in partner versions that are"
-+-                          " missing")
-+-        select.add_option("", "--new-merges", action="store_const", 
-+-                          const="new-merges", dest="type",
-+-                          help="List revisions that have had patchlogs added"
-+-                          " to the tree since the last commit")
-+-        select.add_option("", "--direct-merges", action="store_const", 
-+-                          const="direct-merges", dest="type",
-+-                          help="List revisions that have been directly added"
-+-                          " to tree since the last commit ")
-+-        select.add_option("", "--library", action="store_const", 
-+-                          const="library", dest="type",
-+-                          help="List revisions in the revision library")
-+-        select.add_option("", "--ancestry", action="store_const", 
-+-                          const="ancestry", dest="type",
-+-                          help="List revisions that are ancestors of the "
-+-                          "current tree version")
-+-
-+-        select.add_option("", "--dependencies", action="store_const", 
-+-                          const="dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "depends on")
-+-
-+-        select.add_option("", "--non-dependencies", action="store_const", 
-+-                          const="non-dependencies", dest="type",
-+-                          help="List revisions that the given revision "
-+-                          "does not depend on")
-+-
-+-        select.add_option("--micro", action="store_const", 
-+-                          const="micro", dest="type",
-+-                          help="List partner revisions aimed for this "
-+-                          "micro-branch")
-+-
-+-        select.add_option("", "--modified", dest="modified", 
-+-                          help="List tree ancestor revisions that modified a "
-+-                          "given file", metavar="FILE[:LINE]")
-+ 
-++        cmdutil.add_revision_iter_options(select)
-+         parser.add_option("", "--skip", dest="skip", 
-+                           help="Skip revisions.  Positive numbers skip from "
-+                           "beginning, negative skip from end.",
-+@@ -1312,6 +1174,9 @@
-+         format.add_option("--cacherev", action="store_const", 
-+                          const=paths.determine_cacherev_path, dest="display",
-+                          help="Show location of cacherev file")
-++        format.add_option("--changelog", action="store_const", 
-++                         const=self.changelog_append, dest="display",
-++                         help="Show location of cacherev file")
-+         parser.add_option_group(format)
-+         display = cmdutil.OptionGroup(parser, "Display format options",
-+                           "These control the display of data")
-+@@ -1448,6 +1313,7 @@
-+         if os.access(self.history_file, os.R_OK) and \
-+             os.path.isfile(self.history_file):
-+             readline.read_history_file(self.history_file)
-++        self.cwd = os.getcwd()
-+ 
-+     def write_history(self):
-+         readline.write_history_file(self.history_file)
-+@@ -1470,16 +1336,21 @@
-+     def set_prompt(self):
-+         if self.tree is not None:
-+             try:
-+-                version = " "+self.tree.tree_version.nonarch
-++                prompt = pylon.alias_or_version(self.tree.tree_version, 
-++                                                self.tree, 
-++                                                full=False)
-++                if prompt is not None:
-++                    prompt = " " + prompt
-+             except:
-+-                version = ""
-++                prompt = ""
-+         else:
-+-            version = ""
-+-        self.prompt = "Fai%s> " % version
-++            prompt = ""
-++        self.prompt = "Fai%s> " % prompt
-+ 
-+     def set_title(self, command=None):
-+         try:
-+-            version = self.tree.tree_version.nonarch
-++            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-++                                             full=False)
-+         except:
-+             version = "[no version]"
-+         if command is None:
-+@@ -1489,8 +1360,15 @@
-+     def do_cd(self, line):
-+         if line == "":
-+             line = "~"
-++        line = os.path.expanduser(line)
-++        if os.path.isabs(line):
-++            newcwd = line
-++        else:
-++            newcwd = self.cwd+'/'+line
-++        newcwd = os.path.normpath(newcwd)
-+         try:
-+-            os.chdir(os.path.expanduser(line))
-++            os.chdir(newcwd)
-++            self.cwd = newcwd
-+         except Exception, e:
-+             print e
-+         try:
-+@@ -1523,7 +1401,7 @@
-+             except cmdutil.CantDetermineRevision, e:
-+                 print e
-+             except Exception, e:
-+-                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-++                print "Unhandled error:\n%s" % errors.exception_str(e)
-+ 
-+         elif suggestions.has_key(args[0]):
-+             print suggestions[args[0]]
-+@@ -1574,7 +1452,7 @@
-+                 arg = line.split()[-1]
-+             else:
-+                 arg = ""
-+-            iter = iter_munged_completions(iter, arg, text)
-++            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+         except Exception, e:
-+             print e
-+         return list(iter)
-+@@ -1604,10 +1482,11 @@
-+                 else:
-+                     arg = ""
-+                 if arg.startswith("-"):
-+-                    return list(iter_munged_completions(iter, arg, text))
-++                    return list(cmdutil.iter_munged_completions(iter, arg, 
-++                                                                text))
-+                 else:
-+-                    return list(iter_munged_completions(
-+-                        iter_file_completions(arg), arg, text))
-++                    return list(cmdutil.iter_munged_completions(
-++                        cmdutil.iter_file_completions(arg), arg, text))
-+ 
-+ 
-+             elif cmd == "cd":
-+@@ -1615,13 +1494,13 @@
-+                     arg = args.split()[-1]
-+                 else:
-+                     arg = ""
-+-                iter = iter_dir_completions(arg)
-+-                iter = iter_munged_completions(iter, arg, text)
-++                iter = cmdutil.iter_dir_completions(arg)
-++                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                 return list(iter)
-+             elif len(args)>0:
-+                 arg = args.split()[-1]
-+-                return list(iter_munged_completions(iter_file_completions(arg),
-+-                                                    arg, text))
-++                iter = cmdutil.iter_file_completions(arg)
-++                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+             else:
-+                 return self.completenames(text, line, begidx, endidx)
-+         except Exception, e:
-+@@ -1636,44 +1515,8 @@
-+             yield entry
-+ 
-+ 
-+-def iter_file_completions(arg, only_dirs = False):
-+-    """Generate an iterator that iterates through filename completions.
-+-
-+-    :param arg: The filename fragment to match
-+-    :type arg: str
-+-    :param only_dirs: If true, match only directories
-+-    :type only_dirs: bool
-+-    """
-+-    cwd = os.getcwd()
-+-    if cwd != "/":
-+-        extras = [".", ".."]
-+-    else:
-+-        extras = []
-+-    (dir, file) = os.path.split(arg)
-+-    if dir != "":
-+-        listingdir = os.path.expanduser(dir)
-+-    else:
-+-        listingdir = cwd
-+-    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+-        if dir != "":
-+-            userfile = dir+'/'+file
-+-        else:
-+-            userfile = file
-+-        if userfile.startswith(arg):
-+-            if os.path.isdir(listingdir+'/'+file):
-+-                userfile+='/'
-+-                yield userfile
-+-            elif not only_dirs:
-+-                yield userfile
-+-
-+-def iter_munged_completions(iter, arg, text):
-+-    for completion in iter:
-+-        completion = str(completion)
-+-        if completion.startswith(arg):
-+-            yield completion[len(arg)-len(text):]
-+-
-+ def iter_source_file_completions(tree, arg):
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1701,7 +1544,7 @@
-+     :return: An iterator of all matching untagged files
-+     :rtype: iterator of str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-++    treepath = arch_compound.tree_cwd(tree)
-+     if len(treepath) > 0:
-+         dirs = [treepath]
-+     else:
-+@@ -1743,8 +1586,8 @@
-+     :param arg: The prefix to match
-+     :type arg: str
-+     """
-+-    treepath = cmdutil.tree_cwd(tree)
-+-    tmpdir = cmdutil.tmpdir()
-++    treepath = arch_compound.tree_cwd(tree)
-++    tmpdir = util.tmpdir()
-+     changeset = tmpdir+"/changeset"
-+     completions = []
-+     revision = cmdutil.determine_revision_tree(tree)
-+@@ -1756,14 +1599,6 @@
-+     shutil.rmtree(tmpdir)
-+     return completions
-+ 
-+-def iter_dir_completions(arg):
-+-    """Generate an iterator that iterates through directory name completions.
-+-
-+-    :param arg: The directory name fragment to match
-+-    :type arg: str
-+-    """
-+-    return iter_file_completions(arg, True)
-+-
-+ class Shell(BaseCommand):
-+     def __init__(self):
-+         self.description = "Runs Fai as a shell"
-+@@ -1795,7 +1630,11 @@
-+         parser=self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+ 
-+-        tree = arch.tree_root()
-++        try:
-++            tree = arch.tree_root()
-++        except arch.errors.TreeRootError, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++            
-+ 
-+         if (len(args) == 0) == (options.untagged == False):
-+             raise cmdutil.GetHelp
-+@@ -1809,13 +1648,22 @@
-+         if options.id_type == "tagline":
-+             if method != "tagline":
-+                 if not cmdutil.prompt("Tagline in other tree"):
-+-                    if method == "explicit":
-+-                        options.id_type == explicit
-++                    if method == "explicit" or method == "implicit":
-++                        options.id_type == method
-+                     else:
-+                         print "add-id not supported for \"%s\" tagging method"\
-+                             % method 
-+                         return
-+         
-++        elif options.id_type == "implicit":
-++            if method != "implicit":
-++                if not cmdutil.prompt("Implicit in other tree"):
-++                    if method == "explicit" or method == "tagline":
-++                        options.id_type == method
-++                    else:
-++                        print "add-id not supported for \"%s\" tagging method"\
-++                            % method 
-++                        return
-+         elif options.id_type == "explicit":
-+             if method != "tagline" and method != explicit:
-+                 if not prompt("Explicit in other tree"):
-+@@ -1824,7 +1672,8 @@
-+                     return
-+         
-+         if options.id_type == "auto":
-+-            if method != "tagline" and method != "explicit":
-++            if method != "tagline" and method != "explicit" \
-++                and method !="implicit":
-+                 print "add-id not supported for \"%s\" tagging method" % method
-+                 return
-+             else:
-+@@ -1852,10 +1701,12 @@
-+             previous_files.extend(files)
-+             if id_type == "explicit":
-+                 cmdutil.add_id(files)
-+-            elif id_type == "tagline":
-++            elif id_type == "tagline" or id_type == "implicit":
-+                 for file in files:
-+                     try:
-+-                        cmdutil.add_tagline_or_explicit_id(file)
-++                        implicit = (id_type == "implicit")
-++                        cmdutil.add_tagline_or_explicit_id(file, False,
-++                                                           implicit)
-+                     except cmdutil.AlreadyTagged:
-+                         print "\"%s\" already has a tagline." % file
-+                     except cmdutil.NoCommentSyntax:
-+@@ -1888,6 +1739,9 @@
-+         parser.add_option("--tagline", action="store_const", 
-+                          const="tagline", dest="id_type", 
-+                          help="Use a tagline id")
-++        parser.add_option("--implicit", action="store_const", 
-++                         const="implicit", dest="id_type", 
-++                         help="Use an implicit id (deprecated)")
-+         parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", default=False, 
-+                          help="tag all untagged files")
-+@@ -1926,27 +1780,7 @@
-+     def get_completer(self, arg, index):
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+-        completions = list(ancillary.iter_partners(self.tree, 
-+-                                                   self.tree.tree_version))
-+-        if len(completions) == 0:
-+-            completions = list(self.tree.iter_log_versions())
-+-
-+-        aliases = []
-+-        try:
-+-            for completion in completions:
-+-                alias = ancillary.compact_alias(str(completion), self.tree)
-+-                if alias:
-+-                    aliases.extend(alias)
-+-
-+-            for completion in completions:
-+-                if completion.archive == self.tree.tree_version.archive:
-+-                    aliases.append(completion.nonarch)
-+-
-+-        except Exception, e:
-+-            print e
-+-            
-+-        completions.extend(aliases)
-+-        return completions
-++        return cmdutil.merge_completions(self.tree, arg, index)
-+ 
-+     def do_command(self, cmdargs):
-+         """
-+@@ -1961,7 +1795,7 @@
-+         
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError(os.getcwd())
-+-        if cmdutil.has_changed(self.tree.tree_version):
-++        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+             raise UncommittedChanges(self.tree)
-+ 
-+         if len(args) > 0:
-+@@ -2027,14 +1861,14 @@
-+         :type other_revision: `arch.Revision`
-+         :return: 0 if the merge was skipped, 1 if it was applied
-+         """
-+-        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-++        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+         try:
-+             if action == "native-merge":
-+-                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+-                                                   other_revision)
-++                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-++                                                         other_revision)
-+             elif action == "update":
-+-                ancestor = cmdutil.tree_latest(self.tree, 
-+-                                               other_revision.version)
-++                ancestor = arch_compound.tree_latest(self.tree, 
-++                                                     other_revision.version)
-+         except CantDetermineRevision, e:
-+             raise CommandFailedWrapper(e)
-+         cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+@@ -2104,7 +1938,10 @@
-+         if self.tree is None:
-+             raise arch.errors.TreeRootError
-+ 
-+-        edit_log(self.tree)
-++        try:
-++            edit_log(self.tree, self.tree.tree_version)
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-+ 
-+     def get_parser(self):
-+         """
-+@@ -2132,7 +1969,7 @@
-+         """
-+         return
-+ 
-+-def edit_log(tree):
-++def edit_log(tree, version):
-+     """Makes and edits the log for a tree.  Does all kinds of fancy things
-+     like log templates and merge summaries and log-for-merge
-+     
-+@@ -2141,28 +1978,29 @@
-+     """
-+     #ensure we have an editor before preparing the log
-+     cmdutil.find_editor()
-+-    log = tree.log_message(create=False)
-++    log = tree.log_message(create=False, version=version)
-+     log_is_new = False
-+     if log is None or cmdutil.prompt("Overwrite log"):
-+         if log is not None:
-+            os.remove(log.name)
-+-        log = tree.log_message(create=True)
-++        log = tree.log_message(create=True, version=version)
-+         log_is_new = True
-+         tmplog = log.name
-+-        template = tree+"/{arch}/=log-template"
-+-        if not os.path.exists(template):
-+-            template = os.path.expanduser("~/.arch-params/=log-template")
-+-            if not os.path.exists(template):
-+-                template = None
-++        template = pylon.log_template_path(tree)
-+         if template:
-+             shutil.copyfile(template, tmplog)
-+-        
-+-        new_merges = list(cmdutil.iter_new_merges(tree, 
-+-                                                  tree.tree_version))
-+-        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-++        comp_version = ancillary.comp_revision(tree).version
-++        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-++        new_merges = cmdutil.direct_merges(new_merges)
-++        log["Summary"] = pylon.merge_summary(new_merges, 
-++                                         version)
-+         if len(new_merges) > 0:   
-+             if cmdutil.prompt("Log for merge"):
-+-                mergestuff = cmdutil.log_for_merge(tree)
-++                if cmdutil.prompt("changelog for merge"):
-++                    mergestuff = "Patches applied:\n"
-++                    mergestuff += pylon.changelog_for_merge(new_merges)
-++                else:
-++                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                 log.description += mergestuff
-+         log.save()
-+     try:
-+@@ -2172,29 +2010,6 @@
-+             os.remove(log.name)
-+         raise
-+ 
-+-def merge_summary(new_merges, tree_version):
-+-    if len(new_merges) == 0:
-+-        return ""
-+-    if len(new_merges) == 1:
-+-        summary = new_merges[0].summary
-+-    else:
-+-        summary = "Merge"
-+-
-+-    credits = []
-+-    for merge in new_merges:
-+-        if arch.my_id() != merge.creator:
-+-            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+-            if not name in credits:
-+-                credits.append(name)
-+-        else:
-+-            version = merge.revision.version
-+-            if version.archive == tree_version.archive:
-+-                if not version.nonarch in credits:
-+-                    credits.append(version.nonarch)
-+-            elif not str(version) in credits:
-+-                credits.append(str(version))
-+-
-+-    return ("%s (%s)") % (summary, ", ".join(credits))
-+ 
-+ class MirrorArchive(BaseCommand):
-+     """
-+@@ -2268,31 +2083,73 @@
-+ 
-+ Use "alias" to list available (user and automatic) aliases."""
-+ 
-++auto_alias = [
-++"acur", 
-++"The latest revision in the archive of the tree-version.  You can specify \
-++a different version like so: acur:foo--bar--0 (aliases can be used)",
-++"tcur",
-++"""(tree current) The latest revision in the tree of the tree-version. \
-++You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-++used).""",
-++"tprev" , 
-++"""(tree previous) The previous revision in the tree of the tree-version.  To \
-++specify an older revision, use a number, e.g. "tprev:4" """,
-++"tanc" , 
-++"""(tree ancestor) The ancestor revision of the tree To specify an older \
-++revision, use a number, e.g. "tanc:4".""",
-++"tdate" , 
-++"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-++"tmod" , 
-++""" (tree modified) The latest revision to modify a given file, e.g. \
-++"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-++"ttag" , 
-++"""(tree tag) The revision that was tagged into the current tree revision, \
-++according to the tree""",
-++"tagcur", 
-++"""(tag current) The latest revision of the version that the current tree \
-++was tagged from.""",
-++"mergeanc" , 
-++"""The common ancestor of the current tree and the specified revision. \
-++Defaults to the first partner-version's latest revision or to tagcur.""",
-++]
-++
-++
-++def is_auto_alias(name):
-++    """Determine whether a name is an auto alias name
-++
-++    :param name: the name to check
-++    :type name: str
-++    :return: True if the name is an auto alias, false if not
-++    :rtype: bool
-++    """
-++    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-++
-++
-++def display_def(iter, wrap = 80):
-++    """Display a list of definitions
-++
-++    :param iter: iter of name, definition pairs
-++    :type iter: iter of (str, str)
-++    :param wrap: The width for text wrapping
-++    :type wrap: int
-++    """
-++    vals = list(iter)
-++    maxlen = 0
-++    for (key, value) in vals:
-++        if len(key) > maxlen:
-++            maxlen = len(key)
-++    for (key, value) in vals:
-++        tw=textwrap.TextWrapper(width=wrap, 
-++                                initial_indent=key.rjust(maxlen)+" : ",
-++                                subsequent_indent="".rjust(maxlen+3))
-++        print tw.fill(value)
-++
-++
-+ def help_aliases(tree):
-+-    print """Auto-generated aliases
-+- acur : The latest revision in the archive of the tree-version.  You can specfy
-+-        a different version like so: acur:foo--bar--0 (aliases can be used)
-+- tcur : (tree current) The latest revision in the tree of the tree-version.
-+-        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+-        can be used).
-+-tprev : (tree previous) The previous revision in the tree of the tree-version.
-+-        To specify an older revision, use a number, e.g. "tprev:4"
-+- tanc : (tree ancestor) The ancestor revision of the tree
-+-        To specify an older revision, use a number, e.g. "tanc:4"
-+-tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+- tmod : (tree modified) The latest revision to modify a given file 
-+-        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+- ttag : (tree tag) The revision that was tagged into the current tree revision,
-+-        according to the tree.
-+-tagcur: (tag current) The latest revision of the version that the current tree
-+-        was tagged from.
-+-mergeanc : The common ancestor of the current tree and the specified revision.
-+-        Defaults to the first partner-version's latest revision or to tagcur.
-+-   """
-++    print """Auto-generated aliases"""
-++    display_def(pylon.util.iter_pairs(auto_alias))
-+     print "User aliases"
-+-    for parts in ancillary.iter_all_alias(tree):
-+-        print parts[0].rjust(10)+" : "+parts[1]
-+-
-++    display_def(ancillary.iter_all_alias(tree))
-+ 
-+ class Inventory(BaseCommand):
-+     """List the status of files in the tree"""
-+@@ -2428,6 +2285,11 @@
-+         except cmdutil.ForbiddenAliasSyntax, e:
-+             raise CommandFailedWrapper(e)
-+ 
-++    def no_prefix(self, alias):
-++        if alias.startswith("^"):
-++            alias = alias[1:]
-++        return alias
-++        
-+     def arg_dispatch(self, args, options):
-+         """Add, modify, or list aliases, depending on number of arguments
-+ 
-+@@ -2438,15 +2300,20 @@
-+         if len(args) == 0:
-+             help_aliases(self.tree)
-+             return
-+-        elif len(args) == 1:
-+-            self.print_alias(args[0])
-+-        elif (len(args)) == 2:
-+-            self.add(args[0], args[1], options)
-+         else:
-+-            raise cmdutil.GetHelp
-++            alias = self.no_prefix(args[0])
-++            if len(args) == 1:
-++                self.print_alias(alias)
-++            elif (len(args)) == 2:
-++                self.add(alias, args[1], options)
-++            else:
-++                raise cmdutil.GetHelp
-+ 
-+     def print_alias(self, alias):
-+         answer = None
-++        if is_auto_alias(alias):
-++            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-++                "  Use \"revision\" to expand auto aliases." % alias)
-+         for pair in ancillary.iter_all_alias(self.tree):
-+             if pair[0] == alias:
-+                 answer = pair[1]
-+@@ -2464,6 +2331,8 @@
-+         :type expansion: str
-+         :param options: The commandline options
-+         """
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         written = False
-+         new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+@@ -2490,14 +2359,17 @@
-+         deleted = False
-+         if len(args) != 1:
-+             raise cmdutil.GetHelp
-++        alias = self.no_prefix(args[0])
-++        if is_auto_alias(alias):
-++            raise IsAutoAlias(alias)
-+         newlist = ""
-+         for pair in self.get_iterator(options):
-+-            if pair[0] != args[0]:
-++            if pair[0] != alias:
-+                 newlist+="%s=%s\n" % (pair[0], pair[1])
-+             else:
-+                 deleted = True
-+         if not deleted:
-+-            raise errors.NoSuchAlias(args[0])
-++            raise errors.NoSuchAlias(alias)
-+         self.write_aliases(newlist, options)
-+ 
-+     def get_alias_file(self, options):
-+@@ -2526,7 +2398,7 @@
-+         :param options: The commandline options
-+         """
-+         filename = os.path.expanduser(self.get_alias_file(options))
-+-        file = cmdutil.NewFileVersion(filename)
-++        file = util.NewFileVersion(filename)
-+         file.write(newlist)
-+         file.commit()
-+ 
-+@@ -2588,10 +2460,13 @@
-+         :param cmdargs: The commandline arguments
-+         :type cmdargs: list of str
-+         """
-+-        cmdutil.find_editor()
-+         parser = self.get_parser()
-+         (options, args) = parser.parse_args(cmdargs)
-+         try:
-++            cmdutil.find_editor()
-++        except pylon.errors.NoEditorSpecified, e:
-++            raise pylon.errors.CommandFailedWrapper(e)
-++        try:
-+             self.tree=arch.tree_root()
-+         except:
-+             self.tree=None
-+@@ -2655,7 +2530,7 @@
-+             target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                               args[0])
-+         else:
-+-            target_revision = cmdutil.tree_latest(self.tree)
-++            target_revision = arch_compound.tree_latest(self.tree)
-+         if len(args) > 1:
-+             merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                        self.tree, f)) for f in args[1:] ]
-+@@ -2711,7 +2586,7 @@
-+ 
-+         :param message: The message to send
-+         :type message: `email.Message`"""
-+-        server = smtplib.SMTP()
-++        server = smtplib.SMTP("localhost")
-+         server.sendmail(message['From'], message['To'], message.as_string())
-+         server.quit()
-+ 
-+@@ -2763,6 +2638,22 @@
-+ 'alias' : Alias,
-+ 'request-merge': RequestMerge,
-+ }
-++
-++def my_import(mod_name):
-++    module = __import__(mod_name)
-++    components = mod_name.split('.')
-++    for comp in components[1:]:
-++        module = getattr(module, comp)
-++    return module
-++
-++def plugin(mod_name):
-++    module = my_import(mod_name)
-++    module.add_command(commands)
-++
-++for file in os.listdir(sys.path[0]+"/command"):
-++    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-++        plugin("command."+file[:-3])
-++
-+ suggestions = {
-+ 'apply-delta' : "Try \"apply-changes\".",
-+ 'delta' : "To compare two revisions, use \"changes\".",
-+@@ -2784,6 +2675,7 @@
-+ 'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+ 'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+ 'library-revisions' : "Use revisions --library",
-+-'file-revert' : "Use revert FILE"
-++'file-revert' : "Use revert FILE",
-++'join-branch' : "Use replay --logs-only"
-+ }
-+ # arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/insert_top.patch'
---- /dev/null 
-+++ testdata/insert_top.patch 
-@@ -0,0 +1,7 @@
-+--- orig/pylon/patches.py
-++++ mod/pylon/patches.py
-+@@ -1,3 +1,4 @@
-++#test
-+ import util
-+ import sys
-+ class PatchSyntax(Exception):
-
-*** added file 'testdata/mod'
---- /dev/null 
-+++ testdata/mod 
-@@ -0,0 +1,2681 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+
-+import pylon.errors
-+from pylon.errors import *
-+from pylon import errors
-+from pylon import util
-+from pylon import arch_core
-+from pylon import arch_compound
-+from pylon import ancillary
-+from pylon import misc
-+from pylon import paths 
-+
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import terminal
-+import email
-+import smtplib
-+import textwrap
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = ancillary.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=util.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = arch_compound.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if options.diffopts is not None:
-+                    if isinstance(b_spec, arch.Revision):
-+                        b_dir = b_spec.library_find()
-+                    else:
-+                        b_dir = b_spec
-+                    a_dir = a_spec.library_find()
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision = arch_compound.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        if options.version is None:
-+            return options, tree.tree_version, args
-+
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        ancestor = None
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+            ancestor = base
-+        else:
-+            base = ancillary.submit_revision(tree)
-+            ancestor = base
-+        if ancestor is None:
-+            ancestor = arch_compound.tree_latest(tree, version)
-+
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            last_revision = None
-+            if ancestor is None:
-+                if cmdutil.prompt("Import from commit"):
-+                    return do_import(version)
-+                else:
-+                    raise NoVersionLogs(version)
-+        try:
-+            arch_last_revision = version.iter_revisions(True).next()
-+        except StopIteration, e:
-+            arch_last_revision = None
-+ 
-+        if last_revision != arch_last_revision:
-+            print "Tree is not up to date with %s" % str(version)
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(ancestor):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False, version=version)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree, version)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False, version=version)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = ancillary.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = arch_compound.MungeOpts()
-+            munger.set_hunk_prompt(cmdutil.colorize, cmdutil.user_hunk_confirm,
-+                                   options.hunk_prompt)
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.arch_compound.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = arch_compound.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in arch_compound.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+        self.cl_revisions = []
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        if options.type == "default":
-+            options.type = "archive"
-+        try:
-+            iter = cmdutil.revision_iterator(self.tree, options.type, args, 
-+                                             options.reverse, options.modified,
-+                                             options.shallow)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        except cmdutil.CantDetermineVersion, e:
-+            raise CommandFailedWrapper(e)
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        try:
-+            for revision in iter:
-+                log = None
-+                if isinstance(revision, arch.Patchlog):
-+                    log = revision
-+                    revision=revision.revision
-+                out = options.display(revision)
-+                if out is not None:
-+                    print out
-+                if log is None and (options.summary or options.creator or 
-+                                    options.date or options.merges):
-+                    log = revision.patchlog
-+                if options.creator:
-+                    print "    %s" % log.creator
-+                if options.date:
-+                    print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+                if options.summary:
-+                    print "    %s" % log.summary
-+                if options.merges:
-+                    showed_title = False
-+                    for revision in log.merged_patches:
-+                        if not showed_title:
-+                            print "    Merged:"
-+                            showed_title = True
-+                        print "    %s" % revision
-+            if len(self.cl_revisions) > 0:
-+                print pylon.changelog_for_merge(self.cl_revisions)
-+        except pylon.errors.TreeRootNone:
-+            raise CommandFailedWrapper(
-+                Exception("This option can only be used in a project tree."))
-+
-+    def changelog_append(self, revision):
-+        if isinstance(revision, arch.Revision):
-+            revision=arch.Patchlog(revision)
-+        self.cl_revisions.append(revision)
-+   
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [version/revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+
-+        cmdutil.add_revision_iter_options(select)
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        format.add_option("--changelog", action="store_const", 
-+                         const=self.changelog_append, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+        self.cwd = os.getcwd()
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                prompt = pylon.alias_or_version(self.tree.tree_version, 
-+                                                self.tree, 
-+                                                full=False)
-+                if prompt is not None:
-+                    prompt = " " + prompt
-+            except:
-+                prompt = ""
-+        else:
-+            prompt = ""
-+        self.prompt = "Fai%s> " % prompt
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = pylon.alias_or_version(self.tree.tree_version, self.tree, 
-+                                             full=False)
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        line = os.path.expanduser(line)
-+        if os.path.isabs(line):
-+            newcwd = line
-+        else:
-+            newcwd = self.cwd+'/'+line
-+        newcwd = os.path.normpath(newcwd)
-+        try:
-+            os.chdir(newcwd)
-+            self.cwd = newcwd
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % errors.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = cmdutil.iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(cmdutil.iter_munged_completions(iter, arg, 
-+                                                                text))
-+                else:
-+                    return list(cmdutil.iter_munged_completions(
-+                        cmdutil.iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = cmdutil.iter_dir_completions(arg)
-+                iter = cmdutil.iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                iter = cmdutil.iter_file_completions(arg)
-+                return list(cmdutil.iter_munged_completions(iter, arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = arch_compound.tree_cwd(tree)
-+    tmpdir = util.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+            
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit" or method == "implicit":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "implicit":
-+            if method != "implicit":
-+                if not cmdutil.prompt("Implicit in other tree"):
-+                    if method == "explicit" or method == "tagline":
-+                        options.id_type == method
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit" \
-+                and method !="implicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline" or id_type == "implicit":
-+                for file in files:
-+                    try:
-+                        implicit = (id_type == "implicit")
-+                        cmdutil.add_tagline_or_explicit_id(file, False,
-+                                                           implicit)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--implicit", action="store_const", 
-+                         const="implicit", dest="id_type", 
-+                         help="Use an implicit id (deprecated)")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        return cmdutil.merge_completions(self.tree, arg, index)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(ancillary.comp_revision(self.tree)):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = arch_compound.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = arch_compound.merge_ancestor2(self.tree, other_tree, 
-+                                                         other_revision)
-+            elif action == "update":
-+                ancestor = arch_compound.tree_latest(self.tree, 
-+                                                     other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        try:
-+            edit_log(self.tree, self.tree.tree_version)
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree, version):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False, version=version)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True, version=version)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = pylon.log_template_path(tree)
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        comp_version = ancillary.comp_revision(tree).version
-+        new_merges = cmdutil.iter_new_merges(tree, comp_version)
-+        new_merges = cmdutil.direct_merges(new_merges)
-+        log["Summary"] = pylon.merge_summary(new_merges, 
-+                                         version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                if cmdutil.prompt("changelog for merge"):
-+                    mergestuff = "Patches applied:\n"
-+                    mergestuff += pylon.changelog_for_merge(new_merges)
-+                else:
-+                    mergestuff = cmdutil.log_for_merge(tree, comp_version)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+auto_alias = [
-+"acur", 
-+"The latest revision in the archive of the tree-version.  You can specify \
-+a different version like so: acur:foo--bar--0 (aliases can be used)",
-+"tcur",
-+"""(tree current) The latest revision in the tree of the tree-version. \
-+You can specify a different version like so: tcur:foo--bar--0 (aliases can be \
-+used).""",
-+"tprev" , 
-+"""(tree previous) The previous revision in the tree of the tree-version.  To \
-+specify an older revision, use a number, e.g. "tprev:4" """,
-+"tanc" , 
-+"""(tree ancestor) The ancestor revision of the tree To specify an older \
-+revision, use a number, e.g. "tanc:4".""",
-+"tdate" , 
-+"""(tree date) The latest revision from a given date, e.g. "tdate:July 6".""",
-+"tmod" , 
-+""" (tree modified) The latest revision to modify a given file, e.g. \
-+"tmod:engine.cpp" or "tmod:engine.cpp:16".""",
-+"ttag" , 
-+"""(tree tag) The revision that was tagged into the current tree revision, \
-+according to the tree""",
-+"tagcur", 
-+"""(tag current) The latest revision of the version that the current tree \
-+was tagged from.""",
-+"mergeanc" , 
-+"""The common ancestor of the current tree and the specified revision. \
-+Defaults to the first partner-version's latest revision or to tagcur.""",
-+]
-+
-+
-+def is_auto_alias(name):
-+    """Determine whether a name is an auto alias name
-+
-+    :param name: the name to check
-+    :type name: str
-+    :return: True if the name is an auto alias, false if not
-+    :rtype: bool
-+    """
-+    return name in [f for (f, v) in pylon.util.iter_pairs(auto_alias)]
-+
-+
-+def display_def(iter, wrap = 80):
-+    """Display a list of definitions
-+
-+    :param iter: iter of name, definition pairs
-+    :type iter: iter of (str, str)
-+    :param wrap: The width for text wrapping
-+    :type wrap: int
-+    """
-+    vals = list(iter)
-+    maxlen = 0
-+    for (key, value) in vals:
-+        if len(key) > maxlen:
-+            maxlen = len(key)
-+    for (key, value) in vals:
-+        tw=textwrap.TextWrapper(width=wrap, 
-+                                initial_indent=key.rjust(maxlen)+" : ",
-+                                subsequent_indent="".rjust(maxlen+3))
-+        print tw.fill(value)
-+
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases"""
-+    display_def(pylon.util.iter_pairs(auto_alias))
-+    print "User aliases"
-+    display_def(ancillary.iter_all_alias(tree))
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def no_prefix(self, alias):
-+        if alias.startswith("^"):
-+            alias = alias[1:]
-+        return alias
-+        
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        else:
-+            alias = self.no_prefix(args[0])
-+            if len(args) == 1:
-+                self.print_alias(alias)
-+            elif (len(args)) == 2:
-+                self.add(alias, args[1], options)
-+            else:
-+                raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        if is_auto_alias(alias):
-+            raise pylon.errors.IsAutoAlias(alias, "\"%s\" is an auto alias."
-+                "  Use \"revision\" to expand auto aliases." % alias)
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        alias = self.no_prefix(args[0])
-+        if is_auto_alias(alias):
-+            raise IsAutoAlias(alias)
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(alias)
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = util.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            cmdutil.find_editor()
-+        except pylon.errors.NoEditorSpecified, e:
-+            raise pylon.errors.CommandFailedWrapper(e)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = arch_compound.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP("localhost")
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+
-+def my_import(mod_name):
-+    module = __import__(mod_name)
-+    components = mod_name.split('.')
-+    for comp in components[1:]:
-+        module = getattr(module, comp)
-+    return module
-+
-+def plugin(mod_name):
-+    module = my_import(mod_name)
-+    module.add_command(commands)
-+
-+for file in os.listdir(sys.path[0]+"/command"):
-+    if len(file) > 3 and file[-3:] == ".py" and file != "__init__.py":
-+        plugin("command."+file[:-3])
-+
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE",
-+'join-branch' : "Use replay --logs-only"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** added file 'testdata/orig'
---- /dev/null 
-+++ testdata/orig 
-@@ -0,0 +1,2789 @@
-+# Copyright (C) 2004 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import arch
-+import arch.util
-+import arch.arch
-+import abacmds
-+import cmdutil
-+import shutil
-+import os
-+import options
-+import paths 
-+import time
-+import cmd
-+import readline
-+import re
-+import string
-+import arch_core
-+from errors import *
-+import errors
-+import terminal
-+import ancillary
-+import misc
-+import email
-+import smtplib
-+
-+__docformat__ = "restructuredtext"
-+__doc__ = "Implementation of user (sub) commands"
-+commands = {}
-+
-+def find_command(cmd):
-+    """
-+    Return an instance of a command type.  Return None if the type isn't
-+    registered.
-+
-+    :param cmd: the name of the command to look for
-+    :type cmd: the type of the command
-+    """
-+    if commands.has_key(cmd):
-+        return commands[cmd]()
-+    else:
-+        return None
-+
-+class BaseCommand:
-+    def __call__(self, cmdline):
-+        try:
-+            self.do_command(cmdline.split())
-+        except cmdutil.GetHelp, e:
-+            self.help()
-+        except Exception, e:
-+            print e
-+
-+    def get_completer(index):
-+        return None
-+
-+    def complete(self, args, text):
-+        """
-+        Returns a list of possible completions for the given text.
-+
-+        :param args: The complete list of arguments
-+        :type args: List of str
-+        :param text: text to complete (may be shorter than args[-1])
-+        :type text: str
-+        :rtype: list of str
-+        """
-+        matches = []
-+        candidates = None
-+
-+        if len(args) > 0: 
-+            realtext = args[-1]
-+        else:
-+            realtext = ""
-+
-+        try:
-+            parser=self.get_parser()
-+            if realtext.startswith('-'):
-+                candidates = parser.iter_options()
-+            else:
-+                (options, parsed_args) = parser.parse_args(args)
-+
-+                if len (parsed_args) > 0:
-+                    candidates = self.get_completer(parsed_args[-1], len(parsed_args) -1)
-+                else:
-+                    candidates = self.get_completer("", 0)
-+        except:
-+            pass
-+        if candidates is None:
-+            return
-+        for candidate in candidates:
-+            candidate = str(candidate)
-+            if candidate.startswith(realtext):
-+                matches.append(candidate[len(realtext)- len(text):])
-+        return matches
-+
-+
-+class Help(BaseCommand):
-+    """
-+    Lists commands, prints help messages.
-+    """
-+    def __init__(self):
-+        self.description="Prints help mesages"
-+        self.parser = None
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Prints a help message.
-+        """
-+        options, args = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+
-+        if options.native or options.suggestions or options.external:
-+            native = options.native
-+            suggestions = options.suggestions
-+            external = options.external
-+        else:
-+            native = True
-+            suggestions = False
-+            external = True
-+        
-+        if len(args) == 0:
-+            self.list_commands(native, suggestions, external)
-+            return
-+        elif len(args) == 1:
-+            command_help(args[0])
-+            return
-+
-+    def help(self):
-+        self.get_parser().print_help()
-+        print """
-+If no command is specified, commands are listed.  If a command is
-+specified, help for that command is listed.
-+        """
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        if self.parser is not None:
-+            return self.parser
-+        parser=cmdutil.CmdOptionParser("fai help [command]")
-+        parser.add_option("-n", "--native", action="store_true", 
-+                         dest="native", help="Show native commands")
-+        parser.add_option("-e", "--external", action="store_true", 
-+                         dest="external", help="Show external commands")
-+        parser.add_option("-s", "--suggest", action="store_true", 
-+                         dest="suggestions", help="Show suggestions")
-+        self.parser = parser
-+        return parser 
-+      
-+    def list_commands(self, native=True, suggest=False, external=True):
-+        """
-+        Lists supported commands.
-+
-+        :param native: list native, python-based commands
-+        :type native: bool
-+        :param external: list external aba-style commands
-+        :type external: bool
-+        """
-+        if native:
-+            print "Native Fai commands"
-+            keys=commands.keys()
-+            keys.sort()
-+            for k in keys:
-+                space=""
-+                for i in range(28-len(k)):
-+                    space+=" "
-+                print space+k+" : "+commands[k]().description
-+            print
-+        if suggest:
-+            print "Unavailable commands and suggested alternatives"
-+            key_list = suggestions.keys()
-+            key_list.sort()
-+            for key in key_list:
-+                print "%28s : %s" % (key, suggestions[key])
-+            print
-+        if external:
-+            fake_aba = abacmds.AbaCmds()
-+            if (fake_aba.abadir == ""):
-+                return
-+            print "External commands"
-+            fake_aba.list_commands()
-+            print
-+        if not suggest:
-+            print "Use help --suggest to list alternatives to tla and aba"\
-+                " commands."
-+        if options.tla_fallthrough and (native or external):
-+            print "Fai also supports tla commands."
-+
-+def command_help(cmd):
-+    """
-+    Prints help for a command.
-+
-+    :param cmd: The name of the command to print help for
-+    :type cmd: str
-+    """
-+    fake_aba = abacmds.AbaCmds()
-+    cmdobj = find_command(cmd)
-+    if cmdobj != None:
-+        cmdobj.help()
-+    elif suggestions.has_key(cmd):
-+        print "Not available\n" + suggestions[cmd]
-+    else:
-+        abacmd = fake_aba.is_command(cmd)
-+        if abacmd:
-+            abacmd.help()
-+        else:
-+            print "No help is available for \""+cmd+"\". Maybe try \"tla "+cmd+" -H\"?"
-+
-+
-+
-+class Changes(BaseCommand):
-+    """
-+    the "changes" command: lists differences between trees/revisions:
-+    """
-+    
-+    def __init__(self):
-+        self.description="Lists what files have changed in the project tree"
-+
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        tree=arch.tree_root()
-+        if len(args) == 0:
-+            a_spec = cmdutil.comp_revision(tree)
-+        else:
-+            a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        if len(args) == 2:
-+            b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+            cmdutil.ensure_archive_registered(b_spec.archive)
-+        else:
-+            b_spec=tree
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "changes" command.
-+        """
-+        try:
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        if options.changeset:
-+            changeset=options.changeset
-+            tmpdir = None
-+        else:
-+            tmpdir=cmdutil.tmpdir()
-+            changeset=tmpdir+"/changeset"
-+        try:
-+            delta=arch.iter_delta(a_spec, b_spec, changeset)
-+            try:
-+                for line in delta:
-+                    if cmdutil.chattermatch(line, "changeset:"):
-+                        pass
-+                    else:
-+                        cmdutil.colorize(line, options.suppress_chatter)
-+            except arch.util.ExecProblem, e:
-+                if e.proc.error and e.proc.error.startswith(
-+                    "missing explicit id for file"):
-+                    raise MissingID(e)
-+                else:
-+                    raise
-+            status=delta.status
-+            if status > 1:
-+                return
-+            if (options.perform_diff):
-+                chan = cmdutil.ChangesetMunger(changeset)
-+                chan.read_indices()
-+                if isinstance(b_spec, arch.Revision):
-+                    b_dir = b_spec.library_find()
-+                else:
-+                    b_dir = b_spec
-+                a_dir = a_spec.library_find()
-+                if options.diffopts is not None:
-+                    diffopts = options.diffopts.split()
-+                    cmdutil.show_custom_diffs(chan, diffopts, a_dir, b_dir)
-+                else:
-+                    cmdutil.show_diffs(delta.changeset)
-+        finally:
-+            if tmpdir and (os.access(tmpdir, os.X_OK)):
-+                shutil.rmtree(tmpdir)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai changes [options] [revision]"
-+                                       " [revision]")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        parser.add_option("--diffopts", dest="diffopts", 
-+                          help="Use the specified diff options", 
-+                          metavar="OPTIONS")
-+
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs source-tree comparisons
-+
-+If no revision is specified, the current project tree is compared to the
-+last-committed revision.  If one revision is specified, the current project
-+tree is compared to that revision.  If two revisions are specified, they are
-+compared to each other.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class ApplyChanges(BaseCommand):
-+    """
-+    Apply differences between two revisions to a tree
-+    """
-+    
-+    def __init__(self):
-+        self.description="Applies changes to a project tree"
-+    
-+    def get_completer(self, arg, index):
-+        if index > 1:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) != 2:
-+            raise cmdutil.GetHelp
-+
-+        a_spec = cmdutil.determine_revision_tree(tree, args[0])
-+        cmdutil.ensure_archive_registered(a_spec.archive)
-+        b_spec = cmdutil.determine_revision_tree(tree, args[1])
-+        cmdutil.ensure_archive_registered(b_spec.archive)
-+        return options, a_spec, b_spec
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that performs "apply-changes".
-+        """
-+        try:
-+            tree = arch.tree_root()
-+            options, a_spec, b_spec = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        delta=cmdutil.apply_delta(a_spec, b_spec, tree)
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line, options.suppress_chatter)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "apply-changes" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai apply-changes [options] revision"
-+                                       " revision")
-+        parser.add_option("-d", "--diff", action="store_true", 
-+                          dest="perform_diff", default=False, 
-+                          help="Show diffs in summary")
-+        parser.add_option("-c", "--changeset", dest="changeset", 
-+                          help="Store a changeset in the given directory", 
-+                          metavar="DIRECTORY")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Applies changes to a project tree
-+
-+Compares two revisions and applies the difference between them to the current
-+tree.
-+        """
-+        help_tree_spec() 
-+        return
-+
-+class Update(BaseCommand):
-+    """
-+    Updates a project tree to a given revision, preserving un-committed hanges. 
-+    """
-+    
-+    def __init__(self):
-+        self.description="Apply the latest changes to the current directory"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raises cmdutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+        if len(args) > 2:
-+            raise cmdutil.GetHelp
-+
-+        spec=None
-+        if len(args)>0:
-+            spec=args[0]
-+        revision=cmdutil.determine_revision_arch(tree, spec)
-+        cmdutil.ensure_archive_registered(revision.archive)
-+
-+        mirror_source = cmdutil.get_mirror_source(revision.archive)
-+        if mirror_source != None:
-+            if cmdutil.prompt("Mirror update"):
-+                cmd=cmdutil.mirror_archive(mirror_source, 
-+                    revision.archive, arch.NameParser(revision).get_package_version())
-+                for line in arch.chatter_classifier(cmd):
-+                    cmdutil.colorize(line, options.suppress_chatter)
-+
-+                revision=cmdutil.determine_revision_arch(tree, spec)
-+
-+        return options, revision 
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "update" command.
-+        """
-+        tree=arch.tree_root()
-+        try:
-+            options, to_revision = self.parse_commandline(cmdargs, tree);
-+        except cmdutil.CantDetermineRevision, e:
-+            print e
-+            return
-+        except arch.errors.TreeRootError, e:
-+            print e
-+            return
-+        from_revision=cmdutil.tree_latest(tree)
-+        if from_revision==to_revision:
-+            print "Tree is already up to date with:\n"+str(to_revision)+"."
-+            return
-+        cmdutil.ensure_archive_registered(from_revision.archive)
-+        cmd=cmdutil.apply_delta(from_revision, to_revision, tree,
-+            options.patch_forward)
-+        for line in cmdutil.iter_apply_delta_filter(cmd):
-+            cmdutil.colorize(line)
-+        if to_revision.version != tree.tree_version:
-+            if cmdutil.prompt("Update version"):
-+                tree.tree_version = to_revision.version
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "update" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai update [options]"
-+                                       " [revision/version]")
-+        parser.add_option("-f", "--forward", action="store_true", 
-+                          dest="patch_forward", default=False, 
-+                          help="pass the --forward option to 'patch'")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a revision or version is specified, that is used instead 
-+        """
-+        help_tree_spec() 
-+        return
-+
-+
-+class Commit(BaseCommand):
-+    """
-+    Create a revision based on the changes in the current tree.
-+    """
-+    
-+    def __init__(self):
-+        self.description="Write local changes to the archive"
-+
-+    def get_completer(self, arg, index):
-+        if arg is None:
-+            arg = ""
-+        return iter_modified_file_completions(arch.tree_root(), arg)
-+#        return iter_source_file_completions(arch.tree_root(), arg)
-+    
-+    def parse_commandline(self, cmdline, tree):
-+        """
-+        Parse commandline arguments.  Raise cmtutil.GetHelp if help is needed.
-+        
-+        :param cmdline: A list of arguments to parse
-+        :rtype: (options, Revision, Revision/WorkingTree)
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdline)
-+
-+        if len(args) == 0:
-+            args = None
-+        revision=cmdutil.determine_revision_arch(tree, options.version)
-+        return options, revision.get_version(), args
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "commit" command.
-+        """
-+        tree=arch.tree_root()
-+        options, version, files = self.parse_commandline(cmdargs, tree)
-+        if options.__dict__.has_key("base") and options.base:
-+            base = cmdutil.determine_revision_tree(tree, options.base)
-+        else:
-+            base = cmdutil.submit_revision(tree)
-+        
-+        writeversion=version
-+        archive=version.archive
-+        source=cmdutil.get_mirror_source(archive)
-+        allow_old=False
-+        writethrough="implicit"
-+
-+        if source!=None:
-+            if writethrough=="explicit" and \
-+                cmdutil.prompt("Writethrough"):
-+                writeversion=arch.Version(str(source)+"/"+str(version.get_nonarch()))
-+            elif writethrough=="none":
-+                raise CommitToMirror(archive)
-+
-+        elif archive.is_mirror:
-+            raise CommitToMirror(archive)
-+
-+        try:
-+            last_revision=tree.iter_logs(version, True).next().revision
-+        except StopIteration, e:
-+            if cmdutil.prompt("Import from commit"):
-+                return do_import(version)
-+            else:
-+                raise NoVersionLogs(version)
-+        if last_revision!=version.iter_revisions(True).next():
-+            if not cmdutil.prompt("Out of date"):
-+                raise OutOfDate
-+            else:
-+                allow_old=True
-+
-+        try:
-+            if not cmdutil.has_changed(version):
-+                if not cmdutil.prompt("Empty commit"):
-+                    raise EmptyCommit
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "missing explicit id for file"):
-+                raise MissingID(e)
-+            else:
-+                raise
-+        log = tree.log_message(create=False)
-+        if log is None:
-+            try:
-+                if cmdutil.prompt("Create log"):
-+                    edit_log(tree)
-+
-+            except cmdutil.NoEditorSpecified, e:
-+                raise CommandFailed(e)
-+            log = tree.log_message(create=False)
-+        if log is None: 
-+            raise NoLogMessage
-+        if log["Summary"] is None or len(log["Summary"].strip()) == 0:
-+            if not cmdutil.prompt("Omit log summary"):
-+                raise errors.NoLogSummary
-+        try:
-+            for line in tree.iter_commit(version, seal=options.seal_version,
-+                base=base, out_of_date_ok=allow_old, file_list=files):
-+                cmdutil.colorize(line, options.suppress_chatter)
-+
-+        except arch.util.ExecProblem, e:
-+            if e.proc.error and e.proc.error.startswith(
-+                "These files violate naming conventions:"):
-+                raise LintFailure(e.proc.error)
-+            else:
-+                raise
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "commit" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+
-+        parser=cmdutil.CmdOptionParser("fai commit [options] [file1]"
-+                                       " [file2...]")
-+        parser.add_option("--seal", action="store_true", 
-+                          dest="seal_version", default=False, 
-+                          help="seal this version")
-+        parser.add_option("-v", "--version", dest="version", 
-+                          help="Use the specified version", 
-+                          metavar="VERSION")
-+        parser.add_option("-s", "--silent", action="store_true", 
-+                          dest="suppress_chatter", default=False, 
-+                          help="Suppress chatter messages")
-+        if cmdutil.supports_switch("commit", "--base"):
-+            parser.add_option("--base", dest="base", help="", 
-+                              metavar="REVISION")
-+        return parser
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser is None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a working tree to the current archive revision
-+
-+If a version is specified, that is used instead 
-+        """
-+#        help_tree_spec() 
-+        return
-+
-+
-+
-+class CatLog(BaseCommand):
-+    """
-+    Print the log of a given file (from current tree)
-+    """
-+    def __init__(self):
-+        self.description="Prints the patch log for a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "cat-log" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            tree = None
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp()
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        log = None
-+        
-+        use_tree = (options.source == "tree" or \
-+            (options.source == "any" and tree))
-+        use_arch = (options.source == "archive" or options.source == "any")
-+        
-+        log = None
-+        if use_tree:
-+            for log in tree.iter_logs(revision.get_version()):
-+                if log.revision == revision:
-+                    break
-+                else:
-+                    log = None
-+        if log is None and use_arch:
-+            cmdutil.ensure_revision_exists(revision)
-+            log = arch.Patchlog(revision)
-+        if log is not None:
-+            for item in log.items():
-+                print "%s: %s" % item
-+            print log.description
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai cat-log [revision]")
-+        parser.add_option("--archive", action="store_const", dest="source",
-+                          const="archive", default="any",
-+                          help="Always get the log from the archive")
-+        parser.add_option("--tree", action="store_const", dest="source",
-+                          const="tree", help="Always get the log from the tree")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Prints the log for the specified revision
-+        """
-+        help_tree_spec()
-+        return
-+
-+class Revert(BaseCommand):
-+    """ Reverts a tree (or aspects of it) to a revision
-+    """
-+    def __init__(self):
-+        self.description="Reverts a tree (or aspects of it) to a revision "
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return iter_modified_file_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revert" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError, e:
-+            raise CommandFailed(e)
-+        spec=None
-+        if options.revision is not None:
-+            spec=options.revision
-+        try:
-+            if spec is not None:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.comp_revision(tree)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        munger = None
-+
-+        if options.file_contents or options.file_perms or options.deletions\
-+            or options.additions or options.renames or options.hunk_prompt:
-+            munger = cmdutil.MungeOpts()
-+            munger.hunk_prompt = options.hunk_prompt
-+
-+        if len(args) > 0 or options.logs or options.pattern_files or \
-+            options.control:
-+            if munger is None:
-+                munger = cmdutil.MungeOpts(True)
-+                munger.all_types(True)
-+        if len(args) > 0:
-+            t_cwd = cmdutil.tree_cwd(tree)
-+            for name in args:
-+                if len(t_cwd) > 0:
-+                    t_cwd += "/"
-+                name = "./" + t_cwd + name
-+                munger.add_keep_file(name);
-+
-+        if options.file_perms:
-+            munger.file_perms = True
-+        if options.file_contents:
-+            munger.file_contents = True
-+        if options.deletions:
-+            munger.deletions = True
-+        if options.additions:
-+            munger.additions = True
-+        if options.renames:
-+            munger.renames = True
-+        if options.logs:
-+            munger.add_keep_pattern('^\./\{arch\}/[^=].*')
-+        if options.control:
-+            munger.add_keep_pattern("/\.arch-ids|^\./\{arch\}|"\
-+                                    "/\.arch-inventory$")
-+        if options.pattern_files:
-+            munger.add_keep_pattern(options.pattern_files)
-+                
-+        for line in cmdutil.revert(tree, revision, munger, 
-+                                   not options.no_output):
-+            cmdutil.colorize(line)
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "cat-log" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revert [options] [FILE...]")
-+        parser.add_option("", "--contents", action="store_true", 
-+                          dest="file_contents", 
-+                          help="Revert file content changes")
-+        parser.add_option("", "--permissions", action="store_true", 
-+                          dest="file_perms", 
-+                          help="Revert file permissions changes")
-+        parser.add_option("", "--deletions", action="store_true", 
-+                          dest="deletions", 
-+                          help="Restore deleted files")
-+        parser.add_option("", "--additions", action="store_true", 
-+                          dest="additions", 
-+                          help="Remove added files")
-+        parser.add_option("", "--renames", action="store_true", 
-+                          dest="renames", 
-+                          help="Revert file names")
-+        parser.add_option("--hunks", action="store_true", 
-+                          dest="hunk_prompt", default=False,
-+                          help="Prompt which hunks to revert")
-+        parser.add_option("--pattern-files", dest="pattern_files", 
-+                          help="Revert files that match this pattern", 
-+                          metavar="REGEX")
-+        parser.add_option("--logs", action="store_true", 
-+                          dest="logs", default=False,
-+                          help="Revert only logs")
-+        parser.add_option("--control-files", action="store_true", 
-+                          dest="control", default=False,
-+                          help="Revert logs and other control files")
-+        parser.add_option("-n", "--no-output", action="store_true", 
-+                          dest="no_output", 
-+                          help="Don't keep an undo changeset")
-+        parser.add_option("--revision", dest="revision", 
-+                          help="Revert to the specified revision", 
-+                          metavar="REVISION")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Reverts changes in the current working tree.  If no flags are specified, all
-+types of changes are reverted.  Otherwise, only selected types of changes are
-+reverted.  
-+
-+If a revision is specified on the commandline, differences between the current
-+tree and that revision are reverted.  If a version is specified, the current
-+tree is used to determine the revision.
-+
-+If files are specified, only those files listed will have any changes applied.
-+To specify a renamed file, you can use either the old or new name. (or both!)
-+
-+Unless "-n" is specified, reversions can be undone with "redo".
-+        """
-+        return
-+
-+class Revision(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Prints the name of a revision"
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+
-+        spec=None
-+        if len(args) > 0:
-+            spec=args[0]
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            if tree:
-+                revision = cmdutil.determine_revision_tree(tree, spec)
-+            else:
-+                revision = cmdutil.determine_revision_arch(tree, spec)
-+        except cmdutil.CantDetermineRevision, e:
-+            print str(e)
-+            return
-+        print options.display(revision)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revision [revision]")
-+        parser.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        parser.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        parser.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        parser.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        parser.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        parser.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and prints the name of the specified revision.  Instead of
-+the name, several options can be used to print locations.  If more than one is
-+specified, the last one is used.
-+        """
-+        help_tree_spec()
-+        return
-+
-+def require_version_exists(version, spec):
-+    if not version.exists():
-+        raise cmdutil.CantDetermineVersion(spec, 
-+                                           "The version %s does not exist." \
-+                                           % version)
-+
-+class Revisions(BaseCommand):
-+    """
-+    Print a revision name based on a revision specifier
-+    """
-+    def __init__(self):
-+        self.description="Lists revisions"
-+    
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        (options, args) = self.get_parser().parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise cmdutil.GetHelp
-+        try:
-+            self.tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            self.tree = None
-+        try:
-+            iter = self.get_iterator(options.type, args, options.reverse, 
-+                                     options.modified)
-+        except cmdutil.CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        if options.skip is not None:
-+            iter = cmdutil.iter_skip(iter, int(options.skip))
-+
-+        for revision in iter:
-+            log = None
-+            if isinstance(revision, arch.Patchlog):
-+                log = revision
-+                revision=revision.revision
-+            print options.display(revision)
-+            if log is None and (options.summary or options.creator or 
-+                                options.date or options.merges):
-+                log = revision.patchlog
-+            if options.creator:
-+                print "    %s" % log.creator
-+            if options.date:
-+                print "    %s" % time.strftime('%Y-%m-%d %H:%M:%S %Z', log.date)
-+            if options.summary:
-+                print "    %s" % log.summary
-+            if options.merges:
-+                showed_title = False
-+                for revision in log.merged_patches:
-+                    if not showed_title:
-+                        print "    Merged:"
-+                        showed_title = True
-+                    print "    %s" % revision
-+
-+    def get_iterator(self, type, args, reverse, modified):
-+        if len(args) > 0:
-+            spec = args[0]
-+        else:
-+            spec = None
-+        if modified is not None:
-+            iter = cmdutil.modified_iter(modified, self.tree)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+        elif type == "archive":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return version.iter_revisions(reverse)
-+        elif type == "cacherevs":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+                cmdutil.ensure_archive_registered(version.archive)
-+                require_version_exists(version, spec)
-+            return cmdutil.iter_cacherevs(version, reverse)
-+        elif type == "library":
-+            if spec is None:
-+                if self.tree is None:
-+                    raise cmdutil.CantDetermineRevision("", 
-+                                                        "Not in a project tree")
-+                version = cmdutil.determine_version_tree(spec, self.tree)
-+            else:
-+                version = cmdutil.determine_version_arch(spec, self.tree)
-+            return version.iter_library_revisions(reverse)
-+        elif type == "logs":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            return self.tree.iter_logs(cmdutil.determine_version_tree(spec, \
-+                                  self.tree), reverse)
-+        elif type == "missing" or type == "skip-present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            skip = (type == "skip-present")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_missing(self.tree, version, reverse,
-+                                        skip_present=skip)
-+
-+        elif type == "present":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            return cmdutil.iter_present(self.tree, version, reverse)
-+
-+        elif type == "new-merges" or type == "direct-merges":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            version = cmdutil.determine_version_tree(spec, self.tree)
-+            cmdutil.ensure_archive_registered(version.archive)
-+            require_version_exists(version, spec)
-+            iter = cmdutil.iter_new_merges(self.tree, version, reverse)
-+            if type == "new-merges":
-+                return iter
-+            elif type == "direct-merges":
-+                return cmdutil.direct_merges(iter)
-+
-+        elif type == "missing-from":
-+            if self.tree is None:
-+                raise cmdutil.CantDetermineRevision("", "Not in a project tree")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            libtree = cmdutil.find_or_make_local_revision(revision)
-+            return cmdutil.iter_missing(libtree, self.tree.tree_version,
-+                                        reverse)
-+
-+        elif type == "partner-missing":
-+            return cmdutil.iter_partner_missing(self.tree, reverse)
-+
-+        elif type == "ancestry":
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            iter = cmdutil._iter_ancestry(self.tree, revision)
-+            if reverse:
-+                return iter
-+            else:
-+                return cmdutil.iter_reverse(iter)
-+
-+        elif type == "dependencies" or type == "non-dependencies":
-+            nondeps = (type == "non-dependencies")
-+            revision = cmdutil.determine_revision_tree(self.tree, spec)
-+            anc_iter = cmdutil._iter_ancestry(self.tree, revision)
-+            iter_depends = cmdutil.iter_depends(anc_iter, nondeps)
-+            if reverse:
-+                return iter_depends
-+            else:
-+                return cmdutil.iter_reverse(iter_depends)
-+        elif type == "micro":
-+            return cmdutil.iter_micro(self.tree)
-+
-+    
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai revisions [revision]")
-+        select = cmdutil.OptionGroup(parser, "Selection options",
-+                          "Control which revisions are listed.  These options"
-+                          " are mutually exclusive.  If more than one is"
-+                          " specified, the last is used.")
-+        select.add_option("", "--archive", action="store_const", 
-+                          const="archive", dest="type", default="archive",
-+                          help="List all revisions in the archive")
-+        select.add_option("", "--cacherevs", action="store_const", 
-+                          const="cacherevs", dest="type",
-+                          help="List all revisions stored in the archive as "
-+                          "complete copies")
-+        select.add_option("", "--logs", action="store_const", 
-+                          const="logs", dest="type",
-+                          help="List revisions that have a patchlog in the "
-+                          "tree")
-+        select.add_option("", "--missing", action="store_const", 
-+                          const="missing", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree")
-+        select.add_option("", "--skip-present", action="store_const", 
-+                          const="skip-present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlogs at all in the tree")
-+        select.add_option("", "--present", action="store_const", 
-+                          const="present", dest="type",
-+                          help="List revisions from the specified version that"
-+                          " have no patchlog in the tree, but can't be merged")
-+        select.add_option("", "--missing-from", action="store_const", 
-+                          const="missing-from", dest="type",
-+                          help="List revisions from the specified revision "
-+                          "that have no patchlog for the tree version")
-+        select.add_option("", "--partner-missing", action="store_const", 
-+                          const="partner-missing", dest="type",
-+                          help="List revisions in partner versions that are"
-+                          " missing")
-+        select.add_option("", "--new-merges", action="store_const", 
-+                          const="new-merges", dest="type",
-+                          help="List revisions that have had patchlogs added"
-+                          " to the tree since the last commit")
-+        select.add_option("", "--direct-merges", action="store_const", 
-+                          const="direct-merges", dest="type",
-+                          help="List revisions that have been directly added"
-+                          " to tree since the last commit ")
-+        select.add_option("", "--library", action="store_const", 
-+                          const="library", dest="type",
-+                          help="List revisions in the revision library")
-+        select.add_option("", "--ancestry", action="store_const", 
-+                          const="ancestry", dest="type",
-+                          help="List revisions that are ancestors of the "
-+                          "current tree version")
-+
-+        select.add_option("", "--dependencies", action="store_const", 
-+                          const="dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "depends on")
-+
-+        select.add_option("", "--non-dependencies", action="store_const", 
-+                          const="non-dependencies", dest="type",
-+                          help="List revisions that the given revision "
-+                          "does not depend on")
-+
-+        select.add_option("--micro", action="store_const", 
-+                          const="micro", dest="type",
-+                          help="List partner revisions aimed for this "
-+                          "micro-branch")
-+
-+        select.add_option("", "--modified", dest="modified", 
-+                          help="List tree ancestor revisions that modified a "
-+                          "given file", metavar="FILE[:LINE]")
-+
-+        parser.add_option("", "--skip", dest="skip", 
-+                          help="Skip revisions.  Positive numbers skip from "
-+                          "beginning, negative skip from end.",
-+                          metavar="NUMBER")
-+
-+        parser.add_option_group(select)
-+
-+        format = cmdutil.OptionGroup(parser, "Revision format options",
-+                          "These control the appearance of listed revisions")
-+        format.add_option("", "--location", action="store_const", 
-+                         const=paths.determine_path, dest="display", 
-+                         help="Show location instead of name", default=str)
-+        format.add_option("--import", action="store_const", 
-+                         const=paths.determine_import_path, dest="display",  
-+                         help="Show location of import file")
-+        format.add_option("--log", action="store_const", 
-+                         const=paths.determine_log_path, dest="display", 
-+                         help="Show location of log file")
-+        format.add_option("--patch", action="store_const", 
-+                         dest="display", const=paths.determine_patch_path,
-+                         help="Show location of patchfile")
-+        format.add_option("--continuation", action="store_const", 
-+                         const=paths.determine_continuation_path, 
-+                         dest="display",
-+                         help="Show location of continuation file")
-+        format.add_option("--cacherev", action="store_const", 
-+                         const=paths.determine_cacherev_path, dest="display",
-+                         help="Show location of cacherev file")
-+        parser.add_option_group(format)
-+        display = cmdutil.OptionGroup(parser, "Display format options",
-+                          "These control the display of data")
-+        display.add_option("-r", "--reverse", action="store_true", 
-+                          dest="reverse", help="Sort from newest to oldest")
-+        display.add_option("-s", "--summary", action="store_true", 
-+                          dest="summary", help="Show patchlog summary")
-+        display.add_option("-D", "--date", action="store_true", 
-+                          dest="date", help="Show patchlog date")
-+        display.add_option("-c", "--creator", action="store_true", 
-+                          dest="creator", help="Show the id that committed the"
-+                          " revision")
-+        display.add_option("-m", "--merges", action="store_true", 
-+                          dest="merges", help="Show the revisions that were"
-+                          " merged")
-+        parser.add_option_group(display)
-+        return parser 
-+    def help(self, parser=None):
-+        """Attempt to explain the revisions command
-+        
-+        :param parser: If supplied, used to determine options
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """List revisions.
-+        """
-+        help_tree_spec()
-+
-+
-+class Get(BaseCommand):
-+    """
-+    Retrieve a revision from the archive
-+    """
-+    def __init__(self):
-+        self.description="Retrieve a revision from the archive"
-+        self.parser=self.get_parser()
-+
-+
-+    def get_completer(self, arg, index):
-+        if index > 0:
-+            return None
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+        return cmdutil.iter_revision_completions(arg, tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "get" command.
-+        """
-+        (options, args) = self.parser.parse_args(cmdargs)
-+        if len(args) < 1:
-+            return self.help()            
-+        try:
-+            tree = arch.tree_root()
-+        except arch.errors.TreeRootError:
-+            tree = None
-+        
-+        arch_loc = None
-+        try:
-+            revision, arch_loc = paths.full_path_decode(args[0])
-+        except Exception, e:
-+            revision = cmdutil.determine_revision_arch(tree, args[0], 
-+                check_existence=False, allow_package=True)
-+        if len(args) > 1:
-+            directory = args[1]
-+        else:
-+            directory = str(revision.nonarch)
-+        if os.path.exists(directory):
-+            raise DirectoryExists(directory)
-+        cmdutil.ensure_archive_registered(revision.archive, arch_loc)
-+        try:
-+            cmdutil.ensure_revision_exists(revision)
-+        except cmdutil.NoSuchRevision, e:
-+            raise CommandFailedWrapper(e)
-+
-+        link = cmdutil.prompt ("get link")
-+        for line in cmdutil.iter_get(revision, directory, link,
-+                                     options.no_pristine,
-+                                     options.no_greedy_add):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "get" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai get revision [dir]")
-+        parser.add_option("--no-pristine", action="store_true", 
-+                         dest="no_pristine", 
-+                         help="Do not make pristine copy for reference")
-+        parser.add_option("--no-greedy-add", action="store_true", 
-+                         dest="no_greedy_add", 
-+                         help="Never add to greedy libraries")
-+
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Expands aliases and constructs a project tree for a revision.  If the optional
-+"dir" argument is provided, the project tree will be stored in this directory.
-+        """
-+        help_tree_spec()
-+        return
-+
-+class PromptCmd(cmd.Cmd):
-+    def __init__(self):
-+        cmd.Cmd.__init__(self)
-+        self.prompt = "Fai> "
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+        self.set_title()
-+        self.set_prompt()
-+        self.fake_aba = abacmds.AbaCmds()
-+        self.identchars += '-'
-+        self.history_file = os.path.expanduser("~/.fai-history")
-+        readline.set_completer_delims(string.whitespace)
-+        if os.access(self.history_file, os.R_OK) and \
-+            os.path.isfile(self.history_file):
-+            readline.read_history_file(self.history_file)
-+
-+    def write_history(self):
-+        readline.write_history_file(self.history_file)
-+
-+    def do_quit(self, args):
-+        self.write_history()
-+        sys.exit(0)
-+
-+    def do_exit(self, args):
-+        self.do_quit(args)
-+
-+    def do_EOF(self, args):
-+        print
-+        self.do_quit(args)
-+
-+    def postcmd(self, line, bar):
-+        self.set_title()
-+        self.set_prompt()
-+
-+    def set_prompt(self):
-+        if self.tree is not None:
-+            try:
-+                version = " "+self.tree.tree_version.nonarch
-+            except:
-+                version = ""
-+        else:
-+            version = ""
-+        self.prompt = "Fai%s> " % version
-+
-+    def set_title(self, command=None):
-+        try:
-+            version = self.tree.tree_version.nonarch
-+        except:
-+            version = "[no version]"
-+        if command is None:
-+            command = ""
-+        sys.stdout.write(terminal.term_title("Fai %s %s" % (command, version)))
-+
-+    def do_cd(self, line):
-+        if line == "":
-+            line = "~"
-+        try:
-+            os.chdir(os.path.expanduser(line))
-+        except Exception, e:
-+            print e
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+    def do_help(self, line):
-+        Help()(line)
-+
-+    def default(self, line):
-+        args = line.split()
-+        if find_command(args[0]):
-+            try:
-+                find_command(args[0]).do_command(args[1:])
-+            except cmdutil.BadCommandOption, e:
-+                print e
-+            except cmdutil.GetHelp, e:
-+                find_command(args[0]).help()
-+            except CommandFailed, e:
-+                print e
-+            except arch.errors.ArchiveNotRegistered, e:
-+                print e
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+            except arch.util.ExecProblem, e:
-+                print e.proc.error.rstrip('\n')
-+            except cmdutil.CantDetermineVersion, e:
-+                print e
-+            except cmdutil.CantDetermineRevision, e:
-+                print e
-+            except Exception, e:
-+                print "Unhandled error:\n%s" % cmdutil.exception_str(e)
-+
-+        elif suggestions.has_key(args[0]):
-+            print suggestions[args[0]]
-+
-+        elif self.fake_aba.is_command(args[0]):
-+            tree = None
-+            try:
-+                tree = arch.tree_root()
-+            except arch.errors.TreeRootError:
-+                pass
-+            cmd = self.fake_aba.is_command(args[0])
-+            try:
-+                cmd.run(cmdutil.expand_prefix_alias(args[1:], tree))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+        elif options.tla_fallthrough and args[0] != "rm" and \
-+            cmdutil.is_tla_command(args[0]):
-+            try:
-+                tree = None
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    pass
-+                args = cmdutil.expand_prefix_alias(args, tree)
-+                arch.util.exec_safe('tla', args, stderr=sys.stderr,
-+                expected=(0, 1))
-+            except arch.util.ExecProblem, e:
-+                pass
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+        else:
-+            try:
-+                try:
-+                    tree = arch.tree_root()
-+                except arch.errors.TreeRootError:
-+                    tree = None
-+                args=line.split()
-+                os.system(" ".join(cmdutil.expand_prefix_alias(args, tree)))
-+            except KeyboardInterrupt, e:
-+                print "Interrupted"
-+
-+    def completenames(self, text, line, begidx, endidx):
-+        completions = []
-+        iter = iter_command_names(self.fake_aba)
-+        try:
-+            if len(line) > 0:
-+                arg = line.split()[-1]
-+            else:
-+                arg = ""
-+            iter = iter_munged_completions(iter, arg, text)
-+        except Exception, e:
-+            print e
-+        return list(iter)
-+
-+    def completedefault(self, text, line, begidx, endidx):
-+        """Perform completion for native commands.
-+        
-+        :param text: The text to complete
-+        :type text: str
-+        :param line: The entire line to complete
-+        :type line: str
-+        :param begidx: The start of the text in the line
-+        :type begidx: int
-+        :param endidx: The end of the text in the line
-+        :type endidx: int
-+        """
-+        try:
-+            (cmd, args, foo) = self.parseline(line)
-+            command_obj=find_command(cmd)
-+            if command_obj is not None:
-+                return command_obj.complete(args.split(), text)
-+            elif not self.fake_aba.is_command(cmd) and \
-+                cmdutil.is_tla_command(cmd):
-+                iter = cmdutil.iter_supported_switches(cmd)
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                if arg.startswith("-"):
-+                    return list(iter_munged_completions(iter, arg, text))
-+                else:
-+                    return list(iter_munged_completions(
-+                        iter_file_completions(arg), arg, text))
-+
-+
-+            elif cmd == "cd":
-+                if len(args) > 0:
-+                    arg = args.split()[-1]
-+                else:
-+                    arg = ""
-+                iter = iter_dir_completions(arg)
-+                iter = iter_munged_completions(iter, arg, text)
-+                return list(iter)
-+            elif len(args)>0:
-+                arg = args.split()[-1]
-+                return list(iter_munged_completions(iter_file_completions(arg),
-+                                                    arg, text))
-+            else:
-+                return self.completenames(text, line, begidx, endidx)
-+        except Exception, e:
-+            print e
-+
-+
-+def iter_command_names(fake_aba):
-+    for entry in cmdutil.iter_combine([commands.iterkeys(), 
-+                                     fake_aba.get_commands(), 
-+                                     cmdutil.iter_tla_commands(False)]):
-+        if not suggestions.has_key(str(entry)):
-+            yield entry
-+
-+
-+def iter_file_completions(arg, only_dirs = False):
-+    """Generate an iterator that iterates through filename completions.
-+
-+    :param arg: The filename fragment to match
-+    :type arg: str
-+    :param only_dirs: If true, match only directories
-+    :type only_dirs: bool
-+    """
-+    cwd = os.getcwd()
-+    if cwd != "/":
-+        extras = [".", ".."]
-+    else:
-+        extras = []
-+    (dir, file) = os.path.split(arg)
-+    if dir != "":
-+        listingdir = os.path.expanduser(dir)
-+    else:
-+        listingdir = cwd
-+    for file in cmdutil.iter_combine([os.listdir(listingdir), extras]):
-+        if dir != "":
-+            userfile = dir+'/'+file
-+        else:
-+            userfile = file
-+        if userfile.startswith(arg):
-+            if os.path.isdir(listingdir+'/'+file):
-+                userfile+='/'
-+                yield userfile
-+            elif not only_dirs:
-+                yield userfile
-+
-+def iter_munged_completions(iter, arg, text):
-+    for completion in iter:
-+        completion = str(completion)
-+        if completion.startswith(arg):
-+            yield completion[len(arg)-len(text):]
-+
-+def iter_source_file_completions(tree, arg):
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+    for file in tree.iter_inventory(dirs, source=True, both=True):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def iter_untagged(tree, dirs):
-+    for file in arch_core.iter_inventory_filter(tree, dirs, tagged=False, 
-+                                                categories=arch_core.non_root,
-+                                                control_files=True):
-+        yield file.name 
-+
-+
-+def iter_untagged_completions(tree, arg):
-+    """Generate an iterator for all visible untagged files that match arg.
-+
-+    :param tree: The tree to look for untagged files in
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The argument to match
-+    :type arg: str
-+    :return: An iterator of all matching untagged files
-+    :rtype: iterator of str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    if len(treepath) > 0:
-+        dirs = [treepath]
-+    else:
-+        dirs = None
-+
-+    for file in iter_untagged(tree, dirs):
-+        file = file_completion_match(file, treepath, arg)
-+        if file is not None:
-+            yield file
-+
-+
-+def file_completion_match(file, treepath, arg):
-+    """Determines whether a file within an arch tree matches the argument.
-+
-+    :param file: The rooted filename
-+    :type file: str
-+    :param treepath: The path to the cwd within the tree
-+    :type treepath: str
-+    :param arg: The prefix to match
-+    :return: The completion name, or None if not a match
-+    :rtype: str
-+    """
-+    if not file.startswith(treepath):
-+        return None
-+    if treepath != "":
-+        file = file[len(treepath)+1:]
-+
-+    if not file.startswith(arg):
-+        return None 
-+    if os.path.isdir(file):
-+        file += '/'
-+    return file
-+
-+def iter_modified_file_completions(tree, arg):
-+    """Returns a list of modified files that match the specified prefix.
-+
-+    :param tree: The current tree
-+    :type tree: `arch.WorkingTree`
-+    :param arg: The prefix to match
-+    :type arg: str
-+    """
-+    treepath = cmdutil.tree_cwd(tree)
-+    tmpdir = cmdutil.tmpdir()
-+    changeset = tmpdir+"/changeset"
-+    completions = []
-+    revision = cmdutil.determine_revision_tree(tree)
-+    for line in arch.iter_delta(revision, tree, changeset):
-+        if isinstance(line, arch.FileModification):
-+            file = file_completion_match(line.name[1:], treepath, arg)
-+            if file is not None:
-+                completions.append(file)
-+    shutil.rmtree(tmpdir)
-+    return completions
-+
-+def iter_dir_completions(arg):
-+    """Generate an iterator that iterates through directory name completions.
-+
-+    :param arg: The directory name fragment to match
-+    :type arg: str
-+    """
-+    return iter_file_completions(arg, True)
-+
-+class Shell(BaseCommand):
-+    def __init__(self):
-+        self.description = "Runs Fai as a shell"
-+
-+    def do_command(self, cmdargs):
-+        if len(cmdargs)!=0:
-+            raise cmdutil.GetHelp
-+        prompt = PromptCmd()
-+        try:
-+            prompt.cmdloop()
-+        finally:
-+            prompt.write_history()
-+
-+class AddID(BaseCommand):
-+    """
-+    Adds an inventory id for the given file
-+    """
-+    def __init__(self):
-+        self.description="Add an inventory id for a given file"
-+
-+    def get_completer(self, arg, index):
-+        tree = arch.tree_root()
-+        return iter_untagged_completions(tree, arg)
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+
-+        tree = arch.tree_root()
-+
-+        if (len(args) == 0) == (options.untagged == False):
-+            raise cmdutil.GetHelp
-+
-+       #if options.id and len(args) != 1:
-+       #    print "If --id is specified, only one file can be named."
-+       #    return
-+        
-+        method = tree.tagging_method
-+        
-+        if options.id_type == "tagline":
-+            if method != "tagline":
-+                if not cmdutil.prompt("Tagline in other tree"):
-+                    if method == "explicit":
-+                        options.id_type == explicit
-+                    else:
-+                        print "add-id not supported for \"%s\" tagging method"\
-+                            % method 
-+                        return
-+        
-+        elif options.id_type == "explicit":
-+            if method != "tagline" and method != explicit:
-+                if not prompt("Explicit in other tree"):
-+                    print "add-id not supported for \"%s\" tagging method" % \
-+                        method
-+                    return
-+        
-+        if options.id_type == "auto":
-+            if method != "tagline" and method != "explicit":
-+                print "add-id not supported for \"%s\" tagging method" % method
-+                return
-+            else:
-+                options.id_type = method
-+        if options.untagged:
-+            args = None
-+        self.add_ids(tree, options.id_type, args)
-+
-+    def add_ids(self, tree, id_type, files=()):
-+        """Add inventory ids to files.
-+        
-+        :param tree: the tree the files are in
-+        :type tree: `arch.WorkingTree`
-+        :param id_type: the type of id to add: "explicit" or "tagline"
-+        :type id_type: str
-+        :param files: The list of files to add.  If None do all untagged.
-+        :type files: tuple of str
-+        """
-+
-+        untagged = (files is None)
-+        if untagged:
-+            files = list(iter_untagged(tree, None))
-+        previous_files = []
-+        while len(files) > 0:
-+            previous_files.extend(files)
-+            if id_type == "explicit":
-+                cmdutil.add_id(files)
-+            elif id_type == "tagline":
-+                for file in files:
-+                    try:
-+                        cmdutil.add_tagline_or_explicit_id(file)
-+                    except cmdutil.AlreadyTagged:
-+                        print "\"%s\" already has a tagline." % file
-+                    except cmdutil.NoCommentSyntax:
-+                        pass
-+            #do inventory after tagging until no untagged files are encountered
-+            if untagged:
-+                files = []
-+                for file in iter_untagged(tree, None):
-+                    if not file in previous_files:
-+                        files.append(file)
-+
-+            else:
-+                break
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai add-id file1 [file2] [file3]...")
-+# ddaa suggests removing this to promote GUIDs.  Let's see who squalks.
-+#        parser.add_option("-i", "--id", dest="id", 
-+#                         help="Specify id for a single file", default=None)
-+        parser.add_option("--tltl", action="store_true", 
-+                         dest="lord_style",  help="Use Tom Lord's style of id.")
-+        parser.add_option("--explicit", action="store_const", 
-+                         const="explicit", dest="id_type", 
-+                         help="Use an explicit id", default="auto")
-+        parser.add_option("--tagline", action="store_const", 
-+                         const="tagline", dest="id_type", 
-+                         help="Use a tagline id")
-+        parser.add_option("--untagged", action="store_true", 
-+                         dest="untagged", default=False, 
-+                         help="tag all untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Adds an inventory to the specified file(s) and directories.  If --untagged is
-+specified, adds inventory to all untagged files and directories.
-+        """
-+        return
-+
-+
-+class Merge(BaseCommand):
-+    """
-+    Merges changes from other versions into the current tree
-+    """
-+    def __init__(self):
-+        self.description="Merges changes from other versions"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def get_completer(self, arg, index):
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+        completions = list(ancillary.iter_partners(self.tree, 
-+                                                   self.tree.tree_version))
-+        if len(completions) == 0:
-+            completions = list(self.tree.iter_log_versions())
-+
-+        aliases = []
-+        try:
-+            for completion in completions:
-+                alias = ancillary.compact_alias(str(completion), self.tree)
-+                if alias:
-+                    aliases.extend(alias)
-+
-+            for completion in completions:
-+                if completion.archive == self.tree.tree_version.archive:
-+                    aliases.append(completion.nonarch)
-+
-+        except Exception, e:
-+            print e
-+            
-+        completions.extend(aliases)
-+        return completions
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "merge" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if options.diff3:
-+            action="star-merge"
-+        else:
-+            action = options.action
-+        
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError(os.getcwd())
-+        if cmdutil.has_changed(self.tree.tree_version):
-+            raise UncommittedChanges(self.tree)
-+
-+        if len(args) > 0:
-+            revisions = []
-+            for arg in args:
-+                revisions.append(cmdutil.determine_revision_arch(self.tree, 
-+                                                                 arg))
-+            source = "from commandline"
-+        else:
-+            revisions = ancillary.iter_partner_revisions(self.tree, 
-+                                                         self.tree.tree_version)
-+            source = "from partner version"
-+        revisions = misc.rewind_iterator(revisions)
-+        try:
-+            revisions.next()
-+            revisions.rewind()
-+        except StopIteration, e:
-+            revision = cmdutil.tag_cur(self.tree)
-+            if revision is None:
-+                raise CantDetermineRevision("", "No version specified, no "
-+                                            "partner-versions, and no tag"
-+                                            " source")
-+            revisions = [revision]
-+            source = "from tag source"
-+        for revision in revisions:
-+            cmdutil.ensure_archive_registered(revision.archive)
-+            cmdutil.colorize(arch.Chatter("* Merging %s [%s]" % 
-+                             (revision, source)))
-+            if action=="native-merge" or action=="update":
-+                if self.native_merge(revision, action) == 0:
-+                    continue
-+            elif action=="star-merge":
-+                try: 
-+                    self.star_merge(revision, options.diff3)
-+                except errors.MergeProblem, e:
-+                    break
-+            if cmdutil.has_changed(self.tree.tree_version):
-+                break
-+
-+    def star_merge(self, revision, diff3):
-+        """Perform a star-merge on the current tree.
-+        
-+        :param revision: The revision to use for the merge
-+        :type revision: `arch.Revision`
-+        :param diff3: If true, do a diff3 merge
-+        :type diff3: bool
-+        """
-+        try:
-+            for line in self.tree.iter_star_merge(revision, diff3=diff3):
-+                cmdutil.colorize(line)
-+        except arch.util.ExecProblem, e:
-+            if e.proc.status is not None and e.proc.status == 1:
-+                if e.proc.error:
-+                    print e.proc.error
-+                raise MergeProblem
-+            else:
-+                raise
-+
-+    def native_merge(self, other_revision, action):
-+        """Perform a native-merge on the current tree.
-+        
-+        :param other_revision: The revision to use for the merge
-+        :type other_revision: `arch.Revision`
-+        :return: 0 if the merge was skipped, 1 if it was applied
-+        """
-+        other_tree = cmdutil.find_or_make_local_revision(other_revision)
-+        try:
-+            if action == "native-merge":
-+                ancestor = cmdutil.merge_ancestor2(self.tree, other_tree, 
-+                                                   other_revision)
-+            elif action == "update":
-+                ancestor = cmdutil.tree_latest(self.tree, 
-+                                               other_revision.version)
-+        except CantDetermineRevision, e:
-+            raise CommandFailedWrapper(e)
-+        cmdutil.colorize(arch.Chatter("* Found common ancestor %s" % ancestor))
-+        if (ancestor == other_revision):
-+            cmdutil.colorize(arch.Chatter("* Skipping redundant merge" 
-+                                          % ancestor))
-+            return 0
-+        delta = cmdutil.apply_delta(ancestor, other_tree, self.tree)    
-+        for line in cmdutil.iter_apply_delta_filter(delta):
-+            cmdutil.colorize(line)
-+        return 1
-+
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai merge [VERSION]")
-+        parser.add_option("-s", "--star-merge", action="store_const",
-+                          dest="action", help="Use star-merge",
-+                          const="star-merge", default="native-merge")
-+        parser.add_option("--update", action="store_const",
-+                          dest="action", help="Use update picker",
-+                          const="update")
-+        parser.add_option("--diff3", action="store_true", 
-+                         dest="diff3",  
-+                         help="Use diff3 for merge (implies star-merge)")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Performs a merge operation using the specified version.
-+        """
-+        return
-+
-+class ELog(BaseCommand):
-+    """
-+    Produces a raw patchlog and invokes the user's editor
-+    """
-+    def __init__(self):
-+        self.description="Edit a patchlog to commit"
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "elog" command.
-+        """
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if self.tree is None:
-+            raise arch.errors.TreeRootError
-+
-+        edit_log(self.tree)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "merge" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai elog")
-+        return parser 
-+
-+
-+    def help(self, parser=None):
-+        """
-+        Invokes $EDITOR to produce a log for committing.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Invokes $EDITOR to produce a log for committing.
-+        """
-+        return
-+
-+def edit_log(tree):
-+    """Makes and edits the log for a tree.  Does all kinds of fancy things
-+    like log templates and merge summaries and log-for-merge
-+    
-+    :param tree: The tree to edit the log for
-+    :type tree: `arch.WorkingTree`
-+    """
-+    #ensure we have an editor before preparing the log
-+    cmdutil.find_editor()
-+    log = tree.log_message(create=False)
-+    log_is_new = False
-+    if log is None or cmdutil.prompt("Overwrite log"):
-+        if log is not None:
-+           os.remove(log.name)
-+        log = tree.log_message(create=True)
-+        log_is_new = True
-+        tmplog = log.name
-+        template = tree+"/{arch}/=log-template"
-+        if not os.path.exists(template):
-+            template = os.path.expanduser("~/.arch-params/=log-template")
-+            if not os.path.exists(template):
-+                template = None
-+        if template:
-+            shutil.copyfile(template, tmplog)
-+        
-+        new_merges = list(cmdutil.iter_new_merges(tree, 
-+                                                  tree.tree_version))
-+        log["Summary"] = merge_summary(new_merges, tree.tree_version)
-+        if len(new_merges) > 0:   
-+            if cmdutil.prompt("Log for merge"):
-+                mergestuff = cmdutil.log_for_merge(tree)
-+                log.description += mergestuff
-+        log.save()
-+    try:
-+        cmdutil.invoke_editor(log.name)
-+    except:
-+        if log_is_new:
-+            os.remove(log.name)
-+        raise
-+
-+def merge_summary(new_merges, tree_version):
-+    if len(new_merges) == 0:
-+        return ""
-+    if len(new_merges) == 1:
-+        summary = new_merges[0].summary
-+    else:
-+        summary = "Merge"
-+
-+    credits = []
-+    for merge in new_merges:
-+        if arch.my_id() != merge.creator:
-+            name = re.sub("<.*>", "", merge.creator).rstrip(" ");
-+            if not name in credits:
-+                credits.append(name)
-+        else:
-+            version = merge.revision.version
-+            if version.archive == tree_version.archive:
-+                if not version.nonarch in credits:
-+                    credits.append(version.nonarch)
-+            elif not str(version) in credits:
-+                credits.append(str(version))
-+
-+    return ("%s (%s)") % (summary, ", ".join(credits))
-+
-+class MirrorArchive(BaseCommand):
-+    """
-+    Updates a mirror from an archive
-+    """
-+    def __init__(self):
-+        self.description="Update a mirror from an archive"
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        if len(args) > 1:
-+            raise GetHelp
-+        try:
-+            tree = arch.tree_root()
-+        except:
-+            tree = None
-+
-+        if len(args) == 0:
-+            if tree is not None:
-+                name = tree.tree_version()
-+        else:
-+            name = cmdutil.expand_alias(args[0], tree)
-+            name = arch.NameParser(name)
-+
-+        to_arch = name.get_archive()
-+        from_arch = cmdutil.get_mirror_source(arch.Archive(to_arch))
-+        limit = name.get_nonarch()
-+
-+        iter = arch_core.mirror_archive(from_arch,to_arch, limit)
-+        for line in arch.chatter_classifier(iter):
-+            cmdutil.colorize(line)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai mirror-archive ARCHIVE")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Updates a mirror from an archive.  If a branch, package, or version is
-+supplied, only changes under it are mirrored.
-+        """
-+        return
-+
-+def help_tree_spec():
-+    print """Specifying revisions (default: tree)
-+Revisions may be specified by alias, revision, version or patchlevel.
-+Revisions or versions may be fully qualified.  Unqualified revisions, versions, 
-+or patchlevels use the archive of the current project tree.  Versions will
-+use the latest patchlevel in the tree.  Patchlevels will use the current tree-
-+version.
-+
-+Use "alias" to list available (user and automatic) aliases."""
-+
-+def help_aliases(tree):
-+    print """Auto-generated aliases
-+ acur : The latest revision in the archive of the tree-version.  You can specfy
-+        a different version like so: acur:foo--bar--0 (aliases can be used)
-+ tcur : (tree current) The latest revision in the tree of the tree-version.
-+        You can specify a different version like so: tcur:foo--bar--0 (aliases
-+        can be used).
-+tprev : (tree previous) The previous revision in the tree of the tree-version.
-+        To specify an older revision, use a number, e.g. "tprev:4"
-+ tanc : (tree ancestor) The ancestor revision of the tree
-+        To specify an older revision, use a number, e.g. "tanc:4"
-+tdate : (tree date) The latest revision from a given date (e.g. "tdate:July 6")
-+ tmod : (tree modified) The latest revision to modify a given file 
-+        (e.g. "tmod:engine.cpp" or "tmod:engine.cpp:16")
-+ ttag : (tree tag) The revision that was tagged into the current tree revision,
-+        according to the tree.
-+tagcur: (tag current) The latest revision of the version that the current tree
-+        was tagged from.
-+mergeanc : The common ancestor of the current tree and the specified revision.
-+        Defaults to the first partner-version's latest revision or to tagcur.
-+   """
-+    print "User aliases"
-+    for parts in ancillary.iter_all_alias(tree):
-+        print parts[0].rjust(10)+" : "+parts[1]
-+
-+
-+class Inventory(BaseCommand):
-+    """List the status of files in the tree"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        tree = arch.tree_root()
-+        categories = []
-+
-+        if (options.source):
-+            categories.append(arch_core.SourceFile)
-+        if (options.precious):
-+            categories.append(arch_core.PreciousFile)
-+        if (options.backup):
-+            categories.append(arch_core.BackupFile)
-+        if (options.junk):
-+            categories.append(arch_core.JunkFile)
-+
-+        if len(categories) == 1:
-+            show_leading = False
-+        else:
-+            show_leading = True
-+
-+        if len(categories) == 0:
-+            categories = None
-+
-+        if options.untagged:
-+            categories = arch_core.non_root
-+            show_leading = False
-+            tagged = False
-+        else:
-+            tagged = None
-+        
-+        for file in arch_core.iter_inventory_filter(tree, None, 
-+            control_files=options.control_files, 
-+            categories = categories, tagged=tagged):
-+            print arch_core.file_line(file, 
-+                                      category = show_leading, 
-+                                      untagged = show_leading,
-+                                      id = options.ids)
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "revision" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai inventory [options]")
-+        parser.add_option("--ids", action="store_true", dest="ids", 
-+                          help="Show file ids")
-+        parser.add_option("--control", action="store_true", 
-+                          dest="control_files", help="include control files")
-+        parser.add_option("--source", action="store_true", dest="source",
-+                          help="List source files")
-+        parser.add_option("--backup", action="store_true", dest="backup",
-+                          help="List backup files")
-+        parser.add_option("--precious", action="store_true", dest="precious",
-+                          help="List precious files")
-+        parser.add_option("--junk", action="store_true", dest="junk",
-+                          help="List junk files")
-+        parser.add_option("--unrecognized", action="store_true", 
-+                          dest="unrecognized", help="List unrecognized files")
-+        parser.add_option("--untagged", action="store_true", 
-+                          dest="untagged", help="List only untagged files")
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists the status of files in the archive:
-+S source
-+P precious
-+B backup
-+J junk
-+U unrecognized
-+T tree root
-+? untagged-source
-+Leading letter are not displayed if only one kind of file is shown
-+        """
-+        return
-+
-+
-+class Alias(BaseCommand):
-+    """List or adjust aliases"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def get_completer(self, arg, index):
-+        if index > 2:
-+            return ()
-+        try:
-+            self.tree = arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+        if index == 0:
-+            return [part[0]+" " for part in ancillary.iter_all_alias(self.tree)]
-+        elif index == 1:
-+            return cmdutil.iter_revision_completions(arg, self.tree)
-+
-+
-+    def do_command(self, cmdargs):
-+        """
-+        Master function that perfoms the "revision" command.
-+        """
-+
-+        parser=self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree =  arch.tree_root()
-+        except:
-+            self.tree = None
-+
-+
-+        try:
-+            options.action(args, options)
-+        except cmdutil.ForbiddenAliasSyntax, e:
-+            raise CommandFailedWrapper(e)
-+
-+    def arg_dispatch(self, args, options):
-+        """Add, modify, or list aliases, depending on number of arguments
-+
-+        :param args: The list of commandline arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        if len(args) == 0:
-+            help_aliases(self.tree)
-+            return
-+        elif len(args) == 1:
-+            self.print_alias(args[0])
-+        elif (len(args)) == 2:
-+            self.add(args[0], args[1], options)
-+        else:
-+            raise cmdutil.GetHelp
-+
-+    def print_alias(self, alias):
-+        answer = None
-+        for pair in ancillary.iter_all_alias(self.tree):
-+            if pair[0] == alias:
-+                answer = pair[1]
-+        if answer is not None:
-+            print answer
-+        else:
-+            print "The alias %s is not assigned." % alias
-+
-+    def add(self, alias, expansion, options):
-+        """Add or modify aliases
-+
-+        :param alias: The alias name to create/modify
-+        :type alias: str
-+        :param expansion: The expansion to assign to the alias name
-+        :type expansion: str
-+        :param options: The commandline options
-+        """
-+        newlist = ""
-+        written = False
-+        new_line = "%s=%s\n" % (alias, cmdutil.expand_alias(expansion, 
-+            self.tree))
-+        ancillary.check_alias(new_line.rstrip("\n"), [alias, expansion])
-+
-+        for pair in self.get_iterator(options):
-+            if pair[0] != alias:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            elif not written:
-+                newlist+=new_line
-+                written = True
-+        if not written:
-+            newlist+=new_line
-+        self.write_aliases(newlist, options)
-+            
-+    def delete(self, args, options):
-+        """Delete the specified alias
-+
-+        :param args: The list of arguments
-+        :type args: list of str
-+        :param options: The commandline options
-+        """
-+        deleted = False
-+        if len(args) != 1:
-+            raise cmdutil.GetHelp
-+        newlist = ""
-+        for pair in self.get_iterator(options):
-+            if pair[0] != args[0]:
-+                newlist+="%s=%s\n" % (pair[0], pair[1])
-+            else:
-+                deleted = True
-+        if not deleted:
-+            raise errors.NoSuchAlias(args[0])
-+        self.write_aliases(newlist, options)
-+
-+    def get_alias_file(self, options):
-+        """Return the name of the alias file to use
-+
-+        :param options: The commandline options
-+        """
-+        if options.tree:
-+            if self.tree is None:
-+                self.tree == arch.tree_root()
-+            return str(self.tree)+"/{arch}/+aliases"
-+        else:
-+            return "~/.aba/aliases"
-+
-+    def get_iterator(self, options):
-+        """Return the alias iterator to use
-+
-+        :param options: The commandline options
-+        """
-+        return ancillary.iter_alias(self.get_alias_file(options))
-+
-+    def write_aliases(self, newlist, options):
-+        """Safely rewrite the alias file
-+        :param newlist: The new list of aliases
-+        :type newlist: str
-+        :param options: The commandline options
-+        """
-+        filename = os.path.expanduser(self.get_alias_file(options))
-+        file = cmdutil.NewFileVersion(filename)
-+        file.write(newlist)
-+        file.commit()
-+
-+
-+    def get_parser(self):
-+        """
-+        Returns the options parser to use for the "alias" command.
-+
-+        :rtype: cmdutil.CmdOptionParser
-+        """
-+        parser=cmdutil.CmdOptionParser("fai alias [ALIAS] [NAME]")
-+        parser.add_option("-d", "--delete", action="store_const", dest="action",
-+                          const=self.delete, default=self.arg_dispatch, 
-+                          help="Delete an alias")
-+        parser.add_option("--tree", action="store_true", dest="tree", 
-+                          help="Create a per-tree alias", default=False)
-+        return parser 
-+
-+    def help(self, parser=None):
-+        """
-+        Prints a help message.
-+
-+        :param parser: If supplied, the parser to use for generating help.  If \
-+        not supplied, it is retrieved.
-+        :type parser: cmdutil.CmdOptionParser
-+        """
-+        if parser==None:
-+            parser=self.get_parser()
-+        parser.print_help()
-+        print """
-+Lists current aliases or modifies the list of aliases.
-+
-+If no arguments are supplied, aliases will be listed.  If two arguments are
-+supplied, the specified alias will be created or modified.  If -d or --delete
-+is supplied, the specified alias will be deleted.
-+
-+You can create aliases that refer to any fully-qualified part of the
-+Arch namespace, e.g. 
-+archive, 
-+archive/category, 
-+archive/category--branch, 
-+archive/category--branch--version (my favourite)
-+archive/category--branch--version--patchlevel
-+
-+Aliases can be used automatically by native commands.  To use them
-+with external or tla commands, prefix them with ^ (you can do this
-+with native commands, too).
-+"""
-+
-+
-+class RequestMerge(BaseCommand):
-+    """Submit a merge request to Bug Goo"""
-+    def __init__(self):
-+        self.description=self.__doc__
-+
-+    def do_command(self, cmdargs):
-+        """Submit a merge request
-+
-+        :param cmdargs: The commandline arguments
-+        :type cmdargs: list of str
-+        """
-+        cmdutil.find_editor()
-+        parser = self.get_parser()
-+        (options, args) = parser.parse_args(cmdargs)
-+        try:
-+            self.tree=arch.tree_root()
-+        except:
-+            self.tree=None
-+        base, revisions = self.revision_specs(args)
-+        message = self.make_headers(base, revisions)
-+        message += self.make_summary(revisions)
-+        path = self.edit_message(message)
-+        message = self.tidy_message(path)
-+        if cmdutil.prompt("Send merge"):
-+            self.send_message(message)
-+            print "Merge request sent"
-+
-+    def make_headers(self, base, revisions):
-+        """Produce email and Bug Goo header strings
-+
-+        :param base: The base revision to apply merges to
-+        :type base: `arch.Revision`
-+        :param revisions: The revisions to replay into the base
-+        :type revisions: list of `arch.Patchlog`
-+        :return: The headers
-+        :rtype: str
-+        """
-+        headers = "To: gnu-arch-users@gnu.org\n"
-+        headers += "From: %s\n" % options.fromaddr
-+        if len(revisions) == 1:
-+            headers += "Subject: [MERGE REQUEST] %s\n" % revisions[0].summary
-+        else:
-+            headers += "Subject: [MERGE REQUEST]\n"
-+        headers += "\n"
-+        headers += "Base-Revision: %s\n" % base
-+        for revision in revisions:
-+            headers += "Revision: %s\n" % revision.revision
-+        headers += "Bug: \n\n"
-+        return headers
-+
-+    def make_summary(self, logs):
-+        """Generate a summary of merges
-+
-+        :param logs: the patchlogs that were directly added by the merges
-+        :type logs: list of `arch.Patchlog`
-+        :return: the summary
-+        :rtype: str
-+        """ 
-+        summary = ""
-+        for log in logs:
-+            summary+=str(log.revision)+"\n"
-+            summary+=log.summary+"\n"
-+            if log.description.strip():
-+                summary+=log.description.strip('\n')+"\n\n"
-+        return summary
-+
-+    def revision_specs(self, args):
-+        """Determine the base and merge revisions from tree and arguments.
-+
-+        :param args: The parsed arguments
-+        :type args: list of str
-+        :return: The base revision and merge revisions 
-+        :rtype: `arch.Revision`, list of `arch.Patchlog`
-+        """
-+        if len(args) > 0:
-+            target_revision = cmdutil.determine_revision_arch(self.tree, 
-+                                                              args[0])
-+        else:
-+            target_revision = cmdutil.tree_latest(self.tree)
-+        if len(args) > 1:
-+            merges = [ arch.Patchlog(cmdutil.determine_revision_arch(
-+                       self.tree, f)) for f in args[1:] ]
-+        else:
-+            if self.tree is None:
-+                raise CantDetermineRevision("", "Not in a project tree")
-+            merge_iter = cmdutil.iter_new_merges(self.tree, 
-+                                                 target_revision.version, 
-+                                                 False)
-+            merges = [f for f in cmdutil.direct_merges(merge_iter)]
-+        return (target_revision, merges)
-+
-+    def edit_message(self, message):
-+        """Edit an email message in the user's standard editor
-+
-+        :param message: The message to edit
-+        :type message: str
-+        :return: the path of the edited message
-+        :rtype: str
-+        """
-+        if self.tree is None:
-+            path = os.get_cwd()
-+        else:
-+            path = self.tree
-+        path += "/,merge-request"
-+        file = open(path, 'w')
-+        file.write(message)
-+        file.flush()
-+        cmdutil.invoke_editor(path)
-+        return path
-+
-+    def tidy_message(self, path):
-+        """Validate and clean up message.
-+
-+        :param path: The path to the message to clean up
-+        :type path: str
-+        :return: The parsed message
-+        :rtype: `email.Message`
-+        """
-+        mail = email.message_from_file(open(path))
-+        if mail["Subject"].strip() == "[MERGE REQUEST]":
-+            raise BlandSubject
-+        
-+        request = email.message_from_string(mail.get_payload())
-+        if request.has_key("Bug"):
-+            if request["Bug"].strip()=="":
-+                del request["Bug"]
-+        mail.set_payload(request.as_string())
-+        return mail
-+
-+    def send_message(self, message):
-+        """Send a message, using its headers to address it.
-+
-+        :param message: The message to send
-+        :type message: `email.Message`"""
-+        server = smtplib.SMTP()
-+        server.sendmail(message['From'], message['To'], message.as_string())
-+        server.quit()
-+
-+    def help(self, parser=None):
-+        """Print a usage message
-+
-+        :param parser: The options parser to use
-+        :type parser: `cmdutil.CmdOptionParser`
-+        """
-+        if parser is None:
-+            parser = self.get_parser()
-+        parser.print_help()
-+        print """
-+Sends a merge request formatted for Bug Goo.  Intended use: get the tree
-+you'd like to merge into.  Apply the merges you want.  Invoke request-merge.
-+The merge request will open in your $EDITOR.
-+
-+When no TARGET is specified, it uses the current tree revision.  When
-+no MERGE is specified, it uses the direct merges (as in "revisions
-+--direct-merges").  But you can specify just the TARGET, or all the MERGE
-+revisions.
-+"""
-+
-+    def get_parser(self):
-+        """Produce a commandline parser for this command.
-+
-+        :rtype: `cmdutil.CmdOptionParser`
-+        """
-+        parser=cmdutil.CmdOptionParser("request-merge [TARGET] [MERGE1...]")
-+        return parser
-+
-+commands = { 
-+'changes' : Changes,
-+'help' : Help,
-+'update': Update,
-+'apply-changes':ApplyChanges,
-+'cat-log': CatLog,
-+'commit': Commit,
-+'revision': Revision,
-+'revisions': Revisions,
-+'get': Get,
-+'revert': Revert,
-+'shell': Shell,
-+'add-id': AddID,
-+'merge': Merge,
-+'elog': ELog,
-+'mirror-archive': MirrorArchive,
-+'ninventory': Inventory,
-+'alias' : Alias,
-+'request-merge': RequestMerge,
-+}
-+suggestions = {
-+'apply-delta' : "Try \"apply-changes\".",
-+'delta' : "To compare two revisions, use \"changes\".",
-+'diff-rev' : "To compare two revisions, use \"changes\".",
-+'undo' : "To undo local changes, use \"revert\".",
-+'undelete' : "To undo only deletions, use \"revert --deletions\"",
-+'missing-from' : "Try \"revisions --missing-from\".",
-+'missing' : "Try \"revisions --missing\".",
-+'missing-merge' : "Try \"revisions --partner-missing\".",
-+'new-merges' : "Try \"revisions --new-merges\".",
-+'cachedrevs' : "Try \"revisions --cacherevs\". (no 'd')",
-+'logs' : "Try \"revisions --logs\"",
-+'tree-source' : "Use the \"^ttag\" alias (\"revision ^ttag\")",
-+'latest-revision' : "Use the \"^acur\" alias (\"revision ^acur\")",
-+'change-version' : "Try \"update REVISION\"",
-+'tree-revision' : "Use the \"^tcur\" alias (\"revision ^tcur\")",
-+'rev-depends' : "Use revisions --dependencies",
-+'auto-get' : "Plain get will do archive lookups",
-+'tagline' : "Use add-id.  It uses taglines in tagline trees",
-+'emlog' : "Use elog.  It automatically adds log-for-merge text, if any",
-+'library-revisions' : "Use revisions --library",
-+'file-revert' : "Use revert FILE"
-+}
-+# arch-tag: 19d5739d-3708-486c-93ba-deecc3027fc7
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -31,6 +31,8 @@
- from revision import Revision
- from errors import bailout, BzrError
- from textui import show_status
-+import patches
-+from bzrlib import progress
- 
- BZR_BRANCH_FORMAT = "Bazaar-NG branch, format 0.0.4\n"
- ## TODO: Maybe include checks for common corruption of newlines, etc?
-@@ -864,3 +866,36 @@
- 
-     s = hexlify(rand_bytes(8))
-     return '-'.join((name, compact_date(time.time()), s))
-+
-+
-+def iter_anno_data(branch, file_id):
-+    later_revision = branch.revno()
-+    q = range(branch.revno())
-+    q.reverse()
-+    later_text_id = branch.basis_tree().inventory[file_id].text_id
-+    i = 0
-+    for revno in q:
-+        i += 1
-+        cur_tree = branch.revision_tree(branch.lookup_revision(revno))
-+        if file_id not in cur_tree.inventory:
-+            text_id = None
-+        else:
-+            text_id = cur_tree.inventory[file_id].text_id
-+        if text_id != later_text_id:
-+            patch = get_patch(branch, revno, later_revision, file_id)
-+            yield revno, patch.iter_inserted(), patch
-+            later_revision = revno
-+            later_text_id = text_id
-+        yield progress.Progress("revisions", i)
-+
-+def get_patch(branch, old_revno, new_revno, file_id):
-+    old_tree = branch.revision_tree(branch.lookup_revision(old_revno))
-+    new_tree = branch.revision_tree(branch.lookup_revision(new_revno))
-+    if file_id in old_tree.inventory:
-+        old_file = old_tree.get_file(file_id).readlines()
-+    else:
-+        old_file = []
-+    ud = difflib.unified_diff(old_file, new_tree.get_file(file_id).readlines())
-+    return patches.parse_patch(ud)
-+
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py 
-+++ bzrlib/commands.py 
-@@ -27,6 +27,9 @@
- from bzrlib import Branch, Inventory, InventoryEntry, ScratchBranch, BZRDIR, \
-      format_date
- from bzrlib import merge
-+from bzrlib.branch import iter_anno_data
-+from bzrlib import patches
-+from bzrlib import progress
- 
- 
- def _squish_command_name(cmd):
-@@ -882,7 +885,15 @@
-                 print '%3d FAILED!' % mf
-             else:
-                 print
--
-+        result = bzrlib.patches.test()
-+        resultFailed = len(result.errors) + len(result.failures)
-+        print '%-40s %3d tests' % ('bzrlib.patches', result.testsRun),
-+        if resultFailed:
-+            print '%3d FAILED!' % resultFailed
-+        else:
-+            print
-+        tests += result.testsRun
-+        failures += resultFailed
-         print '%-40s %3d tests' % ('total', tests),
-         if failures:
-             print '%3d FAILED!' % failures
-@@ -897,6 +908,34 @@
-     """Show version of bzr"""
-     def run(self):
-         show_version()
-+
-+class cmd_annotate(Command):
-+    """Show which revision added each line in a file"""
-+
-+    takes_args = ['filename']
-+    def run(self, filename):
-+        if not os.path.exists(filename):
-+            raise BzrCommandError("The file %s does not exist." % filename)
-+        branch = (Branch(filename))
-+        file_id = branch.working_tree().path2id(filename)
-+        if file_id is None:
-+            raise BzrCommandError("The file %s is not versioned." % filename)
-+        lines = branch.basis_tree().get_file(file_id)
-+        total = branch.revno()
-+        anno_d_iter = iter_anno_data(branch, file_id)
-+        progress_bar = progress.ProgressBar()
-+        try:
-+            for result in patches.iter_annotate_file(lines, anno_d_iter):
-+                if isinstance(result, progress.Progress):
-+                    result.total = total
-+                    progress_bar(result)
-+                else:
-+                    anno_lines = result
-+        finally:
-+            progress.clear_progress_bar()
-+        for line in anno_lines:
-+            sys.stdout.write("%4s:%s" % (str(line.log), line.text))
-+
- 
- def show_version():
-     print "bzr (bazaar-ng) %s" % bzrlib.__version__
-

=== added file 'patches/cache_weave_inclusions.diff'
--- patches/cache_weave_inclusions.diff	1970-01-01 00:00:00 +0000
+++ patches/cache_weave_inclusions.diff	2005-09-19 10:46:28 +0000
@@ -0,0 +1,85 @@
+=== modified file 'bzrlib/weave.py'
+--- bzrlib/weave.py
++++ bzrlib/weave.py
+@@ -83,6 +83,10 @@
+ 
+ # TODO: Perhaps the API should work only in names to hide the integer
+ # indexes from the user?
++
++# TODO: Is there any potential performance win by having an add()
++# variant that is passed a pre-cooked version of the single basis
++# version?
+ 
+ 
+ 
+@@ -191,8 +195,8 @@
+         Set by read_weave.
+     """
+ 
+-    __slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
+-                 '_weave_name']
++    ##__slots__ = ['_weave', '_parents', '_sha1s', '_names', '_name_map',
++    ##             '_weave_name', '_inclusion_cache']
+     
+     def __init__(self, weave_name=None):
+         self._weave = []
+@@ -201,7 +205,7 @@
+         self._names = []
+         self._name_map = {}
+         self._weave_name = weave_name
+-
++        self._inclusion_cache = {}
+ 
+     def __eq__(self, other):
+         if not isinstance(other, Weave):
+@@ -300,7 +304,6 @@
+                 self._weave.append(('{', new_version))
+                 self._weave.extend(text)
+                 self._weave.append(('}', None))
+-        
+             return new_version
+ 
+         if len(parents) == 1:
+@@ -308,10 +311,9 @@
+             if sha1 == self._sha1s[pv]:
+                 # special case: same as the single parent
+                 return new_version
+-            
+ 
+         ancestors = self.inclusions(parents)
+-
++        self._inclusion_cache[(new_version,)] = ancestors | set([new_version])
+         l = self._weave
+ 
+         # basis a list of (origin, lineno, line)
+@@ -379,17 +381,19 @@
+ 
+     def inclusions(self, versions):
+         """Return set of all ancestors of given version(s)."""
++        tv = tuple(sorted(versions))
++        cached_val = self._inclusion_cache.get(tv)
++        if cached_val is not None:
++            return cached_val
+         i = set(versions)
+-        v = max(versions)
+-        try:
+-            while v >= 0:
+-                if v in i:
+-                    # include all its parents
+-                    i.update(self._parents[v])
+-                v -= 1
+-            return i
+-        except IndexError:
+-            raise ValueError("version %d not present in weave" % v)
++        for v in xrange(max(versions), 0, -1):
++            if v in i:
++                # include all its parents
++                i.update(self._parents[v])
++        self._inclusion_cache[tv] = i
++        return i
++        ## except IndexError:
++        ##     raise ValueError("version %d not present in weave" % v)
+ 
+ 
+     def parents(self, version):
+

=== removed file 'patches/plugins-no-plugins.patch'
--- patches/plugins-no-plugins.patch	2005-06-22 06:38:18 +0000
+++ patches/plugins-no-plugins.patch	1970-01-01 00:00:00 +0000
@@ -1,519 +0,0 @@
-*** added file 'bzrlib/plugin.py'
---- /dev/null
-+++ bzrlib/plugin.py
-@@ -0,0 +1,92 @@
-+# Copyright (C) 2004, 2005 by Canonical Ltd
-+
-+# This program is free software; you can redistribute it and/or modify
-+# it under the terms of the GNU General Public License as published by
-+# the Free Software Foundation; either version 2 of the License, or
-+# (at your option) any later version.
-+
-+# This program is distributed in the hope that it will be useful,
-+# but WITHOUT ANY WARRANTY; without even the implied warranty of
-+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+# GNU General Public License for more details.
-+
-+# You should have received a copy of the GNU General Public License
-+# along with this program; if not, write to the Free Software
-+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+
-+# This module implements plug-in support.
-+# Any python module in $BZR_PLUGIN_PATH will be imported upon initialization
-+# of bzrlib (and then forgotten about).  In the plugin's main body, it should
-+# update any bzrlib registries it wants to extend; for example, to add new
-+# commands, import bzrlib.commands and add your new command to the
-+# plugin_cmds variable.
-+
-+import sys, os, imp
-+try:
-+    set
-+except NameError:
-+    from sets import Set as set
-+from bzrlib.trace import log_error
-+
-+
-+def load_plugins():
-+    """Find all python files which are plugins, and load them
-+
-+    The environment variable BZR_PLUGIN_PATH is considered a delimited set of
-+    paths to look through. Each entry is searched for *.py files (and whatever
-+    other extensions are used in the platform, such as *.pyd).
-+    """
-+    bzrpath = os.environ.get('BZR_PLUGIN_PATH', os.path.expanduser('~/.bzr/plugins'))
-+
-+    # The problem with imp.get_suffixes() is that it doesn't include
-+    # .pyo which is technically valid
-+    # It also means that "testmodule.so" will show up as both test and testmodule
-+    # though it is only valid as 'test'
-+    # but you should be careful, because "testmodule.py" loads as testmodule.
-+    suffixes = imp.get_suffixes()
-+    suffixes.append(('.pyo', 'rb', imp.PY_COMPILED))
-+    package_entries = ['__init__.py', '__init__.pyc', '__init__.pyo']
-+    for d in bzrpath.split(os.pathsep):
-+        # going trough them one by one allows different plugins with the same
-+        # filename in different directories in the path
-+        if not d:
-+            continue
-+        plugin_names = set()
-+        if not os.path.isdir(d):
-+            continue
-+        for f in os.listdir(d):
-+            path = os.path.join(d, f)
-+            if os.path.isdir(path):
-+                for entry in package_entries:
-+                    # This directory should be a package, and thus added to
-+                    # the list
-+                    if os.path.isfile(os.path.join(path, entry)):
-+                        break
-+                else: # This directory is not a package
-+                    continue
-+            else:
-+                for suffix_info in suffixes:
-+                    if f.endswith(suffix_info[0]):
-+                        f = f[:-len(suffix_info[0])]
-+                        if suffix_info[2] == imp.C_EXTENSION and f.endswith('module'):
-+                            f = f[:-len('module')]
-+                        break
-+                else:
-+                    continue
-+            plugin_names.add(f)
-+
-+        plugin_names = list(plugin_names)
-+        plugin_names.sort()
-+        for name in plugin_names:
-+            try:
-+                plugin_info = imp.find_module(name, [d])
-+                try:
-+                    plugin = imp.load_module('bzrlib.plugin.' + name,
-+                                             *plugin_info)
-+                finally:
-+                    if plugin_info[0] is not None:
-+                        plugin_info[0].close()
-+            except Exception, e:
-+                log_error('Unable to load plugin: %r from %r\n%s' % (name, d, e))
-+
-
-*** modified file 'bzrlib/__init__.py'
---- bzrlib/__init__.py
-+++ bzrlib/__init__.py
-@@ -23,6 +23,7@@
- from diff import compare_trees
- from trace import mutter, warning, open_tracefile
- from log import show_log
-+from plugin import load_plugins
- import add
-
- BZRDIR = ".bzr"
-@@ -62,4 +63,4 @@
-             return None
-     except BzrError:
-         return None
--
-+
-
-*** modified file 'bzrlib/commands.py'
---- bzrlib/commands.py
-+++ bzrlib/commands.py
-@@ -24,6 +24,24 @@
- from bzrlib.osutils import quotefn
- from bzrlib import Branch, Inventory, InventoryEntry, BZRDIR, \
-      format_date
-+
-+
-+plugin_cmds = {}
-+
-+
-+def register_plugin_command(cmd):
-+    "Utility function to help register a command"
-+    global plugin_cmds
-+    k = cmd.__name__
-+    if k.startswith("cmd_"):
-+        k_unsquished = _unsquish_command_name(k)
-+    else:
-+        k_unsquished = k
-+    if not plugin_cmds.has_key(k_unsquished):
-+        plugin_cmds[k_unsquished] = cmd
-+    else:
-+        log_error('Two plugins defined the same command: %r' % k)
-+        log_error('Not loading the one in %r' % sys.modules[cmd.__module__])
-
-
- def _squish_command_name(cmd):
-@@ -68,100 +86,34 @@
-         revs = int(revstr)
-     return revs
-
--def _find_plugins():
--    """Find all python files which are plugins, and load their commands
--    to add to the list of "all commands"
--
--    The environment variable BZRPATH is considered a delimited set of
--    paths to look through. Each entry is searched for *.py files.
--    If a directory is found, it is also searched, but they are
--    not searched recursively. This allows you to revctl the plugins.
--
--    Inside the plugin should be a series of cmd_* function, which inherit from
--    the bzrlib.commands.Command class.
--    """
--    bzrpath = os.environ.get('BZRPLUGINPATH', '')
--
--    plugin_cmds = {}
--    if not bzrpath:
--        return plugin_cmds
--    _platform_extensions = {
--        'win32':'.pyd',
--        'cygwin':'.dll',
--        'darwin':'.dylib',
--        'linux2':'.so'
--        }
--    if _platform_extensions.has_key(sys.platform):
--        platform_extension = _platform_extensions[sys.platform]
--    else:
--        platform_extension = None
--    for d in bzrpath.split(os.pathsep):
--        plugin_names = {} # This should really be a set rather than a dict
--        for f in os.listdir(d):
--            if f.endswith('.py'):
--                f = f[:-3]
--            elif f.endswith('.pyc') or f.endswith('.pyo'):
--                f = f[:-4]
--            elif platform_extension and f.endswith(platform_extension):
--                f = f[:-len(platform_extension)]
--                if f.endswidth('module'):
--                    f = f[:-len('module')]
--            else:
--                continue
--            if not plugin_names.has_key(f):
--                plugin_names[f] = True
--
--        plugin_names = plugin_names.keys()
--        plugin_names.sort()
--        try:
--            sys.path.insert(0, d)
--            for name in plugin_names:
--                try:
--                    old_module = None
--                    try:
--                        if sys.modules.has_key(name):
--                            old_module = sys.modules[name]
--                            del sys.modules[name]
--                        plugin = __import__(name, locals())
--                        for k in dir(plugin):
--                            if k.startswith('cmd_'):
--                                k_unsquished = _unsquish_command_name(k)
--                                if not plugin_cmds.has_key(k_unsquished):
--                                    plugin_cmds[k_unsquished] = getattr(plugin, k)
--                                else:
--                                    log_error('Two plugins defined the same command: %r' % k)
--                                    log_error('Not loading the one in %r in dir %r' % (name, d))
--                    finally:
--                        if old_module:
--                            sys.modules[name] = old_module
--                except ImportError, e:
--                    log_error('Unable to load plugin: %r from %r\n%s' % (name, d, e))
--        finally:
--            sys.path.pop(0)
--    return plugin_cmds
--
--def _get_cmd_dict(include_plugins=True):
-+def _get_cmd_dict(plugins_override=True):
-     d = {}
-     for k, v in globals().iteritems():
-         if k.startswith("cmd_"):
-             d[_unsquish_command_name(k)] = v
--    if include_plugins:
--        d.update(_find_plugins())
-+    # If we didn't load plugins, the plugin_cmds dict will be empty
-+    if plugins_override:
-+        d.update(plugin_cmds)
-+    else:
-+        d2 = {}
-+        d2.update(plugin_cmds)
-+        d2.update(d)
-+        d = d2
-     return d
-
--def get_all_cmds(include_plugins=True):
-+def get_all_cmds(plugins_override=True):
-     """Return canonical name and class for all registered commands."""
--    for k, v in _get_cmd_dict(include_plugins=include_plugins).iteritems():
-+    for k, v in _get_cmd_dict(plugins_override=plugins_override).iteritems():
-         yield k,v
-
-
--def get_cmd_class(cmd,include_plugins=True):
-+def get_cmd_class(cmd, plugins_override=True):
-     """Return the canonical name and command class for a command.
-     """
-     cmd = str(cmd)                      # not unicode
-
-     # first look up this command under the specified name
--    cmds = _get_cmd_dict(include_plugins=include_plugins)
-+    cmds = _get_cmd_dict(plugins_override=plugins_override)
-     try:
-         return cmd, cmds[cmd]
-     except KeyError:
-@@ -1461,6 +1413,75 @@
-     return argdict
-
-
-+def _parse_master_args(argv):
-+    """Parse the arguments that always go with the original command.
-+    These are things like bzr --no-plugins, etc.
-+
-+    There are now 2 types of option flags. Ones that come *before* the command,
-+    and ones that come *after* the command.
-+    Ones coming *before* the command are applied against all possible commands.
-+    And are generally applied before plugins are loaded.
-+
-+    The current list are:
-+        --builtin   Allow plugins to load, but don't let them override builtin commands,
-+                    they will still be allowed if they do not override a builtin.
-+        --no-plugins    Don't load any plugins. This lets you get back to official source
-+                        behavior.
-+        --profile   Enable the hotspot profile before running the command.
-+                    For backwards compatibility, this is also a non-master option.
-+        --version   Spit out the version of bzr that is running and exit.
-+                    This is also a non-master option.
-+        --help      Run help and exit, also a non-master option (I think that should stay, though)
-+
-+    >>> argv, opts = _parse_master_args(['bzr', '--test'])
-+    Traceback (most recent call last):
-+    ...
-+    BzrCommandError: Invalid master option: 'test'
-+    >>> argv, opts = _parse_master_args(['bzr', '--version', 'command'])
-+    >>> print argv
-+    ['command']
-+    >>> print opts['version']
-+    True
-+    >>> argv, opts = _parse_master_args(['bzr', '--profile', 'command', '--more-options'])
-+    >>> print argv
-+    ['command', '--more-options']
-+    >>> print opts['profile']
-+    True
-+    >>> argv, opts = _parse_master_args(['bzr', '--no-plugins', 'command'])
-+    >>> print argv
-+    ['command']
-+    >>> print opts['no-plugins']
-+    True
-+    >>> print opts['profile']
-+    False
-+    >>> argv, opts = _parse_master_args(['bzr', 'command', '--profile'])
-+    >>> print argv
-+    ['command', '--profile']
-+    >>> print opts['profile']
-+    False
-+    """
-+    master_opts = {'builtin':False,
-+        'no-plugins':False,
-+        'version':False,
-+        'profile':False,
-+        'help':False
-+    }
-+
-+    # This is the point where we could hook into argv[0] to determine
-+    # what front-end is supposed to be run
-+    # For now, we are just ignoring it.
-+    cmd_name = argv.pop(0)
-+    for arg in argv[:]:
-+        if arg[:2] != '--': # at the first non-option, we return the rest
-+            break
-+        arg = arg[2:] # Remove '--'
-+        if arg not in master_opts:
-+            # We could say that this is not an error, that we should
-+            # just let it be handled by the main section instead
-+            raise BzrCommandError('Invalid master option: %r' % arg)
-+        argv.pop(0) # We are consuming this entry
-+        master_opts[arg] = True
-+    return argv, master_opts
-
- def run_bzr(argv):
-     """Execute a command.
-@@ -1470,22 +1491,21 @@
-     """
-     argv = [a.decode(bzrlib.user_encoding) for a in argv]
-
--    include_plugins=True
-     try:
--        args, opts = parse_args(argv[1:])
--        if 'help' in opts:
-+        argv, master_opts = _parse_master_args(argv)
-+        if not master_opts['no-plugins']:
-+            bzrlib.load_plugins()
-+        args, opts = parse_args(argv)
-+        if 'help' in opts or master_opts['help']:
-             import help
-             if args:
-                 help.help(args[0])
-             else:
-                 help.help()
-             return 0
--        elif 'version' in opts:
-+        elif 'version' in opts or master_opts['version']:
-             show_version()
-             return 0
--        elif args and args[0] == 'builtin':
--            include_plugins=False
--            args = args[1:]
-         cmd = str(args.pop(0))
-     except IndexError:
-         import help
-@@ -1493,14 +1513,15 @@
-         return 1
-
-
--    canonical_cmd, cmd_class = get_cmd_class(cmd,include_plugins=include_plugins)
--
--    # global option
-+    plugins_override = not (master_opts['builtin'])
-+    canonical_cmd, cmd_class = get_cmd_class(cmd, plugins_override=plugins_override)
-+
-+    profile = master_opts['profile']
-+    # For backwards compatibility, I would rather stick with --profile being a
-+    # master/global option
-     if 'profile' in opts:
-         profile = True
-         del opts['profile']
--    else:
--        profile = False
-
-     # check options are reasonable
-     allowed = cmd_class.takes_options
-
-*** modified file 'testbzr'
---- testbzr
-+++ testbzr
-@@ -149,6 +149,7 @@
-     """Run a test involving creating a plugin to load,
-     and making sure it is seen properly.
-     """
-+    orig_help = backtick('bzr help commands') # No plugins yet
-     mkdir('plugin_test')
-     f = open(os.path.join('plugin_test', 'myplug.py'), 'wb')
-     f.write("""import bzrlib, bzrlib.commands
-@@ -157,24 +158,36 @@
-     aliases = ['mplg']
-     def run(self):
-         print 'Hello from my plugin'
-+class cmd_myplug_with_opt(bzrlib.commands.Command):
-+    '''A simple plugin that requires a special option'''
-+    takes_options = ['aspecialoptionthatdoesntexist']
-+    def run(self, aspecialoptionthatdoesntexist=None):
-+        print 'Found: %s' % aspecialoptionthatdoesntexist
-+
-+bzrlib.commands.register_plugin_command(cmd_myplug)
-+bzrlib.commands.register_plugin_command(cmd_myplug_with_opt)
-+bzrlib.commands.OPTIONS['aspecialoptionthatdoesntexist'] = str
- """)
-     f.close()
-
--    os.environ['BZRPLUGINPATH'] = os.path.abspath('plugin_test')
--    help = backtick('bzr help commands')
-+    os.environ['BZR_PLUGIN_PATH'] = os.path.abspath('plugin_test')
-+    help = backtick('bzr help commands') #Help with user-visible plugins
-     assert help.find('myplug') != -1
-     assert help.find('Just a simple test plugin.') != -1
-
-
-     assert backtick('bzr myplug') == 'Hello from my plugin\n'
-     assert backtick('bzr mplg') == 'Hello from my plugin\n'
-+    assert backtick('bzr myplug-with-opt') == 'Found: None\n'
-+    assert backtick('bzr myplug-with-opt --aspecialoptionthatdoesntexist=2') == 'Found: 2\n'
-
-     f = open(os.path.join('plugin_test', 'override.py'), 'wb')
-     f.write("""import bzrlib, bzrlib.commands
--class cmd_commit(bzrlib.commands.cmd_commit):
--    '''Commit changes into a new revision.'''
-+class cmd_revno(bzrlib.commands.cmd_revno):
-+    '''Show current revision number.'''
-     def run(self, *args, **kwargs):
-         print "I'm sorry dave, you can't do that"
-+        return 1
-
- class cmd_help(bzrlib.commands.cmd_help):
-     '''Show help on a command or other topic.'''
-@@ -182,16 +195,67 @@
-         print "You have been overridden"
-         bzrlib.commands.cmd_help.run(self, *args, **kwargs)
-
-+bzrlib.commands.register_plugin_command(cmd_revno)
-+bzrlib.commands.register_plugin_command(cmd_help)
- """)
-     f.close()
-
--    newhelp = backtick('bzr help commands')
-+    newhelp = backtick('bzr help commands') # Help with no new commands,
-     assert newhelp.startswith('You have been overridden\n')
-     # We added a line, but the rest should work
-     assert newhelp[25:] == help
--
--    assert backtick('bzr commit -m test') == "I'm sorry dave, you can't do that\n"
--
-+    # Make sure we can get back to the original command
-+    # Not overridden, and no extra commands present
-+    assert backtick('bzr --builtin help commands') == help
-+    assert backtick('bzr --no-plugins help commands') == orig_help
-+
-+    assert backtick('bzr revno', retcode=1) == "I'm sorry dave, you can't do that\n"
-+
-+    print_txt = '** Loading noop plugin'
-+    f = open(os.path.join('plugin_test', 'loading.py'), 'wb')
-+    f.write("""import bzrlib, bzrlib.commands
-+class cmd_noop(bzrlib.commands.Command):
-+    def run(self, *args, **kwargs):
-+        pass
-+
-+print %r
-+bzrlib.commands.register_plugin_command(cmd_noop)
-+""" % print_txt)
-+    f.close()
-+    print_txt += '\n'
-+
-+    # Check that --builtin still loads the plugin, and enables it as
-+    # an extra command, but not as an override
-+    # and that --no-plugins doesn't load the command at all
-+    assert backtick('bzr noop') == print_txt
-+    assert backtick('bzr --builtin help')[:len(print_txt)] == print_txt
-+    assert backtick('bzr --no-plugins help')[:len(print_txt)] != print_txt
-+    runcmd('bzr revno', retcode=1)
-+    runcmd('bzr --builtin revno', retcode=0)
-+    runcmd('bzr --no-plugins revno', retcode=0)
-+    runcmd('bzr --builtin noop', retcode=0)
-+    runcmd('bzr --no-plugins noop', retcode=1)
-+
-+    # Check that packages can also be loaded
-+    test_str = 'packages work'
-+    os.mkdir(os.path.join('plugin_test', 'testpkg'))
-+    f = open(os.path.join('plugin_test', 'testpkg', '__init__.py'), 'wb')
-+    f.write("""import bzrlib, bzrlib.commands
-+class testpkgcmd(bzrlib.commands.Command):
-+    def run(self, *args, **kwargs):
-+        print %r
-+
-+bzrlib.commands.register_plugin_command(testpkgcmd)
-+""" % test_str)
-+    f.close()
-+    test_str += '\n'
-+    assert backtick('bzr testpkgcmd') == print_txt + test_str
-+    runcmd('bzr --no-plugins testpkgcmd', retcode=1)
-+
-+    # Make sure that setting BZR_PLUGIN_PATH to empty is the same as using --no-plugins
-+    os.environ['BZR_PLUGIN_PATH'] = ''
-+    assert backtick('bzr help commands') == orig_help
-+
-     shutil.rmtree('plugin_test')
-
- try:
-@@ -221,6 +285,9 @@
-
-     runcmd(['mkdir', TESTDIR])
-     cd(TESTDIR)
-+    # This means that any command that is naively run in this directory
-+    # Won't affect the parent directory.
-+    runcmd('bzr init')
-     test_root = os.getcwd()
-
-     progress("introductory commands")
-

=== removed file 'patches/progress.diff'
--- patches/progress.diff	2005-06-10 06:34:26 +0000
+++ patches/progress.diff	1970-01-01 00:00:00 +0000
@@ -1,143 +0,0 @@
-*** added file 'bzrlib/progress.py'
---- /dev/null 
-+++ bzrlib/progress.py 
-@@ -0,0 +1,138 @@
-+# Copyright (C) 2005 Aaron Bentley
-+# <aaron.bentley@utoronto.ca>
-+#
-+#    This program is free software; you can redistribute it and/or modify
-+#    it under the terms of the GNU General Public License as published by
-+#    the Free Software Foundation; either version 2 of the License, or
-+#    (at your option) any later version.
-+#
-+#    This program is distributed in the hope that it will be useful,
-+#    but WITHOUT ANY WARRANTY; without even the implied warranty of
-+#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-+#    GNU General Public License for more details.
-+#
-+#    You should have received a copy of the GNU General Public License
-+#    along with this program; if not, write to the Free Software
-+#    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-+
-+import sys
-+import datetime
-+
-+class Progress(object):
-+    def __init__(self, units, current, total=None):
-+        self.units = units
-+        self.current = current
-+        self.total = total
-+
-+    def _get_percent(self):
-+        if self.total is not None and self.current is not None:
-+            return 100.0 * self.current / self.total
-+
-+    percent = property(_get_percent)
-+
-+    def __str__(self):
-+        if self.total is not None:
-+            return "%i of %i %s %.1f%%" % (self.current, self.total, self.units,
-+                                         self.percent)
-+        else:
-+            return "%i %s" (self.current, self.units) 
-+
-+class ProgressBar(object):
-+    def __init__(self):
-+        self.start = None
-+        object.__init__(self)
-+
-+    def __call__(self, progress):
-+        if self.start is None:
-+            self.start = datetime.datetime.now()
-+        progress_bar(progress, start_time=self.start)
-+        
-+def divide_timedelta(delt, divisor):
-+    """Divides a timedelta object"""
-+    return datetime.timedelta(float(delt.days)/divisor, 
-+                              float(delt.seconds)/divisor, 
-+                              float(delt.microseconds)/divisor)
-+
-+def str_tdelta(delt):
-+    if delt is None:
-+        return "-:--:--"
-+    return str(datetime.timedelta(delt.days, delt.seconds))
-+
-+def get_eta(start_time, progress, enough_samples=20):
-+    if start_time is None or progress.current == 0:
-+        return None
-+    elif progress.current < enough_samples:
-+        return None
-+    elapsed = datetime.datetime.now() - start_time
-+    total_duration = divide_timedelta((elapsed) * long(progress.total), 
-+                                      progress.current)
-+    if elapsed < total_duration:
-+        eta = total_duration - elapsed
-+    else:
-+        eta = total_duration - total_duration
-+    return eta
-+
-+def progress_bar(progress, start_time=None):
-+    eta = get_eta(start_time, progress)
-+    if start_time is not None:
-+        eta_str = " "+str_tdelta(eta)
-+    else:
-+        eta_str = ""
-+
-+    fmt = " %i of %i %s (%.1f%%)"
-+    f = fmt % (progress.total, progress.total, progress.units, 100.0)
-+    max = len(f)
-+    cols = 77 - max
-+    if start_time is not None:
-+        cols -= len(eta_str)
-+    markers = int (float(cols) * progress.current / progress.total)
-+    txt = fmt % (progress.current, progress.total, progress.units,
-+                 progress.percent)
-+    sys.stderr.write("\r[%s%s]%s%s" % ('='*markers, ' '*(cols-markers), txt, 
-+                                       eta_str))
-+
-+def clear_progress_bar():
-+    sys.stderr.write('\r%s\r' % (' '*79))
-+
-+def spinner_str(progress, show_text=False):
-+    """
-+    Produces the string for a textual "spinner" progress indicator
-+    :param progress: an object represinting current progress
-+    :param show_text: If true, show progress text as well
-+    :return: The spinner string
-+
-+    >>> spinner_str(Progress("baloons", 0))
-+    '|'
-+    >>> spinner_str(Progress("baloons", 5))
-+    '/'
-+    >>> spinner_str(Progress("baloons", 6), show_text=True)
-+    '- 6 baloons'
-+    """
-+    positions = ('|', '/', '-', '\\')
-+    text = positions[progress.current % 4]
-+    if show_text:
-+        text+=" %i %s" % (progress.current, progress.units)
-+    return text
-+
-+def spinner(progress, show_text=False, output=sys.stderr):
-+    """
-+    Update a spinner progress indicator on an output
-+    :param progress: The progress to display
-+    :param show_text: If true, show text as well as spinner
-+    :param output: The output to write to
-+
-+    >>> spinner(Progress("baloons", 6), show_text=True, output=sys.stdout)
-+    \r- 6 baloons
-+    """
-+    output.write('\r%s' % spinner_str(progress, show_text))
-+
-+def run_tests():
-+    import doctest
-+    result = doctest.testmod()
-+    if result[1] > 0:
-+        if result[0] == 0:
-+            print "All tests passed"
-+    else:
-+        print "No tests to run"
-+if __name__ == "__main__":
-+    run_tests()
-

=== removed file 'patches/symlink-support.patch'
--- patches/symlink-support.patch	2005-05-17 07:01:47 +0000
+++ patches/symlink-support.patch	1970-01-01 00:00:00 +0000
@@ -1,629 +0,0 @@
-Return-Path: <erik@tntech.dk>
-X-Original-To: mbp@sourcefrog.net
-Delivered-To: mbp@ozlabs.org
-X-Greylist: delayed 1826 seconds by postgrey-1.21 at ozlabs; Sun, 15 May 2005 06:59:11 EST
-Received: from upstroke.tntech.dk (cpe.atm2-0-1041078.0x503eaf62.odnxx4.customer.tele.dk [80.62.175.98])
-	by ozlabs.org (Postfix) with ESMTP id B968E679EA
-	for <mbp@sourcefrog.net>; Sun, 15 May 2005 06:59:11 +1000 (EST)
-Received: by upstroke.tntech.dk (Postfix, from userid 1001)
-	id 63F83542FF; Sat, 14 May 2005 22:28:37 +0200 (CEST)
-To: Martin Pool <mbp@sourcefrog.net>
-Subject: [PATCH] symlink support patch
-From: Erik Toubro Nielsen <erik@tntech.dk>
-Date: Sat, 14 May 2005 22:28:37 +0200
-Message-ID: <86u0l57dsa.fsf@upstroke.tntech.dk>
-User-Agent: Gnus/5.1006 (Gnus v5.10.6) XEmacs/21.4 (Security Through
- Obscurity, linux)
-MIME-Version: 1.0
-Content-Type: multipart/mixed; boundary="=-=-="
-X-Spam-Checker-Version: SpamAssassin 3.0.3 (2005-04-27) on ozlabs.org
-X-Spam-Level: 
-X-Spam-Status: No, score=-1.4 required=3.2 tests=BAYES_00,NO_MORE_FUNN,
-	RCVD_IN_BLARS_RBL autolearn=no version=3.0.3
-Content-Length: 19688
-Lines: 604
-
---=-=-=
-
-I'm not sending this to the list as it is pretty large. 
-
-Let me know if its usefull and if I should rework anything.
-
-Erik
-
-
-Overview:
-
-        (Bugfix: in class TreeDelta I've moved kind= one level up, since
-        kind is also used in the else part)
-
-        Since both the InventoryEntry and stat cache is changed, perhaps
-        the branch format number should be increased?
-
-        Added test cases for symlinks to testbzr
-
-        Cannot use realpath since it expands path/L to path/LinkTarget
-        Cannot use exists, use new osutils.lexists
-
-        I'm overloading the text_modified to signify that a link        
-        target is changed. Perhaps text_modified should be renamed
-        content_modified?
-
-        InventoryEntry has a new member "symlink_target", 
-
-        The stat cache entry has been extended to contain the symlink
-        target and the st_mode. I try to ignore an old format cache
-        file.
-
-
-
---=-=-=
-Content-Type: text/x-patch
-Content-Disposition: inline; filename=symlinksupport.patch
-
-*** modified file 'bzrlib/add.py'
---- bzrlib/add.py 
-+++ bzrlib/add.py 
-@@ -38,7 +38,7 @@
-     for f in file_list:
-         kind = bzrlib.osutils.file_kind(f)
- 
--        if kind != 'file' and kind != 'directory':
-+        if kind != 'file' and kind != 'directory' and kind != 'symlink':
-             if f not in user_list:
-                 print "Skipping %s (can't add file of kind '%s')" % (f, kind)
-                 continue
-@@ -56,7 +56,7 @@
- 
-         kind = bzrlib.osutils.file_kind(f)
- 
--        if kind != 'file' and kind != 'directory':
-+        if kind != 'file' and kind != 'directory' and kind != 'symlink':
-             bailout("can't add file '%s' of kind %r" % (f, kind))
-             
-         versioned = (inv.path2id(rf) != None)
-
-*** modified file 'bzrlib/branch.py'
---- bzrlib/branch.py 
-+++ bzrlib/branch.py 
-@@ -58,11 +58,9 @@
-     run into the root."""
-     if f == None:
-         f = os.getcwd()
--    elif hasattr(os.path, 'realpath'):
--        f = os.path.realpath(f)
-     else:
--        f = os.path.abspath(f)
--    if not os.path.exists(f):
-+        f = bzrlib.osutils.normalizepath(f)
-+    if not bzrlib.osutils.lexists(f):
-         raise BzrError('%r does not exist' % f)
-         
- 
-@@ -189,7 +187,7 @@
-         """Return path relative to this branch of something inside it.
- 
-         Raises an error if path is not in this branch."""
--        rp = os.path.realpath(path)
-+        rp = bzrlib.osutils.normalizepath(path)
-         # FIXME: windows
-         if not rp.startswith(self.base):
-             bailout("path %r is not within branch %r" % (rp, self.base))
-@@ -531,7 +529,9 @@
-             file_id = entry.file_id
-             mutter('commit prep file %s, id %r ' % (p, file_id))
- 
--            if not os.path.exists(p):
-+            # it should be enough to use os.lexists instead of exists
-+            # but lexists in an 2.4 function
-+            if not bzrlib.osutils.lexists(p):
-                 mutter("    file is missing, removing from inventory")
-                 if verbose:
-                     show_status('D', entry.kind, quotefn(path))
-@@ -554,6 +554,10 @@
-             if entry.kind == 'directory':
-                 if not isdir(p):
-                     bailout("%s is entered as directory but not a directory" % quotefn(p))
-+            elif entry.kind == 'symlink':
-+                if not os.path.islink(p):
-+                    bailout("%s is entered as symbolic link but is not a symbolic link" % quotefn(p))
-+                entry.read_symlink_target(p)
-             elif entry.kind == 'file':
-                 if not isfile(p):
-                     bailout("%s is entered as file but is not a file" % quotefn(p))
-
-*** modified file 'bzrlib/diff.py'
---- bzrlib/diff.py 
-+++ bzrlib/diff.py 
-@@ -66,6 +66,11 @@
-         print >>to_file, "\\ No newline at end of file"
-     print >>to_file
- 
-+def _diff_symlink(old_tree, new_tree, file_id):
-+    t1 = old_tree.get_symlink_target(file_id)
-+    t2 = new_tree.get_symlink_target(file_id)
-+    print '*** *** target changed %r => %r' % (t1, t2)
-+    
- 
- def show_diff(b, revision, specific_files):
-     import sys
-@@ -112,12 +117,15 @@
- 
-     for old_path, new_path, file_id, kind, text_modified in delta.renamed:
-         print '*** renamed %s %r => %r' % (kind, old_path, new_path)
--        if text_modified:
-+        if kind == 'file' and text_modified:
-             _diff_one(old_tree.get_file(file_id).readlines(),
-                    new_tree.get_file(file_id).readlines(),
-                    sys.stdout,
-                    fromfile=old_label + old_path,
-                    tofile=new_label + new_path)
-+
-+        elif kind == 'symlink' and text_modified:
-+            _diff_symlink(old_tree, new_tree, file_id)
- 
-     for path, file_id, kind in delta.modified:
-         print '*** modified %s %r' % (kind, path)
-@@ -128,6 +136,8 @@
-                    fromfile=old_label + path,
-                    tofile=new_label + path)
- 
-+        elif kind == 'symlink':
-+            _diff_symlink(old_tree, new_tree, file_id)
- 
- 
- class TreeDelta:
-@@ -149,7 +159,9 @@
-     Each id is listed only once.
- 
-     Files that are both modified and renamed are listed only in
--    renamed, with the text_modified flag true.
-+    renamed, with the text_modified flag true. The text_modified
-+    applies either to the the content of the file or the target of the
-+    symbolic link, depending of the kind of file.
- 
-     The lists are normally sorted when the delta is created.
-     """
-@@ -224,8 +236,8 @@
-         specific_files = ImmutableSet(specific_files)
- 
-     for file_id in old_tree:
-+        kind = old_inv.get_file_kind(file_id)
-         if file_id in new_tree:
--            kind = old_inv.get_file_kind(file_id)
-             assert kind == new_inv.get_file_kind(file_id)
-             
-             assert kind in ('file', 'directory', 'symlink', 'root_directory'), \
-@@ -246,6 +258,14 @@
-                 old_sha1 = old_tree.get_file_sha1(file_id)
-                 new_sha1 = new_tree.get_file_sha1(file_id)
-                 text_modified = (old_sha1 != new_sha1)
-+            elif kind == 'symlink':
-+                t1 = old_tree.get_symlink_target(file_id)
-+                t2 = new_tree.get_symlink_target(file_id)
-+                if t1 != t2:
-+                    mutter("    symlink target changed")
-+                    text_modified = True
-+                else:
-+                    text_modified = False
-             else:
-                 ## mutter("no text to check for %r %r" % (file_id, kind))
-                 text_modified = False
-
-*** modified file 'bzrlib/inventory.py'
---- bzrlib/inventory.py 
-+++ bzrlib/inventory.py 
-@@ -125,14 +125,22 @@
-         self.kind = kind
-         self.text_id = text_id
-         self.parent_id = parent_id
-+        self.symlink_target = None
-         if kind == 'directory':
-             self.children = {}
-         elif kind == 'file':
-             pass
-+        elif kind == 'symlink':
-+            pass
-         else:
-             raise BzrError("unhandled entry kind %r" % kind)
- 
--
-+    def read_symlink_target(self, path):
-+        if self.kind == 'symlink':
-+            try:
-+                self.symlink_target = os.readlink(path)
-+            except OSError,e:
-+                raise BzrError("os.readlink error, %s" % e)
- 
-     def sorted_children(self):
-         l = self.children.items()
-@@ -145,6 +153,7 @@
-                                self.parent_id, text_id=self.text_id)
-         other.text_sha1 = self.text_sha1
-         other.text_size = self.text_size
-+        other.symlink_target = self.symlink_target
-         return other
- 
- 
-@@ -168,7 +177,7 @@
-         if self.text_size != None:
-             e.set('text_size', '%d' % self.text_size)
-             
--        for f in ['text_id', 'text_sha1']:
-+        for f in ['text_id', 'text_sha1', 'symlink_target']:
-             v = getattr(self, f)
-             if v != None:
-                 e.set(f, v)
-@@ -198,6 +207,7 @@
-         self = cls(elt.get('file_id'), elt.get('name'), elt.get('kind'), parent_id)
-         self.text_id = elt.get('text_id')
-         self.text_sha1 = elt.get('text_sha1')
-+        self.symlink_target = elt.get('symlink_target')
-         
-         ## mutter("read inventoryentry: %r" % (elt.attrib))
- 
-
-*** modified file 'bzrlib/osutils.py'
---- bzrlib/osutils.py 
-+++ bzrlib/osutils.py 
-@@ -58,7 +58,30 @@
-     else:
-         raise BzrError("can't handle file kind with mode %o of %r" % (mode, f)) 
- 
--
-+def lexists(f):
-+    try:
-+        if hasattr(os, 'lstat'):
-+            os.lstat(f)
-+        else:
-+            os.stat(f)
-+        return True
-+    except OSError,e:
-+        if e.errno == errno.ENOENT:
-+            return False;
-+        else:
-+            raise BzrError("lstat/stat of (%r): %r" % (f, e))
-+
-+def normalizepath(f):
-+    if hasattr(os.path, 'realpath'):
-+        F = os.path.realpath
-+    else:
-+        F = os.path.abspath
-+    [p,e] = os.path.split(f)
-+    if e == "" or e == "." or e == "..":
-+        return F(f)
-+    else:
-+        return os.path.join(F(p), e)
-+    
- 
- def isdir(f):
-     """True if f is an accessible directory."""
-
-*** modified file 'bzrlib/statcache.py'
---- bzrlib/statcache.py 
-+++ bzrlib/statcache.py 
-@@ -53,7 +53,7 @@
- to use a tdb instead.
- 
- The cache is represented as a map from file_id to a tuple of (file_id,
--sha1, path, size, mtime, ctime, ino, dev).
-+sha1, path, symlink target, size, mtime, ctime, ino, dev, mode).
- """
- 
- 
-@@ -62,11 +62,13 @@
- FP_CTIME = 2
- FP_INO   = 3
- FP_DEV   = 4
--
-+FP_ST_MODE=5
- 
- SC_FILE_ID = 0
- SC_SHA1    = 1 
--
-+SC_SYMLINK_TARGET = 3
-+
-+CACHE_ENTRY_SIZE = 10
- 
- def fingerprint(abspath):
-     try:
-@@ -79,7 +81,7 @@
-         return None
- 
-     return (fs.st_size, fs.st_mtime,
--            fs.st_ctime, fs.st_ino, fs.st_dev)
-+            fs.st_ctime, fs.st_ino, fs.st_dev, fs.st_mode)
- 
- 
- def _write_cache(basedir, entry_iter, dangerfiles):
-@@ -93,7 +95,9 @@
-                 continue
-             outf.write(entry[0] + ' ' + entry[1] + ' ')
-             outf.write(b2a_qp(entry[2], True))
--            outf.write(' %d %d %d %d %d\n' % entry[3:])
-+            outf.write(' ')
-+            outf.write(b2a_qp(entry[3], True)) # symlink_target
-+            outf.write(' %d %d %d %d %d %d\n' % entry[4:])
- 
-         outf.commit()
-     finally:
-@@ -114,10 +118,13 @@
-     
-     for l in cachefile:
-         f = l.split(' ')
-+        if len(f) != CACHE_ENTRY_SIZE:
-+            mutter("cache is in old format, must recreate it")
-+            return {}
-         file_id = f[0]
-         if file_id in cache:
-             raise BzrError("duplicated file_id in cache: {%s}" % file_id)
--        cache[file_id] = (f[0], f[1], a2b_qp(f[2])) + tuple([long(x) for x in f[3:]])
-+        cache[file_id] = (f[0], f[1], a2b_qp(f[2]), a2b_qp(f[3])) + tuple([long(x) for x in f[4:]])
-     return cache
- 
- 
-@@ -125,7 +132,7 @@
- 
- def _files_from_inventory(inv):
-     for path, ie in inv.iter_entries():
--        if ie.kind != 'file':
-+        if ie.kind != 'file' and ie.kind != 'symlink':
-             continue
-         yield ie.file_id, path
-     
-@@ -190,17 +197,24 @@
-         if (fp[FP_MTIME] >= now) or (fp[FP_CTIME] >= now):
-             dangerfiles.add(file_id)
- 
--        if cacheentry and (cacheentry[3:] == fp):
-+        if cacheentry and (cacheentry[4:] == fp):
-             continue                    # all stat fields unchanged
- 
-         hardcheck += 1
- 
--        dig = sha.new(file(abspath, 'rb').read()).hexdigest()
--
-+        mode = fp[FP_ST_MODE]
-+        if stat.S_ISREG(mode):
-+            link_target = '-' # can be anything, but must be non-empty
-+            dig = sha.new(file(abspath, 'rb').read()).hexdigest()
-+        elif stat.S_ISLNK(mode):
-+            link_target = os.readlink(abspath)
-+            dig = sha.new(link_target).hexdigest()
-+        else:
-+            raise BzrError("file %r: unknown file stat mode: %o"%(abspath,mode))
-         if cacheentry == None or dig != cacheentry[1]: 
-             # if there was no previous entry for this file, or if the
-             # SHA has changed, then update the cache
--            cacheentry = (file_id, dig, path) + fp
-+            cacheentry = (file_id, dig, path, link_target) + fp
-             cache[file_id] = cacheentry
-             change_cnt += 1
- 
-
-*** modified file 'bzrlib/tree.py'
---- bzrlib/tree.py 
-+++ bzrlib/tree.py 
-@@ -125,6 +125,11 @@
-                 os.mkdir(fullpath)
-             elif kind == 'file':
-                 pumpfile(self.get_file(ie.file_id), file(fullpath, 'wb'))
-+            elif kind == 'symlink':
-+                try:
-+                    os.symlink(ie.symlink_target, fullpath)
-+                except OSError,e:
-+                    bailout("Failed to create symlink %r -> %r, error: %s" % (fullpath, ie.symlink_target, e))
-             else:
-                 bailout("don't know how to export {%s} of kind %r" % (ie.file_id, kind))
-             mutter("  export {%s} kind %s to %s" % (ie.file_id, kind, fullpath))
-@@ -167,6 +172,9 @@
-         for path, entry in self.inventory.iter_entries():
-             yield path, 'V', entry.kind, entry.file_id
- 
-+    def get_symlink_target(self, file_id):
-+        ie = self._inventory[file_id]
-+        return ie.symlink_target;
- 
- class EmptyTree(Tree):
-     def __init__(self):
-@@ -179,7 +187,8 @@
-         if False:  # just to make it a generator
-             yield None
-     
--
-+    def get_symlink_target(self, file_id):
-+        return None
- 
- ######################################################################
- # diff
-@@ -245,3 +254,7 @@
-         if old_name != new_name:
-             yield (old_name, new_name)
-             
-+
-+    def get_symlink_target(self, file_id):
-+        ie = self._inventory[file_id]        
-+        return ie.symlink_target
-
-*** modified file 'bzrlib/workingtree.py'
---- bzrlib/workingtree.py 
-+++ bzrlib/workingtree.py 
-@@ -53,7 +53,7 @@
-             ie = inv[file_id]
-             if ie.kind == 'file':
-                 if ((file_id in self._statcache)
--                    or (os.path.exists(self.abspath(inv.id2path(file_id))))):
-+                    or (bzrlib.osutils.lexists(self.abspath(inv.id2path(file_id))))):
-                     yield file_id
- 
- 
-@@ -66,7 +66,7 @@
-         return os.path.join(self.basedir, filename)
- 
-     def has_filename(self, filename):
--        return os.path.exists(self.abspath(filename))
-+        return bzrlib.osutils.lexists(self.abspath(filename))
- 
-     def get_file(self, file_id):
-         return self.get_file_byname(self.id2path(file_id))
-@@ -86,7 +86,7 @@
-         self._update_statcache()
-         if file_id in self._statcache:
-             return True
--        return os.path.exists(self.abspath(self.id2path(file_id)))
-+        return bzrlib.osutils.lexists(self.abspath(self.id2path(file_id)))
- 
- 
-     __contains__ = has_id
-@@ -108,6 +108,12 @@
-         return self._statcache[file_id][statcache.SC_SHA1]
- 
- 
-+    def get_symlink_target(self, file_id):
-+        import statcache
-+        self._update_statcache()
-+        target = self._statcache[file_id][statcache.SC_SYMLINK_TARGET]
-+        return target
-+        
-     def file_class(self, filename):
-         if self.path2id(filename):
-             return 'V'
-
-*** modified file 'testbzr'
---- testbzr 
-+++ testbzr 
-@@ -1,4 +1,4 @@
--#! /usr/bin/python
-+#! /usr/bin/env python
- 
- # Copyright (C) 2005 Canonical Ltd
- 
-@@ -113,6 +113,17 @@
-     logfile.write('   at %s:%d\n' % stack[:2])
- 
- 
-+def has_symlinks():
-+    import os;
-+    if hasattr(os, 'symlink'):
-+        return True
-+    else:
-+        return False
-+
-+def listdir_sorted(dir):
-+    L = os.listdir(dir)
-+    L.sort()
-+    return L
- 
- # prepare an empty scratch directory
- if os.path.exists(TESTDIR):
-@@ -320,8 +331,105 @@
-     runcmd('bzr ignore *.blah')
-     assert backtick('bzr unknowns') == ''
-     assert file('.bzrignore', 'rt').read() == '*.blah\n'
--
--
-+    cd("..")
-+    
-+    if has_symlinks():
-+        progress("symlinks")
-+        mkdir('symlinks')
-+        cd('symlinks')
-+        runcmd('bzr init')
-+        os.symlink("NOWHERE1", "link1")
-+        runcmd('bzr add link1')
-+        assert backtick('bzr unknowns') == ''
-+        runcmd(['bzr', 'commit', '-m', '1: added symlink link1'])
-+
-+        mkdir('d1')
-+        runcmd('bzr add d1')
-+        assert backtick('bzr unknowns') == ''
-+        os.symlink("NOWHERE2", "d1/link2")
-+        assert backtick('bzr unknowns') == 'd1/link2\n'
-+        # is d1/link2 found when adding d1
-+        runcmd('bzr add d1')
-+        assert backtick('bzr unknowns') == ''
-+        os.symlink("NOWHERE3", "d1/link3")
-+        assert backtick('bzr unknowns') == 'd1/link3\n'
-+        runcmd(['bzr', 'commit', '-m', '2: added dir, symlink'])
-+
-+        runcmd('bzr rename d1 d2')
-+        runcmd('bzr move d2/link2 .')
-+        runcmd('bzr move link1 d2')
-+        assert os.readlink("./link2") == "NOWHERE2"
-+        assert os.readlink("d2/link1") == "NOWHERE1"
-+        runcmd('bzr add d2/link3')
-+        runcmd('bzr diff')
-+        runcmd(['bzr', 'commit', '-m', '3: rename of dir, move symlinks, add link3'])
-+
-+        os.unlink("link2")
-+        os.symlink("TARGET 2", "link2")
-+        os.unlink("d2/link1")
-+        os.symlink("TARGET 1", "d2/link1")
-+        runcmd('bzr diff')
-+        assert backtick("bzr relpath d2/link1") == "d2/link1\n"
-+        runcmd(['bzr', 'commit', '-m', '4: retarget of two links'])
-+
-+        runcmd('bzr remove d2/link1')
-+        assert backtick('bzr unknowns') == 'd2/link1\n'
-+        runcmd(['bzr', 'commit', '-m', '5: remove d2/link1'])
-+
-+        os.mkdir("d1")
-+        runcmd('bzr add d1')
-+        runcmd('bzr rename d2/link3 d1/link3new')
-+        assert backtick('bzr unknowns') == 'd2/link1\n'
-+        runcmd(['bzr', 'commit', '-m', '6: remove d2/link1, move/rename link3'])
-+        
-+        runcmd(['bzr', 'check'])
-+        
-+        runcmd(['bzr', 'export', '-r', '1', 'exp1.tmp'])
-+        cd("exp1.tmp")
-+        assert listdir_sorted(".") == [ "link1" ]
-+        assert os.readlink("link1") == "NOWHERE1"
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '2', 'exp2.tmp'])
-+        cd("exp2.tmp")
-+        assert listdir_sorted(".") == [ "d1", "link1" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '3', 'exp3.tmp'])
-+        cd("exp3.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert listdir_sorted("d2") == [ "link1", "link3" ]
-+        assert os.readlink("d2/link1") == "NOWHERE1"
-+        assert os.readlink("link2")    == "NOWHERE2"
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '4', 'exp4.tmp'])
-+        cd("exp4.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert os.readlink("d2/link1") == "TARGET 1"
-+        assert os.readlink("link2")    == "TARGET 2"
-+        assert listdir_sorted("d2") == [ "link1", "link3" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '5', 'exp5.tmp'])
-+        cd("exp5.tmp")
-+        assert listdir_sorted(".") == [ "d2", "link2" ]
-+        assert os.path.islink("link2")
-+        assert listdir_sorted("d2")== [ "link3" ]
-+        cd("..")
-+        
-+        runcmd(['bzr', 'export', '-r', '6', 'exp6.tmp'])
-+        cd("exp6.tmp")
-+        assert listdir_sorted(".") == [ "d1", "d2", "link2" ]
-+        assert listdir_sorted("d1") == [ "link3new" ]
-+        assert listdir_sorted("d2") == []
-+        assert os.readlink("d1/link3new") == "NOWHERE3"
-+        cd("..")
-+
-+        cd("..")
-+    else:
-+        progress("skipping symlink tests")
-+        
-     progress("all tests passed!")
- except Exception, e:
-     sys.stderr.write('*' * 50 + '\n'
-
-
---=-=-=--

=== removed directory 'plugins/changeset'
=== removed file 'plugins/changeset/__init__.py'
--- plugins/changeset/__init__.py	2005-06-22 09:34:59 +0000
+++ plugins/changeset/__init__.py	1970-01-01 00:00:00 +0000
@@ -1,105 +0,0 @@
-#!/usr/bin/env python
-"""\
-This is an attempt to take the internal delta object, and represent
-it as a single-file text-only changeset.
-This should have commands for both generating a changeset,
-and for applying a changeset.
-"""
-
-import bzrlib, bzrlib.commands
-
-class cmd_changeset(bzrlib.commands.Command):
-    """Generate a bundled up changeset.
-
-    This changeset contains all of the meta-information of a
-    diff, rather than just containing the patch information.
-
-    If a file is specified, only changes affecting that file are
-    specified; this gives a changeset that is probably not useful.
-
-    Right now, rollup changesets, or working tree changesets are
-    not supported. This will only generate a changeset that has been
-    committed. You can use "--revision" to specify a certain change
-    to display.
-    """
-    takes_options = ['revision', 'diff-options']
-    takes_args = ['file*']
-    aliases = ['cset']
-
-    def run(self, revision=None, file_list=None, diff_options=None):
-        from bzrlib import find_branch
-        import gen_changeset
-        import sys
-
-        if isinstance(revision, (list, tuple)):
-            if len(revision) > 1:
-                raise BzrCommandError('We do not support rollup-changesets yet.')
-            revision = revision[0]
-        if file_list:
-            b = find_branch(file_list[0])
-            file_list = [b.relpath(f) for f in file_list]
-            if file_list == ['']:
-                # just pointing to top-of-tree
-                file_list = None
-        else:
-            b = find_branch('.')
-
-        gen_changeset.show_changeset(b, revision,
-                specific_files=file_list,
-                external_diff_options=diff_options,
-                to_file=sys.stdout)
-
-class cmd_verify_changeset(bzrlib.commands.Command):
-    """Read a written changeset, and make sure it is valid.
-
-    """
-    takes_args = ['filename?']
-
-    def run(self, filename=None):
-        import sys, read_changeset
-        if filename is None or filename == '-':
-            f = sys.stdin
-        else:
-            f = open(filename, 'rb')
-
-        cset_info = read_changeset.read_changeset(f)
-        print cset_info
-        cset = cset_info.get_changeset()
-        print cset.entries
-
-class cmd_apply_changeset(bzrlib.commands.Command):
-    """Read in the given changeset, and apply it to the
-    current tree.
-
-    """
-    takes_args = ['filename?']
-    takes_options = []
-
-    def run(self, filename=None, reverse=False, auto_commit=False):
-        from bzrlib import find_branch
-        import sys
-        import apply_changeset
-
-        b = find_branch('.') # Make sure we are in a branch
-        if filename is None or filename == '-':
-            f = sys.stdin
-        else:
-            f = open(filename, 'rb')
-
-        apply_changeset.apply_changeset(b, f, reverse=reverse,
-                auto_commit=auto_commit)
-
-
-##if hasattr(bzrlib.commands, 'register_plugin_cmd'):
-
-## print dir(bzrlib.commands)
-
-bzrlib.commands.register_command(cmd_changeset)
-bzrlib.commands.register_command(cmd_verify_changeset)
-bzrlib.commands.register_command(cmd_apply_changeset)
-
-bzrlib.commands.OPTIONS['reverse'] = None
-bzrlib.commands.OPTIONS['auto-commit'] = None
-cmd_apply_changeset.takes_options.append('reverse')
-cmd_apply_changeset.takes_options.append('auto-commit')
-

=== removed file 'plugins/changeset/apply_changeset.py'
--- plugins/changeset/apply_changeset.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/apply_changeset.py	1970-01-01 00:00:00 +0000
@@ -1,41 +0,0 @@
-#!/usr/bin/env python
-"""\
-This contains the apply changset function for bzr
-"""
-
-import bzrlib
-
-def apply_changeset(branch, from_file, reverse=False, auto_commit=False):
-    from bzrlib.changeset import apply_changeset as _apply_changeset
-    from bzrlib.merge import regen_inventory
-    import sys, read_changeset
-
-
-    cset_info = read_changeset.read_changeset(from_file)
-    cset = cset_info.get_changeset()
-    inv = {}
-    for file_id in branch.inventory:
-        inv[file_id] = branch.inventory.id2path(file_id)
-    changes = _apply_changeset(cset, inv, branch.base,
-            reverse=reverse)
-
-    adjust_ids = []
-    for id, path in changes.iteritems():
-        if path is not None:
-            if path == '.':
-                path = ''
-        adjust_ids.append((path, id))
-
-    branch.set_inventory(regen_inventory(branch, branch.base, adjust_ids))
-
-    if auto_commit:
-        from bzrlib.commit import commit
-        if branch.last_patch() == cset_info.precursor:
-            # This patch can be applied directly
-            commit(branch, message = cset_info.message,
-                    timestamp=float(cset_info.timestamp),
-                    timezone=float(cset_info.timezone),
-                    committer=cset_info.committer,
-                    rev_id=cset_info.revision)
-
-

=== removed file 'plugins/changeset/common.py'
--- plugins/changeset/common.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/common.py	1970-01-01 00:00:00 +0000
@@ -1,15 +0,0 @@
-#!/usr/bin/env python
-"""\
-Common entries, like strings, etc, for the changeset reading + writing code.
-"""
-
-header_str = 'Bazaar-NG (bzr) changeset v'
-version = (0, 0, 5)
-
-def get_header():
-    return [
-        header_str + '.'.join([str(v) for v in version]),
-        'This changeset can be applied with bzr apply-changeset',
-        ''
-    ]
-

=== removed file 'plugins/changeset/gen_changeset.py'
--- plugins/changeset/gen_changeset.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/gen_changeset.py	1970-01-01 00:00:00 +0000
@@ -1,345 +0,0 @@
-#!/usr/bin/env python
-"""\
-Just some work for generating a changeset.
-"""
-
-import bzrlib, bzrlib.errors
-
-import common
-
-from bzrlib.inventory import ROOT_ID
-
-try:
-    set
-except NameError:
-    from sets import Set as set
-
-def _canonicalize_revision(branch, revno):
-    """Turn some sort of revision information into a single
-    set of from-to revision ids.
-
-    A revision id can be None if there is no associated revison.
-
-    :return: (old, new)
-    """
-    # This is a little clumsy because revision parsing may return
-    # a single entry, or a list
-    if revno is None:
-        new = branch.last_patch()
-    else:
-        new = branch.lookup_revision(revno)
-
-    if new is None:
-        raise BzrCommandError('Cannot generate a changset with no commits in tree.')
-
-    old = branch.get_revision(new).precursor
-
-    return old, new
-
-def _get_trees(branch, revisions):
-    """Get the old and new trees based on revision.
-    """
-    from bzrlib.tree import EmptyTree
-    if revisions[0] is None:
-        if hasattr(branch, 'get_root_id'): # Watch out for trees with labeled ROOT ids
-            old_tree = EmptyTree(branch.get_root_id) 
-        else:
-            old_tree = EmptyTree()
-    else:
-        old_tree = branch.revision_tree(revisions[0])
-
-    if revisions[1] is None:
-        # This is for the future, once we support rollup revisions
-        # Or working tree revisions
-        new_tree = branch.working_tree()
-    else:
-        new_tree = branch.revision_tree(revisions[1])
-    return old_tree, new_tree
-
-def _fake_working_revision(branch):
-    """Fake a Revision object for the working tree.
-    
-    This is for the future, to support changesets against the working tree.
-    """
-    from bzrlib.revision import Revision
-    import time
-    from bzrlib.osutils import local_time_offset, \
-            username
-
-    precursor = branch.last_patch()
-    precursor_sha1 = branch.get_revision_sha1(precursor)
-
-    return Revision(timestamp=time.time(),
-            timezone=local_time_offset(),
-            committer=username(),
-            precursor=precursor,
-            precursor_sha1=precursor_sha1)
-
-
-class MetaInfoHeader(object):
-    """Maintain all of the header information about this
-    changeset.
-    """
-
-    def __init__(self, branch, revisions, delta,
-            full_remove=True, full_rename=False,
-            external_diff_options = None,
-            new_tree=None, old_tree=None,
-            old_label = '', new_label = ''):
-        """
-        :param full_remove: Include the full-text for a delete
-        :param full_rename: Include an add+delete patch for a rename
-        """
-        self.branch = branch
-        self.delta = delta
-        self.full_remove=full_remove
-        self.full_rename=full_rename
-        self.external_diff_options = external_diff_options
-        self.old_label = old_label
-        self.new_label = new_label
-        self.old_tree = old_tree
-        self.new_tree = new_tree
-        self.to_file = None
-        self.revno = None
-        self.precursor_revno = None
-
-        self._get_revision_list(revisions)
-
-    def _get_revision_list(self, revisions):
-        """This generates the list of all revisions from->to.
-
-        This is for the future, when we support having a rollup changeset.
-        For now, the list should only be one long.
-        """
-        old_revno = None
-        new_revno = None
-        rh = self.branch.revision_history()
-        for revno, rev in enumerate(rh):
-            if rev == revisions[0]:
-                old_revno = revno
-            if rev == revisions[1]:
-                new_revno = revno
-
-        self.revision_list = []
-        if old_revno is None:
-            self.base_revision = None # Effectively the EmptyTree()
-            old_revno = -1
-        else:
-            self.base_revision = self.branch.get_revision(rh[old_revno])
-        if new_revno is None:
-            # For the future, when we support working tree changesets.
-            for rev_id in rh[old_revno+1:]:
-                self.revision_list.append(self.branch.get_revision(rev_id))
-            self.revision_list.append(_fake_working_revision(self.branch))
-        else:
-            for rev_id in rh[old_revno+1:new_revno+1]:
-                self.revision_list.append(self.branch.get_revision(rev_id))
-        self.precursor_revno = old_revno
-        self.revno = new_revno
-
-    def _write(self, txt, key=None):
-        if key:
-            self.to_file.write('# %s: %s\n' % (key, txt))
-        else:
-            self.to_file.write('# %s\n' % (txt,))
-
-    def write_meta_info(self, to_file):
-        """Write out the meta-info portion to the supplied file.
-
-        :param to_file: Write out the meta information to the supplied
-                        file
-        """
-        self.to_file = to_file
-
-        self._write_header()
-        self._write_diffs()
-        self._write_footer()
-
-    def _write_header(self):
-        """Write the stuff that comes before the patches."""
-        from bzrlib.osutils import username, format_date
-        write = self._write
-
-        for line in common.get_header():
-            write(line)
-
-        # This grabs the current username, what we really want is the
-        # username from the actual patches.
-        #write(username(), key='committer')
-        assert len(self.revision_list) == 1
-        rev = self.revision_list[0]
-        write(rev.committer, key='committer')
-        write(format_date(rev.timestamp, offset=rev.timezone), key='date')
-        write(str(self.revno), key='revno')
-        if rev.message:
-            self.to_file.write('# message:\n')
-            for line in rev.message.split('\n'):
-                self.to_file.write('#    %s\n' % line)
-        write(rev.revision_id, key='revision')
-
-        if self.base_revision:
-            write(self.base_revision.revision_id, key='precursor')
-            write(str(self.precursor_revno), key='precursor revno')
-
-
-        write('')
-        self.to_file.write('\n')
-
-    def _write_footer(self):
-        """Write the stuff that comes after the patches.
-
-        This is meant to be more meta-information, which people probably don't want
-        to read, but which is required for proper bzr operation.
-        """
-        write = self._write
-
-        write('BEGIN BZR FOOTER')
-
-        assert len(self.revision_list) == 1 # We only handle single revision entries
-        rev = self.revision_list[0]
-        write(self.branch.get_revision_sha1(rev.revision_id),
-                key='revision sha1')
-        if self.base_revision:
-            rev_id = self.base_revision.revision_id
-            write(self.branch.get_revision_sha1(rev_id),
-                    key='precursor sha1')
-
-        write('%.9f' % rev.timestamp, key='timestamp')
-        write(str(rev.timezone), key='timezone')
-
-        self._write_ids()
-
-        write('END BZR FOOTER')
-
-    def _write_revisions(self):
-        """Not used. Used for writing multiple revisions."""
-        first = True
-        for rev in self.revision_list:
-            if rev.revision_id is not None:
-                if first:
-                    self._write('revisions:')
-                    first = False
-                self._write(' '*4 + rev.revision_id + '\t' + self.branch.get_revision_sha1(rev.revision_id))
-
-
-    def _write_ids(self):
-        if hasattr(self.branch, 'get_root_id'):
-            root_id = self.branch.get_root_id()
-        else:
-            root_id = ROOT_ID
-
-        old_ids = set()
-        new_ids = set()
-
-        for path, file_id, kind in self.delta.removed:
-            old_ids.add(file_id)
-        for path, file_id, kind in self.delta.added:
-            new_ids.add(file_id)
-        for old_path, new_path, file_id, kind, text_modified in self.delta.renamed:
-            old_ids.add(file_id)
-            new_ids.add(file_id)
-        for path, file_id, kind in self.delta.modified:
-            new_ids.add(file_id)
-
-        self._write(root_id, key='tree root id')
-
-        def write_ids(tree, id_set, name):
-            if len(id_set) > 0:
-                self.to_file.write('# %s ids:\n' % name)
-            seen_ids = set([root_id])
-            while len(id_set) > 0:
-                file_id = id_set.pop()
-                if file_id in seen_ids:
-                    continue
-                seen_ids.add(file_id)
-                ie = tree.inventory[file_id]
-                if ie.parent_id not in seen_ids:
-                    id_set.add(ie.parent_id)
-                path = tree.inventory.id2path(file_id)
-                self.to_file.write('#    %s\t%s\t%s\n'
-                        % (path.encode('utf8'), file_id.encode('utf8'),
-                            ie.parent_id.encode('utf8')))
-        write_ids(self.new_tree, new_ids, 'file')
-        write_ids(self.old_tree, old_ids, 'old file')
-
-    def _write_diffs(self):
-        """Write out the specific diffs"""
-        from bzrlib.diff import internal_diff, external_diff
-        DEVNULL = '/dev/null'
-
-        if self.external_diff_options:
-            assert isinstance(self.external_diff_options, basestring)
-            opts = self.external_diff_options.split()
-            def diff_file(olab, olines, nlab, nlines, to_file):
-                external_diff(olab, olines, nlab, nlines, to_file, opts)
-        else:
-            diff_file = internal_diff
-
-        for path, file_id, kind in self.delta.removed:
-            print >>self.to_file, '*** removed %s %r' % (kind, path)
-            if kind == 'file' and self.full_remove:
-                diff_file(self.old_label + path,
-                          self.old_tree.get_file(file_id).readlines(),
-                          DEVNULL, 
-                          [],
-                          self.to_file)
-    
-        for path, file_id, kind in self.delta.added:
-            print >>self.to_file, '*** added %s %r' % (kind, path)
-            if kind == 'file':
-                diff_file(DEVNULL,
-                          [],
-                          self.new_label + path,
-                          self.new_tree.get_file(file_id).readlines(),
-                          self.to_file)
-    
-        for old_path, new_path, file_id, kind, text_modified in self.delta.renamed:
-            print >>self.to_file, '*** renamed %s %r => %r' % (kind, old_path, new_path)
-            if self.full_rename and kind == 'file':
-                diff_file(self.old_label + old_path,
-                          self.old_tree.get_file(file_id).readlines(),
-                          DEVNULL, 
-                          [],
-                          self.to_file)
-                diff_file(DEVNULL,
-                          [],
-                          self.new_label + new_path,
-                          self.new_tree.get_file(file_id).readlines(),
-                          self.to_file)
-            elif text_modified:
-                    diff_file(self.old_label + old_path,
-                              self.old_tree.get_file(file_id).readlines(),
-                              self.new_label + new_path,
-                              self.new_tree.get_file(file_id).readlines(),
-                              self.to_file)
-    
-        for path, file_id, kind in self.delta.modified:
-            print >>self.to_file, '*** modified %s %r' % (kind, path)
-            if kind == 'file':
-                diff_file(self.old_label + path,
-                          self.old_tree.get_file(file_id).readlines(),
-                          self.new_label + path,
-                          self.new_tree.get_file(file_id).readlines(),
-                          self.to_file)
-
-def show_changeset(branch, revision=None, specific_files=None,
-        external_diff_options=None, to_file=None,
-        include_full_diff=False):
-    from bzrlib.diff import compare_trees
-
-    if to_file is None:
-        import sys
-        to_file = sys.stdout
-    revisions = _canonicalize_revision(branch, revision)
-
-    old_tree, new_tree = _get_trees(branch, revisions)
-
-    delta = compare_trees(old_tree, new_tree, want_unchanged=False,
-                          specific_files=specific_files)
-
-    meta = MetaInfoHeader(branch, revisions, delta,
-            external_diff_options=external_diff_options,
-            old_tree=old_tree, new_tree=new_tree)
-    meta.write_meta_info(to_file)
-
-

=== removed file 'plugins/changeset/read_changeset.py'
--- plugins/changeset/read_changeset.py	2005-06-22 09:08:43 +0000
+++ plugins/changeset/read_changeset.py	1970-01-01 00:00:00 +0000
@@ -1,342 +0,0 @@
-#!/usr/bin/env python
-"""\
-Read in a changeset output, and process it into a Changeset object.
-"""
-
-import bzrlib, bzrlib.changeset
-import common
-
-class BadChangeset(Exception): pass
-class MalformedHeader(BadChangeset): pass
-class MalformedPatches(BadChangeset): pass
-class MalformedFooter(BadChangeset): pass
-
-def _unescape(name):
-    """Now we want to find the filename effected.
-    Unfortunately the filename is written out as
-    repr(filename), which means that it surrounds
-    the name with quotes which may be single or double
-    (single is preferred unless there is a single quote in
-    the filename). And some characters will be escaped.
-
-    TODO:   There has to be some pythonic way of undo-ing the
-            representation of a string rather than using eval.
-    """
-    delimiter = name[0]
-    if name[-1] != delimiter:
-        raise BadChangeset('Could not properly parse the'
-                ' filename: %r' % name)
-    # We need to handle escaped hexadecimals too.
-    return name[1:-1].replace('\"', '"').replace("\'", "'")
-
-class ChangesetInfo(object):
-    """This is the intermediate class that gets filled out as
-    the file is read.
-    """
-    def __init__(self):
-        self.committer = None
-        self.date = None
-        self.message = None
-        self.revno = None
-        self.revision = None
-        self.revision_sha1 = None
-        self.precursor = None
-        self.precursor_sha1 = None
-        self.precursor_revno = None
-
-        self.timestamp = None
-        self.timezone = None
-
-        self.tree_root_id = None
-        self.file_ids = None
-        self.old_file_ids = None
-
-        self.actions = [] #this is the list of things that happened
-        self.id2path = {} # A mapping from file id to path name
-        self.path2id = {} # The reverse mapping
-        self.id2parent = {} # A mapping from a given id to it's parent id
-
-        self.old_id2path = {}
-        self.old_path2id = {}
-        self.old_id2parent = {}
-
-    def __str__(self):
-        import pprint
-        return pprint.pformat(self.__dict__)
-
-    def create_maps(self):
-        """Go through the individual id sections, and generate the 
-        id2path and path2id maps.
-        """
-        # Rather than use an empty path, the changeset code seems 
-        # to like to use "./." for the tree root.
-        self.id2path[self.tree_root_id] = './.'
-        self.path2id['./.'] = self.tree_root_id
-        self.id2parent[self.tree_root_id] = bzrlib.changeset.NULL_ID
-        self.old_id2path = self.id2path.copy()
-        self.old_path2id = self.path2id.copy()
-        self.old_id2parent = self.id2parent.copy()
-
-        if self.file_ids:
-            for info in self.file_ids:
-                path, f_id, parent_id = info.split('\t')
-                self.id2path[f_id] = path
-                self.path2id[path] = f_id
-                self.id2parent[f_id] = parent_id
-        if self.old_file_ids:
-            for info in self.old_file_ids:
-                path, f_id, parent_id = info.split('\t')
-                self.old_id2path[f_id] = path
-                self.old_path2id[path] = f_id
-                self.old_id2parent[f_id] = parent_id
-
-    def get_changeset(self):
-        """Create a changeset from the data contained within."""
-        from bzrlib.changeset import Changeset, ChangesetEntry, \
-            PatchApply, ReplaceContents
-        cset = Changeset()
-        
-        entry = ChangesetEntry(self.tree_root_id, 
-                bzrlib.changeset.NULL_ID, './.')
-        cset.add_entry(entry)
-        for info, lines in self.actions:
-            parts = info.split(' ')
-            action = parts[0]
-            kind = parts[1]
-            extra = ' '.join(parts[2:])
-            if action == 'renamed':
-                old_path, new_path = extra.split(' => ')
-                old_path = _unescape(old_path)
-                new_path = _unescape(new_path)
-
-                new_id = self.path2id[new_path]
-                old_id = self.old_path2id[old_path]
-                assert old_id == new_id
-
-                new_parent = self.id2parent[new_id]
-                old_parent = self.old_id2parent[old_id]
-
-                entry = ChangesetEntry(old_id, old_parent, old_path)
-                entry.new_path = new_path
-                entry.new_parent = new_parent
-                if lines:
-                    entry.contents_change = PatchApply(''.join(lines))
-            elif action == 'removed':
-                old_path = _unescape(extra)
-                old_id = self.old_path2id[old_path]
-                old_parent = self.old_id2parent[old_id]
-                entry = ChangesetEntry(old_id, old_parent, old_path)
-                entry.new_path = None
-                entry.new_parent = None
-                if lines:
-                    # Technically a removed should be a ReplaceContents()
-                    # Where you need to have the old contents
-                    # But at most we have a remove style patch.
-                    #entry.contents_change = ReplaceContents()
-                    pass
-            elif action == 'added':
-                new_path = _unescape(extra)
-                new_id = self.path2id[new_path]
-                new_parent = self.id2parent[new_id]
-                entry = ChangesetEntry(new_id, new_parent, new_path)
-                entry.path = None
-                entry.parent = None
-                if lines:
-                    # Technically an added should be a ReplaceContents()
-                    # Where you need to have the old contents
-                    # But at most we have an add style patch.
-                    #entry.contents_change = ReplaceContents()
-                    entry.contents_change = PatchApply(''.join(lines))
-            elif action == 'modified':
-                new_path = _unescape(extra)
-                new_id = self.path2id[new_path]
-                new_parent = self.id2parent[new_id]
-                entry = ChangesetEntry(new_id, new_parent, new_path)
-                entry.path = None
-                entry.parent = None
-                if lines:
-                    # Technically an added should be a ReplaceContents()
-                    # Where you need to have the old contents
-                    # But at most we have an add style patch.
-                    #entry.contents_change = ReplaceContents()
-                    entry.contents_change = PatchApply(''.join(lines))
-            else:
-                raise BadChangeset('Unrecognized action: %r' % action)
-            cset.add_entry(entry)
-        return cset
-
-class ChangesetReader(object):
-    """This class reads in a changeset from a file, and returns
-    a Changeset object, which can then be applied against a tree.
-    """
-    def __init__(self, from_file):
-        """Read in the changeset from the file.
-
-        :param from_file: A file-like object (must have iterator support).
-        """
-        object.__init__(self)
-        self.from_file = from_file
-        
-        self.info = ChangesetInfo()
-        # We put the actual inventory ids in the footer, so that the patch
-        # is easier to read for humans.
-        # Unfortunately, that means we need to read everything before we
-        # can create a proper changeset.
-        self._read_header()
-        next_line = self._read_patches()
-        if next_line is not None:
-            self._read_footer(next_line)
-
-    def get_info(self):
-        """Create the actual changeset object.
-        """
-        self.info.create_maps()
-        return self.info
-
-    def _read_header(self):
-        """Read the bzr header"""
-        header = common.get_header()
-        for head_line, line in zip(header, self.from_file):
-            if (line[:2] != '# '
-                    or line[-1] != '\n'
-                    or line[2:-1] != head_line):
-                raise MalformedHeader('Did not read the opening'
-                    ' header information.')
-
-        for line in self.from_file:
-            if self._handle_info_line(line) is not None:
-                break
-
-    def _handle_info_line(self, line, in_footer=False):
-        """Handle reading a single line.
-
-        This may call itself, in the case that we read_multi,
-        and then had a dangling line on the end.
-        """
-        # The bzr header is terminated with a blank line
-        # which does not start with #
-        next_line = None
-        if line[:1] == '\n':
-            return 'break'
-        if line[:2] != '# ':
-            raise MalformedHeader('Opening bzr header did not start with #')
-
-        line = line[2:-1] # Remove the '# '
-        if not line:
-            return # Ignore blank lines
-
-        if in_footer and line in ('BEGIN BZR FOOTER', 'END BZR FOOTER'):
-            return
-
-        loc = line.find(': ')
-        if loc != -1:
-            key = line[:loc]
-            value = line[loc+2:]
-            if not value:
-                value, next_line = self._read_many()
-        else:
-            if line[-1:] == ':':
-                key = line[:-1]
-                value, next_line = self._read_many()
-            else:
-                raise MalformedHeader('While looking for key: value pairs,'
-                        ' did not find the colon %r' % (line))
-
-        key = key.replace(' ', '_')
-        if hasattr(self.info, key):
-            if getattr(self.info, key) is None:
-                setattr(self.info, key, value)
-            else:
-                raise MalformedHeader('Duplicated Key: %s' % key)
-        else:
-            # What do we do with a key we don't recognize
-            raise MalformedHeader('Unknown Key: %s' % key)
-        
-        if next_line:
-            self._handle_info_line(next_line, in_footer=in_footer)
-
-    def _read_many(self):
-        """If a line ends with no entry, that means that it should be
-        followed with multiple lines of values.
-
-        This detects the end of the list, because it will be a line that
-        does not start with '#    '. Because it has to read that extra
-        line, it returns the tuple: (values, next_line)
-        """
-        values = []
-        for line in self.from_file:
-            if line[:5] != '#    ':
-                return values, line
-            values.append(line[5:-1])
-        return values, None
-
-    def _read_one_patch(self, first_line=None):
-        """Read in one patch, return the complete patch, along with
-        the next line.
-
-        :return: action, lines, next_line, do_continue
-        """
-        first = True
-        action = None
-
-        def parse_firstline(line):
-            if line[:1] == '#':
-                return None
-            if line[:3] != '***':
-                raise MalformedPatches('The first line of all patches'
-                    ' should be a bzr meta line "***"')
-            return line[4:-1]
-
-        if first_line is not None:
-            action = parse_firstline(first_line)
-            first = False
-            if action is None:
-                return None, [], first_line, False
-
-        lines = []
-        for line in self.from_file:
-            if first:
-                action = parse_firstline(line)
-                first = False
-                if action is None:
-                    return None, [], line, False
-            else:
-                if line[:3] == '***':
-                    return action, lines, line, True
-                elif line[:1] == '#':
-                    return action, lines, line, False
-                lines.append(line)
-        return action, lines, None, False
-            
-    def _read_patches(self):
-        next_line = None
-        do_continue = True
-        while do_continue:
-            action, lines, next_line, do_continue = \
-                    self._read_one_patch(next_line)
-            if action is not None:
-                self.info.actions.append((action, lines))
-        return next_line
-
-    def _read_footer(self, first_line=None):
-        """Read the rest of the meta information.
-
-        :param first_line:  The previous step iterates past what it
-                            can handle. That extra line is given here.
-        """
-        if first_line is not None:
-            if self._handle_info_line(first_line, in_footer=True) is not None:
-                return
-        for line in self.from_file:
-            if self._handle_info_line(line, in_footer=True) is not None:
-                break
-
-
-def read_changeset(from_file):
-    """Read in a changeset from a filelike object (must have "readline" support), and
-    parse it into a Changeset object.
-    """
-    cr = ChangesetReader(from_file)
-    info = cr.get_info()
-    return info
-

=== removed file 'plugins/checkperms'
--- plugins/checkperms	2005-06-27 05:14:15 +0000
+++ plugins/checkperms	1970-01-01 00:00:00 +0000
@@ -1,5 +0,0 @@
-#! /bin/sh -ex
-
-root=`bzr root`
-
-find $root \! -perm -0444 
\ No newline at end of file

=== modified file 'setup.py'
--- setup.py	2005-03-14 07:07:24 +0000
+++ setup.py	2005-10-06 12:34:22 +0000
@@ -6,12 +6,64 @@
 
 from distutils.core import setup
 
+# more sophisticated setup script, based on pychecker setup.py
+import sys, os
+from distutils.command.build_scripts import build_scripts
+
+
+###############################
+# Overridden distutils actions
+###############################
+
+class my_build_scripts(build_scripts):
+    """Customized build_scripts distutils action.
+
+    Create bzr.bat for win32.
+    """
+
+    def run(self):
+        if sys.platform == "win32":
+            bat_path = os.path.join(self.build_dir, "bzr.bat")
+            self.scripts.append(bat_path)
+            self.mkpath(self.build_dir)
+            scripts_dir = self.distribution.get_command_obj("install").\
+                                                            install_scripts
+            self.execute(func=self._create_bat,
+                         args=[bat_path, scripts_dir],
+                         msg="Create %s" % bat_path)
+        build_scripts.run(self) # invoke "standard" action
+
+    def _create_bat(self, bat_path, scripts_dir):
+        """ Creates the batch file for bzr on win32.
+        """
+        try:
+            script_path = os.path.join(scripts_dir, "bzr")
+            bat_str = "@%s %s %%*\n" % (sys.executable, script_path)
+            file(bat_path, "w").write(bat_str)
+            print "file written"
+        except Exception, e:
+            print "ERROR: Unable to create %s: %s" % (bat_path, e)
+            raise e
+
+
+########################
+## Setup
+########################
+
 setup(name='bzr',
-      version='0.0.0',
+      version='0.1',
       author='Martin Pool',
       author_email='mbp@sourcefrog.net',
       url='http://www.bazaar-ng.org/',
       description='Friendly distributed version control system',
       license='GNU GPL v2',
-      packages=['bzrlib'],
+      packages=['bzrlib',
+                'bzrlib.plugins',
+                'bzrlib.selftest',
+                'bzrlib.util',
+                'bzrlib.transport',
+                'bzrlib.store',
+                'bzrlib.util.elementtree',
+                'bzrlib.util.effbot.org',
+                ],
       scripts=['bzr'])

=== removed file 'testsweet.py'
--- testsweet.py	2005-07-22 23:31:09 +0000
+++ testsweet.py	1970-01-01 00:00:00 +0000
@@ -1,366 +0,0 @@
-# Copyright (C) 2005 by Canonical Ltd
-
-# This program is free software; you can redistribute it and/or modify
-# it under the terms of the GNU General Public License as published by
-# the Free Software Foundation; either version 2 of the License, or
-# (at your option) any later version.
-
-# This program is distributed in the hope that it will be useful,
-# but WITHOUT ANY WARRANTY; without even the implied warranty of
-# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-# GNU General Public License for more details.
-
-# You should have received a copy of the GNU General Public License
-# along with this program; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
-
-
-"""Enhanced layer on unittest.
-
-This does several things:
-
-* nicer reporting as tests run
-
-* test code can log messages into a buffer that is recorded to disk
-  and displayed if the test fails
-
-* tests can be run in a separate directory, which is useful for code that
-  wants to create files
-
-* utilities to run external commands and check their return code
-  and/or output
-
-Test cases should normally subclass TestBase.  The test runner should
-call runsuite().
-
-This is meant to become independent of bzr, though that's not quite
-true yet.
-"""  
-
-
-from unittest import TestResult, TestCase
-
-def _need_subprocess():
-    sys.stderr.write("sorry, this test suite requires the subprocess module\n"
-                     "this is shipped with python2.4 and available separately for 2.3\n")
-    
-
-class CommandFailed(Exception):
-    pass
-
-
-
-class TestSkipped(Exception):
-    """Indicates that a test was intentionally skipped, rather than failing."""
-    # XXX: Not used yet
-
-
-class TestBase(TestCase):
-    """Base class for bzr test cases.
-
-    Just defines some useful helper functions; doesn't actually test
-    anything.
-    """
-    
-    # TODO: Special methods to invoke bzr, so that we can run it
-    # through a specified Python intepreter
-
-    OVERRIDE_PYTHON = None # to run with alternative python 'python'
-    BZRPATH = 'bzr'
-
-    _log_buf = ""
-
-
-    def setUp(self):
-        super(TestBase, self).setUp()
-        self.log("%s setup" % self.id())
-
-
-    def tearDown(self):
-        super(TestBase, self).tearDown()
-        self.log("%s teardown" % self.id())
-        self.log('')
-
-
-    def formcmd(self, cmd):
-        if isinstance(cmd, basestring):
-            cmd = cmd.split()
-
-        if cmd[0] == 'bzr':
-            cmd[0] = self.BZRPATH
-            if self.OVERRIDE_PYTHON:
-                cmd.insert(0, self.OVERRIDE_PYTHON)
-
-        self.log('$ %r' % cmd)
-
-        return cmd
-
-
-    def runcmd(self, cmd, retcode=0):
-        """Run one command and check the return code.
-
-        Returns a tuple of (stdout,stderr) strings.
-
-        If a single string is based, it is split into words.
-        For commands that are not simple space-separated words, please
-        pass a list instead."""
-        try:
-            import shutil
-            from subprocess import call
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
-
-        cmd = self.formcmd(cmd)
-
-        self.log('$ ' + ' '.join(cmd))
-        actual_retcode = call(cmd, stdout=self.TEST_LOG, stderr=self.TEST_LOG)
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-
-    def backtick(self, cmd, retcode=0):
-        """Run a command and return its output"""
-        try:
-            import shutil
-            from subprocess import Popen, PIPE
-        except ImportError, e:
-            _need_subprocess()
-            raise
-
-        cmd = self.formcmd(cmd)
-        child = Popen(cmd, stdout=PIPE, stderr=self.TEST_LOG)
-        outd, errd = child.communicate()
-        self.log(outd)
-        actual_retcode = child.wait()
-
-        outd = outd.replace('\r', '')
-
-        if retcode != actual_retcode:
-            raise CommandFailed("test failed: %r returned %d, expected %d"
-                                % (cmd, actual_retcode, retcode))
-
-        return outd
-
-
-
-    def build_tree(self, shape):
-        """Build a test tree according to a pattern.
-
-        shape is a sequence of file specifications.  If the final
-        character is '/', a directory is created.
-
-        This doesn't add anything to a branch.
-        """
-        # XXX: It's OK to just create them using forward slashes on windows?
-        import os
-        for name in shape:
-            assert isinstance(name, basestring)
-            if name[-1] == '/':
-                os.mkdir(name[:-1])
-            else:
-                f = file(name, 'wt')
-                print >>f, "contents of", name
-                f.close()
-
-
-    def log(self, msg):
-        """Log a message to a progress file"""
-        self._log_buf = self._log_buf + str(msg) + '\n'
-        print >>self.TEST_LOG, msg
-
-
-    def check_inventory_shape(self, inv, shape):
-        """
-        Compare an inventory to a list of expected names.
-
-        Fail if they are not precisely equal.
-        """
-        extras = []
-        shape = list(shape)             # copy
-        for path, ie in inv.entries():
-            name = path.replace('\\', '/')
-            if ie.kind == 'dir':
-                name = name + '/'
-            if name in shape:
-                shape.remove(name)
-            else:
-                extras.append(name)
-        if shape:
-            self.fail("expected paths not found in inventory: %r" % shape)
-        if extras:
-            self.fail("unexpected paths found in inventory: %r" % extras)
-
-
-    def check_file_contents(self, filename, expect):
-        self.log("check contents of file %s" % filename)
-        contents = file(filename, 'r').read()
-        if contents != expect:
-            self.log("expected: %r" % expect)
-            self.log("actually: %r" % contents)
-            self.fail("contents of %s not as expected")
-            
-
-
-class InTempDir(TestBase):
-    """Base class for tests run in a temporary branch."""
-    def setUp(self):
-        import os
-        self.test_dir = os.path.join(self.TEST_ROOT, self.__class__.__name__)
-        os.mkdir(self.test_dir)
-        os.chdir(self.test_dir)
-        
-    def tearDown(self):
-        import os
-        os.chdir(self.TEST_ROOT)
-
-
-
-
-
-class _MyResult(TestResult):
-    """
-    Custom TestResult.
-
-    No special behaviour for now.
-    """
-    def __init__(self, out, style):
-        self.out = out
-        TestResult.__init__(self)
-        assert style in ('none', 'progress', 'verbose')
-        self.style = style
-
-
-    def startTest(self, test):
-        # TODO: Maybe show test.shortDescription somewhere?
-        what = test.id()
-        # python2.3 has the bad habit of just "runit" for doctests
-        if what == 'runit':
-            what = test.shortDescription()
-        
-        if self.style == 'verbose':
-            print >>self.out, '%-60.60s' % what,
-            self.out.flush()
-        elif self.style == 'progress':
-            self.out.write('~')
-            self.out.flush()
-        TestResult.startTest(self, test)
-
-
-    def stopTest(self, test):
-        # print
-        TestResult.stopTest(self, test)
-
-
-    def addError(self, test, err):
-        if self.style == 'verbose':
-            print >>self.out, 'ERROR'
-        TestResult.addError(self, test, err)
-        _show_test_failure('error', test, err, self.out)
-
-    def addFailure(self, test, err):
-        if self.style == 'verbose':
-            print >>self.out, 'FAILURE'
-        TestResult.addFailure(self, test, err)
-        _show_test_failure('failure', test, err, self.out)
-
-    def addSuccess(self, test):
-        if self.style == 'verbose':
-            print >>self.out, 'OK'
-        TestResult.addSuccess(self, test)
-
-
-
-def run_suite(suite, name='test', verbose=False):
-    import os
-    import shutil
-    import time
-    import sys
-    
-    _setup_test_log(name)
-    _setup_test_dir(name)
-    print
-
-    # save stdout & stderr so there's no leakage from code-under-test
-    real_stdout = sys.stdout
-    real_stderr = sys.stderr
-    sys.stdout = sys.stderr = TestBase.TEST_LOG
-    try:
-        if verbose:
-            style = 'verbose'
-        else:
-            style = 'progress'
-        result = _MyResult(real_stdout, style)
-        suite.run(result)
-    finally:
-        sys.stdout = real_stdout
-        sys.stderr = real_stderr
-
-    _show_results(result)
-
-    return result.wasSuccessful()
-
-
-
-def _setup_test_log(name):
-    import time
-    import os
-    
-    log_filename = os.path.abspath(name + '.log')
-    TestBase.TEST_LOG = open(log_filename, 'wt', buffering=1) # line buffered
-
-    print >>TestBase.TEST_LOG, "tests run at " + time.ctime()
-    print '%-30s %s' % ('test log', log_filename)
-
-
-def _setup_test_dir(name):
-    import os
-    import shutil
-    
-    TestBase.ORIG_DIR = os.getcwdu()
-    TestBase.TEST_ROOT = os.path.abspath(name + '.tmp')
-
-    print '%-30s %s' % ('running tests in', TestBase.TEST_ROOT)
-
-    if os.path.exists(TestBase.TEST_ROOT):
-        shutil.rmtree(TestBase.TEST_ROOT)
-    os.mkdir(TestBase.TEST_ROOT)
-    os.chdir(TestBase.TEST_ROOT)
-
-    # make a fake bzr directory there to prevent any tests propagating
-    # up onto the source directory's real branch
-    os.mkdir(os.path.join(TestBase.TEST_ROOT, '.bzr'))
-
-    
-
-def _show_results(result):
-     print
-     print '%4d tests run' % result.testsRun
-     print '%4d errors' % len(result.errors)
-     print '%4d failures' % len(result.failures)
-
-
-
-def _show_test_failure(kind, case, exc_info, out):
-    from traceback import print_exception
-    
-    print >>out, '-' * 60
-    print >>out, case
-    
-    desc = case.shortDescription()
-    if desc:
-        print >>out, '   (%s)' % desc
-         
-    print_exception(exc_info[0], exc_info[1], exc_info[2], None, out)
-        
-    if isinstance(case, TestBase):
-        print >>out
-        print >>out, 'log from this test:'
-        print >>out, case._log_buf
-         
-    print >>out, '-' * 60
-    
-

=== added file 'tools/capture_tree.py'
--- tools/capture_tree.py	1970-01-01 00:00:00 +0000
+++ tools/capture_tree.py	2005-10-04 11:13:33 +0000
@@ -0,0 +1,40 @@
+#! /usr/bin/env python2.4
+
+# Copyright (C) 2005 Canonical Ltd
+
+"""Print to stdout a description of the current directory, 
+formatted as a Python data structure.
+
+This can be useful in tests that need to recreate directory
+contents."""
+
+import sys
+import os
+
+from bzrlib.trace import enable_default_logging
+enable_default_logging()
+from bzrlib.selftest.treeshape import capture_tree_contents
+
+def main(argv):
+    # a lame reimplementation of pformat that splits multi-line
+    # strings into concatenated string literals.
+    print '['
+    for tt in capture_tree_contents('.'):
+        assert isinstance(tt, tuple)
+        print '    (', repr(tt[0]) + ',',
+        if len(tt) == 1:
+            print '),'
+        else:
+            assert len(tt) == 2
+            val = tt[1]
+            print
+            if val == '':
+                print "        ''"
+            else:
+                for valline in val.splitlines(True):
+                    print '       ', repr(valline)
+            print '    ),'
+    print ']'
+
+if __name__ == '__main__':
+    sys.exit(main(sys.argv))

=== added file 'tools/history2revfiles.py'
--- tools/history2revfiles.py	1970-01-01 00:00:00 +0000
+++ tools/history2revfiles.py	2005-08-25 05:58:05 +0000
@@ -0,0 +1,99 @@
+#! /usr/bin/python
+
+# Copyright (C) 2005 Canonical Ltd
+
+# This program is free software; you can redistribute it and/or modify
+# it under the terms of the GNU General Public License as published by
+# the Free Software Foundation; either version 2 of the License, or
+# (at your option) any later version.
+
+# This program is distributed in the hope that it will be useful,
+# but WITHOUT ANY WARRANTY; without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+# GNU General Public License for more details.
+
+# You should have received a copy of the GNU General Public License
+# along with this program; if not, write to the Free Software
+# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+
+"""Experiment in converting existing bzr branches to weaves."""
+
+
+import bzrlib.branch
+from bzrlib.revfile import Revfile
+from bzrlib.progress import ProgressBar
+import tempfile
+import hotshot, hotshot.stats
+import sys
+
+def convert():
+    pb = ProgressBar()
+
+    inv_rf = Revfile('revfiles/inventory', 'w')
+    last_text_sha = {}
+    text_rfs = {}
+
+    b = bzrlib.branch.find_branch('.')
+
+    revno = 1
+    rev_history = b.revision_history()
+    last_idx = None
+    for rev_id in rev_history:
+        pb.update('converting inventory', revno, len(rev_history))
+        inv_xml = b.get_inventory_xml(rev_id).read()
+        new_idx = inv_rf.add(inv_xml, last_idx, compress=False)
+
+        tree = b.revision_tree(rev_id)
+        inv = tree.inventory
+
+        # for each file in the inventory, put it into its own revfile
+        for file_id in inv:
+            ie = inv[file_id]
+            if ie.kind != 'file':
+                continue
+            if last_text_sha.get(file_id) == ie.text_sha1:
+                # same as last time
+                continue
+            last_text_sha[file_id] = ie.text_sha1
+
+            # new text (though possibly already stored); need to store it
+            text = tree.get_file(file_id).read()
+            
+            if file_id not in text_rfs:
+                text_rfs[file_id] = Revfile('revfiles/' + file_id, 'w')
+            rf = text_rfs[file_id]
+
+            last = len(rf)
+            if last == 0:
+                last = None
+            else:
+                last -= 1
+            rf.add(text, last, compress=True)
+        
+        last_idx = new_idx
+        revno += 1
+
+    pb.clear()
+
+
+def profile_convert(): 
+    prof_f = tempfile.NamedTemporaryFile()
+
+    prof = hotshot.Profile(prof_f.name)
+
+    prof.runcall(convert) 
+    prof.close()
+
+    stats = hotshot.stats.load(prof_f.name)
+    #stats.strip_dirs()
+    stats.sort_stats('time')
+    ## XXX: Might like to write to stderr or the trace file instead but
+    ## print_stats seems hardcoded to stdout
+    stats.print_stats(20)
+            
+
+if '-p' in sys.argv[1:]:
+    profile_convert()
+else:
+    convert()
+    

=== added file 'tools/http_client.py'
--- tools/http_client.py	1970-01-01 00:00:00 +0000
+++ tools/http_client.py	2005-08-25 12:46:42 +0000
@@ -0,0 +1,501 @@
+#! /usr/bin/python
+
+# $Id: http_client.py 271 2004-10-09 10:50:59Z fredrik $
+# a simple asynchronous http client (based on SimpleAsyncHTTP.py from
+# "Python Standard Library" by Fredrik Lundh, O'Reilly 2001)
+#
+# HTTP/1.1 and GZIP support added in January 2003 by Fredrik Lundh.
+#
+# changes:
+# 2004-08-26 fl   unified http callback
+# 2004-10-09 fl   factored out gzip_consumer support
+# 2005-07-08 mbp  experimental support for keepalive connections
+#
+# Copyright (c) 2001-2004 by Fredrik Lundh.  All rights reserved.
+#
+
+
+
+"""async/pipelined http client
+
+Use
+===
+
+Users of this library pass in URLs they want to see, and consumer
+objects that will receive the results at some point in the future.
+Any number of requests may be queued up, and more may be added while
+the download is in progress.
+
+Requests can be both superscalar and superpipelined.  That is to say,
+for each server there can be multiple sockets open, and each socket
+may have more than one request in flight.
+
+Design
+======
+
+There is a single DownloadManager, and a connection object for each
+open socket.
+
+Request/consumer pairs are maintained in queues.  Each connection has
+a list of transmitted requests whose response has not yet been
+received.  There is also a per-server list of requests that have not
+yet been submitted.
+
+When a connection is ready to transmit a new request, it takes one
+from the unsubmitted list, sends the request, and adds the request to
+its unfulfilled list.  This should happen when the connection has
+space for more transmissions or when a new request is added by the
+user.  If the connection terminates with unfulfilled requests they are
+put back onto the unsubmitted list, to be retried elsewhere.
+
+Because responses come back precisely in order, the connection always
+knows what it should expect next: the response for the next
+unfulfilled request.
+"""
+
+# Note that (as of ubuntu python 2.4.1) every socket.connect() call
+# with a hostname does a remote DNS resolution, which is pretty sucky.
+# Shouldn't there be a cache in glibc?  We should probably cache the
+# address in, say, the DownloadManager.
+
+# TODO: A default consumer operation that writes the received data
+# into a file; by default the file is named the same as the last
+# component of the URL.
+
+# TODO: A utility function that is given a list of URLs, and downloads
+# them all parallel/pipelined.  If any fail, it raises an exception
+# (and discards the rest), or perhaps can be told to continue anyhow.
+# The content is written into temporary files.  It returns a list of
+# readable file objects.
+
+# TODO: If we try pipelined or keepalive and the connection drop out
+# then retry the request on a new connection; eventually we should perhaps
+# learn that a given host or network just won't allow keepalive.
+
+
+import asyncore
+import socket, string, time, sys
+import StringIO
+import mimetools, urlparse, urllib
+import logging
+
+logging.basicConfig(level=logging.DEBUG,
+                    format='%(asctime)s %(levelname)s %(message)s',
+                    filename='/tmp/http_client.log',
+                    filemode='w')
+
+logger = logging.getLogger('bzr.http_client')
+debug = logger.debug
+info = logger.info
+error = logger.error
+
+
+##
+# Close connection.   Request handlers can raise this exception to
+# indicate that the connection should be closed.
+
+class CloseConnection(Exception):
+    pass
+
+##
+# Redirect connection.  Request handlers can raise this exception to
+# indicate that the a new request should be issued.
+
+class Redirect(CloseConnection):
+    def __init__(self, location):
+        self.location = location
+
+
+class DownloadManager(object):
+    """Handles pipelined/overlapped downloads.
+
+    Pass in a series of URLs with handlers to receive the response.
+    This object will spread the requests over however many sockets
+    seem useful.
+
+    queued_requests
+        Requests not assigned to any channel
+
+    running_requests
+        Currently assigned to a channel
+    """
+    def __init__(self):
+        self.queued_requests = []
+        # self.channel = HttpChannel('localhost', 8000, self)
+        self.channels = []
+        self.try_pipelined = False
+        self.try_keepalive = False
+        self.max_channels = 5
+
+
+    def enqueue(self, url, consumer):
+        self.queued_requests.append((url, consumer))
+        self._wake_up_channel()
+
+
+    def _channel_closed(self, channel):
+        """Called by the channel when its socket closes.
+        """
+        self.channels.remove(channel)
+        if self.queued_requests:
+            # might recreate one
+            self._wake_up_channel()
+
+
+    def _make_channel(self):
+        # proxy2 203.17.154.69
+        # return HttpChannel('82.211.81.161', 80, self)         # bazaar-ng.org 
+        # return HttpChannel('203.17.154.69', 8080, self)
+        return HttpChannel('127.0.0.1', 8000, self)  # forwarded
+            
+
+    def _wake_up_channel(self):
+        """Try to wake up one channel to send the newly-added request.
+
+        There may be more than one request pending, and this may cause
+        more than one channel to take requests.  That's OK; some of
+        them may be frustrated.
+        """
+        from random import shuffle, choice
+        
+        # first, wake up any idle channels
+        done = False
+        for ch in self.channels:
+            if not ch.sent_requests:
+                ch.take_one()
+                done = True
+        if done:
+            debug("woke existing idle channel(s)")
+            return
+
+        if len(self.channels) < self.max_channels:
+            newch = self._make_channel()
+            self.channels.append(newch)
+            newch.take_one()
+            debug("created new channel")
+            return
+
+        if self.try_pipelined:
+            # ask existing channels to take it
+            debug("woke busy channel")
+            choice(self.channels).take_one()
+
+
+        # debug("request postponed until a channel's idle")
+        
+
+
+
+    def run(self):
+        """Run until all outstanding requests have been served."""
+        #while self.running_requests or self.queued_requests \
+        #          or not self.channel.is_idle():
+        #    asyncore.loop(count=1)
+        asyncore.loop()
+
+
+
+class Response(object):
+    """Holds in-flight response."""
+
+
+
+def _parse_response_http10(header):
+    from cStringIO import StringIO
+
+    fp = StringIO(header)
+    r = Response()
+
+    r.status = fp.readline().split(" ", 2)
+    r.headers = mimetools.Message(fp)
+
+    # we can only(?) expect to do keepalive if we got either a 
+    # content-length or chunked encoding; otherwise there's no way to know
+    # when the content ends apart from through the connection close
+    r.content_type = r.headers.get("content-type")
+    try:
+        r.content_length = int(r.headers.get("content-length"))
+    except (ValueError, TypeError):
+        r.content_length = None
+    debug("seen content length of %r" % r.content_length)
+
+    r.transfer_encoding = r.headers.get("transfer-encoding")
+    r.content_encoding = r.headers.get("content-encoding")
+    r.connection_reply = r.headers.get("connection")
+
+    # TODO: pass status code to consumer?
+
+    if r.transfer_encoding:
+        raise NotImplementedError()
+
+    if r.transfer_encoding:
+        raise NotImplementedError()
+
+    if int(r.status[1]) != 200:
+        debug("can't handle response status %r" % r.status)
+        raise NotImplementedError()
+
+    if r.content_length == None:
+        raise NotImplementedError()
+
+    if r.content_length == 0:
+        raise NotImplementedError()
+
+    r.content_remaining = r.content_length                
+
+    return r
+
+
+    
+    
+        
+
+
+class HttpChannel(asyncore.dispatcher_with_send):
+    """One http socket, pipelining if possible."""
+    # asynchronous http client
+
+    user_agent = "http_client.py 1.3ka (based on effbot)"
+
+    proxies = urllib.getproxies()
+
+    def __init__(self, ip_host, ip_port, manager):
+        asyncore.dispatcher_with_send.__init__(self)
+        self.manager = manager
+
+        # if a response header has been seen, this holds it
+        self.response = None
+        
+        self.data = ""
+
+        self.chunk_size = None
+
+        self.timestamp = time.time()
+
+        self.create_socket(socket.AF_INET, socket.SOCK_STREAM)
+        debug('connecting...')
+        self.connect((ip_host, ip_port))
+
+        # sent_requests holds (url, consumer) 
+        self.sent_requests = []
+
+        self._outbuf = ''
+
+
+    def __repr__(self):
+        return 'HttpChannel(local_port=%r)' % (self.getsockname(),)
+
+
+    def is_idle(self):
+        return (not self.sent_requests)
+
+
+    def handle_connect(self):
+        debug("connected")
+        self.take_one()
+
+
+    def take_one(self):
+        """Accept one request from the manager if possible."""
+        if self.manager.try_pipelined:
+            if len(self.sent_requests) > 4:
+                return
+        else:
+            if len(self.sent_requests) > 0:
+                return 
+        
+        try:
+            url, consumer = self.manager.queued_requests.pop(0)
+            debug('request accepted by channel')
+        except IndexError:
+            return
+        
+        # TODO: If there are too many already in flight, don't take one.
+        # TODO: If the socket's not writable (tx buffer full), don't take.
+        self._push_request_http10(url, consumer)
+
+
+
+    def _push_request_http10(self, url, consumer):
+        """Send a request, and add it to the outstanding queue."""
+        # TODO: check the url requested is appropriate for this connection
+
+        # TODO: If there are too many requests outstanding or (less likely) the 
+        # connection fails, queue it for later use.
+
+        # TODO: Keep track of requests that have been sent but not yet fulfilled,
+        # because we might need to retransmit them if the connection fails. (Or
+        # should the caller do that?)
+
+        request = self._form_request_http10(url)
+        debug('send request for %s from %r' % (url, self))
+
+        # dispatcher_with_send handles buffering the data until it can
+        # be written, and hooks handle_write.
+
+        self.send(request)
+
+        self.sent_requests.append((url, consumer))
+
+
+    def _form_request_http10(self, url):
+        # TODO: get right vhost name
+        request = [
+            "GET %s HTTP/1.0" % (url),
+            "Host: www.bazaar-ng.org",
+            ]
+
+        if self.manager.try_keepalive or self.manager.try_pipelined:
+            request.extend([
+                "Keep-Alive: 60", 
+                "Connection: keep-alive",
+                ])
+
+        # make sure to include a user agent
+        for header in request:
+            if string.lower(header).startswith("user-agent:"):
+                break
+        else:
+            request.append("User-Agent: %s" % self.user_agent)
+
+        return string.join(request, "\r\n") + "\r\n\r\n"
+
+
+    def handle_read(self):
+        # handle incoming data
+        data = self.recv(2048)
+
+        self.data = self.data + data
+
+        if len(data):
+            debug('got %d bytes from socket' % len(data))
+        else:
+            debug('server closed connection')
+
+        while self.data:
+            consumer = self.sent_requests[0][1]
+            if not self.response:
+                # do not have a full response header yet
+
+                # check if we've seen a full header
+                debug('getting header for %s' % self.sent_requests[0][0])
+
+                header = self.data.split("\r\n\r\n", 1)
+                if len(header) <= 1:
+                    return
+                header, self.data = header
+
+                self.response = _parse_response_http10(header)
+                self.content_remaining = self.response.content_length
+
+            if not self.data:
+                return
+
+            # we now know how many (more) content bytes we have, and how much
+            # is in the data buffer. there are two main possibilities:
+            # too much data, and some must be left behind containing the next
+            # response headers, or too little, or possibly just right
+
+            want = self.content_remaining
+            if want > 0:
+                got_data = self.data[:want]
+                self.data = self.data[want:]
+                
+                assert got_data
+
+                self.content_remaining -= len(got_data)
+
+                debug('pass back %d bytes of %s, %d remain'
+                      % (len(got_data),
+                         self.sent_requests[0][0],
+                         self.content_remaining))
+                consumer.feed(data)
+
+            if self.content_remaining == 0:
+                del self.sent_requests[0]
+
+                debug('content complete')
+                consumer.content_complete()
+                
+                # reset lots of things and try to get the next response header
+                if self.response.connection_reply == 'close':
+                    debug('server requested close')
+                    self.manager._channel_closed(self)
+                    self.close()
+                elif not self.manager.try_keepalive:
+                    debug('no keepalive for this socket')
+                    self.manager._channel_closed(self)
+                    self.close()
+                else:
+                    debug("ready for next header...")
+                    self.take_one()
+                self.response = None
+
+
+
+    def handle_close(self):
+        debug('async told us of close on %r' % self)
+        # if there are outstanding requests should probably reopen and 
+        # retransmit, but if we're not making any progress then give up
+        self.manager._channel_closed(self)
+        self.close()
+
+
+class DummyConsumer:
+    def __init__(self, url, pb):
+        self.url = url
+        self.outf = None
+        self._pb = pb
+
+    def feed(self, data):
+        # print "feed", repr(data)
+        # print "feed", repr(data[:20]), repr(data[-20:]), len(data)
+        if not self.outf:
+            base = self.url[self.url.rindex('/')+1:]
+            self.outf = file('/tmp/download/' + base, 'wb')
+        self.outf.write(data)
+
+    def error(self, err_info):
+        import traceback
+        error('error reported to consumer')
+        traceback.print_exception(err_info[0], err_info[1], err_info[2])
+        sys.exit(1)
+
+    def content_complete(self):
+        info('content complete from %s' % self.url)
+        self.outf.close()
+        self.outf = None
+        # using last_cnt is cheating
+        self._pb.update('downloading inventory',
+                        self._pb.last_cnt+1,
+                        self._pb.last_total)
+
+
+
+if __name__ == "__main__":
+    logging.basicConfig(level=logging.DEBUG)
+
+    mgr = DownloadManager()
+
+    from bzrlib.branch import Branch
+    from bzrlib.progress import ProgressBar
+
+    pb = ProgressBar()
+    revs = Branch('/home/mbp/work/bzr').revision_history()
+    pb.update('downloading inventories', 0, len(revs))
+
+    for rev in revs:
+        url = 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/inventory-store/' \
+              + rev + '.gz'
+        mgr.enqueue(url, DummyConsumer(url, pb))
+
+    mgr.run()
+    
+
+
+    
+#     for url in ['http://www.bazaar-ng.org/',
+#                 'http://www.bazaar-ng.org/tutorial.html',
+#                 'http://www.bazaar-ng.org/download.html',
+#                 'http://www.bazaar-ng.org/bzr/bzr.dev/.bzr/revision-store/mbp@hope-20050415013653-3b3c9c3d33fae0a6.gz',
+#                 ]:

=== modified file 'tools/weavebench.py'
--- tools/weavebench.py	2005-07-18 11:23:40 +0000
+++ tools/weavebench.py	2005-08-25 07:48:27 +0000
@@ -51,7 +51,7 @@
             if lines:
                 del lines[randrange(0, len(lines))]
 
-        rev_id = wf.add(parents, lines)
+        rev_id = wf.add("%s" % i, parents, lines)
         parents = [rev_id]
 
     write_weave(wf, file(WEAVE_NAME, 'wb'))

=== added file 'tutorial.txt'
--- tutorial.txt	1970-01-01 00:00:00 +0000
+++ tutorial.txt	2005-09-14 09:03:27 +0000
@@ -0,0 +1,312 @@
+==================
+Bazaar-NG Tutorial
+==================
+
+current for bzr 0.0.6pre, July 2005
+
+
+*NOTE* For a more current and user-editable version of this 
+document, see the wiki at http://bazaar.canonical.com/IntroductionToBzr
+
+
+Introduction
+============
+
+Bazaar-NG is a version control tool.  It manages trees of files and subdirectories.  In particular, it records *revisions* of trees, representing their state at a particular point in time, and information about those revisions and their relationships.  Recording and retrieving tree revisions is useful in several ways if you are writing software or documents or doing similar creative work.
+
+ * Keeping previous revisions lets you go back if you make a mistake or want to check your work.  It acts as a high-level unlimited undo.
+
+ * By recording comments on every revision, you produce an annotated history of the project, describing what, who, why, and when.
+
+ * Using a version control tool can be an aid to thinking about a project: getting to a stable state at regular intervals and then writing a description of what you did is an easy way to stay organized and on track.
+
+Bazaar-NG remembers the *ancestry* of a revision: the previous revisions that it is based upon.  A single revision may have more than one direct descendant, each with different changes, representing a divergence in the evolution of the tree.  
+By branching, Bazaar-NG allows multiple people to cooperate on the evolution of a project, without all needing to work in strict
+lock-step.  Branching can be useful even for a single developer.
+
+Bazaar-NG installs a single new command,
+*bzr*.  Everything else is a subcommand of this.  You can get
+some help with ``bzr help``.  There will be more in the future.
+
+
+
+Introducing yourself to bzr
+===========================
+
+One function of a version control system is to keep track of who
+changed what.  In a distributed system that requires an identifier for
+each author that is globally unique.  Most people already have one of
+these: an email address.
+
+[after 0.0.4] To tell bzr which email address to use, put it in the file
+``$HOME/.bzr.conf/email``, or the environment variable ``$BZREMAIL``.
+If neither of these are set, bzr will use the ``$EMAIL``
+variable, or use your username and hostname.
+
+To check this has taken effect, or if you forget your own name, use
+the ``whoami`` ("who am i?") command::
+
+  % bzr whoami
+
+Some people want to avoid sharing their email address so as not to
+get spam.  bzr will never
+disclose your email address unless you tell it to by publishing an
+archive or transmitting a changeset.  It's recommended that you do use
+a real address, so that people can contact you about your work, but
+it's not required.  You can use an address which is obfuscated, which
+bounces, or which goes through an anti-spam service such as spamgourmet.com.
+
+
+
+Creating a branch
+==================
+
+
+History is by default stored in the .bzr directory of the branch.
+There will be a facility to store it in a separate repository, which
+may be remote.  We create a new branch by running *bzr init* in
+an existing directory::
+
+    % mkdir tutorial
+    % cd tutorial
+    % ls -a
+    ls -a
+    ./  ../
+    % pwd
+    /home/mbp/work/bzr.test/tutorial
+    %
+    % bzr init
+    % ls -aF
+    ./  ../  .bzr/
+    %
+
+As for CVS, there are three classes of file: unknown, ignored, and
+versioned.  The *add* command makes a file versioned: that is,
+changes to it will be recorded by the system::
+
+    % echo 'hello world' > hello.txt
+    % bzr status
+    ?       hello.txt
+    % bzr unknowns
+    hello.txt
+    % bzr add -v hello.txt
+    A       hello.txt
+    % bzr unknowns
+
+
+If you add the wrong file, simply use ``bzr remove`` to make
+it unversioned again.  This does not delete the working copy.
+
+
+Reviewing changes
+=================
+
+Once you have completed some work, you will want to *commit*
+it to the version history.  It is good to commit fairly often:
+whenever you get a new feature working, fix a bug, or improve some
+code or documentation.  It's also a good practice to make sure that
+the code compiles and passes its test suite before committing, to make
+sure that every revision is a known-good state.  You can also review
+your changes, to make sure you're committing what you intend to, and
+as a chance to rethink your work before you permanently record it. 
+
+Two bzr commands are particularly useful here: *status* and
+*diff*.  The *status* command
+shows a listing with one line per file, indicating whether it has been
+Added, Deleted, Modified, or Renamed in the current revision.  Unknown
+files are shown as '?'.  With the ``--all`` option, the status
+command also shows unmodified versioned files as '.', and ignored
+files as 'I'::
+
+    % bzr status
+    A       hello.txt
+
+The *diff* command shows the full text of changes to all
+files as a standard unified diff.  This can be piped through many
+programs such as ``patch``, ``diffstat``,
+``filterdiff`` and ``colordiff``::
+
+    % bzr diff
+    *** added file 'hello.txt'
+    --- /dev/null 
+    +++ hello.txt 
+    @@ -1,0 +1,1 @@
+    +hello world
+
+With the ``-r`` option, the tree is compared to an earlier
+revision.
+
+[TODO: options to run external diff; to get context diff or other
+formats; to diff only selected files; to compare two historical
+revisions.]
+
+
+
+Committing changes
+==================
+
+When the working tree state is satisfactory, it can be
+*committed* to the branch, creating a new revision holding a
+snapshot of that state.  
+
+The ``commit`` command takes a message describing the changes
+in the revision.  It also records your userid, the current time and
+timezone, and the inventory and contents of the tree.  The commit
+message is specified by the ``-m`` or ``--message`` option.
+You can enter a multi-line commit message; in most shells you can
+enter this just by leaving the quotes open at the end of the line. ::
+
+    % bzr commit -m "added my first file"
+
+[TODO: commit message interactively, through an editor or from a
+file.]
+
+[TODO: commit only selected files, including renamed/added/deleted
+files.]
+
+
+
+Removing uncommitted changes
+============================
+
+If you've made some changes and don't want to keep them, use the
+``revert`` command to go back to the previous head version.  It's a
+good idea to use ``bzr diff`` first to see what will be removed. 
+By default the revert command reverts the whole tree; if file or
+directory names are given then only those ones will be affected.
+revert also clears the list of pending merges revisions.
+
+
+
+
+
+Ignoring files
+==============
+
+Many source trees contain some files that do not need to be
+versioned, such as editor backups, object or bytecode files, and built
+programs.  You can simply not add them, but then they'll always crop
+up as unknown files.  You can also tell bzr to ignore these files by
+adding them to a file called ``.bzrignore`` at the top of the
+tree.
+
+This file contains a list of file wildcards (or "globs"), one
+per line.  Typical contents are like this::
+
+    *.o
+    *~
+    *.tmp
+    *.py[co]
+
+If a glob contains a slash, it is matched against the whole path
+from the top of the tree; otherwise it is matched against only the
+filename.  So the previous example ignores ``*.o`` in all
+subdirectories, but this example ignores only config.h at the top
+level and HTML files in ``doc/``::
+
+    ./config.h
+    doc/*.html
+
+To get a list of which files are ignored and what pattern they matched, use ``bzr ignored``::
+
+    % bzr ignored
+    config.h                                           ./config.h
+    configure.in~                                      *~
+
+It is OK to have an ignore pattern match a versioned file, or to
+add an ignored file.  Ignore patterns have no effect on versioned
+files; they only determine whether unversioned files are reported as
+unknown or ignored.
+
+The ``.bzrignore`` file should normally be versioned, so that new
+copies of the branch see the same patterns::
+
+    % bzr add .bzrignore
+    % bzr commit -m "Add ignore patterns"
+
+
+Examining history
+=================
+
+bzr log
+-------
+
+The ``log`` command shows a list of previous revisions.
+
+
+Branch statistics
+=================
+
+The ``bzr info`` command shows some summary information about
+the working tree and the branch history.  
+
+
+Versioning directories
+======================
+
+bzr versions files and directories in a way that can keep track of
+renames and intelligently merge them::
+
+    % mkdir src
+    % echo 'int main() {}' > src/simple.c
+    % bzr add src
+    % bzr status
+    A       src/
+    ?       src/simple.c
+    % bzr add src/simple.c
+    % bzr status
+    A       src/
+    A       src/simple.c
+
+
+Deleting and removing files
+===========================
+
+You can delete files or directories by just deleting them from the
+working directory.  This is a bit different to CVS, which requires
+that you also do *cvs remove*.
+
+*bzr remove* makes the file un-versioned, but does not
+delete the working copy.  This is useful when you add the wrong file,
+or decide that a file should actually not be versioned. ::
+
+    % rm -r src
+    % bzr remove -v hello.txt
+    ?       hello.txt
+    % bzr status
+    ?       hello.txt
+    D       src/
+    D       src/simple.c
+
+
+Branching
+=========
+
+Often rather than starting your own project, you will want to
+submit a change to an existing project.  You can get a copy of an
+existing branch by copying its directory, expanding a tarball, or by a
+remote copy using something like rsync.  You can also use bzr to fetch
+a copy.  Because this new copy is potentially a new branch, the
+command is called *branch*::
+
+    % bzr branch http://bazaar-ng.org/bzr/bzr.dev
+    % cd bzr.dev
+
+This copies down the complete history of this branch, so we can
+do all operations on it locally: log, annotate, making and merging
+branches.  There will be an option to get only part of the history if
+you wish.
+
+
+
+Following upstream changes
+==========================
+
+You can stay up-to-date with the parent branch by *pulling*
+in their changes::
+
+    % bzr pull
+
+This only works if your local branch includes only changes from the
+parent branch.  Otherwise, the branches are said to have *diverged*,
+and they must be merged instead.

